-------------------------------------------------------------------------------------------
-- Copyright  2011-2017, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    09:21:15 12/24/2011 
-- Design Name: 
-- Module Name:    fmcmc1sys - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity fmcmc1sys is
	generic
	(
		C_FAMILY        : string  := "zynq";
		--
		C_SYSTEMCLOCK   : integer := 100_000_000;
		C_XADC          : integer := 1
	);
	port 
	(
		clock           : in  std_logic;
		reset           : in  std_logic;
		--
		-- host interface
		--
		host_en         : in  std_logic;
		host_we         : in  std_logic;
		host_addr       : in  std_logic_vector(31 downto 0);
		host_din        : in  std_logic_vector(31 downto 0);
		host_dout       : out std_logic_vector(31 downto 0);
		host_ack        : out std_logic;
		--
		-- ads1204(x) system clock 20 MHz tip
		ads_clock       : in  std_logic;
		-- M0
		-- ads1204
		ads_clkin_0     : out std_logic;
		ads_clkout_0    : in  std_logic;
		ads_ia_0        : in  std_logic;
		ads_ib_0        : in  std_logic;
		ads_it_0        : in  std_logic;
		ads_vbus_0      : in  std_logic;
		-- drv8312
		drv_pwm_a_0     : out std_logic;
		drv_pwm_b_0     : out std_logic;
		drv_pwm_c_0     : out std_logic;
		drv_rst_a_0     : out std_logic;
		drv_rst_b_0     : out std_logic;
		drv_rst_c_0     : out std_logic;
		drv_fault_0     : in  std_logic;
		drv_otw_0       : in  std_logic;
		-- bemf zero crossing
		bemf_a_0        : in  std_logic;
		bemf_b_0        : in  std_logic;
		bemf_c_0        : in  std_logic;
		-- encoder/hall sensor
		sensor1_0       : in  std_logic;
		sensor2_0       : in  std_logic;
		sensor3_0       : in  std_logic;
		-- M1
		-- ads1204
		ads_clkin_1     : out std_logic;
		ads_clkout_1    : in  std_logic;
		ads_ia_1        : in  std_logic;
		ads_ib_1        : in  std_logic;
		ads_it_1        : in  std_logic;
		-- drv8312
		drv_pwm_a_1     : out std_logic;
		drv_pwm_b_1     : out std_logic;
		drv_pwm_c_1     : out std_logic;
		drv_rst_a_1     : out std_logic;
		drv_rst_b_1     : out std_logic;
		drv_rst_c_1     : out std_logic;
		drv_fault_1     : in  std_logic;
		drv_otw_1       : in  std_logic;
		-- bemf zero crossing
		bemf_a_1        : in  std_logic;
		bemf_b_1        : in  std_logic;
		bemf_c_1        : in  std_logic;
		-- encoder/hall sensor
		sensor1_1       : in  std_logic;
		sensor2_1       : in  std_logic;
		sensor3_1       : in  std_logic;
		-- xadc signals
		xadc_vp         : in  std_logic;
		xadc_vn         : in  std_logic;
		xadc_vaux0_p    : in  std_logic;
		xadc_vaux0_n    : in  std_logic;
		xadc_vaux8_p    : in  std_logic;
		xadc_vaux8_n    : in  std_logic;
		-- xadc external mux
		xadc_mux_a0     : out std_logic;
		xadc_mux_a1     : out std_logic;
		xadc_mux_en     : out std_logic;
		-- dip switches
		dip_sw          : in  std_logic_vector(3 downto 0);
		-- general purpose I/O
		gpio_i          : in  std_logic_vector(7 downto 0);
		gpio_o          : out std_logic_vector(7 downto 0);
		gpio_t          : out std_logic_vector(7 downto 0);
		--
		-- external angle in
		ext_x_angle_0   : in  std_logic_vector(31 downto 0);
		ext_x_angle_1   : in  std_logic_vector(31 downto 0);
		--
		-- resolver in
		rsv_x_priexc_0  : in  std_logic_vector(17 downto 0);
		rsv_x_secsin_0  : in  std_logic_vector(17 downto 0);
		rsv_x_seccos_0  : in  std_logic_vector(17 downto 0);
		--
		rsv_x_priexc_1  : in  std_logic_vector(17 downto 0);
		rsv_x_secsin_1  : in  std_logic_vector(17 downto 0);
		rsv_x_seccos_1  : in  std_logic_vector(17 downto 0);
		--
		-- motor #0 control IP

		-- external angle
		ext_angle_0     : out std_logic_vector(31 downto 0);
		
		-- resolver
		rsv_priexc_0    : out std_logic_vector(17 downto 0);
		rsv_secsin_0    : out std_logic_vector(17 downto 0);
		rsv_seccos_0    : out std_logic_vector(17 downto 0);
		
		-- hall sensor
		hls_hallsig_0   : out std_logic_vector(5 downto 0);
		
		-- digital encoder
		enc_cha_0       : out std_logic;
		enc_chb_0       : out std_logic;
		enc_chi_0       : out std_logic;
		
		-- input data
		--
		acq_iphs_a_0    : out std_logic_vector(17 downto 0);
		acq_iphs_b_0    : out std_logic_vector(17 downto 0);
		acq_iphs_c_0    : out std_logic_vector(17 downto 0);
		acq_ibus_x_0    : out std_logic_vector(17 downto 0);
		acq_vphs_a_0    : out std_logic_vector(17 downto 0);
		acq_vphs_b_0    : out std_logic_vector(17 downto 0);
		acq_vphs_c_0    : out std_logic_vector(17 downto 0);
		acq_vbus_x_0    : out std_logic_vector(17 downto 0);
		acq_vphs_n_0    : out std_logic_vector(17 downto 0);
		--
		acq_sync_0      : out std_logic;
		
		-- output offset
		--
		ofs_iphs_a_0    : in  std_logic_vector(17 downto 0);
		ofs_iphs_b_0    : in  std_logic_vector(17 downto 0);
		ofs_iphs_c_0    : in  std_logic_vector(17 downto 0);
		ofs_ibus_x_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_a_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_b_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_c_0    : in  std_logic_vector(17 downto 0);
		ofs_vbus_x_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_n_0    : in  std_logic_vector(17 downto 0);

		-- normalized values (offset removed)
		--
		nrm_iphs_a_0    : in  std_logic_vector(17 downto 0);
		nrm_iphs_b_0    : in  std_logic_vector(17 downto 0);
		nrm_iphs_c_0    : in  std_logic_vector(17 downto 0);
		nrm_ibus_x_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_a_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_b_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_c_0    : in  std_logic_vector(17 downto 0);
		nrm_vbus_x_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_n_0    : in  std_logic_vector(17 downto 0);
		--
		nrm_sync_0      : in  std_logic;

		-- filtered values
		--
		flt_iphs_a_0    : in  std_logic_vector(17 downto 0);
		flt_iphs_b_0    : in  std_logic_vector(17 downto 0);
		flt_iphs_c_0    : in  std_logic_vector(17 downto 0);
		flt_ibus_x_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_a_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_b_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_c_0    : in  std_logic_vector(17 downto 0);
		flt_vbus_x_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_n_0    : in  std_logic_vector(17 downto 0);
		--
		flt_sync_0      : in  std_logic;

		-- current overflow
		--
		ovi_iphs_a_0    : in  std_logic;
		ovi_iphs_b_0    : in  std_logic;
		ovi_iphs_c_0    : in  std_logic;
		ovi_ibus_x_0    : in  std_logic;

		-- Is overflow
		ismodover_0     : in  std_logic;
		
		-- 1=generator, 0=motor
		generator_0     : in  std_logic;
		
		-- modulator type
		modtype_0       : in  std_logic_vector(0 downto 0);
		
		-- modulator levels
		modlevels_0     : in  std_logic_vector(0 downto 0);
		
		-- motor is 3-phases (0=bipolar)
		is3phases_0     : in  std_logic;
		
		-- pwm 3-phases outputs
		pwm2p_l2c1w1_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w1_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c1w2_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w2_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2sync_0  : in  std_logic;
		
		-- pwm 2-phases outputs
		pwm3p_l2cxw1_0  : in  std_logic_vector(2 downto 0);
		pwm3p_l2cxw2_0  : in  std_logic_vector(2 downto 0);
		pwm3p_l2sync_0  : in  std_logic;
		
		-- rpfm 3-ph two levels outputs
		rpfm3p_l2cxw1_0 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2cxw2_0 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2sync_0 : in  std_logic;
		
		-- rpfm 3-ph three levels outputs
		rpfm3p_l3c1w1_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w1_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w1_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c1w2_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w2_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w2_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3sync_0 : in  std_logic;
		
		-- dead time value
		deadtval_0      : in  std_logic_vector(7 downto 0);
		
		-- gate unit switches
		guswcnt_0       : out std_logic_vector(31 downto 0);
		
		-- in-failure
		hwfail_0        : out std_logic;
		
		-- in-kill
		hwkill_0        : out std_logic;
		
		-- coil driver
		coilgear_0      : in  std_logic;
		coilenab_0      : in  std_logic;
		
		-- system error
		syserr_0        : in  std_logic;
		
		-- reset pending errors
		reseterr_0      : in  std_logic;
		
		-- zero current offset 
		zerocurr_0      : in  std_logic;
		
		-- zero vbus offset 
		zerovbusx_0     : in  std_logic;

		-- zero vphs offset 
		zerovphsx_0     : in  std_logic;
		
		-- control probes
		pix_setval_0    : in std_logic_vector(17 downto 0);
		piy_setval_0    : in std_logic_vector(17 downto 0);
		
		pix_fbkval_0    : in std_logic_vector(17 downto 0);
		piy_fbkval_0    : in std_logic_vector(17 downto 0);
		
		pix_outval_0    : in std_logic_vector(17 downto 0);
		piy_outval_0    : in std_logic_vector(17 downto 0);
		
		-- Vs
		vsmodval_0      : in  std_logic_vector(16 downto 0);
		vsangval_0      : in  std_logic_vector(31 downto 0);
		
		-- Is
		ismodval_0      : in  std_logic_vector(16 downto 0);
		isangval_0      : in  std_logic_vector(31 downto 0);

		-- smo
		smo_bemf_p_0    : in  std_logic_vector(31 downto 0);
		smo_bemf_m_0    : in  std_logic_vector(16 downto 0);
		
		-- e-angles
		mrt_angle_0     : in  std_logic_vector(31 downto 0);
		enc_angle_0     : in  std_logic_vector(31 downto 0);
		hls_angle_0     : in  std_logic_vector(31 downto 0);
		rsv_angle_0     : in  std_logic_vector(31 downto 0);
		smo_angle_0     : in  std_logic_vector(31 downto 0);
		
		-- Rotor angle
		rot_angle_0     : in  std_logic_vector(31 downto 0);
		
		-- rot selector
		rot_select_0    : in  std_logic_vector( 2 downto 0);
		
		-- actual speed
		spd_speed_0     : in  std_logic_vector(31 downto 0);
		
		-- position
		pos_position_0  : in  std_logic_vector(31 downto 0);
		
		-- foc cycle time
		tsclocks_0      : in  std_logic_vector(11 downto 0);
		
		-- motor #1 control IP

		-- external angle
		ext_angle_1     : out std_logic_vector(31 downto 0);
		
		-- resolver
		rsv_priexc_1    : out std_logic_vector(17 downto 0);
		rsv_secsin_1    : out std_logic_vector(17 downto 0);
		rsv_seccos_1    : out std_logic_vector(17 downto 0);
		
		-- hall sensor
		hls_hallsig_1   : out std_logic_vector(5 downto 0);
		
		-- digital encoder
		enc_cha_1       : out std_logic;
		enc_chb_1       : out std_logic;
		enc_chi_1       : out std_logic;
		
		-- input data
		--
		acq_iphs_a_1    : out std_logic_vector(17 downto 0);
		acq_iphs_b_1    : out std_logic_vector(17 downto 0);
		acq_iphs_c_1    : out std_logic_vector(17 downto 0);
		acq_ibus_x_1    : out std_logic_vector(17 downto 0);
		acq_vphs_a_1    : out std_logic_vector(17 downto 0);
		acq_vphs_b_1    : out std_logic_vector(17 downto 0);
		acq_vphs_c_1    : out std_logic_vector(17 downto 0);
		acq_vbus_x_1    : out std_logic_vector(17 downto 0);
		acq_vphs_n_1    : out std_logic_vector(17 downto 0);
		--
		acq_sync_1      : out std_logic;
		
		-- output offset
		--
		ofs_iphs_a_1    : in  std_logic_vector(17 downto 0);
		ofs_iphs_b_1    : in  std_logic_vector(17 downto 0);
		ofs_iphs_c_1    : in  std_logic_vector(17 downto 0);
		ofs_ibus_x_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_a_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_b_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_c_1    : in  std_logic_vector(17 downto 0);
		ofs_vbus_x_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_n_1    : in  std_logic_vector(17 downto 0);

		-- normalized values (offset removed)
		--
		nrm_iphs_a_1    : in  std_logic_vector(17 downto 0);
		nrm_iphs_b_1    : in  std_logic_vector(17 downto 0);
		nrm_iphs_c_1    : in  std_logic_vector(17 downto 0);
		nrm_ibus_x_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_a_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_b_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_c_1    : in  std_logic_vector(17 downto 0);
		nrm_vbus_x_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_n_1    : in  std_logic_vector(17 downto 0);
		--
		nrm_sync_1      : in  std_logic;

		-- filtered values
		--
		flt_iphs_a_1    : in  std_logic_vector(17 downto 0);
		flt_iphs_b_1    : in  std_logic_vector(17 downto 0);
		flt_iphs_c_1    : in  std_logic_vector(17 downto 0);
		flt_ibus_x_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_a_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_b_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_c_1    : in  std_logic_vector(17 downto 0);
		flt_vbus_x_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_n_1    : in  std_logic_vector(17 downto 0);
		--
		flt_sync_1      : in  std_logic;

		-- current overflow
		--
		ovi_iphs_a_1    : in  std_logic;
		ovi_iphs_b_1    : in  std_logic;
		ovi_iphs_c_1    : in  std_logic;
		ovi_ibus_x_1    : in  std_logic;

		-- Is overflow
		ismodover_1     : in  std_logic;
		
		-- 1=generator, 0=motor
		generator_1     : in  std_logic;
		
		-- modulator type
		modtype_1       : in  std_logic_vector(0 downto 0);
		
		-- modulator levels
		modlevels_1     : in  std_logic_vector(0 downto 0);
		
		-- motor is 3-phases (0=bipolar)
		is3phases_1     : in  std_logic;
		
		-- pwm 3-phases outputs
		pwm2p_l2c1w1_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w1_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c1w2_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w2_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2sync_1  : in  std_logic;
		
		-- pwm 2-phases outputs
		pwm3p_l2cxw1_1  : in  std_logic_vector(2 downto 0);
		pwm3p_l2cxw2_1  : in  std_logic_vector(2 downto 0);
		pwm3p_l2sync_1  : in  std_logic;
		
		-- rpfm 3-ph two levels outputs
		rpfm3p_l2cxw1_1 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2cxw2_1 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2sync_1 : in  std_logic;
		
		-- rpfm 3-ph three levels outputs
		rpfm3p_l3c1w1_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w1_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w1_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c1w2_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w2_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w2_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3sync_1 : in  std_logic;
		
		-- dead time value
		deadtval_1      : in  std_logic_vector(7 downto 0);
		
		-- gate unit switches
		guswcnt_1       : out std_logic_vector(31 downto 0);
		
		-- in-failure
		hwfail_1        : out std_logic;
		
		-- in-kill
		hwkill_1        : out std_logic;
		
		-- coil driver
		coilgear_1      : in  std_logic;
		coilenab_1      : in  std_logic;
		
		-- system error
		syserr_1        : in  std_logic;
		
		-- reset pending errors
		reseterr_1      : in  std_logic;
		
		-- zero current offset 
		zerocurr_1      : in  std_logic;
		
		-- zero vbus offset 
		zerovbusx_1     : in  std_logic;

		-- zero vphs offset 
		zerovphsx_1     : in  std_logic;
		
		-- control probes
		pix_setval_1    : in std_logic_vector(17 downto 0);
		piy_setval_1    : in std_logic_vector(17 downto 0);
		
		pix_fbkval_1    : in std_logic_vector(17 downto 0);
		piy_fbkval_1    : in std_logic_vector(17 downto 0);
		
		pix_outval_1    : in std_logic_vector(17 downto 0);
		piy_outval_1    : in std_logic_vector(17 downto 0);
		
		-- Vs
		vsmodval_1      : in  std_logic_vector(16 downto 0);
		vsangval_1      : in  std_logic_vector(31 downto 0);
		
		-- Is
		ismodval_1      : in  std_logic_vector(16 downto 0);
		isangval_1      : in  std_logic_vector(31 downto 0);

		-- smo
		smo_bemf_p_1    : in  std_logic_vector(31 downto 0);
		smo_bemf_m_1    : in  std_logic_vector(16 downto 0);
		
		-- e-angles
		mrt_angle_1     : in  std_logic_vector(31 downto 0);
		enc_angle_1     : in  std_logic_vector(31 downto 0);
		hls_angle_1     : in  std_logic_vector(31 downto 0);
		rsv_angle_1     : in  std_logic_vector(31 downto 0);
		smo_angle_1     : in  std_logic_vector(31 downto 0);
		
		-- Rotor angle
		rot_angle_1     : in  std_logic_vector(31 downto 0);
		
		-- rot selector
		rot_select_1    : in  std_logic_vector( 2 downto 0);
		
		-- actual speed
		spd_speed_1     : in  std_logic_vector(31 downto 0);
		
		-- position
		pos_position_1  : in  std_logic_vector(31 downto 0);
		
		-- foc cycle time
		tsclocks_1      : in  std_logic_vector(11 downto 0)
	);
end fmcmc1sys;

architecture Behavioral of fmcmc1sys is

	-- input processing: lock metastability
	component inlkmeta
        generic
        (
            C_NCLKS : integer := 3
        );
        port 
        (
            clock  : in  std_logic;
            datin  : in  std_logic;
            datout : out std_logic
        );
	end component;

	-- de glitch
	component unglitch
		generic
		(
			C_CLOCK_FREQ  : integer := 100_000_000;
			C_FILTER_TIME : real    := 1.0e-6
		);
		port
		(
			clock  : in  std_logic;
			reset  : in  std_logic;
			datin  : in  std_logic;
			datout : out std_logic
		);
	end component;
	
	-- s3 multi channel
	component s3acqmult
		generic
		(
			C_RESULT_WIDTH : integer := 16;
			C_NUM_CHANNELS : integer := 16
		);
		port 
		(
			clock   : in  std_logic;
			reset   : in  std_logic;
			--
			sclock  : in  std_logic;
			sdata   : in  std_logic_vector(C_NUM_CHANNELS-1 downto 0);
			--
			pdata0  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata1  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata2  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata3  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata4  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata5  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata6  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata7  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata8  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata9  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata10 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata11 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata12 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata13 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata14 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata15 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			finish  : out std_logic
		);
	end component;

	-- s/d 2-channel acquisition
	component s3acqimot
		port 
		(
			clock  : in  std_logic;
			reset  : in  std_logic;
			--
			sclock : in  std_logic;
			sdata1 : in  std_logic;
			sdata2 : in  std_logic;
			--
			modsyn : in  std_logic;
			--
			syncmd : in  std_logic;
			syntos : in  std_logic_vector(5 downto 0);
			--
			declim : in  std_logic_vector(5 downto 0);
			skewms : in  std_logic_vector(5 downto 0);
			--
			pdata1 : out std_logic_vector(17 downto 0);
			pdata2 : out std_logic_vector(17 downto 0);
			focsyn : out std_logic
		);
	end component;
	
	-- XADC acquisition module
	component mcxadc
		generic
		(
			C_SYSTEMCLOCK   : integer := 100_000_000
		);
		port
		(
			clock          : in  std_logic;
			reset          : in  std_logic;
			-- on-chip temperature
			val_temp       : out std_logic_vector(17 downto 0);
			-- M0/1 values
			val_ia_0       : out std_logic_vector(17 downto 0);
			val_ib_0       : out std_logic_vector(17 downto 0);
			val_vbus_0     : out std_logic_vector(17 downto 0);
			val_ia_1       : out std_logic_vector(17 downto 0);
			val_ib_1       : out std_logic_vector(17 downto 0);
			val_vbus_1     : out std_logic_vector(17 downto 0);
			-- sync
			val_sync       : out std_logic;
			-- xadc drp access
			xadc_drp_daddr : in  std_logic_vector(6 downto 0);
			xadc_drp_di    : in  std_logic_vector(15 downto 0);
			xadc_drp_den   : in  std_logic;
			xadc_drp_dwe   : in  std_logic;
			xadc_drp_do    : out std_logic_vector(15 downto 0);
			xadc_drp_drdy  : out std_logic;
			-- xadc analog signals
			xadc_vp        : in  std_logic;
			xadc_vn        : in  std_logic;
			xadc_vaux0_p   : in  std_logic;
			xadc_vaux0_n   : in  std_logic;
			xadc_vaux8_p   : in  std_logic;
			xadc_vaux8_n   : in  std_logic;
			-- xadc external mux
			xadc_mux_a0    : out std_logic;
			xadc_mux_a1    : out std_logic;
			-- sync
			-- clk div. to sync
			synclkdiv      : in  std_logic_vector(9 downto 0);
			-- values trigger
			syncreq        : in  std_logic;
			syncout        : out std_logic;
			syncack        : out std_logic
		);
	end component;
	
	-- lpf1
	component filtiir
		generic
		(
			C_FAMILY : string := "spartan3adsp"
		);
		port 
		( 
			clock  : in  std_logic;
			reset  : in  std_logic;
			
			start  : in  std_logic;
			fktau  : in  std_logic_vector(16 downto 0);
			inraw  : in  std_logic_vector(17 downto 0);
			outflt : out std_logic_vector(17 downto 0);
			finish : out std_logic
		);
	end component;

	-- event counter
	component evxcount
		generic
		(
			C_DATA_WIDTH  : integer := 16;
			C_EVENT_WIDTH : integer := 8;
			C_COUNT_MODE  : integer := 0
		);
		port
		(
			clock   : in  std_logic;
			reset   : in  std_logic;
			sigevt  : in  std_logic_vector(C_EVENT_WIDTH-1 downto 0);
			counter : out std_logic_vector(C_DATA_WIDTH-1 downto 0)
		);
	end component;

	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	-- modulator type
	constant MODTYPE_PWM  : std_logic_vector(0 downto 0) := "0";
	constant MODTYPE_RPFM : std_logic_vector(0 downto 0) := "1";
	
	-- modulator levels
	constant MODLEVELS_2  : std_logic_vector(0 downto 0) := "0";
	constant MODLEVELS_3  : std_logic_vector(0 downto 0) := "1";
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------

	-- digital input handling for anti-metastability
	
	-- M0
	signal lk_ads_ia_0    : std_logic;
	signal lk_ads_ib_0    : std_logic;
	--
	signal lk_drv_fault_0 : std_logic;
	signal lk_drv_otw_0   : std_logic;
	--
	signal lk_sensor1_0   : std_logic;
	signal lk_sensor2_0   : std_logic;
	signal lk_sensor3_0   : std_logic;
	
	-- M1
	signal lk_ads_ia_1    : std_logic;
	signal lk_ads_ib_1    : std_logic;
	--
	signal lk_drv_fault_1 : std_logic;
	signal lk_drv_otw_1   : std_logic;
	--
	signal lk_sensor1_1   : std_logic;
	signal lk_sensor2_1   : std_logic;
	signal lk_sensor3_1   : std_logic;
	
	---------------------------------------------------------------------------

	-- digital input handling for de-glitch
	
	-- M0
	signal dg_drv_fault_0 : std_logic;
	signal dg_drv_otw_0   : std_logic;
	--
	signal dg_sensor1_0   : std_logic;
	signal dg_sensor2_0   : std_logic;
	signal dg_sensor3_0   : std_logic;
	
	-- M1
	signal dg_drv_fault_1 : std_logic;
	signal dg_drv_otw_1   : std_logic;
	--
	signal dg_sensor1_1   : std_logic;
	signal dg_sensor2_1   : std_logic;
	signal dg_sensor3_1   : std_logic;
	
	---------------------------------------------------------------------------

	-- de-glitch for inputs
	constant C_DEGLITCH_TIME : real := 1.0e-6;
	
	---------------------------------------------------------------------------
	
	-- param for sinc3 and sync dim
	constant C_SYNTOS_WIDTH : integer := 6;
	constant C_DECLIM_WIDTH : integer := 6;
	constant C_SKEWMS_WIDTH : integer := 6;
	
	-- modulator sync for acq.
	
	signal modsyn_0 : std_logic;
	signal modsyn_1 : std_logic;
	
	-- s/d acq & sync param
	
	-- M0
	signal syncmd_0 : std_logic;
	signal syntos_0 : std_logic_vector(C_SYNTOS_WIDTH-1 downto 0);
	signal declim_0 : std_logic_vector(C_DECLIM_WIDTH-1 downto 0);
	signal skewms_0 : std_logic_vector(C_SKEWMS_WIDTH-1 downto 0);
	
	-- M1
	signal syncmd_1 : std_logic;
	signal syntos_1 : std_logic_vector(C_SYNTOS_WIDTH-1 downto 0);
	signal declim_1 : std_logic_vector(C_DECLIM_WIDTH-1 downto 0);
	signal skewms_1 : std_logic_vector(C_SKEWMS_WIDTH-1 downto 0);
	
	-- M0/M1 S/D control registers
	constant C_SYNCMD_BITOFS : integer := 0;
	constant C_SYNTOS_BITOFS : integer := 1;
	constant C_DECLIM_BITOFS : integer := 7;
	constant C_SKEWMS_BITOFS : integer := 13;
	constant C_SD_CTRL_WIDTH : integer := 19;
	signal sd_ctrl_reg_0 : std_logic_vector(C_SD_CTRL_WIDTH-1 downto 0);
	signal sd_ctrl_reg_1 : std_logic_vector(C_SD_CTRL_WIDTH-1 downto 0);
	
	constant I_SD_CTRL_REG : integer := ((64-1) * 2 ** C_DECLIM_BITOFS) + ((64/2-1) * 2 ** C_SKEWMS_BITOFS);
	constant K_SD_CTRL_REG : std_logic_vector(C_SD_CTRL_WIDTH-1 downto 0) := std_logic_vector(to_unsigned(I_SD_CTRL_REG,C_SD_CTRL_WIDTH));
	
	---------------------------------------------------------------------------
	-- xadc work regs ---------------------------------------------------------
	
	-- clk div. to sync
	signal xa_synclkdiv    : std_logic_vector(9 downto 0);
	-- on-chip temperature
	signal xa_val_temp     : std_logic_vector(17 downto 0);
	-- m0 adj values
	signal xa_val_ia_0     : std_logic_vector(17 downto 0);
	signal xa_val_ib_0     : std_logic_vector(17 downto 0);
	signal xa_val_vbus_0   : std_logic_vector(17 downto 0);
	-- m1 adj values
	signal xa_val_ia_1     : std_logic_vector(17 downto 0);
	signal xa_val_ib_1     : std_logic_vector(17 downto 0);
	signal xa_val_vbus_1   : std_logic_vector(17 downto 0);
	-- sync
	signal xa_val_sync     : std_logic;
	-- values trigger
	signal xa_syncreq      : std_logic;
	signal xa_syncout      : std_logic;
	signal xa_syncack      : std_logic;
	-- xadc reset
	signal xadc_reset      : std_logic;
	-- xadc drp access
	signal xadc_drp_daddr : std_logic_vector(6 downto 0);
	signal xadc_drp_di    : std_logic_vector(15 downto 0);
	signal xadc_drp_den   : std_logic;
	signal xadc_drp_dwe   : std_logic;
	signal xadc_drp_do    : std_logic_vector(15 downto 0);
	signal xadc_drp_drdy  : std_logic;
	
	---------------------------------------------------------------------------

	-- xa normalized aft uni/bip
	
	signal xa_bip_ia_0     : std_logic_vector(17 downto 0);
	signal xa_bip_ib_0     : std_logic_vector(17 downto 0);
	signal xa_bip_vbus_0   : std_logic_vector(17 downto 0);
	
	signal xa_bip_ia_1     : std_logic_vector(17 downto 0);
	signal xa_bip_ib_1     : std_logic_vector(17 downto 0);
	signal xa_bip_vbus_1   : std_logic_vector(17 downto 0);
	
	---------------------------------------------------------------------------
	
	-- xadc write ctrl
	signal xadc_wctreg     : std_logic_vector(31 downto 0);
	signal xadc_rctreg     : std_logic_vector(31 downto 0);

	type   stdrp_type is ( idle, w4rdy );
	signal stdrp : stdrp_type;
	
	---------------------------------------------------------------------------
	
	-- data packed
	signal ads_sdata : std_logic_vector(0 downto 0);
	
	constant C_SD_VBUS_WIDTH : integer := 12;
	
	signal sd_vbus_raw : std_logic_vector(C_SD_VBUS_WIDTH-1 downto 0);
	
	---------------------------------------------------------------------------
	
	-- s/d output
	
	signal sd_val_ia_0   : std_logic_vector(17 downto 0);
	signal sd_val_ib_0   : std_logic_vector(17 downto 0);
	signal sd_val_vbus_0 : std_logic_vector(17 downto 0);
	signal sd_val_isyn_0 : std_logic;
	
	signal sd_val_ia_1   : std_logic_vector(17 downto 0);
	signal sd_val_ib_1   : std_logic_vector(17 downto 0);
	signal sd_val_vbus_1 : std_logic_vector(17 downto 0);
	signal sd_val_isyn_1 : std_logic;
	
	---------------------------------------------------------------------------

	-- sd normalized aft uni/bip
	
	signal sd_bip_ia_0     : std_logic_vector(17 downto 0);
	signal sd_bip_ib_0     : std_logic_vector(17 downto 0);
	signal sd_bip_vbus_0   : std_logic_vector(17 downto 0);
	
	signal sd_bip_ia_1     : std_logic_vector(17 downto 0);
	signal sd_bip_ib_1     : std_logic_vector(17 downto 0);
	signal sd_bip_vbus_1   : std_logic_vector(17 downto 0);

	---------------------------------------------------------------------------
	-- xadc mux selector
	
	signal xa_iab_0        : std_logic;
	signal xa_vbus_0       : std_logic;
	
	signal xa_iab_1        : std_logic;
	signal xa_vbus_1       : std_logic;
	
	---------------------------------------------------------------------------
	
	-- aft mux
	
	signal mux_iphs_a_0 : std_logic_vector(17 downto 0);
	signal mux_iphs_b_0 : std_logic_vector(17 downto 0);
	signal mux_vbus_x_0 : std_logic_vector(17 downto 0);

	signal mux_iphs_a_1 : std_logic_vector(17 downto 0);
	signal mux_iphs_b_1 : std_logic_vector(17 downto 0);
	signal mux_vbus_x_1 : std_logic_vector(17 downto 0);

	signal mux_sync_0   : std_logic;
	signal mux_sync_1   : std_logic;
	
	---------------------------------------------------------------------------
	-- filter K ---------------------------------------------------------------
	
	constant K_FILTER_FAST  : std_logic_vector(16 downto 0) := (others => '1');
	
	signal oc_temp_fkt   : std_logic_vector(16 downto 0);
	
	---------------------------------------------------------------------------
	-- filtered values --------------------------------------------------------
	
	signal oc_temp_val   : std_logic_vector(17 downto 0);
	
	---------------------------------------------------------------------------
	constant C_WCTREG_DATA_LO   : integer :=  0;
	constant C_WCTREG_DATA_HI   : integer := 15;
	constant C_WCTREG_ADDR_LO   : integer := 16;
	constant C_WCTREG_ADDR_HI   : integer := 22;
	constant C_WCTREG_DRP_EN    : integer := 23;
	constant C_WCTREG_DRP_WE    : integer := 24;
	constant C_WCTREG_DRP_DC    : integer := 25;
	constant C_WCTREG_SYNC      : integer := 26;
	constant C_WCTREG_XA_RESET  : integer := 27;
	constant C_WCTREG_XA_VBUS_0 : integer := 28;
	constant C_WCTREG_XA_IABS_0 : integer := 29;
	constant C_WCTREG_XA_VBUS_1 : integer := 30;
	constant C_WCTREG_XA_IABS_1 : integer := 31;
	---------------------------------------------------------------------------
	constant C_XA_SYNCTIME  : integer := integer(real(C_SYSTEMCLOCK) * 3.2e-6);
	constant K_XA_SYNCLKDIV : std_logic_vector(9 downto 0) := std_logic_vector(to_unsigned(C_XA_SYNCTIME - 1,10));
	---------------------------------------------------------------------------
	
	-- motor driver arbiter: set the owner
	signal mot_drv_0_owner  : std_logic;
	signal mot_drv_1_owner  : std_logic;
	
	-- gate unit failure
	signal gu_fault_0 : std_logic;
	signal gu_fault_1 : std_logic;
	
	---------------------------------------------------------------------------
	
	-- coil driver
	signal coildrv           : std_logic_vector(11 downto 0);
	
	constant DEF_DRV_pwm_a_0 : integer := 0;
	constant DEF_DRV_pwm_b_0 : integer := 1;
	constant DEF_DRV_pwm_c_0 : integer := 2;

	constant DEF_DRV_rst_a_0 : integer := 3;
	constant DEF_DRV_rst_b_0 : integer := 4;
	constant DEF_DRV_rst_c_0 : integer := 5;

	constant DEF_DRV_pwm_a_1 : integer := 6;
	constant DEF_DRV_pwm_b_1 : integer := 7;
	constant DEF_DRV_pwm_c_1 : integer := 8;

	constant DEF_DRV_rst_a_1 : integer := 9;
	constant DEF_DRV_rst_b_1 : integer := 10;
	constant DEF_DRV_rst_c_1 : integer := 11;
	
	---------------------------------------------------------------------------
	
	-- gate unit switches counters
	
	-- event captures
	signal guevsrc_0    : std_logic_vector(2 downto 0);
	signal guevsrc_1    : std_logic_vector(2 downto 0);
	
	-- switches counter per leg
	signal guswcnt_leg_0 : std_logic_vector(30 downto 0);
	signal guswcnt_leg_1 : std_logic_vector(30 downto 0);
	
	---------------------------------------------------------------------------
	
	-- status register
	signal status_reg  : std_logic_vector(3 downto 0);
	-- M0
	constant DEF_STATUS_REG_DRV_FAULT_0  : integer := 0;
	constant DEF_STATUS_REG_DRV_OTW_0    : integer := 1;

	-- M1
	constant DEF_STATUS_REG_DRV_FAULT_1  : integer := 2;
	constant DEF_STATUS_REG_DRV_OTW_1    : integer := 3;

	---------------------------------------------------------------------------
	
	-- access delay
	signal host_en_dl1  : std_logic;
	
	---------------------------------------------------------------------------

	-- bus2reg
	function loadreg(s : std_logic_vector) return std_logic_vector is
		variable r : std_logic_vector(31 downto 0);
	begin
		r := (others => '0');
		r(s'range) := s;
		return r;
	end function;

	-- reg2bus
	procedure storereg(signal d : out std_logic_vector; signal s : in std_logic_vector) is
	begin
		d <= s(d'range);
	end procedure;

	---------------------------------------------------------------------------
	
  	constant RA_ip_ident        : integer := (4*0);
	constant RA_status_reg      : integer := (4*1);

	constant RA_sd_ctrl_0       : integer := (4*2);
	constant RA_sd_ctrl_1       : integer := (4*3);

	constant RA_oc_temp_val     : integer := (4*4);
	constant RA_oc_temp_fkt     : integer := (4*5);

	constant RA_xa_synclkdiv    : integer := (4*6);

	constant RA_xadc_wctreg     : integer := (4*7);
	constant RA_xadc_rctreg     : integer := (4*8);

	constant RA_dip_sw          : integer := (4*9);

	constant RA_gpio_i          : integer := (4*10);
	constant RA_gpio_o          : integer := (4*11);
	constant RA_gpio_t          : integer := (4*12);
	
	---------------------------------------------------------------------------
	
	constant C_DECAD_WIDTH : integer := 4;
	
	---------------------------------------------------------------------------
	
	function adrtoint(ad : std_logic_vector) return integer is
		variable ar : std_logic_vector(C_DECAD_WIDTH+1 downto 0);
	begin
		ar := ad(ar'left downto 2) & "00";
		return to_integer(unsigned(ar));
	end function;
	
	---------------------------------------------------------------------------

	-- ip identification
	constant ip_id_code  : integer := 7;
	constant ip_id_majv  : integer := 1;
	constant ip_id_minv  : integer := 1;
	constant ip_id_hscr  : integer := 97;
	
	constant ip_ident_k : std_logic_vector(31 downto 0) :=
		std_logic_vector(to_unsigned(ip_id_code,8)) &
		std_logic_vector(to_unsigned(ip_id_majv,8)) &
		std_logic_vector(to_unsigned(ip_id_minv,8)) &
		std_logic_vector(to_unsigned(ip_id_hscr,8));


	-- outputs
	
	-- host bus
	signal host_dout_i  : std_logic_vector(31 downto 0);
	signal host_ack_i   : std_logic;
	
	
	-- motor board i/f
	
	signal ads_clkin_0_i   : std_logic;
	signal drv_pwm_a_0_i   : std_logic;
	signal drv_pwm_b_0_i   : std_logic;
	signal drv_pwm_c_0_i   : std_logic;
	signal drv_rst_a_0_i   : std_logic;
	signal drv_rst_b_0_i   : std_logic;
	signal drv_rst_c_0_i   : std_logic;
	
	signal ads_clkin_1_i   : std_logic;
	signal drv_pwm_a_1_i   : std_logic;
	signal drv_pwm_b_1_i   : std_logic;
	signal drv_pwm_c_1_i   : std_logic;
	signal drv_rst_a_1_i   : std_logic;
	signal drv_rst_b_1_i   : std_logic;
	signal drv_rst_c_1_i   : std_logic;
	
	signal xadc_mux_a0_i   : std_logic;
	signal xadc_mux_a1_i   : std_logic;
	signal xadc_mux_en_i   : std_logic;
	
	signal gpio_o_i        : std_logic_vector(7 downto 0);
	signal gpio_t_i        : std_logic_vector(7 downto 0);
	

	---------------------------------------------------------------------------
	
	-- motor #0 control
	
	-- external angle
	signal ext_angle_0_i   : std_logic_vector(31 downto 0);

	-- resolver            
	signal rsv_priexc_0_i  : std_logic_vector(17 downto 0);
	signal rsv_secsin_0_i  : std_logic_vector(17 downto 0);
	signal rsv_seccos_0_i  : std_logic_vector(17 downto 0);

	-- hall sensor         
	signal hls_hallsig_0_i : std_logic_vector(5 downto 0);

	-- digital encoder     
	signal enc_cha_0_i     : std_logic;
	signal enc_chb_0_i     : std_logic;
	signal enc_chi_0_i     : std_logic;

	-- input data          
	--                     
	signal acq_iphs_a_0_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_b_0_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_c_0_i  : std_logic_vector(17 downto 0);
	signal acq_ibus_x_0_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_a_0_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_b_0_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_c_0_i  : std_logic_vector(17 downto 0);
	signal acq_vbus_x_0_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_n_0_i  : std_logic_vector(17 downto 0);
	--                     
	signal acq_sync_0_i    : std_logic;

	-- gate unit switches   
	signal guswcnt_0_i     : std_logic_vector(31 downto 0);

	-- in-failure          
	signal hwfail_0_i      : std_logic;

	-- in-kill             
	signal hwkill_0_i      : std_logic;

	---------------------------------------------------------------------------
	
	-- motor #1 control
	
	-- external angle
	signal ext_angle_1_i   : std_logic_vector(31 downto 0);

	-- resolver            
	signal rsv_priexc_1_i  : std_logic_vector(17 downto 0);
	signal rsv_secsin_1_i  : std_logic_vector(17 downto 0);
	signal rsv_seccos_1_i  : std_logic_vector(17 downto 0);

	-- hall sensor         
	signal hls_hallsig_1_i : std_logic_vector(5 downto 0);

	-- digital encoder     
	signal enc_cha_1_i     : std_logic;
	signal enc_chb_1_i     : std_logic;
	signal enc_chi_1_i     : std_logic;

	-- input data          
	--                     
	signal acq_iphs_a_1_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_b_1_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_c_1_i  : std_logic_vector(17 downto 0);
	signal acq_ibus_x_1_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_a_1_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_b_1_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_c_1_i  : std_logic_vector(17 downto 0);
	signal acq_vbus_x_1_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_n_1_i  : std_logic_vector(17 downto 0);
	--                     
	signal acq_sync_1_i    : std_logic;

	-- gate unit switches   
	signal guswcnt_1_i     : std_logic_vector(31 downto 0);

	-- in-failure          
	signal hwfail_1_i      : std_logic;

	-- in-kill             
	signal hwkill_1_i      : std_logic;

	---------------------------------------------------------------------------

begin

	---------------------------------------------------------------------------
	
	-- fwd output

	-- host bus
	host_dout     <= host_dout_i;
	host_ack      <= host_ack_i;
	
	
	-- motor board i/f
	
	ads_clkin_0   <= ads_clkin_0_i;
	drv_pwm_a_0   <= drv_pwm_a_0_i;
	drv_pwm_b_0   <= drv_pwm_b_0_i;
	drv_pwm_c_0   <= drv_pwm_c_0_i;
	drv_rst_a_0   <= drv_rst_a_0_i;
	drv_rst_b_0   <= drv_rst_b_0_i;
	drv_rst_c_0   <= drv_rst_c_0_i;

	ads_clkin_1   <= ads_clkin_1_i;
	drv_pwm_a_1   <= drv_pwm_a_1_i;
	drv_pwm_b_1   <= drv_pwm_b_1_i;
	drv_pwm_c_1   <= drv_pwm_c_1_i;
	drv_rst_a_1   <= drv_rst_a_1_i;
	drv_rst_b_1   <= drv_rst_b_1_i;
	drv_rst_c_1   <= drv_rst_c_1_i;

	xadc_mux_a0   <= xadc_mux_a0_i;
	xadc_mux_a1   <= xadc_mux_a1_i;
	xadc_mux_en   <= xadc_mux_en_i;

	gpio_o        <= gpio_o_i;
	gpio_t        <= gpio_t_i;

	
	---------------------------------------------------------------------------
	
	-- motor #0 control
	
	-- external angle
	ext_angle_0    <=  ext_angle_0_i;
	
	-- resolver
	rsv_priexc_0   <=  rsv_priexc_0_i;
	rsv_secsin_0   <=  rsv_secsin_0_i;
	rsv_seccos_0   <=  rsv_seccos_0_i;
	
	-- hall sensor
	hls_hallsig_0  <=  hls_hallsig_0_i;
	
	-- digital encoder
	enc_cha_0      <=  enc_cha_0_i;
	enc_chb_0      <=  enc_chb_0_i;
	enc_chi_0      <=  enc_chi_0_i;
	
	-- input data
	--
	acq_iphs_a_0   <=  acq_iphs_a_0_i;
	acq_iphs_b_0   <=  acq_iphs_b_0_i;
	acq_iphs_c_0   <=  acq_iphs_c_0_i;
	acq_ibus_x_0   <=  acq_ibus_x_0_i;
	acq_vphs_a_0   <=  acq_vphs_a_0_i;
	acq_vphs_b_0   <=  acq_vphs_b_0_i;
	acq_vphs_c_0   <=  acq_vphs_c_0_i;
	acq_vbus_x_0   <=  acq_vbus_x_0_i;
	acq_vphs_n_0   <=  acq_vphs_n_0_i;
	--
	acq_sync_0     <=  acq_sync_0_i;
	
	-- gate unit switches
	guswcnt_0      <=  guswcnt_0_i;
	
	-- in-failure
	hwfail_0       <=  hwfail_0_i;
	
	-- in-kill
	hwkill_0       <=  hwkill_0_i;
	
	---------------------------------------------------------------------------
	
	-- motor #1 control
	
	-- external angle
	ext_angle_1    <=  ext_angle_1_i;
	
	-- resolver
	rsv_priexc_1   <=  rsv_priexc_1_i;
	rsv_secsin_1   <=  rsv_secsin_1_i;
	rsv_seccos_1   <=  rsv_seccos_1_i;
	
	-- hall sensor
	hls_hallsig_1  <=  hls_hallsig_1_i;
	
	-- digital encoder
	enc_cha_1      <=  enc_cha_1_i;
	enc_chb_1      <=  enc_chb_1_i;
	enc_chi_1      <=  enc_chi_1_i;
	
	-- input data
	--
	acq_iphs_a_1   <=  acq_iphs_a_1_i;
	acq_iphs_b_1   <=  acq_iphs_b_1_i;
	acq_iphs_c_1   <=  acq_iphs_c_1_i;
	acq_ibus_x_1   <=  acq_ibus_x_1_i;
	acq_vphs_a_1   <=  acq_vphs_a_1_i;
	acq_vphs_b_1   <=  acq_vphs_b_1_i;
	acq_vphs_c_1   <=  acq_vphs_c_1_i;
	acq_vbus_x_1   <=  acq_vbus_x_1_i;
	acq_vphs_n_1   <=  acq_vphs_n_1_i;
	--
	acq_sync_1     <=  acq_sync_1_i;
	
	-- gate unit switches
	guswcnt_1      <=  guswcnt_1_i;
	
	-- in-failure
	hwfail_1       <=  hwfail_1_i;
	
	-- in-kill
	hwkill_1       <=  hwkill_1_i;
	
	---------------------------------------------------------------------------
	
	---------------------------------------------------------------------------

	-- anti-metastability
	
	-- M0

	LK_M0_ADS_IA    : inlkmeta generic map ( C_NCLKS => 2 ) port map ( clock => ads_clkout_0, datin => ads_ia_0, datout => lk_ads_ia_0   );
	LK_M0_ADS_IB    : inlkmeta generic map ( C_NCLKS => 2 ) port map ( clock => ads_clkout_0, datin => ads_ib_0, datout => lk_ads_ib_0   );
	
	LK_M0_DRV_FAULT : inlkmeta generic map ( C_NCLKS => 1 ) port map ( clock => clock, datin => drv_fault_0    , datout => lk_drv_fault_0);
	LK_M0_DRV_OTW   : inlkmeta generic map ( C_NCLKS => 1 ) port map ( clock => clock, datin => drv_otw_0      , datout => lk_drv_otw_0  );
																																		 
	LK_M0_SENSOR1   : inlkmeta generic map ( C_NCLKS => 2 ) port map ( clock => clock, datin => sensor1_0      , datout => lk_sensor1_0  );
	LK_M0_SENSOR2   : inlkmeta generic map ( C_NCLKS => 2 ) port map ( clock => clock, datin => sensor2_0      , datout => lk_sensor2_0  );
	LK_M0_SENSOR3   : inlkmeta generic map ( C_NCLKS => 2 ) port map ( clock => clock, datin => sensor3_0      , datout => lk_sensor3_0  );

	-- M1

	LK_M1_ADS_IA    : inlkmeta generic map ( C_NCLKS => 2 ) port map ( clock => ads_clkout_1, datin => ads_ia_1, datout => lk_ads_ia_1   );
	LK_M1_ADS_IB    : inlkmeta generic map ( C_NCLKS => 2 ) port map ( clock => ads_clkout_1, datin => ads_ib_1, datout => lk_ads_ib_1   );

	LK_M1_DRV_FAULT : inlkmeta generic map ( C_NCLKS => 1 ) port map ( clock => clock, datin => drv_fault_1    , datout => lk_drv_fault_1);
	LK_M1_DRV_OTW   : inlkmeta generic map ( C_NCLKS => 1 ) port map ( clock => clock, datin => drv_otw_1      , datout => lk_drv_otw_1  );
																																		 
	LK_M1_SENSOR1   : inlkmeta generic map ( C_NCLKS => 2 ) port map ( clock => clock, datin => sensor1_1      , datout => lk_sensor1_1  );
	LK_M1_SENSOR2   : inlkmeta generic map ( C_NCLKS => 2 ) port map ( clock => clock, datin => sensor2_1      , datout => lk_sensor2_1  );
	LK_M1_SENSOR3   : inlkmeta generic map ( C_NCLKS => 2 ) port map ( clock => clock, datin => sensor3_1      , datout => lk_sensor3_1  );

	---------------------------------------------------------------------------

	-- M0 deglitch
    
	DG_M0_DRV_FAULT : unglitch
		generic map
		(
			C_CLOCK_FREQ  => C_SYSTEMCLOCK,
			C_FILTER_TIME => C_DEGLITCH_TIME
		)
		port map
		(
			clock  => clock,
			reset  => reset,
			datin  => lk_drv_fault_0,
			datout => dg_drv_fault_0
		);

    DG_M0_DRV_OTW   : unglitch
		generic map
		(
			C_CLOCK_FREQ  => C_SYSTEMCLOCK,
			C_FILTER_TIME => C_DEGLITCH_TIME
		)
		port map
		(
			clock  => clock,
			reset  => reset,
			datin  => lk_drv_otw_0,
			datout => dg_drv_otw_0
		);

	DG_M0_SENSOR1   : unglitch
		generic map
		(
			C_CLOCK_FREQ  => C_SYSTEMCLOCK,
			C_FILTER_TIME => C_DEGLITCH_TIME
		)
		port map
		(
			clock  => clock,
			reset  => reset,
			datin  => lk_sensor1_0,
			datout => dg_sensor1_0
		);
	
	DG_M0_SENSOR2   : unglitch
		generic map
		(
			C_CLOCK_FREQ  => C_SYSTEMCLOCK,
			C_FILTER_TIME => C_DEGLITCH_TIME
		)
		port map
		(
			clock  => clock,
			reset  => reset,
			datin  => lk_sensor2_0,
			datout => dg_sensor2_0
		);
	
	DG_M0_SENSOR3   : unglitch
		generic map
		(
			C_CLOCK_FREQ  => C_SYSTEMCLOCK,
			C_FILTER_TIME => C_DEGLITCH_TIME
		)
		port map
		(
			clock  => clock,
			reset  => reset,
			datin  => lk_sensor3_0,
			datout => dg_sensor3_0
		);

	---------------------------------------------------------------------------

	-- M1 deglitch
    
	DG_M1_DRV_FAULT : unglitch
		generic map
		(
			C_CLOCK_FREQ  => C_SYSTEMCLOCK,
			C_FILTER_TIME => C_DEGLITCH_TIME
		)
		port map
		(
			clock  => clock,
			reset  => reset,
			datin  => lk_drv_fault_1,
			datout => dg_drv_fault_1
		);

	DG_M1_DRV_OTW   : unglitch
		generic map
		(
			C_CLOCK_FREQ  => C_SYSTEMCLOCK,
			C_FILTER_TIME => C_DEGLITCH_TIME
		)
		port map
		(
			clock  => clock,
			reset  => reset,
			datin  => lk_drv_otw_1,
			datout => dg_drv_otw_1
		);
	
	DG_M1_SENSOR1   : unglitch
		generic map
		(
			C_CLOCK_FREQ  => C_SYSTEMCLOCK,
			C_FILTER_TIME => C_DEGLITCH_TIME
		)
		port map
		(
			clock  => clock,
			reset  => reset,
			datin  => lk_sensor1_1,
			datout => dg_sensor1_1
		);
	
	DG_M1_SENSOR2   : unglitch
		generic map
		(
			C_CLOCK_FREQ  => C_SYSTEMCLOCK,
			C_FILTER_TIME => C_DEGLITCH_TIME
		)
		port map
		(
			clock  => clock,
			reset  => reset,
			datin  => lk_sensor2_1,
			datout => dg_sensor2_1
		);
	
	DG_M1_SENSOR3   : unglitch
		generic map
		(
			C_CLOCK_FREQ  => C_SYSTEMCLOCK,
			C_FILTER_TIME => C_DEGLITCH_TIME
		)
		port map
		(
			clock  => clock,
			reset  => reset,
			datin  => lk_sensor3_1,
			datout => dg_sensor3_1
		);

	---------------------------------------------------------------------------
    
	-- motor sensor signals
	
	-- external angle: 0/1
	ext_angle_0_i <= ext_x_angle_0;
	ext_angle_1_i <= ext_x_angle_1;
	
	-- M0
	rsv_priexc_0_i <= rsv_x_priexc_0;
	rsv_secsin_0_i <= rsv_x_secsin_0;
	rsv_seccos_0_i <= rsv_x_seccos_0;

	-- M1
	rsv_priexc_1_i <= rsv_x_priexc_1;
	rsv_secsin_1_i <= rsv_x_secsin_1;
	rsv_seccos_1_i <= rsv_x_seccos_1;
	
	-- M0
	hls_hallsig_0_i(0) <= dg_sensor1_0;
	hls_hallsig_0_i(1) <= dg_sensor2_0;
	hls_hallsig_0_i(2) <= dg_sensor3_0;
	hls_hallsig_0_i(5 downto 3) <= (others => '0');
	-- M1
	hls_hallsig_1_i(0) <= dg_sensor1_1;
	hls_hallsig_1_i(1) <= dg_sensor2_1;
	hls_hallsig_1_i(2) <= dg_sensor3_1;
	hls_hallsig_1_i(5 downto 3) <= (others => '0');
	
	-- M0
	enc_chi_0_i <= dg_sensor1_0;
	enc_cha_0_i <= dg_sensor2_0;
	enc_chb_0_i <= dg_sensor3_0;
	
	-- M1
	enc_chi_1_i <= dg_sensor1_1;
	enc_cha_1_i <= dg_sensor2_1;
	enc_chb_1_i <= dg_sensor3_1;
	
	---------------------------------------------------------------------------

	-- clkin to ADS1204 (just a wire !)
	ads_clkin_0_i <= ads_clock;
	ads_clkin_1_i <= ads_clock;
	
	---------------------------------------------------------------------------
	-- s/d control register split
	-- M0
	syncmd_0 <= sd_ctrl_reg_0(C_SYNCMD_BITOFS);
	syntos_0 <= sd_ctrl_reg_0(C_SYNTOS_BITOFS + C_SYNTOS_WIDTH - 1 downto C_SYNTOS_BITOFS);
	declim_0 <= sd_ctrl_reg_0(C_DECLIM_BITOFS + C_DECLIM_WIDTH - 1 downto C_DECLIM_BITOFS);
	skewms_0 <= sd_ctrl_reg_0(C_SKEWMS_BITOFS + C_SKEWMS_WIDTH - 1 downto C_SKEWMS_BITOFS);
	-- M1
	syncmd_1 <= sd_ctrl_reg_1(C_SYNCMD_BITOFS);
	syntos_1 <= sd_ctrl_reg_1(C_SYNTOS_BITOFS + C_SYNTOS_WIDTH - 1 downto C_SYNTOS_BITOFS);
	declim_1 <= sd_ctrl_reg_1(C_DECLIM_BITOFS + C_DECLIM_WIDTH - 1 downto C_DECLIM_BITOFS);
	skewms_1 <= sd_ctrl_reg_1(C_SKEWMS_BITOFS + C_SKEWMS_WIDTH - 1 downto C_SKEWMS_BITOFS);
	
	---------------------------------------------------------------------------
	-- modulator sync for acq
	
	modsyn_0 <=
				rpfm3p_l2sync_0 when modtype_0 = MODTYPE_RPFM and is3phases_0 = '1' else
				pwm3p_l2sync_0  when modtype_0 = MODTYPE_PWM  and is3phases_0 = '1' else
				pwm2p_l2sync_0  when modtype_0 = MODTYPE_PWM  and is3phases_0 = '0' else '0';
				
	modsyn_1 <=
				rpfm3p_l2sync_1 when modtype_1 = MODTYPE_RPFM and is3phases_1 = '1' else
				pwm3p_l2sync_1  when modtype_1 = MODTYPE_PWM  and is3phases_1 = '1' else
				pwm2p_l2sync_1  when modtype_1 = MODTYPE_PWM  and is3phases_1 = '0' else '0';
				
	---------------------------------------------------------------------------
	
	ads_sdata(0) <= ads_vbus_0;
	
	-- s3 multi channel
	SD_ACQ_VBUS_INST : s3acqmult
		generic map
		(
			C_RESULT_WIDTH => C_SD_VBUS_WIDTH,
			C_NUM_CHANNELS => 1
		)
		port map
		(
			clock   => clock,
			reset   => reset,
			--
			sclock  => ads_clkout_0,
			sdata   => ads_sdata,
			--
			pdata0  => sd_vbus_raw,
			pdata1  => open,
			pdata2  => open,
			pdata3  => open,
			pdata4  => open,
			pdata5  => open,
			pdata6  => open,
			pdata7  => open,
			pdata8  => open,
			pdata9  => open,
			pdata10 => open,
			pdata11 => open,
			pdata12 => open,
			pdata13 => open,
			pdata14 => open,
			pdata15 => open,
			finish  => open
		);
		
	sd_val_vbus_0 <= sd_vbus_raw & "000000";
	sd_val_vbus_1 <= sd_vbus_raw & "000000";
	
	---------------------------------------------------------------------------

	-- s/d M0 3-channel acquisition
	SD_ACQ_M0_INST : s3acqimot
		port map
		(
			clock  => clock,
			reset  => reset,
			--
			sclock => ads_clkout_0,
			sdata1 => lk_ads_ia_0,
			sdata2 => lk_ads_ib_0,
			--
			modsyn => modsyn_0,
			--
			syncmd => syncmd_0,
			syntos => syntos_0,
			--
			declim => declim_0,
			skewms => skewms_0,
			--
			pdata1 => sd_val_ia_0,
			pdata2 => sd_val_ib_0,
			focsyn => sd_val_isyn_0
		);
		
	---------------------------------------------------------------------------

	-- s/d M1 3-channel acquisition
	SD_ACQ_M1_INST : s3acqimot
		port map
		(
			clock  => clock,
			reset  => reset,
			--
			sclock => ads_clkout_1,
			sdata1 => lk_ads_ia_1,
			sdata2 => lk_ads_ib_1,
			--
			modsyn => modsyn_1,
			--
			syncmd => syncmd_1,
			syntos => syntos_1,
			--
			declim => declim_1,
			skewms => skewms_1,
			--
			pdata1 => sd_val_ia_1,
			pdata2 => sd_val_ib_1,
			focsyn => sd_val_isyn_1
		);

	---------------------------------------------------------------------------
	
	DO_XADC_GEN : if C_XADC > 0 generate
	
		xa_vbus_0     <= xadc_rctreg(C_WCTREG_XA_VBUS_0);
		xa_iab_0      <= xadc_rctreg(C_WCTREG_XA_IABS_0);

		xa_vbus_1     <= xadc_rctreg(C_WCTREG_XA_VBUS_1);
		xa_iab_1      <= xadc_rctreg(C_WCTREG_XA_IABS_1);
		
		xadc_mux_en_i <= '1';

		XADC_DRP_PROC : process(clock)
		begin
			if rising_edge(clock) then
				if reset = '1' or xadc_mux_en_i = '0' then
					xadc_drp_di    <= (others => '0');
					xadc_drp_daddr <= (others => '0');
					xadc_drp_dwe   <= '0';
					xadc_drp_den   <= '0';
					xadc_rctreg    <= (others => '0');
					xa_syncreq     <= '0';
					stdrp          <= idle;
				else
					--
					xadc_rctreg(C_WCTREG_XA_RESET)  <= xadc_wctreg(C_WCTREG_XA_RESET);
					--
					xadc_rctreg(C_WCTREG_XA_VBUS_0) <= xadc_wctreg(C_WCTREG_XA_VBUS_0);
					xadc_rctreg(C_WCTREG_XA_IABS_0) <= xadc_wctreg(C_WCTREG_XA_IABS_0);
					--
					xadc_rctreg(C_WCTREG_XA_VBUS_1) <= xadc_wctreg(C_WCTREG_XA_VBUS_1);
					xadc_rctreg(C_WCTREG_XA_IABS_1) <= xadc_wctreg(C_WCTREG_XA_IABS_1);
					--
					xa_syncreq                 <= xadc_wctreg(C_WCTREG_SYNC);
					xadc_rctreg(C_WCTREG_SYNC) <= xa_syncack;
					--
					case stdrp is
						when idle =>
							if xadc_rctreg(C_WCTREG_DRP_DC) /= xadc_wctreg(C_WCTREG_DRP_DC) then
								if xadc_wctreg(C_WCTREG_DRP_EN) = '1' then
									xadc_drp_di    <= xadc_wctreg(C_WCTREG_DATA_HI downto C_WCTREG_DATA_LO);
									xadc_drp_daddr <= xadc_wctreg(C_WCTREG_ADDR_HI downto C_WCTREG_ADDR_LO);
									xadc_drp_dwe   <= xadc_wctreg(C_WCTREG_DRP_WE);
									xadc_drp_den   <= '1';
									stdrp          <= w4rdy;
								end if;
							end if;
						when w4rdy =>
							if xadc_drp_drdy = '1' or xadc_reset = '1' then
								xadc_rctreg(C_WCTREG_DATA_HI downto C_WCTREG_DATA_LO) <= xadc_drp_do;
								xadc_rctreg(C_WCTREG_ADDR_HI downto C_WCTREG_ADDR_LO) <= xadc_drp_daddr;
								xadc_drp_dwe   <= '0';
								xadc_drp_den   <= '0';
								xadc_rctreg(C_WCTREG_DRP_DC) <= xadc_wctreg(C_WCTREG_DRP_DC);
								stdrp          <= idle;
							end if;
					end case;
				end if;
			end if;
		end process;
		
		-- XADC reset
		xadc_reset <= '1' when xadc_wctreg(C_WCTREG_XA_RESET) = '1' or reset = '1' else '0';

		-- XADC acquisiton module
		XADC_ACQ_INST : mcxadc
			generic map
			(
				C_SYSTEMCLOCK  => C_SYSTEMCLOCK
			)
			port map
			(
				clock          => clock,
				reset          => xadc_reset,
				-- on-chip temperature
				val_temp       => xa_val_temp,
				-- M0/1 values
				val_ia_0       => xa_val_ia_0,
				val_ib_0       => xa_val_ib_0,
				val_vbus_0     => xa_val_vbus_0,
				val_ia_1       => xa_val_ia_1,
				val_ib_1       => xa_val_ib_1,
				val_vbus_1     => xa_val_vbus_1,
				-- sync
				val_sync       => xa_val_sync,
				-- xadc drp access
				xadc_drp_daddr => xadc_drp_daddr,
				xadc_drp_di    => xadc_drp_di,
				xadc_drp_den   => xadc_drp_den,
				xadc_drp_dwe   => xadc_drp_dwe,
				xadc_drp_do    => xadc_drp_do,
				xadc_drp_drdy  => xadc_drp_drdy,
				-- xadc analog signals
				xadc_vp        => xadc_vp,
				xadc_vn        => xadc_vn,
				xadc_vaux0_p   => xadc_vaux0_p,
				xadc_vaux0_n   => xadc_vaux0_n,
				xadc_vaux8_p   => xadc_vaux8_p,
				xadc_vaux8_n   => xadc_vaux8_n,
				-- xadc external mux
				xadc_mux_a0    => xadc_mux_a0_i,
				xadc_mux_a1    => xadc_mux_a1_i,
				-- sync
				-- clk div. to sync
				synclkdiv      => xa_synclkdiv,
				-- values trigger
				syncreq        => xa_syncreq,
				syncout        => xa_syncout,
				syncack        => xa_syncack
			);
			
	end generate;
	
	
	NOT_XADC_GEN : if C_XADC <= 0 generate
	
		xadc_drp_di    <= (others => '0');
		xadc_drp_daddr <= (others => '0');
		xadc_drp_dwe   <= '0';
		xadc_drp_den   <= '0';
		xadc_rctreg    <= (others => '0');
		xa_syncreq     <= '0';
	
		xa_vbus_0      <= '0';
		xa_iab_0       <= '0';

		xa_vbus_1      <= '0';
		xa_iab_1       <= '0';
		
		xadc_reset     <= '0';
		
		xa_val_temp    <= (others => '0');
		
		xa_val_ia_0    <= (others => '0');
		xa_val_ib_0    <= (others => '0');
		xa_val_vbus_0  <= (others => '0');
		xa_val_ia_1    <= (others => '0');
		xa_val_ib_1    <= (others => '0');
		xa_val_vbus_1  <= (others => '0');
		
		xa_val_sync    <= '0';
		
		xa_syncout     <= '0';
		xa_syncack     <= '0';
		
		xadc_drp_do    <= (others => '0');
		xadc_drp_drdy  <= '0';
		
		xadc_mux_a0_i  <= '0';
		xadc_mux_a1_i  <= '0';
		
		xadc_mux_en_i  <= '0';
		
	end generate;
	
	---------------------------------------------------------------------------

	-- sd normalized
	
	sd_bip_ia_0   <= std_logic_vector(signed(sd_val_ia_0  ) - 2**17);
	sd_bip_ib_0   <= std_logic_vector(signed(sd_val_ib_0  ) - 2**17);
	sd_bip_vbus_0 <= std_logic_vector(signed(sd_val_vbus_0) - 2**17);
	
	sd_bip_ia_1   <= std_logic_vector(signed(sd_val_ia_1  ) - 2**17);
	sd_bip_ib_1   <= std_logic_vector(signed(sd_val_ib_1  ) - 2**17);
	sd_bip_vbus_1 <= std_logic_vector(signed(sd_val_vbus_1) - 2**17);
	
	---------------------------------------------------------------------------

	-- xa normalized
	
	xa_bip_ia_0   <= std_logic_vector(signed(xa_val_ia_0  ) - 0);
	xa_bip_ib_0   <= std_logic_vector(signed(xa_val_ib_0  ) - 0);
	xa_bip_vbus_0 <= std_logic_vector(signed(xa_val_vbus_0) - 0);
	
	xa_bip_ia_1   <= std_logic_vector(signed(xa_val_ia_1  ) - 0);
	xa_bip_ib_1   <= std_logic_vector(signed(xa_val_ib_1  ) - 0);
	xa_bip_vbus_1 <= std_logic_vector(signed(xa_val_vbus_1) - 0);
	
	---------------------------------------------------------------------------
	-- mux sd/xadc ------------------------------------------------------------
	---------------------------------------------------------------------------
	
	mux_iphs_a_0  <= xa_bip_ia_0   when xa_iab_0  = '1' else sd_bip_ia_0;
	mux_iphs_b_0  <= xa_bip_ib_0   when xa_iab_0  = '1' else sd_bip_ib_0;
	mux_vbus_x_0  <= xa_bip_vbus_0 when xa_vbus_0 = '1' else sd_bip_vbus_0;
	mux_sync_0    <= xa_syncout    when xa_iab_0  = '1' else sd_val_isyn_0;

	mux_iphs_a_1  <= xa_bip_ia_1   when xa_iab_1  = '1' else sd_bip_ia_1;
	mux_iphs_b_1  <= xa_bip_ib_1   when xa_iab_1  = '1' else sd_bip_ib_1;
	mux_vbus_x_1  <= xa_bip_vbus_1 when xa_vbus_1 = '1' else sd_bip_vbus_1;
	mux_sync_1    <= xa_syncout    when xa_iab_1  = '1' else sd_val_isyn_1;

	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	
	-----------------------------------------------------------------------------------------

	-- M0 a/d
	acq_iphs_a_0_i <= mux_iphs_a_0;
	acq_iphs_b_0_i <= mux_iphs_b_0;
	acq_iphs_c_0_i <= (others => '0');
	acq_ibus_x_0_i <= (others => '0');
	acq_vphs_a_0_i <= (others => '0');
	acq_vphs_b_0_i <= (others => '0');
	acq_vphs_c_0_i <= (others => '0');
	acq_vbus_x_0_i <= mux_vbus_x_0;
	acq_vphs_n_0_i <= (others => '0');
	acq_sync_0_i   <= mux_sync_0;
	
	-----------------------------------------------------------------------------------------

	-- M1 a/d
	acq_iphs_a_1_i <= mux_iphs_a_1;
	acq_iphs_b_1_i <= mux_iphs_b_1;
	acq_iphs_c_1_i <= (others => '0');
	acq_ibus_x_1_i <= (others => '0');
	acq_vphs_a_1_i <= (others => '0');
	acq_vphs_b_1_i <= (others => '0');
	acq_vphs_c_1_i <= (others => '0');
	acq_vbus_x_1_i <= mux_vbus_x_1;
	acq_vphs_n_1_i <= (others => '0');
	acq_sync_1_i   <= mux_sync_1;
	
	-----------------------------------------------------------------------------------------
	
	-- M0 failure/kill
	hwfail_0_i <= gu_fault_0;
	hwkill_0_i <= '0';
	
	-- M1 failure/kill
	hwfail_1_i <= gu_fault_1;
	hwkill_1_i <= '0';
	
	-----------------------------------------------------------------------------------------
	
	
	---------------------------------------------------------------------------
	
	---------------------------------------------------------------------------
	-- filtering --------------------------------------------------------------
	---------------------------------------------------------------------------
	
	DO_OC_TEMP_LPF_GEN : if C_XADC > 0 generate
		OC_TEMP_LPF_INST : filtiir
			generic map
			(
				C_FAMILY => C_FAMILY
			)
			port map
			( 
				clock  => clock,
				reset  => reset,
				
				start  => xa_val_sync,
				fktau  => oc_temp_fkt,
				inraw  => xa_val_temp,
				outflt => oc_temp_val,
				finish => open
			);
	end generate;
			
	NOT_OC_TEMP_LPF_GEN : if C_XADC <= 0 generate
		oc_temp_val <= (others => '0');
	end generate;
	
	---------------------------------------------------------------------------
	
	-- M0: pwm driver
	drv_pwm_a_0_i <= coildrv(DEF_DRV_pwm_a_0);
	drv_pwm_b_0_i <= coildrv(DEF_DRV_pwm_b_0);
	drv_pwm_c_0_i <= coildrv(DEF_DRV_pwm_c_0);
	
	-- M0: pwm enable
	drv_rst_a_0_i <= coildrv(DEF_DRV_rst_a_0);
	drv_rst_b_0_i <= coildrv(DEF_DRV_rst_b_0);
	drv_rst_c_0_i <= coildrv(DEF_DRV_rst_c_0);
	
	-- M1: pwm driver
	drv_pwm_a_1_i <= coildrv(DEF_DRV_pwm_a_1);
	drv_pwm_b_1_i <= coildrv(DEF_DRV_pwm_b_1);
	drv_pwm_c_1_i <= coildrv(DEF_DRV_pwm_c_1);
	
	-- M1: pwm enable
	drv_rst_a_1_i <= coildrv(DEF_DRV_rst_a_1);
	drv_rst_b_1_i <= coildrv(DEF_DRV_rst_b_1);
	drv_rst_c_1_i <= coildrv(DEF_DRV_rst_c_1);

	---------------------------------------------------------------------------
	
	-- GU swtch count M0
	
	guevsrc_0(0) <= drv_pwm_a_0_i;
	guevsrc_0(1) <= drv_pwm_b_0_i;
	guevsrc_0(2) <= drv_pwm_c_0_i;

	-- event counter
	GUSWCNT_M0 : evxcount
		generic map
		(
			C_DATA_WIDTH  => 31,
			C_EVENT_WIDTH => 3,
			C_COUNT_MODE  => 0
		)
		port map
		(
			clock   => clock,
			reset   => reset,
			sigevt  => guevsrc_0,
			counter => guswcnt_leg_0
		);
		
	guswcnt_0_i <= guswcnt_leg_0 & "0";

	---------------------------------------------------------------------------
	
	-- GU swtch count M1
	
	guevsrc_1(0) <= drv_pwm_a_1_i;
	guevsrc_1(1) <= drv_pwm_b_1_i;
	guevsrc_1(2) <= drv_pwm_c_1_i;

	-- event counter
	GUSWCNT_M1 : evxcount
		generic map
		(
			C_DATA_WIDTH  => 31,
			C_EVENT_WIDTH => 3,
			C_COUNT_MODE  => 0
		)
		port map
		(
			clock   => clock,
			reset   => reset,
			sigevt  => guevsrc_1,
			counter => guswcnt_leg_1
		);
		
	guswcnt_1_i <= guswcnt_leg_1 & "0";
	
	---------------------------------------------------------------------------

	-- arbiter is required in case of bipolar stepper motor
	motdrv_arbiter_proc : process(clock)
	begin
		if rising_edge(clock) then
			if reset = '1' then
				mot_drv_0_owner <= '0';
				mot_drv_1_owner <= '0';
			else
				if is3phases_0 = '1' and is3phases_1 = '1' then
					mot_drv_0_owner <= '1';
					mot_drv_1_owner <= '1';
				elsif coilenab_0 = '1' and coilenab_1 = '0' then
					mot_drv_0_owner <= '1';
					mot_drv_1_owner <= '0';
				elsif coilenab_0 = '0' and coilenab_1 = '1' then
					mot_drv_0_owner <= '0';
					mot_drv_1_owner <= '1';
				elsif coilenab_0 = '1' and coilenab_1 = '1' then
					if mot_drv_0_owner = '1' and mot_drv_1_owner = '1' then
						if is3phases_0 = '0' then
							mot_drv_0_owner <= '1';
							mot_drv_1_owner <= '0';
						else
							mot_drv_0_owner <= '0';
							mot_drv_1_owner <= '1';
						end if;
					end if;
				end if;
			end if;
		end if;
	end process;

	---------------------------------------------------------------------------
	-- gate unit fault
	
	GU_FAULT_PROC : process(clock)
	begin
		if rising_edge(clock) then
			if reset = '1' then
				gu_fault_0 <= '0';
				gu_fault_1 <= '0';
			else
				-- M0
				if dg_drv_fault_0 = '0' or dg_drv_otw_0 = '0' then
					gu_fault_0 <= '1';
				elsif reseterr_0 = '1' then
					gu_fault_0 <= '0';
				end if;
				-- M1
				if dg_drv_fault_1 = '0' or dg_drv_otw_1 = '0' then
					gu_fault_1 <= '1';
				elsif reseterr_1 = '1' then
					gu_fault_1 <= '0';
				end if;
			end if;
		end if;
	end process;
	
	---------------------------------------------------------------------------
	-- build status register
	--
    
    -- M0
	status_reg(DEF_STATUS_REG_DRV_FAULT_0) <= not dg_drv_fault_0;
	status_reg(DEF_STATUS_REG_DRV_OTW_0  ) <= not dg_drv_otw_0  ;
	
    -- M1
	status_reg(DEF_STATUS_REG_DRV_FAULT_1) <= not dg_drv_fault_1;
	status_reg(DEF_STATUS_REG_DRV_OTW_1  ) <= not dg_drv_otw_1  ;
    
	---------------------------------------------------------------------------
	
	COIL_MOT_DRV_PROC : process(clock)
		variable cdv : std_logic_vector(coildrv'range);
	begin
		if rising_edge(clock) then
			if reset = '1' then
				coildrv <= (others => '0');
			else
				cdv := (others => '0');
				-- M0
				if mot_drv_0_owner = '1' then
				
					if is3phases_0 = '1' then
					
						if modtype_0 = MODTYPE_RPFM then
							cdv(DEF_DRV_pwm_a_0) := rpfm3p_l2cxw1_0(0); 
							cdv(DEF_DRV_pwm_b_0) := rpfm3p_l2cxw1_0(1); 
							cdv(DEF_DRV_pwm_c_0) := rpfm3p_l2cxw1_0(2);
						else
							cdv(DEF_DRV_pwm_a_0) := pwm3p_l2cxw1_0(0); 
							cdv(DEF_DRV_pwm_b_0) := pwm3p_l2cxw1_0(1); 
							cdv(DEF_DRV_pwm_c_0) := pwm3p_l2cxw1_0(2);
						end if;
						
						cdv(DEF_DRV_rst_a_0) := coilgear_0;
						cdv(DEF_DRV_rst_b_0) := coilgear_0;
						cdv(DEF_DRV_rst_c_0) := coilgear_0;
						
					else
					
						cdv(DEF_DRV_pwm_a_0) := pwm2p_l2c1w1_0(0);
						cdv(DEF_DRV_pwm_b_0) := pwm2p_l2c2w1_0(0);
						cdv(DEF_DRV_pwm_c_0) := '0';
						
						cdv(DEF_DRV_rst_a_0) := coilgear_0;
						cdv(DEF_DRV_rst_b_0) := coilgear_0;
						cdv(DEF_DRV_rst_c_0) := '0';
						
						cdv(DEF_DRV_pwm_a_1) := pwm2p_l2c1w1_0(1);
						cdv(DEF_DRV_pwm_b_1) := pwm2p_l2c2w1_0(1);
						cdv(DEF_DRV_pwm_c_1) := '0';
						
						cdv(DEF_DRV_rst_a_1) := coilgear_0;
						cdv(DEF_DRV_rst_b_1) := coilgear_0;
						cdv(DEF_DRV_rst_c_1) := '0';
						
					end if;
				end if;
				-- M1
				if mot_drv_1_owner = '1' then
				
					if is3phases_1 = '1' then
					
						if modtype_1 = MODTYPE_RPFM then
							cdv(DEF_DRV_pwm_a_1) := rpfm3p_l2cxw1_1(0); 
							cdv(DEF_DRV_pwm_b_1) := rpfm3p_l2cxw1_1(1); 
							cdv(DEF_DRV_pwm_c_1) := rpfm3p_l2cxw1_1(2);
						else
							cdv(DEF_DRV_pwm_a_1) := pwm3p_l2cxw1_1(0); 
							cdv(DEF_DRV_pwm_b_1) := pwm3p_l2cxw1_1(1); 
							cdv(DEF_DRV_pwm_c_1) := pwm3p_l2cxw1_1(2);
						end if;
						
						cdv(DEF_DRV_rst_a_1) := coilgear_1;
						cdv(DEF_DRV_rst_b_1) := coilgear_1;
						cdv(DEF_DRV_rst_c_1) := coilgear_1;
						
					else
					
						cdv(DEF_DRV_pwm_a_1) := pwm2p_l2c1w1_1(0);
						cdv(DEF_DRV_pwm_b_1) := pwm2p_l2c2w1_1(0);
						cdv(DEF_DRV_pwm_c_1) := '0';
						
						cdv(DEF_DRV_rst_a_1) := coilgear_1;
						cdv(DEF_DRV_rst_b_1) := coilgear_1;
						cdv(DEF_DRV_rst_c_1) := '0';
						
						cdv(DEF_DRV_pwm_a_0) := pwm2p_l2c1w1_1(1);
						cdv(DEF_DRV_pwm_b_0) := pwm2p_l2c2w1_1(1);
						cdv(DEF_DRV_pwm_c_0) := '0';
						
						cdv(DEF_DRV_rst_a_0) := coilgear_1;
						cdv(DEF_DRV_rst_b_0) := coilgear_1;
						cdv(DEF_DRV_rst_c_0) := '0';
						
					end if;
				end if;
				--
				coildrv <= cdv;
			end if;
		end if;
	end process;
	
	---------------------------------------------------------------------------
	
	-- implement slave model software accessible register(s)
	SLAVE_WRITE_PROC : process(clock)
	begin
		if rising_edge(clock) then
			if reset = '1' then
				--
				sd_ctrl_reg_0 <= K_SD_CTRL_REG;
				sd_ctrl_reg_1 <= K_SD_CTRL_REG;
				--
				xa_synclkdiv  <= K_XA_SYNCLKDIV;
				--
				oc_temp_fkt   <= K_FILTER_FAST;
				--
				xadc_wctreg   <= (others => '0');
				--
				gpio_o_i      <= (others => '0');
				gpio_t_i      <= (others => '1');
				--
			elsif host_we = '1' and host_en = '1' then
				case adrtoint(host_addr) is
				
					when RA_ip_ident     => null;
					when RA_status_reg   => null;

					when RA_sd_ctrl_0    => storereg(sd_ctrl_reg_0  ,host_din);
					when RA_sd_ctrl_1    => storereg(sd_ctrl_reg_1  ,host_din);

					when RA_oc_temp_val  => null;
					when RA_oc_temp_fkt  => storereg(oc_temp_fkt ,host_din);

					when RA_xa_synclkdiv => storereg(xa_synclkdiv,host_din);

					when RA_xadc_wctreg  => storereg(xadc_wctreg ,host_din);
					when RA_xadc_rctreg  => null;

					when RA_dip_sw       => null;

					when RA_gpio_i       => null;
					when RA_gpio_o       => storereg(gpio_o_i,host_din);
					when RA_gpio_t       => storereg(gpio_t_i,host_din);

					when others          => null;
				end case;
			end if;
		end if;
	end process;

	-- implement slave model software accessible register(s) read mux
	SLAVE_READ_PROC : process(clock)
		variable v : std_logic_vector(31 downto 0);
	begin
		if rising_edge(clock) then
			v := (others => '0');
			case adrtoint(host_addr) is
			
				when RA_ip_ident     => v := loadreg(ip_ident_k);
				when RA_status_reg   => v := loadreg(status_reg);

				when RA_sd_ctrl_0    => v := loadreg(sd_ctrl_reg_0);
				when RA_sd_ctrl_1    => v := loadreg(sd_ctrl_reg_1);

				when RA_oc_temp_val  => v := loadreg(oc_temp_val);
				when RA_oc_temp_fkt  => v := loadreg(oc_temp_fkt);

				when RA_xa_synclkdiv => v := loadreg(xa_synclkdiv);

				when RA_xadc_wctreg  => v := loadreg(xadc_wctreg);
				when RA_xadc_rctreg  => v := loadreg(xadc_rctreg);

				when RA_dip_sw       => v := loadreg(dip_sw);

				when RA_gpio_i       => v := loadreg(gpio_i);
				when RA_gpio_o       => v := loadreg(gpio_o_i);
				when RA_gpio_t       => v := loadreg(gpio_t_i);

				when others          => null;
			end case;
			host_dout_i <= v;
		end if;
	end process;
	
	-- data transfer ack
	host_ack_i <= host_we or host_en_dl1;
	
	-- host access delay
	HOSTDLY_PROC : process(clock)
	begin
		if rising_edge(clock) then
			host_en_dl1 <= host_en;
		end if;
	end process;

end Behavioral;
