-------------------------------------------------------------------------------------------
-- Copyright  2011-2015, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    09:22:03 01/05/2012 
-- Design Name: 
-- Module Name:    s3acqimot - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity s3acqimot is
	port 
	(
		clock  : in  std_logic;
		reset  : in  std_logic;
		--
		sclock : in  std_logic;
		sdata1 : in  std_logic;
		sdata2 : in  std_logic;
		--
		modsyn : in  std_logic;
		--
		syncmd : in  std_logic;
		syntos : in  std_logic_vector(5 downto 0);
		--
		declim : in  std_logic_vector(5 downto 0);
		skewms : in  std_logic_vector(5 downto 0);
		--
		pdata1 : out std_logic_vector(17 downto 0);
		pdata2 : out std_logic_vector(17 downto 0);
		focsyn : out std_logic
	);
end s3acqimot;

architecture Behavioral of s3acqimot is

	-- sinc3 tandem filter
	component s3tandem
		port
		(
			clock  : in  std_logic;
			reset  : in  std_logic;
			--
			sclock : in  std_logic;
			sdata  : in  std_logic;
			--
			wloadm : in  std_logic;
			wloads : in  std_logic;
			--
			pdata  : out std_logic_vector(17 downto 0);
			focsyn : out std_logic
		);
	end component;


	-- sinc3 parameters
	constant C_RESULT_WIDTH : integer := 12;
	constant C_DECIM_WIDTH  : integer := C_RESULT_WIDTH / 2;
	
	-- s/d clock domain
	signal decim  : std_logic_vector(C_DECIM_WIDTH-1 downto 0);
	signal wloadm : std_logic; -- master S3
	signal wloads : std_logic; -- slave  S3
	
	-- sys-clock
	signal syncmr  : std_logic; -- rising edge detection of sync command
	signal synw4ev : std_logic; -- waiting for sync 
	signal syncnt  : std_logic; -- sync command counter (just a toggle bit)
	
	-- s/d-clock
	signal synack  : std_logic; -- ack sync counter command from sys
	signal synrun  : std_logic; -- sync running proc.
	signal s3synct : std_logic; -- syn count command to s3 decimator
	signal tmsync  : std_logic_vector(5 downto 0); -- skew counter modulator to s3
	signal s3synak : std_logic; -- sync count ack on s3 decimator

begin

	-- decimation syncronization
	
	-- sys domain: sync clock counte
	S3_SYNH_PROC : process(clock)
	begin
		if rising_edge(clock) then
			if reset = '1' then
				syncmr  <= '0';
				synw4ev <= '0';
				syncnt  <= '0';
			else
				syncmr <= syncmd;
				if syncmr = '0' and syncmd = '1' then
					synw4ev <= '1';
				elsif synw4ev = '1' and modsyn = '1' then
					synw4ev <= '0';
					syncnt  <= not syncnt;
				end if;
			end if;
		end if;
	end process;
	
	
	-- s/d domain: sync clock counter
	S3_SYNC_PROC : process(sclock)
	begin
		if rising_edge(sclock) then
			if reset = '1' then
				synack  <= '0';
				synrun  <= '0';
				s3synct <= '0';
				tmsync  <= (others => '0');
			else
				synack <= syncnt;
				if synack /= syncnt then
					synrun <= '1';
					tmsync <= (others => '0');
				elsif tmsync = syntos and synrun = '1' then
					synrun  <= '0';
					s3synct <= not s3synct;
					tmsync  <= tmsync + 1;
				elsif synrun = '1' then
					tmsync <= tmsync + 1;
				end if;
			end if;
		end if;
	end process;

	-- decimation counter
	S3_DECT_PROC : process(sclock)
	begin
		if rising_edge(sclock) then
			if reset = '1' then
				s3synak <= '0';
				decim   <= (others => '0');
				wloadm  <= '0';
				wloads  <= '0';
			else
				s3synak <= s3synct;
				if decim = declim or s3synak /= s3synct then
					decim  <= (others => '0');
					wloadm <= '1';
				else
					decim  <= decim + 1;
					wloadm <= '0';
				end if;
				if decim = skewms then
					wloads <= '1';
				else
					wloads <= '0';
				end if;
			end if;
		end if;
	end process;
	
	-- acq+filter+offadj
	
	S3_AFO_1_INST : s3tandem
		port map
		(
			clock  => clock,
			reset  => reset,
			--
			sclock => sclock,
			sdata  => sdata1,
			--
			wloadm => wloadm,
			wloads => wloads,
			--
			pdata  => pdata1,
			focsyn => focsyn
		);

	S3_AFO_2_INST : s3tandem
		port map
		(
			clock  => clock,
			reset  => reset,
			--
			sclock => sclock,
			sdata  => sdata2,
			--
			wloadm => wloadm,
			wloads => wloads,
			--
			pdata  => pdata2,
			focsyn => open
		);

end Behavioral;
