-------------------------------------------------------------------------------------------
-- Copyright  2011-2015, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    15:00:31 01/08/2014 
-- Design Name: 
-- Module Name:    s3acqmult - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity s3acqmult is
	generic
	(
		C_RESULT_WIDTH : integer := 16;
		C_NUM_CHANNELS : integer := 16
	);
	port 
	(
		clock   : in  std_logic;
		reset   : in  std_logic;
		--
		sclock  : in  std_logic;
		sdata   : in  std_logic_vector(C_NUM_CHANNELS-1 downto 0);
		--
		pdata0  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata1  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata2  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata3  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata4  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata5  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata6  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata7  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata8  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata9  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata10 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata11 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata12 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata13 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata14 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		pdata15 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
		finish  : out std_logic
	);
end s3acqmult;

architecture Behavioral of s3acqmult is

	component sinc3flt
		generic
		(
			C_MATH_WIDTH   : integer := 24;
			C_RESULT_WIDTH : integer := 16
		);
		port
		(
			clock  : in  std_logic;
			reset  : in  std_logic;
			sdata  : in  std_logic;
			wload  : in  std_logic;
			wdata  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0)
		);
	end component;

	-- sinc3 parameters
	constant C_MATH_WIDTH   : integer := (C_RESULT_WIDTH * 3) / 2;
	constant C_DECIM_WIDTH  : integer := C_RESULT_WIDTH / 2;

	-- s/d clock domain
	constant k_decim_zero : std_logic_vector(C_DECIM_WIDTH-1 downto 0) := (others => '0');
	signal   decim        : std_logic_vector(C_DECIM_WIDTH-1 downto 0) := (others => '0');
	signal   wload        : std_logic;
	type     wdata_t is array(0 to 15) of std_logic_vector(C_RESULT_WIDTH-1 downto 0);
	signal   wdata        : wdata_t := (others => (others =>'0'));
	
	-- system clock domain
	signal   dload        : std_logic_vector(2 downto 0) := (others => '0');
	
	-- output
	signal pdata_i  : wdata_t;
	signal finish_i : std_logic;
	
begin

	-- result
	pdata0  <= pdata_i(0 );
	pdata1  <= pdata_i(1 );
	pdata2  <= pdata_i(2 );
	pdata3  <= pdata_i(3 );
	pdata4  <= pdata_i(4 );
	pdata5  <= pdata_i(5 );
	pdata6  <= pdata_i(6 );
	pdata7  <= pdata_i(7 );
	pdata8  <= pdata_i(8 );
	pdata9  <= pdata_i(9 );
	pdata10 <= pdata_i(10);
	pdata11 <= pdata_i(11);
	pdata12 <= pdata_i(12);
	pdata13 <= pdata_i(13);
	pdata14 <= pdata_i(14);
	pdata15 <= pdata_i(15);
	finish  <= finish_i;

	KTK_PROC : process(clock)
	begin
		if rising_edge(clock) then
			if reset = '1' then
				finish_i <= '0';
			else
				dload <= dload(dload'left-1 downto 0) & wload;
				if dload(dload'left) = '1' and dload(dload'left-1) = '0' then
					pdata_i  <= wdata;
					finish_i <= '1';
				else
					finish_i <= '0';
				end if;
			end if;
		end if;
	end process;

	S3_DECIM_PROC : process(sclock)
	begin
		if rising_edge(sclock) then
			decim <= decim + 1;
			if decim = k_decim_zero then
				wload <= '1';
			else
				wload <= '0';
			end if;
		end if;
	end process;
	
	S3_FLT_GEN : for i in sdata'range generate
		S3_FLT_INST : sinc3flt
			generic map
			(
				C_MATH_WIDTH   => C_MATH_WIDTH,
				C_RESULT_WIDTH => C_RESULT_WIDTH
			)
			port map
			(
				clock  => sclock,
				reset  => reset,
				sdata  => sdata(i),
				wload  => wload,
				wdata  => wdata(i)
			);
	end generate;

end Behavioral;
