-------------------------------------------------------------------------------------------
-- Copyright  2011-2015, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    11:02:27 01/05/2012 
-- Design Name: 
-- Module Name:    s3tandem - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity s3tandem is
	port
	(
		clock  : in  std_logic;
		reset  : in  std_logic;
		--
		sclock : in  std_logic;
		sdata  : in  std_logic;
		--
		wloadm : in  std_logic;
		wloads : in  std_logic;
		--
		pdata  : out std_logic_vector(17 downto 0);
		focsyn : out std_logic
	);
end s3tandem;

architecture Behavioral of s3tandem is

	-- sinc3 filter core
	component sinc3flt
		generic
		(
			C_MATH_WIDTH   : integer := 24;
			C_RESULT_WIDTH : integer := 16
		);
		port
		(
			clock  : in  std_logic;
			reset  : in  std_logic;
			sdata  : in  std_logic;
			wload  : in  std_logic;
			wdata  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0)
		);
	end component;
	
	-- sinc3 parameters
	constant C_RESULT_WIDTH : integer := 12;
	constant C_MATH_WIDTH   : integer := (C_RESULT_WIDTH * 3) / 2;
	
	-- s/d clock domain
	signal   wdatam      : std_logic_vector(C_RESULT_WIDTH-1 downto 0);
	signal   wdatas      : std_logic_vector(C_RESULT_WIDTH-1 downto 0);

	-- system clock domain
	constant k_cload_smp : integer := 2;
	constant k_cload_hld : integer := k_cload_smp + 1;
	signal   cload       : natural range 0 to k_cload_hld;
	signal   selmas      : std_logic; -- mux selector (master/slave)
	-- raw dato @ 18 bits
	signal   pdata_i     : std_logic_Vector(17 downto 0);
	signal   focsyn_i    : std_logic;
	
	-- padding data for output width
	constant C_EXTRA_WIDTH : integer := 18 - C_RESULT_WIDTH;
	constant k_dat_pad     : std_logic_vector(C_EXTRA_WIDTH-1 downto 0) := (others => '0');

begin

	pdata  <= pdata_i;
	focsyn <= focsyn_i;

	KTK_PROC : process(clock)
		variable sylod : std_logic;
	begin
		if rising_edge(clock) then
			if reset = '1' then
				selmas   <= '0';
				cload    <= 0;
				pdata_i  <= (others => '0');
				focsyn_i <= '0';
			else
				sylod := '0';
				if wloadm = '1' or wloads = '1' then
					selmas <= wloadm;
					cload  <= 0;
				elsif cload = k_cload_smp then
					sylod := '1';
					if selmas = '1' then
						pdata_i <= wdatam & k_dat_pad;
					else
						pdata_i <= wdatas & k_dat_pad;
					end if;
					cload <= cload + 1;
				elsif cload /= k_cload_hld then
					cload <= cload + 1;
				end if;
				focsyn_i <= sylod;
			end if;
		end if;
	end process;

	S3_MAS_INST : sinc3flt
		generic map
		(
			C_MATH_WIDTH   => C_MATH_WIDTH,
			C_RESULT_WIDTH => C_RESULT_WIDTH
		)
		port map
		(
			clock  => sclock,
			reset  => reset,
			sdata  => sdata,
			wload  => wloadm,
			wdata  => wdatam
		);

	S3_SLV_INST : sinc3flt
		generic map
		(
			C_MATH_WIDTH   => C_MATH_WIDTH,
			C_RESULT_WIDTH => C_RESULT_WIDTH
		)
		port map
		(
			clock  => sclock,
			reset  => reset,
			sdata  => sdata,
			wload  => wloads,
			wdata  => wdatas
		);

end Behavioral;

