-------------------------------------------------------------------------------------------
-- Copyright © 2011-2019, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    15:42:51 01/07/2016 
-- Design Name: 
-- Module Name:    axi_gplogger - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity axi_gplogger is
	generic
	(
		-- FPGA Family Parameter      
		C_FAMILY             : string := "zynq";

		-- AXI bus block speed
		C_S_AXI_ACLK_FREQ_HZ : integer := 100_000_000;

		-- Parameters of Axi Slave Bus Interface S_AXI
		C_S_AXI_DATA_WIDTH	: integer range 32 to 32 := 32;
		C_S_AXI_ADDR_WIDTH	: integer range 16 to 16 := 16;

		-- AXI bus block speed
		C_M_AXI_ACLK_FREQ_HZ : integer := 100_000_000;

		-- Parameters of Axi Master Bus Interface M_AXI
		C_M_AXI_DATA_WIDTH   : integer range 32 to 32 := 32;
		C_M_AXI_ADDR_WIDTH   : integer range 32 to 32 := 32;
		
		-- IP generics
		C_SYSTEMCLOCK        : integer := 100_000_000;
		--
		C_HW_LINK_REF        : integer := 0;
		C_MEMORY_BASE        : std_logic_vector(31 downto 0) := X"00000000";
		C_MEMORY_SIZE        : std_logic_vector(31 downto 0) := X"00002000";
		C_REC_DATA_WIDTH     : integer                       := 128
	);
	port
	(
		-- Ports of Axi Slave Bus Interface S_AXI
		S_AXI_ACLK      : in  std_logic;
		S_AXI_ARESETN   : in  std_logic;
		S_AXI_AWADDR    : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 DOWNTO 0);
		S_AXI_AWVALID   : in  std_logic;
		S_AXI_AWREADY   : out std_logic;
		S_AXI_WDATA     : in  std_logic_vector(C_S_AXI_DATA_WIDTH-1 DOWNTO 0);
		S_AXI_WSTRB     : in  std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 DOWNTO 0);
		S_AXI_WVALID    : in  std_logic;
		S_AXI_WREADY    : out std_logic;
		S_AXI_BRESP     : out std_logic_vector(1 DOWNTO 0);
		S_AXI_BVALID    : out std_logic;
		S_AXI_BREADY    : in  std_logic;
		S_AXI_ARADDR    : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 DOWNTO 0);
		S_AXI_ARVALID   : in  std_logic;
		S_AXI_ARREADY   : out std_logic;
		S_AXI_RDATA     : out std_logic_vector(C_S_AXI_DATA_WIDTH-1 DOWNTO 0);
		S_AXI_RRESP     : out std_logic_vector(1 DOWNTO 0);
		S_AXI_RVALID    : out std_logic;
		S_AXI_RREADY    : in  std_logic;
		--
		-- Ports of Axi Master Bus Interface M_AXI
		M_AXI_ACLK    : in  std_logic;
		M_AXI_ARESETN : in  std_logic;
		M_AXI_AWADDR  : out std_logic_vector(C_M_AXI_ADDR_WIDTH-1 downto 0);
		M_AXI_AWPROT  : out std_logic_vector(2 downto 0);
		M_AXI_AWVALID : out std_logic;
		M_AXI_AWREADY : in  std_logic;
		M_AXI_WDATA   : out std_logic_vector(C_M_AXI_DATA_WIDTH-1 downto 0);
		M_AXI_WSTRB   : out std_logic_vector(C_M_AXI_DATA_WIDTH/8-1 downto 0);
		M_AXI_WVALID  : out std_logic;
		M_AXI_WREADY  : in  std_logic;
		M_AXI_BRESP   : in  std_logic_vector(1 downto 0);
		M_AXI_BVALID  : in  std_logic;
		M_AXI_BREADY  : out std_logic;
		M_AXI_ARADDR  : out std_logic_vector(C_M_AXI_ADDR_WIDTH-1 downto 0);
		M_AXI_ARPROT  : out std_logic_vector(2 downto 0);
		M_AXI_ARVALID : out std_logic;
		M_AXI_ARREADY : in  std_logic;
		M_AXI_RDATA   : in  std_logic_vector(C_M_AXI_DATA_WIDTH-1 downto 0);
		M_AXI_RRESP   : in  std_logic_vector(1 downto 0);
		M_AXI_RVALID  : in  std_logic;
		M_AXI_RREADY  : out std_logic;
		--
		-- acquisition ctrl/stat
		acq_start     : out std_logic;
		acq_trig      : out std_logic;
		acq_ready     : out std_logic;
		acq_w4trg     : out std_logic;
		acq_level     : out std_logic_vector(31 downto 0);
		-- incoming triggers
		acq_trigers   : in  std_logic_vector(15 downto 0);
		-- record
		rec_data      : in  std_logic_vector(C_REC_DATA_WIDTH-1 downto 0);
		rec_sync      : in  std_logic
	);
end axi_gplogger;

architecture Behavioral of axi_gplogger is

	component axi4ltiny
		generic
		(
			C_S_AXI_DATA_WIDTH : integer range 32 to 32 := 32;
			C_S_AXI_ADDR_WIDTH : integer range  1 to 32 := 16
		);
		port
		(
			-- AXI4LITE
			S_AXI_ACLK    : in  std_logic;
			S_AXI_ARESETN : in  std_logic;
			S_AXI_AWADDR  : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
			S_AXI_AWVALID : in  std_logic;
			S_AXI_AWREADY : out std_logic;
			S_AXI_WDATA   : in  std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
			S_AXI_WSTRB   : in  std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 downto 0);
			S_AXI_WVALID  : in  std_logic;
			S_AXI_WREADY  : out std_logic;
			S_AXI_BRESP   : out std_logic_vector(1 downto 0);
			S_AXI_BVALID  : out std_logic;
			S_AXI_BREADY  : in  std_logic;
			S_AXI_ARADDR  : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
			S_AXI_ARVALID : in  std_logic;
			S_AXI_ARREADY : out std_logic;
			S_AXI_RDATA   : out std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
			S_AXI_RRESP   : out std_logic_vector(1 downto 0);
			S_AXI_RVALID  : out std_logic;
			S_AXI_RREADY  : in  std_logic;
			-- local bus  
			host_clk      : out std_logic;
			host_rst      : out std_logic;
			host_en       : out std_logic;
			host_we       : out std_logic;
			host_addr     : out std_logic_vector(31 downto 0);
			host_din      : out std_logic_vector(31 downto 0);
			host_dout     : in  std_logic_vector(31 downto 0);
			host_ack      : in  std_logic
		);
	end component;
	
	component axi4mtiny
		generic
		(
			C_M_AXI_DATA_WIDTH : integer range 32 to 32 := 32;
			C_M_AXI_ADDR_WIDTH : integer range 32 to 32 := 32
		);
		port
		(
			-- Global Signals
			M_AXI_ACLK    : in  std_logic;
			M_AXI_ARESETN : in  std_logic;
			-- Write Address Channel
			M_AXI_AWADDR  : out std_logic_vector(C_M_AXI_ADDR_WIDTH-1 downto 0);
			M_AXI_AWPROT  : out std_logic_vector(2 downto 0);
			M_AXI_AWVALID : out std_logic;
			M_AXI_AWREADY : in  std_logic;
			-- Write Channel
			M_AXI_WDATA   : out std_logic_vector(C_M_AXI_DATA_WIDTH-1 downto 0);
			M_AXI_WSTRB   : out std_logic_vector(C_M_AXI_DATA_WIDTH/8-1 downto 0);
			M_AXI_WVALID  : out std_logic;
			M_AXI_WREADY  : in  std_logic;
			-- Write Response Channel
			M_AXI_BRESP   : in  std_logic_vector(1 downto 0);
			M_AXI_BVALID  : in  std_logic;
			M_AXI_BREADY  : out std_logic;
			-- Read Address Channel
			M_AXI_ARADDR  : out std_logic_vector(C_M_AXI_ADDR_WIDTH-1 downto 0);
			M_AXI_ARPROT  : out std_logic_vector(2 downto 0);
			M_AXI_ARVALID : out std_logic;
			M_AXI_ARREADY : in  std_logic;
			-- Read Channel
			M_AXI_RDATA   : in  std_logic_vector(C_M_AXI_DATA_WIDTH-1 downto 0);
			M_AXI_RRESP   : in  std_logic_vector(1 downto 0);
			M_AXI_RVALID  : in  std_logic;
			M_AXI_RREADY  : out std_logic;
			-- master local bus  
			master_clk    : out std_logic;
			master_rst    : out std_logic;
			-- write i/f
			master_wen    : in  std_logic;
			master_waddr  : in  std_logic_vector(31 downto 0);
			master_wdata  : in  std_logic_vector(31 downto 0);
			master_wack   : out std_logic;
			-- read i/f
			master_ren    : in  std_logic;
			master_raddr  : in  std_logic_vector(31 downto 0);
			master_rdata  : out std_logic_vector(31 downto 0);
			master_rack   : out std_logic
		);
	end component;
	
	component gplogger
		generic
		(
			C_FAMILY           : string                        := "zynq";
			C_HOST_CLK_FREQ_HZ : integer                       := 100_000_000;
			C_MEMORY_BASE      : std_logic_vector(31 downto 0) := X"00000000";
			C_MEMORY_SIZE      : std_logic_vector(31 downto 0) := X"00002000";
			C_REC_DATA_WIDTH   : integer                       := 128
		);
		port 
		(
			-- host interface
			host_clk     : in  std_logic;
			host_rst     : in  std_logic;
			host_en      : in  std_logic;
			host_we      : in  std_logic;
			host_addr    : in  std_logic_vector(31 downto 0);
			host_din     : in  std_logic_vector(31 downto 0);
			host_dout    : out std_logic_vector(31 downto 0);
			host_ack     : out std_logic;
			-- master local bus  
			master_clk   : in  std_logic;
			master_rst   : in  std_logic;
			-- write i/f
			master_wen   : out std_logic;
			master_waddr : out std_logic_vector(31 downto 0);
			master_wdata : out std_logic_vector(31 downto 0);
			master_wack  : in  std_logic;
			-- read i/f
			master_ren   : out std_logic;
			master_raddr : out std_logic_vector(31 downto 0);
			master_rdata : in  std_logic_vector(31 downto 0);
			master_rack  : in  std_logic;
			-- acquisition ctrl/stat
			acq_start    : out std_logic;
			acq_trig     : out std_logic;
			acq_ready    : out std_logic;
			acq_w4trg    : out std_logic;
			acq_level    : out std_logic_vector(31 downto 0);
			-- incoming triggers
			acq_trigers  : in  std_logic_vector(15 downto 0);
			-- record
			rec_data     : in  std_logic_vector(C_REC_DATA_WIDTH-1 downto 0);
			rec_sync     : in  std_logic
		);
	end component;
	
	-- wrapper
	
	constant C_HOST_CLK_FREQ_HZ : integer := C_S_AXI_ACLK_FREQ_HZ;

	signal host_clk        : std_logic;
	signal host_rst        : std_logic;
	--
	signal host_en         : std_logic;
	signal host_we         : std_logic;
	signal host_addr       : std_logic_vector(31 downto 0);
	signal host_din        : std_logic_vector(31 downto 0);
	signal host_dout       : std_logic_vector(31 downto 0);
	signal host_ack        : std_logic;
	
	-- slave -> master
    
	signal master_clk      : std_logic;
	signal master_rst      : std_logic;

	signal master_wen      : std_logic;
	signal master_waddr    : std_logic_vector(31 downto 0);
	signal master_wdata    : std_logic_vector(31 downto 0);
	signal master_wack     : std_logic;

	signal master_ren      : std_logic;
	signal master_raddr    : std_logic_vector(31 downto 0);
	signal master_rdata    : std_logic_vector(31 downto 0);
	signal master_rack     : std_logic;


begin

	assert C_S_AXI_ACLK_FREQ_HZ = C_SYSTEMCLOCK report "AXI clock /= system clock" severity failure;
	assert false report "C_SYSTEMCLOCK=" & integer'image(C_SYSTEMCLOCK) severity note;

	AXI_S_INST : axi4ltiny
		generic map
		(
			C_S_AXI_DATA_WIDTH => C_S_AXI_DATA_WIDTH,
			C_S_AXI_ADDR_WIDTH => C_S_AXI_ADDR_WIDTH
		)
		port map
		(
			--
			S_AXI_ACLK    => S_AXI_ACLK,
			S_AXI_ARESETN => S_AXI_ARESETN,
			S_AXI_AWADDR  => S_AXI_AWADDR,
			S_AXI_AWVALID => S_AXI_AWVALID,
			S_AXI_AWREADY => S_AXI_AWREADY,
			S_AXI_WDATA   => S_AXI_WDATA,
			S_AXI_WSTRB   => S_AXI_WSTRB,
			S_AXI_WVALID  => S_AXI_WVALID,
			S_AXI_WREADY  => S_AXI_WREADY,
			S_AXI_BRESP   => S_AXI_BRESP,
			S_AXI_BVALID  => S_AXI_BVALID,
			S_AXI_BREADY  => S_AXI_BREADY,
			S_AXI_ARADDR  => S_AXI_ARADDR,
			S_AXI_ARVALID => S_AXI_ARVALID,
			S_AXI_ARREADY => S_AXI_ARREADY,
			S_AXI_RDATA   => S_AXI_RDATA,
			S_AXI_RRESP   => S_AXI_RRESP,
			S_AXI_RVALID  => S_AXI_RVALID,
			S_AXI_RREADY  => S_AXI_RREADY,
			--
			host_clk      => host_clk,
			host_rst      => host_rst,
			host_en       => host_en,
			host_we       => host_we,
			host_addr     => host_addr,
			host_din      => host_din,
			host_dout     => host_dout,
			host_ack      => host_ack
		);
	
	AXI_M_INST : axi4mtiny
		generic map
		(
			C_M_AXI_DATA_WIDTH => C_M_AXI_DATA_WIDTH,
			C_M_AXI_ADDR_WIDTH => C_M_AXI_ADDR_WIDTH
		)
		port map
		(
			--
			M_AXI_ACLK    => M_AXI_ACLK,
			M_AXI_ARESETN => M_AXI_ARESETN,
			--
			M_AXI_AWADDR  => M_AXI_AWADDR,
			M_AXI_AWPROT  => M_AXI_AWPROT,
			M_AXI_AWVALID => M_AXI_AWVALID,
			M_AXI_AWREADY => M_AXI_AWREADY,
			--
			M_AXI_WDATA   => M_AXI_WDATA,
			M_AXI_WSTRB   => M_AXI_WSTRB,
			M_AXI_WVALID  => M_AXI_WVALID,
			M_AXI_WREADY  => M_AXI_WREADY,
			--
			M_AXI_BRESP   => M_AXI_BRESP,
			M_AXI_BVALID  => M_AXI_BVALID,
			M_AXI_BREADY  => M_AXI_BREADY,
			--
			M_AXI_ARADDR  => M_AXI_ARADDR,
			M_AXI_ARPROT  => M_AXI_ARPROT,
			M_AXI_ARVALID => M_AXI_ARVALID,
			M_AXI_ARREADY => M_AXI_ARREADY,
			--
			M_AXI_RDATA   => M_AXI_RDATA,
			M_AXI_RRESP   => M_AXI_RRESP,
			M_AXI_RVALID  => M_AXI_RVALID,
			M_AXI_RREADY  => M_AXI_RREADY,
			--
			master_clk    => master_clk,
			master_rst    => master_rst,
			--
			master_wen    => master_wen,
			master_waddr  => master_waddr,
			master_wdata  => master_wdata,
			master_wack   => master_wack,
			--
			master_ren    => master_ren,
			master_raddr  => master_raddr,
			master_rdata  => master_rdata,
			master_rack   => master_rack
		);
	
	LOGGER_INST : gplogger
		generic map
		(
			C_FAMILY           => C_FAMILY,
			C_HOST_CLK_FREQ_HZ => C_HOST_CLK_FREQ_HZ,
			C_MEMORY_BASE      => C_MEMORY_BASE,
			C_MEMORY_SIZE      => C_MEMORY_SIZE,
			C_REC_DATA_WIDTH   => C_REC_DATA_WIDTH  
		) 
		port map
		(
			--
			host_clk     => host_clk,
			host_rst     => host_rst,
			host_en      => host_en,
			host_we      => host_we,
			host_addr    => host_addr,
			host_din     => host_din,
			host_dout    => host_dout,
			host_ack     => host_ack,
			--
			master_clk   => master_clk,
			master_rst   => master_rst,
			--
			master_wen   => master_wen,
			master_waddr => master_waddr,
			master_wdata => master_wdata,
			master_wack  => master_wack,
			--
			master_ren   => master_ren,
			master_raddr => master_raddr,
			master_rdata => master_rdata,
			master_rack  => master_rack,
			--
			acq_start    => acq_start,
			acq_trig     => acq_trig,
			acq_ready    => acq_ready,
			acq_w4trg    => acq_w4trg,
			acq_level    => acq_level,
			--
			acq_trigers  => acq_trigers,
			--
			rec_data     => rec_data,
			rec_sync     => rec_sync
		);

end Behavioral;
