-------------------------------------------------------------------------------------------
-- Copyright © 2011-2017, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    11:32:17 01/06/2014 
-- Design Name: 
-- Module Name:    axi_motcon1_drv - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity axi_motcon1_drv is
	generic
	(
		-- FPGA Family Parameter      
		C_FAMILY             : string := "zynq";

		-- AXI bus block speed
		C_S_AXI_ACLK_FREQ_HZ : integer := 100_000_000;

		-- Parameters of Axi Slave Bus Interface S_AXI
		C_S_AXI_DATA_WIDTH	: integer range 32 to 32 := 32;
		C_S_AXI_ADDR_WIDTH	: integer range 16 to 16 := 16;

		-- IP generics
		C_SYSTEMCLOCK       : integer := 100_000_000;
		C_CARRIER           : integer := 1;
		C_HW_LINK_REF       : integer := 0;
		C_XADC              : integer := 1
	);
	port
	(
		-- Ports of Axi Slave Bus Interface S_AXI
		S_AXI_ACLK      : in  std_logic;
		S_AXI_ARESETN   : in  std_logic;
		S_AXI_AWADDR    : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 DOWNTO 0);
		S_AXI_AWVALID   : in  std_logic;
		S_AXI_AWREADY   : out std_logic;
		S_AXI_WDATA     : in  std_logic_vector(C_S_AXI_DATA_WIDTH-1 DOWNTO 0);
		S_AXI_WSTRB     : in  std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 DOWNTO 0);
		S_AXI_WVALID    : in  std_logic;
		S_AXI_WREADY    : out std_logic;
		S_AXI_BRESP     : out std_logic_vector(1 DOWNTO 0);
		S_AXI_BVALID    : out std_logic;
		S_AXI_BREADY    : in  std_logic;
		S_AXI_ARADDR    : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 DOWNTO 0);
		S_AXI_ARVALID   : in  std_logic;
		S_AXI_ARREADY   : out std_logic;
		S_AXI_RDATA     : out std_logic_vector(C_S_AXI_DATA_WIDTH-1 DOWNTO 0);
		S_AXI_RRESP     : out std_logic_vector(1 DOWNTO 0);
		S_AXI_RVALID    : out std_logic;
		S_AXI_RREADY    : in  std_logic;
		-- ip ports
		--
		-- s/d ADC
		sd_clock        : in  std_logic;
		--
		fmc_ia_clk      : out std_logic;
		fmc_ib_clk      : out std_logic;
		fmc_it_clk      : out std_logic;
		fmc_vbus_clk    : out std_logic;
		fmc_ia_dat      : in  std_logic;
		fmc_ib_dat      : in  std_logic;
		fmc_it_dat      : in  std_logic;
		fmc_vbus_dat    : in  std_logic;
		-- encoder/hall sensors
		fmc_sensor_a    : in  std_logic;
		fmc_sensor_b    : in  std_logic;
		fmc_sensor_c    : in  std_logic;
		-- inverter
		fmc_pwm_en      : out std_logic;
		fmc_m_fault     : in  std_logic;
		fmc_pwm_ah      : out std_logic;
		fmc_pwm_al      : out std_logic;
		fmc_pwm_bh      : out std_logic;
		fmc_pwm_bl      : out std_logic;
		fmc_pwm_ch      : out std_logic;
		fmc_pwm_cl      : out std_logic;
		-- gpio
		fmc_gpo         : out std_logic_vector(10 downto 0);
		fmc_gpi         : in  std_logic_vector(4 downto 0);
		--
		-- xadc signals
		xadc_vp         : in  std_logic;
		xadc_vn         : in  std_logic;
		xadc_vaux0_p    : in  std_logic;
		xadc_vaux0_n    : in  std_logic;
		xadc_vaux8_p    : in  std_logic;
		xadc_vaux8_n    : in  std_logic;
		-- xadc external mux
		xadc_mux_a0     : out std_logic;
		xadc_mux_a1     : out std_logic;
		xadc_mux_en     : out std_logic;
		--
		-- external angle in
		ext_x_angle     : in  std_logic_vector(31 downto 0) := (others => '0');
		--
		-- resolver in
		rsv_x_priexc    : in  std_logic_vector(17 downto 0) := (others => '0');
		rsv_x_secsin    : in  std_logic_vector(17 downto 0) := (others => '0');
		rsv_x_seccos    : in  std_logic_vector(17 downto 0) := (others => '0');

		-- motor control IP

		-- external angle
		ext_angle     : out   std_logic_vector(31 downto 0);
		
		-- resolver
		rsv_priexc    : out   std_logic_vector(17 downto 0);
		rsv_secsin    : out   std_logic_vector(17 downto 0);
		rsv_seccos    : out   std_logic_vector(17 downto 0);
		
		-- hall sensor
		hls_hallsig   : out   std_logic_vector(5 downto 0);
		
		-- digital encoder
		enc_cha       : out   std_logic;
		enc_chb       : out   std_logic;
		enc_chi       : out   std_logic;
		
		-- input data
		--
		acq_iphs_a    : out  std_logic_vector(17 downto 0);
		acq_iphs_b    : out  std_logic_vector(17 downto 0);
		acq_iphs_c    : out  std_logic_vector(17 downto 0);
		acq_ibus_x    : out  std_logic_vector(17 downto 0);
		acq_vphs_a    : out  std_logic_vector(17 downto 0);
		acq_vphs_b    : out  std_logic_vector(17 downto 0);
		acq_vphs_c    : out  std_logic_vector(17 downto 0);
		acq_vbus_x    : out  std_logic_vector(17 downto 0);
		acq_vphs_n    : out  std_logic_vector(17 downto 0);
		--
		acq_sync      : out  std_logic;
		
		-- output offset
		--
		ofs_iphs_a    : in  std_logic_vector(17 downto 0);
		ofs_iphs_b    : in  std_logic_vector(17 downto 0);
		ofs_iphs_c    : in  std_logic_vector(17 downto 0);
		ofs_ibus_x    : in  std_logic_vector(17 downto 0);
		ofs_vphs_a    : in  std_logic_vector(17 downto 0);
		ofs_vphs_b    : in  std_logic_vector(17 downto 0);
		ofs_vphs_c    : in  std_logic_vector(17 downto 0);
		ofs_vbus_x    : in  std_logic_vector(17 downto 0);
		ofs_vphs_n    : in  std_logic_vector(17 downto 0);

		-- normalized values (offset removed)
		--
		nrm_iphs_a    : in  std_logic_vector(17 downto 0);
		nrm_iphs_b    : in  std_logic_vector(17 downto 0);
		nrm_iphs_c    : in  std_logic_vector(17 downto 0);
		nrm_ibus_x    : in  std_logic_vector(17 downto 0);
		nrm_vphs_a    : in  std_logic_vector(17 downto 0);
		nrm_vphs_b    : in  std_logic_vector(17 downto 0);
		nrm_vphs_c    : in  std_logic_vector(17 downto 0);
		nrm_vbus_x    : in  std_logic_vector(17 downto 0);
		nrm_vphs_n    : in  std_logic_vector(17 downto 0);
		--
		nrm_sync      : in  std_logic;

		-- filtered values
		--
		flt_iphs_a    : in  std_logic_vector(17 downto 0);
		flt_iphs_b    : in  std_logic_vector(17 downto 0);
		flt_iphs_c    : in  std_logic_vector(17 downto 0);
		flt_ibus_x    : in  std_logic_vector(17 downto 0);
		flt_vphs_a    : in  std_logic_vector(17 downto 0);
		flt_vphs_b    : in  std_logic_vector(17 downto 0);
		flt_vphs_c    : in  std_logic_vector(17 downto 0);
		flt_vbus_x    : in  std_logic_vector(17 downto 0);
		flt_vphs_n    : in  std_logic_vector(17 downto 0);
		--
		flt_sync      : in  std_logic;

		-- current overflow
		--
		ovi_iphs_a    : in  std_logic;
		ovi_iphs_b    : in  std_logic;
		ovi_iphs_c    : in  std_logic;
		ovi_ibus_x    : in  std_logic;

		-- Is overflow
		ismodover     : in  std_logic;
		
		-- 1=generator, 0=motor
		generator     : in  std_logic;
		
		-- modulator type
		modtype       : in  std_logic_vector(0 downto 0);
		
		-- modulator levels
		modlevels     : in  std_logic_vector(0 downto 0);
		
		-- motor is 3-phases (0=bipolar)
		is3phases     : in  std_logic;
		
		-- pwm 3-phases outputs
		pwm2p_l2c1w1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c1w2  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w2  : in  std_logic_vector(1 downto 0);
		pwm2p_l2sync  : in  std_logic;
		
		-- pwm 2-phases outputs
		pwm3p_l2cxw1  : in  std_logic_vector(2 downto 0);
		pwm3p_l2cxw2  : in  std_logic_vector(2 downto 0);
		pwm3p_l2sync  : in  std_logic;
		
		-- rpfm 3-ph two levels outputs
		rpfm3p_l2cxw1 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2cxw2 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2sync : in  std_logic;
		
		-- rpfm 3-ph three levels outputs
		rpfm3p_l3c1w1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c1w2 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w2 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w2 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3sync : in  std_logic;
		
		-- dead time value
		deadtval      : in  std_logic_vector(7 downto 0);
		
		-- gate unit switches
		guswcnt       : out  std_logic_vector(31 downto 0);
		
		-- in-failure
		hwfail        : out  std_logic;
		
		-- in-kill
		hwkill        : out  std_logic;
		
		-- coil driver
		coilgear      : in  std_logic;
		coilenab      : in  std_logic;
		
		-- system error
		syserr        : in  std_logic;
		
		-- reset pending errors
		reseterr      : in  std_logic;
		
		-- zero current offset 
		zerocurr      : in  std_logic;
		
		-- zero vbus offset 
		zerovbusx     : in  std_logic;

		-- zero vphs offset 
		zerovphsx     : in  std_logic;
		
		-- control probes
		pix_setval    : in std_logic_vector(17 downto 0);
		piy_setval    : in std_logic_vector(17 downto 0);
		
		pix_fbkval    : in std_logic_vector(17 downto 0);
		piy_fbkval    : in std_logic_vector(17 downto 0);
		
		pix_outval    : in std_logic_vector(17 downto 0);
		piy_outval    : in std_logic_vector(17 downto 0);
		
		-- Vs
		vsmodval      : in  std_logic_vector(16 downto 0);
		vsangval      : in  std_logic_vector(31 downto 0);
		
		-- Is
		ismodval      : in  std_logic_vector(16 downto 0);
		isangval      : in  std_logic_vector(31 downto 0);

		-- smo
		smo_bemf_p    : in  std_logic_vector(31 downto 0);
		smo_bemf_m    : in  std_logic_vector(16 downto 0);
		
		-- e-angles
		mrt_angle     : in  std_logic_vector(31 downto 0);
		enc_angle     : in  std_logic_vector(31 downto 0);
		hls_angle     : in  std_logic_vector(31 downto 0);
		rsv_angle     : in  std_logic_vector(31 downto 0);
		smo_angle     : in  std_logic_vector(31 downto 0);
		
		-- Rotor angle
		rot_angle     : in  std_logic_vector(31 downto 0);
		
		-- rot selector
		rot_select    : in  std_logic_vector( 2 downto 0);
		
		-- actual speed
		spd_speed     : in  std_logic_vector(31 downto 0);
		
		-- position
		pos_position  : in  std_logic_vector(31 downto 0);
		
		-- foc cycle time
		tsclocks      : in  std_logic_vector(11 downto 0)
	);
end entity axi_motcon1_drv;

architecture Behavioral of axi_motcon1_drv is

	component axi4ltiny
		generic
		(
			C_S_AXI_DATA_WIDTH : integer range 32 to 32 := 32;
			C_S_AXI_ADDR_WIDTH : integer range  1 to 32 := 16
		);
		port
		(
			-- AXI4LITE
			S_AXI_ACLK    : in  std_logic;
			S_AXI_ARESETN : in  std_logic;
			S_AXI_AWADDR  : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
			S_AXI_AWVALID : in  std_logic;
			S_AXI_AWREADY : out std_logic;
			S_AXI_WDATA   : in  std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
			S_AXI_WSTRB   : in  std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 downto 0);
			S_AXI_WVALID  : in  std_logic;
			S_AXI_WREADY  : out std_logic;
			S_AXI_BRESP   : out std_logic_vector(1 downto 0);
			S_AXI_BVALID  : out std_logic;
			S_AXI_BREADY  : in  std_logic;
			S_AXI_ARADDR  : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
			S_AXI_ARVALID : in  std_logic;
			S_AXI_ARREADY : out std_logic;
			S_AXI_RDATA   : out std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
			S_AXI_RRESP   : out std_logic_vector(1 downto 0);
			S_AXI_RVALID  : out std_logic;
			S_AXI_RREADY  : in  std_logic;
			-- local bus  
			host_clk      : out std_logic;
			host_rst      : out std_logic;
			host_en       : out std_logic;
			host_we       : out std_logic;
			host_addr     : out std_logic_vector(31 downto 0);
			host_din      : out std_logic_vector(31 downto 0);
			host_dout     : in  std_logic_vector(31 downto 0);
			host_ack      : in  std_logic
		);
	end component;
	
	component motcon1sys
		generic
		(
			C_FAMILY      : string  := "zynq";
			C_SYSTEMCLOCK : integer := 100_000_000;
			C_XADC        : integer := 1
		);
		port 
		(
			clock           : in  std_logic;
			reset           : in  std_logic;
			--
			-- host interface
			--
			host_en         : in  std_logic;
			host_we         : in  std_logic;
			host_addr       : in  std_logic_vector(31 downto 0);
			host_din        : in  std_logic_vector(31 downto 0);
			host_dout       : out std_logic_vector(31 downto 0);
			host_ack        : out std_logic;
			--
			-- s/d ADC
			sd_clock        : in  std_logic;
			--
			fmc_ia_clk      : out std_logic;
			fmc_ib_clk      : out std_logic;
			fmc_it_clk      : out std_logic;
			fmc_vbus_clk    : out std_logic;
			fmc_ia_dat      : in  std_logic;
			fmc_ib_dat      : in  std_logic;
			fmc_it_dat      : in  std_logic;
			fmc_vbus_dat    : in  std_logic;
			-- encoder/hall sensors
			fmc_sensor_a    : in  std_logic;
			fmc_sensor_b    : in  std_logic;
			fmc_sensor_c    : in  std_logic;
			-- inverter
			fmc_pwm_en      : out std_logic;
			fmc_m_fault     : in  std_logic;
			fmc_pwm_ah      : out std_logic;
			fmc_pwm_al      : out std_logic;
			fmc_pwm_bh      : out std_logic;
			fmc_pwm_bl      : out std_logic;
			fmc_pwm_ch      : out std_logic;
			fmc_pwm_cl      : out std_logic;
			-- gpio
			fmc_gpo         : out std_logic_vector(10 downto 0);
			fmc_gpi         : in  std_logic_vector(4 downto 0);
			--
			-- xadc signals
			xadc_vp         : in  std_logic;
			xadc_vn         : in  std_logic;
			xadc_vaux0_p    : in  std_logic;
			xadc_vaux0_n    : in  std_logic;
			xadc_vaux8_p    : in  std_logic;
			xadc_vaux8_n    : in  std_logic;
			-- xadc external mux
			xadc_mux_a0     : out std_logic;
			xadc_mux_a1     : out std_logic;
			xadc_mux_en     : out std_logic;
			--
			--
			-- external angle in
			ext_x_angle     : in  std_logic_vector(31 downto 0);
			--
			-- resolver in
			rsv_x_priexc    : in  std_logic_vector(17 downto 0);
			rsv_x_secsin    : in  std_logic_vector(17 downto 0);
			rsv_x_seccos    : in  std_logic_vector(17 downto 0);

			-- motor control IP

			-- external angle
			ext_angle     : out   std_logic_vector(31 downto 0);
			
			-- resolver
			rsv_priexc    : out   std_logic_vector(17 downto 0);
			rsv_secsin    : out   std_logic_vector(17 downto 0);
			rsv_seccos    : out   std_logic_vector(17 downto 0);
			
			-- hall sensor
			hls_hallsig   : out   std_logic_vector(5 downto 0);
			
			-- digital encoder
			enc_cha       : out   std_logic;
			enc_chb       : out   std_logic;
			enc_chi       : out   std_logic;
			
			-- input data
			--
			acq_iphs_a    : out  std_logic_vector(17 downto 0);
			acq_iphs_b    : out  std_logic_vector(17 downto 0);
			acq_iphs_c    : out  std_logic_vector(17 downto 0);
			acq_ibus_x    : out  std_logic_vector(17 downto 0);
			acq_vphs_a    : out  std_logic_vector(17 downto 0);
			acq_vphs_b    : out  std_logic_vector(17 downto 0);
			acq_vphs_c    : out  std_logic_vector(17 downto 0);
			acq_vbus_x    : out  std_logic_vector(17 downto 0);
			acq_vphs_n    : out  std_logic_vector(17 downto 0);
			--
			acq_sync      : out  std_logic;
			
			-- output offset
			--
			ofs_iphs_a    : in  std_logic_vector(17 downto 0);
			ofs_iphs_b    : in  std_logic_vector(17 downto 0);
			ofs_iphs_c    : in  std_logic_vector(17 downto 0);
			ofs_ibus_x    : in  std_logic_vector(17 downto 0);
			ofs_vphs_a    : in  std_logic_vector(17 downto 0);
			ofs_vphs_b    : in  std_logic_vector(17 downto 0);
			ofs_vphs_c    : in  std_logic_vector(17 downto 0);
			ofs_vbus_x    : in  std_logic_vector(17 downto 0);
			ofs_vphs_n    : in  std_logic_vector(17 downto 0);

			-- normalized values (offset removed)
			--
			nrm_iphs_a    : in  std_logic_vector(17 downto 0);
			nrm_iphs_b    : in  std_logic_vector(17 downto 0);
			nrm_iphs_c    : in  std_logic_vector(17 downto 0);
			nrm_ibus_x    : in  std_logic_vector(17 downto 0);
			nrm_vphs_a    : in  std_logic_vector(17 downto 0);
			nrm_vphs_b    : in  std_logic_vector(17 downto 0);
			nrm_vphs_c    : in  std_logic_vector(17 downto 0);
			nrm_vbus_x    : in  std_logic_vector(17 downto 0);
			nrm_vphs_n    : in  std_logic_vector(17 downto 0);
			--
			nrm_sync      : in  std_logic;

			-- filtered values
			--
			flt_iphs_a    : in  std_logic_vector(17 downto 0);
			flt_iphs_b    : in  std_logic_vector(17 downto 0);
			flt_iphs_c    : in  std_logic_vector(17 downto 0);
			flt_ibus_x    : in  std_logic_vector(17 downto 0);
			flt_vphs_a    : in  std_logic_vector(17 downto 0);
			flt_vphs_b    : in  std_logic_vector(17 downto 0);
			flt_vphs_c    : in  std_logic_vector(17 downto 0);
			flt_vbus_x    : in  std_logic_vector(17 downto 0);
			flt_vphs_n    : in  std_logic_vector(17 downto 0);
			--
			flt_sync      : in  std_logic;

			-- current overflow
			--
			ovi_iphs_a    : in  std_logic;
			ovi_iphs_b    : in  std_logic;
			ovi_iphs_c    : in  std_logic;
			ovi_ibus_x    : in  std_logic;

			-- Is overflow
			ismodover     : in  std_logic;
			
			-- 1=generator, 0=motor
			generator     : in  std_logic;
			
			-- modulator type
			modtype       : in  std_logic_vector(0 downto 0);
			
			-- modulator levels
			modlevels     : in  std_logic_vector(0 downto 0);
			
			-- motor is 3-phases (0=bipolar)
			is3phases     : in  std_logic;
			
			-- pwm 3-phases outputs
			pwm2p_l2c1w1  : in  std_logic_vector(1 downto 0);
			pwm2p_l2c2w1  : in  std_logic_vector(1 downto 0);
			pwm2p_l2c1w2  : in  std_logic_vector(1 downto 0);
			pwm2p_l2c2w2  : in  std_logic_vector(1 downto 0);
			pwm2p_l2sync  : in  std_logic;
			
			-- pwm 2-phases outputs
			pwm3p_l2cxw1  : in  std_logic_vector(2 downto 0);
			pwm3p_l2cxw2  : in  std_logic_vector(2 downto 0);
			pwm3p_l2sync  : in  std_logic;
			
			-- rpfm 3-ph two levels outputs
			rpfm3p_l2cxw1 : in  std_logic_vector(2 downto 0);
			rpfm3p_l2cxw2 : in  std_logic_vector(2 downto 0);
			rpfm3p_l2sync : in  std_logic;
			
			-- rpfm 3-ph three levels outputs
			rpfm3p_l3c1w1 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c2w1 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c3w1 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c1w2 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c2w2 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c3w2 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3sync : in  std_logic;
			
			-- dead time value
			deadtval      : in  std_logic_vector(7 downto 0);
			
			-- gate unit switches
			guswcnt       : out  std_logic_vector(31 downto 0);
			
			-- in-failure
			hwfail        : out  std_logic;
			
			-- in-kill
			hwkill        : out  std_logic;
			
			-- coil driver
			coilgear      : in  std_logic;
			coilenab      : in  std_logic;
			
			-- system error
			syserr        : in  std_logic;
			
			-- reset pending errors
			reseterr      : in  std_logic;
			
			-- zero current offset 
			zerocurr      : in  std_logic;
			
			-- zero vbus offset 
			zerovbusx     : in  std_logic;

			-- zero vphs offset 
			zerovphsx     : in  std_logic;
			
			-- control probes
			pix_setval    : in std_logic_vector(17 downto 0);
			piy_setval    : in std_logic_vector(17 downto 0);
			
			pix_fbkval    : in std_logic_vector(17 downto 0);
			piy_fbkval    : in std_logic_vector(17 downto 0);
			
			pix_outval    : in std_logic_vector(17 downto 0);
			piy_outval    : in std_logic_vector(17 downto 0);
			
			-- Vs
			vsmodval      : in  std_logic_vector(16 downto 0);
			vsangval      : in  std_logic_vector(31 downto 0);
			
			-- Is
			ismodval      : in  std_logic_vector(16 downto 0);
			isangval      : in  std_logic_vector(31 downto 0);

			-- smo
			smo_bemf_p    : in  std_logic_vector(31 downto 0);
			smo_bemf_m    : in  std_logic_vector(16 downto 0);
			
			-- e-angles
			mrt_angle     : in  std_logic_vector(31 downto 0);
			enc_angle     : in  std_logic_vector(31 downto 0);
			hls_angle     : in  std_logic_vector(31 downto 0);
			rsv_angle     : in  std_logic_vector(31 downto 0);
			smo_angle     : in  std_logic_vector(31 downto 0);
			
			-- Rotor angle
			rot_angle     : in  std_logic_vector(31 downto 0);
			
			-- rot selector
			rot_select    : in  std_logic_vector( 2 downto 0);
			
			-- actual speed
			spd_speed     : in  std_logic_vector(31 downto 0);
			
			-- position
			pos_position  : in  std_logic_vector(31 downto 0);
			
			-- foc cycle time
			tsclocks      : in  std_logic_vector(11 downto 0)
		);
	end component;

	-- wrapper

	signal host_clk        : std_logic;
	signal host_rst        : std_logic;
	--
	signal host_en         : std_logic;
	signal host_we         : std_logic;
	signal host_addr       : std_logic_vector(31 downto 0);
	signal host_din        : std_logic_vector(31 downto 0);
	signal host_dout       : std_logic_vector(31 downto 0);
	signal host_ack        : std_logic;

begin

	assert C_S_AXI_ACLK_FREQ_HZ = C_SYSTEMCLOCK report "AXI clock /= system clock" severity failure;
	assert false report "C_SYSTEMCLOCK=" & integer'image(C_SYSTEMCLOCK) severity note;
	
	AXI_LITE_IPIF_I : axi4ltiny
		generic map
		(
			C_S_AXI_DATA_WIDTH => C_S_AXI_DATA_WIDTH,
			C_S_AXI_ADDR_WIDTH => C_S_AXI_ADDR_WIDTH
		)
		port map
		(
			-- AXI4LITE
			S_AXI_ACLK    => S_AXI_ACLK,
			S_AXI_ARESETN => S_AXI_ARESETN,
			S_AXI_AWADDR  => S_AXI_AWADDR,
			S_AXI_AWVALID => S_AXI_AWVALID,
			S_AXI_AWREADY => S_AXI_AWREADY,
			S_AXI_WDATA   => S_AXI_WDATA,
			S_AXI_WSTRB   => S_AXI_WSTRB,
			S_AXI_WVALID  => S_AXI_WVALID,
			S_AXI_WREADY  => S_AXI_WREADY,
			S_AXI_BRESP   => S_AXI_BRESP,
			S_AXI_BVALID  => S_AXI_BVALID,
			S_AXI_BREADY  => S_AXI_BREADY,
			S_AXI_ARADDR  => S_AXI_ARADDR,
			S_AXI_ARVALID => S_AXI_ARVALID,
			S_AXI_ARREADY => S_AXI_ARREADY,
			S_AXI_RDATA   => S_AXI_RDATA,
			S_AXI_RRESP   => S_AXI_RRESP,
			S_AXI_RVALID  => S_AXI_RVALID,
			S_AXI_RREADY  => S_AXI_RREADY,
			-- local bus  
			host_clk      => host_clk,
			host_rst      => host_rst,
			host_en       => host_en,
			host_we       => host_we,
			host_addr     => host_addr,
			host_din      => host_din,
			host_dout     => host_dout,
			host_ack      => host_ack
		);

  motcon1sys_inst : motcon1sys
		generic map
		(
			C_FAMILY      => C_FAMILY,
			C_SYSTEMCLOCK => C_SYSTEMCLOCK,
			C_XADC        => C_XADC
		)
		port  map
		(
			clock           => host_clk,
			reset           => host_rst,
			--
			-- host interface
			--
			host_en         => host_en,
			host_we         => host_we,
			host_addr       => host_addr,
			host_din        => host_din,
			host_dout       => host_dout,
			host_ack        => host_ack,
			--
			-- s/d ADC
			sd_clock        => sd_clock,
			--
			fmc_ia_clk      => fmc_ia_clk,
			fmc_ib_clk      => fmc_ib_clk,
			fmc_it_clk      => fmc_it_clk,
			fmc_vbus_clk    => fmc_vbus_clk,
			fmc_ia_dat      => fmc_ia_dat,
			fmc_ib_dat      => fmc_ib_dat,
			fmc_it_dat      => fmc_it_dat,
			fmc_vbus_dat    => fmc_vbus_dat,
			-- encoder/hall sensors
			fmc_sensor_a    => fmc_sensor_a,
			fmc_sensor_b    => fmc_sensor_b,
			fmc_sensor_c    => fmc_sensor_c,
			-- inverter
			fmc_pwm_en      => fmc_pwm_en,
			fmc_m_fault     => fmc_m_fault,
			fmc_pwm_ah      => fmc_pwm_ah,
			fmc_pwm_al      => fmc_pwm_al,
			fmc_pwm_bh      => fmc_pwm_bh,
			fmc_pwm_bl      => fmc_pwm_bl,
			fmc_pwm_ch      => fmc_pwm_ch,
			fmc_pwm_cl      => fmc_pwm_cl,
			-- gpio
			fmc_gpo         => fmc_gpo,
			fmc_gpi         => fmc_gpi,
			--
			-- xadc signals
			xadc_vp         => xadc_vp,
			xadc_vn         => xadc_vn,
			xadc_vaux0_p    => xadc_vaux0_p,
			xadc_vaux0_n    => xadc_vaux0_n,
			xadc_vaux8_p    => xadc_vaux8_p,
			xadc_vaux8_n    => xadc_vaux8_n,
			-- xadc external mux
			xadc_mux_a0     => xadc_mux_a0,
			xadc_mux_a1     => xadc_mux_a1,
			xadc_mux_en     => xadc_mux_en,

			-- external angle in
			ext_x_angle     => ext_x_angle,
			
			-- resolver in
			rsv_x_priexc    => rsv_x_priexc,
			rsv_x_secsin    => rsv_x_secsin,
			rsv_x_seccos    => rsv_x_seccos,
			
			-- motor control IP

			-- external angle
			ext_angle     => ext_angle,
			
			-- resolver
			rsv_priexc    => rsv_priexc,
			rsv_secsin    => rsv_secsin,
			rsv_seccos    => rsv_seccos,
			
			-- hall sensor
			hls_hallsig   => hls_hallsig,
			
			-- digital encoder
			enc_cha       => enc_cha,
			enc_chb       => enc_chb,
			enc_chi       => enc_chi,
			
			-- input data
			--
			acq_iphs_a    => acq_iphs_a,
			acq_iphs_b    => acq_iphs_b,
			acq_iphs_c    => acq_iphs_c,
			acq_ibus_x    => acq_ibus_x,
			acq_vphs_a    => acq_vphs_a,
			acq_vphs_b    => acq_vphs_b,
			acq_vphs_c    => acq_vphs_c,
			acq_vbus_x    => acq_vbus_x,
			acq_vphs_n    => acq_vphs_n,
			--
			acq_sync      => acq_sync,
			
			-- output offset
			--
			ofs_iphs_a    => ofs_iphs_a,
			ofs_iphs_b    => ofs_iphs_b,
			ofs_iphs_c    => ofs_iphs_c,
			ofs_ibus_x    => ofs_ibus_x,
			ofs_vphs_a    => ofs_vphs_a,
			ofs_vphs_b    => ofs_vphs_b,
			ofs_vphs_c    => ofs_vphs_c,
			ofs_vbus_x    => ofs_vbus_x,
			ofs_vphs_n    => ofs_vphs_n,

			-- normalized values (offset removed)
			--
			nrm_iphs_a    => nrm_iphs_a,
			nrm_iphs_b    => nrm_iphs_b,
			nrm_iphs_c    => nrm_iphs_c,
			nrm_ibus_x    => nrm_ibus_x,
			nrm_vphs_a    => nrm_vphs_a,
			nrm_vphs_b    => nrm_vphs_b,
			nrm_vphs_c    => nrm_vphs_c,
			nrm_vbus_x    => nrm_vbus_x,
			nrm_vphs_n    => nrm_vphs_n,
			--
			nrm_sync      => nrm_sync,

			-- filtered values
			--
			flt_iphs_a    => flt_iphs_a,
			flt_iphs_b    => flt_iphs_b,
			flt_iphs_c    => flt_iphs_c,
			flt_ibus_x    => flt_ibus_x,
			flt_vphs_a    => flt_vphs_a,
			flt_vphs_b    => flt_vphs_b,
			flt_vphs_c    => flt_vphs_c,
			flt_vbus_x    => flt_vbus_x,
			flt_vphs_n    => flt_vphs_n,
			--
			flt_sync      => flt_sync,

			-- current overflow
			--
			ovi_iphs_a    => ovi_iphs_a,
			ovi_iphs_b    => ovi_iphs_b,
			ovi_iphs_c    => ovi_iphs_c,
			ovi_ibus_x    => ovi_ibus_x,

			-- Is overflow
			ismodover     => ismodover,
			
			-- 1=generator, 0=motor
			generator     => generator,
			
			-- modulator type
			modtype       => modtype,
			
			-- modulator levels
			modlevels     => modlevels,
			
			-- motor is 3-phases (0=bipolar)
			is3phases     => is3phases,
			
			-- pwm 3-phases outputs
			pwm2p_l2c1w1  => pwm2p_l2c1w1,
			pwm2p_l2c2w1  => pwm2p_l2c2w1,
			pwm2p_l2c1w2  => pwm2p_l2c1w2,
			pwm2p_l2c2w2  => pwm2p_l2c2w2,
			pwm2p_l2sync  => pwm2p_l2sync,
			
			-- pwm 2-phases outputs
			pwm3p_l2cxw1  => pwm3p_l2cxw1,
			pwm3p_l2cxw2  => pwm3p_l2cxw2,
			pwm3p_l2sync  => pwm3p_l2sync,
			
			-- rpfm 3-ph two levels outputs
			rpfm3p_l2cxw1 => rpfm3p_l2cxw1,
			rpfm3p_l2cxw2 => rpfm3p_l2cxw2,
			rpfm3p_l2sync => rpfm3p_l2sync,
			
			-- rpfm 3-ph three levels outputs
			rpfm3p_l3c1w1 => rpfm3p_l3c1w1,
			rpfm3p_l3c2w1 => rpfm3p_l3c2w1,
			rpfm3p_l3c3w1 => rpfm3p_l3c3w1,
			rpfm3p_l3c1w2 => rpfm3p_l3c1w2,
			rpfm3p_l3c2w2 => rpfm3p_l3c2w2,
			rpfm3p_l3c3w2 => rpfm3p_l3c3w2,
			rpfm3p_l3sync => rpfm3p_l3sync,
			
			-- dead time value
			deadtval      => deadtval,
			
			-- gate unit switches
			guswcnt       => guswcnt,
			
			-- in-failure
			hwfail        => hwfail,
			
			-- in-kill
			hwkill        => hwkill,
			
			-- coil driver
			coilgear      => coilgear,
			coilenab      => coilenab,
			
			-- system error
			syserr        => syserr,
			
			-- reset pending errors
			reseterr      => reseterr,
			
			-- zero current offset 
			zerocurr      => zerocurr,
			
			-- zero vbus offset 
			zerovbusx     => zerovbusx,

			-- zero vphs offset 
			zerovphsx     => zerovphsx,
			
			-- control probes
			pix_setval    => pix_setval,
			piy_setval    => piy_setval,
			
			pix_fbkval    => pix_fbkval,
			piy_fbkval    => piy_fbkval,
			
			pix_outval    => pix_outval,
			piy_outval    => piy_outval,
			
			-- Vs
			vsmodval      => vsmodval,
			vsangval      => vsangval,
			
			-- Is
			ismodval      => ismodval,
			isangval      => isangval,

			-- smo
			smo_bemf_p    => smo_bemf_p,
			smo_bemf_m    => smo_bemf_m,

			-- e-angles
			mrt_angle     => mrt_angle,
			enc_angle     => enc_angle,
			hls_angle     => hls_angle,
			rsv_angle     => rsv_angle,
			smo_angle     => smo_angle,

			-- Rotor angle
			rot_angle     => rot_angle,
			
			-- rot selector
			rot_select    => rot_select,
			
			-- actual speed
			spd_speed     => spd_speed,
			
			-- position
			pos_position  => pos_position,
			
			-- foc cycle time
			tsclocks      => tsclocks
		);

end Behavioral;
