-------------------------------------------------------------------------------------------
-- Copyright  2011-2014, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    15:33:27 01/08/2014 
-- Design Name: 
-- Module Name:    mcbxadc - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity mcbxadc is
	generic
	(
		C_SYSTEMCLOCK   : integer := 100_000_000
	);
	port
	(
		clock          : in  std_logic;
		reset          : in  std_logic;
		-- temp & vbus values
		val_temp       : out std_logic_vector(17 downto 0);
		val_vbus       : out std_logic_vector(17 downto 0);
		-- motor currents
		val_ia         : out std_logic_vector(17 downto 0);
		val_ib         : out std_logic_vector(17 downto 0);
		-- sync
		val_sync       : out std_logic;
		-- xadc drp access
		xadc_drp_daddr : in  std_logic_vector(6 downto 0);
		xadc_drp_di    : in  std_logic_vector(15 downto 0);
		xadc_drp_den   : in  std_logic;
		xadc_drp_dwe   : in  std_logic;
		xadc_drp_do    : out std_logic_vector(15 downto 0);
		xadc_drp_drdy  : out std_logic;
		-- xadc analog signals
		xadc_vp        : in  std_logic;
		xadc_vn        : in  std_logic;
		xadc_vaux0_p   : in  std_logic;
		xadc_vaux0_n   : in  std_logic;
		xadc_vaux8_p   : in  std_logic;
		xadc_vaux8_n   : in  std_logic;
		-- xadc external mux
		xadc_mux_a0    : out std_logic;
		xadc_mux_a1    : out std_logic
	);
end mcbxadc;

architecture Behavioral of mcbxadc is

	---------------------------------------------------------------------------
	
	function xadc_eval_divclk(sysclock : integer; xadcclk : integer) return integer is
		variable divisor : integer;
	begin
		divisor := 1;
		loop
			exit when (sysclock / divisor) <= xadcclk;
			divisor := divisor + 1;
		end loop;
		return divisor;
	end function;
	
	---------------------------------------------------------------------------
	constant C_XADC_ADCCLK      : integer := 26_000_000;
	constant C_XADC_INIT_42_CD  : integer := xadc_eval_divclk(C_SYSTEMCLOCK,C_XADC_ADCCLK);
	constant C_XADC_INIT_42_INT : integer := C_XADC_INIT_42_CD * 256;
	constant C_XADC_INIT_42_REG : bit_vector(15 downto 0) := to_bitvector(conv_std_logic_vector(C_XADC_INIT_42_INT,16));
	---------------------------------------------------------------------------
	--
	-- simultaneous sampling mode with external mux
	-- 48h: 0x0100: 8=on-chip temp
	-- 49h: 0x0005: 0=ia,ib, 2=vbus
	-- 4ch: 0x0000: unipolar
	-- 4dh: 0x0000: unipolar
	-- 4eh: 0x0000: settling time of 48h, oc-temp @ 26 clocks
	-- 4fh: 0x0005: settling time of 49f, ia/ib, vbus @ 32 clocks
	-- 3 conversions, 2 @ 32 clocks + 1 @ 26 clocks = 90 ADCCLKs
	--
	-- configuration parameters
	constant C_XADC_INIT_40 : bit_vector(15 downto 0) := X"1D10";
	constant C_XADC_INIT_41 : bit_vector(15 downto 0) := X"410F";
	constant C_XADC_INIT_42 : bit_vector(15 downto 0) := C_XADC_INIT_42_REG;
	--
	constant C_XADC_INIT_48 : bit_vector(15 downto 0) := X"0100";
	constant C_XADC_INIT_49 : bit_vector(15 downto 0) := X"0005";
	constant C_XADC_INIT_4A : bit_vector(15 downto 0) := X"0000";
	constant C_XADC_INIT_4B : bit_vector(15 downto 0) := X"0000";
	constant C_XADC_INIT_4C : bit_vector(15 downto 0) := X"0000";
	constant C_XADC_INIT_4D : bit_vector(15 downto 0) := X"0000";
	constant C_XADC_INIT_4E : bit_vector(15 downto 0) := X"0000";
	constant C_XADC_INIT_4F : bit_vector(15 downto 0) := X"0005";
	--
	constant C_XADC_INIT_50 : bit_vector(15 downto 0) := X"b5ed";
	constant C_XADC_INIT_51 : bit_vector(15 downto 0) := X"5999";
	constant C_XADC_INIT_52 : bit_vector(15 downto 0) := X"e000";
	constant C_XADC_INIT_53 : bit_vector(15 downto 0) := X"ca33";
	constant C_XADC_INIT_54 : bit_vector(15 downto 0) := X"a93a";
	constant C_XADC_INIT_55 : bit_vector(15 downto 0) := X"5111";
	constant C_XADC_INIT_56 : bit_vector(15 downto 0) := X"caaa";
	constant C_XADC_INIT_57 : bit_vector(15 downto 0) := X"ae4e";
	constant C_XADC_INIT_58 : bit_vector(15 downto 0) := X"5999";
	constant C_XADC_INIT_5C : bit_vector(15 downto 0) := X"5111";
	--
	constant C_XADC_SIM_DEVICE       : string := "7SERIES";
	constant C_XADC_SIM_MONITOR_FILE : string := "design.txt";

	-- XADC signals wrapper to save orignal name
	

      -- ALARMS: 8-bit (each) output: ALM, OT
	signal ALM_i          : std_logic_vector(7 downto 0);  -- 8-bit output: Output alarm for temp, Vccint, Vccaux and Vccbram
	signal OT_i           : std_logic;                     -- 1-bit output: Over-Temperature alarm
      -- Dynamic Reconfiguration Port (DRP): 16-bit (each) output: Dynamic Reconfiguration Ports
	signal DO_i           : std_logic_vector(15 downto 0); -- 16-bit output: DRP output data bus
	signal DRDY_i         : std_logic;                     -- 1-bit output: DRP data ready
      -- STATUS: 1-bit (each) output: XADC status ports
	signal BUSY_i         : std_logic;                     -- 1-bit output: ADC busy output
	signal CHANNEL_i      : std_logic_vector(4 downto 0);  -- 5-bit output: Channel selection outputs
	signal EOC_i          : std_logic;                     -- 1-bit output: End of Conversion
	signal EOS_i          : std_logic;                     -- 1-bit output: End of Sequence
	signal JTAGBUSY_i     : std_logic;                     -- 1-bit output: JTAG DRP transaction in progress output
	signal JTAGLOCKED_i   : std_logic;                     -- 1-bit output: JTAG requested DRP port lock
	signal JTAGMODIFIED_i : std_logic;                     -- 1-bit output: JTAG Write to the DRP has occurred
	signal MUXADDR_i      : std_logic_vector(4 downto 0);  -- 5-bit output: External MUX channel decode
      -- Auxiliary Analog-Input Pairs: 16-bit (each) input: VAUXP[15:0], VAUXN[15:0]
	signal VAUXN_i        : std_logic_vector(15 downto 0); -- 16-bit input: N-side auxiliary analog input
	signal VAUXP_i        : std_logic_vector(15 downto 0); -- 16-bit input: P-side auxiliary analog input
      -- CONTROL and CLOCK: 1-bit (each) input: Reset, conversion start and clock inputs
	signal CONVST_i       : std_logic;                     -- 1-bit input: Convert start input
	signal CONVSTCLK_i    : std_logic;                     -- 1-bit input: Convert start input
	signal RESET_i        : std_logic;                     -- 1-bit input: Active-high reset
      -- Dedicated Analog Input Pair: 1-bit (each) input: VP/VN
	signal VN_i           : std_logic;                     -- 1-bit input: N-side analog input
	signal VP_i           : std_logic;                     -- 1-bit input: P-side analog input
      -- Dynamic Reconfiguration Port (DRP): 7-bit (each) input: Dynamic Reconfiguration Ports
	signal DADDR_i        : std_logic_vector(6 downto 0);  -- 7-bit input: DRP address bus
	signal DCLK_i         : std_logic;                     -- 1-bit input: DRP clock
	signal DEN_i          : std_logic;                     -- 1-bit input: DRP enable signal
	signal DI_i           : std_logic_vector(15 downto 0); -- 16-bit input: DRP input data bus
	signal DWE_i          : std_logic;                     -- 1-bit input: DRP write enable
	
	-- states of main process state machine
	type   st_type is ( s0,s1,s2,s3 );
	signal stm : st_type;
	
	-- capture event
	signal eoscnt    : std_logic_vector(1 downto 0);
	signal eosocn    : std_logic_vector(eoscnt'range);
	
	-- XADC Status Register addresses
	
	constant C_DADDR_temp : std_logic_vector(6 downto 0) := "0000000"; -- 00h
	constant C_DADDR_vbus : std_logic_vector(6 downto 0) := "0010010"; -- 12h
	
	constant C_DADDR_ia   : std_logic_vector(6 downto 0) := "0010000"; -- 10h
	constant C_DADDR_ib   : std_logic_vector(6 downto 0) := "0011000"; -- 18h
	
	-- XADC Status Register readed values
	
	signal adc_temp  : std_logic_vector(17 downto 0);
	signal adc_vbus  : std_logic_vector(17 downto 0);
	
	signal adc_ia    : std_logic_vector(17 downto 0);
	signal adc_ib    : std_logic_vector(17 downto 0);
	
	signal acqed     : std_logic;
	
	-- XADC power on reset
	signal   xadc_reset     : std_logic;
	signal   xadc_porc      : std_logic_vector(3 downto 0) := (others => '0');
	constant k_xadc_porc_hi : std_logic_vector(xadc_porc'range) := (others => '1');
	
begin

	------------------------------------------------------------------------
	
	-- on-chip temperature
	val_temp <= adc_temp;
	
	-- dc_link
	val_vbus <= adc_vbus;
	
	-- motor currents
	val_ia   <= adc_ia;
	val_ib   <= adc_ib;
	
	-- end-of-acquisition
	val_sync <= acqed;
	
	------------------------------------------------------------------------
	
	xadc_reset <= '1' when xadc_porc /= k_xadc_porc_hi else '0';
	
	POR_PROC : process(clock)
	begin
		if rising_edge(clock) then
			if xadc_reset = '1' then
				xadc_porc <= xadc_porc + 1;
			end if;
		end if;
	end process;
	
	------------------------------------------------------------------------

	MAIN_PROC : process(clock)
	begin
		if rising_edge(clock) then
			if reset = '1' then
				--
				DI_i           <= (others => '0');
				DADDR_i        <= (others => '0');
				DWE_i          <= '0';
				DEN_i          <= '0';
				--
				xadc_drp_do   <= (others => '0');
				xadc_drp_drdy <= '0';
				--
				eosocn        <= (others => '0');
				eoscnt        <= (others => '0');
				--
				adc_temp      <= (others => '0');
				adc_vbus      <= (others => '0');
				adc_ia        <= (others => '0');
				adc_ib        <= (others => '0');
				--
				acqed         <= '0';
				--
				stm           <= s0;
			else
				-- count end-of-sequence
				if EOS_i = '1' then
					eoscnt <= eoscnt + 1;
				end if;
				-- main state machine
				case stm is
					-- initial state.
					when s0 =>
						xadc_drp_drdy <= '0'; -- single clock pulse for DRP ready
						acqed         <= '0'; -- single clock pulse for end of acquisition
						-- DRP request 
						if xadc_drp_den = '1' then
							-- set for r/w access
							DI_i           <= xadc_drp_di;
							DADDR_i        <= xadc_drp_daddr;
							DWE_i          <= xadc_drp_dwe;
							DEN_i          <= '1';
							stm            <= s3;
						else
							-- operating mode: w4-end-of-sequence event
							if eoscnt /= eosocn then
								eosocn <= eoscnt;
								-- initiate read process
								DADDR_i <= C_DADDR_temp;
								stm     <= s1;
							end if;
						end if;
					when s1 =>
						DEN_i   <= '1';
						stm     <= s2;
					when s2 =>
						DEN_i <= '0';
						if DRDY_i = '1' or RESET_i = '1' then
							case DADDR_i is
								when C_DADDR_temp =>
									adc_temp <= DO_i & "00";
									DADDR_i  <= C_DADDR_vbus;
									stm      <= s1;
								when C_DADDR_vbus =>
									adc_vbus <= DO_i & "00";
									DADDR_i  <= C_DADDR_ia;
									stm      <= s1;
								when C_DADDR_ia =>
									adc_ia   <= DO_i & "00";
									DADDR_i  <= C_DADDR_ib;
									stm      <= s1;
								when C_DADDR_ib =>
									adc_ib   <= DO_i & "00";
									acqed    <= '1';
									stm      <= s0;
								when others =>
									stm      <= s0;
							end case;
						end if;
					-- w4-end-drp
					when s3 =>
						DWE_i <= '0';
						DEN_i <= '0';
						if DRDY_i = '1' or RESET_i = '1' then
							xadc_drp_do   <= DO_i;
							xadc_drp_drdy <= '1';
							stm           <= s0;
						end if;
				end case;
			end if;
		end if;
	end process;
	
	------------------------------------------------------------------------

	CONVST_i    <= '0'; -- not used external trigger
	CONVSTCLK_i <= '0'; -- not used external trigger
	RESET_i     <= reset or xadc_reset;
	DCLK_i      <= clock;

	-- xadc vp/vn
	VP_i        <= xadc_vp;
	VN_i        <= xadc_vn;
	
	-- xadc analog input mux

	-- ADC_A
	VAUXP_i(0)  <= xadc_vaux0_p;
	VAUXN_i(0)  <= xadc_vaux0_n;

	-- ADC_B
	VAUXP_i(8)  <= xadc_vaux8_p;
	VAUXN_i(8)  <= xadc_vaux8_n;
	
	-- unused
	xan : for i in 1 to 7 generate
	begin
		VAUXP_i(i)   <= '0';
		VAUXN_i(i)   <= '0';
		VAUXP_i(i+8) <= '0';
		VAUXN_i(i+8) <= '0';
	end generate;
	
	-- external mux address
	xadc_mux_a0 <= MUXADDR_i(0);
	xadc_mux_a1 <= MUXADDR_i(1);
	
	------------------------------------------------------------------------

	-- the XADC instance
	XADC_inst : XADC
	generic map (
		-- INIT_40 - INIT_42: XADC configuration registers
		INIT_40 => C_XADC_INIT_40,
		INIT_41 => C_XADC_INIT_41,
		INIT_42 => C_XADC_INIT_42,
		-- INIT_48 - INIT_4F: Sequence Registers
		INIT_48 => C_XADC_INIT_48,
		INIT_49 => C_XADC_INIT_49,
		INIT_4A => C_XADC_INIT_4A,
		INIT_4B => C_XADC_INIT_4B,
		INIT_4C => C_XADC_INIT_4C,
		INIT_4D => C_XADC_INIT_4D,
		INIT_4E => C_XADC_INIT_4E,
		INIT_4F => C_XADC_INIT_4F,
		-- INIT_50 - INIT_58, INIT5C: Alarm Limit Registers
		INIT_50 => C_XADC_INIT_50,
		INIT_51 => C_XADC_INIT_51,
		INIT_52 => C_XADC_INIT_52,
		INIT_53 => C_XADC_INIT_53,
		INIT_54 => C_XADC_INIT_54,
		INIT_55 => C_XADC_INIT_55,
		INIT_56 => C_XADC_INIT_56,
		INIT_57 => C_XADC_INIT_57,
		INIT_58 => C_XADC_INIT_58,
		INIT_5C => C_XADC_INIT_5C,
		-- Simulation attributes: Set for proepr simulation behavior
		SIM_DEVICE       => C_XADC_SIM_DEVICE,
		SIM_MONITOR_FILE => C_XADC_SIM_MONITOR_FILE
	)
	port map (
		-- ALARMS: 8-bit (each) output: ALM, OT
		ALM          => ALM_i,          -- 8-bit output: Output alarm for temp, Vccint, Vccaux and Vccbram
		OT           => OT_i,           -- 1-bit output: Over-Temperature alarm
		-- Dynamic Reconfiguration Port (DRP): 16-bit (each) output: Dynamic Reconfiguration Ports
		DO           => DO_i,           -- 16-bit output: DRP output data bus
		DRDY         => DRDY_i,         -- 1-bit output: DRP data ready
		-- STATUS: 1-bit (each) output: XADC status ports
		BUSY         => BUSY_i,         -- 1-bit output: ADC busy output
		CHANNEL      => CHANNEL_i,      -- 5-bit output: Channel selection outputs
		EOC          => EOC_i,          -- 1-bit output: End of Conversion
		EOS          => EOS_i,          -- 1-bit output: End of Sequence
		JTAGBUSY     => JTAGBUSY_i,     -- 1-bit output: JTAG DRP transaction in progress output
		JTAGLOCKED   => JTAGLOCKED_i,   -- 1-bit output: JTAG requested DRP port lock
		JTAGMODIFIED => JTAGMODIFIED_i, -- 1-bit output: JTAG Write to the DRP has occurred
		MUXADDR      => MUXADDR_i,      -- 5-bit output: External MUX channel decode
		-- Auxiliary Analog-Input Pairs: 16-bit (each) input: VAUXP[15:0], VAUXN[15:0]
		VAUXN        => VAUXN_i,        -- 16-bit input: N-side auxiliary analog input
		VAUXP        => VAUXP_i,        -- 16-bit input: P-side auxiliary analog input
		-- CONTROL and CLOCK: 1-bit (each) input: Reset, conversion start and clock inputs
		CONVST       => CONVST_i,       -- 1-bit input: Convert start input
		CONVSTCLK    => CONVSTCLK_i,    -- 1-bit input: Convert start input
		RESET        => RESET_i,        -- 1-bit input: Active-high reset
		-- Dedicated Analog Input Pair: 1-bit (each) input: VP/VN
		VN           => VN_i,           -- 1-bit input: N-side analog input
		VP           => VP_i,           -- 1-bit input: P-side analog input
		-- Dynamic Reconfiguration Port (DRP): 7-bit (each) input: Dynamic Reconfiguration Ports
		DADDR        => DADDR_i,        -- 7-bit input: DRP address bus
		DCLK         => DCLK_i,         -- 1-bit input: DRP clock
		DEN          => DEN_i,          -- 1-bit input: DRP enable signal
		DI           => DI_i,           -- 16-bit input: DRP input data bus
		DWE          => DWE_i           -- 1-bit input: DRP write enable
	);

end Behavioral;

