-------------------------------------------------------------------------------------------
-- Copyright  2011-2019, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    11:32:17 01/06/2014 
-- Design Name: 
-- Module Name:    motcon1sys - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity motcon1sys is
	generic
	(
		C_FAMILY      : string  := "zynq";
		C_SYSTEMCLOCK : integer := 100_000_000;
		C_XADC        : integer := 1
	);
	port 
	(
		clock           : in  std_logic;
		reset           : in  std_logic;
		--
		-- host interface
		--
		host_en         : in  std_logic;
		host_we         : in  std_logic;
		host_addr       : in  std_logic_vector(31 downto 0);
		host_din        : in  std_logic_vector(31 downto 0);
		host_dout       : out std_logic_vector(31 downto 0);
		host_ack        : out std_logic;
		--
		-- s/d ADC
		sd_clock        : in  std_logic;
		--
		fmc_ia_clk      : out std_logic;
		fmc_ib_clk      : out std_logic;
		fmc_it_clk      : out std_logic;
		fmc_vbus_clk    : out std_logic;
		fmc_ia_dat      : in  std_logic;
		fmc_ib_dat      : in  std_logic;
		fmc_it_dat      : in  std_logic;
		fmc_vbus_dat    : in  std_logic;
		-- encoder/hall sensors
		fmc_sensor_a    : in  std_logic;
		fmc_sensor_b    : in  std_logic;
		fmc_sensor_c    : in  std_logic;
		-- inverter
		fmc_pwm_en      : out std_logic;
		fmc_m_fault     : in  std_logic;
		fmc_pwm_ah      : out std_logic;
		fmc_pwm_al      : out std_logic;
		fmc_pwm_bh      : out std_logic;
		fmc_pwm_bl      : out std_logic;
		fmc_pwm_ch      : out std_logic;
		fmc_pwm_cl      : out std_logic;
		-- gpio
		fmc_gpo         : out std_logic_vector(10 downto 0);
		fmc_gpi         : in  std_logic_vector(4 downto 0);
		--
		-- xadc signals
		xadc_vp         : in  std_logic;
		xadc_vn         : in  std_logic;
		xadc_vaux0_p    : in  std_logic;
		xadc_vaux0_n    : in  std_logic;
		xadc_vaux8_p    : in  std_logic;
		xadc_vaux8_n    : in  std_logic;
		-- xadc external mux
		xadc_mux_a0     : out std_logic;
		xadc_mux_a1     : out std_logic;
		xadc_mux_en     : out std_logic;
		--
		--
		-- external angle in
		ext_x_angle     : in  std_logic_vector(31 downto 0);
		--
		-- resolver in
		rsv_x_priexc    : in  std_logic_vector(17 downto 0);
		rsv_x_secsin    : in  std_logic_vector(17 downto 0);
		rsv_x_seccos    : in  std_logic_vector(17 downto 0);

		-- motor control IP

		-- external angle
		ext_angle     : out   std_logic_vector(31 downto 0);
		
		-- resolver
		rsv_priexc    : out   std_logic_vector(17 downto 0);
		rsv_secsin    : out   std_logic_vector(17 downto 0);
		rsv_seccos    : out   std_logic_vector(17 downto 0);
		
		-- hall sensor
		hls_hallsig   : out   std_logic_vector(5 downto 0);
		
		-- digital encoder
		enc_cha       : out   std_logic;
		enc_chb       : out   std_logic;
		enc_chi       : out   std_logic;
		
		-- input data
		--
		acq_iphs_a    : out  std_logic_vector(17 downto 0);
		acq_iphs_b    : out  std_logic_vector(17 downto 0);
		acq_iphs_c    : out  std_logic_vector(17 downto 0);
		acq_ibus_x    : out  std_logic_vector(17 downto 0);
		acq_vphs_a    : out  std_logic_vector(17 downto 0);
		acq_vphs_b    : out  std_logic_vector(17 downto 0);
		acq_vphs_c    : out  std_logic_vector(17 downto 0);
		acq_vbus_x    : out  std_logic_vector(17 downto 0);
		acq_vphs_n    : out  std_logic_vector(17 downto 0);
		--
		acq_sync      : out  std_logic;
		
		-- output offset
		--
		ofs_iphs_a    : in  std_logic_vector(17 downto 0);
		ofs_iphs_b    : in  std_logic_vector(17 downto 0);
		ofs_iphs_c    : in  std_logic_vector(17 downto 0);
		ofs_ibus_x    : in  std_logic_vector(17 downto 0);
		ofs_vphs_a    : in  std_logic_vector(17 downto 0);
		ofs_vphs_b    : in  std_logic_vector(17 downto 0);
		ofs_vphs_c    : in  std_logic_vector(17 downto 0);
		ofs_vbus_x    : in  std_logic_vector(17 downto 0);
		ofs_vphs_n    : in  std_logic_vector(17 downto 0);

		-- normalized values (offset removed)
		--
		nrm_iphs_a    : in  std_logic_vector(17 downto 0);
		nrm_iphs_b    : in  std_logic_vector(17 downto 0);
		nrm_iphs_c    : in  std_logic_vector(17 downto 0);
		nrm_ibus_x    : in  std_logic_vector(17 downto 0);
		nrm_vphs_a    : in  std_logic_vector(17 downto 0);
		nrm_vphs_b    : in  std_logic_vector(17 downto 0);
		nrm_vphs_c    : in  std_logic_vector(17 downto 0);
		nrm_vbus_x    : in  std_logic_vector(17 downto 0);
		nrm_vphs_n    : in  std_logic_vector(17 downto 0);
		--
		nrm_sync      : in  std_logic;

		-- filtered values
		--
		flt_iphs_a    : in  std_logic_vector(17 downto 0);
		flt_iphs_b    : in  std_logic_vector(17 downto 0);
		flt_iphs_c    : in  std_logic_vector(17 downto 0);
		flt_ibus_x    : in  std_logic_vector(17 downto 0);
		flt_vphs_a    : in  std_logic_vector(17 downto 0);
		flt_vphs_b    : in  std_logic_vector(17 downto 0);
		flt_vphs_c    : in  std_logic_vector(17 downto 0);
		flt_vbus_x    : in  std_logic_vector(17 downto 0);
		flt_vphs_n    : in  std_logic_vector(17 downto 0);
		--
		flt_sync      : in  std_logic;

		-- current overflow
		--
		ovi_iphs_a    : in  std_logic;
		ovi_iphs_b    : in  std_logic;
		ovi_iphs_c    : in  std_logic;
		ovi_ibus_x    : in  std_logic;

		-- Is overflow
		ismodover     : in  std_logic;
		
		-- 1=generator, 0=motor
		generator     : in  std_logic;
		
		-- modulator type
		modtype       : in  std_logic_vector(0 downto 0);
		
		-- modulator levels
		modlevels     : in  std_logic_vector(0 downto 0);
		
		-- motor is 3-phases (0=bipolar)
		is3phases     : in  std_logic;
		
		-- pwm 3-phases outputs
		pwm2p_l2c1w1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c1w2  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w2  : in  std_logic_vector(1 downto 0);
		pwm2p_l2sync  : in  std_logic;
		
		-- pwm 2-phases outputs
		pwm3p_l2cxw1  : in  std_logic_vector(2 downto 0);
		pwm3p_l2cxw2  : in  std_logic_vector(2 downto 0);
		pwm3p_l2sync  : in  std_logic;
		
		-- rpfm 3-ph two levels outputs
		rpfm3p_l2cxw1 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2cxw2 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2sync : in  std_logic;
		
		-- rpfm 3-ph three levels outputs
		rpfm3p_l3c1w1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c1w2 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w2 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w2 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3sync : in  std_logic;
		
		-- dead time value
		deadtval      : in  std_logic_vector(7 downto 0);
		
		-- gate unit switches
		guswcnt       : out  std_logic_vector(31 downto 0);
		
		-- in-failure
		hwfail        : out  std_logic;
		
		-- in-kill
		hwkill        : out  std_logic;
		
		-- coil driver
		coilgear      : in  std_logic;
		coilenab      : in  std_logic;
		
		-- system error
		syserr        : in  std_logic;
		
		-- reset pending errors
		reseterr      : in  std_logic;
		
		-- zero current offset 
		zerocurr      : in  std_logic;
		
		-- zero vbus offset 
		zerovbusx     : in  std_logic;

		-- zero vphs offset 
		zerovphsx     : in  std_logic;
		
		-- control probes
		pix_setval    : in std_logic_vector(17 downto 0);
		piy_setval    : in std_logic_vector(17 downto 0);
		
		pix_fbkval    : in std_logic_vector(17 downto 0);
		piy_fbkval    : in std_logic_vector(17 downto 0);
		
		pix_outval    : in std_logic_vector(17 downto 0);
		piy_outval    : in std_logic_vector(17 downto 0);
		
		-- Vs
		vsmodval      : in  std_logic_vector(16 downto 0);
		vsangval      : in  std_logic_vector(31 downto 0);
		
		-- Is
		ismodval      : in  std_logic_vector(16 downto 0);
		isangval      : in  std_logic_vector(31 downto 0);

		-- smo
		smo_bemf_p    : in  std_logic_vector(31 downto 0);
		smo_bemf_m    : in  std_logic_vector(16 downto 0);
		
		-- e-angles
		mrt_angle     : in  std_logic_vector(31 downto 0);
		enc_angle     : in  std_logic_vector(31 downto 0);
		hls_angle     : in  std_logic_vector(31 downto 0);
		rsv_angle     : in  std_logic_vector(31 downto 0);
		smo_angle     : in  std_logic_vector(31 downto 0);
		
		-- Rotor angle
		rot_angle     : in  std_logic_vector(31 downto 0);
		
		-- rot selector
		rot_select    : in  std_logic_vector( 2 downto 0);
		
		-- actual speed
		spd_speed     : in  std_logic_vector(31 downto 0);
		
		-- position
		pos_position  : in  std_logic_vector(31 downto 0);
		
		-- foc cycle time
		tsclocks      : in  std_logic_vector(11 downto 0)
	);
end motcon1sys;

architecture Behavioral of motcon1sys is

	component inlkmeta
		generic
		(
			C_NCLKS : integer := 3
		);
		port 
		(
			clock  : in  std_logic;
			datin  : in  std_logic;
			datout : out std_logic
		);
	end component;
	
	component unglitch
		generic
		(
			C_CLOCK_FREQ  : integer := 100_000_000;
			C_FILTER_TIME : real    := 1.0e-6
		);
		port
		(
			clock  : in  std_logic;
			reset  : in  std_logic;
			datin  : in  std_logic;
			datout : out std_logic
		);
	end component;

	component s3acqmult
		generic
		(
			C_RESULT_WIDTH : integer := 16;
			C_NUM_CHANNELS : integer := 16
		);
		port 
		(
			clock   : in  std_logic;
			reset   : in  std_logic;
			--
			sclock  : in  std_logic;
			sdata   : in  std_logic_vector(C_NUM_CHANNELS-1 downto 0);
			--
			pdata0  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata1  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata2  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata3  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata4  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata5  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata6  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata7  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata8  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata9  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata10 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata11 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata12 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata13 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata14 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata15 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			finish  : out std_logic
		);
	end component;
	
	component mcbxadc is
		generic
		(
			C_SYSTEMCLOCK   : integer := 100_000_000
		);
		port
		(
			clock          : in  std_logic;
			reset          : in  std_logic;
			-- temp & vbus values
			val_temp       : out std_logic_vector(17 downto 0);
			val_vbus       : out std_logic_vector(17 downto 0);
			-- motor currents
			val_ia         : out std_logic_vector(17 downto 0);
			val_ib         : out std_logic_vector(17 downto 0);
			-- sync
			val_sync       : out std_logic;
			-- xadc drp access
			xadc_drp_daddr : in  std_logic_vector(6 downto 0);
			xadc_drp_di    : in  std_logic_vector(15 downto 0);
			xadc_drp_den   : in  std_logic;
			xadc_drp_dwe   : in  std_logic;
			xadc_drp_do    : out std_logic_vector(15 downto 0);
			xadc_drp_drdy  : out std_logic;
			-- xadc analog signals
			xadc_vp        : in  std_logic;
			xadc_vn        : in  std_logic;
			xadc_vaux0_p   : in  std_logic;
			xadc_vaux0_n   : in  std_logic;
			xadc_vaux8_p   : in  std_logic;
			xadc_vaux8_n   : in  std_logic;
			-- xadc external mux
			xadc_mux_a0    : out std_logic;
			xadc_mux_a1    : out std_logic
		);
	end component;
	
	component filtiir
		generic
		(
			C_FAMILY : string := "spartan3adsp"
		);
		port 
		( 
			clock  : in  std_logic;
			reset  : in  std_logic;
			
			start  : in  std_logic;
			fktau  : in  std_logic_vector(16 downto 0);
			inraw  : in  std_logic_vector(17 downto 0);
			outflt : out std_logic_vector(17 downto 0);
			finish : out std_logic
		);
	end component;

	component evxcount
		generic
		(
			C_DATA_WIDTH  : integer := 16;
			C_EVENT_WIDTH : integer := 8;
			C_COUNT_MODE  : integer := 0
		);
		port
		(
			clock   : in  std_logic;
			reset   : in  std_logic;
			sigevt  : in  std_logic_vector(C_EVENT_WIDTH-1 downto 0);
			counter : out std_logic_vector(C_DATA_WIDTH-1 downto 0)
		);
	end component;

	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	-- modulator type
	constant MODTYPE_PWM  : std_logic_vector(0 downto 0) := "0";
	constant MODTYPE_RPFM : std_logic_vector(0 downto 0) := "1";
	
	-- modulator levels
	constant MODLEVELS_2  : std_logic_vector(0 downto 0) := "0";
	constant MODLEVELS_3  : std_logic_vector(0 downto 0) := "1";
	
	---------------------------------------------------------------------------
  
	constant RA_ip_ident    : integer := (4*0);
	constant RA_status_reg  : integer := (4*1);
	constant RA_control_reg : integer := (4*2);
	constant RA_coil_ctrl   : integer := (4*3);
	constant RA_oc_temp_val : integer := (4*4);
	constant RA_oc_temp_fkt : integer := (4*5);
	
	---------------------------------------------------------------------------
	
	constant C_DECAD_WIDTH : integer := 3;
	
	---------------------------------------------------------------------------
	
	function adrtoint(ad : std_logic_vector) return integer is
		variable ar : std_logic_vector(C_DECAD_WIDTH+1 downto 0);
	begin
		ar := ad(ar'left downto 2) & "00";
		return to_integer(unsigned(ar));
	end function;
	
	---------------------------------------------------------------------------

	-- ip identification
	constant ip_id_code  : integer := 8;
	constant ip_id_majv  : integer := 1;
	constant ip_id_minv  : integer := 0;
	constant ip_id_hscr  : integer := 97;
	
	constant ip_ident_k : std_logic_vector(31 downto 0) :=
		std_logic_vector(to_unsigned(ip_id_code,8)) &
		std_logic_vector(to_unsigned(ip_id_majv,8)) &
		std_logic_vector(to_unsigned(ip_id_minv,8)) &
		std_logic_vector(to_unsigned(ip_id_hscr,8));
	
	---------------------------------------------------------------------------
	-- host registers
	---------------------------------------------------------------------------
	
	constant C_COIL_CTRL_VALUE_OFS   : integer := 0;
	constant C_COIL_CTRL_GX_AUTO     : integer := 4;
	constant C_COIL_CTRL_GX_GEAR     : integer := 5;
	constant C_COIL_CTRL_WIDTH       : integer := 6;
	signal coil_ctrl : std_logic_vector(C_COIL_CTRL_WIDTH-1 downto 0);
	
	constant C_STATUS_REG_PWM_FAULT    : integer := 0;
	constant C_STATUS_REG_OVC_LATCHED  : integer := 1;
	constant C_STATUS_REG_WIDTH        : integer := 2;
	signal status_reg : std_logic_vector(C_STATUS_REG_WIDTH-1 downto 0);
	
	constant C_CONTROL_REG_ANGAIN0     : integer := 0;
	constant C_CONTROL_REG_ANGAIN1     : integer := 1;
	constant C_CONTROL_REG_SD_IABS_LVM : integer := 2;
	constant C_CONTROL_REG_XA_VBUS     : integer := 3;
	constant C_CONTROL_REG_XA_IABS     : integer := 4;
	constant C_CONTROL_REG_WIDTH       : integer := 5;
	signal control_reg : std_logic_vector(C_CONTROL_REG_WIDTH-1 downto 0);

	-- on-chip temp
	signal oc_temp_val : std_logic_vector(17 downto 0);

	-- on-chip temp filter k
	signal oc_temp_fkt : std_logic_vector(16 downto 0);
	
	constant K_FILTER_FAST  : std_logic_vector(16 downto 0) := (others => '1');
	
	---------------------------------------------------------------------------

	-- access delay
	signal host_en_dl1  : std_logic;
	
	---------------------------------------------------------------------------

	-- bus2reg
	function loadreg(s : std_logic_vector) return std_logic_vector is
		variable r : std_logic_vector(31 downto 0);
	begin
		r := (others => '0');
		r(s'range) := s;
		return r;
	end function;

	-- reg2bus
	procedure storereg(signal d : out std_logic_vector; signal s : in std_logic_vector) is
	begin
		d <= s(d'range);
	end procedure;
	
	---------------------------------------------------------------------------
	
	
	-- antimetastability # FF
	constant K_UMSK : integer := 2;

	-- antimetastabilized s/d inputs
	signal lk_fmc_ia_dat   : std_logic;
	signal lk_fmc_ib_dat   : std_logic;
	signal lk_fmc_vbus_dat : std_logic;

	-- antimetastabilized motor sentors inputs
	signal lk_fmc_sensor_a : std_logic;
	signal lk_fmc_sensor_b : std_logic;
	signal lk_fmc_sensor_c : std_logic;
	
	-- low voltage motor driver board antimetastabilized s/d inputs
	signal lk_lvm_ia_dat   : std_logic;
	signal lk_lvm_ib_dat   : std_logic;
	
	-- deglitch filter
	constant K_DEGLITH_TIME : real := 200.0e-9;
	
	-- deglitched motor sentors inputs
	signal dg_fmc_sensor_a : std_logic;
	signal dg_fmc_sensor_b : std_logic;
	signal dg_fmc_sensor_c : std_logic;
	
	---------------------------------------------------------------------------
	-- Inverter Values
	
	---------------------------------------------------------------------------
	-- indexes
	
	constant C_IVD_IPHS_A : integer := 0;
	constant C_IVD_IPHS_B : integer := 1;
	constant C_IVD_VBUS_X : integer := 2;
	constant C_IVD_NUM    : integer := 3;
	
	---------------------------------------------------------------------------
	-- ACQ (acquisition)
	
	signal invdat_sdat  : std_logic_vector(C_IVD_NUM-1 downto 0);
	
	constant C_INVERTER_SD_WIDTH : integer := 12;
	
	type   invdat_acq_t is array(0 to C_IVD_NUM-1) of std_logic_vector(C_INVERTER_SD_WIDTH-1 downto 0);
	signal invdat_acq : invdat_acq_t := (others => (others => '0'));
	signal invdat_eoc : std_logic;
	
	---------------------------------------------------------------------------
	-- RAW (raw: final size)
	
	constant C_INVDAT_ACQ2RAW_PAD : std_logic_vector(18-C_INVERTER_SD_WIDTH-1 downto 0) := (others => '0');
	
	type   invdat_raw_t is array(0 to C_IVD_NUM-1) of std_logic_vector(17 downto 0);
	signal invdat_raw : invdat_raw_t := (others => (others => '0'));
	
	---------------------------------------------------------------------------
	-- BIP (unipolar to bipolar)
	
	type   invdat_bip_t is array(0 to C_IVD_NUM-1) of std_logic_vector(17 downto 0);
	signal invdat_bip : invdat_bip_t := (others => (others => '0'));
	
	---------------------------------------------------------------------------
	
	
	-- xadc work registers
	
	-- temp & vbus values
	signal xa_val_temp     : std_logic_vector(17 downto 0);
	signal xa_val_vbus     : std_logic_vector(17 downto 0);
	-- motor current values
	signal xa_val_ia       : std_logic_vector(17 downto 0);
	signal xa_val_ib       : std_logic_vector(17 downto 0);
	-- sampled data
	signal xa_val_sync     : std_logic;
	-- xadc drp access
	signal xadc_drp_daddr  : std_logic_vector(6 downto 0);
	signal xadc_drp_di     : std_logic_vector(15 downto 0);
	signal xadc_drp_den    : std_logic;
	signal xadc_drp_dwe    : std_logic;
	signal xadc_drp_do     : std_logic_vector(15 downto 0);
	signal xadc_drp_drdy   : std_logic;
	---------------------------------------------------------------------------
	-- mux selector
	signal xa_vbus         : std_logic;
	signal xa_iabs         : std_logic;
	---------------------------------------------------------------------------
	-- bipolar
	signal xa_bip_ia       : std_logic_vector(17 downto 0);
	signal xa_bip_ib       : std_logic_vector(17 downto 0);
	signal xa_bip_vbus     : std_logic_vector(17 downto 0);
	---------------------------------------------------------------------------
	signal oc_temp_ftin    : std_logic_vector(17 downto 0);
	signal oc_temp_stf     : std_logic;
	---------------------------------------------------------------------------
	-- filtered values
	---------------------------------------------------------------------------
	signal oc_temp_fted    : std_logic_vector(17 downto 0);
	signal oc_temp_eof     : std_logic;
	---------------------------------------------------------------------------
	-- low voltage board remap GPIO
	
	-- GPO
	signal lvm_analog_gain    : std_logic_vector(1 downto 0);
	signal lvm_clr_ovc_fault  : std_logic;
	signal lvm_not_used_gpo4  : std_logic;
	signal lvm_coil_gate_dh   : std_logic;
	signal lvm_coil_gate_dl   : std_logic;
	signal lvm_not_used_gpo10 : std_logic;
	signal lvm_not_used_gpo11 : std_logic;
	
	-- GPI
	signal lvm_ovc_ct_latched : std_logic;
	signal lvm_not_used_gpi2  : std_logic;
	signal lvm_ia_dat         : std_logic;
	signal lvm_ib_dat         : std_logic;
	signal lvm_it_dat         : std_logic;
	---------------------------------------------------------------------------

	-- coil enable
	signal coilenx      : std_logic;
	
	-- coil commands hi/lo to gate unit/driver
	signal coildrv : std_logic_vector(3 downto 0);
	signal coilcmx : std_logic_vector(3 downto 0);
	
	---------------------------------------------------------------------------
	
	-- events to counts
	signal guevsrc : std_logic_vector(7 downto 0);
	
	---------------------------------------------------------------------------
	
	-- outputs
	
	-- host bus
	
	signal host_dout_i  : std_logic_vector(31 downto 0);
	signal host_ack_i   : std_logic;

	---------------------------------------------------------------------------
	
	-- board outputs
	
	signal fmc_ia_clk_i      : std_logic;
	signal fmc_ib_clk_i      : std_logic;
	signal fmc_it_clk_i      : std_logic;
	signal fmc_vbus_clk_i    : std_logic;
	signal fmc_pwm_en_i      : std_logic;
	signal fmc_pwm_ah_i      : std_logic;
	signal fmc_pwm_al_i      : std_logic;
	signal fmc_pwm_bh_i      : std_logic;
	signal fmc_pwm_bl_i      : std_logic;
	signal fmc_pwm_ch_i      : std_logic;
	signal fmc_pwm_cl_i      : std_logic;
	signal fmc_gpo_i         : std_logic_vector(10 downto 0);
	signal xadc_mux_a0_i     : std_logic;
	signal xadc_mux_a1_i     : std_logic;
	signal xadc_mux_en_i     : std_logic;
	
	---------------------------------------------------------------------------
	
	-- motor control
	
	-- external angle
	signal ext_angle_i   : std_logic_vector(31 downto 0);
	
	-- resolver
	signal rsv_priexc_i  : std_logic_vector(17 downto 0);
	signal rsv_secsin_i  : std_logic_vector(17 downto 0);
	signal rsv_seccos_i  : std_logic_vector(17 downto 0);
	
	-- hall sensor
	signal hls_hallsig_i : std_logic_vector(5 downto 0);
	
	-- digital encoder
	signal enc_cha_i     : std_logic;
	signal enc_chb_i     : std_logic;
	signal enc_chi_i     : std_logic;
	
	-- input data
	--
	signal acq_iphs_a_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_b_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_c_i  : std_logic_vector(17 downto 0);
	signal acq_ibus_x_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_a_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_b_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_c_i  : std_logic_vector(17 downto 0);
	signal acq_vbus_x_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_n_i  : std_logic_vector(17 downto 0);
	--
	signal acq_sync_i    : std_logic;
	
	-- gate unit switches
	signal guswcnt_i     : std_logic_vector(31 downto 0);
	
	-- in-failure
	signal hwfail_i      : std_logic;
	
	-- in-kill
	signal hwkill_i      : std_logic;
	
	---------------------------------------------------------------------------

begin

	---------------------------------------------------------------------------
	
	-- output host interface
	host_dout   <= host_dout_i;
	host_ack    <= host_ack_i;
	
	---------------------------------------------------------------------------
	-- output motor ip
	
	-- motor control
	
	-- external angle
	ext_angle   <= ext_angle_i;
	
	-- resolver
	rsv_priexc  <= rsv_priexc_i;
	rsv_secsin  <= rsv_secsin_i;
	rsv_seccos  <= rsv_seccos_i;
	
	-- hall sensor
	hls_hallsig <= hls_hallsig_i;
	
	-- digital encoder
	enc_cha     <= enc_cha_i;
	enc_chb     <= enc_chb_i;
	enc_chi     <= enc_chi_i;
	
	-- input data
	--
	acq_iphs_a  <= acq_iphs_a_i;
	acq_iphs_b  <= acq_iphs_b_i;
	acq_iphs_c  <= acq_iphs_c_i;
	acq_ibus_x  <= acq_ibus_x_i;
	acq_vphs_a  <= acq_vphs_a_i;
	acq_vphs_b  <= acq_vphs_b_i;
	acq_vphs_c  <= acq_vphs_c_i;
	acq_vbus_x  <= acq_vbus_x_i;
	acq_vphs_n  <= acq_vphs_n_i;
	--
	acq_sync    <= acq_sync_i;
	
	-- gate unit switches
	guswcnt     <= guswcnt_i;
	
	-- in-failure
	hwfail      <= hwfail_i;
	
	-- in-kill
	hwkill      <= hwkill_i;
	
	---------------------------------------------------------------------------
	
	hwfail_i <= (not lvm_ovc_ct_latched) or (not fmc_m_fault);
	hwkill_i <= '0'; -- n/a Kill input
	
	---------------------------------------------------------------------------
	
	-- output data board
	
	fmc_ia_clk   <= fmc_ia_clk_i;
	fmc_ib_clk   <= fmc_ib_clk_i;
	fmc_it_clk   <= fmc_it_clk_i;
	fmc_vbus_clk <= fmc_vbus_clk_i;
	fmc_pwm_en   <= fmc_pwm_en_i;
	fmc_pwm_ah   <= fmc_pwm_ah_i;
	fmc_pwm_al   <= fmc_pwm_al_i;
	fmc_pwm_bh   <= fmc_pwm_bh_i;
	fmc_pwm_bl   <= fmc_pwm_bl_i;
	fmc_pwm_ch   <= fmc_pwm_ch_i;
	fmc_pwm_cl   <= fmc_pwm_cl_i;
	fmc_gpo      <= fmc_gpo_i;
	xadc_mux_a0  <= xadc_mux_a0_i;
	xadc_mux_a1  <= xadc_mux_a1_i;
	xadc_mux_en  <= xadc_mux_en_i;

	---------------------------------------------------------------------------

	-- AD7401A mclkin
	fmc_ia_clk_i   <= sd_clock;
	fmc_ib_clk_i   <= sd_clock;
	fmc_it_clk_i   <= sd_clock;
	fmc_vbus_clk_i <= sd_clock;

	-- xadc mux enable
	xadc_mux_en_i  <= '1';
	
	---------------------------------------------------------------------------
	
	-- antimetastability s/d inputs
	LK_CB_SD_IA   : inlkmeta generic map ( C_NCLKS => K_UMSK ) port map ( clock => sd_clock, datin => fmc_ia_dat  , datout => lk_fmc_ia_dat   );
	LK_CB_SD_IB   : inlkmeta generic map ( C_NCLKS => K_UMSK ) port map ( clock => sd_clock, datin => fmc_ib_dat  , datout => lk_fmc_ib_dat   );
	LK_CB_SD_VBUS : inlkmeta generic map ( C_NCLKS => K_UMSK ) port map ( clock => sd_clock, datin => fmc_vbus_dat, datout => lk_fmc_vbus_dat );
	
	-- antimetastability motor sensor
	LK_CB_SENSOR_A : inlkmeta generic map ( C_NCLKS => K_UMSK ) port map ( clock => clock, datin => fmc_sensor_a  , datout => lk_fmc_sensor_a );
	LK_CB_SENSOR_B : inlkmeta generic map ( C_NCLKS => K_UMSK ) port map ( clock => clock, datin => fmc_sensor_b  , datout => lk_fmc_sensor_b );
	LK_CB_SENSOR_C : inlkmeta generic map ( C_NCLKS => K_UMSK ) port map ( clock => clock, datin => fmc_sensor_c  , datout => lk_fmc_sensor_c );
	
	-- low voltage motor control board 
	LK_LV_SD_IA : inlkmeta generic map ( C_NCLKS => K_UMSK ) port map ( clock => sd_clock, datin => lvm_ia_dat  , datout => lk_lvm_ia_dat );
	LK_LV_SD_IB : inlkmeta generic map ( C_NCLKS => K_UMSK ) port map ( clock => sd_clock, datin => lvm_ib_dat  , datout => lk_lvm_ib_dat );
	
	---------------------------------------------------------------------------
	
	-- deglitch
	
	DG_SENSOR_A : unglitch
		generic map
			(
				C_CLOCK_FREQ => C_SYSTEMCLOCK,
				C_FILTER_TIME => K_DEGLITH_TIME
			)
			port map
			(
				clock  => clock,
				reset  => reset,
				datin  => lk_fmc_sensor_a,
				datout => dg_fmc_sensor_a
			);
			
	DG_SENSOR_B : unglitch
		generic map
			(
				C_CLOCK_FREQ => C_SYSTEMCLOCK,
				C_FILTER_TIME => K_DEGLITH_TIME
			)
			port map
			(
				clock  => clock,
				reset  => reset,
				datin  => lk_fmc_sensor_b,
				datout => dg_fmc_sensor_b
			);
			
	DG_SENSOR_C : unglitch
		generic map
			(
				C_CLOCK_FREQ => C_SYSTEMCLOCK,
				C_FILTER_TIME => K_DEGLITH_TIME
			)
			port map
			(
				clock  => clock,
				reset  => reset,
				datin  => lk_fmc_sensor_c,
				datout => dg_fmc_sensor_c
			);

	---------------------------------------------------------------------------
	
	-- motor sensor signals
	
	-- external angle
	ext_angle_i <= ext_x_angle;
	
	-- resolver
	rsv_priexc_i <= rsv_x_priexc;
	rsv_secsin_i <= rsv_x_secsin;
	rsv_seccos_i <= rsv_x_seccos;
	
	-- hall
	hls_hallsig_i(0) <= dg_fmc_sensor_a;
	hls_hallsig_i(1) <= dg_fmc_sensor_b;
	hls_hallsig_i(2) <= dg_fmc_sensor_c;
	
	-- 2nd: unused
	hls_hallsig_i(5 downto 3) <= (others => '0');

	-- encoder
	enc_chi_i <= dg_fmc_sensor_a;
	enc_cha_i <= dg_fmc_sensor_b;
	enc_chb_i <= dg_fmc_sensor_c;
	
	---------------------------------------------------------------------------
	
	-- s/d input
	invdat_sdat(C_IVD_IPHS_A) <= lk_lvm_ia_dat when control_reg(C_CONTROL_REG_SD_IABS_LVM) = '1' else lk_fmc_ia_dat;
	invdat_sdat(C_IVD_IPHS_B) <= lk_lvm_ib_dat when control_reg(C_CONTROL_REG_SD_IABS_LVM) = '1' else lk_fmc_ib_dat;
	invdat_sdat(C_IVD_VBUS_X) <= lk_fmc_vbus_dat;
	
	INV_ACQ_INST : s3acqmult
		generic map
		(
			C_RESULT_WIDTH => C_INVERTER_SD_WIDTH,
			C_NUM_CHANNELS => C_IVD_NUM
		)
		port map
		(
			clock   => clock,
			reset   => reset,
			--
			sclock  => sd_clock,
			sdata   => invdat_sdat,
			--
			pdata0  => invdat_acq(0),
			pdata1  => invdat_acq(1),
			pdata2  => invdat_acq(2),
			pdata3  => open,
			pdata4  => open,
			pdata5  => open,
			pdata6  => open,
			pdata7  => open,
			pdata8  => open,
			pdata9  => open,
			pdata10 => open,
			pdata11 => open,
			pdata12 => open,
			pdata13 => open,
			pdata14 => open,
			pdata15 => open,
			finish  => invdat_eoc
		);
		
	-- final size
	INV_ACQ2RAW_GEN : for i in invdat_sdat'range generate
		invdat_raw(i) <= invdat_acq(i) & C_INVDAT_ACQ2RAW_PAD;
	end generate;
	
	-- unipolar to bipolar
	INV_RAW2BIP_GEN : for i in invdat_sdat'range generate
		invdat_bip(i) <= std_logic_vector(signed(invdat_raw(i)) - 2**17);
	end generate;
	
	---------------------------------------------------------------------------

	-- xadc DRP for test only
	xadc_drp_daddr <= (others => '0');
	xadc_drp_di    <= (others => '0');
	xadc_drp_den   <= '0';
	xadc_drp_dwe   <= '0';

	
	NOT_XADC_GEN : if (C_XADC <= 0) generate
	
		xa_vbus       <= '0';
		xa_iabs       <= '0';
	
		xa_val_temp   <= (others => '0');
		xa_val_vbus   <= (others => '0');
		
		xa_val_ia     <= (others => '0');
		xa_val_ib     <= (others => '0');
		
		xa_val_sync   <= '0';
		
		xadc_drp_do   <= (others => '0');
		xadc_drp_drdy <= '0';
		
		xadc_mux_a0_i <= '0';
		xadc_mux_a1_i <= '0';
		
	end generate;
	
	
	DO_XADC_GEN : if (C_XADC >= 1) generate
	
		xa_vbus       <= control_reg(C_CONTROL_REG_XA_VBUS);
		xa_iabs       <= control_reg(C_CONTROL_REG_XA_IABS);
		
		XADC_ACQ_INST : mcbxadc
			generic map
			(
				C_SYSTEMCLOCK   => C_SYSTEMCLOCK
			)
			port map
			(
				clock          => clock,
				reset          => reset,
				-- temp & vbus values
				val_temp       => xa_val_temp,
				val_vbus       => xa_val_vbus,
				-- motor currents
				val_ia         => xa_val_ia,
				val_ib         => xa_val_ib,
				-- sync
				val_sync       => xa_val_sync,
				-- xadc drp access
				xadc_drp_daddr => xadc_drp_daddr,
				xadc_drp_di    => xadc_drp_di,
				xadc_drp_den   => xadc_drp_den,
				xadc_drp_dwe   => xadc_drp_dwe,
				xadc_drp_do    => xadc_drp_do,
				xadc_drp_drdy  => xadc_drp_drdy,
				-- xadc analog signals
				xadc_vp        => xadc_vp,
				xadc_vn        => xadc_vn,
				xadc_vaux0_p   => xadc_vaux0_p,
				xadc_vaux0_n   => xadc_vaux0_n,
				xadc_vaux8_p   => xadc_vaux8_p,
				xadc_vaux8_n   => xadc_vaux8_n,
				-- xadc external mux
				xadc_mux_a0    => xadc_mux_a0_i,
				xadc_mux_a1    => xadc_mux_a1_i
			);
			
	
	end generate;
	
	---------------------------------------------------------------------------
	
	oc_temp_ftin <= xa_val_temp;
	oc_temp_stf  <= xa_val_sync;
	
	---------------------------------------------------------------------------
	
	xa_bip_ia   <= std_logic_vector(signed(xa_val_ia  ) - 2**17);
	xa_bip_ib   <= std_logic_vector(signed(xa_val_ib  ) - 2**17);
	xa_bip_vbus <= std_logic_vector(signed(xa_val_vbus) - 2**17);
	
	---------------------------------------------------------------------------
	
	-- output to motor control ip
	
	acq_iphs_a_i <= xa_bip_ia   when xa_iabs = '1' else invdat_bip(C_IVD_IPHS_A);
	acq_iphs_b_i <= xa_bip_ib   when xa_iabs = '1' else invdat_bip(C_IVD_IPHS_B);
	acq_iphs_c_i <= (others => '0');
	acq_ibus_x_i <= (others => '0');
	acq_vphs_a_i <= (others => '0');
	acq_vphs_b_i <= (others => '0');
	acq_vphs_c_i <= (others => '0');
	acq_vbus_x_i <= xa_bip_vbus when xa_vbus = '1' else invdat_bip(C_IVD_VBUS_X);
	acq_vphs_n_i <= (others => '0');
	acq_sync_i   <= xa_val_sync when xa_iabs = '1' else invdat_eoc;
	
	---------------------------------------------------------------------------
	
	OC_TEMP_LPF_INST : filtiir
		generic map
		(
			C_FAMILY => C_FAMILY
		)
		port map
		( 
			clock  => clock,
			reset  => reset,
			
			start  => oc_temp_stf,
			fktau  => oc_temp_fkt,
			inraw  => oc_temp_ftin,
			outflt => oc_temp_fted,
			finish => oc_temp_eof
		);
	
	---------------------------------------------------------------------------
	
	-- filtered oc-temp
	oc_temp_val <= oc_temp_fted;
	
	---------------------------------------------------------------------------
	-- low voltage motor driver board gpio remap
	
	-- not used GPO
	lvm_not_used_gpo4  <= '0';
	lvm_not_used_gpo10 <= '0';
	lvm_not_used_gpo11 <= '0';
	
	-- remap GPO
	
	-- analog gain
	fmc_gpo_i(0)  <= lvm_analog_gain(0);
	fmc_gpo_i(1)  <= lvm_analog_gain(1);
	
	-- overvoltage fault clear
	fmc_gpo_i(2)  <= lvm_clr_ovc_fault;
	
	fmc_gpo_i(3)  <= lvm_not_used_gpo4; -- spare
	
	-- AD7401A mclkin
	fmc_gpo_i(4)  <= sd_clock;
	fmc_gpo_i(5)  <= sd_clock;
	fmc_gpo_i(6)  <= sd_clock;
	
	-- gate D
	fmc_gpo_i(7)  <= lvm_coil_gate_dh;
	fmc_gpo_i(8)  <= lvm_coil_gate_dl;
	
	fmc_gpo_i(9)  <= lvm_not_used_gpo10; -- spare
	fmc_gpo_i(10) <= lvm_not_used_gpo11; -- spare
	
	-- remap GPI
	
	lvm_ovc_ct_latched <= fmc_gpi(0);
	
	lvm_not_used_gpi2  <= fmc_gpi(1); -- spare
	
	-- AD7401A mdat
	lvm_ia_dat     <= fmc_gpi(2);
	lvm_ib_dat     <= fmc_gpi(3);
	lvm_it_dat     <= fmc_gpi(4);
	
	---------------------------------------------------------------------------

	-- enable coil... some conditions
	coilenx <= 
		'1' when (
				coil_ctrl(C_COIL_CTRL_GX_AUTO) = '0' and -- manual
				coil_ctrl(C_COIL_CTRL_GX_GEAR) = '1' and -- gear-on
				lvm_clr_ovc_fault  = '0' and -- no overcurrent reset command
				lvm_ovc_ct_latched = '1' and -- no overcurrent latched
				fmc_m_fault        = '1'     -- no PWM driver fault
			) else
		'1' when (
				coil_ctrl(C_COIL_CTRL_GX_AUTO) = '1' and -- automatic
				coil_ctrl(C_COIL_CTRL_GX_GEAR) = '1' and -- gear-on
				syserr             = '0' and -- no error from motor control IP
				coilenab           = '1' and -- coil enable from motor control IP
				coilgear           = '1' and -- coil gear from motor control IP
				lvm_clr_ovc_fault  = '0' and -- no overcurrent reset command
				lvm_ovc_ct_latched = '1' and -- no overcurrent latched
				fmc_m_fault        = '1'     -- no PWM driver fault
			) else
		'0';
		
	---------------------------------------------------------------------------
	
	coildrv(0) <= 
		rpfm3p_l2cxw1(0) when modtype = MODTYPE_RPFM and is3phases = '1' else
		pwm3p_l2cxw1(0)  when modtype = MODTYPE_PWM  and is3phases = '1' else
		pwm2p_l2c1w1(0)  when modtype = MODTYPE_PWM  and is3phases = '0' else '0';
		
	coildrv(1) <= 
		rpfm3p_l2cxw1(1) when modtype = MODTYPE_RPFM and is3phases = '1' else
		pwm3p_l2cxw1(1)  when modtype = MODTYPE_PWM  and is3phases = '1' else
		pwm2p_l2c2w1(0)  when modtype = MODTYPE_PWM  and is3phases = '0' else '0';
		
	coildrv(2) <= 
		rpfm3p_l2cxw1(2) when modtype = MODTYPE_RPFM and is3phases = '1' else
		pwm3p_l2cxw1(2)  when modtype = MODTYPE_PWM  and is3phases = '1' else
		pwm2p_l2c1w1(1)  when modtype = MODTYPE_PWM  and is3phases = '0' else '0';
		
	coildrv(3) <= 
		pwm2p_l2c2w1(1)  when modtype = MODTYPE_PWM  and is3phases = '0' else '0';

	---------------------------------------------------------------------------
	
	CMDREG_GEN :
		for i in coilcmx'range generate
			coilcmx(i) <= '1' when ((coil_ctrl(C_COIL_CTRL_GX_AUTO) = '0' and coil_ctrl(C_COIL_CTRL_VALUE_OFS + i) = '1') or (coil_ctrl(C_COIL_CTRL_GX_AUTO) = '1' and coildrv(i) = '1')) else '0';
		end generate;
		
	---------------------------------------------------------------------------
	
	-- PWM driver enable
	fmc_pwm_en_i <= '1' when coilenx = '1' else '0';
	
	-- coil commands with dead time
	
	-- coil-A=>U
	fmc_pwm_ah_i <= '0' when coilcmx(0) = '1' and coilenx = '1' else '1';
	fmc_pwm_al_i <= '0' when coilcmx(0) = '0' and coilenx = '1' else '1';
	
	-- coil-B=>V
	fmc_pwm_bh_i <= '0' when coilcmx(1) = '1' and coilenx = '1' else '1';
	fmc_pwm_bl_i <= '0' when coilcmx(1) = '0' and coilenx = '1' else '1';
	
	-- coil-C=>W
	fmc_pwm_ch_i <= '0' when coilcmx(2) = '1' and coilenx = '1' else '1';
	fmc_pwm_cl_i <= '0' when coilcmx(2) = '0' and coilenx = '1' else '1';
	
	-- coil-D=>X bipolar stepper motor only
	lvm_coil_gate_dh <= '1' when coilcmx(3) = '1' and is3phases = '0' and coilenx = '1' else '0';
	lvm_coil_gate_dl <= '0' when coilcmx(3) = '0' and is3phases = '0' and coilenx = '1' else '1';
	
	---------------------------------------------------------------------------
	
	-- gate switches counter
	
	guevsrc(0) <= fmc_pwm_ah_i;
	guevsrc(1) <= fmc_pwm_al_i;
	guevsrc(2) <= fmc_pwm_bh_i;
	guevsrc(3) <= fmc_pwm_bl_i;
	guevsrc(4) <= fmc_pwm_ch_i;
	guevsrc(5) <= fmc_pwm_cl_i;
	guevsrc(6) <= lvm_coil_gate_dh;
	guevsrc(7) <= lvm_coil_gate_dl;
	
	GUSWCNT_INST : evxcount
		generic map
		(
			C_DATA_WIDTH  => 32,
			C_EVENT_WIDTH => 8,
			C_COUNT_MODE  => 0
		)
		port map
		(
			clock   => clock,
			reset   => reset,
			sigevt  => guevsrc,
			counter => guswcnt_i
		);
	
	
	---------------------------------------------------------------------------
	
	-- status register
	status_reg(C_STATUS_REG_PWM_FAULT)   <= fmc_m_fault;
	status_reg(C_STATUS_REG_OVC_LATCHED) <= lvm_ovc_ct_latched;
	
	-- control register
	lvm_analog_gain(0) <= control_reg(C_CONTROL_REG_ANGAIN1);
	lvm_analog_gain(1) <= control_reg(C_CONTROL_REG_ANGAIN0);
	
	-- reset error
	lvm_clr_ovc_fault  <= reseterr;
	
	---------------------------------------------------------------------------
	
	-- implement slave model software accessible register(s)
	SLAVE_WRITE_PROC : process(clock)
	begin
		if rising_edge(clock) then
			if reset = '1' then
				control_reg    <= (others => '0');
				coil_ctrl      <= (others => '0');
				oc_temp_fkt    <= K_FILTER_FAST;
			elsif host_we = '1' and host_en = '1' then
				case adrtoint(host_addr) is
					when RA_ip_ident    => null;
					when RA_status_reg  => null;
					when RA_control_reg => storereg(control_reg ,host_din);
					when RA_coil_ctrl   => storereg(coil_ctrl   ,host_din);
					when RA_oc_temp_val => null;
					when RA_oc_temp_fkt => storereg(oc_temp_fkt ,host_din);
					when others         => null;
				end case;
			end if;
		end if;
	end process;

	-- implement slave model software accessible register(s) read mux
	SLAVE_READ_PROC : process(clock)
		variable v : std_logic_vector(31 downto 0);
	begin
		if rising_edge(clock) then
			v := (others => '0');
			case adrtoint(host_addr) is
				when RA_ip_ident    => v := loadreg(ip_ident_k );
				when RA_status_reg  => v := loadreg(status_reg );
				when RA_control_reg => v := loadreg(control_reg);
				when RA_coil_ctrl   => v := loadreg(coil_ctrl  );
				when RA_oc_temp_val => v := loadreg(oc_temp_val);
				when RA_oc_temp_fkt => v := loadreg(oc_temp_fkt);
				when others         => null;
			end case;
			host_dout_i <= v;
		end if;
	end process;
	
	-- data transfer ack
	host_ack_i <= host_we or host_en_dl1;
	
	-- host access delay
	HOSTDLY_PROC : process(clock)
	begin
		if rising_edge(clock) then
			host_en_dl1 <= host_en;
		end if;
	end process;

end Behavioral;
