-------------------------------------------------------------------------------------------
-- Copyright  2011-2014, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    18:04:50 12/27/2014 
-- Design Name: 
-- Module Name:    dthlcmd - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity dthlcmd is
	generic
	(
		C_CLOCK_FREQ : integer := 100_000_000;
		C_DEAD_TIME  : real    := 100.0e-9
	);
	port
	(
		clock   : in  std_logic;
		reset   : in  std_logic;
		incmd   : in  std_logic;
		outhi   : out std_logic;
		outlo   : out std_logic
	);
end dthlcmd;

architecture Behavioral of dthlcmd is

	constant K_COUNT  : integer := integer(C_DEAD_TIME * real(C_CLOCK_FREQ));
	
	constant K_CNTMAX : integer := K_COUNT - 1;
	signal   counter  : natural range 0 to K_CNTMAX;
	
	signal   outhi_i  : std_logic;
	signal   outlo_i  : std_logic;

begin

	outhi <= outhi_i;
	outlo <= outlo_i;
	
	DT_PROC : process(clock)
	begin
		if rising_edge(clock) then
			if reset = '1' or (outhi_i /= incmd and outhi_i /= outlo_i) then
				outhi_i <= '0';
				outlo_i <= '0';
				counter <= 0;
			elsif counter = K_CNTMAX then
				outhi_i <= incmd;
				outlo_i <= not incmd;
			else
				counter <= counter + 1;
			end if;
		end if;
	end process;
	
end Behavioral;
