-------------------------------------------------------------------------------------------
-- Copyright  2011-2012, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    20:52:33 01/04/2012 
-- Design Name: 
-- Module Name:    inlkmeta - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity inlkmeta is
	generic
	(
		C_NCLKS : integer := 3
	);
	port 
	(
		clock  : in  std_logic;
		datin  : in  std_logic;
		datout : out std_logic
	);
end inlkmeta;

architecture Behavioral of inlkmeta is

	attribute IOB   : string;  
	signal    datpl : std_logic_vector(C_NCLKS-1 downto 0);

begin

	-- void function: pass-trought
	BYP_GEN : if (C_NCLKS <= 0) generate
		datout <= datin;
	end generate;
	
	-- 1..n FF registers.
	WREG_GEN : if (C_NCLKS > 0) generate
		attribute IOB of BF_1_INST : label is "true";
	begin
		-- result
		datout <= datpl(datpl'left);
		
		-- first FD is input pin....
		BF_1_INST : FD generic map (INIT => '0') port map (Q => datpl(0)  , C => clock, D => datin );
		
		-- ... others FD to reduce metastability
		FD_GEN : for i in 2 to C_NCLKS generate
			FD_INST : FD generic map (INIT => '0') port map (Q => datpl(i-1)  , C => clock, D => datpl(i-2) );
		end generate;
		
	end generate;

end Behavioral;
