-------------------------------------------------------------------------------------------
-- Copyright  2011-2019, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    10:27:28 12/27/2014 
-- Design Name: 
-- Module Name:    motcon2sys - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity motcon2sys is
	generic
	(
		C_FAMILY           : string  := "zynq";
		C_HOST_CLK_FREQ_HZ : integer := 100_000_000;
		C_XADC             : integer := 1
	);
	port 
	(
		-- host interface
		host_clk        : in  std_logic;
		host_rst        : in  std_logic;
		host_en         : in  std_logic;
		host_we         : in  std_logic;
		host_addr       : in  std_logic_vector(31 downto 0);
		host_din        : in  std_logic_vector(31 downto 0);
		host_dout       : out std_logic_vector(31 downto 0);
		host_ack        : out std_logic;
		-- ip ports
		--
		-- s/d ADC
		sd_clock        : in  std_logic;
		--
		sd_clock_x      : out std_logic;
		--
		-- over-current reset
		ovc_reset       : out std_logic;
		--
		-- M0 phy
		sd_vbus_dat_0   : in  std_logic;
		sd_ia_dat_0     : in  std_logic;
		sd_ib_dat_0     : in  std_logic;
		--
		drv_en_0        : out std_logic;
		drv_ah_0        : out std_logic;
		drv_al_0        : out std_logic;
		drv_bh_0        : out std_logic;
		drv_bl_0        : out std_logic;
		drv_ch_0        : out std_logic;
		drv_cl_0        : out std_logic;
		drv_dh_0        : out std_logic;
		drv_dl_0        : out std_logic;
		ovc_ct_0        : in  std_logic;
		--
		sensor_a_0      : in  std_logic;
		sensor_b_0      : in  std_logic;
		sensor_c_0      : in  std_logic;
		--
		-- M1 phy
		sd_vbus_dat_1   : in  std_logic;
		sd_ia_dat_1     : in  std_logic;
		sd_ib_dat_1     : in  std_logic;
		--
		drv_en_1        : out std_logic;
		drv_ah_1        : out std_logic;
		drv_al_1        : out std_logic;
		drv_bh_1        : out std_logic;
		drv_bl_1        : out std_logic;
		drv_ch_1        : out std_logic;
		drv_cl_1        : out std_logic;
		drv_dh_1        : out std_logic;
		drv_dl_1        : out std_logic;
		ovc_ct_1        : in  std_logic;
		--
		sensor_a_1      : in  std_logic;
		sensor_b_1      : in  std_logic;
		sensor_c_1      : in  std_logic;
		--
		-- hall sensor driver direction
		hlsdir_a_1      : out std_logic;
		hlsdir_b_1      : out std_logic;
		--
		-- i2c
		fmc_scl_o       : out std_logic;
		fmc_scl_t       : out std_logic;
		fmc_scl_i       : in  std_logic;
		--
		fmc_sda_o       : out std_logic;
		fmc_sda_t       : out std_logic;
		fmc_sda_i       : in  std_logic;
		--
		-- RDC
		rdc_sample      : out std_logic;
		rdc_mosi        : out std_logic;
		rdc_miso        : in  std_logic;
		rdc_sclk        : out std_logic;
		rdc_ss          : out std_logic;
		--
		-- bus enable
		vt_enable       : out std_logic;
		--
		-- xadc signals
		xadc_vp         : in  std_logic;
		xadc_vn         : in  std_logic;
		xadc_vaux0_p    : in  std_logic;
		xadc_vaux0_n    : in  std_logic;
		xadc_vaux8_p    : in  std_logic;
		xadc_vaux8_n    : in  std_logic;
		-- xadc external mux
		xadc_mux_a0     : out std_logic;
		xadc_mux_a1     : out std_logic;
		xadc_mux_en     : out std_logic;
		--
		-- external angle in
		ext_x_angle_0   : in  std_logic_vector(31 downto 0);
		ext_x_angle_1   : in  std_logic_vector(31 downto 0);
		--
		-- resolver in
		rsv_x_priexc_0  : in  std_logic_vector(17 downto 0);
		rsv_x_secsin_0  : in  std_logic_vector(17 downto 0);
		rsv_x_seccos_0  : in  std_logic_vector(17 downto 0);
		--
		rsv_x_priexc_1  : in  std_logic_vector(17 downto 0);
		rsv_x_secsin_1  : in  std_logic_vector(17 downto 0);
		rsv_x_seccos_1  : in  std_logic_vector(17 downto 0);
		--
		-- motor #0 control IP

		-- external angle
		ext_angle_0     : out std_logic_vector(31 downto 0);
		
		-- resolver
		rsv_priexc_0    : out std_logic_vector(17 downto 0);
		rsv_secsin_0    : out std_logic_vector(17 downto 0);
		rsv_seccos_0    : out std_logic_vector(17 downto 0);
		
		-- hall sensor
		hls_hallsig_0   : out std_logic_vector(5 downto 0);
		
		-- digital encoder
		enc_cha_0       : out std_logic;
		enc_chb_0       : out std_logic;
		enc_chi_0       : out std_logic;
		
		-- input data
		--
		acq_iphs_a_0    : out std_logic_vector(17 downto 0);
		acq_iphs_b_0    : out std_logic_vector(17 downto 0);
		acq_iphs_c_0    : out std_logic_vector(17 downto 0);
		acq_ibus_x_0    : out std_logic_vector(17 downto 0);
		acq_vphs_a_0    : out std_logic_vector(17 downto 0);
		acq_vphs_b_0    : out std_logic_vector(17 downto 0);
		acq_vphs_c_0    : out std_logic_vector(17 downto 0);
		acq_vbus_x_0    : out std_logic_vector(17 downto 0);
		acq_vphs_n_0    : out std_logic_vector(17 downto 0);
		--
		acq_sync_0      : out std_logic;
		
		-- output offset
		--
		ofs_iphs_a_0    : in  std_logic_vector(17 downto 0);
		ofs_iphs_b_0    : in  std_logic_vector(17 downto 0);
		ofs_iphs_c_0    : in  std_logic_vector(17 downto 0);
		ofs_ibus_x_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_a_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_b_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_c_0    : in  std_logic_vector(17 downto 0);
		ofs_vbus_x_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_n_0    : in  std_logic_vector(17 downto 0);

		-- normalized values (offset removed)
		--
		nrm_iphs_a_0    : in  std_logic_vector(17 downto 0);
		nrm_iphs_b_0    : in  std_logic_vector(17 downto 0);
		nrm_iphs_c_0    : in  std_logic_vector(17 downto 0);
		nrm_ibus_x_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_a_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_b_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_c_0    : in  std_logic_vector(17 downto 0);
		nrm_vbus_x_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_n_0    : in  std_logic_vector(17 downto 0);
		--
		nrm_sync_0      : in  std_logic;

		-- filtered values
		--
		flt_iphs_a_0    : in  std_logic_vector(17 downto 0);
		flt_iphs_b_0    : in  std_logic_vector(17 downto 0);
		flt_iphs_c_0    : in  std_logic_vector(17 downto 0);
		flt_ibus_x_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_a_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_b_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_c_0    : in  std_logic_vector(17 downto 0);
		flt_vbus_x_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_n_0    : in  std_logic_vector(17 downto 0);
		--
		flt_sync_0      : in  std_logic;

		-- current overflow
		--
		ovi_iphs_a_0    : in  std_logic;
		ovi_iphs_b_0    : in  std_logic;
		ovi_iphs_c_0    : in  std_logic;
		ovi_ibus_x_0    : in  std_logic;

		-- Is overflow
		ismodover_0     : in  std_logic;
		
		-- 1=generator, 0=motor
		generator_0     : in  std_logic;
		
		-- modulator type
		modtype_0       : in  std_logic_vector(0 downto 0);
		
		-- modulator levels
		modlevels_0     : in  std_logic_vector(0 downto 0);
		
		-- motor is 3-phases (0=bipolar)
		is3phases_0     : in  std_logic;
		
		-- pwm 3-phases outputs
		pwm2p_l2c1w1_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w1_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c1w2_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w2_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2sync_0  : in  std_logic;
		
		-- pwm 2-phases outputs
		pwm3p_l2cxw1_0  : in  std_logic_vector(2 downto 0);
		pwm3p_l2cxw2_0  : in  std_logic_vector(2 downto 0);
		pwm3p_l2sync_0  : in  std_logic;
		
		-- rpfm 3-ph two levels outputs
		rpfm3p_l2cxw1_0 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2cxw2_0 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2sync_0 : in  std_logic;
		
		-- rpfm 3-ph three levels outputs
		rpfm3p_l3c1w1_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w1_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w1_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c1w2_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w2_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w2_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3sync_0 : in  std_logic;
		
		-- dead time value
		deadtval_0      : in  std_logic_vector(7 downto 0);
		
		-- gate unit switches
		guswcnt_0       : out std_logic_vector(31 downto 0);
		
		-- in-failure
		hwfail_0        : out std_logic;
		
		-- in-kill
		hwkill_0        : out std_logic;
		
		-- coil driver
		coilgear_0      : in  std_logic;
		coilenab_0      : in  std_logic;
		
		-- system error
		syserr_0        : in  std_logic;
		
		-- reset pending errors
		reseterr_0      : in  std_logic;
		
		-- zero current offset 
		zerocurr_0      : in  std_logic;
		
		-- zero vbus offset 
		zerovbusx_0     : in  std_logic;

		-- zero vphs offset 
		zerovphsx_0     : in  std_logic;
		
		-- control probes
		pix_setval_0    : in std_logic_vector(17 downto 0);
		piy_setval_0    : in std_logic_vector(17 downto 0);
		
		pix_fbkval_0    : in std_logic_vector(17 downto 0);
		piy_fbkval_0    : in std_logic_vector(17 downto 0);
		
		pix_outval_0    : in std_logic_vector(17 downto 0);
		piy_outval_0    : in std_logic_vector(17 downto 0);
		
		-- Vs
		vsmodval_0      : in  std_logic_vector(16 downto 0);
		vsangval_0      : in  std_logic_vector(31 downto 0);
		
		-- Is
		ismodval_0      : in  std_logic_vector(16 downto 0);
		isangval_0      : in  std_logic_vector(31 downto 0);

		-- smo
		smo_bemf_p_0    : in  std_logic_vector(31 downto 0);
		smo_bemf_m_0    : in  std_logic_vector(16 downto 0);
		
		-- e-angles
		mrt_angle_0     : in  std_logic_vector(31 downto 0);
		enc_angle_0     : in  std_logic_vector(31 downto 0);
		hls_angle_0     : in  std_logic_vector(31 downto 0);
		rsv_angle_0     : in  std_logic_vector(31 downto 0);
		smo_angle_0     : in  std_logic_vector(31 downto 0);
		
		-- Rotor angle
		rot_angle_0     : in  std_logic_vector(31 downto 0);
		
		-- rot selector
		rot_select_0    : in  std_logic_vector( 2 downto 0);
		
		-- actual speed
		spd_speed_0     : in  std_logic_vector(31 downto 0);
		
		-- position
		pos_position_0  : in  std_logic_vector(31 downto 0);
		
		-- foc cycle time
		tsclocks_0      : in  std_logic_vector(11 downto 0);
		
		-- motor #1 control IP

		-- external angle
		ext_angle_1     : out std_logic_vector(31 downto 0);
		
		-- resolver
		rsv_priexc_1    : out std_logic_vector(17 downto 0);
		rsv_secsin_1    : out std_logic_vector(17 downto 0);
		rsv_seccos_1    : out std_logic_vector(17 downto 0);
		
		-- hall sensor
		hls_hallsig_1   : out std_logic_vector(5 downto 0);
		
		-- digital encoder
		enc_cha_1       : out std_logic;
		enc_chb_1       : out std_logic;
		enc_chi_1       : out std_logic;
		
		-- input data
		--
		acq_iphs_a_1    : out std_logic_vector(17 downto 0);
		acq_iphs_b_1    : out std_logic_vector(17 downto 0);
		acq_iphs_c_1    : out std_logic_vector(17 downto 0);
		acq_ibus_x_1    : out std_logic_vector(17 downto 0);
		acq_vphs_a_1    : out std_logic_vector(17 downto 0);
		acq_vphs_b_1    : out std_logic_vector(17 downto 0);
		acq_vphs_c_1    : out std_logic_vector(17 downto 0);
		acq_vbus_x_1    : out std_logic_vector(17 downto 0);
		acq_vphs_n_1    : out std_logic_vector(17 downto 0);
		--
		acq_sync_1      : out std_logic;
		
		-- output offset
		--
		ofs_iphs_a_1    : in  std_logic_vector(17 downto 0);
		ofs_iphs_b_1    : in  std_logic_vector(17 downto 0);
		ofs_iphs_c_1    : in  std_logic_vector(17 downto 0);
		ofs_ibus_x_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_a_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_b_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_c_1    : in  std_logic_vector(17 downto 0);
		ofs_vbus_x_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_n_1    : in  std_logic_vector(17 downto 0);

		-- normalized values (offset removed)
		--
		nrm_iphs_a_1    : in  std_logic_vector(17 downto 0);
		nrm_iphs_b_1    : in  std_logic_vector(17 downto 0);
		nrm_iphs_c_1    : in  std_logic_vector(17 downto 0);
		nrm_ibus_x_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_a_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_b_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_c_1    : in  std_logic_vector(17 downto 0);
		nrm_vbus_x_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_n_1    : in  std_logic_vector(17 downto 0);
		--
		nrm_sync_1      : in  std_logic;

		-- filtered values
		--
		flt_iphs_a_1    : in  std_logic_vector(17 downto 0);
		flt_iphs_b_1    : in  std_logic_vector(17 downto 0);
		flt_iphs_c_1    : in  std_logic_vector(17 downto 0);
		flt_ibus_x_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_a_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_b_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_c_1    : in  std_logic_vector(17 downto 0);
		flt_vbus_x_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_n_1    : in  std_logic_vector(17 downto 0);
		--
		flt_sync_1      : in  std_logic;

		-- current overflow
		--
		ovi_iphs_a_1    : in  std_logic;
		ovi_iphs_b_1    : in  std_logic;
		ovi_iphs_c_1    : in  std_logic;
		ovi_ibus_x_1    : in  std_logic;

		-- Is overflow
		ismodover_1     : in  std_logic;
		
		-- 1=generator, 0=motor
		generator_1     : in  std_logic;
		
		-- modulator type
		modtype_1       : in  std_logic_vector(0 downto 0);
		
		-- modulator levels
		modlevels_1     : in  std_logic_vector(0 downto 0);
		
		-- motor is 3-phases (0=bipolar)
		is3phases_1     : in  std_logic;
		
		-- pwm 3-phases outputs
		pwm2p_l2c1w1_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w1_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c1w2_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w2_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2sync_1  : in  std_logic;
		
		-- pwm 2-phases outputs
		pwm3p_l2cxw1_1  : in  std_logic_vector(2 downto 0);
		pwm3p_l2cxw2_1  : in  std_logic_vector(2 downto 0);
		pwm3p_l2sync_1  : in  std_logic;
		
		-- rpfm 3-ph two levels outputs
		rpfm3p_l2cxw1_1 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2cxw2_1 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2sync_1 : in  std_logic;
		
		-- rpfm 3-ph three levels outputs
		rpfm3p_l3c1w1_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w1_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w1_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c1w2_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w2_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w2_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3sync_1 : in  std_logic;
		
		-- dead time value
		deadtval_1      : in  std_logic_vector(7 downto 0);
		
		-- gate unit switches
		guswcnt_1       : out std_logic_vector(31 downto 0);
		
		-- in-failure
		hwfail_1        : out std_logic;
		
		-- in-kill
		hwkill_1        : out std_logic;
		
		-- coil driver
		coilgear_1      : in  std_logic;
		coilenab_1      : in  std_logic;
		
		-- system error
		syserr_1        : in  std_logic;
		
		-- reset pending errors
		reseterr_1      : in  std_logic;
		
		-- zero current offset 
		zerocurr_1      : in  std_logic;
		
		-- zero vbus offset 
		zerovbusx_1     : in  std_logic;

		-- zero vphs offset 
		zerovphsx_1     : in  std_logic;
		
		-- control probes
		pix_setval_1    : in std_logic_vector(17 downto 0);
		piy_setval_1    : in std_logic_vector(17 downto 0);
		
		pix_fbkval_1    : in std_logic_vector(17 downto 0);
		piy_fbkval_1    : in std_logic_vector(17 downto 0);
		
		pix_outval_1    : in std_logic_vector(17 downto 0);
		piy_outval_1    : in std_logic_vector(17 downto 0);
		
		-- Vs
		vsmodval_1      : in  std_logic_vector(16 downto 0);
		vsangval_1      : in  std_logic_vector(31 downto 0);
		
		-- Is
		ismodval_1      : in  std_logic_vector(16 downto 0);
		isangval_1      : in  std_logic_vector(31 downto 0);

		-- smo
		smo_bemf_p_1    : in  std_logic_vector(31 downto 0);
		smo_bemf_m_1    : in  std_logic_vector(16 downto 0);
		
		-- e-angles
		mrt_angle_1     : in  std_logic_vector(31 downto 0);
		enc_angle_1     : in  std_logic_vector(31 downto 0);
		hls_angle_1     : in  std_logic_vector(31 downto 0);
		rsv_angle_1     : in  std_logic_vector(31 downto 0);
		smo_angle_1     : in  std_logic_vector(31 downto 0);
		
		-- Rotor angle
		rot_angle_1     : in  std_logic_vector(31 downto 0);
		
		-- rot selector
		rot_select_1    : in  std_logic_vector( 2 downto 0);
		
		-- actual speed
		spd_speed_1     : in  std_logic_vector(31 downto 0);
		
		-- position
		pos_position_1  : in  std_logic_vector(31 downto 0);
		
		-- foc cycle time
		tsclocks_1      : in  std_logic_vector(11 downto 0)
	);
end motcon2sys;

architecture Behavioral of motcon2sys is

	component mcbxadc
		generic
		(
			C_SYSTEMCLOCK   : integer := 100_000_000
		);
		port
		(
			clock          : in  std_logic;
			reset          : in  std_logic;
			-- on-chip temperature
			val_temp       : out std_logic_vector(17 downto 0);
			-- M0/1 values
			val_ia_0       : out std_logic_vector(17 downto 0);
			val_ib_0       : out std_logic_vector(17 downto 0);
			val_vbus_0     : out std_logic_vector(17 downto 0);
			val_ia_1       : out std_logic_vector(17 downto 0);
			val_ib_1       : out std_logic_vector(17 downto 0);
			val_vbus_1     : out std_logic_vector(17 downto 0);
			-- sync
			val_sync       : out std_logic;
			-- xadc drp access
			xadc_drp_daddr : in  std_logic_vector(6 downto 0);
			xadc_drp_di    : in  std_logic_vector(15 downto 0);
			xadc_drp_den   : in  std_logic;
			xadc_drp_dwe   : in  std_logic;
			xadc_drp_do    : out std_logic_vector(15 downto 0);
			xadc_drp_drdy  : out std_logic;
			-- xadc analog signals
			xadc_vp        : in  std_logic;
			xadc_vn        : in  std_logic;
			xadc_vaux0_p   : in  std_logic;
			xadc_vaux0_n   : in  std_logic;
			xadc_vaux8_p   : in  std_logic;
			xadc_vaux8_n   : in  std_logic;
			-- xadc external mux
			xadc_mux_a0    : out std_logic;
			xadc_mux_a1    : out std_logic
		);
	end component;
	
	component inlkmeta
		generic
		(
			C_NCLKS : integer := 3
		);
		port 
		(
			clock  : in  std_logic;
			datin  : in  std_logic;
			datout : out std_logic
		);
	end component;
	
	component unglitch
		generic
		(
			C_CLOCK_FREQ  : integer := 100_000_000;
			C_FILTER_TIME : real    := 1.0e-6
		);
		port
		(
			clock  : in  std_logic;
			reset  : in  std_logic;
			datin  : in  std_logic;
			datout : out std_logic
		);
	end component;
	
	component s3acqmult
		generic
		(
			C_RESULT_WIDTH : integer := 16;
			C_NUM_CHANNELS : integer := 16
		);
		port 
		(
			clock   : in  std_logic;
			reset   : in  std_logic;
			--
			sclock  : in  std_logic;
			sdata   : in  std_logic_vector(C_NUM_CHANNELS-1 downto 0);
			--
			pdata0  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata1  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata2  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata3  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata4  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata5  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata6  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata7  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata8  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata9  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata10 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata11 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata12 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata13 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata14 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata15 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			finish  : out std_logic
		);
	end component;
	
	component filtiir
		generic
		(
			C_FAMILY : string := "spartan3adsp"
		);
		port 
		( 
			clock  : in  std_logic;
			reset  : in  std_logic;
			
			start  : in  std_logic;
			fktau  : in  std_logic_vector(16 downto 0);
			inraw  : in  std_logic_vector(17 downto 0);
			outflt : out std_logic_vector(17 downto 0);
			finish : out std_logic
		);
	end component;
	
	component dthlcmd
		generic
		(
			C_CLOCK_FREQ : integer := 100_000_000;
			C_DEAD_TIME  : real    := 100.0e-9
		);
		port
		(
			clock   : in  std_logic;
			reset   : in  std_logic;
			incmd   : in  std_logic;
			outhi   : out std_logic;
			outlo   : out std_logic
		);
	end component;
	
	component evxcount
		generic
		(
			C_DATA_WIDTH  : integer := 16;
			C_EVENT_WIDTH : integer := 8;
			C_COUNT_MODE  : integer := 0
		);
		port
		(
			clock   : in  std_logic;
			reset   : in  std_logic;
			sigevt  : in  std_logic_vector(C_EVENT_WIDTH-1 downto 0);
			counter : out std_logic_vector(C_DATA_WIDTH-1 downto 0)
		);
	end component;

	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	-- modulator type
	constant MODTYPE_PWM  : std_logic_vector(0 downto 0) := "0";
	constant MODTYPE_RPFM : std_logic_vector(0 downto 0) := "1";
	
	-- modulator levels
	constant MODLEVELS_2  : std_logic_vector(0 downto 0) := "0";
	constant MODLEVELS_3  : std_logic_vector(0 downto 0) := "1";
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------

	---------------------------------------------------------------------------
	
	constant C_CONTROL_REG_XA_VBUS_0  : integer :=  0;
	constant C_CONTROL_REG_XA_IAB_0   : integer :=  1;
	constant C_CONTROL_REG_XA_VBUS_1  : integer :=  2;
	constant C_CONTROL_REG_XA_IAB_1   : integer :=  3;
	constant C_CONTROL_REG_VT_ENABLE  : integer :=  4;
	constant C_CONTROL_REG_WIDTH      : integer :=  5;
	signal control_reg : std_logic_vector(C_CONTROL_REG_WIDTH-1 downto 0);
	
	---------------------------------------------------------------------------
	
	constant C_STATUS_REG_OVC_0        : integer := 0;
	constant C_STATUS_REG_OVC_1        : integer := 1;
	constant C_STATUS_REG_WIDTH        : integer := 2;
	signal status_reg : std_logic_vector(C_STATUS_REG_WIDTH-1 downto 0);
	
	---------------------------------------------------------------------------
	-- input sensors
	
	signal lk_sensor_a_0   : std_logic;
	signal lk_sensor_b_0   : std_logic;
	signal lk_sensor_c_0   : std_logic;
	
	signal lk_sensor_a_1   : std_logic;
	signal lk_sensor_b_1   : std_logic;
	signal lk_sensor_c_1   : std_logic;
	
	---------------------------------------------------------------------------
	-- de-glitch filter
	constant K_DEGLITH_TIME : real := 200.0e-9;
	
	-- de-glitched motor input sensors

	signal dg_sensor_a_0 : std_logic;
	signal dg_sensor_b_0 : std_logic;
	signal dg_sensor_c_0 : std_logic;
	
	signal dg_sensor_a_1 : std_logic;
	signal dg_sensor_b_1 : std_logic;
	signal dg_sensor_c_1 : std_logic;
	
	---------------------------------------------------------------------------

	-- xadc mux selector
	
	signal xa_iab_0        : std_logic;
	signal xa_vbus_0       : std_logic;
	
	signal xa_iab_1        : std_logic;
	signal xa_vbus_1       : std_logic;
	
	-- output values
	
	signal xa_val_temp     : std_logic_vector(17 downto 0);
	
	signal xa_val_ia_0     : std_logic_vector(17 downto 0);
	signal xa_val_ib_0     : std_logic_vector(17 downto 0);
	signal xa_val_vbus_0   : std_logic_vector(17 downto 0);
	
	signal xa_val_ia_1     : std_logic_vector(17 downto 0);
	signal xa_val_ib_1     : std_logic_vector(17 downto 0);
	signal xa_val_vbus_1   : std_logic_vector(17 downto 0);

	-- xadc end-of-conversion
	signal xa_val_sync     : std_logic;
	
	-- xadc drp access
	signal xadc_drp_daddr  : std_logic_vector(6 downto 0);
	signal xadc_drp_di     : std_logic_vector(15 downto 0);
	signal xadc_drp_den    : std_logic;
	signal xadc_drp_dwe    : std_logic;
	signal xadc_drp_do     : std_logic_vector(15 downto 0);
	signal xadc_drp_drdy   : std_logic;

	---------------------------------------------------------------------------

	-- xa normalized aft uni/bip
	
	signal xa_bip_ia_0     : std_logic_vector(17 downto 0);
	signal xa_bip_ib_0     : std_logic_vector(17 downto 0);
	signal xa_bip_vbus_0   : std_logic_vector(17 downto 0);
	
	signal xa_bip_ia_1     : std_logic_vector(17 downto 0);
	signal xa_bip_ib_1     : std_logic_vector(17 downto 0);
	signal xa_bip_vbus_1   : std_logic_vector(17 downto 0);
	
	---------------------------------------------------------------------------
	
	-- anti-metastability # FF
	constant K_UMSK : integer := 2;
	
	-- locked
	
	signal lk_sd_ia_0   : std_logic;
	signal lk_sd_ib_0   : std_logic;
	signal lk_sd_vbus_0 : std_logic;

	signal lk_sd_ia_1   : std_logic;
	signal lk_sd_ib_1   : std_logic;
	signal lk_sd_vbus_1 : std_logic;
	
	-- s/d input
	
	signal sd_data_mux   : std_logic_vector(5 downto 0);
	
	-- s/d output
	
	constant C_INVERTER_SD_WIDTH : integer := 10; -- 10 bits @ 10 MHz -> 3.2 us
	
	signal sd_raw_ia_0   : std_logic_vector(C_INVERTER_SD_WIDTH-1 downto 0);
	signal sd_raw_ib_0   : std_logic_vector(C_INVERTER_SD_WIDTH-1 downto 0);
	signal sd_raw_vbus_0 : std_logic_vector(C_INVERTER_SD_WIDTH-1 downto 0);

	signal sd_raw_ia_1   : std_logic_vector(C_INVERTER_SD_WIDTH-1 downto 0);
	signal sd_raw_ib_1   : std_logic_vector(C_INVERTER_SD_WIDTH-1 downto 0);
	signal sd_raw_vbus_1 : std_logic_vector(C_INVERTER_SD_WIDTH-1 downto 0);
	
	signal sd_raw_sync   : std_logic;
	
	-- s/d output normalized
	
	constant C_INVDAT_RAW2VAL_PAD : std_logic_vector(18-C_INVERTER_SD_WIDTH-1 downto 0) := (others => '0');
	
	signal sd_val_ia_0   : std_logic_vector(17 downto 0);
	signal sd_val_ib_0   : std_logic_vector(17 downto 0);
	signal sd_val_vbus_0 : std_logic_vector(17 downto 0);
	
	signal sd_val_ia_1   : std_logic_vector(17 downto 0);
	signal sd_val_ib_1   : std_logic_vector(17 downto 0);
	signal sd_val_vbus_1 : std_logic_vector(17 downto 0);
	
	signal sd_val_sync   : std_logic;
	
	---------------------------------------------------------------------------

	-- sd normalized aft uni/bip
	
	signal sd_bip_ia_0     : std_logic_vector(17 downto 0);
	signal sd_bip_ib_0     : std_logic_vector(17 downto 0);
	signal sd_bip_vbus_0   : std_logic_vector(17 downto 0);
	
	signal sd_bip_ia_1     : std_logic_vector(17 downto 0);
	signal sd_bip_ib_1     : std_logic_vector(17 downto 0);
	signal sd_bip_vbus_1   : std_logic_vector(17 downto 0);

	---------------------------------------------------------------------------
	
	-- aft mux
	
	signal mux_iphs_a_0  : std_logic_vector(17 downto 0);
	signal mux_iphs_b_0  : std_logic_vector(17 downto 0);
	signal mux_vbus_x_0  : std_logic_vector(17 downto 0);
	
	signal mux_iphs_a_1  : std_logic_vector(17 downto 0);
	signal mux_iphs_b_1  : std_logic_vector(17 downto 0);
	signal mux_vbus_x_1  : std_logic_vector(17 downto 0);
	
	signal mux_sync_0    : std_logic;
	signal mux_sync_1    : std_logic;
	
	---------------------------------------------------------------------------
	-- filter K ---------------------------------------------------------------
	
	constant K_FILTER_FAST  : std_logic_vector(16 downto 0) := (others => '1');
	
	signal oc_temp_fkt   : std_logic_vector(16 downto 0);
	
	---------------------------------------------------------------------------
	-- filtered values --------------------------------------------------------
	
	signal oc_temp_val   : std_logic_vector(17 downto 0);

	---------------------------------------------------------------------------
	-- coil mux pwm/rpfm 3-phases 2-phases ------------------------------------
	
	-- M0
	
	signal rpfm_3phl2_0 : std_logic;
	signal pwm_3phl2_0  : std_logic;
	signal pwm_2phl2_0  : std_logic;
	
	signal coil_a_0 : std_logic;
	signal coil_b_0 : std_logic;
	signal coil_c_0 : std_logic;
	signal coil_d_0 : std_logic;
	
	-- M1
	
	signal rpfm_3phl2_1 : std_logic;
	signal pwm_3phl2_1  : std_logic;
	signal pwm_2phl2_1  : std_logic;
	
	signal coil_a_1 : std_logic;
	signal coil_b_1 : std_logic;
	signal coil_c_1 : std_logic;
	signal coil_d_1 : std_logic;
	
	---------------------------------------------------------------------------
	
	constant C_DEAD_TIME : real := 100.0e-9;
	
	constant DRV_A_0 : integer := 0;
	constant DRV_B_0 : integer := 1;
	constant DRV_C_0 : integer := 2;
	constant DRV_D_0 : integer := 3;
	
	constant DRV_A_1 : integer := 4;
	constant DRV_B_1 : integer := 5;
	constant DRV_C_1 : integer := 6;
	constant DRV_D_1 : integer := 7;
	
	constant DRV_WIDTH : integer := 8;
	
	signal coil_cmt : std_logic_vector(DRV_WIDTH-1 downto 0);
	signal coil_cmh : std_logic_vector(DRV_WIDTH-1 downto 0);
	signal coil_cml : std_logic_vector(DRV_WIDTH-1 downto 0);
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	-- gate unit switches counters
	
	-- event captures
	signal guevsrc_0 : std_logic_vector(7 downto 0);
	signal guevsrc_1 : std_logic_vector(7 downto 0);
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	-- access delay
	signal host_en_dl1  : std_logic;
	
	---------------------------------------------------------------------------

	-- bus2reg
	function loadreg(s : std_logic_vector) return std_logic_vector is
		variable r : std_logic_vector(31 downto 0);
	begin
		r := (others => '0');
		r(s'range) := s;
		return r;
	end function;

	-- reg2bus
	procedure storereg(signal d : out std_logic_vector; signal s : in std_logic_vector) is
	begin
		d <= s(d'range);
	end procedure;
	
	
	---------------------------------------------------------------------------
	
  	constant RA_ip_ident    : integer := (4*0);
	constant RA_status_reg  : integer := (4*1);
	constant RA_control_reg : integer := (4*2);
	constant RA_oc_temp_val : integer := (4*3);
	constant RA_oc_temp_fkt : integer := (4*4);
	
	---------------------------------------------------------------------------
	
	constant C_DECAD_WIDTH      : integer := 3;
	
	---------------------------------------------------------------------------
	
	function adrtoint(ad : std_logic_vector) return integer is
		variable ar : std_logic_vector(C_DECAD_WIDTH+1 downto 0);
	begin
		ar := ad(ar'left downto 2) & "00";
		return to_integer(unsigned(ar));
	end function;
	
	---------------------------------------------------------------------------

	-- ip identification
	constant ip_id_code  : integer := 9;
	constant ip_id_majv  : integer := 1;
	constant ip_id_minv  : integer := 1;
	constant ip_id_hscr  : integer := 97;
	
	constant ip_ident_k : std_logic_vector(31 downto 0) :=
		std_logic_vector(to_unsigned(ip_id_code,8)) &
		std_logic_vector(to_unsigned(ip_id_majv,8)) &
		std_logic_vector(to_unsigned(ip_id_minv,8)) &
		std_logic_vector(to_unsigned(ip_id_hscr,8));
		
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	-- output signals 

	-- host interface
	signal host_dout_i     :  std_logic_vector(31 downto 0);
	signal host_ack_i      :  std_logic;

	-- s/d clock
	signal sd_clock_x_i    :  std_logic;
	
	-- over-current reset
	signal ovc_reset_i     :  std_logic;

	-- M0 phy
	signal drv_en_0_i      :  std_logic;
	signal drv_ah_0_i      :  std_logic;
	signal drv_al_0_i      :  std_logic;
	signal drv_bh_0_i      :  std_logic;
	signal drv_bl_0_i      :  std_logic;
	signal drv_ch_0_i      :  std_logic;
	signal drv_cl_0_i      :  std_logic;
	signal drv_dh_0_i      :  std_logic;
	signal drv_dl_0_i      :  std_logic;
	
	-- M1 phy
	signal drv_en_1_i      :  std_logic;
	signal drv_ah_1_i      :  std_logic;
	signal drv_al_1_i      :  std_logic;
	signal drv_bh_1_i      :  std_logic;
	signal drv_bl_1_i      :  std_logic;
	signal drv_ch_1_i      :  std_logic;
	signal drv_cl_1_i      :  std_logic;
	signal drv_dh_1_i      :  std_logic;
	signal drv_dl_1_i      :  std_logic;
	
	-- hall sensor driver direction
	signal hlsdir_a_1_i    : std_logic;
	signal hlsdir_b_1_i    : std_logic;
	
	-- i2c
	signal fmc_scl_o_i     :  std_logic;
	signal fmc_scl_t_i     :  std_logic;
	signal fmc_sda_o_i     :  std_logic;
	signal fmc_sda_t_i     :  std_logic;

	-- RDC
	signal rdc_sample_i    :  std_logic;
	signal rdc_mosi_i      :  std_logic;
	signal rdc_sclk_i      :  std_logic;
	signal rdc_ss_i        :  std_logic;
	
	-- bus enable
	signal vt_enable_i     :  std_logic;
	
	
	-- xadc external mux
	signal xadc_mux_a0_i   :  std_logic;
	signal xadc_mux_a1_i   :  std_logic;
	signal xadc_mux_en_i   :  std_logic;

	---------------------------------------------------------------------------
	
	-- motor #0 control
	
	-- external angle
	signal ext_angle_0_i   : std_logic_vector(31 downto 0);

	-- resolver            
	signal rsv_priexc_0_i  : std_logic_vector(17 downto 0);
	signal rsv_secsin_0_i  : std_logic_vector(17 downto 0);
	signal rsv_seccos_0_i  : std_logic_vector(17 downto 0);

	-- hall sensor         
	signal hls_hallsig_0_i : std_logic_vector(5 downto 0);

	-- digital encoder     
	signal enc_cha_0_i     : std_logic;
	signal enc_chb_0_i     : std_logic;
	signal enc_chi_0_i     : std_logic;

	-- input data          
	--                     
	signal acq_iphs_a_0_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_b_0_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_c_0_i  : std_logic_vector(17 downto 0);
	signal acq_ibus_x_0_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_a_0_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_b_0_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_c_0_i  : std_logic_vector(17 downto 0);
	signal acq_vbus_x_0_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_n_0_i  : std_logic_vector(17 downto 0);
	--                     
	signal acq_sync_0_i    : std_logic;

	-- gate unit switches   
	signal guswcnt_0_i     : std_logic_vector(31 downto 0);

	-- in-failure          
	signal hwfail_0_i      : std_logic;

	-- in-kill             
	signal hwkill_0_i      : std_logic;

	---------------------------------------------------------------------------
	
	-- motor #1 control
	
	-- external angle
	signal ext_angle_1_i   : std_logic_vector(31 downto 0);

	-- resolver            
	signal rsv_priexc_1_i  : std_logic_vector(17 downto 0);
	signal rsv_secsin_1_i  : std_logic_vector(17 downto 0);
	signal rsv_seccos_1_i  : std_logic_vector(17 downto 0);

	-- hall sensor         
	signal hls_hallsig_1_i : std_logic_vector(5 downto 0);

	-- digital encoder     
	signal enc_cha_1_i     : std_logic;
	signal enc_chb_1_i     : std_logic;
	signal enc_chi_1_i     : std_logic;

	-- input data          
	--                     
	signal acq_iphs_a_1_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_b_1_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_c_1_i  : std_logic_vector(17 downto 0);
	signal acq_ibus_x_1_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_a_1_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_b_1_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_c_1_i  : std_logic_vector(17 downto 0);
	signal acq_vbus_x_1_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_n_1_i  : std_logic_vector(17 downto 0);
	--                     
	signal acq_sync_1_i    : std_logic;

	-- gate unit switches   
	signal guswcnt_1_i     : std_logic_vector(31 downto 0);

	-- in-failure          
	signal hwfail_1_i      : std_logic;

	-- in-kill             
	signal hwkill_1_i      : std_logic;

	---------------------------------------------------------------------------


begin

	---------------------------------------------------------------------------
	-- fwd output
	---------------------------------------------------------------------------

	-- host interface
	host_dout           <= host_dout_i;
	host_ack            <= host_ack_i;

	-- s/d clock
	sd_clock_x          <= sd_clock_x_i;
	
	-- over-current reset
	ovc_reset           <= ovc_reset_i;

	-- M0 phy
	drv_en_0            <= drv_en_0_i;
	drv_ah_0            <= drv_ah_0_i;
	drv_al_0            <= drv_al_0_i;
	drv_bh_0            <= drv_bh_0_i;
	drv_bl_0            <= drv_bl_0_i;
	drv_ch_0            <= drv_ch_0_i;
	drv_cl_0            <= drv_cl_0_i;
	drv_dh_0            <= drv_dh_0_i;
	drv_dl_0            <= drv_dl_0_i;

	-- M1 phy
	drv_en_1            <= drv_en_1_i;
	drv_ah_1            <= drv_ah_1_i;
	drv_al_1            <= drv_al_1_i;
	drv_bh_1            <= drv_bh_1_i;
	drv_bl_1            <= drv_bl_1_i;
	drv_ch_1            <= drv_ch_1_i;
	drv_cl_1            <= drv_cl_1_i;
	drv_dh_1            <= drv_dh_1_i;
	drv_dl_1            <= drv_dl_1_i;
	
	-- hall sensor driver direction
	hlsdir_a_1          <= hlsdir_a_1_i;
	hlsdir_b_1          <= hlsdir_b_1_i;

	-- i2c
	fmc_scl_o           <= fmc_scl_o_i;
	fmc_scl_t           <= fmc_scl_t_i;
	fmc_sda_o           <= fmc_sda_o_i;
	fmc_sda_t           <= fmc_sda_t_i;

	-- RDC
	rdc_sample          <= rdc_sample_i;
	rdc_mosi            <= rdc_mosi_i;
	rdc_sclk            <= rdc_sclk_i;
	rdc_ss              <= rdc_ss_i;
	
	-- bus enable
	vt_enable           <= vt_enable_i;

	-- xadc external mux
	xadc_mux_a0         <= xadc_mux_a0_i;
	xadc_mux_a1         <= xadc_mux_a1_i;
	xadc_mux_en         <= xadc_mux_en_i;

	---------------------------------------------------------------------------
	
	-- motor #0 control
	
	-- external angle
	ext_angle_0    <=  ext_angle_0_i;
	
	-- resolver
	rsv_priexc_0   <=  rsv_priexc_0_i;
	rsv_secsin_0   <=  rsv_secsin_0_i;
	rsv_seccos_0   <=  rsv_seccos_0_i;
	
	-- hall sensor
	hls_hallsig_0  <=  hls_hallsig_0_i;
	
	-- digital encoder
	enc_cha_0      <=  enc_cha_0_i;
	enc_chb_0      <=  enc_chb_0_i;
	enc_chi_0      <=  enc_chi_0_i;
	
	-- input data
	--
	acq_iphs_a_0   <=  acq_iphs_a_0_i;
	acq_iphs_b_0   <=  acq_iphs_b_0_i;
	acq_iphs_c_0   <=  acq_iphs_c_0_i;
	acq_ibus_x_0   <=  acq_ibus_x_0_i;
	acq_vphs_a_0   <=  acq_vphs_a_0_i;
	acq_vphs_b_0   <=  acq_vphs_b_0_i;
	acq_vphs_c_0   <=  acq_vphs_c_0_i;
	acq_vbus_x_0   <=  acq_vbus_x_0_i;
	acq_vphs_n_0   <=  acq_vphs_n_0_i;
	--
	acq_sync_0     <=  acq_sync_0_i;
	
	-- gate unit switches
	guswcnt_0      <=  guswcnt_0_i;
	
	-- in-failure
	hwfail_0       <=  hwfail_0_i;
	
	-- in-kill
	hwkill_0       <=  hwkill_0_i;
	
	---------------------------------------------------------------------------
	
	-- motor #1 control
	
	-- external angle
	ext_angle_1    <=  ext_angle_1_i;
	
	-- resolver
	rsv_priexc_1   <=  rsv_priexc_1_i;
	rsv_secsin_1   <=  rsv_secsin_1_i;
	rsv_seccos_1   <=  rsv_seccos_1_i;
	
	-- hall sensor
	hls_hallsig_1  <=  hls_hallsig_1_i;
	
	-- digital encoder
	enc_cha_1      <=  enc_cha_1_i;
	enc_chb_1      <=  enc_chb_1_i;
	enc_chi_1      <=  enc_chi_1_i;
	
	-- input data
	--
	acq_iphs_a_1   <=  acq_iphs_a_1_i;
	acq_iphs_b_1   <=  acq_iphs_b_1_i;
	acq_iphs_c_1   <=  acq_iphs_c_1_i;
	acq_ibus_x_1   <=  acq_ibus_x_1_i;
	acq_vphs_a_1   <=  acq_vphs_a_1_i;
	acq_vphs_b_1   <=  acq_vphs_b_1_i;
	acq_vphs_c_1   <=  acq_vphs_c_1_i;
	acq_vbus_x_1   <=  acq_vbus_x_1_i;
	acq_vphs_n_1   <=  acq_vphs_n_1_i;
	--
	acq_sync_1     <=  acq_sync_1_i;
	
	-- gate unit switches
	guswcnt_1      <=  guswcnt_1_i;
	
	-- in-failure
	hwfail_1       <=  hwfail_1_i;
	
	-- in-kill
	hwkill_1       <=  hwkill_1_i;
	
	---------------------------------------------------------------------------
	
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	-- status register
	status_reg(C_STATUS_REG_OVC_0) <= ovc_ct_0;
	status_reg(C_STATUS_REG_OVC_1) <= ovc_ct_1;
	
	---------------------------------------------------------------------------
	
	-- hall sensor differential Input enable (motor #2 only)
	hlsdir_a_1_i <= '0';
	hlsdir_b_1_i <= '0';
	
	-- overcurrent reset
	ovc_reset_i  <= '1' when (reseterr_0 = '1' or reseterr_1 = '1') and coilenab_0 = '0' and coilgear_0 = '0' and coilenab_1 = '0' and coilgear_1 = '0' else '0';
	
	-- bus enable
	vt_enable_i  <= control_reg(C_CONTROL_REG_VT_ENABLE);
	
	---------------------------------------------------------------------------
	-- I2C --------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	fmc_scl_o_i  <= '1';
	fmc_scl_t_i  <= '1';
	
	fmc_sda_o_i  <= '1';
	fmc_sda_t_i  <= '1';
	
	---------------------------------------------------------------------------
	-- RDC --------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	rdc_sample_i <= '1';
	rdc_mosi_i   <= '1';
	rdc_sclk_i   <= '1';
	rdc_ss_i     <= '1';
	
	---------------------------------------------------------------------------
	-- antimetastability motor sensor
	
	LK_SENSOR_A_0_INST : inlkmeta generic map (C_NCLKS => K_UMSK) port map (clock => host_clk, datin => sensor_a_0, datout => lk_sensor_a_0);
	LK_SENSOR_B_0_INST : inlkmeta generic map (C_NCLKS => K_UMSK) port map (clock => host_clk, datin => sensor_b_0, datout => lk_sensor_b_0);
	LK_SENSOR_C_0_INST : inlkmeta generic map (C_NCLKS => K_UMSK) port map (clock => host_clk, datin => sensor_c_0, datout => lk_sensor_c_0);
	
	LK_SENSOR_A_1_INST : inlkmeta generic map (C_NCLKS => K_UMSK) port map (clock => host_clk, datin => sensor_a_1, datout => lk_sensor_a_1);
	LK_SENSOR_B_1_INST : inlkmeta generic map (C_NCLKS => K_UMSK) port map (clock => host_clk, datin => sensor_b_1, datout => lk_sensor_b_1);
	LK_SENSOR_C_1_INST : inlkmeta generic map (C_NCLKS => K_UMSK) port map (clock => host_clk, datin => sensor_c_1, datout => lk_sensor_c_1);
	
	---------------------------------------------------------------------------
	-- deglitch motor sensors
	
	DG_SENSOR_A_0_INST : unglitch generic map (C_CLOCK_FREQ => C_HOST_CLK_FREQ_HZ, C_FILTER_TIME => K_DEGLITH_TIME) port map (clock  => host_clk, reset  => host_rst, datin  => lk_sensor_a_0, datout => dg_sensor_a_0);
	DG_SENSOR_B_0_INST : unglitch generic map (C_CLOCK_FREQ => C_HOST_CLK_FREQ_HZ, C_FILTER_TIME => K_DEGLITH_TIME) port map (clock  => host_clk, reset  => host_rst, datin  => lk_sensor_b_0, datout => dg_sensor_b_0);
	DG_SENSOR_C_0_INST : unglitch generic map (C_CLOCK_FREQ => C_HOST_CLK_FREQ_HZ, C_FILTER_TIME => K_DEGLITH_TIME) port map (clock  => host_clk, reset  => host_rst, datin  => lk_sensor_c_0, datout => dg_sensor_c_0);
	
	DG_SENSOR_A_1_INST : unglitch generic map (C_CLOCK_FREQ => C_HOST_CLK_FREQ_HZ, C_FILTER_TIME => K_DEGLITH_TIME) port map (clock  => host_clk, reset  => host_rst, datin  => lk_sensor_a_1, datout => dg_sensor_a_1);
	DG_SENSOR_B_1_INST : unglitch generic map (C_CLOCK_FREQ => C_HOST_CLK_FREQ_HZ, C_FILTER_TIME => K_DEGLITH_TIME) port map (clock  => host_clk, reset  => host_rst, datin  => lk_sensor_b_1, datout => dg_sensor_b_1);
	DG_SENSOR_C_1_INST : unglitch generic map (C_CLOCK_FREQ => C_HOST_CLK_FREQ_HZ, C_FILTER_TIME => K_DEGLITH_TIME) port map (clock  => host_clk, reset  => host_rst, datin  => lk_sensor_c_1, datout => dg_sensor_c_1);
	
	---------------------------------------------------------------------------
	
	-- motor sensor signals
	
	-- external angle: 0/1
	ext_angle_0_i <= ext_x_angle_0;
	ext_angle_1_i <= ext_x_angle_1;
	
	-- M0
	rsv_priexc_0_i <= rsv_x_priexc_0;
	rsv_secsin_0_i <= rsv_x_secsin_0;
	rsv_seccos_0_i <= rsv_x_seccos_0;

	-- M1
	rsv_priexc_1_i <= rsv_x_priexc_1;
	rsv_secsin_1_i <= rsv_x_secsin_1;
	rsv_seccos_1_i <= rsv_x_seccos_1;
	
	-- M0
	
	-- hall
	hls_hallsig_0_i(0) <= dg_sensor_a_0;
	hls_hallsig_0_i(1) <= dg_sensor_b_0;
	hls_hallsig_0_i(2) <= dg_sensor_c_0;
	
	-- 2nd: unused
	hls_hallsig_0_i(5 downto 3) <= (others => '0');

	-- encoder
	enc_chi_0_i <= dg_sensor_a_0;
	enc_cha_0_i <= dg_sensor_b_0;
	enc_chb_0_i <= dg_sensor_c_0;
	
	-- M1
	
	-- hall
	hls_hallsig_1_i(0) <= dg_sensor_a_1;
	hls_hallsig_1_i(1) <= dg_sensor_b_1;
	hls_hallsig_1_i(2) <= dg_sensor_c_1;
	
	-- 2nd: unused
	hls_hallsig_1_i(5 downto 3) <= (others => '0');

	-- encoder
	enc_chi_1_i <= dg_sensor_a_1;
	enc_cha_1_i <= dg_sensor_b_1;
	enc_chb_1_i <= dg_sensor_c_1;
	
	---------------------------------------------------------------------------
	-- XADC -------------------------------------------------------------------
	
	
	-- xadc DRP for test only
	xadc_drp_daddr <= (others => '0');
	xadc_drp_di    <= (others => '0');
	xadc_drp_den   <= '0';
	xadc_drp_dwe   <= '0';
	
	
	DO_XADC_GEN : if C_XADC > 0 generate
	
		xa_vbus_0     <= control_reg(C_CONTROL_REG_XA_VBUS_0);
		xa_iab_0      <= control_reg(C_CONTROL_REG_XA_IAB_0);

		xa_vbus_1     <= control_reg(C_CONTROL_REG_XA_VBUS_1);
		xa_iab_1      <= control_reg(C_CONTROL_REG_XA_IAB_1);
		
		xadc_mux_en_i <= '1';
	
		XADC_ACQ_INST : mcbxadc
			generic map
			(
				C_SYSTEMCLOCK => C_HOST_CLK_FREQ_HZ
			)
			port map
			(
				clock          => host_clk,
				reset          => host_rst,
				-- on-chip temperature
				val_temp       => xa_val_temp,
				-- M0/1 values
				val_ia_0       => xa_val_ia_0,
				val_ib_0       => xa_val_ib_0,
				val_vbus_0     => xa_val_vbus_0,
				val_ia_1       => xa_val_ia_1,
				val_ib_1       => xa_val_ib_1,
				val_vbus_1     => xa_val_vbus_1,
				-- sync
				val_sync       => xa_val_sync,
				-- xadc drp access
				xadc_drp_daddr => xadc_drp_daddr,
				xadc_drp_di    => xadc_drp_di,
				xadc_drp_den   => xadc_drp_den,
				xadc_drp_dwe   => xadc_drp_dwe,
				xadc_drp_do    => xadc_drp_do,
				xadc_drp_drdy  => xadc_drp_drdy,
				-- xadc analog signals
				xadc_vp        => xadc_vp,
				xadc_vn        => xadc_vn,
				xadc_vaux0_p   => xadc_vaux0_p,
				xadc_vaux0_n   => xadc_vaux0_n,
				xadc_vaux8_p   => xadc_vaux8_p,
				xadc_vaux8_n   => xadc_vaux8_n,
				-- xadc external mux
				xadc_mux_a0    => xadc_mux_a0_i,
				xadc_mux_a1    => xadc_mux_a1_i
			);
	end generate;
			
	NOT_XADC_GEN : if C_XADC <= 0 generate
	
		xa_vbus_0     <= '0';
		xa_iab_0      <= '0';

		xa_vbus_1     <= '0';
		xa_iab_1      <= '0';
		
		xa_val_temp   <= (others => '0');
		
		xa_val_ia_0   <= (others => '0');
		xa_val_ib_0   <= (others => '0');
		xa_val_vbus_0 <= (others => '0');
		xa_val_ia_1   <= (others => '0');
		xa_val_ib_1   <= (others => '0');
		xa_val_vbus_1 <= (others => '0');
		
		xa_val_sync   <= '0';
		
		xadc_drp_do   <= (others => '0');
		xadc_drp_drdy <= '0';
		
		xadc_mux_a0_i <= '0';
		xadc_mux_a1_i <= '0';
		
		xadc_mux_en_i <= '0';
		
	end generate;
	
	---------------------------------------------------------------------------
	
	-- output s/d clock
	sd_clock_x_i <= not sd_clock;
	
	LK_SD_IA_0_INST   : inlkmeta generic map (C_NCLKS => K_UMSK) port map (clock => sd_clock, datin => sd_ia_dat_0,   datout => lk_sd_ia_0  );
	LK_SD_IB_0_INST   : inlkmeta generic map (C_NCLKS => K_UMSK) port map (clock => sd_clock, datin => sd_ib_dat_0,   datout => lk_sd_ib_0  );
	LK_SD_VBUS_0_INST : inlkmeta generic map (C_NCLKS => K_UMSK) port map (clock => sd_clock, datin => sd_vbus_dat_0, datout => lk_sd_vbus_0);
	
	LK_SD_IA_1_INST   : inlkmeta generic map (C_NCLKS => K_UMSK) port map (clock => sd_clock, datin => sd_ia_dat_1,   datout => lk_sd_ia_1  );
	LK_SD_IB_1_INST   : inlkmeta generic map (C_NCLKS => K_UMSK) port map (clock => sd_clock, datin => sd_ib_dat_1,   datout => lk_sd_ib_1  );
	LK_SD_VBUS_1_INST : inlkmeta generic map (C_NCLKS => K_UMSK) port map (clock => sd_clock, datin => sd_vbus_dat_1, datout => lk_sd_vbus_1);
	
	---------------------------------------------------------------------------
	
	sd_data_mux(0) <= lk_sd_ia_0;
	sd_data_mux(1) <= lk_sd_ib_0;
	sd_data_mux(2) <= lk_sd_vbus_0;
	
	sd_data_mux(3) <= lk_sd_ia_1;
	sd_data_mux(4) <= lk_sd_ib_1;
	sd_data_mux(5) <= lk_sd_vbus_1;
	
	S3_ACQ_INST : s3acqmult
		generic map
		(
			C_RESULT_WIDTH => C_INVERTER_SD_WIDTH,
			C_NUM_CHANNELS => 6
		)
		port map
		(
			clock   => host_clk,
			reset   => host_rst,
			--
			sclock  => sd_clock,
			sdata   => sd_data_mux,
			--
			pdata0  => sd_raw_ia_0,
			pdata1  => sd_raw_ib_0,
			pdata2  => sd_raw_vbus_0,
			pdata3  => sd_raw_ia_1,
			pdata4  => sd_raw_ib_1,
			pdata5  => sd_raw_vbus_1,
			pdata6  => open,
			pdata7  => open,
			pdata8  => open,
			pdata9  => open,
			pdata10 => open,
			pdata11 => open,
			pdata12 => open,
			pdata13 => open,
			pdata14 => open,
			pdata15 => open,
			
			finish => sd_raw_sync
		);
		
	sd_val_ia_0   <= sd_raw_ia_0   & C_INVDAT_RAW2VAL_PAD;
	sd_val_ib_0   <= sd_raw_ib_0   & C_INVDAT_RAW2VAL_PAD;
	sd_val_vbus_0 <= sd_raw_vbus_0 & C_INVDAT_RAW2VAL_PAD;

	sd_val_ia_1   <= sd_raw_ia_1   & C_INVDAT_RAW2VAL_PAD;
	sd_val_ib_1   <= sd_raw_ib_1   & C_INVDAT_RAW2VAL_PAD;
	sd_val_vbus_1 <= sd_raw_vbus_1 & C_INVDAT_RAW2VAL_PAD;
	
	sd_val_sync   <= sd_raw_sync;
	
	---------------------------------------------------------------------------

	-- sd normalized
	
	sd_bip_ia_0   <= std_logic_vector(signed(sd_val_ia_0  ) - 2**17);
	sd_bip_ib_0   <= std_logic_vector(signed(sd_val_ib_0  ) - 2**17);
	sd_bip_vbus_0 <= std_logic_vector(signed(sd_val_vbus_0) - 2**17);
	
	sd_bip_ia_1   <= std_logic_vector(signed(sd_val_ia_1  ) - 2**17);
	sd_bip_ib_1   <= std_logic_vector(signed(sd_val_ib_1  ) - 2**17);
	sd_bip_vbus_1 <= std_logic_vector(signed(sd_val_vbus_1) - 2**17);
	
	---------------------------------------------------------------------------

	-- xa normalized
	
	xa_bip_ia_0   <= std_logic_vector(signed(xa_val_ia_0  ) - 2**17);
	xa_bip_ib_0   <= std_logic_vector(signed(xa_val_ib_0  ) - 2**17);
	xa_bip_vbus_0 <= std_logic_vector(signed(xa_val_vbus_0) - 2**17);
	
	xa_bip_ia_1   <= std_logic_vector(signed(xa_val_ia_1  ) - 2**17);
	xa_bip_ib_1   <= std_logic_vector(signed(xa_val_ib_1  ) - 2**17);
	xa_bip_vbus_1 <= std_logic_vector(signed(xa_val_vbus_1) - 2**17);
	
	---------------------------------------------------------------------------
	-- mux sd/xadc ------------------------------------------------------------
	---------------------------------------------------------------------------
	
	mux_iphs_a_0 <= xa_bip_ia_0   when xa_iab_0  = '1' else sd_bip_ia_0;
	mux_iphs_b_0 <= xa_bip_ib_0   when xa_iab_0  = '1' else sd_bip_ib_0;
	mux_vbus_x_0 <= xa_bip_vbus_0 when xa_vbus_0 = '1' else sd_bip_vbus_0;
	mux_sync_0   <= xa_val_sync   when xa_iab_0  = '1' else sd_val_sync;

	mux_iphs_a_1 <= xa_bip_ia_1   when xa_iab_1  = '1' else sd_bip_ia_1;
	mux_iphs_b_1 <= xa_bip_ib_1   when xa_iab_1  = '1' else sd_bip_ib_1;
	mux_vbus_x_1 <= xa_bip_vbus_1 when xa_vbus_1 = '1' else sd_bip_vbus_1;
	mux_sync_1   <= xa_val_sync   when xa_iab_1  = '1' else sd_val_sync;
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	
	-----------------------------------------------------------------------------------------

	-- M0 a/d
	acq_iphs_a_0_i <= mux_iphs_a_0;
	acq_iphs_b_0_i <= mux_iphs_b_0;
	acq_iphs_c_0_i <= (others => '0');
	acq_ibus_x_0_i <= (others => '0');
	acq_vphs_a_0_i <= (others => '0');
	acq_vphs_b_0_i <= (others => '0');
	acq_vphs_c_0_i <= (others => '0');
	acq_vbus_x_0_i <= mux_vbus_x_0;
	acq_vphs_n_0_i <= (others => '0');
	acq_sync_0_i   <= mux_sync_0;
	
	-----------------------------------------------------------------------------------------

	-- M1 a/d
	acq_iphs_a_1_i <= mux_iphs_a_1;
	acq_iphs_b_1_i <= mux_iphs_b_1;
	acq_iphs_c_1_i <= (others => '0');
	acq_ibus_x_1_i <= (others => '0');
	acq_vphs_a_1_i <= (others => '0');
	acq_vphs_b_1_i <= (others => '0');
	acq_vphs_c_1_i <= (others => '0');
	acq_vbus_x_1_i <= mux_vbus_x_1;
	acq_vphs_n_1_i <= (others => '0');
	acq_sync_1_i   <= mux_sync_1;
	
	-----------------------------------------------------------------------------------------
	
	-- M0 failure/kill
	hwfail_0_i <= '0';
	hwkill_0_i <= '0';
	
	-- M1 failure/kill
	hwfail_1_i <= '0';
	hwkill_1_i <= '0';
	

	---------------------------------------------------------------------------
	-- filtering --------------------------------------------------------------
	---------------------------------------------------------------------------
	
	DO_OC_TEMP_LPF_GEN : if C_XADC > 0 generate
		OC_TEMP_LPF_INST : filtiir
			generic map
			(
				C_FAMILY => C_FAMILY
			)
			port map
			( 
				clock  => host_clk,
				reset  => host_rst,
				
				start  => xa_val_sync,
				fktau  => oc_temp_fkt,
				inraw  => xa_val_temp,
				outflt => oc_temp_val,
				finish => open
			);
	end generate;
			
	NOT_OC_TEMP_LPF_GEN : if C_XADC <= 0 generate
		oc_temp_val <= (others => '0');
	end generate;
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	-- gate unit control ------------------------------------------------------
	
	-- M0
	
	rpfm_3phl2_0 <= '1' when modtype_0 = MODTYPE_RPFM and is3phases_0 = '1' and modlevels_0 = MODLEVELS_2 else '0';
	pwm_3phl2_0  <= '1' when modtype_0 = MODTYPE_PWM  and is3phases_0 = '1' and modlevels_0 = MODLEVELS_2 else '0';
	pwm_2phl2_0  <= '1' when modtype_0 = MODTYPE_PWM  and is3phases_0 = '0' and modlevels_0 = MODLEVELS_2 else '0';
	
	coil_a_0 <=
			rpfm3p_l2cxw1_0(0) when rpfm_3phl2_0 = '1' else
			pwm3p_l2cxw1_0(0)  when pwm_3phl2_0  = '1' else
			pwm2p_l2c1w1_0(0)  when pwm_2phl2_0  = '1' else '0';
	
	coil_b_0 <=
			rpfm3p_l2cxw1_0(1) when rpfm_3phl2_0 = '1' else
			pwm3p_l2cxw1_0(1)  when pwm_3phl2_0  = '1' else
			pwm2p_l2c2w1_0(0)  when pwm_2phl2_0  = '1' else '0';
	
	coil_c_0 <=
			rpfm3p_l2cxw1_0(2) when rpfm_3phl2_0 = '1' else
			pwm3p_l2cxw1_0(2)  when pwm_3phl2_0  = '1' else
			pwm2p_l2c1w1_0(1)  when pwm_2phl2_0  = '1' else '0';
	
	coil_d_0 <=
			'0'                when rpfm_3phl2_0 = '1' else
			'0'                when pwm_3phl2_0  = '1' else
			pwm2p_l2c2w1_0(1)  when pwm_2phl2_0  = '1' else '0';
			
			
	-- M1
	
	rpfm_3phl2_1 <= '1' when modtype_1 = MODTYPE_RPFM and is3phases_1 = '1' and modlevels_1 = MODLEVELS_2 else '0';
	pwm_3phl2_1  <= '1' when modtype_1 = MODTYPE_PWM  and is3phases_1 = '1' and modlevels_1 = MODLEVELS_2 else '0';
	pwm_2phl2_1  <= '1' when modtype_1 = MODTYPE_PWM  and is3phases_1 = '0' and modlevels_1 = MODLEVELS_2 else '0';
	
	coil_a_1 <=
			rpfm3p_l2cxw1_1(0) when rpfm_3phl2_1 = '1' else
			pwm3p_l2cxw1_1(0)  when pwm_3phl2_1  = '1' else
			pwm2p_l2c1w1_1(0)  when pwm_2phl2_1  = '1' else '0';
	
	coil_b_1 <=
			rpfm3p_l2cxw1_1(1) when rpfm_3phl2_1 = '1' else
			pwm3p_l2cxw1_1(1)  when pwm_3phl2_1  = '1' else
			pwm2p_l2c2w1_1(0)  when pwm_2phl2_1  = '1' else '0';
	
	coil_c_1 <=
			rpfm3p_l2cxw1_1(2) when rpfm_3phl2_1 = '1' else
			pwm3p_l2cxw1_1(2)  when pwm_3phl2_1  = '1' else
			pwm2p_l2c1w1_1(1)  when pwm_2phl2_1  = '1' else '0';
	
	coil_d_1 <=
			'0'                when rpfm_3phl2_1 = '1' else
			'0'                when pwm_3phl2_1  = '1' else
			pwm2p_l2c2w1_1(1)  when pwm_2phl2_1  = '1' else '0';
			
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	-- enable -----------------------------------------------------------------
	
	drv_en_0_i <= '1' when coilgear_0 = '1' and modlevels_0 = MODLEVELS_2 else '0';
	drv_en_1_i <= '1' when coilgear_1 = '1' and modlevels_1 = MODLEVELS_2 else '0';
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	-- dead time --------------------------------------------------------------
	
	coil_cmt(DRV_A_0) <= coil_a_0 and drv_en_0_i;
	coil_cmt(DRV_B_0) <= coil_b_0 and drv_en_0_i;
	coil_cmt(DRV_C_0) <= coil_c_0 and drv_en_0_i;
	coil_cmt(DRV_D_0) <= coil_d_0 and drv_en_0_i;
	
	coil_cmt(DRV_A_1) <= coil_a_1 and drv_en_1_i;
	coil_cmt(DRV_B_1) <= coil_b_1 and drv_en_1_i;
	coil_cmt(DRV_C_1) <= coil_c_1 and drv_en_1_i;
	coil_cmt(DRV_D_1) <= coil_d_1 and drv_en_1_i;

	DTCOIL_GEN : for i in coil_cmt'range generate
	
		DTCOIL_INST : dthlcmd
			generic map
			(
				C_CLOCK_FREQ => C_HOST_CLK_FREQ_HZ,
				C_DEAD_TIME  => C_DEAD_TIME
			)
			port map
			(
				clock   => host_clk,
				reset   => host_rst,
				incmd   => coil_cmt(i),
				outhi   => coil_cmh(i),
				outlo   => coil_cml(i)
			);
	end generate;
	
	-- M0
	
	drv_ah_0_i <= coil_cmh(DRV_A_0);
	drv_al_0_i <= coil_cml(DRV_A_0);

	drv_bh_0_i <= coil_cmh(DRV_B_0);
	drv_bl_0_i <= coil_cml(DRV_B_0);
	
	drv_ch_0_i <= coil_cmh(DRV_C_0);
	drv_cl_0_i <= coil_cml(DRV_C_0);
	
	drv_dh_0_i <= coil_cmh(DRV_D_0);
	drv_dl_0_i <= coil_cml(DRV_D_0);
	
	-- M1
	
	drv_ah_1_i <= coil_cmh(DRV_A_1);
	drv_al_1_i <= coil_cml(DRV_A_1);

	drv_bh_1_i <= coil_cmh(DRV_B_1);
	drv_bl_1_i <= coil_cml(DRV_B_1);
	
	drv_ch_1_i <= coil_cmh(DRV_C_1);
	drv_cl_1_i <= coil_cml(DRV_C_1);
	
	drv_dh_1_i <= coil_cmh(DRV_D_1);
	drv_dl_1_i <= coil_cml(DRV_D_1);
	
	---------------------------------------------------------------------------
	-- GU swtch count M0
	
	guevsrc_0(0) <= drv_ah_0_i;
	guevsrc_0(1) <= drv_al_0_i;
	
	guevsrc_0(2) <= drv_bh_0_i;
	guevsrc_0(3) <= drv_bl_0_i;
	
	guevsrc_0(4) <= drv_ch_0_i;
	guevsrc_0(5) <= drv_cl_0_i;
	
	guevsrc_0(6) <= drv_dh_0_i;
	guevsrc_0(7) <= drv_dl_0_i;

	-- event counter
	GUSWCNT_M0 : evxcount
		generic map
		(
			C_DATA_WIDTH  => 32,
			C_EVENT_WIDTH => 8,
			C_COUNT_MODE  => 0
		)
		port map
		(
			clock   => host_clk,
			reset   => host_rst,
			sigevt  => guevsrc_0,
			counter => guswcnt_0_i
		);

	---------------------------------------------------------------------------
	-- GU swtch count M1
	
	guevsrc_1(0) <= drv_ah_1_i;
	guevsrc_1(1) <= drv_al_1_i;
	
	guevsrc_1(2) <= drv_bh_1_i;
	guevsrc_1(3) <= drv_bl_1_i;
	
	guevsrc_1(4) <= drv_ch_1_i;
	guevsrc_1(5) <= drv_cl_1_i;
	
	guevsrc_1(6) <= drv_dh_1_i;
	guevsrc_1(7) <= drv_dl_1_i;

	-- event counter
	GUSWCNT_M1 : evxcount
		generic map
		(
			C_DATA_WIDTH  => 32,
			C_EVENT_WIDTH => 8,
			C_COUNT_MODE  => 0
		)
		port map
		(
			clock   => host_clk,
			reset   => host_rst,
			sigevt  => guevsrc_1,
			counter => guswcnt_1_i
		);

	---------------------------------------------------------------------------
	
	-- implement slave model software accessible register(s)
	SLAVE_WRITE_PROC : process(host_clk)
	begin
		if rising_edge(host_clk) then
			if host_rst = '1' then
				control_reg <= (others => '0');
				oc_temp_fkt <= K_FILTER_FAST;
			elsif host_we = '1' and host_en = '1' then
				case adrtoint(host_addr) is
					when RA_ip_ident    => null;
					when RA_status_reg  => null;
					when RA_control_reg => storereg(control_reg,host_din);
					when RA_oc_temp_val => null;
					when RA_oc_temp_fkt => storereg(oc_temp_fkt,host_din);
					when others         => null;
				end case;
			end if;
		end if;
	end process;

	-- implement slave model software accessible register(s) read mux
	SLAVE_READ_PROC : process(host_clk)
		variable v : std_logic_vector(31 downto 0);
	begin
		if rising_edge(host_clk) then
			v := (others => '0');
			case adrtoint(host_addr) is
				when RA_ip_ident    => v := loadreg(ip_ident_k );
				when RA_status_reg  => v := loadreg(status_reg );
				when RA_control_reg => v := loadreg(control_reg);
				when RA_oc_temp_val => v := loadreg(oc_temp_val);
				when RA_oc_temp_fkt => v := loadreg(oc_temp_fkt);
				when others         => null;
			end case;
			host_dout_i <= v;
		end if;
	end process;
	
	-- data transfer ack
	host_ack_i <= host_we or host_en_dl1;
	
	-- host access delay
	HOSTDLY_PROC : process(host_clk)
	begin
		if rising_edge(host_clk) then
			host_en_dl1 <= host_en;
		end if;
	end process;
	
end Behavioral;
