-------------------------------------------------------------------------------------------
-- Copyright  2011-2012, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    16:00:00 12/25/2011 
-- Design Name: 
-- Module Name:    sinc3flt - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity sinc3flt is
	generic
	(
		C_MATH_WIDTH   : integer := 24;
		C_RESULT_WIDTH : integer := 16
	);
	port
	(
		clock  : in  std_logic;
		reset  : in  std_logic;
		sdata  : in  std_logic;
		wload  : in  std_logic;
		wdata  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0)
	);
end sinc3flt;

architecture Behavioral of sinc3flt is

	signal acc1     : std_logic_vector(C_MATH_WIDTH-1 downto 0);
	signal acc2     : std_logic_vector(C_MATH_WIDTH-1 downto 0);
	signal acc3     : std_logic_vector(C_MATH_WIDTH-1 downto 0);
	signal acc3_d2  : std_logic_vector(C_MATH_WIDTH-1 downto 0);
	signal diff1    : std_logic_vector(C_MATH_WIDTH-1 downto 0);
	signal diff2    : std_logic_vector(C_MATH_WIDTH-1 downto 0);
	signal diff3    : std_logic_vector(C_MATH_WIDTH-1 downto 0);
	signal diff1_d  : std_logic_vector(C_MATH_WIDTH-1 downto 0);
	signal diff2_d  : std_logic_vector(C_MATH_WIDTH-1 downto 0);

begin

	SINC3_PROC : process(clock) is
	begin
	
		if rising_edge(clock) then
	
			if reset = '1' then
			
				acc1  <= (others => '0');
				acc2  <= (others => '0');
				acc3  <= (others => '0');
			
				diff1 <= (others => '0');
				diff2 <= (others => '0');
				diff3 <= (others => '0');
				
				acc3_d2 <= (others => '0');
				diff1_d <= (others => '0');
				diff2_d <= (others => '0');

			else
			
				if sdata = '1' then
					acc1 <= acc1 + 1;
				--else
				--	acc1 <= acc1 - 1;
				end if;
				
				acc2 <= acc2 + acc1;
				acc3 <= acc3 + acc2;
				
				if wload = '1' then
					diff1   <= acc3 - acc3_d2;
					diff2   <= diff1 - diff1_d;
					diff3   <= diff2 - diff2_d;
					acc3_d2 <= acc3;
					diff1_d <= diff1;
					diff2_d <= diff2;
				end if;
			
			end if;
	
		end if;

	end process;

	wdata <= diff3(C_MATH_WIDTH-1 downto C_MATH_WIDTH-C_RESULT_WIDTH);

end Behavioral;
