-------------------------------------------------------------------------------------------
-- Copyright © 2011-2018, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    09:51:35 04/20/2011 
-- Design Name: 
-- Module Name:    axi_motorfoc - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity axi_motorfoc is
	generic
	(
		-- FPGA Family Parameter      
		C_FAMILY             : string := "zynq";

		-- AXI bus block speed
		C_S_AXI_ACLK_FREQ_HZ : integer := 100_000_000;

		-- Parameters of Axi Slave Bus Interface S_AXI
		C_S_AXI_DATA_WIDTH : integer range 32 to 32 := 32;
		C_S_AXI_ADDR_WIDTH : integer range 16 to 16 := 16;

		-- IP generics
		C_SYSTEMCLOCK     : integer := 100_000_000;
		C_HW_LINK_REF     : integer := 0;
		--
		C_INV_IN_MAP      : std_logic_vector(11 downto 0) := x"083";
		C_INV_IN_NOT      : std_logic_vector(11 downto 0) := x"000";
		C_INV_OFSV_MODE   : integer := 1;
		C_INV_OVER_IPHS   : integer := 1;
		C_INV_OVER_IBUS   : integer := 1;
		C_INV_FILTER      : integer := 1;
		C_INV_EVAL_VPHS   : integer := 1;
		C_CLARKE_NPHS     : integer := 3;
		C_PWM_MODULATOR   : integer := 1;
		C_RPFM_MODULATOR  : integer := 1;
		C_RPFM_3_LEVEL    : integer := 1;
		C_RPFM_TPNC       : integer := 1;
		C_SMO_EVAL        : integer := 1;
		C_SPD_EVAL        : integer := 1;
		C_SPEED_CTRL      : integer := 1;
		C_RESOLVER        : integer := 1;
		C_RSV_MINANV      : integer := 8192;
		C_HALLSENSOR      : integer := 1;
		C_ENCODER         : integer := 1;
		C_MANROT          : integer := 2;
		C_MRT_ACCMAX_DLN2 : integer := 16;
		C_SMO_ZS_DLN2     : integer := 24;
		C_SMO_F2_DLN2     : integer := 28;
		C_SLP_PRO_DLN2    : integer := 1;
		C_SLP_INT_DLN2    : integer := 5;
		C_SLP_INDWP_DLN2  : integer := 1;
		C_SLP_INDWP_KDIV  : integer := 1;
		C_ISOVERFLOW_CMAX : integer := 1;
		C_PI_ERPRO_DLN2   : integer := 12;
		C_PI_ERINT_DLN2   : integer := 18
	);
	port
	(
		-- Ports of Axi Slave Bus Interface S_AXI
		S_AXI_ACLK      : in  std_logic;
		S_AXI_ARESETN   : in  std_logic;
		S_AXI_AWADDR    : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 DOWNTO 0);
		S_AXI_AWVALID   : in  std_logic;
		S_AXI_AWREADY   : out std_logic;
		S_AXI_WDATA     : in  std_logic_vector(C_S_AXI_DATA_WIDTH-1 DOWNTO 0);
		S_AXI_WSTRB     : in  std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 DOWNTO 0);
		S_AXI_WVALID    : in  std_logic;
		S_AXI_WREADY    : out std_logic;
		S_AXI_BRESP     : out std_logic_vector(1 DOWNTO 0);
		S_AXI_BVALID    : out std_logic;
		S_AXI_BREADY    : in  std_logic;
		S_AXI_ARADDR    : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 DOWNTO 0);
		S_AXI_ARVALID   : in  std_logic;
		S_AXI_ARREADY   : out std_logic;
		S_AXI_RDATA     : out std_logic_vector(C_S_AXI_DATA_WIDTH-1 DOWNTO 0);
		S_AXI_RRESP     : out std_logic_vector(1 DOWNTO 0);
		S_AXI_RVALID    : out std_logic;
		S_AXI_RREADY    : in  std_logic;
		
		-- ip ports

		-- external angle
		ext_angle     : in  std_logic_vector(31 downto 0);
		
		-- resolver
		rsv_priexc    : in  std_logic_vector(17 downto 0);
		rsv_secsin    : in  std_logic_vector(17 downto 0);
		rsv_seccos    : in  std_logic_vector(17 downto 0);
		
		-- hall sensor
		hls_hallsig   : in  std_logic_vector(5 downto 0);
		
		-- digital encoder
		enc_cha       : in  std_logic;
		enc_chb       : in  std_logic;
		enc_chi       : in  std_logic;
		
		-- input data
		--
		acq_iphs_a    : in std_logic_vector(17 downto 0);
		acq_iphs_b    : in std_logic_vector(17 downto 0);
		acq_iphs_c    : in std_logic_vector(17 downto 0);
		acq_ibus_x    : in std_logic_vector(17 downto 0);
		acq_vphs_a    : in std_logic_vector(17 downto 0);
		acq_vphs_b    : in std_logic_vector(17 downto 0);
		acq_vphs_c    : in std_logic_vector(17 downto 0);
		acq_vbus_x    : in std_logic_vector(17 downto 0);
		acq_vphs_n    : in std_logic_vector(17 downto 0);
		--
		acq_sync      : in std_logic;
		
		-- output offset
		--
		ofs_iphs_a    : out std_logic_vector(17 downto 0);
		ofs_iphs_b    : out std_logic_vector(17 downto 0);
		ofs_iphs_c    : out std_logic_vector(17 downto 0);
		ofs_ibus_x    : out std_logic_vector(17 downto 0);
		ofs_vphs_a    : out std_logic_vector(17 downto 0);
		ofs_vphs_b    : out std_logic_vector(17 downto 0);
		ofs_vphs_c    : out std_logic_vector(17 downto 0);
		ofs_vbus_x    : out std_logic_vector(17 downto 0);
		ofs_vphs_n    : out std_logic_vector(17 downto 0);

		-- normalized values (offset removed)
		--
		nrm_iphs_a    : out std_logic_vector(17 downto 0);
		nrm_iphs_b    : out std_logic_vector(17 downto 0);
		nrm_iphs_c    : out std_logic_vector(17 downto 0);
		nrm_ibus_x    : out std_logic_vector(17 downto 0);
		nrm_vphs_a    : out std_logic_vector(17 downto 0);
		nrm_vphs_b    : out std_logic_vector(17 downto 0);
		nrm_vphs_c    : out std_logic_vector(17 downto 0);
		nrm_vbus_x    : out std_logic_vector(17 downto 0);
		nrm_vphs_n    : out std_logic_vector(17 downto 0);
		--
		nrm_sync      : out std_logic;

		-- filtered values
		--
		flt_iphs_a    : out std_logic_vector(17 downto 0);
		flt_iphs_b    : out std_logic_vector(17 downto 0);
		flt_iphs_c    : out std_logic_vector(17 downto 0);
		flt_ibus_x    : out std_logic_vector(17 downto 0);
		flt_vphs_a    : out std_logic_vector(17 downto 0);
		flt_vphs_b    : out std_logic_vector(17 downto 0);
		flt_vphs_c    : out std_logic_vector(17 downto 0);
		flt_vbus_x    : out std_logic_vector(17 downto 0);
		flt_vphs_n    : out std_logic_vector(17 downto 0);
		--
		flt_sync      : out std_logic;

		-- current overflow
		--
		ovi_iphs_a    : out std_logic;
		ovi_iphs_b    : out std_logic;
		ovi_iphs_c    : out std_logic;
		ovi_ibus_x    : out std_logic;

		-- Is overflow
		ismodover     : out std_logic;
		
		-- 1=generator, 0=motor
		generator     : out std_logic;
		
		-- modulator type
		modtype       : out std_logic_vector(0 downto 0);
		
		-- modulator levels
		modlevels     : out std_logic_vector(0 downto 0);
		
		-- motor is 3-phases (0=bipolar)
		is3phases     : out std_logic;
		
		-- pwm 3-phases outputs
		pwm2p_l2c1w1  : out std_logic_vector(1 downto 0);
		pwm2p_l2c2w1  : out std_logic_vector(1 downto 0);
		pwm2p_l2c1w2  : out std_logic_vector(1 downto 0);
		pwm2p_l2c2w2  : out std_logic_vector(1 downto 0);
		pwm2p_l2sync  : out std_logic;
		
		-- pwm 2-phases outputs
		pwm3p_l2cxw1  : out std_logic_vector(2 downto 0);
		pwm3p_l2cxw2  : out std_logic_vector(2 downto 0);
		pwm3p_l2sync  : out std_logic;
		
		-- rpfm 3-ph two levels outputs
		rpfm3p_l2cxw1 : out std_logic_vector(2 downto 0);
		rpfm3p_l2cxw2 : out std_logic_vector(2 downto 0);
		rpfm3p_l2sync : out std_logic;
		
		-- rpfm 3-ph three levels outputs
		rpfm3p_l3c1w1 : out std_logic_vector(1 downto 0);
		rpfm3p_l3c2w1 : out std_logic_vector(1 downto 0);
		rpfm3p_l3c3w1 : out std_logic_vector(1 downto 0);
		rpfm3p_l3c1w2 : out std_logic_vector(1 downto 0);
		rpfm3p_l3c2w2 : out std_logic_vector(1 downto 0);
		rpfm3p_l3c3w2 : out std_logic_vector(1 downto 0);
		rpfm3p_l3sync : out std_logic;
		
		-- dead time value
		deadtval      : out std_logic_vector(7 downto 0);
		
		-- gate unit switches
		guswcnt       : in std_logic_vector(31 downto 0);
		
		-- in-failure
		hwfail        : in std_logic;
		
		-- in-kill
		hwkill        : in std_logic;
		
		-- coil driver
		coilgear      : out std_logic;
		coilenab      : out std_logic;
		
		-- system error
		syserr        : out std_logic;
		
		-- reset pending errors
		reseterr      : out std_logic;
		
		-- zero current offset 
		zerocurr      : out std_logic;
		
		-- zero vbus offset 
		zerovbusx     : out std_logic;

		-- zero vphs offset 
		zerovphsx     : out std_logic;
		
		-- control probes
		pix_setval    : out std_logic_vector(17 downto 0);
		piy_setval    : out std_logic_vector(17 downto 0);
		
		pix_fbkval    : out std_logic_vector(17 downto 0);
		piy_fbkval    : out std_logic_vector(17 downto 0);
		
		pix_outval    : out std_logic_vector(17 downto 0);
		piy_outval    : out std_logic_vector(17 downto 0);
		
		-- Vs
		vsmodval      : out std_logic_vector(16 downto 0);
		vsangval      : out std_logic_vector(31 downto 0);
		
		-- Is
		ismodval      : out std_logic_vector(16 downto 0);
		isangval      : out std_logic_vector(31 downto 0);

		-- smo
		smo_bemf_p    : out std_logic_vector(31 downto 0);
		smo_bemf_m    : out std_logic_vector(16 downto 0);
		
		-- e-angles
		mrt_angle     : out std_logic_vector(31 downto 0);
		enc_angle     : out std_logic_vector(31 downto 0);
		hls_angle     : out std_logic_vector(31 downto 0);
		rsv_angle     : out std_logic_vector(31 downto 0);
		smo_angle     : out std_logic_vector(31 downto 0);
		
		-- Rotor angle
		rot_angle     : out std_logic_vector(31 downto 0);
		
		-- rot selector
		rot_select    : out std_logic_vector( 2 downto 0);
		
		-- actual speed
		spd_speed     : out std_logic_vector(31 downto 0);
		
		-- position
		pos_position  : out std_logic_vector(31 downto 0);
		
		-- foc cycle time
		tsclocks      : out std_logic_vector(11 downto 0)
	);
end entity axi_motorfoc;

architecture IMP of axi_motorfoc is

	component axi4ltiny
		generic
		(
			C_S_AXI_DATA_WIDTH : integer range 32 to 32 := 32;
			C_S_AXI_ADDR_WIDTH : integer range  1 to 32 := 16
		);
		port
		(
			-- AXI4LITE
			S_AXI_ACLK    : in  std_logic;
			S_AXI_ARESETN : in  std_logic;
			S_AXI_AWADDR  : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
			S_AXI_AWVALID : in  std_logic;
			S_AXI_AWREADY : out std_logic;
			S_AXI_WDATA   : in  std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
			S_AXI_WSTRB   : in  std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 downto 0);
			S_AXI_WVALID  : in  std_logic;
			S_AXI_WREADY  : out std_logic;
			S_AXI_BRESP   : out std_logic_vector(1 downto 0);
			S_AXI_BVALID  : out std_logic;
			S_AXI_BREADY  : in  std_logic;
			S_AXI_ARADDR  : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
			S_AXI_ARVALID : in  std_logic;
			S_AXI_ARREADY : out std_logic;
			S_AXI_RDATA   : out std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
			S_AXI_RRESP   : out std_logic_vector(1 downto 0);
			S_AXI_RVALID  : out std_logic;
			S_AXI_RREADY  : in  std_logic;
			-- local bus  
			host_clk      : out std_logic;
			host_rst      : out std_logic;
			host_en       : out std_logic;
			host_we       : out std_logic;
			host_addr     : out std_logic_vector(31 downto 0);
			host_din      : out std_logic_vector(31 downto 0);
			host_dout     : in  std_logic_vector(31 downto 0);
			host_ack      : in  std_logic
		);
	end component;
	
	component mosysfoc
		generic
		(
			C_FAMILY          : string := "zynq";
			--
			C_INV_IN_MAP      : std_logic_vector(11 downto 0) := x"083";
			C_INV_IN_NOT      : std_logic_vector(11 downto 0) := x"000";
			C_INV_OFSV_MODE   : integer := 1;
			C_INV_OVER_IPHS   : integer := 1;
			C_INV_OVER_IBUS   : integer := 1;
			C_INV_FILTER      : integer := 1;
			C_INV_EVAL_VPHS   : integer := 1;
			C_CLARKE_NPHS     : integer := 3;
			C_PWM_MODULATOR   : integer := 1;
			C_RPFM_MODULATOR  : integer := 1;
			C_RPFM_3_LEVEL    : integer := 1;
			C_RPFM_TPNC       : integer := 1;
			C_SMO_EVAL        : integer := 1;
			C_SPD_EVAL        : integer := 1;
			C_SPEED_CTRL      : integer := 1;
			C_RESOLVER        : integer := 1;
			C_RSV_MINANV      : integer := 8192;
			C_HALLSENSOR      : integer := 1;
			C_ENCODER         : integer := 1;
			C_MANROT          : integer := 2;
			C_MRT_ACCMAX_DLN2 : integer := 16;
			C_SMO_ZS_DLN2     : integer := 24;
			C_SMO_F2_DLN2     : integer := 28;
			C_SLP_PRO_DLN2    : integer := 1;
			C_SLP_INT_DLN2    : integer := 5;
			C_SLP_INDWP_DLN2  : integer := 1;
			C_SLP_INDWP_KDIV  : integer := 1;
			C_ISOVERFLOW_CMAX : integer := 1;
			C_PI_ERPRO_DLN2   : integer := 12;
			C_PI_ERINT_DLN2   : integer := 18
		);
		port 
		( 
			clock         : in  std_logic;
			reset         : in  std_logic;
			
			-- host interface
			host_en       : in  std_logic;
			host_we       : in  std_logic;
			host_addr     : in  std_logic_vector(31 downto 0);
			host_din      : in  std_logic_vector(31 downto 0);
			host_dout     : out std_logic_vector(31 downto 0);
			host_ack      : out std_logic;

			-- external angle
			ext_angle     : in  std_logic_vector(31 downto 0);
			
			-- resolver
			rsv_priexc    : in  std_logic_vector(17 downto 0);
			rsv_secsin    : in  std_logic_vector(17 downto 0);
			rsv_seccos    : in  std_logic_vector(17 downto 0);
			
			-- hall sensor
			hls_hallsig   : in  std_logic_vector(5 downto 0);
			
			-- digital encoder
			enc_cha       : in  std_logic;
			enc_chb       : in  std_logic;
			enc_chi       : in  std_logic;
			
			-- input data
			--
			acq_iphs_a    : in std_logic_vector(17 downto 0);
			acq_iphs_b    : in std_logic_vector(17 downto 0);
			acq_iphs_c    : in std_logic_vector(17 downto 0);
			acq_ibus_x    : in std_logic_vector(17 downto 0);
			acq_vphs_a    : in std_logic_vector(17 downto 0);
			acq_vphs_b    : in std_logic_vector(17 downto 0);
			acq_vphs_c    : in std_logic_vector(17 downto 0);
			acq_vbus_x    : in std_logic_vector(17 downto 0);
			acq_vphs_n    : in std_logic_vector(17 downto 0);
			--
			acq_sync      : in std_logic;
			
			-- output offset
			--
			ofs_iphs_a    : out std_logic_vector(17 downto 0);
			ofs_iphs_b    : out std_logic_vector(17 downto 0);
			ofs_iphs_c    : out std_logic_vector(17 downto 0);
			ofs_ibus_x    : out std_logic_vector(17 downto 0);
			ofs_vphs_a    : out std_logic_vector(17 downto 0);
			ofs_vphs_b    : out std_logic_vector(17 downto 0);
			ofs_vphs_c    : out std_logic_vector(17 downto 0);
			ofs_vbus_x    : out std_logic_vector(17 downto 0);
			ofs_vphs_n    : out std_logic_vector(17 downto 0);

			-- normalized values (offset removed)
			--
			nrm_iphs_a    : out std_logic_vector(17 downto 0);
			nrm_iphs_b    : out std_logic_vector(17 downto 0);
			nrm_iphs_c    : out std_logic_vector(17 downto 0);
			nrm_ibus_x    : out std_logic_vector(17 downto 0);
			nrm_vphs_a    : out std_logic_vector(17 downto 0);
			nrm_vphs_b    : out std_logic_vector(17 downto 0);
			nrm_vphs_c    : out std_logic_vector(17 downto 0);
			nrm_vbus_x    : out std_logic_vector(17 downto 0);
			nrm_vphs_n    : out std_logic_vector(17 downto 0);
			--
			nrm_sync      : out std_logic;

			-- filtered values
			--
			flt_iphs_a    : out std_logic_vector(17 downto 0);
			flt_iphs_b    : out std_logic_vector(17 downto 0);
			flt_iphs_c    : out std_logic_vector(17 downto 0);
			flt_ibus_x    : out std_logic_vector(17 downto 0);
			flt_vphs_a    : out std_logic_vector(17 downto 0);
			flt_vphs_b    : out std_logic_vector(17 downto 0);
			flt_vphs_c    : out std_logic_vector(17 downto 0);
			flt_vbus_x    : out std_logic_vector(17 downto 0);
			flt_vphs_n    : out std_logic_vector(17 downto 0);
			--
			flt_sync      : out std_logic;

			-- current overflow
			--
			ovi_iphs_a    : out std_logic;
			ovi_iphs_b    : out std_logic;
			ovi_iphs_c    : out std_logic;
			ovi_ibus_x    : out std_logic;

			-- Is overflow
			ismodover     : out std_logic;
			
			-- 1=generator, 0=motor
			generator     : out std_logic;
			
			-- modulator type
			modtype       : out std_logic_vector(0 downto 0);
			
			-- modulator levels
			modlevels     : out std_logic_vector(0 downto 0);
			
			-- motor is 3-phases (0=bipolar)
			is3phases     : out std_logic;
			
			-- pwm 3-phases outputs
			pwm2p_l2c1w1  : out std_logic_vector(1 downto 0);
			pwm2p_l2c2w1  : out std_logic_vector(1 downto 0);
			pwm2p_l2c1w2  : out std_logic_vector(1 downto 0);
			pwm2p_l2c2w2  : out std_logic_vector(1 downto 0);
			pwm2p_l2sync  : out std_logic;
			
			-- pwm 2-phases outputs
			pwm3p_l2cxw1  : out std_logic_vector(2 downto 0);
			pwm3p_l2cxw2  : out std_logic_vector(2 downto 0);
			pwm3p_l2sync  : out std_logic;
			
			-- rpfm 3-ph two levels outputs
			rpfm3p_l2cxw1 : out std_logic_vector(2 downto 0);
			rpfm3p_l2cxw2 : out std_logic_vector(2 downto 0);
			rpfm3p_l2sync : out std_logic;
			
			-- rpfm 3-ph three levels outputs
			rpfm3p_l3c1w1 : out std_logic_vector(1 downto 0);
			rpfm3p_l3c2w1 : out std_logic_vector(1 downto 0);
			rpfm3p_l3c3w1 : out std_logic_vector(1 downto 0);
			rpfm3p_l3c1w2 : out std_logic_vector(1 downto 0);
			rpfm3p_l3c2w2 : out std_logic_vector(1 downto 0);
			rpfm3p_l3c3w2 : out std_logic_vector(1 downto 0);
			rpfm3p_l3sync : out std_logic;
			
			-- dead time value
			deadtval      : out std_logic_vector(7 downto 0);
			
			-- gate unit switches
			guswcnt       : in std_logic_vector(31 downto 0);
			
			-- in-failure
			hwfail        : in std_logic;
			
			-- in-kill
			hwkill        : in std_logic;
			
			-- coil driver
			coilgear      : out std_logic;
			coilenab      : out std_logic;
			
			-- system error
			syserr        : out std_logic;
			
			-- reset pending errors
			reseterr      : out std_logic;
			
			-- zero current offset 
			zerocurr      : out std_logic;
			
			-- zero vbus offset 
			zerovbusx     : out std_logic;

			-- zero vphs offset 
			zerovphsx     : out std_logic;
			
			-- control probes
			pix_setval    : out std_logic_vector(17 downto 0);
			piy_setval    : out std_logic_vector(17 downto 0);
			
			pix_fbkval    : out std_logic_vector(17 downto 0);
			piy_fbkval    : out std_logic_vector(17 downto 0);
			
			pix_outval    : out std_logic_vector(17 downto 0);
			piy_outval    : out std_logic_vector(17 downto 0);
			
			-- Vs
			vsmodval      : out std_logic_vector(16 downto 0);
			vsangval      : out std_logic_vector(31 downto 0);
			
			-- Is
			ismodval      : out std_logic_vector(16 downto 0);
			isangval      : out std_logic_vector(31 downto 0);

			-- smo
			smo_bemf_p    : out std_logic_vector(31 downto 0);
			smo_bemf_m    : out std_logic_vector(16 downto 0);
			
			-- e-angles
			mrt_angle     : out std_logic_vector(31 downto 0);
			enc_angle     : out std_logic_vector(31 downto 0);
			hls_angle     : out std_logic_vector(31 downto 0);
			rsv_angle     : out std_logic_vector(31 downto 0);
			smo_angle     : out std_logic_vector(31 downto 0);
			
			-- Rotor angle
			rot_angle     : out std_logic_vector(31 downto 0);
			
			-- rot selector
			rot_select    : out std_logic_vector( 2 downto 0);
			
			-- actual speed
			spd_speed     : out std_logic_vector(31 downto 0);
			
			-- position
			pos_position  : out std_logic_vector(31 downto 0);
			
			-- foc cycle time
			tsclocks      : out std_logic_vector(11 downto 0)
		);
	end component;

	-- wrapper

	signal host_clk        : std_logic;
	signal host_rst        : std_logic;
	--
	signal host_en         : std_logic;
	signal host_we         : std_logic;
	signal host_addr       : std_logic_vector(31 downto 0);
	signal host_din        : std_logic_vector(31 downto 0);
	signal host_dout       : std_logic_vector(31 downto 0);
	signal host_ack        : std_logic;

begin

	assert C_S_AXI_ACLK_FREQ_HZ = C_SYSTEMCLOCK report "AXI clock /= system clock" severity failure;
	assert false report "C_SYSTEMCLOCK=" & integer'image(C_SYSTEMCLOCK) severity note;
	
	AXI_LITE_IPIF_I : axi4ltiny
		generic map
		(
			C_S_AXI_DATA_WIDTH => C_S_AXI_DATA_WIDTH,
			C_S_AXI_ADDR_WIDTH => C_S_AXI_ADDR_WIDTH
		)
		port map
		(
			-- AXI4LITE
			S_AXI_ACLK    => S_AXI_ACLK,
			S_AXI_ARESETN => S_AXI_ARESETN,
			S_AXI_AWADDR  => S_AXI_AWADDR,
			S_AXI_AWVALID => S_AXI_AWVALID,
			S_AXI_AWREADY => S_AXI_AWREADY,
			S_AXI_WDATA   => S_AXI_WDATA,
			S_AXI_WSTRB   => S_AXI_WSTRB,
			S_AXI_WVALID  => S_AXI_WVALID,
			S_AXI_WREADY  => S_AXI_WREADY,
			S_AXI_BRESP   => S_AXI_BRESP,
			S_AXI_BVALID  => S_AXI_BVALID,
			S_AXI_BREADY  => S_AXI_BREADY,
			S_AXI_ARADDR  => S_AXI_ARADDR,
			S_AXI_ARVALID => S_AXI_ARVALID,
			S_AXI_ARREADY => S_AXI_ARREADY,
			S_AXI_RDATA   => S_AXI_RDATA,
			S_AXI_RRESP   => S_AXI_RRESP,
			S_AXI_RVALID  => S_AXI_RVALID,
			S_AXI_RREADY  => S_AXI_RREADY,
			-- local bus  
			host_clk      => host_clk,
			host_rst      => host_rst,
			host_en       => host_en,
			host_we       => host_we,
			host_addr     => host_addr,
			host_din      => host_din,
			host_dout     => host_dout,
			host_ack      => host_ack
		);

  mosysfoc_inst : mosysfoc
		generic map
		(
			C_FAMILY          => C_FAMILY,
			--
			C_INV_IN_MAP      => C_INV_IN_MAP,
			C_INV_IN_NOT      => C_INV_IN_NOT,
			C_INV_OFSV_MODE   => C_INV_OFSV_MODE,
			C_INV_OVER_IPHS   => C_INV_OVER_IPHS,
			C_INV_OVER_IBUS   => C_INV_OVER_IBUS,
			C_INV_FILTER      => C_INV_FILTER,
			C_INV_EVAL_VPHS   => C_INV_EVAL_VPHS,
			C_CLARKE_NPHS     => C_CLARKE_NPHS,
			C_PWM_MODULATOR   => C_PWM_MODULATOR,
			C_RPFM_MODULATOR  => C_RPFM_MODULATOR,
			C_RPFM_3_LEVEL    => C_RPFM_3_LEVEL,
			C_RPFM_TPNC       => C_RPFM_TPNC,
			C_SMO_EVAL        => C_SMO_EVAL,
			C_SPD_EVAL        => C_SPD_EVAL,
			C_SPEED_CTRL      => C_SPEED_CTRL,
			C_RESOLVER        => C_RESOLVER,
			C_RSV_MINANV      => C_RSV_MINANV,
			C_HALLSENSOR      => C_HALLSENSOR,
			C_ENCODER         => C_ENCODER,
			C_MANROT          => C_MANROT,
			C_MRT_ACCMAX_DLN2 => C_MRT_ACCMAX_DLN2,
			C_SMO_ZS_DLN2     => C_SMO_ZS_DLN2,
			C_SMO_F2_DLN2     => C_SMO_F2_DLN2,
			C_SLP_PRO_DLN2    => C_SLP_PRO_DLN2,
			C_SLP_INT_DLN2    => C_SLP_INT_DLN2,
			C_SLP_INDWP_DLN2  => C_SLP_INDWP_DLN2,
			C_SLP_INDWP_KDIV  => C_SLP_INDWP_KDIV,
			C_ISOVERFLOW_CMAX => C_ISOVERFLOW_CMAX,
			C_PI_ERPRO_DLN2   => C_PI_ERPRO_DLN2,
			C_PI_ERINT_DLN2   => C_PI_ERINT_DLN2
		)
		port map
		( 
			clock         => host_clk,
			reset         => host_rst,
			--
			-- host interface
			host_en       => host_en  ,
			host_we       => host_we  ,
			host_addr     => host_addr,
			host_din      => host_din ,
			host_dout     => host_dout,
			host_ack      => host_ack ,

			-- external angle
			ext_angle     => ext_angle,
			
			-- resolver
			rsv_priexc    => rsv_priexc,
			rsv_secsin    => rsv_secsin,
			rsv_seccos    => rsv_seccos,
			
			-- hall sensor
			hls_hallsig   => hls_hallsig,
			
			-- digital encoder
			enc_cha       => enc_cha,
			enc_chb       => enc_chb,
			enc_chi       => enc_chi,
			
			-- input data
			--
			acq_iphs_a    => acq_iphs_a,
			acq_iphs_b    => acq_iphs_b,
			acq_iphs_c    => acq_iphs_c,
			acq_ibus_x    => acq_ibus_x,
			acq_vphs_a    => acq_vphs_a,
			acq_vphs_b    => acq_vphs_b,
			acq_vphs_c    => acq_vphs_c,
			acq_vbus_x    => acq_vbus_x,
			acq_vphs_n    => acq_vphs_n,
			--
			acq_sync      => acq_sync,
			
			-- output offset
			--
			ofs_iphs_a    => ofs_iphs_a,
			ofs_iphs_b    => ofs_iphs_b,
			ofs_iphs_c    => ofs_iphs_c,
			ofs_ibus_x    => ofs_ibus_x,
			ofs_vphs_a    => ofs_vphs_a,
			ofs_vphs_b    => ofs_vphs_b,
			ofs_vphs_c    => ofs_vphs_c,
			ofs_vbus_x    => ofs_vbus_x,
			ofs_vphs_n    => ofs_vphs_n,

			-- normalized values (offset removed)
			--
			nrm_iphs_a    => nrm_iphs_a,
			nrm_iphs_b    => nrm_iphs_b,
			nrm_iphs_c    => nrm_iphs_c,
			nrm_ibus_x    => nrm_ibus_x,
			nrm_vphs_a    => nrm_vphs_a,
			nrm_vphs_b    => nrm_vphs_b,
			nrm_vphs_c    => nrm_vphs_c,
			nrm_vbus_x    => nrm_vbus_x,
			nrm_vphs_n    => nrm_vphs_n,
			--
			nrm_sync      => nrm_sync,

			-- filtered values
			--
			flt_iphs_a    => flt_iphs_a,
			flt_iphs_b    => flt_iphs_b,
			flt_iphs_c    => flt_iphs_c,
			flt_ibus_x    => flt_ibus_x,
			flt_vphs_a    => flt_vphs_a,
			flt_vphs_b    => flt_vphs_b,
			flt_vphs_c    => flt_vphs_c,
			flt_vbus_x    => flt_vbus_x,
			flt_vphs_n    => flt_vphs_n,
			--
			flt_sync      => flt_sync,

			-- current overflow
			--
			ovi_iphs_a    => ovi_iphs_a,
			ovi_iphs_b    => ovi_iphs_b,
			ovi_iphs_c    => ovi_iphs_c,
			ovi_ibus_x    => ovi_ibus_x,

			-- Is overflow
			ismodover     => ismodover,
			
			-- 1=generator, 0=motor
			generator     => generator,
			
			-- modulator type
			modtype       => modtype,
			
			-- modulator levels
			modlevels     => modlevels,
			
			-- motor is 3-phases (0=bipolar)
			is3phases     => is3phases,
			
			-- pwm 3-phases outputs
			pwm2p_l2c1w1  => pwm2p_l2c1w1,
			pwm2p_l2c2w1  => pwm2p_l2c2w1,
			pwm2p_l2c1w2  => pwm2p_l2c1w2,
			pwm2p_l2c2w2  => pwm2p_l2c2w2,
			pwm2p_l2sync  => pwm2p_l2sync,
			
			-- pwm 2-phases outputs
			pwm3p_l2cxw1  => pwm3p_l2cxw1,
			pwm3p_l2cxw2  => pwm3p_l2cxw2,
			pwm3p_l2sync  => pwm3p_l2sync,
			
			-- rpfm 3-ph two levels outputs
			rpfm3p_l2cxw1 => rpfm3p_l2cxw1,
			rpfm3p_l2cxw2 => rpfm3p_l2cxw2,
			rpfm3p_l2sync => rpfm3p_l2sync,
			
			-- rpfm 3-ph three levels outputs
			rpfm3p_l3c1w1 => rpfm3p_l3c1w1,
			rpfm3p_l3c2w1 => rpfm3p_l3c2w1,
			rpfm3p_l3c3w1 => rpfm3p_l3c3w1,
			rpfm3p_l3c1w2 => rpfm3p_l3c1w2,
			rpfm3p_l3c2w2 => rpfm3p_l3c2w2,
			rpfm3p_l3c3w2 => rpfm3p_l3c3w2,
			rpfm3p_l3sync => rpfm3p_l3sync,
			
			-- dead time value
			deadtval      => deadtval,
			
			-- gate unit switches
			guswcnt       => guswcnt,
			
			-- in-failure
			hwfail        => hwfail,
			
			-- in-kill
			hwkill        => hwkill,
			
			-- coil driver
			coilgear      => coilgear,
			coilenab      => coilenab,
			
			-- system error
			syserr        => syserr,
			
			-- reset pending errors
			reseterr      => reseterr,
			
			-- zero current offset 
			zerocurr      => zerocurr,
			
			-- zero vbus offset 
			zerovbusx     => zerovbusx,

			-- zero vphs offset 
			zerovphsx     => zerovphsx,
			
			-- control probes
			pix_setval    => pix_setval,
			piy_setval    => piy_setval,
			
			pix_fbkval    => pix_fbkval,
			piy_fbkval    => piy_fbkval,
			
			pix_outval    => pix_outval,
			piy_outval    => piy_outval,
			
			-- Vs
			vsmodval      => vsmodval,
			vsangval      => vsangval,
			
			-- Is
			ismodval      => ismodval,
			isangval      => isangval,

			-- smo
			smo_bemf_p    => smo_bemf_p,
			smo_bemf_m    => smo_bemf_m,

			-- e-angles
			mrt_angle     => mrt_angle,
			enc_angle     => enc_angle,
			hls_angle     => hls_angle,
			rsv_angle     => rsv_angle,
			smo_angle     => smo_angle,

			-- Rotor angle
			rot_angle     => rot_angle,
			
			-- rot selector
			rot_select    => rot_select,
			
			-- actual speed
			spd_speed     => spd_speed,
			
			-- position
			pos_position  => pos_position,
			
			-- foc cycle time
			tsclocks      => tsclocks
		);

end IMP;
