-------------------------------------------------------------------------------------------
-- Copyright  2011, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    08:32:35 03/27/2011 
-- Design Name: 
-- Module Name:    acqsync - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity acqsync is
	generic
	(
		C_SYSCLOCK_PERIOD_PS : integer := 20000;
		C_ACQUI_PERIOD_PS    : integer := 3200000
	);
	port 
	(
		clock   : in  std_logic;
		reset   : in  std_logic;
		datsync : in  std_logic;
		engsync : out std_logic
	);
end acqsync;

architecture Behavioral of acqsync is

	-- sample timing
	constant SCNTMAX  : integer := C_ACQUI_PERIOD_PS / C_SYSCLOCK_PERIOD_PS;
	
	-- sampling window
	constant SCNTLOW  : integer := SCNTMAX - 3;
	constant SCNTHIG  : integer := SCNTMAX + 2;
	
	-- sampling counter
	signal scntval      : natural range 0 to SCNTHIG;

	signal wdsena    : std_logic; -- window sampling enable
	signal holdsync  : std_logic; -- saved sync
	signal engsync_i : std_logic; -- sync output

begin

	engsync <= engsync_i;

	main_proc : process(clock)
		variable savesync : std_logic;
		variable execsync : std_logic;
	begin
		if rising_edge(clock) then
			if reset = '1' then
				scntval   <= 0;
				wdsena    <= '0';
				holdsync  <= '0';
				engsync_i <= '0';
			else
			
				savesync := '0';
				execsync := '0';
				
				if datsync = '1' or holdsync = '1' then
					if wdsena = '1' then
						execsync := '1';
					else
						savesync := '1';
					end if;
				end if;
				
				if scntval = SCNTHIG or execsync = '1' then
					scntval <= 0;
					wdsena  <= '0';
				elsif scntval = SCNTLOW then
					scntval <= scntval + 1;
					wdsena  <= '1';
				else
					scntval <= scntval + 1;
				end if;
				
				engsync_i <= execsync;
				holdsync  <= savesync;
				
			end if;
		end if;
	end process;

end Behavioral;
