-------------------------------------------------------------------------------------------
-- Copyright © 2011-2017, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    11:52:27 10/31/2011 
-- Design Name: 
-- Module Name:    axi_netmot_drv - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity axi_netmot_drv is
	generic
	(
		-- FPGA Family Parameter      
		C_FAMILY             : string := "zynq";

		-- AXI bus block speed
		C_S_AXI_ACLK_FREQ_HZ : integer := 100_000_000;

		-- Parameters of Axi Slave Bus Interface S_AXI
		C_S_AXI_DATA_WIDTH	: integer range 32 to 32 := 32;
		C_S_AXI_ADDR_WIDTH	: integer range 16 to 16 := 16;

		-- IP generics
		C_SYSTEMCLOCK       : integer  := 100_000_000;
		C_CARRIER           : integer  := 1;
		C_HW_LINK_REF_0     : integer  := 0;
		C_HW_LINK_REF_1     : integer  := 0;
		C_RXLINK_SPACE      : integer  := 0
	);
	port
	(
		-- Ports of Axi Slave Bus Interface S_AXI
		S_AXI_ACLK      : in  std_logic;
		S_AXI_ARESETN   : in  std_logic;
		S_AXI_AWADDR    : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 DOWNTO 0);
		S_AXI_AWVALID   : in  std_logic;
		S_AXI_AWREADY   : out std_logic;
		S_AXI_WDATA     : in  std_logic_vector(C_S_AXI_DATA_WIDTH-1 DOWNTO 0);
		S_AXI_WSTRB     : in  std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 DOWNTO 0);
		S_AXI_WVALID    : in  std_logic;
		S_AXI_WREADY    : out std_logic;
		S_AXI_BRESP     : out std_logic_vector(1 DOWNTO 0);
		S_AXI_BVALID    : out std_logic;
		S_AXI_BREADY    : in  std_logic;
		S_AXI_ARADDR    : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 DOWNTO 0);
		S_AXI_ARVALID   : in  std_logic;
		S_AXI_ARREADY   : out std_logic;
		S_AXI_RDATA     : out std_logic_vector(C_S_AXI_DATA_WIDTH-1 DOWNTO 0);
		S_AXI_RRESP     : out std_logic_vector(1 DOWNTO 0);
		S_AXI_RVALID    : out std_logic;
		S_AXI_RREADY    : in  std_logic;
		-- ip ports
		-- coil drivers
		Motor_Coil      : out std_logic_vector (7 downto 0);
		-- serial link
		Cmd_Clock       : out std_logic;
		Cmd_Data        : out std_logic;
		Sts_Clock       : in  std_logic;
		Sts_Data        : in  std_logic;
		--
		-- external angle in
		ext_x_angle_0   : in  std_logic_vector(31 downto 0) := (others => '0');
		ext_x_angle_1   : in  std_logic_vector(31 downto 0) := (others => '0');
		--
		-- resolver in
		rsv_x_priexc_0  : in  std_logic_vector(17 downto 0) := (others => '0');
		rsv_x_secsin_0  : in  std_logic_vector(17 downto 0) := (others => '0');
		rsv_x_seccos_0  : in  std_logic_vector(17 downto 0) := (others => '0');
		--
		rsv_x_priexc_1  : in  std_logic_vector(17 downto 0) := (others => '0');
		rsv_x_secsin_1  : in  std_logic_vector(17 downto 0) := (others => '0');
		rsv_x_seccos_1  : in  std_logic_vector(17 downto 0) := (others => '0');
		--
		-- motor #0 control IP

		-- external angle
		ext_angle_0     : out std_logic_vector(31 downto 0);
		
		-- resolver
		rsv_priexc_0    : out std_logic_vector(17 downto 0);
		rsv_secsin_0    : out std_logic_vector(17 downto 0);
		rsv_seccos_0    : out std_logic_vector(17 downto 0);
		
		-- hall sensor
		hls_hallsig_0   : out std_logic_vector(5 downto 0);
		
		-- digital encoder
		enc_cha_0       : out std_logic;
		enc_chb_0       : out std_logic;
		enc_chi_0       : out std_logic;
		
		-- input data
		--
		acq_iphs_a_0    : out std_logic_vector(17 downto 0);
		acq_iphs_b_0    : out std_logic_vector(17 downto 0);
		acq_iphs_c_0    : out std_logic_vector(17 downto 0);
		acq_ibus_x_0    : out std_logic_vector(17 downto 0);
		acq_vphs_a_0    : out std_logic_vector(17 downto 0);
		acq_vphs_b_0    : out std_logic_vector(17 downto 0);
		acq_vphs_c_0    : out std_logic_vector(17 downto 0);
		acq_vbus_x_0    : out std_logic_vector(17 downto 0);
		acq_vphs_n_0    : out std_logic_vector(17 downto 0);
		--
		acq_sync_0      : out std_logic;
		
		-- output offset
		--
		ofs_iphs_a_0    : in  std_logic_vector(17 downto 0);
		ofs_iphs_b_0    : in  std_logic_vector(17 downto 0);
		ofs_iphs_c_0    : in  std_logic_vector(17 downto 0);
		ofs_ibus_x_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_a_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_b_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_c_0    : in  std_logic_vector(17 downto 0);
		ofs_vbus_x_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_n_0    : in  std_logic_vector(17 downto 0);

		-- normalized values (offset removed)
		--
		nrm_iphs_a_0    : in  std_logic_vector(17 downto 0);
		nrm_iphs_b_0    : in  std_logic_vector(17 downto 0);
		nrm_iphs_c_0    : in  std_logic_vector(17 downto 0);
		nrm_ibus_x_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_a_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_b_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_c_0    : in  std_logic_vector(17 downto 0);
		nrm_vbus_x_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_n_0    : in  std_logic_vector(17 downto 0);
		--
		nrm_sync_0      : in  std_logic;

		-- filtered values
		--
		flt_iphs_a_0    : in  std_logic_vector(17 downto 0);
		flt_iphs_b_0    : in  std_logic_vector(17 downto 0);
		flt_iphs_c_0    : in  std_logic_vector(17 downto 0);
		flt_ibus_x_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_a_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_b_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_c_0    : in  std_logic_vector(17 downto 0);
		flt_vbus_x_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_n_0    : in  std_logic_vector(17 downto 0);
		--
		flt_sync_0      : in  std_logic;

		-- current overflow
		--
		ovi_iphs_a_0    : in  std_logic;
		ovi_iphs_b_0    : in  std_logic;
		ovi_iphs_c_0    : in  std_logic;
		ovi_ibus_x_0    : in  std_logic;

		-- Is overflow
		ismodover_0     : in  std_logic;
		
		-- 1=generator, 0=motor
		generator_0     : in  std_logic;
		
		-- modulator type
		modtype_0       : in  std_logic_vector(0 downto 0);
		
		-- modulator levels
		modlevels_0     : in  std_logic_vector(0 downto 0);
		
		-- motor is 3-phases (0=bipolar)
		is3phases_0     : in  std_logic;
		
		-- pwm 3-phases outputs
		pwm2p_l2c1w1_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w1_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c1w2_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w2_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2sync_0  : in  std_logic;
		
		-- pwm 2-phases outputs
		pwm3p_l2cxw1_0  : in  std_logic_vector(2 downto 0);
		pwm3p_l2cxw2_0  : in  std_logic_vector(2 downto 0);
		pwm3p_l2sync_0  : in  std_logic;
		
		-- rpfm 3-ph two levels outputs
		rpfm3p_l2cxw1_0 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2cxw2_0 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2sync_0 : in  std_logic;
		
		-- rpfm 3-ph three levels outputs
		rpfm3p_l3c1w1_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w1_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w1_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c1w2_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w2_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w2_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3sync_0 : in  std_logic;
		
		-- dead time value
		deadtval_0      : in  std_logic_vector(7 downto 0);
		
		-- gate unit switches
		guswcnt_0       : out std_logic_vector(31 downto 0);
		
		-- in-failure
		hwfail_0        : out std_logic;
		
		-- in-kill
		hwkill_0        : out std_logic;
		
		-- coil driver
		coilgear_0      : in  std_logic;
		coilenab_0      : in  std_logic;
		
		-- system error
		syserr_0        : in  std_logic;
		
		-- reset pending errors
		reseterr_0      : in  std_logic;
		
		-- zero current offset 
		zerocurr_0      : in  std_logic;
		
		-- zero vbus offset 
		zerovbusx_0     : in  std_logic;

		-- zero vphs offset 
		zerovphsx_0     : in  std_logic;
		
		-- control probes
		pix_setval_0    : in std_logic_vector(17 downto 0);
		piy_setval_0    : in std_logic_vector(17 downto 0);
		
		pix_fbkval_0    : in std_logic_vector(17 downto 0);
		piy_fbkval_0    : in std_logic_vector(17 downto 0);
		
		pix_outval_0    : in std_logic_vector(17 downto 0);
		piy_outval_0    : in std_logic_vector(17 downto 0);
		
		-- Vs
		vsmodval_0      : in  std_logic_vector(16 downto 0);
		vsangval_0      : in  std_logic_vector(31 downto 0);
		
		-- Is
		ismodval_0      : in  std_logic_vector(16 downto 0);
		isangval_0      : in  std_logic_vector(31 downto 0);

		-- smo
		smo_bemf_p_0    : in  std_logic_vector(31 downto 0);
		smo_bemf_m_0    : in  std_logic_vector(16 downto 0);
		
		-- e-angles
		mrt_angle_0     : in  std_logic_vector(31 downto 0);
		enc_angle_0     : in  std_logic_vector(31 downto 0);
		hls_angle_0     : in  std_logic_vector(31 downto 0);
		rsv_angle_0     : in  std_logic_vector(31 downto 0);
		smo_angle_0     : in  std_logic_vector(31 downto 0);
		
		-- Rotor angle
		rot_angle_0     : in  std_logic_vector(31 downto 0);
		
		-- rot selector
		rot_select_0    : in  std_logic_vector( 2 downto 0);
		
		-- actual speed
		spd_speed_0     : in  std_logic_vector(31 downto 0);
		
		-- position
		pos_position_0  : in  std_logic_vector(31 downto 0);
		
		-- foc cycle time
		tsclocks_0      : in  std_logic_vector(11 downto 0);
		
		-- motor #1 control IP

		-- external angle
		ext_angle_1     : out std_logic_vector(31 downto 0);
		
		-- resolver
		rsv_priexc_1    : out std_logic_vector(17 downto 0);
		rsv_secsin_1    : out std_logic_vector(17 downto 0);
		rsv_seccos_1    : out std_logic_vector(17 downto 0);
		
		-- hall sensor
		hls_hallsig_1   : out std_logic_vector(5 downto 0);
		
		-- digital encoder
		enc_cha_1       : out std_logic;
		enc_chb_1       : out std_logic;
		enc_chi_1       : out std_logic;
		
		-- input data
		--
		acq_iphs_a_1    : out std_logic_vector(17 downto 0);
		acq_iphs_b_1    : out std_logic_vector(17 downto 0);
		acq_iphs_c_1    : out std_logic_vector(17 downto 0);
		acq_ibus_x_1    : out std_logic_vector(17 downto 0);
		acq_vphs_a_1    : out std_logic_vector(17 downto 0);
		acq_vphs_b_1    : out std_logic_vector(17 downto 0);
		acq_vphs_c_1    : out std_logic_vector(17 downto 0);
		acq_vbus_x_1    : out std_logic_vector(17 downto 0);
		acq_vphs_n_1    : out std_logic_vector(17 downto 0);
		--
		acq_sync_1      : out std_logic;
		
		-- output offset
		--
		ofs_iphs_a_1    : in  std_logic_vector(17 downto 0);
		ofs_iphs_b_1    : in  std_logic_vector(17 downto 0);
		ofs_iphs_c_1    : in  std_logic_vector(17 downto 0);
		ofs_ibus_x_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_a_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_b_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_c_1    : in  std_logic_vector(17 downto 0);
		ofs_vbus_x_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_n_1    : in  std_logic_vector(17 downto 0);

		-- normalized values (offset removed)
		--
		nrm_iphs_a_1    : in  std_logic_vector(17 downto 0);
		nrm_iphs_b_1    : in  std_logic_vector(17 downto 0);
		nrm_iphs_c_1    : in  std_logic_vector(17 downto 0);
		nrm_ibus_x_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_a_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_b_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_c_1    : in  std_logic_vector(17 downto 0);
		nrm_vbus_x_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_n_1    : in  std_logic_vector(17 downto 0);
		--
		nrm_sync_1      : in  std_logic;

		-- filtered values
		--
		flt_iphs_a_1    : in  std_logic_vector(17 downto 0);
		flt_iphs_b_1    : in  std_logic_vector(17 downto 0);
		flt_iphs_c_1    : in  std_logic_vector(17 downto 0);
		flt_ibus_x_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_a_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_b_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_c_1    : in  std_logic_vector(17 downto 0);
		flt_vbus_x_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_n_1    : in  std_logic_vector(17 downto 0);
		--
		flt_sync_1      : in  std_logic;

		-- current overflow
		--
		ovi_iphs_a_1    : in  std_logic;
		ovi_iphs_b_1    : in  std_logic;
		ovi_iphs_c_1    : in  std_logic;
		ovi_ibus_x_1    : in  std_logic;

		-- Is overflow
		ismodover_1     : in  std_logic;
		
		-- 1=generator, 0=motor
		generator_1     : in  std_logic;
		
		-- modulator type
		modtype_1       : in  std_logic_vector(0 downto 0);
		
		-- modulator levels
		modlevels_1     : in  std_logic_vector(0 downto 0);
		
		-- motor is 3-phases (0=bipolar)
		is3phases_1     : in  std_logic;
		
		-- pwm 3-phases outputs
		pwm2p_l2c1w1_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w1_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c1w2_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w2_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2sync_1  : in  std_logic;
		
		-- pwm 2-phases outputs
		pwm3p_l2cxw1_1  : in  std_logic_vector(2 downto 0);
		pwm3p_l2cxw2_1  : in  std_logic_vector(2 downto 0);
		pwm3p_l2sync_1  : in  std_logic;
		
		-- rpfm 3-ph two levels outputs
		rpfm3p_l2cxw1_1 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2cxw2_1 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2sync_1 : in  std_logic;
		
		-- rpfm 3-ph three levels outputs
		rpfm3p_l3c1w1_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w1_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w1_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c1w2_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w2_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w2_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3sync_1 : in  std_logic;
		
		-- dead time value
		deadtval_1      : in  std_logic_vector(7 downto 0);
		
		-- gate unit switches
		guswcnt_1       : out std_logic_vector(31 downto 0);
		
		-- in-failure
		hwfail_1        : out std_logic;
		
		-- in-kill
		hwkill_1        : out std_logic;
		
		-- coil driver
		coilgear_1      : in  std_logic;
		coilenab_1      : in  std_logic;
		
		-- system error
		syserr_1        : in  std_logic;
		
		-- reset pending errors
		reseterr_1      : in  std_logic;
		
		-- zero current offset 
		zerocurr_1      : in  std_logic;
		
		-- zero vbus offset 
		zerovbusx_1     : in  std_logic;

		-- zero vphs offset 
		zerovphsx_1     : in  std_logic;
		
		-- control probes
		pix_setval_1    : in std_logic_vector(17 downto 0);
		piy_setval_1    : in std_logic_vector(17 downto 0);
		
		pix_fbkval_1    : in std_logic_vector(17 downto 0);
		piy_fbkval_1    : in std_logic_vector(17 downto 0);
		
		pix_outval_1    : in std_logic_vector(17 downto 0);
		piy_outval_1    : in std_logic_vector(17 downto 0);
		
		-- Vs
		vsmodval_1      : in  std_logic_vector(16 downto 0);
		vsangval_1      : in  std_logic_vector(31 downto 0);
		
		-- Is
		ismodval_1      : in  std_logic_vector(16 downto 0);
		isangval_1      : in  std_logic_vector(31 downto 0);

		-- smo
		smo_bemf_p_1    : in  std_logic_vector(31 downto 0);
		smo_bemf_m_1    : in  std_logic_vector(16 downto 0);
		
		-- e-angles
		mrt_angle_1     : in  std_logic_vector(31 downto 0);
		enc_angle_1     : in  std_logic_vector(31 downto 0);
		hls_angle_1     : in  std_logic_vector(31 downto 0);
		rsv_angle_1     : in  std_logic_vector(31 downto 0);
		smo_angle_1     : in  std_logic_vector(31 downto 0);
		
		-- Rotor angle
		rot_angle_1     : in  std_logic_vector(31 downto 0);
		
		-- rot selector
		rot_select_1    : in  std_logic_vector( 2 downto 0);
		
		-- actual speed
		spd_speed_1     : in  std_logic_vector(31 downto 0);
		
		-- position
		pos_position_1  : in  std_logic_vector(31 downto 0);
		
		-- foc cycle time
		tsclocks_1      : in  std_logic_vector(11 downto 0)
	);

end entity axi_netmot_drv;

architecture Behavioral of axi_netmot_drv is

	component axi4ltiny
		generic
		(
			C_S_AXI_DATA_WIDTH : integer range 32 to 32 := 32;
			C_S_AXI_ADDR_WIDTH : integer range  1 to 32 := 16
		);
		port
		(
			-- AXI4LITE
			S_AXI_ACLK    : in  std_logic;
			S_AXI_ARESETN : in  std_logic;
			S_AXI_AWADDR  : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
			S_AXI_AWVALID : in  std_logic;
			S_AXI_AWREADY : out std_logic;
			S_AXI_WDATA   : in  std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
			S_AXI_WSTRB   : in  std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 downto 0);
			S_AXI_WVALID  : in  std_logic;
			S_AXI_WREADY  : out std_logic;
			S_AXI_BRESP   : out std_logic_vector(1 downto 0);
			S_AXI_BVALID  : out std_logic;
			S_AXI_BREADY  : in  std_logic;
			S_AXI_ARADDR  : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
			S_AXI_ARVALID : in  std_logic;
			S_AXI_ARREADY : out std_logic;
			S_AXI_RDATA   : out std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
			S_AXI_RRESP   : out std_logic_vector(1 downto 0);
			S_AXI_RVALID  : out std_logic;
			S_AXI_RREADY  : in  std_logic;
			-- local bus  
			host_clk      : out std_logic;
			host_rst      : out std_logic;
			host_en       : out std_logic;
			host_we       : out std_logic;
			host_addr     : out std_logic_vector(31 downto 0);
			host_din      : out std_logic_vector(31 downto 0);
			host_dout     : in  std_logic_vector(31 downto 0);
			host_ack      : in  std_logic
		);
	end component;
	
	component nmbrdsys
		generic
		(
			C_FAMILY          : string := "zynq";
			C_CLOCK_PERIOD_PS : integer := 10000;
			C_RXLINK_SPACE    : integer := 0
		);
		port 
		( 
			clock  : in  std_logic;
			reset  : in  std_logic;
			
			--
			-- host interface
			--
			
			host_en    : in  std_logic;
			host_we    : in  std_logic;
			host_addr  : in  std_logic_vector(31 downto 0);
			host_din   : in  std_logic_vector(31 downto 0);
			host_dout  : out std_logic_vector(31 downto 0);
			host_ack   : out std_logic;
			-- ip ports
			-- coil drivers
			Motor_Coil      : out std_logic_vector (7 downto 0);
			-- serial link
			Cmd_Clock       : out std_logic;
			Cmd_Data        : out std_logic;
			Sts_Clock       : in  std_logic;
			Sts_Data        : in  std_logic;
			--
			-- external angle in
			ext_x_angle_0   : in  std_logic_vector(31 downto 0);
			ext_x_angle_1   : in  std_logic_vector(31 downto 0);
			--
			-- resolver in
			rsv_x_priexc_0  : in  std_logic_vector(17 downto 0);
			rsv_x_secsin_0  : in  std_logic_vector(17 downto 0);
			rsv_x_seccos_0  : in  std_logic_vector(17 downto 0);
			--
			rsv_x_priexc_1  : in  std_logic_vector(17 downto 0);
			rsv_x_secsin_1  : in  std_logic_vector(17 downto 0);
			rsv_x_seccos_1  : in  std_logic_vector(17 downto 0);
			--
			-- motor #0 control IP

			-- external angle
			ext_angle_0     : out std_logic_vector(31 downto 0);
			
			-- resolver
			rsv_priexc_0    : out std_logic_vector(17 downto 0);
			rsv_secsin_0    : out std_logic_vector(17 downto 0);
			rsv_seccos_0    : out std_logic_vector(17 downto 0);
			
			-- hall sensor
			hls_hallsig_0   : out std_logic_vector(5 downto 0);
			
			-- digital encoder
			enc_cha_0       : out std_logic;
			enc_chb_0       : out std_logic;
			enc_chi_0       : out std_logic;
			
			-- input data
			--
			acq_iphs_a_0    : out std_logic_vector(17 downto 0);
			acq_iphs_b_0    : out std_logic_vector(17 downto 0);
			acq_iphs_c_0    : out std_logic_vector(17 downto 0);
			acq_ibus_x_0    : out std_logic_vector(17 downto 0);
			acq_vphs_a_0    : out std_logic_vector(17 downto 0);
			acq_vphs_b_0    : out std_logic_vector(17 downto 0);
			acq_vphs_c_0    : out std_logic_vector(17 downto 0);
			acq_vbus_x_0    : out std_logic_vector(17 downto 0);
			acq_vphs_n_0    : out std_logic_vector(17 downto 0);
			--
			acq_sync_0      : out std_logic;
			
			-- output offset
			--
			ofs_iphs_a_0    : in  std_logic_vector(17 downto 0);
			ofs_iphs_b_0    : in  std_logic_vector(17 downto 0);
			ofs_iphs_c_0    : in  std_logic_vector(17 downto 0);
			ofs_ibus_x_0    : in  std_logic_vector(17 downto 0);
			ofs_vphs_a_0    : in  std_logic_vector(17 downto 0);
			ofs_vphs_b_0    : in  std_logic_vector(17 downto 0);
			ofs_vphs_c_0    : in  std_logic_vector(17 downto 0);
			ofs_vbus_x_0    : in  std_logic_vector(17 downto 0);
			ofs_vphs_n_0    : in  std_logic_vector(17 downto 0);

			-- normalized values (offset removed)
			--
			nrm_iphs_a_0    : in  std_logic_vector(17 downto 0);
			nrm_iphs_b_0    : in  std_logic_vector(17 downto 0);
			nrm_iphs_c_0    : in  std_logic_vector(17 downto 0);
			nrm_ibus_x_0    : in  std_logic_vector(17 downto 0);
			nrm_vphs_a_0    : in  std_logic_vector(17 downto 0);
			nrm_vphs_b_0    : in  std_logic_vector(17 downto 0);
			nrm_vphs_c_0    : in  std_logic_vector(17 downto 0);
			nrm_vbus_x_0    : in  std_logic_vector(17 downto 0);
			nrm_vphs_n_0    : in  std_logic_vector(17 downto 0);
			--
			nrm_sync_0      : in  std_logic;

			-- filtered values
			--
			flt_iphs_a_0    : in  std_logic_vector(17 downto 0);
			flt_iphs_b_0    : in  std_logic_vector(17 downto 0);
			flt_iphs_c_0    : in  std_logic_vector(17 downto 0);
			flt_ibus_x_0    : in  std_logic_vector(17 downto 0);
			flt_vphs_a_0    : in  std_logic_vector(17 downto 0);
			flt_vphs_b_0    : in  std_logic_vector(17 downto 0);
			flt_vphs_c_0    : in  std_logic_vector(17 downto 0);
			flt_vbus_x_0    : in  std_logic_vector(17 downto 0);
			flt_vphs_n_0    : in  std_logic_vector(17 downto 0);
			--
			flt_sync_0      : in  std_logic;

			-- current overflow
			--
			ovi_iphs_a_0    : in  std_logic;
			ovi_iphs_b_0    : in  std_logic;
			ovi_iphs_c_0    : in  std_logic;
			ovi_ibus_x_0    : in  std_logic;

			-- Is overflow
			ismodover_0     : in  std_logic;
			
			-- 1=generator, 0=motor
			generator_0     : in  std_logic;
			
			-- modulator type
			modtype_0       : in  std_logic_vector(0 downto 0);
			
			-- modulator levels
			modlevels_0     : in  std_logic_vector(0 downto 0);
			
			-- motor is 3-phases (0=bipolar)
			is3phases_0     : in  std_logic;
			
			-- pwm 3-phases outputs
			pwm2p_l2c1w1_0  : in  std_logic_vector(1 downto 0);
			pwm2p_l2c2w1_0  : in  std_logic_vector(1 downto 0);
			pwm2p_l2c1w2_0  : in  std_logic_vector(1 downto 0);
			pwm2p_l2c2w2_0  : in  std_logic_vector(1 downto 0);
			pwm2p_l2sync_0  : in  std_logic;
			
			-- pwm 2-phases outputs
			pwm3p_l2cxw1_0  : in  std_logic_vector(2 downto 0);
			pwm3p_l2cxw2_0  : in  std_logic_vector(2 downto 0);
			pwm3p_l2sync_0  : in  std_logic;
			
			-- rpfm 3-ph two levels outputs
			rpfm3p_l2cxw1_0 : in  std_logic_vector(2 downto 0);
			rpfm3p_l2cxw2_0 : in  std_logic_vector(2 downto 0);
			rpfm3p_l2sync_0 : in  std_logic;
			
			-- rpfm 3-ph three levels outputs
			rpfm3p_l3c1w1_0 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c2w1_0 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c3w1_0 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c1w2_0 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c2w2_0 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c3w2_0 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3sync_0 : in  std_logic;
			
			-- dead time value
			deadtval_0      : in  std_logic_vector(7 downto 0);
			
			-- gate unit switches
			guswcnt_0       : out std_logic_vector(31 downto 0);
			
			-- in-failure
			hwfail_0        : out std_logic;
			
			-- in-kill
			hwkill_0        : out std_logic;
			
			-- coil driver
			coilgear_0      : in  std_logic;
			coilenab_0      : in  std_logic;
			
			-- system error
			syserr_0        : in  std_logic;
			
			-- reset pending errors
			reseterr_0      : in  std_logic;
			
			-- zero current offset 
			zerocurr_0      : in  std_logic;
			
			-- zero vbus offset 
			zerovbusx_0     : in  std_logic;

			-- zero vphs offset 
			zerovphsx_0     : in  std_logic;
			
			-- control probes
			pix_setval_0    : in std_logic_vector(17 downto 0);
			piy_setval_0    : in std_logic_vector(17 downto 0);
			
			pix_fbkval_0    : in std_logic_vector(17 downto 0);
			piy_fbkval_0    : in std_logic_vector(17 downto 0);
			
			pix_outval_0    : in std_logic_vector(17 downto 0);
			piy_outval_0    : in std_logic_vector(17 downto 0);
			
			-- Vs
			vsmodval_0      : in  std_logic_vector(16 downto 0);
			vsangval_0      : in  std_logic_vector(31 downto 0);
			
			-- Is
			ismodval_0      : in  std_logic_vector(16 downto 0);
			isangval_0      : in  std_logic_vector(31 downto 0);

			-- smo
			smo_bemf_p_0    : in  std_logic_vector(31 downto 0);
			smo_bemf_m_0    : in  std_logic_vector(16 downto 0);
			
			-- e-angles
			mrt_angle_0     : in  std_logic_vector(31 downto 0);
			enc_angle_0     : in  std_logic_vector(31 downto 0);
			hls_angle_0     : in  std_logic_vector(31 downto 0);
			rsv_angle_0     : in  std_logic_vector(31 downto 0);
			smo_angle_0     : in  std_logic_vector(31 downto 0);
			
			-- Rotor angle
			rot_angle_0     : in  std_logic_vector(31 downto 0);
			
			-- rot selector
			rot_select_0    : in  std_logic_vector( 2 downto 0);
			
			-- actual speed
			spd_speed_0     : in  std_logic_vector(31 downto 0);
			
			-- position
			pos_position_0  : in  std_logic_vector(31 downto 0);
			
			-- foc cycle time
			tsclocks_0      : in  std_logic_vector(11 downto 0);
			
			-- motor #1 control IP

			-- external angle
			ext_angle_1     : out std_logic_vector(31 downto 0);
			
			-- resolver
			rsv_priexc_1    : out std_logic_vector(17 downto 0);
			rsv_secsin_1    : out std_logic_vector(17 downto 0);
			rsv_seccos_1    : out std_logic_vector(17 downto 0);
			
			-- hall sensor
			hls_hallsig_1   : out std_logic_vector(5 downto 0);
			
			-- digital encoder
			enc_cha_1       : out std_logic;
			enc_chb_1       : out std_logic;
			enc_chi_1       : out std_logic;
			
			-- input data
			--
			acq_iphs_a_1    : out std_logic_vector(17 downto 0);
			acq_iphs_b_1    : out std_logic_vector(17 downto 0);
			acq_iphs_c_1    : out std_logic_vector(17 downto 0);
			acq_ibus_x_1    : out std_logic_vector(17 downto 0);
			acq_vphs_a_1    : out std_logic_vector(17 downto 0);
			acq_vphs_b_1    : out std_logic_vector(17 downto 0);
			acq_vphs_c_1    : out std_logic_vector(17 downto 0);
			acq_vbus_x_1    : out std_logic_vector(17 downto 0);
			acq_vphs_n_1    : out std_logic_vector(17 downto 0);
			--
			acq_sync_1      : out std_logic;
			
			-- output offset
			--
			ofs_iphs_a_1    : in  std_logic_vector(17 downto 0);
			ofs_iphs_b_1    : in  std_logic_vector(17 downto 0);
			ofs_iphs_c_1    : in  std_logic_vector(17 downto 0);
			ofs_ibus_x_1    : in  std_logic_vector(17 downto 0);
			ofs_vphs_a_1    : in  std_logic_vector(17 downto 0);
			ofs_vphs_b_1    : in  std_logic_vector(17 downto 0);
			ofs_vphs_c_1    : in  std_logic_vector(17 downto 0);
			ofs_vbus_x_1    : in  std_logic_vector(17 downto 0);
			ofs_vphs_n_1    : in  std_logic_vector(17 downto 0);

			-- normalized values (offset removed)
			--
			nrm_iphs_a_1    : in  std_logic_vector(17 downto 0);
			nrm_iphs_b_1    : in  std_logic_vector(17 downto 0);
			nrm_iphs_c_1    : in  std_logic_vector(17 downto 0);
			nrm_ibus_x_1    : in  std_logic_vector(17 downto 0);
			nrm_vphs_a_1    : in  std_logic_vector(17 downto 0);
			nrm_vphs_b_1    : in  std_logic_vector(17 downto 0);
			nrm_vphs_c_1    : in  std_logic_vector(17 downto 0);
			nrm_vbus_x_1    : in  std_logic_vector(17 downto 0);
			nrm_vphs_n_1    : in  std_logic_vector(17 downto 0);
			--
			nrm_sync_1      : in  std_logic;

			-- filtered values
			--
			flt_iphs_a_1    : in  std_logic_vector(17 downto 0);
			flt_iphs_b_1    : in  std_logic_vector(17 downto 0);
			flt_iphs_c_1    : in  std_logic_vector(17 downto 0);
			flt_ibus_x_1    : in  std_logic_vector(17 downto 0);
			flt_vphs_a_1    : in  std_logic_vector(17 downto 0);
			flt_vphs_b_1    : in  std_logic_vector(17 downto 0);
			flt_vphs_c_1    : in  std_logic_vector(17 downto 0);
			flt_vbus_x_1    : in  std_logic_vector(17 downto 0);
			flt_vphs_n_1    : in  std_logic_vector(17 downto 0);
			--
			flt_sync_1      : in  std_logic;

			-- current overflow
			--
			ovi_iphs_a_1    : in  std_logic;
			ovi_iphs_b_1    : in  std_logic;
			ovi_iphs_c_1    : in  std_logic;
			ovi_ibus_x_1    : in  std_logic;

			-- Is overflow
			ismodover_1     : in  std_logic;
			
			-- 1=generator, 0=motor
			generator_1     : in  std_logic;
			
			-- modulator type
			modtype_1       : in  std_logic_vector(0 downto 0);
			
			-- modulator levels
			modlevels_1     : in  std_logic_vector(0 downto 0);
			
			-- motor is 3-phases (0=bipolar)
			is3phases_1     : in  std_logic;
			
			-- pwm 3-phases outputs
			pwm2p_l2c1w1_1  : in  std_logic_vector(1 downto 0);
			pwm2p_l2c2w1_1  : in  std_logic_vector(1 downto 0);
			pwm2p_l2c1w2_1  : in  std_logic_vector(1 downto 0);
			pwm2p_l2c2w2_1  : in  std_logic_vector(1 downto 0);
			pwm2p_l2sync_1  : in  std_logic;
			
			-- pwm 2-phases outputs
			pwm3p_l2cxw1_1  : in  std_logic_vector(2 downto 0);
			pwm3p_l2cxw2_1  : in  std_logic_vector(2 downto 0);
			pwm3p_l2sync_1  : in  std_logic;
			
			-- rpfm 3-ph two levels outputs
			rpfm3p_l2cxw1_1 : in  std_logic_vector(2 downto 0);
			rpfm3p_l2cxw2_1 : in  std_logic_vector(2 downto 0);
			rpfm3p_l2sync_1 : in  std_logic;
			
			-- rpfm 3-ph three levels outputs
			rpfm3p_l3c1w1_1 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c2w1_1 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c3w1_1 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c1w2_1 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c2w2_1 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3c3w2_1 : in  std_logic_vector(1 downto 0);
			rpfm3p_l3sync_1 : in  std_logic;
			
			-- dead time value
			deadtval_1      : in  std_logic_vector(7 downto 0);
			
			-- gate unit switches
			guswcnt_1       : out std_logic_vector(31 downto 0);
			
			-- in-failure
			hwfail_1        : out std_logic;
			
			-- in-kill
			hwkill_1        : out std_logic;
			
			-- coil driver
			coilgear_1      : in  std_logic;
			coilenab_1      : in  std_logic;
			
			-- system error
			syserr_1        : in  std_logic;
			
			-- reset pending errors
			reseterr_1      : in  std_logic;
			
			-- zero current offset 
			zerocurr_1      : in  std_logic;
			
			-- zero vbus offset 
			zerovbusx_1     : in  std_logic;

			-- zero vphs offset 
			zerovphsx_1     : in  std_logic;
			
			-- control probes
			pix_setval_1    : in std_logic_vector(17 downto 0);
			piy_setval_1    : in std_logic_vector(17 downto 0);
			
			pix_fbkval_1    : in std_logic_vector(17 downto 0);
			piy_fbkval_1    : in std_logic_vector(17 downto 0);
			
			pix_outval_1    : in std_logic_vector(17 downto 0);
			piy_outval_1    : in std_logic_vector(17 downto 0);
			
			-- Vs
			vsmodval_1      : in  std_logic_vector(16 downto 0);
			vsangval_1      : in  std_logic_vector(31 downto 0);
			
			-- Is
			ismodval_1      : in  std_logic_vector(16 downto 0);
			isangval_1      : in  std_logic_vector(31 downto 0);

			-- smo
			smo_bemf_p_1    : in  std_logic_vector(31 downto 0);
			smo_bemf_m_1    : in  std_logic_vector(16 downto 0);
			
			-- e-angles
			mrt_angle_1     : in  std_logic_vector(31 downto 0);
			enc_angle_1     : in  std_logic_vector(31 downto 0);
			hls_angle_1     : in  std_logic_vector(31 downto 0);
			rsv_angle_1     : in  std_logic_vector(31 downto 0);
			smo_angle_1     : in  std_logic_vector(31 downto 0);
			
			-- Rotor angle
			rot_angle_1     : in  std_logic_vector(31 downto 0);
			
			-- rot selector
			rot_select_1    : in  std_logic_vector( 2 downto 0);
			
			-- actual speed
			spd_speed_1     : in  std_logic_vector(31 downto 0);
			
			-- position
			pos_position_1  : in  std_logic_vector(31 downto 0);
			
			-- foc cycle time
			tsclocks_1      : in  std_logic_vector(11 downto 0)
		);
	end component;

	-- wrapper

	signal host_clk        : std_logic;
	signal host_rst        : std_logic;
	--
	signal host_en         : std_logic;
	signal host_we         : std_logic;
	signal host_addr       : std_logic_vector(31 downto 0);
	signal host_din        : std_logic_vector(31 downto 0);
	signal host_dout       : std_logic_vector(31 downto 0);
	signal host_ack        : std_logic;

	-- TODO change time to freq !!!!
	constant C_CLOCK_PERIOD_PS : integer := 1000000000 / (C_SYSTEMCLOCK / 1000);
	--
	
begin

	assert C_S_AXI_ACLK_FREQ_HZ = C_SYSTEMCLOCK report "AXI clock /= system clock" severity failure;
	assert false report "C_SYSTEMCLOCK=" & integer'image(C_SYSTEMCLOCK) severity note;
	
	AXI_LITE_IPIF_I : axi4ltiny
		generic map
		(
			C_S_AXI_DATA_WIDTH => C_S_AXI_DATA_WIDTH,
			C_S_AXI_ADDR_WIDTH => C_S_AXI_ADDR_WIDTH
		)
		port map
		(
			-- AXI4LITE
			S_AXI_ACLK    => S_AXI_ACLK,
			S_AXI_ARESETN => S_AXI_ARESETN,
			S_AXI_AWADDR  => S_AXI_AWADDR,
			S_AXI_AWVALID => S_AXI_AWVALID,
			S_AXI_AWREADY => S_AXI_AWREADY,
			S_AXI_WDATA   => S_AXI_WDATA,
			S_AXI_WSTRB   => S_AXI_WSTRB,
			S_AXI_WVALID  => S_AXI_WVALID,
			S_AXI_WREADY  => S_AXI_WREADY,
			S_AXI_BRESP   => S_AXI_BRESP,
			S_AXI_BVALID  => S_AXI_BVALID,
			S_AXI_BREADY  => S_AXI_BREADY,
			S_AXI_ARADDR  => S_AXI_ARADDR,
			S_AXI_ARVALID => S_AXI_ARVALID,
			S_AXI_ARREADY => S_AXI_ARREADY,
			S_AXI_RDATA   => S_AXI_RDATA,
			S_AXI_RRESP   => S_AXI_RRESP,
			S_AXI_RVALID  => S_AXI_RVALID,
			S_AXI_RREADY  => S_AXI_RREADY,
			-- local bus  
			host_clk      => host_clk,
			host_rst      => host_rst,
			host_en       => host_en,
			host_we       => host_we,
			host_addr     => host_addr,
			host_din      => host_din,
			host_dout     => host_dout,
			host_ack      => host_ack
		);

  nmbrdsys_inst : nmbrdsys
		generic map
		(
			C_FAMILY          => C_FAMILY,
			C_CLOCK_PERIOD_PS => C_CLOCK_PERIOD_PS,
			C_RXLINK_SPACE    => C_RXLINK_SPACE
		)
		port map
		(
			clock            => host_clk,
			reset            => host_rst,
			--
			host_en          => host_en,
			host_we          => host_we,
			host_addr        => host_addr,
			host_din         => host_din,
			host_dout        => host_dout,
			host_ack         => host_ack,
			--
			Motor_Coil       => Motor_Coil,
			--
			Cmd_Clock        => Cmd_Clock,
			Cmd_Data         => Cmd_Data,
			Sts_Clock        => Sts_Clock,
			Sts_Data         => Sts_Data,
			--
			-- external angle in
			ext_x_angle_0   => ext_x_angle_0,
			ext_x_angle_1   => ext_x_angle_1,
			--
			-- resolver in
			rsv_x_priexc_0  => rsv_x_priexc_0,
			rsv_x_secsin_0  => rsv_x_secsin_0,
			rsv_x_seccos_0  => rsv_x_seccos_0,
			--
			rsv_x_priexc_1  => rsv_x_priexc_1,
			rsv_x_secsin_1  => rsv_x_secsin_1,
			rsv_x_seccos_1  => rsv_x_seccos_1,
			--
			-- motor #0 control IP

			-- external angle
			ext_angle_0      => ext_angle_0,
			
			-- resolver
			rsv_priexc_0     => rsv_priexc_0,
			rsv_secsin_0     => rsv_secsin_0,
			rsv_seccos_0     => rsv_seccos_0,
			
			-- hall sensor
			hls_hallsig_0    => hls_hallsig_0,
			
			-- digital encoder
			enc_cha_0        => enc_cha_0,
			enc_chb_0        => enc_chb_0,
			enc_chi_0        => enc_chi_0,
			
			-- input data
			--
			acq_iphs_a_0     => acq_iphs_a_0,
			acq_iphs_b_0     => acq_iphs_b_0,
			acq_iphs_c_0     => acq_iphs_c_0,
			acq_ibus_x_0     => acq_ibus_x_0,
			acq_vphs_a_0     => acq_vphs_a_0,
			acq_vphs_b_0     => acq_vphs_b_0,
			acq_vphs_c_0     => acq_vphs_c_0,
			acq_vbus_x_0     => acq_vbus_x_0,
			acq_vphs_n_0     => acq_vphs_n_0,
			--
			acq_sync_0       => acq_sync_0,
			
			-- output offset
			--
			ofs_iphs_a_0     => ofs_iphs_a_0,
			ofs_iphs_b_0     => ofs_iphs_b_0,
			ofs_iphs_c_0     => ofs_iphs_c_0,
			ofs_ibus_x_0     => ofs_ibus_x_0,
			ofs_vphs_a_0     => ofs_vphs_a_0,
			ofs_vphs_b_0     => ofs_vphs_b_0,
			ofs_vphs_c_0     => ofs_vphs_c_0,
			ofs_vbus_x_0     => ofs_vbus_x_0,
			ofs_vphs_n_0     => ofs_vphs_n_0,

			-- normalized values (offset removed)
			--
			nrm_iphs_a_0     => nrm_iphs_a_0,
			nrm_iphs_b_0     => nrm_iphs_b_0,
			nrm_iphs_c_0     => nrm_iphs_c_0,
			nrm_ibus_x_0     => nrm_ibus_x_0,
			nrm_vphs_a_0     => nrm_vphs_a_0,
			nrm_vphs_b_0     => nrm_vphs_b_0,
			nrm_vphs_c_0     => nrm_vphs_c_0,
			nrm_vbus_x_0     => nrm_vbus_x_0,
			nrm_vphs_n_0     => nrm_vphs_n_0,
			--
			nrm_sync_0       => nrm_sync_0,

			-- filtered values
			--
			flt_iphs_a_0     => flt_iphs_a_0,
			flt_iphs_b_0     => flt_iphs_b_0,
			flt_iphs_c_0     => flt_iphs_c_0,
			flt_ibus_x_0     => flt_ibus_x_0,
			flt_vphs_a_0     => flt_vphs_a_0,
			flt_vphs_b_0     => flt_vphs_b_0,
			flt_vphs_c_0     => flt_vphs_c_0,
			flt_vbus_x_0     => flt_vbus_x_0,
			flt_vphs_n_0     => flt_vphs_n_0,
			--
			flt_sync_0       => flt_sync_0,

			-- current overflow
			--
			ovi_iphs_a_0     => ovi_iphs_a_0,
			ovi_iphs_b_0     => ovi_iphs_b_0,
			ovi_iphs_c_0     => ovi_iphs_c_0,
			ovi_ibus_x_0     => ovi_ibus_x_0,

			-- Is overflow
			ismodover_0      => ismodover_0,
			
			-- 1=generator_0, 0=motor
			generator_0      => generator_0,
			
			-- modulator type
			modtype_0        => modtype_0,
			
			-- modulator levels
			modlevels_0      => modlevels_0,
			
			-- motor is 3-phases (0=bipolar)
			is3phases_0      => is3phases_0,
			
			-- pwm 3-phases outputs
			pwm2p_l2c1w1_0   => pwm2p_l2c1w1_0,
			pwm2p_l2c2w1_0   => pwm2p_l2c2w1_0,
			pwm2p_l2c1w2_0   => pwm2p_l2c1w2_0,
			pwm2p_l2c2w2_0   => pwm2p_l2c2w2_0,
			pwm2p_l2sync_0   => pwm2p_l2sync_0,
			
			-- pwm 2-phases outputs
			pwm3p_l2cxw1_0   => pwm3p_l2cxw1_0,
			pwm3p_l2cxw2_0   => pwm3p_l2cxw2_0,
			pwm3p_l2sync_0   => pwm3p_l2sync_0,
			
			-- rpfm 3-ph two levels outputs
			rpfm3p_l2cxw1_0  => rpfm3p_l2cxw1_0,
			rpfm3p_l2cxw2_0  => rpfm3p_l2cxw2_0,
			rpfm3p_l2sync_0  => rpfm3p_l2sync_0,
			
			-- rpfm 3-ph three levels outputs
			rpfm3p_l3c1w1_0  => rpfm3p_l3c1w1_0,
			rpfm3p_l3c2w1_0  => rpfm3p_l3c2w1_0,
			rpfm3p_l3c3w1_0  => rpfm3p_l3c3w1_0,
			rpfm3p_l3c1w2_0  => rpfm3p_l3c1w2_0,
			rpfm3p_l3c2w2_0  => rpfm3p_l3c2w2_0,
			rpfm3p_l3c3w2_0  => rpfm3p_l3c3w2_0,
			rpfm3p_l3sync_0  => rpfm3p_l3sync_0,
			
			-- dead time value
			deadtval_0       => deadtval_0,
			
			-- gate unit switches
			guswcnt_0        => guswcnt_0,
			
			-- in-failure
			hwfail_0         => hwfail_0,
			
			-- in-kill
			hwkill_0         => hwkill_0,
			
			-- coil driver
			coilgear_0       => coilgear_0,
			coilenab_0       => coilenab_0,
			
			-- system error
			syserr_0         => syserr_0,
			
			-- reset pending errors
			reseterr_0       => reseterr_0,
			
			-- zero current offset 
			zerocurr_0       => zerocurr_0,
			
			-- zero vbus offset 
			zerovbusx_0      => zerovbusx_0,

			-- zero vphs offset 
			zerovphsx_0      => zerovphsx_0,
			
			-- control probes
			pix_setval_0    => pix_setval_0,
			piy_setval_0    => piy_setval_0,
			
			pix_fbkval_0    => pix_fbkval_0,
			piy_fbkval_0    => piy_fbkval_0,
			
			pix_outval_0    => pix_outval_0,
			piy_outval_0    => piy_outval_0,
			
			-- Vs
			vsmodval_0       => vsmodval_0,
			vsangval_0       => vsangval_0,
			
			-- Is
			ismodval_0       => ismodval_0,
			isangval_0       => isangval_0,

			-- smo
			smo_bemf_p_0     => smo_bemf_p_0,
			smo_bemf_m_0     => smo_bemf_m_0,

			-- e-angles
			mrt_angle_0      => mrt_angle_0,
			enc_angle_0      => enc_angle_0,
			hls_angle_0      => hls_angle_0,
			rsv_angle_0      => rsv_angle_0,
			smo_angle_0      => smo_angle_0,

			-- Rotor angle
			rot_angle_0      => rot_angle_0,
			
			-- rot selector
			rot_select_0     => rot_select_0,
			
			-- actual speed
			spd_speed_0      => spd_speed_0,
			
			-- position
			pos_position_0   => pos_position_0,
			
			-- foc cycle time
			tsclocks_0       => tsclocks_0,

			-- motor #1 control IP

			-- external angle
			ext_angle_1      => ext_angle_1,
			
			-- resolver
			rsv_priexc_1     => rsv_priexc_1,
			rsv_secsin_1     => rsv_secsin_1,
			rsv_seccos_1     => rsv_seccos_1,
			
			-- hall sensor
			hls_hallsig_1    => hls_hallsig_1,
			
			-- digital encoder
			enc_cha_1        => enc_cha_1,
			enc_chb_1        => enc_chb_1,
			enc_chi_1        => enc_chi_1,
			
			-- input data
			--
			acq_iphs_a_1     => acq_iphs_a_1,
			acq_iphs_b_1     => acq_iphs_b_1,
			acq_iphs_c_1     => acq_iphs_c_1,
			acq_ibus_x_1     => acq_ibus_x_1,
			acq_vphs_a_1     => acq_vphs_a_1,
			acq_vphs_b_1     => acq_vphs_b_1,
			acq_vphs_c_1     => acq_vphs_c_1,
			acq_vbus_x_1     => acq_vbus_x_1,
			acq_vphs_n_1     => acq_vphs_n_1,
			--
			acq_sync_1       => acq_sync_1,
			
			-- output offset
			--
			ofs_iphs_a_1     => ofs_iphs_a_1,
			ofs_iphs_b_1     => ofs_iphs_b_1,
			ofs_iphs_c_1     => ofs_iphs_c_1,
			ofs_ibus_x_1     => ofs_ibus_x_1,
			ofs_vphs_a_1     => ofs_vphs_a_1,
			ofs_vphs_b_1     => ofs_vphs_b_1,
			ofs_vphs_c_1     => ofs_vphs_c_1,
			ofs_vbus_x_1     => ofs_vbus_x_1,
			ofs_vphs_n_1     => ofs_vphs_n_1,

			-- normalized values (offset removed)
			--
			nrm_iphs_a_1     => nrm_iphs_a_1,
			nrm_iphs_b_1     => nrm_iphs_b_1,
			nrm_iphs_c_1     => nrm_iphs_c_1,
			nrm_ibus_x_1     => nrm_ibus_x_1,
			nrm_vphs_a_1     => nrm_vphs_a_1,
			nrm_vphs_b_1     => nrm_vphs_b_1,
			nrm_vphs_c_1     => nrm_vphs_c_1,
			nrm_vbus_x_1     => nrm_vbus_x_1,
			nrm_vphs_n_1     => nrm_vphs_n_1,
			--
			nrm_sync_1       => nrm_sync_1,

			-- filtered values
			--
			flt_iphs_a_1     => flt_iphs_a_1,
			flt_iphs_b_1     => flt_iphs_b_1,
			flt_iphs_c_1     => flt_iphs_c_1,
			flt_ibus_x_1     => flt_ibus_x_1,
			flt_vphs_a_1     => flt_vphs_a_1,
			flt_vphs_b_1     => flt_vphs_b_1,
			flt_vphs_c_1     => flt_vphs_c_1,
			flt_vbus_x_1     => flt_vbus_x_1,
			flt_vphs_n_1     => flt_vphs_n_1,
			--
			flt_sync_1       => flt_sync_1,

			-- current overflow
			--
			ovi_iphs_a_1     => ovi_iphs_a_1,
			ovi_iphs_b_1     => ovi_iphs_b_1,
			ovi_iphs_c_1     => ovi_iphs_c_1,
			ovi_ibus_x_1     => ovi_ibus_x_1,

			-- Is overflow
			ismodover_1      => ismodover_1,
			
			-- 1=generator_1, 0=motor
			generator_1      => generator_1,
			
			-- modulator type
			modtype_1        => modtype_1,
			
			-- modulator levels
			modlevels_1      => modlevels_1,
			
			-- motor is 3-phases (0=bipolar)
			is3phases_1      => is3phases_1,
			
			-- pwm 3-phases outputs
			pwm2p_l2c1w1_1   => pwm2p_l2c1w1_1,
			pwm2p_l2c2w1_1   => pwm2p_l2c2w1_1,
			pwm2p_l2c1w2_1   => pwm2p_l2c1w2_1,
			pwm2p_l2c2w2_1   => pwm2p_l2c2w2_1,
			pwm2p_l2sync_1   => pwm2p_l2sync_1,
			
			-- pwm 2-phases outputs
			pwm3p_l2cxw1_1   => pwm3p_l2cxw1_1,
			pwm3p_l2cxw2_1   => pwm3p_l2cxw2_1,
			pwm3p_l2sync_1   => pwm3p_l2sync_1,
			
			-- rpfm 3-ph two levels outputs
			rpfm3p_l2cxw1_1  => rpfm3p_l2cxw1_1,
			rpfm3p_l2cxw2_1  => rpfm3p_l2cxw2_1,
			rpfm3p_l2sync_1  => rpfm3p_l2sync_1,
			
			-- rpfm 3-ph three levels outputs
			rpfm3p_l3c1w1_1  => rpfm3p_l3c1w1_1,
			rpfm3p_l3c2w1_1  => rpfm3p_l3c2w1_1,
			rpfm3p_l3c3w1_1  => rpfm3p_l3c3w1_1,
			rpfm3p_l3c1w2_1  => rpfm3p_l3c1w2_1,
			rpfm3p_l3c2w2_1  => rpfm3p_l3c2w2_1,
			rpfm3p_l3c3w2_1  => rpfm3p_l3c3w2_1,
			rpfm3p_l3sync_1  => rpfm3p_l3sync_1,
			
			-- dead time value
			deadtval_1       => deadtval_1,
			
			-- gate unit switches
			guswcnt_1        => guswcnt_1,
			
			-- in-failure
			hwfail_1         => hwfail_1,
			
			-- in-kill
			hwkill_1         => hwkill_1,
			
			-- coil driver
			coilgear_1       => coilgear_1,
			coilenab_1       => coilenab_1,
			
			-- system error
			syserr_1         => syserr_1,
			
			-- reset pending errors
			reseterr_1       => reseterr_1,
			
			-- zero current offset 
			zerocurr_1       => zerocurr_1,
			
			-- zero vbus offset 
			zerovbusx_1      => zerovbusx_1,

			-- zero vphs offset 
			zerovphsx_1      => zerovphsx_1,
			
			-- control probes
			pix_setval_1    => pix_setval_1,
			piy_setval_1    => piy_setval_1,
			
			pix_fbkval_1    => pix_fbkval_1,
			piy_fbkval_1    => piy_fbkval_1,
			
			pix_outval_1    => pix_outval_1,
			piy_outval_1    => piy_outval_1,
			
			-- Vs
			vsmodval_1       => vsmodval_1,
			vsangval_1       => vsangval_1,
			
			-- Is
			ismodval_1       => ismodval_1,
			isangval_1       => isangval_1,

			-- smo
			smo_bemf_p_1     => smo_bemf_p_1,
			smo_bemf_m_1     => smo_bemf_m_1,

			-- e-angles
			mrt_angle_1      => mrt_angle_1,
			enc_angle_1      => enc_angle_1,
			hls_angle_1      => hls_angle_1,
			rsv_angle_1      => rsv_angle_1,
			smo_angle_1      => smo_angle_1,

			-- Rotor angle
			rot_angle_1      => rot_angle_1,
			
			-- rot selector
			rot_select_1     => rot_select_1,
			
			-- actual speed
			spd_speed_1      => spd_speed_1,
			
			-- position
			pos_position_1   => pos_position_1,
			
			-- foc cycle time
			tsclocks_1       => tsclocks_1
		);

end Behavioral;
