-------------------------------------------------------------------------------------------
-- Copyright  2011-2017, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    11:52:27 10/31/2011 
-- Design Name: 
-- Module Name:    nmbrdsys - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity nmbrdsys is
	generic
	(
		C_FAMILY          : string := "zynq";
		C_CLOCK_PERIOD_PS : integer := 10000;
		C_RXLINK_SPACE    : integer := 0
	);
	port 
	( 
		clock  : in  std_logic;
		reset  : in  std_logic;
		
		--
		-- host interface
		--
		
		host_en    : in  std_logic;
		host_we    : in  std_logic;
		host_addr  : in  std_logic_vector(31 downto 0);
		host_din   : in  std_logic_vector(31 downto 0);
		host_dout  : out std_logic_vector(31 downto 0);
		host_ack   : out std_logic;
		-- ip ports
		-- coil drivers
		Motor_Coil      : out std_logic_vector (7 downto 0);
		-- serial link
		Cmd_Clock       : out std_logic;
		Cmd_Data        : out std_logic;
		Sts_Clock       : in  std_logic;
		Sts_Data        : in  std_logic;
		--
		-- external angle in
		ext_x_angle_0   : in  std_logic_vector(31 downto 0);
		ext_x_angle_1   : in  std_logic_vector(31 downto 0);
		--
		-- resolver in
		rsv_x_priexc_0  : in  std_logic_vector(17 downto 0);
		rsv_x_secsin_0  : in  std_logic_vector(17 downto 0);
		rsv_x_seccos_0  : in  std_logic_vector(17 downto 0);
		--
		rsv_x_priexc_1  : in  std_logic_vector(17 downto 0);
		rsv_x_secsin_1  : in  std_logic_vector(17 downto 0);
		rsv_x_seccos_1  : in  std_logic_vector(17 downto 0);
		--
		-- motor #0 control IP

		-- external angle
		ext_angle_0     : out std_logic_vector(31 downto 0);
		
		-- resolver
		rsv_priexc_0    : out std_logic_vector(17 downto 0);
		rsv_secsin_0    : out std_logic_vector(17 downto 0);
		rsv_seccos_0    : out std_logic_vector(17 downto 0);
		
		-- hall sensor
		hls_hallsig_0   : out std_logic_vector(5 downto 0);
		
		-- digital encoder
		enc_cha_0       : out std_logic;
		enc_chb_0       : out std_logic;
		enc_chi_0       : out std_logic;
		
		-- input data
		--
		acq_iphs_a_0    : out std_logic_vector(17 downto 0);
		acq_iphs_b_0    : out std_logic_vector(17 downto 0);
		acq_iphs_c_0    : out std_logic_vector(17 downto 0);
		acq_ibus_x_0    : out std_logic_vector(17 downto 0);
		acq_vphs_a_0    : out std_logic_vector(17 downto 0);
		acq_vphs_b_0    : out std_logic_vector(17 downto 0);
		acq_vphs_c_0    : out std_logic_vector(17 downto 0);
		acq_vbus_x_0    : out std_logic_vector(17 downto 0);
		acq_vphs_n_0    : out std_logic_vector(17 downto 0);
		--
		acq_sync_0      : out std_logic;
		
		-- output offset
		--
		ofs_iphs_a_0    : in  std_logic_vector(17 downto 0);
		ofs_iphs_b_0    : in  std_logic_vector(17 downto 0);
		ofs_iphs_c_0    : in  std_logic_vector(17 downto 0);
		ofs_ibus_x_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_a_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_b_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_c_0    : in  std_logic_vector(17 downto 0);
		ofs_vbus_x_0    : in  std_logic_vector(17 downto 0);
		ofs_vphs_n_0    : in  std_logic_vector(17 downto 0);

		-- normalized values (offset removed)
		--
		nrm_iphs_a_0    : in  std_logic_vector(17 downto 0);
		nrm_iphs_b_0    : in  std_logic_vector(17 downto 0);
		nrm_iphs_c_0    : in  std_logic_vector(17 downto 0);
		nrm_ibus_x_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_a_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_b_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_c_0    : in  std_logic_vector(17 downto 0);
		nrm_vbus_x_0    : in  std_logic_vector(17 downto 0);
		nrm_vphs_n_0    : in  std_logic_vector(17 downto 0);
		--
		nrm_sync_0      : in  std_logic;

		-- filtered values
		--
		flt_iphs_a_0    : in  std_logic_vector(17 downto 0);
		flt_iphs_b_0    : in  std_logic_vector(17 downto 0);
		flt_iphs_c_0    : in  std_logic_vector(17 downto 0);
		flt_ibus_x_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_a_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_b_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_c_0    : in  std_logic_vector(17 downto 0);
		flt_vbus_x_0    : in  std_logic_vector(17 downto 0);
		flt_vphs_n_0    : in  std_logic_vector(17 downto 0);
		--
		flt_sync_0      : in  std_logic;

		-- current overflow
		--
		ovi_iphs_a_0    : in  std_logic;
		ovi_iphs_b_0    : in  std_logic;
		ovi_iphs_c_0    : in  std_logic;
		ovi_ibus_x_0    : in  std_logic;

		-- Is overflow
		ismodover_0     : in  std_logic;
		
		-- 1=generator, 0=motor
		generator_0     : in  std_logic;
		
		-- modulator type
		modtype_0       : in  std_logic_vector(0 downto 0);
		
		-- modulator levels
		modlevels_0     : in  std_logic_vector(0 downto 0);
		
		-- motor is 3-phases (0=bipolar)
		is3phases_0     : in  std_logic;
		
		-- pwm 3-phases outputs
		pwm2p_l2c1w1_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w1_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c1w2_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w2_0  : in  std_logic_vector(1 downto 0);
		pwm2p_l2sync_0  : in  std_logic;
		
		-- pwm 2-phases outputs
		pwm3p_l2cxw1_0  : in  std_logic_vector(2 downto 0);
		pwm3p_l2cxw2_0  : in  std_logic_vector(2 downto 0);
		pwm3p_l2sync_0  : in  std_logic;
		
		-- rpfm 3-ph two levels outputs
		rpfm3p_l2cxw1_0 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2cxw2_0 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2sync_0 : in  std_logic;
		
		-- rpfm 3-ph three levels outputs
		rpfm3p_l3c1w1_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w1_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w1_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c1w2_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w2_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w2_0 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3sync_0 : in  std_logic;
		
		-- dead time value
		deadtval_0      : in  std_logic_vector(7 downto 0);
		
		-- gate unit switches
		guswcnt_0       : out std_logic_vector(31 downto 0);
		
		-- in-failure
		hwfail_0        : out std_logic;
		
		-- in-kill
		hwkill_0        : out std_logic;
		
		-- coil driver
		coilgear_0      : in  std_logic;
		coilenab_0      : in  std_logic;
		
		-- system error
		syserr_0        : in  std_logic;
		
		-- reset pending errors
		reseterr_0      : in  std_logic;
		
		-- zero current offset 
		zerocurr_0      : in  std_logic;
		
		-- zero vbus offset 
		zerovbusx_0     : in  std_logic;

		-- zero vphs offset 
		zerovphsx_0     : in  std_logic;
		
		-- control probes
		pix_setval_0    : in std_logic_vector(17 downto 0);
		piy_setval_0    : in std_logic_vector(17 downto 0);
		
		pix_fbkval_0    : in std_logic_vector(17 downto 0);
		piy_fbkval_0    : in std_logic_vector(17 downto 0);
		
		pix_outval_0    : in std_logic_vector(17 downto 0);
		piy_outval_0    : in std_logic_vector(17 downto 0);
		
		-- Vs
		vsmodval_0      : in  std_logic_vector(16 downto 0);
		vsangval_0      : in  std_logic_vector(31 downto 0);
		
		-- Is
		ismodval_0      : in  std_logic_vector(16 downto 0);
		isangval_0      : in  std_logic_vector(31 downto 0);

		-- smo
		smo_bemf_p_0    : in  std_logic_vector(31 downto 0);
		smo_bemf_m_0    : in  std_logic_vector(16 downto 0);
		
		-- e-angles
		mrt_angle_0     : in  std_logic_vector(31 downto 0);
		enc_angle_0     : in  std_logic_vector(31 downto 0);
		hls_angle_0     : in  std_logic_vector(31 downto 0);
		rsv_angle_0     : in  std_logic_vector(31 downto 0);
		smo_angle_0     : in  std_logic_vector(31 downto 0);
		
		-- Rotor angle
		rot_angle_0     : in  std_logic_vector(31 downto 0);
		
		-- rot selector
		rot_select_0    : in  std_logic_vector( 2 downto 0);
		
		-- actual speed
		spd_speed_0     : in  std_logic_vector(31 downto 0);
		
		-- position
		pos_position_0  : in  std_logic_vector(31 downto 0);
		
		-- foc cycle time
		tsclocks_0      : in  std_logic_vector(11 downto 0);
		
		-- motor #1 control IP

		-- external angle
		ext_angle_1     : out std_logic_vector(31 downto 0);
		
		-- resolver
		rsv_priexc_1    : out std_logic_vector(17 downto 0);
		rsv_secsin_1    : out std_logic_vector(17 downto 0);
		rsv_seccos_1    : out std_logic_vector(17 downto 0);
		
		-- hall sensor
		hls_hallsig_1   : out std_logic_vector(5 downto 0);
		
		-- digital encoder
		enc_cha_1       : out std_logic;
		enc_chb_1       : out std_logic;
		enc_chi_1       : out std_logic;
		
		-- input data
		--
		acq_iphs_a_1    : out std_logic_vector(17 downto 0);
		acq_iphs_b_1    : out std_logic_vector(17 downto 0);
		acq_iphs_c_1    : out std_logic_vector(17 downto 0);
		acq_ibus_x_1    : out std_logic_vector(17 downto 0);
		acq_vphs_a_1    : out std_logic_vector(17 downto 0);
		acq_vphs_b_1    : out std_logic_vector(17 downto 0);
		acq_vphs_c_1    : out std_logic_vector(17 downto 0);
		acq_vbus_x_1    : out std_logic_vector(17 downto 0);
		acq_vphs_n_1    : out std_logic_vector(17 downto 0);
		--
		acq_sync_1      : out std_logic;
		
		-- output offset
		--
		ofs_iphs_a_1    : in  std_logic_vector(17 downto 0);
		ofs_iphs_b_1    : in  std_logic_vector(17 downto 0);
		ofs_iphs_c_1    : in  std_logic_vector(17 downto 0);
		ofs_ibus_x_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_a_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_b_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_c_1    : in  std_logic_vector(17 downto 0);
		ofs_vbus_x_1    : in  std_logic_vector(17 downto 0);
		ofs_vphs_n_1    : in  std_logic_vector(17 downto 0);

		-- normalized values (offset removed)
		--
		nrm_iphs_a_1    : in  std_logic_vector(17 downto 0);
		nrm_iphs_b_1    : in  std_logic_vector(17 downto 0);
		nrm_iphs_c_1    : in  std_logic_vector(17 downto 0);
		nrm_ibus_x_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_a_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_b_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_c_1    : in  std_logic_vector(17 downto 0);
		nrm_vbus_x_1    : in  std_logic_vector(17 downto 0);
		nrm_vphs_n_1    : in  std_logic_vector(17 downto 0);
		--
		nrm_sync_1      : in  std_logic;

		-- filtered values
		--
		flt_iphs_a_1    : in  std_logic_vector(17 downto 0);
		flt_iphs_b_1    : in  std_logic_vector(17 downto 0);
		flt_iphs_c_1    : in  std_logic_vector(17 downto 0);
		flt_ibus_x_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_a_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_b_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_c_1    : in  std_logic_vector(17 downto 0);
		flt_vbus_x_1    : in  std_logic_vector(17 downto 0);
		flt_vphs_n_1    : in  std_logic_vector(17 downto 0);
		--
		flt_sync_1      : in  std_logic;

		-- current overflow
		--
		ovi_iphs_a_1    : in  std_logic;
		ovi_iphs_b_1    : in  std_logic;
		ovi_iphs_c_1    : in  std_logic;
		ovi_ibus_x_1    : in  std_logic;

		-- Is overflow
		ismodover_1     : in  std_logic;
		
		-- 1=generator, 0=motor
		generator_1     : in  std_logic;
		
		-- modulator type
		modtype_1       : in  std_logic_vector(0 downto 0);
		
		-- modulator levels
		modlevels_1     : in  std_logic_vector(0 downto 0);
		
		-- motor is 3-phases (0=bipolar)
		is3phases_1     : in  std_logic;
		
		-- pwm 3-phases outputs
		pwm2p_l2c1w1_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w1_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c1w2_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w2_1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2sync_1  : in  std_logic;
		
		-- pwm 2-phases outputs
		pwm3p_l2cxw1_1  : in  std_logic_vector(2 downto 0);
		pwm3p_l2cxw2_1  : in  std_logic_vector(2 downto 0);
		pwm3p_l2sync_1  : in  std_logic;
		
		-- rpfm 3-ph two levels outputs
		rpfm3p_l2cxw1_1 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2cxw2_1 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2sync_1 : in  std_logic;
		
		-- rpfm 3-ph three levels outputs
		rpfm3p_l3c1w1_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w1_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w1_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c1w2_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w2_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w2_1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3sync_1 : in  std_logic;
		
		-- dead time value
		deadtval_1      : in  std_logic_vector(7 downto 0);
		
		-- gate unit switches
		guswcnt_1       : out std_logic_vector(31 downto 0);
		
		-- in-failure
		hwfail_1        : out std_logic;
		
		-- in-kill
		hwkill_1        : out std_logic;
		
		-- coil driver
		coilgear_1      : in  std_logic;
		coilenab_1      : in  std_logic;
		
		-- system error
		syserr_1        : in  std_logic;
		
		-- reset pending errors
		reseterr_1      : in  std_logic;
		
		-- zero current offset 
		zerocurr_1      : in  std_logic;
		
		-- zero vbus offset 
		zerovbusx_1     : in  std_logic;

		-- zero vphs offset 
		zerovphsx_1     : in  std_logic;
		
		-- control probes
		pix_setval_1    : in std_logic_vector(17 downto 0);
		piy_setval_1    : in std_logic_vector(17 downto 0);
		
		pix_fbkval_1    : in std_logic_vector(17 downto 0);
		piy_fbkval_1    : in std_logic_vector(17 downto 0);
		
		pix_outval_1    : in std_logic_vector(17 downto 0);
		piy_outval_1    : in std_logic_vector(17 downto 0);

		-- Vs
		vsmodval_1      : in  std_logic_vector(16 downto 0);
		vsangval_1      : in  std_logic_vector(31 downto 0);
		
		-- Is
		ismodval_1      : in  std_logic_vector(16 downto 0);
		isangval_1      : in  std_logic_vector(31 downto 0);

		-- smo
		smo_bemf_p_1    : in  std_logic_vector(31 downto 0);
		smo_bemf_m_1    : in  std_logic_vector(16 downto 0);
		
		-- e-angles
		mrt_angle_1     : in  std_logic_vector(31 downto 0);
		enc_angle_1     : in  std_logic_vector(31 downto 0);
		hls_angle_1     : in  std_logic_vector(31 downto 0);
		rsv_angle_1     : in  std_logic_vector(31 downto 0);
		smo_angle_1     : in  std_logic_vector(31 downto 0);
		
		-- Rotor angle
		rot_angle_1     : in  std_logic_vector(31 downto 0);
		
		-- rot selector
		rot_select_1    : in  std_logic_vector( 2 downto 0);
		
		-- actual speed
		spd_speed_1     : in  std_logic_vector(31 downto 0);
		
		-- position
		pos_position_1  : in  std_logic_vector(31 downto 0);
		
		-- foc cycle time
		tsclocks_1      : in  std_logic_vector(11 downto 0)
	);
end nmbrdsys;

architecture Behavioral of nmbrdsys is

	-- serial link sender
	component sendvbkd
		generic
		(
			C_SYSCLOCK_PERIOD_PS : integer := 10000;
			C_TXCLOCK_PERIOD_PS  : integer := 100000;
			C_DATA_WIDTH         : integer := 32;
			C_HOLE_SIZE          : integer := 8
		);
		port
		( 
			clock : in  std_logic;
			reset : in  std_logic;
			start : in  std_logic;
			datas : in  std_logic_vector(0 to C_DATA_WIDTH-1);
			txdat : out std_logic;
			txclk : out std_logic
		);
	end component;
	
	-- serial link receiver
	component recvkd
		generic
		(
			C_SYSCLOCK_PERIOD_PS : integer := 10000;
			C_DATCLOCK_PERIOD_PS : integer := 25000;
			C_DATA_WIDTH         : integer := 32;
			C_HOLE_SIZE          : integer := 8;
			C_SPACE_VALUE        : integer := 0
		);
		port 
		(
			clock : in  std_logic;
			reset : in  std_logic;
			rxdat : in  std_logic;
			rxclk : in  std_logic;
			datar : out std_logic_vector(0 to C_DATA_WIDTH-1);
			datac : out std_logic
		);
	end component;

	-- re-sync serial link & current data
	component acqsync
		generic
		(
			C_SYSCLOCK_PERIOD_PS : integer := 20000;
			C_ACQUI_PERIOD_PS    : integer := 3200000
		);
		port 
		(
			clock   : in  std_logic;
			reset   : in  std_logic;
			datsync : in  std_logic;
			engsync : out std_logic
		);
	end component;
	
	-- event counter
	component evxcount
		generic
		(
			C_DATA_WIDTH  : integer := 16;
			C_EVENT_WIDTH : integer := 8;
			C_COUNT_MODE  : integer := 0
		);
		port
		(
			clock   : in  std_logic;
			reset   : in  std_logic;
			sigevt  : in  std_logic_vector(C_EVENT_WIDTH-1 downto 0);
			counter : out std_logic_vector(C_DATA_WIDTH-1 downto 0)
		);
	end component;
	
	
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	-- modulator type
	constant MODTYPE_PWM  : std_logic_vector(0 downto 0) := "0";
	constant MODTYPE_RPFM : std_logic_vector(0 downto 0) := "1";
	
	-- modulator levels
	constant MODLEVELS_2  : std_logic_vector(0 downto 0) := "0";
	constant MODLEVELS_3  : std_logic_vector(0 downto 0) := "1";
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------

	
  -- Command Register
	signal m_cmd_sreg : std_logic_vector(0 to 31);
	signal m_cmd_reg  : std_logic_vector(31 downto 0);
	signal m_cmd_out  : std_logic_vector(31 downto 0);
  
	constant DEF_CMD_TOK_VAL    : integer := 31;
	constant DEF_CMD_DNA_READ   : integer := 30;
	constant DEF_CMD_DNA_SHIFT  : integer := 29;
	constant DEF_CMD_DNA_CLK    : integer := 28;
	constant DEF_CMD_RES_27     : integer := 27;
	constant DEF_CMD_RES_26     : integer := 26;
	constant DEF_CMD_RES_25     : integer := 25;
	constant DEF_CMD_OUT_ENA    : integer := 24;
	constant DEF_CMD_MT7_ENA    : integer := 23;
	constant DEF_CMD_MT6_ENA    : integer := 22;
	constant DEF_CMD_MT5_ENA    : integer := 21;
	constant DEF_CMD_MT4_ENA    : integer := 20;
	constant DEF_CMD_MT3_ENA    : integer := 19;
	constant DEF_CMD_MT2_ENA    : integer := 18;
	constant DEF_CMD_MT1_ENA    : integer := 17;
	constant DEF_CMD_MT0_ENA    : integer := 16;
	constant DEF_CMD_LED7       : integer := 15;
	constant DEF_CMD_LED6       : integer := 14;
	constant DEF_CMD_LED5       : integer := 13;
	constant DEF_CMD_LED4       : integer := 12;
	constant DEF_CMD_LED3       : integer := 11;
	constant DEF_CMD_LED2       : integer := 10;
	constant DEF_CMD_LED1       : integer :=  9;
	constant DEF_CMD_LED0       : integer :=  8;
	constant DEF_CMD_DOUT7      : integer :=  7;
	constant DEF_CMD_DOUT6      : integer :=  6;
	constant DEF_CMD_DOUT5      : integer :=  5;
	constant DEF_CMD_DOUT4      : integer :=  4;
	constant DEF_CMD_DOUT3      : integer :=  3;
	constant DEF_CMD_DOUT2      : integer :=  2;
	constant DEF_CMD_DOUT1      : integer :=  1;
	constant DEF_CMD_DOUT0      : integer :=  0;


	-- Status Register  
	constant STATUS_REGISTER_SIZE : integer := 80;
	signal m_sts_sreg   : std_logic_vector(0 to STATUS_REGISTER_SIZE-1);
	signal m_sts_pdat   : std_logic_vector(STATUS_REGISTER_SIZE-1 downto 0);
	signal m_sts_phbt   : std_logic;
	signal m_sts_ohbt   : std_logic;
	signal m_sts_ready  : std_logic;

	constant DEF_STS_REG_TOK_FLG   : integer := 79;
	constant DEF_STS_REG_DNA_DATA  : integer := 78;
	constant DEF_STS_REG_MCHB1     : integer := 77;
	constant DEF_STS_REG_MCHA1     : integer := 76;
	constant DEF_STS_REG_MIDX1     : integer := 75;
	constant DEF_STS_REG_MCHB0     : integer := 74;
	constant DEF_STS_REG_MCHA0     : integer := 73;
	constant DEF_STS_REG_MIDX0     : integer := 72;
	constant DEF_STS_REG_DINP7     : integer := 71;
	constant DEF_STS_REG_DINP6     : integer := 70;
	constant DEF_STS_REG_DINP5     : integer := 69;
	constant DEF_STS_REG_DINP4     : integer := 68;
	constant DEF_STS_REG_DINP3     : integer := 67;
	constant DEF_STS_REG_DINP2     : integer := 66;
	constant DEF_STS_REG_DINP1     : integer := 65;
	constant DEF_STS_REG_DINP0     : integer := 64;
	constant DEF_STS_REG_ADVCNT    : integer := 63;
	constant DEF_STS_REG_ADICNT    : integer := 62;
	constant DEF_STS_REG_RES_13    : integer := 61;
	constant DEF_STS_REG_RES_12    : integer := 60;
	constant DEF_STS_REG_RES_11    : integer := 59;
	constant DEF_STS_REG_RES_10    : integer := 58;

	constant DEF_STS_REG_ADPWR9    : integer := 57;
	constant DEF_STS_REG_ADPWR8    : integer := 56;
	constant DEF_STS_REG_ADPWR7    : integer := 55;
	constant DEF_STS_REG_ADPWR6    : integer := 54;
	constant DEF_STS_REG_ADPWR5    : integer := 53;
	constant DEF_STS_REG_ADPWR4    : integer := 52;
	constant DEF_STS_REG_ADPWR3    : integer := 51;
	constant DEF_STS_REG_ADPWR2    : integer := 50;
	constant DEF_STS_REG_ADPWR1    : integer := 49;
	constant DEF_STS_REG_ADPWR0    : integer := 48;

	constant DEF_STS_REG_ADC1M1B11 : integer := 47;
	constant DEF_STS_REG_ADC1M1B10 : integer := 46;
	constant DEF_STS_REG_ADC1M1B9  : integer := 45;
	constant DEF_STS_REG_ADC1M1B8  : integer := 44;
	constant DEF_STS_REG_ADC1M1B7  : integer := 43;
	constant DEF_STS_REG_ADC1M1B6  : integer := 42;
	constant DEF_STS_REG_ADC1M1B5  : integer := 41;
	constant DEF_STS_REG_ADC1M1B4  : integer := 40;
	constant DEF_STS_REG_ADC1M1B3  : integer := 39;
	constant DEF_STS_REG_ADC1M1B2  : integer := 38;
	constant DEF_STS_REG_ADC1M1B1  : integer := 37;
	constant DEF_STS_REG_ADC1M1B0  : integer := 36;

	constant DEF_STS_REG_ADC0M1B11 : integer := 35;
	constant DEF_STS_REG_ADC0M1B10 : integer := 34;
	constant DEF_STS_REG_ADC0M1B9  : integer := 33;
	constant DEF_STS_REG_ADC0M1B8  : integer := 32;
	constant DEF_STS_REG_ADC0M1B7  : integer := 31;
	constant DEF_STS_REG_ADC0M1B6  : integer := 30;
	constant DEF_STS_REG_ADC0M1B5  : integer := 29;
	constant DEF_STS_REG_ADC0M1B4  : integer := 28;
	constant DEF_STS_REG_ADC0M1B3  : integer := 27;
	constant DEF_STS_REG_ADC0M1B2  : integer := 26;
	constant DEF_STS_REG_ADC0M1B1  : integer := 25;
	constant DEF_STS_REG_ADC0M1B0  : integer := 24;

	constant DEF_STS_REG_ADC1M0B11 : integer := 23;
	constant DEF_STS_REG_ADC1M0B10 : integer := 22;
	constant DEF_STS_REG_ADC1M0B9  : integer := 21;
	constant DEF_STS_REG_ADC1M0B8  : integer := 20;
	constant DEF_STS_REG_ADC1M0B7  : integer := 19;
	constant DEF_STS_REG_ADC1M0B6  : integer := 18;
	constant DEF_STS_REG_ADC1M0B5  : integer := 17;
	constant DEF_STS_REG_ADC1M0B4  : integer := 16;
	constant DEF_STS_REG_ADC1M0B3  : integer := 15;
	constant DEF_STS_REG_ADC1M0B2  : integer := 14;
	constant DEF_STS_REG_ADC1M0B1  : integer := 13;
	constant DEF_STS_REG_ADC1M0B0  : integer := 12;

	constant DEF_STS_REG_ADC0M0B11 : integer := 11;
	constant DEF_STS_REG_ADC0M0B10 : integer := 10;
	constant DEF_STS_REG_ADC0M0B9  : integer :=  9;
	constant DEF_STS_REG_ADC0M0B8  : integer :=  8;
	constant DEF_STS_REG_ADC0M0B7  : integer :=  7;
	constant DEF_STS_REG_ADC0M0B6  : integer :=  6;
	constant DEF_STS_REG_ADC0M0B5  : integer :=  5;
	constant DEF_STS_REG_ADC0M0B4  : integer :=  4;
	constant DEF_STS_REG_ADC0M0B3  : integer :=  3;
	constant DEF_STS_REG_ADC0M0B2  : integer :=  2;
	constant DEF_STS_REG_ADC0M0B1  : integer :=  1;
	constant DEF_STS_REG_ADC0M0B0  : integer :=  0;

	-- A/D processing

	alias sts_advcnt   is m_sts_pdat(DEF_STS_REG_ADVCNT);
	alias sts_adicnt   is m_sts_pdat(DEF_STS_REG_ADICNT);

	-- old V/I A/D acquisition counter
	signal m_sts_ovcn   : std_logic;
	signal m_sts_oicn   : std_logic;

	-- new V/I A/D acquisition samples
	signal m_sts_nvcs   : std_logic;
	signal m_sts_nics   : std_logic;

	---------------------------------------------------------------------------

	-- current samples time
	constant C_ACQUI_PERIOD_PS : integer := 3200000;
	
	---------------------------------------------------------------------------
	-- Inverter Values indexes
	
	constant C_IVD_IPHS_A0 : integer := 0;
	constant C_IVD_IPHS_B0 : integer := 1;
	constant C_IVD_VBUS_X0 : integer := 2;
	constant C_IVD_IPHS_A1 : integer := 3;
	constant C_IVD_IPHS_B1 : integer := 4;
	constant C_IVD_VBUS_X1 : integer := 5;
	constant C_IVD_NUM     : integer := 6;
		
	---------------------------------------------------------------------------
	-- ACQ (acquisition)
	
	signal invdat_sdat  : std_logic_vector(C_IVD_NUM-1 downto 0);
	
	constant C_INVERTER_SD_WIDTH : integer := 12;
	
	type   invdat_acq_t is array(0 to C_IVD_NUM-1) of std_logic_vector(C_INVERTER_SD_WIDTH-1 downto 0);
	signal invdat_acq : invdat_acq_t := (others => (others => '0'));
	signal invdat_eoc : std_logic;
	
	---------------------------------------------------------------------------
	-- RAW (raw: final size)
	
	constant C_INVDAT_ACQ2RAW_PAD : std_logic_vector(18-C_INVERTER_SD_WIDTH-1 downto 0) := (others => '0');
	
	type   invdat_raw_t is array(0 to C_IVD_NUM-1) of std_logic_vector(17 downto 0);
	signal invdat_raw : invdat_raw_t := (others => (others => '0'));
	
	---------------------------------------------------------------------------
	-- BIP (unipolar to bipolar)
	
	type   invdat_bip_t is array(0 to C_IVD_NUM-1) of std_logic_vector(17 downto 0);
	signal invdat_bip : invdat_bip_t := (others => (others => '0'));
	
	---------------------------------------------------------------------------
	
	-- common motor coil
	
	signal coiljmux_0   : std_logic_vector(3 downto 0);
	signal coiljmux_1   : std_logic_vector(3 downto 0);

	signal coildrv_x    : std_logic_vector(7 downto 0);
	
	-- gate unit switches counters
	
	-- event captures
	signal guevsrc_0    : std_logic_vector(3 downto 0);
	signal guevsrc_1    : std_logic_vector(3 downto 0);
	
	-- switches counter per leg
	signal guswcnt_leg_0 : std_logic_vector(30 downto 0);
	signal guswcnt_leg_1 : std_logic_vector(30 downto 0);
	
	-- registri
	
	-- command register
	signal command_reg : std_logic_vector(31 downto 0);
	
	-- status register
	signal status_reg  : std_logic_vector(31 downto 0);
	
	-- coil control register
	signal   coil_ctrl : std_logic_vector(15 downto 0);
	constant DEF_COIL_CTRL_ENA_OFS : integer := 0;
	constant DEF_COIL_CTRL_VAL_OFS : integer := 8;

	---------------------------------------------------------------------------
	
	-- access delay
	signal host_en_dl1  : std_logic;
	
	---------------------------------------------------------------------------
	
	-- outputs
	signal host_dout_i  : std_logic_vector(31 downto 0);
	signal host_ack_i   : std_logic;
	
	---------------------------------------------------------------------------
	
	-- bus2reg
	function loadreg(s : std_logic_vector) return std_logic_vector is
		variable r : std_logic_vector(31 downto 0);
	begin
		r := (others => '0');
		r(s'range) := s;
		return r;
	end function;

	-- reg2bus
	procedure storereg(signal d : out std_logic_vector; signal s : in std_logic_vector) is
	begin
		d <= s(d'range);
	end procedure;

	---------------------------------------------------------------------------
  
	constant RA_ip_ident     : integer := (4*0);
	constant RA_command      : integer := (4*1);
	constant RA_status       : integer := (4*2);
	constant RA_coil_ctrl    : integer := (4*3);
	
	---------------------------------------------------------------------------
  
	constant C_DECAD_WIDTH : integer := 2;
	
	---------------------------------------------------------------------------
	
	function adrtoint(ad : std_logic_vector) return integer is
		variable ar : std_logic_vector(C_DECAD_WIDTH+1 downto 0);
	begin
		ar := ad(ar'left downto 2) & "00";
		return to_integer(unsigned(ar));
	end function;
	
	---------------------------------------------------------------------------
	
	-- ip identification
	constant ip_id_code  : integer := 2;
	constant ip_id_majv  : integer := 1;
	constant ip_id_minv  : integer := 1;
	constant ip_id_hscr  : integer := 97;
	
	constant ip_ident_k : std_logic_vector(31 downto 0) :=
		std_logic_vector(to_unsigned(ip_id_code,8)) &
		std_logic_vector(to_unsigned(ip_id_majv,8)) &
		std_logic_vector(to_unsigned(ip_id_minv,8)) &
		std_logic_vector(to_unsigned(ip_id_hscr,8));
		
	-- outputs

	-- coil drivers
	signal Motor_Coil_i    : std_logic_vector(7 downto 0);

	-- serial link
	signal Cmd_Clock_i     : std_logic;
	signal Cmd_Data_i      : std_logic;

	---------------------------------------------------------------------------
	
	-- motor #0 control
	
	-- external angle
	signal ext_angle_0_i   : std_logic_vector(31 downto 0);

	-- resolver            
	signal rsv_priexc_0_i  : std_logic_vector(17 downto 0);
	signal rsv_secsin_0_i  : std_logic_vector(17 downto 0);
	signal rsv_seccos_0_i  : std_logic_vector(17 downto 0);

	-- hall sensor         
	signal hls_hallsig_0_i : std_logic_vector(5 downto 0);

	-- digital encoder     
	signal enc_cha_0_i     : std_logic;
	signal enc_chb_0_i     : std_logic;
	signal enc_chi_0_i     : std_logic;

	-- input data          
	--                     
	signal acq_iphs_a_0_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_b_0_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_c_0_i  : std_logic_vector(17 downto 0);
	signal acq_ibus_x_0_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_a_0_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_b_0_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_c_0_i  : std_logic_vector(17 downto 0);
	signal acq_vbus_x_0_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_n_0_i  : std_logic_vector(17 downto 0);
	--                     
	signal acq_sync_0_i    : std_logic;

	-- gate unit switches   
	signal guswcnt_0_i     : std_logic_vector(31 downto 0);

	-- in-failure          
	signal hwfail_0_i      : std_logic;

	-- in-kill             
	signal hwkill_0_i      : std_logic;

	---------------------------------------------------------------------------
	
	-- motor #1 control
	
	-- external angle
	signal ext_angle_1_i   : std_logic_vector(31 downto 0);

	-- resolver            
	signal rsv_priexc_1_i  : std_logic_vector(17 downto 0);
	signal rsv_secsin_1_i  : std_logic_vector(17 downto 0);
	signal rsv_seccos_1_i  : std_logic_vector(17 downto 0);

	-- hall sensor         
	signal hls_hallsig_1_i : std_logic_vector(5 downto 0);

	-- digital encoder     
	signal enc_cha_1_i     : std_logic;
	signal enc_chb_1_i     : std_logic;
	signal enc_chi_1_i     : std_logic;

	-- input data          
	--                     
	signal acq_iphs_a_1_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_b_1_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_c_1_i  : std_logic_vector(17 downto 0);
	signal acq_ibus_x_1_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_a_1_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_b_1_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_c_1_i  : std_logic_vector(17 downto 0);
	signal acq_vbus_x_1_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_n_1_i  : std_logic_vector(17 downto 0);
	--                     
	signal acq_sync_1_i    : std_logic;

	-- gate unit switches   
	signal guswcnt_1_i     : std_logic_vector(31 downto 0);

	-- in-failure          
	signal hwfail_1_i      : std_logic;

	-- in-kill             
	signal hwkill_1_i      : std_logic;

	---------------------------------------------------------------------------

begin

	-- ip results
	host_dout <= host_dout_i;
	host_ack  <= host_ack_i;
	
	---------------------------------------------------------------------------

	-- output
	
	-- coil drivers
	Motor_Coil    <= Motor_Coil_i;

	-- serial link
	Cmd_Clock     <= Cmd_Clock_i;
	Cmd_Data      <= Cmd_Data_i;
	
	---------------------------------------------------------------------------
	
	-- motor #0 control
	
	-- external angle
	ext_angle_0    <=  ext_angle_0_i;
	
	-- resolver
	rsv_priexc_0   <=  rsv_priexc_0_i;
	rsv_secsin_0   <=  rsv_secsin_0_i;
	rsv_seccos_0   <=  rsv_seccos_0_i;
	
	-- hall sensor
	hls_hallsig_0  <=  hls_hallsig_0_i;
	
	-- digital encoder
	enc_cha_0      <=  enc_cha_0_i;
	enc_chb_0      <=  enc_chb_0_i;
	enc_chi_0      <=  enc_chi_0_i;
	
	-- input data
	--
	acq_iphs_a_0   <=  acq_iphs_a_0_i;
	acq_iphs_b_0   <=  acq_iphs_b_0_i;
	acq_iphs_c_0   <=  acq_iphs_c_0_i;
	acq_ibus_x_0   <=  acq_ibus_x_0_i;
	acq_vphs_a_0   <=  acq_vphs_a_0_i;
	acq_vphs_b_0   <=  acq_vphs_b_0_i;
	acq_vphs_c_0   <=  acq_vphs_c_0_i;
	acq_vbus_x_0   <=  acq_vbus_x_0_i;
	acq_vphs_n_0   <=  acq_vphs_n_0_i;
	--
	acq_sync_0     <=  acq_sync_0_i;
	
	-- gate unit switches
	guswcnt_0      <=  guswcnt_0_i;
	
	-- in-failure
	hwfail_0       <=  hwfail_0_i;
	
	-- in-kill
	hwkill_0       <=  hwkill_0_i;
	
	---------------------------------------------------------------------------
	
	-- motor #1 control
	
	-- external angle
	ext_angle_1    <=  ext_angle_1_i;
	
	-- resolver
	rsv_priexc_1   <=  rsv_priexc_1_i;
	rsv_secsin_1   <=  rsv_secsin_1_i;
	rsv_seccos_1   <=  rsv_seccos_1_i;
	
	-- hall sensor
	hls_hallsig_1  <=  hls_hallsig_1_i;
	
	-- digital encoder
	enc_cha_1      <=  enc_cha_1_i;
	enc_chb_1      <=  enc_chb_1_i;
	enc_chi_1      <=  enc_chi_1_i;
	
	-- input data
	--
	acq_iphs_a_1   <=  acq_iphs_a_1_i;
	acq_iphs_b_1   <=  acq_iphs_b_1_i;
	acq_iphs_c_1   <=  acq_iphs_c_1_i;
	acq_ibus_x_1   <=  acq_ibus_x_1_i;
	acq_vphs_a_1   <=  acq_vphs_a_1_i;
	acq_vphs_b_1   <=  acq_vphs_b_1_i;
	acq_vphs_c_1   <=  acq_vphs_c_1_i;
	acq_vbus_x_1   <=  acq_vbus_x_1_i;
	acq_vphs_n_1   <=  acq_vphs_n_1_i;
	--
	acq_sync_1     <=  acq_sync_1_i;
	
	-- gate unit switches
	guswcnt_1      <=  guswcnt_1_i;
	
	-- in-failure
	hwfail_1       <=  hwfail_1_i;
	
	-- in-kill
	hwkill_1       <=  hwkill_1_i;
	
	---------------------------------------------------------------------------

	-- serial link
	m_cmd_reg  <= command_reg;
	m_cmd_sreg <= m_cmd_out;

    -- command transmitter
	CMD_SENDER_INST: sendvbkd
		generic map
		(
			C_SYSCLOCK_PERIOD_PS => C_CLOCK_PERIOD_PS,
			C_TXCLOCK_PERIOD_PS  => 100000, -- speed is fixed to 10 Mhz
			C_DATA_WIDTH         => 32,
			C_HOLE_SIZE          => 8 -- idle time in bits. Fixed value
		)
		port map
		( 
			clock => clock,
			reset => reset,
			start => '1',
			datas => m_cmd_sreg,
			txdat => Cmd_Data_i,
			txclk => Cmd_Clock_i
		);
    
	------------------------------------------------------------------
	------------------------------------------------------------------
	------------------------------------------------------------------
	
	status_reg <= m_sts_pdat(DEF_STS_REG_TOK_FLG downto DEF_STS_REG_ADPWR0);
	
	m_sts_pdat <= m_sts_sreg;

	-- status receiver
	STS_RECEIVER_INST: recvkd
		generic map
		(
			C_SYSCLOCK_PERIOD_PS => C_CLOCK_PERIOD_PS,
			C_DATCLOCK_PERIOD_PS => 25000, -- speed if fixed to 40 Mhz
			C_DATA_WIDTH         => STATUS_REGISTER_SIZE,
			C_HOLE_SIZE          => 8,  -- idle time in bits. Fixed value
			C_SPACE_VALUE        => C_RXLINK_SPACE
		)
		port map
		(
			clock => clock,
			reset => reset,
			rxdat => Sts_Data,
			rxclk => Sts_Clock,
			datar => m_sts_sreg,
			datac => m_sts_phbt
		);
	
    -- netmot capt new tgm
    STS_SETUP_PROC: process(clock)
    begin
		if rising_edge(clock) then
			if reset = '1' then
				m_sts_ohbt  <= '0';
				m_sts_ready <= '0';
			else
				m_sts_ohbt <= m_sts_phbt;
				if m_sts_ohbt /= m_sts_phbt then
					m_sts_ready <= '1';
				else
					m_sts_ready <= '0';
				end if;
			end if;
		end if;
    end process;

    -- netmot capt new A/D values
    STS_ATODEV_PROC: process(clock)
    begin
		if rising_edge(clock) then
			if reset = '1' then
				m_sts_ovcn <= '0';
				m_sts_oicn <= '0';
				m_sts_nvcs <= '0';
				m_sts_nics <= '0';
			else
				m_sts_ovcn <= sts_advcnt;
				m_sts_oicn <= sts_adicnt;
				-- A/D voltage conversion counter chnge
				if m_sts_ovcn /= sts_advcnt then
					m_sts_nvcs <= '1';
				else
					m_sts_nvcs <= '0';
				end if;
				-- A/D current conversion counter chnge
				if m_sts_oicn /= sts_adicnt then
					m_sts_nics <= '1';
				else
					m_sts_nics <= '0';
				end if;
			end if;
		end if;
    end process;

	------------------------------------------------------------------
	------------------------------------------------------------------
	
	BLDCMDREG_PROC : process(clock)
		variable t_cmd : std_logic_vector(31 downto 0);
	begin
		if rising_edge(clock) then
			if reset = '1' then
				m_cmd_out <= (others => '0');
			else
				-- default command registe
				t_cmd := m_cmd_reg;
				--
				-- neutral motor #0 drivers
				if coil_ctrl(DEF_COIL_CTRL_ENA_OFS+3 downto DEF_COIL_CTRL_ENA_OFS+0) /= "1111" then
					if coilgear_0 = '0' then
						t_cmd(DEF_CMD_MT0_ENA) := '0';
						t_cmd(DEF_CMD_MT1_ENA) := '0';
						t_cmd(DEF_CMD_MT2_ENA) := '0';
						t_cmd(DEF_CMD_MT3_ENA) := '0';
					end if;
				end if;
				-- neutral motor #1 drivers
				if coil_ctrl(DEF_COIL_CTRL_ENA_OFS+7 downto DEF_COIL_CTRL_ENA_OFS+4) /= "1111" then
					if coilgear_1 = '0' then
						t_cmd(DEF_CMD_MT4_ENA) := '0';
						t_cmd(DEF_CMD_MT5_ENA) := '0';
						t_cmd(DEF_CMD_MT6_ENA) := '0';
						t_cmd(DEF_CMD_MT7_ENA) := '0';
					end if;
				end if;
				m_cmd_out <= t_cmd;
			end if;
		end if;
	end process;
	
	-------------------------------------------------------------------
	
	-- sync on current data
	SYNC_SLIK_INST : acqsync
		generic map
		(
			C_SYSCLOCK_PERIOD_PS => C_CLOCK_PERIOD_PS,
			C_ACQUI_PERIOD_PS    => C_ACQUI_PERIOD_PS
		)
		port map
		(
			clock   => clock,
			reset   => reset,
			datsync => m_sts_nics,
			engsync => invdat_eoc
		);
	
	-----------------------------------------------------------------------------------------
	
	-- M0 acq
	invdat_acq(C_IVD_IPHS_A0) <= m_sts_pdat(DEF_STS_REG_ADC0M0B11 downto DEF_STS_REG_ADC0M0B0);
	invdat_acq(C_IVD_IPHS_B0) <= m_sts_pdat(DEF_STS_REG_ADC1M0B11 downto DEF_STS_REG_ADC1M0B0);
	invdat_acq(C_IVD_VBUS_X0) <= m_sts_pdat(DEF_STS_REG_ADPWR9    downto DEF_STS_REG_ADPWR0  ) & "00";
	
	-- M1 acq
	invdat_acq(C_IVD_IPHS_A1) <= m_sts_pdat(DEF_STS_REG_ADC0M1B11 downto DEF_STS_REG_ADC0M1B0);
	invdat_acq(C_IVD_IPHS_B1) <= m_sts_pdat(DEF_STS_REG_ADC1M1B11 downto DEF_STS_REG_ADC1M1B0);
	invdat_acq(C_IVD_VBUS_X1) <= m_sts_pdat(DEF_STS_REG_ADPWR9    downto DEF_STS_REG_ADPWR0  ) & "00";
	
	-----------------------------------------------------------------------------------------
	
	-- final size
	INV_ACQ2RAW_GEN : for i in invdat_sdat'range generate
		invdat_raw(i) <= invdat_acq(i) & C_INVDAT_ACQ2RAW_PAD;
	end generate;
	
	-- unipolar to bipolar
	INV_RAW2BIP_GEN : for i in invdat_sdat'range generate
		invdat_bip(i) <= std_logic_vector(signed(invdat_raw(i)) - 2**17);
	end generate;
	
	-----------------------------------------------------------------------------------------

	-- M0 a/d
	acq_iphs_a_0_i <= invdat_bip(C_IVD_IPHS_A0);
	acq_iphs_b_0_i <= invdat_bip(C_IVD_IPHS_B0);
	acq_iphs_c_0_i <= (others => '0');
	acq_ibus_x_0_i <= (others => '0');
	acq_vphs_a_0_i <= (others => '0');
	acq_vphs_b_0_i <= (others => '0');
	acq_vphs_c_0_i <= (others => '0');
	acq_vbus_x_0_i <= invdat_bip(C_IVD_VBUS_X0);
	acq_vphs_n_0_i <= (others => '0');
	acq_sync_0_i   <= invdat_eoc;
	
	-----------------------------------------------------------------------------------------

	-- M1 a/d
	acq_iphs_a_1_i <= invdat_bip(C_IVD_IPHS_A1);
	acq_iphs_b_1_i <= invdat_bip(C_IVD_IPHS_B1);
	acq_iphs_c_1_i <= (others => '0');
	acq_ibus_x_1_i <= (others => '0');
	acq_vphs_a_1_i <= (others => '0');
	acq_vphs_b_1_i <= (others => '0');
	acq_vphs_c_1_i <= (others => '0');
	acq_vbus_x_1_i <= invdat_bip(C_IVD_VBUS_X1);
	acq_vphs_n_1_i <= (others => '0');
	acq_sync_1_i   <= invdat_eoc;
	
	-----------------------------------------------------------------------------------------
	
	-- M0 no failure/kill
	hwfail_0_i <= '0';
	hwkill_0_i <= '0';
	
	-- M1 no failure/kill
	hwfail_1_i <= '0';
	hwkill_1_i <= '0';
	
	-----------------------------------------------------------------------------------------
	
	-- external angle: 0/1
	ext_angle_0_i <= ext_x_angle_0;
	ext_angle_1_i <= ext_x_angle_1;
	
	-- M0
	rsv_priexc_0_i <= rsv_x_priexc_0;
	rsv_secsin_0_i <= rsv_x_secsin_0;
	rsv_seccos_0_i <= rsv_x_seccos_0;

	-- M1
	rsv_priexc_1_i <= rsv_x_priexc_1;
	rsv_secsin_1_i <= rsv_x_secsin_1;
	rsv_seccos_1_i <= rsv_x_seccos_1;

	-- M0 hall sns
	hls_hallsig_0_i(0) <= m_sts_pdat(DEF_STS_REG_MIDX0);
	hls_hallsig_0_i(1) <= m_sts_pdat(DEF_STS_REG_MCHA0);
	hls_hallsig_0_i(2) <= m_sts_pdat(DEF_STS_REG_MCHB0);
	hls_hallsig_0_i(5 downto 3) <= (others => '0');
	
	-- M1 hall sns
	hls_hallsig_1_i(0) <= m_sts_pdat(DEF_STS_REG_MIDX1);
	hls_hallsig_1_i(1) <= m_sts_pdat(DEF_STS_REG_MCHA1);
	hls_hallsig_1_i(2) <= m_sts_pdat(DEF_STS_REG_MCHB1);
	hls_hallsig_1_i(5 downto 3) <= (others => '0');
	
	enc_cha_0_i <= m_sts_pdat(DEF_STS_REG_MCHA0);
	enc_chb_0_i <= m_sts_pdat(DEF_STS_REG_MCHB0);
	enc_chi_0_i <= m_sts_pdat(DEF_STS_REG_MIDX0);
	
	enc_cha_1_i <= m_sts_pdat(DEF_STS_REG_MCHA1);
	enc_chb_1_i <= m_sts_pdat(DEF_STS_REG_MCHB1);
	enc_chi_1_i <= m_sts_pdat(DEF_STS_REG_MIDX1);
	
	-- mux pwm/rpfm

	-- MOTOR #0
	
	coiljmux_0(0) <= rpfm3p_l2cxw1_0(0) when modtype_0 = MODTYPE_RPFM and is3phases_0 = '1' else
					     pwm3p_l2cxw1_0(0)  when modtype_0 = MODTYPE_PWM  and is3phases_0 = '1' else
					     pwm2p_l2c1w1_0(0)  when modtype_0 = MODTYPE_PWM  and is3phases_0 = '0' else '0';
	
	coiljmux_0(1) <= rpfm3p_l2cxw1_0(2) when modtype_0 = MODTYPE_RPFM and is3phases_0 = '1' else
					     pwm3p_l2cxw1_0(2)  when modtype_0 = MODTYPE_PWM  and is3phases_0 = '1' else
					     pwm2p_l2c1w1_0(1)  when modtype_0 = MODTYPE_PWM  and is3phases_0 = '0' else '0';
	
	coiljmux_0(2) <= rpfm3p_l2cxw1_0(1) when modtype_0 = MODTYPE_RPFM and is3phases_0 = '1' else
					     pwm3p_l2cxw1_0(1)  when modtype_0 = MODTYPE_PWM  and is3phases_0 = '1' else
					     pwm2p_l2c2w1_0(0)  when modtype_0 = MODTYPE_PWM  and is3phases_0 = '0' else '0';
						  
	coiljmux_0(3) <= pwm2p_l2c2w1_0(1)  when modtype_0 = MODTYPE_PWM  and is3phases_0 = '0' else '0';
	
	
	-- MOTOR #1
	
	coiljmux_1(0) <= rpfm3p_l2cxw1_1(0) when modtype_1 = MODTYPE_RPFM and is3phases_1 = '1' else
					     pwm3p_l2cxw1_1(0)  when modtype_1 = MODTYPE_PWM  and is3phases_1 = '1' else
					     pwm2p_l2c1w1_1(0)  when modtype_1 = MODTYPE_PWM  and is3phases_1 = '0' else '0';
	
	coiljmux_1(1) <= rpfm3p_l2cxw1_1(2) when modtype_1 = MODTYPE_RPFM and is3phases_1 = '1' else
					     pwm3p_l2cxw1_1(2)  when modtype_1 = MODTYPE_PWM  and is3phases_1 = '1' else
					     pwm2p_l2c1w1_1(1)  when modtype_1 = MODTYPE_PWM  and is3phases_1 = '0' else '0';
	
	coiljmux_1(2) <= rpfm3p_l2cxw1_1(1) when modtype_1 = MODTYPE_RPFM and is3phases_1 = '1' else
					     pwm3p_l2cxw1_1(1)  when modtype_1 = MODTYPE_PWM  and is3phases_1 = '1' else
					     pwm2p_l2c2w1_1(0)  when modtype_1 = MODTYPE_PWM  and is3phases_1 = '0' else '0';
						  
	coiljmux_1(3) <= pwm2p_l2c2w1_1(1)  when modtype_1 = MODTYPE_PWM  and is3phases_1 = '0' else '0';
	

	-- MOTOR #0
	
	coildrv_x(0)  <= coiljmux_0(0);
	coildrv_x(1)  <= coiljmux_0(1);
	coildrv_x(2)  <= coiljmux_0(2);
	coildrv_x(3)  <= coiljmux_0(3);

	-- MOTOR #1
	
	coildrv_x(4) <= coiljmux_1(0);
	coildrv_x(5) <= coiljmux_1(1);
	coildrv_x(6) <= coiljmux_1(2);
	coildrv_x(7) <= coiljmux_1(3);
		
	-- mux coil control
	coilmux_gen : for i in 0 to 7 generate
		Motor_Coil_i(i) <= coildrv_x(i) when coil_ctrl(i+DEF_COIL_CTRL_ENA_OFS) = '0' else coil_ctrl(i+DEF_COIL_CTRL_VAL_OFS);
	end generate;


	-------------------------
	
	-- GU swtch count M0
	
	guevsrc_0(0) <= Motor_Coil_i(0);
	guevsrc_0(1) <= Motor_Coil_i(1);
	guevsrc_0(2) <= Motor_Coil_i(2);
	guevsrc_0(3) <= Motor_Coil_i(3);

	-- event counter
	GUSWCNT_M0 : evxcount
		generic map
		(
			C_DATA_WIDTH  => 31,
			C_EVENT_WIDTH => 4,
			C_COUNT_MODE  => 0
		)
		port map
		(
			clock   => clock,
			reset   => reset,
			sigevt  => guevsrc_0,
			counter => guswcnt_leg_0
		);
		
	guswcnt_0_i <= guswcnt_leg_0 & "0";

	-------------------------
	
	-- GU swtch count M1
	
	guevsrc_1(0) <= Motor_Coil_i(4);
	guevsrc_1(1) <= Motor_Coil_i(5);
	guevsrc_1(2) <= Motor_Coil_i(6);
	guevsrc_1(3) <= Motor_Coil_i(7);

	-- event counter
	GUSWCNT_M1 : evxcount
		generic map
		(
			C_DATA_WIDTH  => 31,
			C_EVENT_WIDTH => 4,
			C_COUNT_MODE  => 0
		)
		port map
		(
			clock   => clock,
			reset   => reset,
			sigevt  => guevsrc_1,
			counter => guswcnt_leg_1
		);
		
	guswcnt_1_i <= guswcnt_leg_1 & "0";
	
	---------------------------------------------------------------------------
	
	-- implement slave model software accessible register(s)
	SLAVE_WRITE_PROC : process(clock)
	begin
		if rising_edge(clock) then
			if reset = '1' then
				command_reg  <= (others => '0');
				coil_ctrl    <= (others => '0');
			elsif host_we = '1' and host_en = '1' then
				case adrtoint(host_addr) is
					when RA_ip_ident  => null;
					when RA_command   => storereg(command_reg ,host_din);
					when RA_status    => null;
					when RA_coil_ctrl => storereg(coil_ctrl   ,host_din);
					when others       => null;
				end case;
			end if;
		end if;
	end process;

	-- implement slave model software accessible register(s) read mux
	SLAVE_READ_PROC : process(clock)
		variable v : std_logic_vector(31 downto 0);
	begin
		if rising_edge(clock) then
			v := (others => '0');
			case adrtoint(host_addr) is
				when RA_ip_ident  => v := loadreg(ip_ident_k  );
				when RA_command   => v := loadreg(command_reg );
				when RA_status    => v := loadreg(status_reg  );
				when RA_coil_ctrl => v := loadreg(coil_ctrl   );
				when others       => null;
			end case;
			host_dout_i <= v;
		end if;
	end process;
	
	-- data transfer ack
	host_ack_i <= host_we or host_en_dl1;
	
	-- host access delay
	HOSTDLY_PROC : process(clock)
	begin
		if rising_edge(clock) then
			host_en_dl1 <= host_en;
		end if;
	end process;

end Behavioral;
