-------------------------------------------------------------------------------------------
-- Copyright  2011-2016, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    11:22:28 04/29/2015 
-- Design Name: 
-- Module Name:    gudtmfx - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity gudtmfx is
	port 
	(
		clock    : in  std_logic;
		reset    : in  std_logic;
		--
		deadtval : in  std_logic_vector(7 downto 0);
		command  : in  std_logic;
		enablehi : in  std_logic;
		enablelo : in  std_logic;
		drivehi  : out std_logic;
		drivelo  : out std_logic
	);
end gudtmfx;

architecture Behavioral of gudtmfx is

	-- dead time eval
	
	signal oldcmd     : std_logic;
	signal chgcmd     : std_logic;
	signal cntsteady  : std_logic_vector(deadtval'range);
	signal dtready    : std_logic;
	
	-- output

	signal drivehi_i  : std_logic;
	signal drivelo_i  : std_logic;

begin

	drivehi <= drivehi_i;
	drivelo <= drivelo_i;
        
	-- deadtime

	dtready <= '1' when cntsteady = deadtval else '0';
	chgcmd  <= '1' when oldcmd /= command else '0';

	DT_PROC : process(clock)
	begin
		if rising_edge(clock) then
			oldcmd <= command;
			if chgcmd = '1' or reset = '1' then
				cntsteady <= (others => '0');
				drivehi_i <= '0';
				drivelo_i <= '0';
			elsif dtready = '0' then
				cntsteady <= cntsteady + 1;
			else
				drivehi_i <= enablehi and command;
				drivelo_i <= enablelo and (not command);
			end if;
		end if;
	end process;
	
end Behavioral;
