-------------------------------------------------------------------------------------------
-- Copyright  2011-2019, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    13:15:25 12/28/2015 
-- Design Name: 
-- Module Name:    tec53sys - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity tec53sys is
	generic
	(
		C_FAMILY            : string  := "zynq";
		C_HOST_CLK_FREQ_HZ  : integer := 100_000_000;
		--
		C_SDADC_CLK_FREQ_HZ : integer := 20_000_000;
		C_INV_ACQTIME_NS    : integer := 3200;
		C_SD_CLK_NOT        : integer := 0
	);
	port 
	(
		-- host interface
		host_clk      : in  std_logic;
		host_rst      : in  std_logic;
		host_en       : in  std_logic;
		host_we       : in  std_logic;
		host_addr     : in  std_logic_vector(31 downto 0);
		host_din      : in  std_logic_vector(31 downto 0);
		host_dout     : out std_logic_vector(31 downto 0);
		host_ack      : out std_logic;
		--
		-- source clock for A/D
		clk_sdadc     : in  std_logic;
		--
		-- board phy
		--
		-- Incremental encoder
		enc_a         : in  std_logic;
		enc_b         : in  std_logic;
		enc_i         : in  std_logic;
		--
		-- Gate unit control
		gu_cmd_h      : out std_logic_vector(2 downto 0);
		gu_cmd_l      : out std_logic_vector(2 downto 0);
		--
		-- Voltage & Current analog acquisiton
		adx_clk       : out std_logic;
		adx_dat       : in  std_logic_vector(3 downto 0);
		--
		
		-- motor control IP

		-- external angle
		ext_angle     : out   std_logic_vector(31 downto 0);
		
		-- resolver
		rsv_priexc    : out   std_logic_vector(17 downto 0);
		rsv_secsin    : out   std_logic_vector(17 downto 0);
		rsv_seccos    : out   std_logic_vector(17 downto 0);
		
		-- hall sensor
		hls_hallsig   : out   std_logic_vector(5 downto 0);
		
		-- digital encoder
		enc_cha       : out   std_logic;
		enc_chb       : out   std_logic;
		enc_chi       : out   std_logic;
		
		-- input data
		--
		acq_iphs_a    : out  std_logic_vector(17 downto 0);
		acq_iphs_b    : out  std_logic_vector(17 downto 0);
		acq_iphs_c    : out  std_logic_vector(17 downto 0);
		acq_ibus_x    : out  std_logic_vector(17 downto 0);
		acq_vphs_a    : out  std_logic_vector(17 downto 0);
		acq_vphs_b    : out  std_logic_vector(17 downto 0);
		acq_vphs_c    : out  std_logic_vector(17 downto 0);
		acq_vbus_x    : out  std_logic_vector(17 downto 0);
		acq_vphs_n    : out  std_logic_vector(17 downto 0);
		--
		acq_sync      : out  std_logic;
		
		-- output offset
		--
		ofs_iphs_a    : in  std_logic_vector(17 downto 0);
		ofs_iphs_b    : in  std_logic_vector(17 downto 0);
		ofs_iphs_c    : in  std_logic_vector(17 downto 0);
		ofs_ibus_x    : in  std_logic_vector(17 downto 0);
		ofs_vphs_a    : in  std_logic_vector(17 downto 0);
		ofs_vphs_b    : in  std_logic_vector(17 downto 0);
		ofs_vphs_c    : in  std_logic_vector(17 downto 0);
		ofs_vbus_x    : in  std_logic_vector(17 downto 0);
		ofs_vphs_n    : in  std_logic_vector(17 downto 0);

		-- normalized values (offset removed)
		--
		nrm_iphs_a    : in  std_logic_vector(17 downto 0);
		nrm_iphs_b    : in  std_logic_vector(17 downto 0);
		nrm_iphs_c    : in  std_logic_vector(17 downto 0);
		nrm_ibus_x    : in  std_logic_vector(17 downto 0);
		nrm_vphs_a    : in  std_logic_vector(17 downto 0);
		nrm_vphs_b    : in  std_logic_vector(17 downto 0);
		nrm_vphs_c    : in  std_logic_vector(17 downto 0);
		nrm_vbus_x    : in  std_logic_vector(17 downto 0);
		nrm_vphs_n    : in  std_logic_vector(17 downto 0);
		--
		nrm_sync      : in  std_logic;

		-- filtered values
		--
		flt_iphs_a    : in  std_logic_vector(17 downto 0);
		flt_iphs_b    : in  std_logic_vector(17 downto 0);
		flt_iphs_c    : in  std_logic_vector(17 downto 0);
		flt_ibus_x    : in  std_logic_vector(17 downto 0);
		flt_vphs_a    : in  std_logic_vector(17 downto 0);
		flt_vphs_b    : in  std_logic_vector(17 downto 0);
		flt_vphs_c    : in  std_logic_vector(17 downto 0);
		flt_vbus_x    : in  std_logic_vector(17 downto 0);
		flt_vphs_n    : in  std_logic_vector(17 downto 0);
		--
		flt_sync      : in  std_logic;

		-- current overflow
		--
		ovi_iphs_a    : in  std_logic;
		ovi_iphs_b    : in  std_logic;
		ovi_iphs_c    : in  std_logic;
		ovi_ibus_x    : in  std_logic;

		-- Is overflow
		ismodover     : in  std_logic;
		
		-- 1=generator, 0=motor
		generator     : in  std_logic;
		
		-- modulator type
		modtype       : in  std_logic_vector(0 downto 0);
		
		-- modulator levels
		modlevels     : in  std_logic_vector(0 downto 0);
		
		-- motor is 3-phases (0=bipolar)
		is3phases     : in  std_logic;
		
		-- pwm 3-phases outputs
		pwm2p_l2c1w1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c1w2  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w2  : in  std_logic_vector(1 downto 0);
		pwm2p_l2sync  : in  std_logic;
		
		-- pwm 2-phases outputs
		pwm3p_l2cxw1  : in  std_logic_vector(2 downto 0);
		pwm3p_l2cxw2  : in  std_logic_vector(2 downto 0);
		pwm3p_l2sync  : in  std_logic;
		
		-- rpfm 3-ph two levels outputs
		rpfm3p_l2cxw1 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2cxw2 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2sync : in  std_logic;
		
		-- rpfm 3-ph three levels outputs
		rpfm3p_l3c1w1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c1w2 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w2 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w2 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3sync : in  std_logic;
		
		-- dead time value
		deadtval      : in  std_logic_vector(7 downto 0);
		
		-- gate unit switches
		guswcnt       : out  std_logic_vector(31 downto 0);
		
		-- in-failure
		hwfail        : out  std_logic;
		
		-- in-kill
		hwkill        : out  std_logic;
		
		-- coil driver
		coilgear      : in  std_logic;
		coilenab      : in  std_logic;
		
		-- system error
		syserr        : in  std_logic;
		
		-- reset pending errors
		reseterr      : in  std_logic;
		
		-- zero current offset 
		zerocurr      : in  std_logic;
		
		-- zero vbus offset 
		zerovbusx     : in  std_logic;

		-- zero vphs offset 
		zerovphsx     : in  std_logic;
		
		-- control probes
		pix_setval    : in std_logic_vector(17 downto 0);
		piy_setval    : in std_logic_vector(17 downto 0);
		
		pix_fbkval    : in std_logic_vector(17 downto 0);
		piy_fbkval    : in std_logic_vector(17 downto 0);
		
		pix_outval    : in std_logic_vector(17 downto 0);
		piy_outval    : in std_logic_vector(17 downto 0);

		-- Vs
		vsmodval      : in  std_logic_vector(16 downto 0);
		vsangval      : in  std_logic_vector(31 downto 0);
		
		-- Is
		ismodval      : in  std_logic_vector(16 downto 0);
		isangval      : in  std_logic_vector(31 downto 0);

		-- smo
		smo_bemf_p    : in  std_logic_vector(31 downto 0);
		smo_bemf_m    : in  std_logic_vector(16 downto 0);
		
		-- e-angles
		mrt_angle     : in  std_logic_vector(31 downto 0);
		enc_angle     : in  std_logic_vector(31 downto 0);
		hls_angle     : in  std_logic_vector(31 downto 0);
		rsv_angle     : in  std_logic_vector(31 downto 0);
		smo_angle     : in  std_logic_vector(31 downto 0);
		
		-- Rotor angle
		rot_angle     : in  std_logic_vector(31 downto 0);
		
		-- rot selector
		rot_select    : in  std_logic_vector( 2 downto 0);
		
		-- actual speed
		spd_speed     : in  std_logic_vector(31 downto 0);
		
		-- position
		pos_position  : in  std_logic_vector(31 downto 0);
		
		-- foc cycle time
		tsclocks      : in  std_logic_vector(11 downto 0)
	);
end tec53sys;

architecture Behavioral of tec53sys is

	component inlkmeta
		generic
		(
			C_NCLKS : integer := 3
		);
		port 
		(
			clock  : in  std_logic;
			datin  : in  std_logic;
			datout : out std_logic
		);
	end component;
	
	component unglitch
		generic
		(
			C_CLOCK_FREQ  : integer := 100_000_000;
			C_FILTER_TIME : real    := 1.0e-6
		);
		port
		(
			clock  : in  std_logic;
			reset  : in  std_logic;
			datin  : in  std_logic;
			datout : out std_logic
		);
	end component;
	
	component s3acqmult
		generic
		(
			C_RESULT_WIDTH : integer := 16;
			C_NUM_CHANNELS : integer := 16
		);
		port 
		(
			clock   : in  std_logic;
			reset   : in  std_logic;
			--
			sclock  : in  std_logic;
			sdata   : in  std_logic_vector(C_NUM_CHANNELS-1 downto 0);
			--
			pdata0  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata1  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata2  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata3  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata4  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata5  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata6  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata7  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata8  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata9  : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata10 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata11 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata12 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata13 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata14 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			pdata15 : out std_logic_vector(C_RESULT_WIDTH-1 downto 0);
			finish  : out std_logic
		);
	end component;

	component gudtmfx
		port 
		(
			clock    : in  std_logic;
			reset    : in  std_logic;
			--
			deadtval : in  std_logic_vector(7 downto 0);
			command  : in  std_logic;
			enablehi : in  std_logic;
			enablelo : in  std_logic;
			drivehi  : out std_logic;
			drivelo  : out std_logic
		);
	end component;
	
	component evxcount
		generic
		(
			C_DATA_WIDTH  : integer := 16;
			C_EVENT_WIDTH : integer := 8;
			C_COUNT_MODE  : integer := 0
		);
		port
		(
			clock   : in  std_logic;
			reset   : in  std_logic;
			sigevt  : in  std_logic_vector(C_EVENT_WIDTH-1 downto 0);
			counter : out std_logic_vector(C_DATA_WIDTH-1 downto 0)
		);
	end component;
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	-- modulator type
	constant MODTYPE_PWM  : std_logic_vector(0 downto 0) := "0";
	constant MODTYPE_RPFM : std_logic_vector(0 downto 0) := "1";
	
	-- modulator levels
	constant MODLEVELS_2  : std_logic_vector(0 downto 0) := "0";
	constant MODLEVELS_3  : std_logic_vector(0 downto 0) := "1";
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	-- 3-phase index
	constant P1      : integer := 0;
	constant P2      : integer := 1;
	constant P3      : integer := 2;
	
	-- leg value for 3 levels
	constant K_LOW   : std_logic_vector(1 downto 0) := "00";
	constant K_MID   : std_logic_vector(1 downto 0) := "01";
	constant K_HIG   : std_logic_vector(1 downto 0) := "10";
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	constant RA_ip_ident         : integer := (4*0);

	---------------------------------------------------------------------------
	
	constant C_DECAD_WIDTH       : integer := 1;
	
	---------------------------------------------------------------------------
	
	function adrtoint(ad : std_logic_vector) return integer is
		variable ar : std_logic_vector(C_DECAD_WIDTH+1 downto 0);
	begin
		ar := ad(ar'left downto 2) & "00";
		return to_integer(unsigned(ar));
	end function;
	
	---------------------------------------------------------------------------

	-- ip identification
	constant ip_id_code  : integer := 14;
	constant ip_id_majv  : integer := 0;
	constant ip_id_minv  : integer := 1;
	constant ip_id_hscr  : integer := 97;
	
	constant ip_ident_k : std_logic_vector(31 downto 0) :=
		std_logic_vector(to_unsigned(ip_id_code,8)) &
		std_logic_vector(to_unsigned(ip_id_majv,8)) &
		std_logic_vector(to_unsigned(ip_id_minv,8)) &
		std_logic_vector(to_unsigned(ip_id_hscr,8));
	
	-- access delay
	signal host_en_dl1  : std_logic;
	
	---------------------------------------------------------------------------

	-- bus2reg
	function loadreg(s : std_logic_vector) return std_logic_vector is
		variable r : std_logic_vector(31 downto 0);
	begin
		r := (others => '0');
		r(s'range) := s;
		return r;
	end function;

	-- reg2bus
	procedure storereg(signal d : out std_logic_vector; signal s : in std_logic_vector) is
	begin
		d <= s(d'range);
	end procedure;
	
	---------------------------------------------------------------------------
	
	-- anti-metastability # FF
	constant K_UMSK : integer := 2;
	
	-- anti-meta-stabilized encoder
	signal lk_enc_cha : std_logic;
	signal lk_enc_chb : std_logic;
	signal lk_enc_chi : std_logic;
	
	-- ADC data line
	signal lk_adx_dat : std_logic_vector(3 downto 0);

	---------------------------------------------------------------------------

	-- deglitch filter
	constant K_DEGLITH_TIME : real := 200.0e-9;
	
	-- deglitched encoder inputs
	signal dg_enc_cha : std_logic;
	signal dg_enc_chb : std_logic;
	signal dg_enc_chi : std_logic;
	
	---------------------------------------------------------------------------
	
	-- s/d acquisition
	
	---------------------------------------------------------------------------

	function invsd_time2bits return integer is
		variable btn : integer;
		variable cnt : real;
		variable cns : real;
	begin
		cnt := real(C_SDADC_CLK_FREQ_HZ) * real(C_INV_ACQTIME_NS) * 1.0e-9;
		btn := 0;
		cns := 1.0;
		while cns < cnt loop
		  btn := btn + 1;
		  cns := cns * 2.0;
        end loop;
        btn := btn * 2;
		return btn;
	end function;
	
	---------------------------------------------------------------------------
	-- Inverter Values indexes
	
	constant C_IVD_IPHS_A : integer := 0;
	constant C_IVD_IPHS_B : integer := 1;
	constant C_IVD_IPHS_C : integer := 2;
	constant C_IVD_VBUS_X : integer := 3;
	constant C_IVD_NUM    : integer := 4;
		
	---------------------------------------------------------------------------
	-- ACQ (acquisition)
	
	signal invdat_sdat  : std_logic_vector(C_IVD_NUM-1 downto 0);
	
	constant C_INVERTER_SD_WIDTH : integer := invsd_time2bits;
	
	type   invdat_acq_t is array(0 to C_IVD_NUM-1) of std_logic_vector(C_INVERTER_SD_WIDTH-1 downto 0);
	signal invdat_acq : invdat_acq_t := (others => (others => '0'));
	signal invdat_eoc : std_logic;
	
	---------------------------------------------------------------------------
	-- RAW (raw: final size)
	
	constant C_INVDAT_ACQ2RAW_PAD : std_logic_vector(18-C_INVERTER_SD_WIDTH-1 downto 0) := (others => '0');
	
	type   invdat_raw_t is array(0 to C_IVD_NUM-1) of std_logic_vector(17 downto 0);
	signal invdat_raw : invdat_raw_t := (others => (others => '0'));
	
	---------------------------------------------------------------------------
	-- BIP (unipolar to bipolar)
	
	type   invdat_bip_t is array(0 to C_IVD_NUM-1) of std_logic_vector(17 downto 0);
	signal invdat_bip : invdat_bip_t := (others => (others => '0'));
	
	---------------------------------------------------------------------------
	-- coil drive
	
	-- gu 3-phase command after modulator mux
	signal p3_gu_cmd   : std_logic_vector(2 downto 0);
	signal p3_gu_ena_h : std_logic_vector(2 downto 0);
	signal p3_gu_ena_l : std_logic_vector(2 downto 0);
	
	-- gate unit output enable for automatic mode
	signal coilenx     : std_logic;
	
	---------------------------------------------------------------------------
	
	-- events to counts
	signal guevsrc : std_logic_vector(5 downto 0);
	
	---------------------------------------------------------------------------
	
	---------------------------------------------------------------------------
	-- outputs
	---------------------------------------------------------------------------
	
	
	---------------------------------------------------------------------------
	
	-- host interface
	signal host_dout_i        :  std_logic_vector(31 downto 0);
	signal host_ack_i         :  std_logic;
	
	---------------------------------------------------------------------------
	-- board phy

	-- Gate unit control
	signal gu_cmd_h_i       : std_logic_vector(2 downto 0);
	signal gu_cmd_l_i       : std_logic_vector(2 downto 0);

	-- S/D Analog acquisiton
	signal adx_clk_i        : std_logic;

	---------------------------------------------------------------------------
	
	-- motor control
	
	-- external angle
	signal ext_angle_i   : std_logic_vector(31 downto 0);
	
	-- resolver
	signal rsv_priexc_i  : std_logic_vector(17 downto 0);
	signal rsv_secsin_i  : std_logic_vector(17 downto 0);
	signal rsv_seccos_i  : std_logic_vector(17 downto 0);
	
	-- hall sensor
	signal hls_hallsig_i : std_logic_vector(5 downto 0);
	
	-- digital encoder
	signal enc_cha_i     : std_logic;
	signal enc_chb_i     : std_logic;
	signal enc_chi_i     : std_logic;
	
	-- input data
	--
	signal acq_iphs_a_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_b_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_c_i  : std_logic_vector(17 downto 0);
	signal acq_ibus_x_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_a_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_b_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_c_i  : std_logic_vector(17 downto 0);
	signal acq_vbus_x_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_n_i  : std_logic_vector(17 downto 0);
	--
	signal acq_sync_i    : std_logic;
	
	-- gate unit switches
	signal guswcnt_i     : std_logic_vector(31 downto 0);
	
	-- in-failure
	signal hwfail_i      : std_logic;
	
	-- in-kill
	signal hwkill_i      : std_logic;

begin

	---------------------------------------------------------------------------
	
	-- host interface
	host_dout      <= host_dout_i;
	host_ack       <= host_ack_i;

	---------------------------------------------------------------------------
	
	-- board phy

	-- gate unit control
	gu_cmd_h       <= gu_cmd_h_i;
	gu_cmd_l       <= gu_cmd_l_i;

	-- analog acq.
	adx_clk        <= adx_clk_i;

	---------------------------------------------------------------------------
	
	-- motor control
	
	-- external angle
	ext_angle   <= ext_angle_i;
	
	-- resolver
	rsv_priexc  <= rsv_priexc_i;
	rsv_secsin  <= rsv_secsin_i;
	rsv_seccos  <= rsv_seccos_i;
	
	-- hall sensor
	hls_hallsig <= hls_hallsig_i;
	
	-- digital encoder
	enc_cha     <= enc_cha_i;
	enc_chb     <= enc_chb_i;
	enc_chi     <= enc_chi_i;
	
	-- input data
	--
	acq_iphs_a  <= acq_iphs_a_i;
	acq_iphs_b  <= acq_iphs_b_i;
	acq_iphs_c  <= acq_iphs_c_i;
	acq_ibus_x  <= acq_ibus_x_i;
	acq_vphs_a  <= acq_vphs_a_i;
	acq_vphs_b  <= acq_vphs_b_i;
	acq_vphs_c  <= acq_vphs_c_i;
	acq_vbus_x  <= acq_vbus_x_i;
	acq_vphs_n  <= acq_vphs_n_i;
	--
	acq_sync    <= acq_sync_i;
	
	-- gate unit switches
	guswcnt     <= guswcnt_i;
	
	-- in-failure
	hwfail      <= hwfail_i;
	
	-- in-kill
	hwkill      <= hwkill_i;
	
	---------------------------------------------------------------------------
	
	-- anti metastability
	LK_ENC_A_INST : inlkmeta generic map ( C_NCLKS => K_UMSK ) port map ( clock => host_clk, datin => enc_a, datout => lk_enc_cha );
	LK_ENC_B_INST : inlkmeta generic map ( C_NCLKS => K_UMSK ) port map ( clock => host_clk, datin => enc_b, datout => lk_enc_chb );
	LK_ENC_I_INST : inlkmeta generic map ( C_NCLKS => K_UMSK ) port map ( clock => host_clk, datin => enc_i, datout => lk_enc_chi );
	
	-- deglitch encoder
	DG_ENC_A_INST : unglitch generic map ( C_CLOCK_FREQ => C_HOST_CLK_FREQ_HZ, C_FILTER_TIME => K_DEGLITH_TIME ) port map ( clock  => host_clk, reset  => host_rst, datin  => lk_enc_cha, datout => dg_enc_cha );
	DG_ENC_B_INST : unglitch generic map ( C_CLOCK_FREQ => C_HOST_CLK_FREQ_HZ, C_FILTER_TIME => K_DEGLITH_TIME ) port map ( clock  => host_clk, reset  => host_rst, datin  => lk_enc_chb, datout => dg_enc_chb );
	DG_ENC_I_INST : unglitch generic map ( C_CLOCK_FREQ => C_HOST_CLK_FREQ_HZ, C_FILTER_TIME => K_DEGLITH_TIME ) port map ( clock  => host_clk, reset  => host_rst, datin  => lk_enc_chi, datout => dg_enc_chi );
	
	-- output to motor control ip
	enc_cha_i <= dg_enc_cha;
	enc_chb_i <= dg_enc_chb;
	enc_chi_i <= dg_enc_chi;
	
	-- motor hall sensors on same digital inputs
	
	-- output to motor control ip
	hls_hallsig_i(0) <= dg_enc_chi;
	hls_hallsig_i(1) <= dg_enc_cha;
	hls_hallsig_i(2) <= dg_enc_chb;
	hls_hallsig_i(5 downto 3) <= (others => '0');
	
	---------------------------------------------------------------------------
	
	-- external angle not inplemented
	ext_angle_i <= (others => '0');
	
	-- resolver not inplemented
	rsv_priexc_i <= (others => '0');
	rsv_secsin_i <= (others => '0');
	rsv_seccos_i <= (others => '0');
	
	---------------------------------------------------------------------------
	
	-- sigma/delta output clocks
	adx_clk_i  <= clk_sdadc xor std_logic(to_unsigned(C_SD_CLK_NOT,1)(0));

	-- anti metastability processing
	LK_ADX_DAT_GEN : for i in adx_dat'range generate LK_ADX_DAT_INST : inlkmeta generic map ( C_NCLKS => K_UMSK ) port map ( clock => clk_sdadc, datin => adx_dat(i), datout => lk_adx_dat(i) ); end generate;
	
	---------------------------------------------------------------------------
	-- report acq time
	assert false
		report "Acquisition time is : " & real'image(real(C_INV_ACQTIME_NS) * 1.0e-3) & " uS, sample bits : " & integer'image(C_INVERTER_SD_WIDTH)
		severity note;
			
	---------------------------------------------------------------------------

	-- inverter acquisition
	invdat_sdat(C_IVD_IPHS_A) <= lk_adx_dat(0);
	invdat_sdat(C_IVD_IPHS_B) <= lk_adx_dat(1);
	invdat_sdat(C_IVD_IPHS_C) <= lk_adx_dat(2);
	invdat_sdat(C_IVD_VBUS_X) <= lk_adx_dat(3);
			
	INV_ACQ_INST : s3acqmult
		generic map
		(
			C_RESULT_WIDTH => C_INVERTER_SD_WIDTH,
			C_NUM_CHANNELS => C_IVD_NUM
		)
		port map
		(
			clock   => host_clk,
			reset   => host_rst,
			--
			sclock  => clk_sdadc,
			sdata   => invdat_sdat,
			--
			pdata0  => invdat_acq(0),
			pdata1  => invdat_acq(1),
			pdata2  => invdat_acq(2),
			pdata3  => invdat_acq(3),
			pdata4  => open,
			pdata5  => open,
			pdata6  => open,
			pdata7  => open,
			pdata8  => open,
			pdata9  => open,
			pdata10 => open,
			pdata11 => open,
			pdata12 => open,
			pdata13 => open,
			pdata14 => open,
			pdata15 => open,
			finish  => invdat_eoc
		);
		
	-- final size
	INV_ACQ2RAW_GEN : for i in invdat_sdat'range generate
		invdat_raw(i) <= invdat_acq(i) & C_INVDAT_ACQ2RAW_PAD;
	end generate;
	
	-- unipolar to bipolar
	INV_RAW2BIP_GEN : for i in invdat_sdat'range generate
		invdat_bip(i) <= std_logic_vector(signed(invdat_raw(i)) - 2**17);
	end generate;
	
	-- output to motor control ip
	acq_iphs_a_i <= invdat_bip(C_IVD_IPHS_A);
	acq_iphs_b_i <= invdat_bip(C_IVD_IPHS_B);
	acq_iphs_c_i <= invdat_bip(C_IVD_IPHS_C);
	acq_ibus_x_i <= (others => '0');
	acq_vphs_a_i <= (others => '0');
	acq_vphs_b_i <= (others => '0');
	acq_vphs_c_i <= (others => '0');
	acq_vbus_x_i <= invdat_bip(C_IVD_VBUS_X);
	acq_vphs_n_i <= (others => '0');
	acq_sync_i   <= invdat_eoc;
	
	---------------------------------------------------------------------------
	
	-- enable coil: enabled and 3-phase only and 2-levels
	coilenx <= '1' when (coilgear = '1' and is3phases = '1' and modlevels = MODLEVELS_2) else '0';
	
	-- mux PWM/RPFM 2 levels
	p3_gu_cmd <= rpfm3p_l2cxw1 when modtype = MODTYPE_RPFM else pwm3p_l2cxw1;
	
	p3_gu_ena_h <= (others => coilenx);
	p3_gu_ena_l <= (others => coilenx);
	
	---------------------------------------------------------------------------
	
	-- cmd to drive (enable and dead time)
	
	GU_DTFX_GEN : for i in 0 to 2 generate
		
		GU_DTFX_INST : gudtmfx
			port map
			(
				clock    => host_clk,
				reset    => host_rst,
				--
				deadtval  => deadtval,
				command   => p3_gu_cmd(i),
				enablehi  => p3_gu_ena_h(i),
				enablelo  => p3_gu_ena_l(i),
				drivehi   => gu_cmd_h_i(i),
				drivelo   => gu_cmd_l_i(i)
			);
			
	end generate;

	---------------------------------------------------------------------------
	
	-- gate switches counter
	
	guevsrc <= gu_cmd_h_i & gu_cmd_l_i;
	
	GUSWCNT_INST : evxcount
		generic map
		(
			C_DATA_WIDTH  => 32,
			C_EVENT_WIDTH => 6,
			C_COUNT_MODE  => 0
		)
		port map
		(
			clock   => host_clk,
			reset   => host_rst,
			sigevt  => guevsrc,
			counter => guswcnt_i
		);
	
	---------------------------------------------------------------------------
	
	-- gate unit not enable is also KILL
	hwkill_i <= '0';
	
	-- gu error (comulative)
	hwfail_i <= '0';
	
	---------------------------------------------------------------------------
	
	-- implement slave model software accessible register(s)
	SLAVE_WRITE_PROC : process(host_clk)
	begin
		if rising_edge(host_clk) then
		
			if host_rst = '1' then
			
				null;
				
			elsif host_we = '1' and host_en = '1' then
			
				case adrtoint(host_addr) is
					when RA_ip_ident     => null;
					when others          => null;
				end case;
			end if;
		end if;
	end process;

	-- implement slave model software accessible register(s)
	SLAVE_READ_PROC : process(host_clk)
		variable v : std_logic_vector(31 downto 0);
	begin
		if rising_edge(host_clk) then
			v := (others => '0');
			case adrtoint(host_addr) is
				when RA_ip_ident     => v := loadreg(ip_ident_k);
				when others          => null;
			end case;
			host_dout_i <= v;
		end if;
	end process;
	
	-- data transfer ack
	host_ack_i <= host_we or host_en_dl1;
	
	-- host access delay
	HOSTDLY_PROC : process(host_clk)
	begin
		if rising_edge(host_clk) then
			host_en_dl1 <= host_en;
		end if;
	end process;

end Behavioral;
