-------------------------------------------------------------------------------------------
-- Copyright  2011-2014, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    09:41:23 11/02/2014 
-- Design Name: 
-- Module Name:    evxcount - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity evxcount is
	generic
	(
		C_DATA_WIDTH  : integer := 16;
		C_EVENT_WIDTH : integer := 8;
		C_COUNT_MODE  : integer := 0
	);
	port
	(
		clock   : in  std_logic;
		reset   : in  std_logic;
		sigevt  : in  std_logic_vector(C_EVENT_WIDTH-1 downto 0);
		counter : out std_logic_vector(C_DATA_WIDTH-1 downto 0)
	);
end evxcount;

architecture Behavioral of evxcount is

	function sommabit(v : std_logic_vector) return integer is
		variable n : integer;
	begin
		n := 0;
		for i in v'range loop
			if v(i) = '1' then
				n := n + 1;
			end if;
		end loop;
		return n;
	end function;

	signal captevt   : std_logic_vector(C_EVENT_WIDTH-1 downto 0);
	signal docount   : std_logic_vector(C_EVENT_WIDTH-1 downto 0);

	signal counter_i : std_logic_vector(C_DATA_WIDTH-1 downto 0);

begin

	counter <= counter_i;
	
	MAIN_PROC : process(clock)
	begin
		if rising_edge(clock) then
			if reset = '1' then
				counter_i <= (others => '0');
			else
				counter_i <= counter_i + sommabit(docount);
			end if;
			captevt <= sigevt;
		end if;
	end process;
	
	CM_NONE_GEN : if C_COUNT_MODE < 0 or C_COUNT_MODE > 2 generate
		docount <= (others => '0');
	end generate;
	
	CM_BOTH_GEN : if C_COUNT_MODE = 0 generate
		CX_BOTH_GEN : for i in sigevt'range generate
			docount(i) <= '1' when sigevt(i) /= captevt(i) else '0';
		end generate;
	end generate;
	
	CM_RISING_GEN : if C_COUNT_MODE = 1 generate
		CX_RISING_GEN : for i in sigevt'range generate
			docount(i) <= '1' when sigevt(i) = '1' and captevt(i) = '0' else '0';
		end generate;
	end generate;
	
	CM_FALLING_GEN : if C_COUNT_MODE = 2 generate
		CX_FALLING_GEN :for i in sigevt'range generate
			docount(i) <= '1' when sigevt(i) = '0' and captevt(i) = '1' else '0';
		end generate;
	end generate;

end Behavioral;
