-------------------------------------------------------------------------------------------
-- Copyright  2011-2018, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    13:15:25 12/28/2015 
-- Design Name: 
-- Module Name:    tep2sys - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity tep2sys is
	generic
	(
		C_FAMILY           : string  := "zynq";
		C_HOST_CLK_FREQ_HZ : integer := 100_000_000;
		--
		C_INV_ACQTIME_NS   : integer := 3200
	);
	port 
	(
		-- host interface
		host_clk      : in  std_logic;
		host_rst      : in  std_logic;
		host_en       : in  std_logic;
		host_we       : in  std_logic;
		host_addr     : in  std_logic_vector(31 downto 0);
		host_din      : in  std_logic_vector(31 downto 0);
		host_dout     : out std_logic_vector(31 downto 0);
		host_ack      : out std_logic;
		--
		-- Gate unit control
		gu_cmd_d      : out std_logic_vector(2 downto 0);
		gu_cmd_e      : out std_logic_vector(2 downto 0);
		--
		-- Voltage & Current analog acquisiton
		adc_sck       : out std_logic;
		adc_cs        : out std_logic;
		adc_ch        : out std_logic;
		adc_dat       : in  std_logic_vector(3 downto 0);
		--
		-- Sensor fault
		isn_fault     : in std_logic;
		
		-- motor control IP

		-- external angle
		ext_angle     : out   std_logic_vector(31 downto 0);
		
		-- resolver
		rsv_priexc    : out   std_logic_vector(17 downto 0);
		rsv_secsin    : out   std_logic_vector(17 downto 0);
		rsv_seccos    : out   std_logic_vector(17 downto 0);
		
		-- hall sensor
		hls_hallsig   : out   std_logic_vector(5 downto 0);
		
		-- digital encoder
		enc_cha       : out   std_logic;
		enc_chb       : out   std_logic;
		enc_chi       : out   std_logic;
		
		-- input data
		--
		acq_iphs_a    : out  std_logic_vector(17 downto 0);
		acq_iphs_b    : out  std_logic_vector(17 downto 0);
		acq_iphs_c    : out  std_logic_vector(17 downto 0);
		acq_ibus_x    : out  std_logic_vector(17 downto 0);
		acq_vphs_a    : out  std_logic_vector(17 downto 0);
		acq_vphs_b    : out  std_logic_vector(17 downto 0);
		acq_vphs_c    : out  std_logic_vector(17 downto 0);
		acq_vbus_x    : out  std_logic_vector(17 downto 0);
		acq_vphs_n    : out  std_logic_vector(17 downto 0);
		--
		acq_sync      : out  std_logic;
		
		-- output offset
		--
		ofs_iphs_a    : in  std_logic_vector(17 downto 0);
		ofs_iphs_b    : in  std_logic_vector(17 downto 0);
		ofs_iphs_c    : in  std_logic_vector(17 downto 0);
		ofs_ibus_x    : in  std_logic_vector(17 downto 0);
		ofs_vphs_a    : in  std_logic_vector(17 downto 0);
		ofs_vphs_b    : in  std_logic_vector(17 downto 0);
		ofs_vphs_c    : in  std_logic_vector(17 downto 0);
		ofs_vbus_x    : in  std_logic_vector(17 downto 0);
		ofs_vphs_n    : in  std_logic_vector(17 downto 0);

		-- normalized values (offset removed)
		--
		nrm_iphs_a    : in  std_logic_vector(17 downto 0);
		nrm_iphs_b    : in  std_logic_vector(17 downto 0);
		nrm_iphs_c    : in  std_logic_vector(17 downto 0);
		nrm_ibus_x    : in  std_logic_vector(17 downto 0);
		nrm_vphs_a    : in  std_logic_vector(17 downto 0);
		nrm_vphs_b    : in  std_logic_vector(17 downto 0);
		nrm_vphs_c    : in  std_logic_vector(17 downto 0);
		nrm_vbus_x    : in  std_logic_vector(17 downto 0);
		nrm_vphs_n    : in  std_logic_vector(17 downto 0);
		--
		nrm_sync      : in  std_logic;

		-- filtered values
		--
		flt_iphs_a    : in  std_logic_vector(17 downto 0);
		flt_iphs_b    : in  std_logic_vector(17 downto 0);
		flt_iphs_c    : in  std_logic_vector(17 downto 0);
		flt_ibus_x    : in  std_logic_vector(17 downto 0);
		flt_vphs_a    : in  std_logic_vector(17 downto 0);
		flt_vphs_b    : in  std_logic_vector(17 downto 0);
		flt_vphs_c    : in  std_logic_vector(17 downto 0);
		flt_vbus_x    : in  std_logic_vector(17 downto 0);
		flt_vphs_n    : in  std_logic_vector(17 downto 0);
		--
		flt_sync      : in  std_logic;

		-- current overflow
		--
		ovi_iphs_a    : in  std_logic;
		ovi_iphs_b    : in  std_logic;
		ovi_iphs_c    : in  std_logic;
		ovi_ibus_x    : in  std_logic;

		-- Is overflow
		ismodover     : in  std_logic;
		
		-- 1=generator, 0=motor
		generator     : in  std_logic;
		
		-- modulator type
		modtype       : in  std_logic_vector(0 downto 0);
		
		-- modulator levels
		modlevels     : in  std_logic_vector(0 downto 0);
		
		-- motor is 3-phases (0=bipolar)
		is3phases     : in  std_logic;
		
		-- pwm 3-phases outputs
		pwm2p_l2c1w1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w1  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c1w2  : in  std_logic_vector(1 downto 0);
		pwm2p_l2c2w2  : in  std_logic_vector(1 downto 0);
		pwm2p_l2sync  : in  std_logic;
		
		-- pwm 2-phases outputs
		pwm3p_l2cxw1  : in  std_logic_vector(2 downto 0);
		pwm3p_l2cxw2  : in  std_logic_vector(2 downto 0);
		pwm3p_l2sync  : in  std_logic;
		
		-- rpfm 3-ph two levels outputs
		rpfm3p_l2cxw1 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2cxw2 : in  std_logic_vector(2 downto 0);
		rpfm3p_l2sync : in  std_logic;
		
		-- rpfm 3-ph three levels outputs
		rpfm3p_l3c1w1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w1 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c1w2 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c2w2 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3c3w2 : in  std_logic_vector(1 downto 0);
		rpfm3p_l3sync : in  std_logic;
		
		-- dead time value
		deadtval      : in  std_logic_vector(7 downto 0);
		
		-- gate unit switches
		guswcnt       : out  std_logic_vector(31 downto 0);
		
		-- in-failure
		hwfail        : out  std_logic;
		
		-- in-kill
		hwkill        : out  std_logic;
		
		-- coil driver
		coilgear      : in  std_logic;
		coilenab      : in  std_logic;
		
		-- system error
		syserr        : in  std_logic;
		
		-- reset pending errors
		reseterr      : in  std_logic;
		
		-- zero current offset 
		zerocurr      : in  std_logic;
		
		-- zero vbus offset 
		zerovbusx     : in  std_logic;

		-- zero vphs offset 
		zerovphsx     : in  std_logic;
		
		-- control probes
		pix_setval    : in std_logic_vector(17 downto 0);
		piy_setval    : in std_logic_vector(17 downto 0);
		
		pix_fbkval    : in std_logic_vector(17 downto 0);
		piy_fbkval    : in std_logic_vector(17 downto 0);
		
		pix_outval    : in std_logic_vector(17 downto 0);
		piy_outval    : in std_logic_vector(17 downto 0);
		
		-- Vs
		vsmodval      : in  std_logic_vector(16 downto 0);
		vsangval      : in  std_logic_vector(31 downto 0);
		
		-- Is
		ismodval      : in  std_logic_vector(16 downto 0);
		isangval      : in  std_logic_vector(31 downto 0);

		-- smo
		smo_bemf_p    : in  std_logic_vector(31 downto 0);
		smo_bemf_m    : in  std_logic_vector(16 downto 0);
		
		-- e-angles
		mrt_angle     : in  std_logic_vector(31 downto 0);
		enc_angle     : in  std_logic_vector(31 downto 0);
		hls_angle     : in  std_logic_vector(31 downto 0);
		rsv_angle     : in  std_logic_vector(31 downto 0);
		smo_angle     : in  std_logic_vector(31 downto 0);
		
		-- Rotor angle
		rot_angle     : in  std_logic_vector(31 downto 0);
		
		-- rot selector
		rot_select    : in  std_logic_vector( 2 downto 0);
		
		-- actual speed
		spd_speed     : in  std_logic_vector(31 downto 0);
		
		-- position
		pos_position  : in  std_logic_vector(31 downto 0);
		
		-- foc cycle time
		tsclocks      : in  std_logic_vector(11 downto 0)
	);
end tep2sys;

architecture Behavioral of tep2sys is

	component inlkmeta
		generic
		(
			C_NCLKS : integer := 3
		);
		port 
		(
			clock  : in  std_logic;
			datin  : in  std_logic;
			datout : out std_logic
		);
	end component;
	
	component unglitch
		generic
		(
			C_CLOCK_FREQ  : integer := 100_000_000;
			C_FILTER_TIME : real    := 1.0e-6
		);
		port
		(
			clock  : in  std_logic;
			reset  : in  std_logic;
			datin  : in  std_logic;
			datout : out std_logic
		);
	end component;
	
	component evxcount
		generic
		(
			C_DATA_WIDTH  : integer := 16;
			C_EVENT_WIDTH : integer := 8;
			C_COUNT_MODE  : integer := 0
		);
		port
		(
			clock   : in  std_logic;
			reset   : in  std_logic;
			sigevt  : in  std_logic_vector(C_EVENT_WIDTH-1 downto 0);
			counter : out std_logic_vector(C_DATA_WIDTH-1 downto 0)
		);
	end component;
	
	
	component ads7947core
		generic
		(
			C_CLK_FREQ_HZ  : integer := 100_000_000;
			C_ACQTIME_NS   : integer := 3200
		);
		port 
		( 
			clock  : in  std_logic;
			reset  : in  std_logic;
			sdo    : in  std_logic_vector(3 downto 0);
			sclk   : out std_logic;
			cs     : out std_logic;
			ch_sel : out std_logic;
			pdo00  : out std_logic_vector(11 downto 0);
			pdo01  : out std_logic_vector(11 downto 0);
			pdo10  : out std_logic_vector(11 downto 0);
			pdo11  : out std_logic_vector(11 downto 0);
			pdo20  : out std_logic_vector(11 downto 0);
			pdo21  : out std_logic_vector(11 downto 0);
			pdo30  : out std_logic_vector(11 downto 0);
			pdo31  : out std_logic_vector(11 downto 0);
			finish : out std_logic
		);
	end component;
	
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	-- modulator type
	constant MODTYPE_PWM  : std_logic_vector(0 downto 0) := "0";
	constant MODTYPE_RPFM : std_logic_vector(0 downto 0) := "1";
	
	-- modulator levels
	constant MODLEVELS_2  : std_logic_vector(0 downto 0) := "0";
	constant MODLEVELS_3  : std_logic_vector(0 downto 0) := "1";
	
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	
	-- 3-phase index
	constant P1      : integer := 0;
	constant P2      : integer := 1;
	constant P3      : integer := 2;
	
	-- leg value for 3 levels
	constant K_LOW   : std_logic_vector(1 downto 0) := "00";
	constant K_MID   : std_logic_vector(1 downto 0) := "01";
	constant K_HIG   : std_logic_vector(1 downto 0) := "10";
	
	---------------------------------------------------------------------------
	
	constant RA_ip_ident         : integer := (4*0);

	---------------------------------------------------------------------------
	
	constant C_DECAD_WIDTH       : integer := 1;
	
	---------------------------------------------------------------------------
	
	function adrtoint(ad : std_logic_vector) return integer is
		variable ar : std_logic_vector(C_DECAD_WIDTH+1 downto 0);
	begin
		ar := ad(ar'left downto 2) & "00";
		return to_integer(unsigned(ar));
	end function;
	
	---------------------------------------------------------------------------

	-- ip identification
	constant ip_id_code  : integer := 15;
	constant ip_id_majv  : integer := 0;
	constant ip_id_minv  : integer := 1;
	constant ip_id_hscr  : integer := 97;
	
	constant ip_ident_k : std_logic_vector(31 downto 0) :=
		std_logic_vector(to_unsigned(ip_id_code,8)) &
		std_logic_vector(to_unsigned(ip_id_majv,8)) &
		std_logic_vector(to_unsigned(ip_id_minv,8)) &
		std_logic_vector(to_unsigned(ip_id_hscr,8));
	
	-- access delay
	signal host_en_dl1  : std_logic;
	
	---------------------------------------------------------------------------

	-- bus2reg
	function loadreg(s : std_logic_vector) return std_logic_vector is
		variable r : std_logic_vector(31 downto 0);
	begin
		r := (others => '0');
		r(s'range) := s;
		return r;
	end function;

	-- reg2bus
	procedure storereg(signal d : out std_logic_vector; signal s : in std_logic_vector) is
	begin
		d <= s(d'range);
	end procedure;
	
	---------------------------------------------------------------------------
	
	---------------------------------------------------------------------------
	-- Inverter Values indexes
	
	constant C_IVD_IPHS_A : integer := 0;
	constant C_IVD_IPHS_B : integer := 1;
	constant C_IVD_IPHS_C : integer := 2;
	constant C_IVD_IBUS_X : integer := 3;
	constant C_IVD_VPHS_A : integer := 4;
	constant C_IVD_VPHS_B : integer := 5;
	constant C_IVD_VPHS_C : integer := 6;
	constant C_IVD_VBUS_X : integer := 7;
	constant C_IVD_NUM    : integer := 8;
		
	---------------------------------------------------------------------------
	-- ACQ (acquisition)
	
	type   invdat_acq_t is array(0 to C_IVD_NUM-1) of std_logic_vector(11 downto 0);
	signal invdat_acq : invdat_acq_t := (others => (others => '0'));
	signal invdat_eoc : std_logic;
	
	---------------------------------------------------------------------------
	-- RAW (raw: final size)
	
	type   invdat_raw_t is array(0 to C_IVD_NUM-1) of std_logic_vector(17 downto 0);
	signal invdat_raw : invdat_raw_t := (others => (others => '0'));
	
	---------------------------------------------------------------------------
	-- BIP (unipolar to bipolar)
	
	type   invdat_bip_t is array(0 to C_IVD_NUM-1) of std_logic_vector(17 downto 0);
	signal invdat_bip : invdat_bip_t := (others => (others => '0'));
	
	---------------------------------------------------------------------------
	-- coil drive
	
	-- gate unit output enable for automatic mode
	signal coilenx     : std_logic;
	
	---------------------------------------------------------------------------
	
	-- events to counts
	signal guevsrc : std_logic_vector(2 downto 0);
	
	-- switches counter per leg
	signal guswcnt_leg : std_logic_vector(30 downto 0);
	
	---------------------------------------------------------------------------
	
	---------------------------------------------------------------------------
	-- outputs
	---------------------------------------------------------------------------
	
	
	---------------------------------------------------------------------------
	
	-- host interface
	signal host_dout_i        :  std_logic_vector(31 downto 0);
	signal host_ack_i         :  std_logic;
	
	---------------------------------------------------------------------------
	-- board phy

	-- Gate unit control
	signal gu_cmd_d_i       : std_logic_vector(2 downto 0);
	signal gu_cmd_e_i       : std_logic_vector(2 downto 0);

	-- Analog acquisiton
	signal adc_sck_i       : std_logic;
	signal adc_cs_i        : std_logic;
	signal adc_ch_i        : std_logic;

	---------------------------------------------------------------------------
	
	-- motor control
	
	-- external angle
	signal ext_angle_i   : std_logic_vector(31 downto 0);
	
	-- resolver
	signal rsv_priexc_i  : std_logic_vector(17 downto 0);
	signal rsv_secsin_i  : std_logic_vector(17 downto 0);
	signal rsv_seccos_i  : std_logic_vector(17 downto 0);
	
	-- hall sensor
	signal hls_hallsig_i : std_logic_vector(5 downto 0);
	
	-- digital encoder
	signal enc_cha_i     : std_logic;
	signal enc_chb_i     : std_logic;
	signal enc_chi_i     : std_logic;
	
	-- input data
	--
	signal acq_iphs_a_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_b_i  : std_logic_vector(17 downto 0);
	signal acq_iphs_c_i  : std_logic_vector(17 downto 0);
	signal acq_ibus_x_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_a_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_b_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_c_i  : std_logic_vector(17 downto 0);
	signal acq_vbus_x_i  : std_logic_vector(17 downto 0);
	signal acq_vphs_n_i  : std_logic_vector(17 downto 0);
	--
	signal acq_sync_i    : std_logic;
	
	-- gate unit switches
	signal guswcnt_i     : std_logic_vector(31 downto 0);
	
	-- in-failure
	signal hwfail_i      : std_logic;
	
	-- in-kill
	signal hwkill_i      : std_logic;

begin

	---------------------------------------------------------------------------
	
	-- host interface
	host_dout      <= host_dout_i;
	host_ack       <= host_ack_i;

	---------------------------------------------------------------------------
	
	-- board phy

	-- gate unit control
	gu_cmd_d       <= gu_cmd_d_i;
	gu_cmd_e       <= gu_cmd_e_i;

	-- analog acq.
	adc_sck        <= adc_sck_i;
	adc_cs         <= adc_cs_i;
	adc_ch         <= adc_ch_i;

	---------------------------------------------------------------------------
	
	-- motor control
	
	-- external angle
	ext_angle   <= ext_angle_i;
	
	-- resolver
	rsv_priexc  <= rsv_priexc_i;
	rsv_secsin  <= rsv_secsin_i;
	rsv_seccos  <= rsv_seccos_i;
	
	-- hall sensor
	hls_hallsig <= hls_hallsig_i;
	
	-- digital encoder
	enc_cha     <= enc_cha_i;
	enc_chb     <= enc_chb_i;
	enc_chi     <= enc_chi_i;
	
	-- input data
	--
	acq_iphs_a  <= acq_iphs_a_i;
	acq_iphs_b  <= acq_iphs_b_i;
	acq_iphs_c  <= acq_iphs_c_i;
	acq_ibus_x  <= acq_ibus_x_i;
	acq_vphs_a  <= acq_vphs_a_i;
	acq_vphs_b  <= acq_vphs_b_i;
	acq_vphs_c  <= acq_vphs_c_i;
	acq_vbus_x  <= acq_vbus_x_i;
	acq_vphs_n  <= acq_vphs_n_i;
	--
	acq_sync    <= acq_sync_i;
	
	-- gate unit switches
	guswcnt     <= guswcnt_i;
	
	-- in-failure
	hwfail      <= hwfail_i;
	
	-- in-kill
	hwkill      <= hwkill_i;
	
	---------------------------------------------------------------------------
	
	-- output to motor control ip
	enc_cha_i <= '0';
	enc_chb_i <= '0';
	enc_chi_i <= '0';
	
	-- output to motor control ip
	hls_hallsig_i <= (others => '0');
	
	---------------------------------------------------------------------------
	
	-- external angle not inplemented
	ext_angle_i <= (others => '0');
	
	-- resolver not inplemented
	rsv_priexc_i <= (others => '0');
	rsv_secsin_i <= (others => '0');
	rsv_seccos_i <= (others => '0');
	
	---------------------------------------------------------------------------
	
	---------------------------------------------------------------------------
	-- report acq time
	assert false
		report "Acquisition time is : " & real'image(real(C_INV_ACQTIME_NS) * 1.0e-3)
		severity note;

	---------------------------------------------------------------------------
	
	adc_inst : ads7947core
		generic map
		(
			C_CLK_FREQ_HZ  => C_HOST_CLK_FREQ_HZ,
			C_ACQTIME_NS   => C_INV_ACQTIME_NS
		)
		port map
		( 
			clock  => host_clk,
			reset  => host_rst,
			sdo    => adc_dat,
			sclk   => adc_sck_i,
			cs     => adc_cs_i,
			ch_sel => adc_ch_i,
			pdo00  => invdat_acq(C_IVD_VPHS_A),
			pdo01  => invdat_acq(C_IVD_IPHS_A),
			pdo10  => invdat_acq(C_IVD_VPHS_B),
			pdo11  => invdat_acq(C_IVD_IPHS_B),
			pdo20  => invdat_acq(C_IVD_VPHS_C),
			pdo21  => invdat_acq(C_IVD_IPHS_C),
			pdo30  => invdat_acq(C_IVD_VBUS_X),
			pdo31  => invdat_acq(C_IVD_IBUS_X),
			finish => invdat_eoc
		);
		
	-- final size
	INV_ACQ2RAW_GEN : for i in invdat_raw'range generate
		
		INV_ACQ2RAW_I_GEN : if (i = C_IVD_IBUS_X or i = C_IVD_IPHS_A or i = C_IVD_IPHS_B or i = C_IVD_IPHS_C) generate
			invdat_raw(i) <= invdat_acq(i) & "000000";
		end generate;
		
		INV_ACQ2RAW_V_GEN : if (i = C_IVD_VBUS_X or i = C_IVD_VPHS_A or i = C_IVD_VPHS_B or i = C_IVD_VPHS_C) generate
			invdat_raw(i) <= "0" & invdat_acq(i) & "00000";
		end generate;
		
	end generate;
	
	-- unipolar to bipolar
	INV_RAW2BIP_GEN : for i in invdat_bip'range generate
	
		INV_RAW2BIP_I_GEN : if (i = C_IVD_IBUS_X or i = C_IVD_IPHS_A or i = C_IVD_IPHS_B or i = C_IVD_IPHS_C) generate
			invdat_bip(i) <= std_logic_vector(signed(invdat_raw(i)) - 2**17);
		end generate;
		
		INV_RAW2BIP_V_GEN : if (i = C_IVD_VBUS_X or i = C_IVD_VPHS_A or i = C_IVD_VPHS_B or i = C_IVD_VPHS_C) generate
			invdat_bip(i) <= std_logic_vector(signed(invdat_raw(i)) - 0);
		end generate;
		
	end generate;
	
	-- output to motor control ip
	acq_iphs_a_i <= invdat_bip(C_IVD_IPHS_A);
	acq_iphs_b_i <= invdat_bip(C_IVD_IPHS_B);
	acq_iphs_c_i <= invdat_bip(C_IVD_IPHS_C);
	acq_ibus_x_i <= invdat_bip(C_IVD_IBUS_X);
	acq_vphs_a_i <= invdat_bip(C_IVD_VPHS_A);
	acq_vphs_b_i <= invdat_bip(C_IVD_VPHS_B);
	acq_vphs_c_i <= invdat_bip(C_IVD_VPHS_C);
	acq_vbus_x_i <= invdat_bip(C_IVD_VBUS_X);
	acq_vphs_n_i <= (others => '0');
	acq_sync_i   <= invdat_eoc;
	
	---------------------------------------------------------------------------
	
	-- enable coil: enabled and 3-phase only and 2-levels
	coilenx <= '1' when (coilgear = '1' and is3phases = '1' and modlevels = MODLEVELS_2) else '0';
	
	-- mux PWM/RPFM 2 levels
	gu_cmd_d_i <= rpfm3p_l2cxw1 when modtype = MODTYPE_RPFM else pwm3p_l2cxw1;

	-- GU enable
	gu_cmd_e_i <= (others => coilenx);
	
	---------------------------------------------------------------------------
	
	-- gate switches counter
	
	guevsrc <= gu_cmd_d_i;
	
	GUSWCNT_INST : evxcount
		generic map
		(
			C_DATA_WIDTH  => 31,
			C_EVENT_WIDTH => 3,
			C_COUNT_MODE  => 0
		)
		port map
		(
			clock   => host_clk,
			reset   => host_rst,
			sigevt  => guevsrc,
			counter => guswcnt_leg
		);
		
	guswcnt_i <= guswcnt_leg & "0";
	
	---------------------------------------------------------------------------
	
	-- gate unit not enable is also KILL
	hwkill_i <= '0';
	
	-- gu error (comulative)
	hwfail_i <= '0'; -- not isn_fault;
	
	---------------------------------------------------------------------------
	
	-- implement slave model software accessible register(s)
	SLAVE_WRITE_PROC : process(host_clk)
	begin
		if rising_edge(host_clk) then
		
			if host_rst = '1' then
			
				null;
				
			elsif host_we = '1' and host_en = '1' then
			
				case adrtoint(host_addr) is
					when RA_ip_ident     => null;
					when others          => null;
				end case;
			end if;
		end if;
	end process;

	-- implement slave model software accessible register(s)
	SLAVE_READ_PROC : process(host_clk)
		variable v : std_logic_vector(31 downto 0);
	begin
		if rising_edge(host_clk) then
			v := (others => '0');
			case adrtoint(host_addr) is
				when RA_ip_ident     => v := loadreg(ip_ident_k);
				when others          => null;
			end case;
			host_dout_i <= v;
		end if;
	end process;
	
	-- data transfer ack
	host_ack_i <= host_we or host_en_dl1;
	
	-- host access delay
	HOSTDLY_PROC : process(host_clk)
	begin
		if rising_edge(host_clk) then
			host_en_dl1 <= host_en;
		end if;
	end process;

end Behavioral;
