-------------------------------------------------------------------------------------------
-- Copyright © 2011-2016, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    11:05:36 05/30/2016 
-- Design Name: 
-- Module Name:    axi_uart4bus - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity axi_uart4bus is
	generic
	(
		-- FPGA Family Parameter      
		C_FAMILY             : string := "zynq";

		-- AXI bus block speed
		C_S_AXI_ACLK_FREQ_HZ : integer := 100_000_000;

		-- Parameters of Axi Slave Bus Interface S_AXI
		C_S_AXI_DATA_WIDTH	: integer range 32 to 32 := 32;
		C_S_AXI_ADDR_WIDTH	: integer range 16 to 16 := 16;

		-- IP generics
		C_SYSTEMCLOCK        : integer := 100_000_000;
		--
		C_UART_NUM           : integer := 16;
		--
		C_SPACE_VALUE        : integer := 0;
		C_EDATA_VALUE        : integer := 1;
		C_CLKDIV_WIDTH       : integer := 16;
		C_WEIGHT_WIDTH       : integer := 8;
		C_TXFIFO_DEPTH       : integer := 128;
		C_RXFIFO_DEPTH       : integer := 128;
		--
		C_GPIO_WIDTH         : integer := 32;
		C_GPIO_IN_ENA        : integer := 1;
		C_GPIO_OUT_ENA       : integer := 1;
		C_GPIO_TRI_ENA       : integer := 1;
		C_GPIO_OUT_DEF       : std_logic_vector (31 downto 0) := X"0000_0000";
		C_GPIO_TRI_DEF       : std_logic_vector (31 downto 0) := X"FFFF_FFFF"
	);
	port
	(
		-- Ports of Axi Slave Bus Interface S_AXI
		S_AXI_ACLK      : in  std_logic;
		S_AXI_ARESETN   : in  std_logic;
		S_AXI_AWADDR    : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 DOWNTO 0);
		S_AXI_AWVALID   : in  std_logic;
		S_AXI_AWREADY   : out std_logic;
		S_AXI_WDATA     : in  std_logic_vector(C_S_AXI_DATA_WIDTH-1 DOWNTO 0);
		S_AXI_WSTRB     : in  std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 DOWNTO 0);
		S_AXI_WVALID    : in  std_logic;
		S_AXI_WREADY    : out std_logic;
		S_AXI_BRESP     : out std_logic_vector(1 DOWNTO 0);
		S_AXI_BVALID    : out std_logic;
		S_AXI_BREADY    : in  std_logic;
		S_AXI_ARADDR    : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 DOWNTO 0);
		S_AXI_ARVALID   : in  std_logic;
		S_AXI_ARREADY   : out std_logic;
		S_AXI_RDATA     : out std_logic_vector(C_S_AXI_DATA_WIDTH-1 DOWNTO 0);
		S_AXI_RRESP     : out std_logic_vector(1 DOWNTO 0);
		S_AXI_RVALID    : out std_logic;
		S_AXI_RREADY    : in  std_logic;
		--
		-- uart
		uart_txe        : out std_logic_vector(C_UART_NUM-1 downto 0);
		uart_txd        : out std_logic_vector(C_UART_NUM-1 downto 0);
		uart_rxd        : in  std_logic_vector(C_UART_NUM-1 downto 0) := (others => '1');
		--
		-- gpio
		gpio_io_in      : in  std_logic_vector(C_GPIO_WIDTH-1 downto 0) := (others => '0');
		gpio_io_out     : out std_logic_vector(C_GPIO_WIDTH-1 downto 0);
		gpio_io_tri     : out std_logic_vector(C_GPIO_WIDTH-1 downto 0)
	);
end axi_uart4bus;

architecture Behavioral of axi_uart4bus is

	component axi4ltiny
		generic
		(
			C_S_AXI_DATA_WIDTH : integer range 32 to 32 := 32;
			C_S_AXI_ADDR_WIDTH : integer range  1 to 32 := 16
		);
		port
		(
			-- AXI4LITE
			S_AXI_ACLK    : in  std_logic;
			S_AXI_ARESETN : in  std_logic;
			S_AXI_AWADDR  : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
			S_AXI_AWVALID : in  std_logic;
			S_AXI_AWREADY : out std_logic;
			S_AXI_WDATA   : in  std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
			S_AXI_WSTRB   : in  std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 downto 0);
			S_AXI_WVALID  : in  std_logic;
			S_AXI_WREADY  : out std_logic;
			S_AXI_BRESP   : out std_logic_vector(1 downto 0);
			S_AXI_BVALID  : out std_logic;
			S_AXI_BREADY  : in  std_logic;
			S_AXI_ARADDR  : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
			S_AXI_ARVALID : in  std_logic;
			S_AXI_ARREADY : out std_logic;
			S_AXI_RDATA   : out std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
			S_AXI_RRESP   : out std_logic_vector(1 downto 0);
			S_AXI_RVALID  : out std_logic;
			S_AXI_RREADY  : in  std_logic;
			-- local bus  
			host_clk      : out std_logic;
			host_rst      : out std_logic;
			host_en       : out std_logic;
			host_we       : out std_logic;
			host_addr     : out std_logic_vector(31 downto 0);
			host_din      : out std_logic_vector(31 downto 0);
			host_dout     : in  std_logic_vector(31 downto 0);
			host_ack      : in  std_logic
		);
	end component;
	
	component uart4bus
		generic
		(
			C_FAMILY           : string  := "zynq";
			C_HOST_CLK_FREQ_HZ : integer := 100_000_000;
			--
			C_UART_NUM         : integer := 16;
			--
			C_SPACE_VALUE      : integer := 0;
			C_EDATA_VALUE      : integer := 1;
			C_CLKDIV_WIDTH     : integer := 16;
			C_WEIGHT_WIDTH     : integer := 8;
			C_TXFIFO_DEPTH     : integer := 128;
			C_RXFIFO_DEPTH     : integer := 128;
			--
			C_GPIO_WIDTH       : integer := 32;
			C_GPIO_IN_ENA      : integer := 1;
			C_GPIO_OUT_ENA     : integer := 1;
			C_GPIO_TRI_ENA     : integer := 1;
			C_GPIO_OUT_DEF     : std_logic_vector (31 downto 0) := X"0000_0000";
			C_GPIO_TRI_DEF     : std_logic_vector (31 downto 0) := X"FFFF_FFFF"
		);
		port 
		(
			-- host interface
			host_clk    : in  std_logic;
			host_rst    : in  std_logic;
			host_en     : in  std_logic;
			host_we     : in  std_logic;
			host_addr   : in  std_logic_vector(31 downto 0);
			host_din    : in  std_logic_vector(31 downto 0);
			host_dout   : out std_logic_vector(31 downto 0);
			host_ack    : out std_logic;
			--
			-- uart
			uart_txe    : out std_logic_vector(C_UART_NUM-1 downto 0);
			uart_txd    : out std_logic_vector(C_UART_NUM-1 downto 0);
			uart_rxd    : in  std_logic_vector(C_UART_NUM-1 downto 0);
			--
			-- gpio
			gpio_io_in  : in  std_logic_vector(C_GPIO_WIDTH-1 downto 0);
			gpio_io_out : out std_logic_vector(C_GPIO_WIDTH-1 downto 0);
			gpio_io_tri : out std_logic_vector(C_GPIO_WIDTH-1 downto 0)
		);
	end component;
	
	-- wrapper
	
	constant C_HOST_CLK_FREQ_HZ : integer := C_S_AXI_ACLK_FREQ_HZ;

	signal host_clk        : std_logic;
	signal host_rst        : std_logic;
	--
	signal host_en         : std_logic;
	signal host_we         : std_logic;
	signal host_addr       : std_logic_vector(31 downto 0);
	signal host_din        : std_logic_vector(31 downto 0);
	signal host_dout       : std_logic_vector(31 downto 0);
	signal host_ack        : std_logic;
	

begin

	assert C_S_AXI_ACLK_FREQ_HZ = C_SYSTEMCLOCK report "AXI clock /= system clock" severity failure;
	assert false report "C_SYSTEMCLOCK=" & integer'image(C_SYSTEMCLOCK) severity note;

	AXI_S_INST : axi4ltiny
		generic map
		(
			C_S_AXI_DATA_WIDTH => C_S_AXI_DATA_WIDTH,
			C_S_AXI_ADDR_WIDTH => C_S_AXI_ADDR_WIDTH
		)
		port map
		(
			--
			S_AXI_ACLK    => S_AXI_ACLK,
			S_AXI_ARESETN => S_AXI_ARESETN,
			S_AXI_AWADDR  => S_AXI_AWADDR,
			S_AXI_AWVALID => S_AXI_AWVALID,
			S_AXI_AWREADY => S_AXI_AWREADY,
			S_AXI_WDATA   => S_AXI_WDATA,
			S_AXI_WSTRB   => S_AXI_WSTRB,
			S_AXI_WVALID  => S_AXI_WVALID,
			S_AXI_WREADY  => S_AXI_WREADY,
			S_AXI_BRESP   => S_AXI_BRESP,
			S_AXI_BVALID  => S_AXI_BVALID,
			S_AXI_BREADY  => S_AXI_BREADY,
			S_AXI_ARADDR  => S_AXI_ARADDR,
			S_AXI_ARVALID => S_AXI_ARVALID,
			S_AXI_ARREADY => S_AXI_ARREADY,
			S_AXI_RDATA   => S_AXI_RDATA,
			S_AXI_RRESP   => S_AXI_RRESP,
			S_AXI_RVALID  => S_AXI_RVALID,
			S_AXI_RREADY  => S_AXI_RREADY,
			--
			host_clk      => host_clk,
			host_rst      => host_rst,
			host_en       => host_en,
			host_we       => host_we,
			host_addr     => host_addr,
			host_din      => host_din,
			host_dout     => host_dout,
			host_ack      => host_ack
		);
	
	
	UART4BUS_INST : uart4bus
		generic map
		(
			C_FAMILY           => C_FAMILY,
			C_HOST_CLK_FREQ_HZ => C_HOST_CLK_FREQ_HZ,

			C_UART_NUM         => C_UART_NUM,

			C_SPACE_VALUE      => C_SPACE_VALUE,
			C_EDATA_VALUE      => C_EDATA_VALUE,
			C_CLKDIV_WIDTH     => C_CLKDIV_WIDTH,
			C_WEIGHT_WIDTH     => C_WEIGHT_WIDTH,
			C_TXFIFO_DEPTH     => C_TXFIFO_DEPTH,
			C_RXFIFO_DEPTH     => C_RXFIFO_DEPTH,

			C_GPIO_WIDTH       => C_GPIO_WIDTH,
			C_GPIO_IN_ENA      => C_GPIO_IN_ENA,
			C_GPIO_OUT_ENA     => C_GPIO_OUT_ENA,
			C_GPIO_TRI_ENA     => C_GPIO_TRI_ENA,
			C_GPIO_OUT_DEF     => C_GPIO_OUT_DEF,
			C_GPIO_TRI_DEF     => C_GPIO_TRI_DEF
		)
		port map
		(
			host_clk    => host_clk,
			host_rst    => host_rst,
			host_en     => host_en,
			host_we     => host_we,
			host_addr   => host_addr,
			host_din    => host_din,
			host_dout   => host_dout,
			host_ack    => host_ack,

			uart_txe    => uart_txe,
			uart_txd    => uart_txd,
			uart_rxd    => uart_rxd,

			gpio_io_in  => gpio_io_in,
			gpio_io_out => gpio_io_out,
			gpio_io_tri => gpio_io_tri
		);

end Behavioral;
