-------------------------------------------------------------------------------------------
-- Copyright © 2011-2017, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    13:16:52 07/31/2013 
-- Design Name: 
-- Module Name:    axi_zedodisp - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity axi_zedodisp is
	generic
	(
		-- FPGA Family Parameter      
		C_FAMILY             : string := "zynq";

		-- AXI bus block speed
		C_S_AXI_ACLK_FREQ_HZ : integer := 100_000_000;

		-- Parameters of Axi Slave Bus Interface S_AXI
		C_S_AXI_DATA_WIDTH	: integer range 32 to 32 := 32;
		C_S_AXI_ADDR_WIDTH	: integer range 16 to 16 := 16;

		-- IP generics
		C_SYSTEMCLOCK       : integer := 100_000_000;
		C_SCLK_FREQ_HZ      : integer := 5000000;
		C_FMEM_READ_EN      : integer := 1
	);
	port
	(
		-- Ports of Axi Slave Bus Interface S_AXI
		S_AXI_ACLK      : in  std_logic;
		S_AXI_ARESETN   : in  std_logic;
		S_AXI_AWADDR    : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 DOWNTO 0);
		S_AXI_AWVALID   : in  std_logic;
		S_AXI_AWREADY   : out std_logic;
		S_AXI_WDATA     : in  std_logic_vector(C_S_AXI_DATA_WIDTH-1 DOWNTO 0);
		S_AXI_WSTRB     : in  std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 DOWNTO 0);
		S_AXI_WVALID    : in  std_logic;
		S_AXI_WREADY    : out std_logic;
		S_AXI_BRESP     : out std_logic_vector(1 DOWNTO 0);
		S_AXI_BVALID    : out std_logic;
		S_AXI_BREADY    : in  std_logic;
		S_AXI_ARADDR    : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 DOWNTO 0);
		S_AXI_ARVALID   : in  std_logic;
		S_AXI_ARREADY   : out std_logic;
		S_AXI_RDATA     : out std_logic_vector(C_S_AXI_DATA_WIDTH-1 DOWNTO 0);
		S_AXI_RRESP     : out std_logic_vector(1 DOWNTO 0);
		S_AXI_RVALID    : out std_logic;
		S_AXI_RREADY    : in  std_logic;
		-- ip ports
		oled_vbat       : out std_logic;
		oled_vdd        : out std_logic;
		oled_res        : out std_logic;
		oled_dc         : out std_logic;
		oled_sclk       : out std_logic;
		oled_sdin       : out std_logic
	);

end entity axi_zedodisp;


architecture IMP of axi_zedodisp is

	component axi4ltiny
		generic
		(
			C_S_AXI_DATA_WIDTH : integer range 32 to 32 := 32;
			C_S_AXI_ADDR_WIDTH : integer range  1 to 32 := 16
		);
		port
		(
			-- AXI4LITE
			S_AXI_ACLK    : in  std_logic;
			S_AXI_ARESETN : in  std_logic;
			S_AXI_AWADDR  : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
			S_AXI_AWVALID : in  std_logic;
			S_AXI_AWREADY : out std_logic;
			S_AXI_WDATA   : in  std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
			S_AXI_WSTRB   : in  std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 downto 0);
			S_AXI_WVALID  : in  std_logic;
			S_AXI_WREADY  : out std_logic;
			S_AXI_BRESP   : out std_logic_vector(1 downto 0);
			S_AXI_BVALID  : out std_logic;
			S_AXI_BREADY  : in  std_logic;
			S_AXI_ARADDR  : in  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
			S_AXI_ARVALID : in  std_logic;
			S_AXI_ARREADY : out std_logic;
			S_AXI_RDATA   : out std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
			S_AXI_RRESP   : out std_logic_vector(1 downto 0);
			S_AXI_RVALID  : out std_logic;
			S_AXI_RREADY  : in  std_logic;
			-- local bus  
			host_clk      : out std_logic;
			host_rst      : out std_logic;
			host_en       : out std_logic;
			host_we       : out std_logic;
			host_addr     : out std_logic_vector(31 downto 0);
			host_din      : out std_logic_vector(31 downto 0);
			host_dout     : in  std_logic_vector(31 downto 0);
			host_ack      : in  std_logic
		);
	end component;
	
	component ssd1306
		generic
		(
			C_FAMILY        : string  := "zynq";
			C_CLOCK_FREQ_HZ : integer := 100000000;
			C_SCLK_FREQ_HZ  : integer := 5000000;
			C_FMEM_READ_EN  : integer := 1
		);
		port 
		(
			clock     : in  std_logic;
			reset     : in  std_logic;
			
			-- host bus
			host_en   : in  std_logic;
			host_we   : in  std_logic;
			host_addr : in  std_logic_vector(31 downto 0);
			host_din  : in  std_logic_vector(31 downto 0);
			host_dout : out std_logic_vector(31 downto 0);
			host_ack  : out std_logic;
			
			-- oled display
			oled_vbat : out std_logic;
			oled_vdd  : out std_logic;
			oled_res  : out std_logic;
			oled_dc   : out std_logic;
			oled_sclk : out std_logic;
			oled_sdin : out std_logic
		);
	end component;

	-- wrapper

	signal host_clk        : std_logic;
	signal host_rst        : std_logic;
	--
	signal host_en         : std_logic;
	signal host_we         : std_logic;
	signal host_addr       : std_logic_vector(31 downto 0);
	signal host_din        : std_logic_vector(31 downto 0);
	signal host_dout       : std_logic_vector(31 downto 0);
	signal host_ack        : std_logic;

begin

	assert C_S_AXI_ACLK_FREQ_HZ = C_SYSTEMCLOCK report "AXI clock /= system clock" severity failure;
	assert false report "C_SYSTEMCLOCK=" & integer'image(C_SYSTEMCLOCK) severity note;
	
	AXI_LITE_IPIF_I : axi4ltiny
		generic map
		(
			C_S_AXI_DATA_WIDTH => C_S_AXI_DATA_WIDTH,
			C_S_AXI_ADDR_WIDTH => C_S_AXI_ADDR_WIDTH
		)
		port map
		(
			-- AXI4LITE
			S_AXI_ACLK    => S_AXI_ACLK,
			S_AXI_ARESETN => S_AXI_ARESETN,
			S_AXI_AWADDR  => S_AXI_AWADDR,
			S_AXI_AWVALID => S_AXI_AWVALID,
			S_AXI_AWREADY => S_AXI_AWREADY,
			S_AXI_WDATA   => S_AXI_WDATA,
			S_AXI_WSTRB   => S_AXI_WSTRB,
			S_AXI_WVALID  => S_AXI_WVALID,
			S_AXI_WREADY  => S_AXI_WREADY,
			S_AXI_BRESP   => S_AXI_BRESP,
			S_AXI_BVALID  => S_AXI_BVALID,
			S_AXI_BREADY  => S_AXI_BREADY,
			S_AXI_ARADDR  => S_AXI_ARADDR,
			S_AXI_ARVALID => S_AXI_ARVALID,
			S_AXI_ARREADY => S_AXI_ARREADY,
			S_AXI_RDATA   => S_AXI_RDATA,
			S_AXI_RRESP   => S_AXI_RRESP,
			S_AXI_RVALID  => S_AXI_RVALID,
			S_AXI_RREADY  => S_AXI_RREADY,
			-- local bus  
			host_clk      => host_clk,
			host_rst      => host_rst,
			host_en       => host_en,
			host_we       => host_we,
			host_addr     => host_addr,
			host_din      => host_din,
			host_dout     => host_dout,
			host_ack      => host_ack
		);
		
  ssd1306_INST : ssd1306
		generic map
		(
			C_FAMILY        => C_FAMILY,
			C_CLOCK_FREQ_HZ => C_SYSTEMCLOCK,
			C_SCLK_FREQ_HZ  => C_SCLK_FREQ_HZ,
			C_FMEM_READ_EN  => C_FMEM_READ_EN
		)
		port map
		(
			clock     => host_clk,
			reset     => host_rst,
			
			-- host bus
			host_en   => host_en,
			host_we   => host_we,
			host_addr => host_addr,
			host_din  => host_din,
			host_dout => host_dout,
			host_ack  => host_ack,
			
			-- oled display
			oled_vbat => oled_vbat,
			oled_vdd  => oled_vdd,
			oled_res  => oled_res,
			oled_dc   => oled_dc,
			oled_sclk => oled_sclk,
			oled_sdin => oled_sdin
		);

end IMP;
