-------------------------------------------------------------------------------------------
-- Copyright  2011-2013, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
-- QDESYS 
-- Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
-------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Company:        QDesys
-- Engineer:       Diego Quagreda
-- 
-- Create Date:    13:40:02 05/16/2013 
-- Design Name: 
-- Module Name:    rgmii_mii - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity rgmii_mii is
	generic
	(
		C_RESET_CLKS_MAX : integer := 250000;
		C_NOT_MAC_TX_CLK : integer range 0 to 1 := 0;
		C_NOT_MAC_RX_CLK : integer range 0 to 1 := 0;
		C_NOT_PHY_TX_CLK : integer range 0 to 1 := 0;
		C_NOT_PHY_RX_CLK : integer range 0 to 1 := 0;
		C_HAS_PHY_TX_REG : integer range 0 to 1 := 0
	);
	port 
	(
		CLK_25_mhz  : in  std_logic;
		MAC_rst_n   : in  std_logic;
		MAC_tx_clk  : out std_logic;
		MAC_tx_data : in  std_logic_vector(3 downto 0);
		MAC_tx_en   : in  std_logic;
		MAC_rx_clk  : out std_logic;
		MAC_rx_data : out std_logic_vector(3 downto 0);
		MAC_rx_er   : out std_logic;
		MAC_col     : out std_logic;
		MAC_crs     : out std_logic;
		MAC_dv      : out std_logic;
		PHY_sysclk  : out std_logic;
		PHY_rst_n   : out std_logic;
		PHY_tx_clk  : out std_logic;
		PHY_tx_data : out std_logic_vector(3 downto 0);
		PHY_tx_ctl  : out std_logic;
		PHY_rx_clk  : in  std_logic;
		PHY_rx_data : in  std_logic_vector(3 downto 0);
		PHY_rx_ctl  : in  std_logic;
		PHY_status  : out std_logic_vector(3 downto 0)
	);
	
end rgmii_mii;

architecture Behavioral of rgmii_mii is

	signal PHY_rx_clk_i  : std_logic;
	
	signal PHY_rx_data_i : std_logic_vector (3 downto 0);
	signal PHY_rx_ctl_i  : std_logic;
	
	signal phy_stmp      : std_logic_vector (3 downto 0) := "0000";
	
	constant C_RESET_CLKS_LIM : integer := C_RESET_CLKS_MAX - 1;
	signal   reset_clks_cnt   : natural range 0 to C_RESET_CLKS_LIM := 0;

begin

	-- RGMII clock
	--   1 Gb :   8 ns -> 125 MHz 
	-- 100 Mb :  40 ns ->  25 MHz
	--  10 Mb : 400 ns -> 2.5 MHz
	
	-------------------------------------------------------------------------------

	DO_AUTO_RESET_GEN : if C_RESET_CLKS_MAX > 1 generate
	
		res_proc : process(CLK_25_mhz)
		begin
			if rising_edge(CLK_25_mhz) then
				if MAC_rst_n = '0' then
					reset_clks_cnt <= 0;
					PHY_rst_n <= '0';
				elsif reset_clks_cnt = C_RESET_CLKS_LIM then
					PHY_rst_n <= '1';
				else 
					reset_clks_cnt <= reset_clks_cnt + 1;
					PHY_rst_n <= '0';
				end if;
			end if;
		end process;
		
	end generate;
	
	NOT_AUTO_RESET_GEN : if C_RESET_CLKS_MAX <= 1 generate
	
		PHY_rst_n <= MAC_rst_n;
		
	end generate;
	

	-------------------------------------------------------------------------------
	
	-- phy system clock
	PHY_sysclk <= CLK_25_mhz;
	
	
	-- TX -------------------------------------------------------------------------
	
	TX_PHY_REG_DO : if C_HAS_PHY_TX_REG = 1 generate
	
		-- reglock of TX control
		TX_CTL_FF_I : FD generic map (INIT => '0') port map ( Q => PHY_tx_ctl, C => CLK_25_mhz, D => MAC_tx_en );
		
		-- reglock of TX data
		TX_DATA_FF_GEN : for i in 3 downto 0 generate
		begin
			TX_DATA_FF_I : FD generic map (INIT => '0') port map ( Q => PHY_tx_data(i), C => CLK_25_mhz, D => MAC_tx_data(i) );
		end generate;
		
		-- PHY TX clock 
		DIR_PHY_TX_CLK_GEN : if C_NOT_PHY_TX_CLK = 0 generate
		begin
			PHY_tx_clk  <= CLK_25_mhz;
		end generate;

		NOT_PHY_TX_CLK_GEN : if C_NOT_PHY_TX_CLK = 1 generate
		begin
			PHY_tx_clk  <= not CLK_25_mhz;
		end generate;
		
		-- MAC TX clock 
		DIR_MAC_TX_CLK_GEN : if C_NOT_MAC_TX_CLK = 0 generate
		begin
			MAC_tx_clk  <= CLK_25_mhz;
		end generate;

		NOT_MAC_TX_CLK_GEN : if C_NOT_MAC_TX_CLK = 1 generate
		begin
			MAC_tx_clk  <= not CLK_25_mhz;
		end generate;
		
	end generate;
	
	TX_PHY_REG_NOT : if C_HAS_PHY_TX_REG = 0 generate
	
		PHY_tx_ctl  <= MAC_tx_en;
		PHY_tx_data <= MAC_tx_data;
		PHY_tx_clk  <= CLK_25_mhz;
		MAC_tx_clk  <= CLK_25_mhz;
		
	end generate;
	
	-- RX -------------------------------------------------------------------------
	
	-- sampling of RX control
	RX_CTL_FF_I : FD generic map (INIT => '0') port map ( Q => PHY_rx_ctl_i, C => PHY_rx_clk_i, D => PHY_rx_ctl );
	
	-- sampling of RX data
	RX_DATA_FF_GEN : for i in 3 downto 0 generate
	begin
		RX_DATA_FF_I : FD generic map (INIT => '0') port map ( Q => PHY_rx_data_i(i), C => PHY_rx_clk_i, D => PHY_rx_data(i) );
	end generate;
	
	-- MAC RX clock 
	DIR_MAC_RX_CLK_GEN : if C_NOT_MAC_RX_CLK = 0 generate
	begin
		MAC_rx_clk  <= PHY_rx_clk;
	end generate;

	NOT_MAC_RX_CLK_GEN : if C_NOT_MAC_RX_CLK = 1 generate
	begin
		MAC_rx_clk  <= not PHY_rx_clk;
	end generate;
	
	
	-- PHY RX clock 
	DIR_PHY_RX_CLK_GEN : if C_NOT_PHY_RX_CLK = 0 generate
	begin
		PHY_rx_clk_i  <= PHY_rx_clk;
	end generate;

	NOT_PHY_RX_CLK_GEN : if C_NOT_PHY_RX_CLK = 1 generate
	begin
		PHY_rx_clk_i  <= not PHY_rx_clk;
	end generate;
	
	
	-- separate data / status
	RX_DEMUX_PROC : process(PHY_rx_clk_i)
	begin
		if rising_edge(PHY_rx_clk_i) then
			if PHY_rx_ctl_i = '0' then
				-- rgmii in-band status
				phy_stmp <= PHY_rx_data_i;
				-- data idle
				MAC_dv      <= '0';
				MAC_rx_data <= (others => '1');
			else
				-- data valid
				MAC_dv      <= '1';
				MAC_rx_data <= PHY_rx_data_i;
			end if;
		end if;
	end process;
	
	-- hold status register
	PHY_status <= phy_stmp;
	
	MAC_col     <= '0'; -- no collision
	MAC_rx_er   <= '0'; -- no error
	
	-- carrier sense
	CRS_PROC : process(CLK_25_mhz)
	begin
		if rising_edge(CLK_25_mhz) then
			if PHY_rx_ctl_i = '1' or MAC_tx_en = '1' then
				MAC_crs <= '1';
			else
				MAC_crs <= '0';
			end if;
		end if;
	end process;
	


end Behavioral;
