/*
-------------------------------------------------------------------------------------------
-- Copyright © 2011-2014, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief ds1820
/// \file ds1820.h
/// \version 0.51
/// \date jun 15, 2013
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2009-2014

#ifndef DS1820_H
#define DS1820_H

/***************************** Include Files *******************************/

#include "xil_types.h"
#include "xil_assert.h"
#include "xil_io.h"
#include "xstatus.h"

/***************************************************************************/

#include "ds1820_lx.h"

/**************************** Type Definitions *****************************/

typedef struct {

	u16 DeviceId;
	u32 BaseAddress;
	
} ds1820_Config;

extern ds1820_Config ds1820_ConfigTable[];

typedef struct {

	ds1820_Config * Config;
	
	u32 IsReady;
	
	u8 lastk; // last conflict index
	u8 lastd;  // last device found
	u8 ident[DS1820_IDENT_SIZE];
	u8 scrpd[DS1820_SCRPD_SIZE];
	
	#define  DS1020_STACK_SIZE 8
	u8 stack[DS1020_STACK_SIZE];
	u8 stkpnt;
	
	u8 state; // state machine code
	
	u8 cmd;
	u8 arg;
	u8 indata;
	u8 wribyte;
	u8 idx;
	u8 maskbit;
	u8 outdata;
	u8 crc;
	u8 bangpos;
	u8 b0;
	
} ds1820;

// command codes
enum {
	DS1820_CMD_INIT,
	DS1820_CMD_RUN,
	DS1820_CMD_KICK
};

// states
enum {

	// initial state
	DS1820_STATE_INIT,

	// Temperature conversion
	DS1820_STATE_TCONV,
	DS1820_STATE_TCONV_STARTED,
	DS1820_STATE_TCONV_W4END,
	DS1820_STATE_TCONV_DONE,

	// Device Search
	DS1820_STATE_SEARCH,
	DS1820_STATE_SEARCH_INIT,
	DS1820_STATE_SEARCH_ITIALIZED,
	DS1820_STATE_SEARCH_LOOP,
	DS1820_STATE_SEARCH_LOOP_B0,
	DS1820_STATE_SEARCH_LOOP_B1,
	DS1820_STATE_SEARCH_LOOP_SAVEBIT,
	DS1820_STATE_SEARCH_DONE_FOUND,

	// read scratch pad memory
	DS1820_STATE_READ_TEMP,
	DS1820_STATE_READ_TEMP_COMD,
	DS1820_STATE_READ_TEMP_LOOP,
	DS1820_STATE_READ_TEMP_LOOP_BIT,
	DS1820_STATE_READ_TEMP_DONE,

	// last device
	DS1820_STATE_LAST_DEVICE,

	// subroutine "command"
	DS1820_STATE_COMMAND,
	DS1820_STATE_COMMAND_INITALIZED,
	DS1820_STATE_COMMAND_ADRMATCH,
	DS1820_STATE_COMMAND_DOCMD,

	// subroutine "initialize"
	DS1820_STATE_XINIT,
	DS1820_STATE_XINIT_W4READY,
	DS1820_STATE_XINIT_W4DONE,


	// Write byte
	DS1820_STATE_WRITE_BYTE,
	DS1820_STATE_WRITE_BYTE_LOOP,

	// Write Bit
	DS1820_STATE_WRITE_BIT,

	// Read Bit
	DS1820_STATE_READ_BIT,

	// Wait for Done
	DS1820_STATE_W4DONE,
	DS1820_STATE_W4DONE_EPLG,

	// Wait for Ready (not busy)
	DS1820_STATE_W4READY
};

// events
enum {
	DS1820_EVENT_ERROR,
	DS1820_EVENT_NONE,
	DS1820_EVENT_TCONV_STARTED,
	DS1820_EVENT_TCONV_RUNNING,
	DS1820_EVENT_DEVICE_FOUND,
	DS1820_EVENT_TEMP_READED,
	DS1820_EVENT_END_OF_SCAN
};


/***************** Macros (Inline Functions) Definitions *******************/

/**
 *
 * Write a value to a DS1820 register. A 32 bit write is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is written.
 *
 * @param   BaseAddress is the base address of the DS1820 device.
 * @param   RegOffset is the register offset from the base to write to.
 * @param   Data is the data written to the register.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void DS1820_mWriteReg(u32 BaseAddress, unsigned RegOffset, u32 Data)
 *
 */
#define DS1820_mWriteReg(BaseAddress, RegOffset, Data) \
 	Xil_Out32((BaseAddress) + (RegOffset), (u32)(Data))

/**
 *
 * Read a value from a DS1820 register. A 32 bit read is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is read from the register. The most significant data
 * will be read as 0.
 *
 * @param   BaseAddress is the base address of the DS1820 device.
 * @param   RegOffset is the register offset from the base to write to.
 *
 * @return  Data is the data from the register.
 *
 * @note
 * C-style signature:
 * 	u32 DS1820_mReadReg(u32 BaseAddress, unsigned RegOffset)
 *
 */
#define DS1820_mReadReg(BaseAddress, RegOffset) \
 	Xil_In32((BaseAddress) + (RegOffset))


/***************************************************************************/

//
// Write registers
//

// #define DS1820_mWrite_ip_ident(BaseAddress,Value)    DS1820_mWriteReg((BaseAddress), DS1820_RA_ip_ident_OFFSET   ,(Value))
// #define DS1820_mWrite_status_reg(BaseAddress,Value)  DS1820_mWriteReg((BaseAddress), DS1820_RA_status_reg_OFFSET ,(Value))
   #define DS1820_mWrite_control_reg(BaseAddress,Value) DS1820_mWriteReg((BaseAddress), DS1820_RA_control_reg_OFFSET,(Value))
	
//
// Read registers
//

   #define DS1820_mRead_ip_ident(BaseAddress)           DS1820_mReadReg((BaseAddress), DS1820_RA_ip_ident_OFFSET   )
   #define DS1820_mRead_status_reg(BaseAddress)         DS1820_mReadReg((BaseAddress), DS1820_RA_status_reg_OFFSET )
   #define DS1820_mRead_control_reg(BaseAddress)        DS1820_mReadReg((BaseAddress), DS1820_RA_control_reg_OFFSET)

/************************** Function Prototypes ****************************/

ds1820_Config *ds1820_LookupConfig(u16 DeviceId);
int ds1820_Initialize(ds1820 *InstancePtr, u16 DeviceId);
int ds1820_Down(u16 DeviceId);

int ds1820_Process_Run(ds1820 *InstancePtr, u8 cmd);
int ds1820_Process_Event(ds1820 *InstancePtr);

#endif /** DS1820_H */
