/*
-------------------------------------------------------------------------------------------
-- Copyright © 2011-2014, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief ds1820 ap lib
/// \file ds1820_apd.c
/// \version 0.51
/// \date jun 15, 2013
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2009-2014

#include <stdio.h>
#include "xparameters.h"
#include "xil_assert.h"

#include "ds1820.h"
#include "ds1820_apc.h"
#include "ds1820_apd.h"

//
// display device identification code
//

void ds1820_ap_ident_disp(ds1820 *InstancePtr)
{
	if (InstancePtr == NULL) {
		return;
	}

	int i;

	printf("IDENT : ");
	for(i = 0; i < DS1820_IDENT_SIZE; i++)
	{
		printf("%02X ",InstancePtr->ident[i]);
	}
	printf("Family code = x%02X : ",InstancePtr->ident[0]);
	switch(InstancePtr->ident[0])
	{
		case DS1820_FAMILY_CODE_DS1820:
			printf("DS1820\n");
			break;
		default:
			printf("Invalid\n");
			break;
	}
}

//
// display scartchpad with temperature
//

void ds1820_ap_temp_disp(ds1820 *InstancePtr)
{
	if (InstancePtr == NULL) {
		return;
	}

	int i;

	printf("SCRPD : ");
	for(i = 0; i < DS1820_SCRPD_SIZE; i++)
	{
		printf("%02X ",InstancePtr->scrpd[i]);
	}

	float temp;
	ds1820_ap_temp_stdres(InstancePtr,&temp);
	printf("Temp = %.4f\n", temp);

}
