/*
-------------------------------------------------------------------------------------------
-- Copyright  2011-2016, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/


#ifndef FMCMC1_DRV_H
#define FMCMC1_DRV_H

/***************************** Include Files *******************************/

#include "xparameters.h"
#include "xil_types.h"
#include "xil_assert.h"
#include "xil_io.h"
#include "xstatus.h"
#include "xenv.h"

/***************************************************************************/

#include "fmcmc1_drv_lx.h"

/**************************** Type Definitions *****************************/

typedef struct {

	u16 DeviceId;
	u32 BaseAddress;

	u32 systemclock;

	u8  carrier;
	u8  hw_link_ref_0;
	u8  hw_link_ref_1;
	u8  xadc;
	
} fmcmc1_drv_Config;

extern fmcmc1_drv_Config fmcmc1_drv_ConfigTable[];

typedef struct {

	u16 r48, r49, r4E, r4F;
	u8  ssr;

} fmcmc1_drv_ap_xadc_t;

typedef struct {

	u32 sd_ctrl_0;
	u32 sd_ctrl_1;

	u32 oc_temp_fkt;
	
	u32 xa_synclkdiv;

	u8  gpio_o;
	u8  gpio_t;

} fmcmc1_drv_ap_ctl_t;

typedef struct {

	u8  status_reg;

	u32 oc_temp_val;

	u8  dip_sw;
	
	u8  gpio_i;
	
} fmcmc1_drv_ap_sts_t;


typedef struct {

	// xadc setup
	fmcmc1_drv_ap_xadc_t xadc;
	
	// board control
	fmcmc1_drv_ap_ctl_t  ctl ;

	// board status
	fmcmc1_drv_ap_sts_t  sts;

} fmcmc1_drv_ap_t;

typedef struct {

	fmcmc1_drv_Config * Config;
	
	fmcmc1_drv_ap_t   * ApPtr;
	
	u32 IsReady;
	
} fmcmc1_drv;

/***************** Macros (Inline Functions) Definitions *******************/

/**
 *
 * Write a value to a FMCMC1_DRV register. A 32 bit write is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is written.
 *
 * @param   BaseAddress is the base address of the FMCMC1_DRV device.
 * @param   RegOffset is the register offset from the base to write to.
 * @param   Data is the data written to the register.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void FMCMC1_DRV_mWriteReg(u32 BaseAddress, unsigned RegOffset, u32 Data)
 *
 */
#define FMCMC1_DRV_mWriteReg(BaseAddress, RegOffset, Data) \
 	Xil_Out32((BaseAddress) + (RegOffset), (u32)(Data))

/**
 *
 * Read a value from a FMCMC1_DRV register. A 32 bit read is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is read from the register. The most significant data
 * will be read as 0.
 *
 * @param   BaseAddress is the base address of the FMCMC1_DRV device.
 * @param   RegOffset is the register offset from the base to write to.
 *
 * @return  Data is the data from the register.
 *
 * @note
 * C-style signature:
 * 	u32 FMCMC1_DRV_mReadReg(u32 BaseAddress, unsigned RegOffset)
 *
 */
#define FMCMC1_DRV_mReadReg(BaseAddress, RegOffset) \
 	Xil_In32((BaseAddress) + (RegOffset))

/***************************************************************************/

//
// Write registers
//

// #define FMCMC1_DRV_mWrite_ip_ident(BaseAddress,Value)      FMCMC1_DRV_mWriteReg((BaseAddress), FMCMC1_DRV_RA_ip_ident_OFFSET     ,(Value))
// #define FMCMC1_DRV_mWrite_status_reg(BaseAddress,Value)    FMCMC1_DRV_mWriteReg((BaseAddress), FMCMC1_DRV_RA_status_reg_OFFSET   ,(Value))

   #define FMCMC1_DRV_mWrite_sd_ctrl_0(BaseAddress,Value)     FMCMC1_DRV_mWriteReg((BaseAddress), FMCMC1_DRV_RA_sd_ctrl_0_OFFSET    ,(Value))
   #define FMCMC1_DRV_mWrite_sd_ctrl_1(BaseAddress,Value)     FMCMC1_DRV_mWriteReg((BaseAddress), FMCMC1_DRV_RA_sd_ctrl_1_OFFSET    ,(Value))

// #define FMCMC1_DRV_mWrite_oc_temp_val(BaseAddress,Value)   FMCMC1_DRV_mWriteReg((BaseAddress), FMCMC1_DRV_RA_oc_temp_val_OFFSET  ,(Value))
   #define FMCMC1_DRV_mWrite_oc_temp_fkt(BaseAddress,Value)   FMCMC1_DRV_mWriteReg((BaseAddress), FMCMC1_DRV_RA_oc_temp_fkt_OFFSET  ,(Value))

   #define FMCMC1_DRV_mWrite_xa_synclkdiv(BaseAddress,Value)  FMCMC1_DRV_mWriteReg((BaseAddress), FMCMC1_DRV_RA_xa_synclkdiv_OFFSET ,(Value))

   #define FMCMC1_DRV_mWrite_xadc_wctreg(BaseAddress,Value)   FMCMC1_DRV_mWriteReg((BaseAddress), FMCMC1_DRV_RA_xadc_wctreg_OFFSET  ,(Value))
// #define FMCMC1_DRV_mWrite_xadc_rctreg(BaseAddress,Value)   FMCMC1_DRV_mWriteReg((BaseAddress), FMCMC1_DRV_RA_xadc_rctreg_OFFSET  ,(Value))

// #define FMCMC1_DRV_mWrite_dip_sw(BaseAddress,Value)        FMCMC1_DRV_mWriteReg((BaseAddress), FMCMC1_DRV_RA_dip_sw_OFFSET       ,(Value))

// #define FMCMC1_DRV_mWrite_gpio_i(BaseAddress,Value)        FMCMC1_DRV_mWriteReg((BaseAddress), FMCMC1_DRV_RA_gpio_i_OFFSET       ,(Value))
   #define FMCMC1_DRV_mWrite_gpio_o(BaseAddress,Value)        FMCMC1_DRV_mWriteReg((BaseAddress), FMCMC1_DRV_RA_gpio_o_OFFSET       ,(Value))
   #define FMCMC1_DRV_mWrite_gpio_t(BaseAddress,Value)        FMCMC1_DRV_mWriteReg((BaseAddress), FMCMC1_DRV_RA_gpio_t_OFFSET       ,(Value))

//                                                            
// Read registers                                             
//                                                            

   #define FMCMC1_DRV_mRead_ip_ident(BaseAddress)             FMCMC1_DRV_mReadReg((BaseAddress), FMCMC1_DRV_RA_ip_ident_OFFSET     )
   #define FMCMC1_DRV_mRead_status_reg(BaseAddress)           FMCMC1_DRV_mReadReg((BaseAddress), FMCMC1_DRV_RA_status_reg_OFFSET   )

   #define FMCMC1_DRV_mRead_sd_ctrl_0(BaseAddress)            FMCMC1_DRV_mReadReg((BaseAddress), FMCMC1_DRV_RA_sd_ctrl_0_OFFSET    )
   #define FMCMC1_DRV_mRead_sd_ctrl_1(BaseAddress)            FMCMC1_DRV_mReadReg((BaseAddress), FMCMC1_DRV_RA_sd_ctrl_1_OFFSET    )

   #define FMCMC1_DRV_mRead_oc_temp_val(BaseAddress)          FMCMC1_DRV_mReadReg((BaseAddress), FMCMC1_DRV_RA_oc_temp_val_OFFSET  )
   #define FMCMC1_DRV_mRead_oc_temp_fkt(BaseAddress)          FMCMC1_DRV_mReadReg((BaseAddress), FMCMC1_DRV_RA_oc_temp_fkt_OFFSET  )

   #define FMCMC1_DRV_mRead_xa_synclkdiv(BaseAddress)         FMCMC1_DRV_mReadReg((BaseAddress), FMCMC1_DRV_RA_xa_synclkdiv_OFFSET )

   #define FMCMC1_DRV_mRead_xadc_wctreg(BaseAddress)          FMCMC1_DRV_mReadReg((BaseAddress), FMCMC1_DRV_RA_xadc_wctreg_OFFSET  )
   #define FMCMC1_DRV_mRead_xadc_rctreg(BaseAddress)          FMCMC1_DRV_mReadReg((BaseAddress), FMCMC1_DRV_RA_xadc_rctreg_OFFSET  )

   #define FMCMC1_DRV_mRead_dip_sw(BaseAddress)               FMCMC1_DRV_mReadReg((BaseAddress), FMCMC1_DRV_RA_dip_sw_OFFSET       )

   #define FMCMC1_DRV_mRead_gpio_i(BaseAddress)               FMCMC1_DRV_mReadReg((BaseAddress), FMCMC1_DRV_RA_gpio_i_OFFSET       )
   #define FMCMC1_DRV_mRead_gpio_o(BaseAddress)               FMCMC1_DRV_mReadReg((BaseAddress), FMCMC1_DRV_RA_gpio_o_OFFSET       )
   #define FMCMC1_DRV_mRead_gpio_t(BaseAddress)               FMCMC1_DRV_mReadReg((BaseAddress), FMCMC1_DRV_RA_gpio_t_OFFSET       )

/************************** Function Prototypes ****************************/


fmcmc1_drv_Config *fmcmc1_drv_LookupConfig(u16 DeviceId);
int fmcmc1_drv_Initialize(fmcmc1_drv *InstancePtr, u16 DeviceId);
int fmcmc1_drv_Down(u16 DeviceId);

// sign-extend macro
#define FMCMC1_DRV_sign_extend(v,b) (((v) & (1 << ((b)-1))) ? ((v) | (-(1 << (b)))) : ((v) & ((1 << (b))-1)))

// XADC Temperature convert

// ad / 4 / 16 * 503.975 / 4096 - 273.15
#define FMCMC1_DRV_xa_adi_mcelsius(value)   ((((value) - 142080) * 123) / 64)

#endif /** FMCMC1_DRV_H */
