/*
-------------------------------------------------------------------------------------------
-- Copyright  2011-2016, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief fmcmc1_drv ap lib
/// \file fmcmc1_drv_apc.c
/// \version 0.51
/// \date jun 15, 2013
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2009-2013

//#include <stdio.h>
#include "xparameters.h"
#include "xil_types.h"
#include "xstatus.h"
#include "fmcmc1_drv.h"
#include "fmcmc1_drv_apc.h"

//#include <math.h>

#define QD_FMCMC1_DRV_DEFLADDR                u32 qd_fmcmc1_drv_laddr = InstancePtr->Config->BaseAddress;
#define QD_FMCMC1_DRV_REFLADDR                qd_fmcmc1_drv_laddr

#define QD_XADC_AUTOMA_NTRY_EXEC 100

#define QD_FMCMC1_DRV_RET4ERROR() { if (XST_SUCCESS != rc) return rc; }


int fmcmc1_drv_ap_compile(fmcmc1_drv *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}
	return XST_SUCCESS;
}

// Reset pulse to XADC automa

int fmcmc1_drv_xadc_reset(fmcmc1_drv *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}
		
    // driver base address
	QD_FMCMC1_DRV_DEFLADDR;
	
	u32 xctw = FMCMC1_DRV_mRead_xadc_wctreg(QD_FMCMC1_DRV_REFLADDR);
	
	xctw |= FMCMC1_DRV_WCTREG_XA_RESET;
	FMCMC1_DRV_mWrite_xadc_wctreg(QD_FMCMC1_DRV_REFLADDR,xctw);
	
	xctw &= ~FMCMC1_DRV_WCTREG_XA_RESET;
	FMCMC1_DRV_mWrite_xadc_wctreg(QD_FMCMC1_DRV_REFLADDR,xctw);
	
	return XST_SUCCESS;
}

// R/W XADC register

int fmcmc1_drv_xadc_reg_io(fmcmc1_drv *InstancePtr, const u8 addr, u16 *rval, const u16 wval, const u8 write)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}
		
    // driver base address
	QD_FMCMC1_DRV_DEFLADDR;
	
	// xadc control register (w & r)
	u32 xctw = FMCMC1_DRV_mRead_xadc_wctreg(QD_FMCMC1_DRV_REFLADDR);
	u32 xctr = FMCMC1_DRV_mRead_xadc_rctreg(QD_FMCMC1_DRV_REFLADDR);

	// set value
	xctw &= ~(0xFFFF * FMCMC1_DRV_WCTREG_DATA_LO);
	xctw |= ( wval   * FMCMC1_DRV_WCTREG_DATA_LO);

	// set address
	xctw &= ~(0x7F * FMCMC1_DRV_WCTREG_ADDR_LO);
	xctw |= ( addr * FMCMC1_DRV_WCTREG_ADDR_LO);

	// r/w access
	if (write)
		xctw |= (FMCMC1_DRV_WCTREG_DRP_WE);
	else
		xctw &= ~(FMCMC1_DRV_WCTREG_DRP_WE);

	// enable
	xctw |= (FMCMC1_DRV_WCTREG_DRP_EN);

	// toggle DC
	if (0 != (xctr & FMCMC1_DRV_WCTREG_DRP_DC))
		xctw &= ~(FMCMC1_DRV_WCTREG_DRP_DC);
	else
		xctw |= (FMCMC1_DRV_WCTREG_DRP_DC);

	// do command to ip automa
	FMCMC1_DRV_mWrite_xadc_wctreg(QD_FMCMC1_DRV_REFLADDR,xctw);

	// wait 4 exec
	int ntry = 0;
	for(;;)
	{
		xctr = FMCMC1_DRV_mRead_xadc_rctreg(QD_FMCMC1_DRV_REFLADDR);
		if ((xctr & FMCMC1_DRV_WCTREG_DRP_DC) == (xctw & FMCMC1_DRV_WCTREG_DRP_DC))
			break;
		if (++ntry == QD_XADC_AUTOMA_NTRY_EXEC)
		{
			#if 0
			printf("XADC: i/o reg fatal error.\n");
			#endif
			return XST_FAILURE;
		}
	}

	// get value
	u16 inval = xctr & 0xFFFF;

	// remove enable
	xctw &= ~(FMCMC1_DRV_WCTREG_DRP_EN);

	// read (no write)
	xctw &= ~(FMCMC1_DRV_WCTREG_DRP_WE);

	FMCMC1_DRV_mWrite_xadc_wctreg(QD_FMCMC1_DRV_REFLADDR,xctw);

	#if 0
	if (write)
	{
		printf("XADC: [%02X] W %04X try=%d\n",addr, wval,ntry);
	}
	else
	{
		printf("XADC: [%02X] R %04X try=%d\n",addr, inval,ntry);
	}
	#endif

	if (rval)
	{
		*rval = inval;
	}
	
	return XST_SUCCESS;
}

// R/W XADC SSR

int fmcmc1_drv_xadc_ssr_io(fmcmc1_drv *InstancePtr, u8 *rval, const u8 wval, const u8 write)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}
		
    // driver base address
	QD_FMCMC1_DRV_DEFLADDR;
	
	// xadc ip ctrl reg.
	u32 xctw = FMCMC1_DRV_mRead_xadc_wctreg(QD_FMCMC1_DRV_REFLADDR);
	
	if (write)
	{
		xctw &= ~(
			FMCMC1_DRV_WCTREG_XA_VBUS_0 |
			FMCMC1_DRV_WCTREG_XA_IABS_0 |
			FMCMC1_DRV_WCTREG_XA_VBUS_1 |
			FMCMC1_DRV_WCTREG_XA_IABS_1 );
		
		if (wval & FMCMC1_DRV_AP_MASK_SSR_VBUS_0) xctw |= FMCMC1_DRV_WCTREG_XA_VBUS_0;
		if (wval & FMCMC1_DRV_AP_MASK_SSR_IABS_0) xctw |= FMCMC1_DRV_WCTREG_XA_IABS_0;
		if (wval & FMCMC1_DRV_AP_MASK_SSR_VBUS_1) xctw |= FMCMC1_DRV_WCTREG_XA_VBUS_1;
		if (wval & FMCMC1_DRV_AP_MASK_SSR_IABS_1) xctw |= FMCMC1_DRV_WCTREG_XA_IABS_1;

		FMCMC1_DRV_mWrite_xadc_wctreg(QD_FMCMC1_DRV_REFLADDR,xctw);
	}
	
	if (rval)
	{
		u8 t = 0;
		
		if (xctw & FMCMC1_DRV_WCTREG_XA_VBUS_0) t |= FMCMC1_DRV_AP_MASK_SSR_VBUS_0;
		if (xctw & FMCMC1_DRV_WCTREG_XA_IABS_0) t |= FMCMC1_DRV_AP_MASK_SSR_IABS_0;
		if (xctw & FMCMC1_DRV_WCTREG_XA_VBUS_1) t |= FMCMC1_DRV_AP_MASK_SSR_VBUS_1;
		if (xctw & FMCMC1_DRV_WCTREG_XA_IABS_1) t |= FMCMC1_DRV_AP_MASK_SSR_IABS_1;
		
		* rval = t;
	}
	
	return XST_SUCCESS;
}

int fmcmc1_drv_ap_get_control(fmcmc1_drv *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	fmcmc1_drv_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
		
    // driver base address
	QD_FMCMC1_DRV_DEFLADDR;
	
	// board control
	{
		fmcmc1_drv_ap_ctl_t * ctl = &ApPtr->ctl;

		ctl->sd_ctrl_0     = FMCMC1_DRV_mRead_sd_ctrl_0(QD_FMCMC1_DRV_REFLADDR);
		ctl->sd_ctrl_1     = FMCMC1_DRV_mRead_sd_ctrl_1(QD_FMCMC1_DRV_REFLADDR);
		
		ctl->oc_temp_fkt   = FMCMC1_DRV_mRead_oc_temp_fkt(QD_FMCMC1_DRV_REFLADDR);

		ctl->xa_synclkdiv  = FMCMC1_DRV_mRead_xa_synclkdiv(QD_FMCMC1_DRV_REFLADDR);
		
		ctl->gpio_o        = FMCMC1_DRV_mRead_gpio_o(QD_FMCMC1_DRV_REFLADDR);
		ctl->gpio_t        = FMCMC1_DRV_mRead_gpio_t(QD_FMCMC1_DRV_REFLADDR);

	}
	
	return XST_SUCCESS;
}

int fmcmc1_drv_ap_set_control(fmcmc1_drv *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	fmcmc1_drv_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
		
    // driver base address
	QD_FMCMC1_DRV_DEFLADDR;
	
	// board control
	{
		fmcmc1_drv_ap_ctl_t * ctl = &ApPtr->ctl;
	
		FMCMC1_DRV_mWrite_sd_ctrl_0(QD_FMCMC1_DRV_REFLADDR       ,ctl->sd_ctrl_0);
		FMCMC1_DRV_mWrite_sd_ctrl_1(QD_FMCMC1_DRV_REFLADDR       ,ctl->sd_ctrl_1);
		
		FMCMC1_DRV_mWrite_oc_temp_fkt(QD_FMCMC1_DRV_REFLADDR     ,ctl->oc_temp_fkt);

		FMCMC1_DRV_mWrite_xa_synclkdiv(QD_FMCMC1_DRV_REFLADDR    ,ctl->xa_synclkdiv);

		FMCMC1_DRV_mWrite_gpio_o(QD_FMCMC1_DRV_REFLADDR          ,ctl->gpio_o);
		FMCMC1_DRV_mWrite_gpio_t(QD_FMCMC1_DRV_REFLADDR          ,ctl->gpio_t);
	}
	
	return XST_SUCCESS;
}

int fmcmc1_drv_ap_get_status(fmcmc1_drv *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	fmcmc1_drv_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
		
    // driver base address
	QD_FMCMC1_DRV_DEFLADDR;
	
	// board status
	{
		fmcmc1_drv_ap_sts_t * sts = &ApPtr->sts;
		
		sts->status_reg  = FMCMC1_DRV_mRead_status_reg(QD_FMCMC1_DRV_REFLADDR);

		sts->oc_temp_val = FMCMC1_DRV_mRead_oc_temp_val(QD_FMCMC1_DRV_REFLADDR);

		sts->dip_sw      = FMCMC1_DRV_mRead_dip_sw(QD_FMCMC1_DRV_REFLADDR);

		sts->gpio_i      = FMCMC1_DRV_mRead_gpio_i(QD_FMCMC1_DRV_REFLADDR);
	}
	
	return XST_SUCCESS;
}

int fmcmc1_drv_ap_get_xadc(fmcmc1_drv *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	fmcmc1_drv_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
		
	// XADC control
	{
		fmcmc1_drv_ap_xadc_t * xadc = &ApPtr->xadc;
		int rc;

		rc = fmcmc1_drv_xadc_reg_io(InstancePtr,0x48,&xadc->r48,0,0); QD_FMCMC1_DRV_RET4ERROR();
		rc = fmcmc1_drv_xadc_reg_io(InstancePtr,0x49,&xadc->r49,0,0); QD_FMCMC1_DRV_RET4ERROR();

		rc = fmcmc1_drv_xadc_reg_io(InstancePtr,0x4E,&xadc->r4E,0,0); QD_FMCMC1_DRV_RET4ERROR();
		rc = fmcmc1_drv_xadc_reg_io(InstancePtr,0x4F,&xadc->r4F,0,0); QD_FMCMC1_DRV_RET4ERROR();

		rc = fmcmc1_drv_xadc_ssr_io(InstancePtr,&xadc->ssr,0,0); QD_FMCMC1_DRV_RET4ERROR();
	}
	
	return XST_SUCCESS;
}

int fmcmc1_drv_ap_set_xadc(fmcmc1_drv *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	fmcmc1_drv_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}
		
	// XADC setup
	{
		fmcmc1_drv_ap_xadc_t * xadc = &ApPtr->xadc;
		int rc;
		u16 t41;

		// get current r41 register....
		rc = fmcmc1_drv_xadc_reg_io(InstancePtr,0x41,&t41,0,0); QD_FMCMC1_DRV_RET4ERROR();

		// SEQ3..SEQ0 = 0
		rc = fmcmc1_drv_xadc_reg_io(InstancePtr,0x41,NULL,(t41 & 0x0FFF),1); QD_FMCMC1_DRV_RET4ERROR();

		rc = fmcmc1_drv_xadc_reg_io(InstancePtr,0x48,NULL,xadc->r48,1); QD_FMCMC1_DRV_RET4ERROR();
		rc = fmcmc1_drv_xadc_reg_io(InstancePtr,0x49,NULL,xadc->r49,1); QD_FMCMC1_DRV_RET4ERROR();

		rc = fmcmc1_drv_xadc_reg_io(InstancePtr,0x4E,NULL,xadc->r4E,1); QD_FMCMC1_DRV_RET4ERROR();
		rc = fmcmc1_drv_xadc_reg_io(InstancePtr,0x4F,NULL,xadc->r4F,1); QD_FMCMC1_DRV_RET4ERROR();
		
		// restore r41
		rc = fmcmc1_drv_xadc_reg_io(InstancePtr,0x41,NULL,t41,1); QD_FMCMC1_DRV_RET4ERROR();
		
		rc = fmcmc1_drv_xadc_ssr_io(InstancePtr,NULL,xadc->ssr,1); QD_FMCMC1_DRV_RET4ERROR();
	}
	
	return XST_SUCCESS;
}

int fmcmc1_drv_ap_get_oc_temp(fmcmc1_drv *InstancePtr, float * oc_temp)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	fmcmc1_drv_ap_t *ApPtr = InstancePtr->ApPtr;

	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// board status
	if (oc_temp != NULL)
	{
		fmcmc1_drv_ap_sts_t * sts = &ApPtr->sts;

		* oc_temp = (float)sts->oc_temp_val / (float)(1 << 18) * (float)FMCMC1_DRV_XADC_TEMP_KONST - (float)FMCMC1_DRV_XADC_KELVIN_ZERO_CENT;
	}

	return XST_SUCCESS;
}

int fmcmc1_drv_ap_init(fmcmc1_drv *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}
	return XST_SUCCESS;
}
