/*
-------------------------------------------------------------------------------------------
-- Copyright  2011-2016, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief fmcmc1_drv ap lib
/// \file fmcmc1_drv_apd.c
/// \version 0.51
/// \date jun 15, 2013
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2009-2013

#include <stdio.h>
#include "xparameters.h"
#include "xil_types.h"
#include "xstatus.h"
#include "fmcmc1_drv.h"
#include "fmcmc1_drv_apc.h"
#include "fmcmc1_drv_apd.h"

int fmcmc1_drv_ap_xadc_disp(fmcmc1_drv *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	fmcmc1_drv_ap_t *ApPtr = InstancePtr->ApPtr;
	
	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// XADC setup
	{
		fmcmc1_drv_ap_xadc_t * xadc = &ApPtr->xadc;
		
		printf("\nXADC\n");
		printf("r48..r49 = %04X %04X\n", xadc->r48, xadc->r49);
		printf("r4E..r4F = %04X %04X\n", xadc->r4E, xadc->r4F);
		printf("ssr: vbus0=%d iabs0=%d vbus1=%d iabs1=%d\n",
			(xadc->ssr & FMCMC1_DRV_AP_MASK_SSR_VBUS_0) != 0,
			(xadc->ssr & FMCMC1_DRV_AP_MASK_SSR_IABS_0) != 0,
			(xadc->ssr & FMCMC1_DRV_AP_MASK_SSR_VBUS_1) != 0,
			(xadc->ssr & FMCMC1_DRV_AP_MASK_SSR_IABS_1) != 0);
	}

	return XST_SUCCESS;
}

int fmcmc1_drv_ap_ctl_disp(fmcmc1_drv *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	fmcmc1_drv_ap_t *ApPtr = InstancePtr->ApPtr;
	
	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// board control
	{
		fmcmc1_drv_ap_ctl_t * ctl = &ApPtr->ctl;
	
		printf("\nCTL\n");

		printf("sd_ctrl_0      =x%X\n",    (unsigned)(ctl->sd_ctrl_0    ));
		printf("sd_ctrl_1      =x%X\n",    (unsigned)(ctl->sd_ctrl_1    ));
		
		printf("oc_temp_fkt    =%d\n",     (unsigned)(ctl->oc_temp_fkt  ));

		printf("xa_synclkdiv   =%d\n",     (unsigned)(ctl->xa_synclkdiv ));

		printf("gpio_o         =x%02X\n",            (ctl->gpio_o       ));
		printf("gpio_t         =x%02X\n",            (ctl->gpio_t       ));

	}

	return XST_SUCCESS;
}

int fmcmc1_drv_ap_sts_disp(fmcmc1_drv *InstancePtr)
{
	if (InstancePtr == NULL) {
		return XST_FAILURE;
	}

	fmcmc1_drv_ap_t *ApPtr = InstancePtr->ApPtr;
	
	if (ApPtr == NULL) {
		return XST_FAILURE;
	}

	// board status
	{
		fmcmc1_drv_ap_sts_t * sts = &ApPtr->sts;
	
		printf("\nSTS\n");

		printf("status_reg      =x%02X\n", (sts->status_reg ));

		printf("oc_temp_val     =%u\n",    (unsigned)(sts->oc_temp_val));

		printf("dip_sw          =x%02X\n", (sts->dip_sw     ));

		printf("gpio_i          =x%02X\n", (sts->gpio_i     ));
	}

	return XST_SUCCESS;
}
