/*
-------------------------------------------------------------------------------------------
-- Copyright © 2011-2019, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

#ifndef GPLOGGER_H
#define GPLOGGER_H

/****************** Include Files ********************/

#include "xil_types.h"
#include "xil_assert.h"
#include "xil_io.h"
#include "xstatus.h"
#include "xenv.h"

/***************************************************************************/

#include "gplogger_lx.h"

/**************************** Type Definitions *****************************/

typedef struct {

	u16 DeviceId;
	u32 BaseAddress;
	//-
	u32 systemclock;
	//-
	u8  hw_link_ref;
	u32 memory_base;
	u32 memory_size;
	u16 rec_data_width;
	
} gplogger_Config;

extern gplogger_Config gplogger_ConfigTable[];


typedef struct {

	gplogger_Config * Config;
	
	u32 IsReady;
	
} gplogger;


// ===========================================================================

/**************************** Type Definitions *****************************/
/**
 *
 * Write a value to a GPLOGGER register. A 32 bit write is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is written.
 *
 * @param   BaseAddress is the base address of the GPLOGGERdevice.
 * @param   RegOffset is the register offset from the base to write to.
 * @param   Data is the data written to the register.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void GPLOGGER_mWriteReg(u32 BaseAddress, unsigned RegOffset, u32 Data)
 *
 */
#define GPLOGGER_mWriteReg(BaseAddress, RegOffset, Data) \
  	Xil_Out32((BaseAddress) + (RegOffset), (u32)(Data))

/**
 *
 * Read a value from a GPLOGGER register. A 32 bit read is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is read from the register. The most significant data
 * will be read as 0.
 *
 * @param   BaseAddress is the base address of the GPLOGGER device.
 * @param   RegOffset is the register offset from the base to write to.
 *
 * @return  Data is the data from the register.
 *
 * @note
 * C-style signature:
 * 	u32 GPLOGGER_mReadReg(u32 BaseAddress, unsigned RegOffset)
 *
 */
#define GPLOGGER_mReadReg(BaseAddress, RegOffset) \
    Xil_In32((BaseAddress) + (RegOffset))

// ===========================================================================

/***************************************************************************/

//
// Write registers
//

// #define GPLOGGER_mWrite_ip_ident(BaseAddress,Value)     GPLOGGER_mWriteReg((BaseAddress), GPLOGGER_RA_ip_ident_OFFSET   ,(Value))
   #define GPLOGGER_mWrite_acq_ctrlst(BaseAddress,Value)   GPLOGGER_mWriteReg((BaseAddress), GPLOGGER_RA_acq_ctrlst_OFFSET ,(Value))
   #define GPLOGGER_mWrite_mem_base(BaseAddress,Value)     GPLOGGER_mWriteReg((BaseAddress), GPLOGGER_RA_mem_base_OFFSET   ,(Value))
   #define GPLOGGER_mWrite_mem_size(BaseAddress,Value)     GPLOGGER_mWriteReg((BaseAddress), GPLOGGER_RA_mem_size_OFFSET   ,(Value))
// #define GPLOGGER_mWrite_acq_level(BaseAddress,Value)    GPLOGGER_mWriteReg((BaseAddress), GPLOGGER_RA_acq_level_OFFSET  ,(Value))
   
//
// Read registers
//

   #define GPLOGGER_mRead_ip_ident(BaseAddress)            GPLOGGER_mReadReg((BaseAddress), GPLOGGER_RA_ip_ident_OFFSET   )
   #define GPLOGGER_mRead_acq_ctrlst(BaseAddress)          GPLOGGER_mReadReg((BaseAddress), GPLOGGER_RA_acq_ctrlst_OFFSET )
   #define GPLOGGER_mRead_mem_base(BaseAddress)            GPLOGGER_mReadReg((BaseAddress), GPLOGGER_RA_mem_base_OFFSET   )
   #define GPLOGGER_mRead_mem_size(BaseAddress)            GPLOGGER_mReadReg((BaseAddress), GPLOGGER_RA_mem_size_OFFSET   )
   #define GPLOGGER_mRead_acq_level(BaseAddress)           GPLOGGER_mReadReg((BaseAddress), GPLOGGER_RA_acq_level_OFFSET  )
   
/***************************************************************************/

gplogger_Config *gplogger_LookupConfig(u16 DeviceId);
int gplogger_Initialize(gplogger *InstancePtr, u16 DeviceId);
int gplogger_Down(u16 DeviceId);

// ===========================================================================

#endif // GPLOGGER_H
