/*
-------------------------------------------------------------------------------------------
-- Copyright © 2011-2016, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

#ifndef MOTCON1_DRV_H
#define MOTCON1_DRV_H

/***************************** Include Files *******************************/

#include "xparameters.h"
#include "xil_types.h"
#include "xil_assert.h"
#include "xil_io.h"
#include "xstatus.h"
#include "xenv.h"

/***************************************************************************/

#include "motcon1_drv_lx.h"

/**************************** Type Definitions *****************************/

typedef struct {

	u16 DeviceId;
	u32 BaseAddress;
	//-
	u32 systemclock;
	//-
	u8  carrier;
	u8  hw_link_ref;
	u8  xadc;
	
} motcon1_drv_Config;

extern motcon1_drv_Config motcon1_drv_ConfigTable[];

typedef struct {

	motcon1_drv_Config * Config;
	
	u32 IsReady;
	
} motcon1_drv;

/***************** Macros (Inline Functions) Definitions *******************/

/**
 *
 * Write a value to a MOTCON1_DRV register. A 32 bit write is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is written.
 *
 * @param   BaseAddress is the base address of the MOTCON1_DRV device.
 * @param   RegOffset is the register offset from the base to write to.
 * @param   Data is the data written to the register.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void MOTCON1_DRV_mWriteReg(u32 BaseAddress, unsigned RegOffset, u32 Data)
 *
 */
#define MOTCON1_DRV_mWriteReg(BaseAddress, RegOffset, Data) \
 	Xil_Out32((BaseAddress) + (RegOffset), (u32)(Data))

/**
 *
 * Read a value from a MOTCON1_DRV register. A 32 bit read is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is read from the register. The most significant data
 * will be read as 0.
 *
 * @param   BaseAddress is the base address of the MOTCON1_DRV device.
 * @param   RegOffset is the register offset from the base to write to.
 *
 * @return  Data is the data from the register.
 *
 * @note
 * C-style signature:
 * 	u32 MOTCON1_DRV_mReadReg(u32 BaseAddress, unsigned RegOffset)
 *
 */
#define MOTCON1_DRV_mReadReg(BaseAddress, RegOffset) \
 	Xil_In32((BaseAddress) + (RegOffset))

/***************************************************************************/

//
// Write registers
//

// #define MOTCON1_DRV_mWrite_ip_ident(BaseAddress,Value)               MOTCON1_DRV_mWriteReg((BaseAddress), MOTCON1_DRV_RA_ip_ident_OFFSET   ,(Value))
// #define MOTCON1_DRV_mWrite_status_reg(BaseAddress,Value)             MOTCON1_DRV_mWriteReg((BaseAddress), MOTCON1_DRV_RA_status_reg_OFFSET ,(Value))
   #define MOTCON1_DRV_mWrite_control_reg(BaseAddress,Value)            MOTCON1_DRV_mWriteReg((BaseAddress), MOTCON1_DRV_RA_control_reg_OFFSET,(Value))
   #define MOTCON1_DRV_mWrite_coil_ctrl(BaseAddress,Value)              MOTCON1_DRV_mWriteReg((BaseAddress), MOTCON1_DRV_RA_coil_ctrl_OFFSET  ,(Value))
// #define MOTCON1_DRV_mWrite_oc_temp_val(BaseAddress,Value)            MOTCON1_DRV_mWriteReg((BaseAddress), MOTCON1_DRV_RA_oc_temp_val_OFFSET,(Value))
   #define MOTCON1_DRV_mWrite_oc_temp_fkt(BaseAddress,Value)            MOTCON1_DRV_mWriteReg((BaseAddress), MOTCON1_DRV_RA_oc_temp_fkt_OFFSET,(Value))

   
//
// Read registers
//

   #define MOTCON1_DRV_mRead_ip_ident(BaseAddress)                      MOTCON1_DRV_mReadReg((BaseAddress), MOTCON1_DRV_RA_ip_ident_OFFSET   )
   #define MOTCON1_DRV_mRead_status_reg(BaseAddress)                    MOTCON1_DRV_mReadReg((BaseAddress), MOTCON1_DRV_RA_status_reg_OFFSET )
   #define MOTCON1_DRV_mRead_control_reg(BaseAddress)                   MOTCON1_DRV_mReadReg((BaseAddress), MOTCON1_DRV_RA_control_reg_OFFSET)
   #define MOTCON1_DRV_mRead_coil_ctrl(BaseAddress)                     MOTCON1_DRV_mReadReg((BaseAddress), MOTCON1_DRV_RA_coil_ctrl_OFFSET  )
   #define MOTCON1_DRV_mRead_oc_temp_val(BaseAddress)                   MOTCON1_DRV_mReadReg((BaseAddress), MOTCON1_DRV_RA_oc_temp_val_OFFSET)
   #define MOTCON1_DRV_mRead_oc_temp_fkt(BaseAddress)                   MOTCON1_DRV_mReadReg((BaseAddress), MOTCON1_DRV_RA_oc_temp_fkt_OFFSET)

/************************** Function Prototypes ****************************/


motcon1_drv_Config *motcon1_drv_LookupConfig(u16 DeviceId);
int motcon1_drv_Initialize(motcon1_drv *InstancePtr, u16 DeviceId);
int motcon1_drv_Down(u16 DeviceId);

// XADC Temperature conf

// ad / 4 / 16 * 503.975 / 4096 - 273.15
#define MOTCON1_DRV_xa_adi_mcelsius(value)   ((((value) - 142080) * 123) / 64)

#endif /** MOTCON1_DRV_H */
