/*
-------------------------------------------------------------------------------------------
-- Copyright © 2011-2016, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/


#ifndef MOTCON1_DRV_LX_H
#define MOTCON1_DRV_LX_H

// register offsets

#define MOTCON1_DRV_RA_ip_ident_OFFSET          (4*0)
#define MOTCON1_DRV_RA_status_reg_OFFSET        (4*1)
#define MOTCON1_DRV_RA_control_reg_OFFSET       (4*2)
#define MOTCON1_DRV_RA_coil_ctrl_OFFSET         (4*3)
#define MOTCON1_DRV_RA_oc_temp_val_OFFSET       (4*4)
#define MOTCON1_DRV_RA_oc_temp_fkt_OFFSET       (4*5)

// IP ident code
#define MOTCON1_DRV_ip_ident_code_value   8


// Coil Control
#define MOTCON1_DRV_C_COIL_CTRL_VALUE_OFS     (1 << 0)
#define MOTCON1_DRV_C_COIL_CTRL_GX_AUTO       (1 << 4)
#define MOTCON1_DRV_C_COIL_CTRL_GX_GEAR       (1 << 5)

// Status Register
#define MOTCON1_DRV_C_STATUS_REG_PWM_FAULT    (1 << 0)
#define MOTCON1_DRV_C_STATUS_REG_OVC_LATCHED  (1 << 1)

// Control Register
#define MOTCON1_DRV_C_CONTROL_REG_ANGAIN0     (1 << 0)
#define MOTCON1_DRV_C_CONTROL_REG_ANGAIN1     (1 << 1)
#define MOTCON1_DRV_C_CONTROL_REG_SD_IABS_LVM (1 << 2)
#define MOTCON1_DRV_C_CONTROL_REG_XA_VBUS     (1 << 3)
#define MOTCON1_DRV_C_CONTROL_REG_XA_IABS     (1 << 4)

// AD8251 A1/A0 analog gain values
#define MOTCON1_DRV_C_CONTROL_REG_ANGAIN_1    0
#define MOTCON1_DRV_C_CONTROL_REG_ANGAIN_2    MOTCON1_DRV_C_CONTROL_REG_ANGAIN0
#define MOTCON1_DRV_C_CONTROL_REG_ANGAIN_4    MOTCON1_DRV_C_CONTROL_REG_ANGAIN1
#define MOTCON1_DRV_C_CONTROL_REG_ANGAIN_8    (MOTCON1_DRV_C_CONTROL_REG_ANGAIN1 | MOTCON1_DRV_C_CONTROL_REG_ANGAIN0)
#define MOTCON1_DRV_C_CONTROL_REG_ANGAIN_MASK MOTCON1_DRV_C_CONTROL_REG_ANGAIN_8

// ----------------------------------------------------------------------------------

#define MOTCON1_DRV_XADC_KELVIN_ZERO_CENT     (273.15)
#define MOTCON1_DRV_XADC_TEMP_KONST           (503.975)

// XADC conversion time per channel
#define MOTCON1_DRV_XADC_CONVTIME_ACQ_ZERO    26
#define MOTCON1_DRV_XADC_CONVTIME_ACQ_ONE     32

// XADC sample rate in clocks
#define MOTCON1_DRV_XA_SAMPLE_CLOCK           (MOTCON1_DRV_XADC_CONVTIME_ACQ_ONE*2+MOTCON1_DRV_XADC_CONVTIME_ACQ_ZERO)

// ----------------------------------------------------------------------------------

// Dead time
#define MOTCON1_DRV_DEF_DEADTIME          (275.0e-9)
#define MOTCON1_DRV_MIN_DEADTIME          (275.0e-9)
#define MOTCON1_DRV_MAX_DEADTIME          (275.0e-9)

// S/D analog conversion
#define MOTCON1_DRV_SD_VBUS_GAIN          (5.7e-4)
#define MOTCON1_DRV_SD_CURR_GAIN          (4.07e-4)

// XA analog conversion
#define MOTCON1_DRV_XA_VBUS_GAIN          (6.50e-4)
#define MOTCON1_DRV_XA_CURR_GAIN          (4.07e-4)

// current limit
#define MOTCON1_DRV_DEF_CURR_LIMIT        (10.1)

#endif /** MOTCON1_DRV_LX_H */
