/*
-------------------------------------------------------------------------------------------
-- Copyright  2011-2018, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

#ifndef MOTORFOC_H
#define MOTORFOC_H

/***************************** Include Files *******************************/

#include "xparameters.h"
#include "xil_types.h"
#include "xil_assert.h"
#include "xil_io.h"
#include "xstatus.h"
#include "xenv.h"

/***************************************************************************/

#include "motorfoc_lx.h"

/**************************** Type Definitions *****************************/

typedef struct {

	u16 DeviceId;
	u32 BaseAddress;
	//-
	u32 systemclock;
	//-
	u8  hw_link_ref;
	//-
	u16 inv_in_map;
	u16 inv_in_not;
	u8  inv_ofsv_mode;
	u8  inv_over_iphs;
	u8  inv_over_ibus;
	u8  inv_filter;
	u8  clarke_nphs;
	u8  pwm_modulator;
	u8  rpfm_modulator;
	u8  rpfm_3_level;
	u8  rpfm_tpnc;
	u8  smo_eval;
	u8  spd_eval;
	u8  speed_ctrl;
	u8  resolver;
	u8  hallsensor;
	u8  encoder;
	u8  manrot;
	u8  mrt_accmax_dln2;
	u8  smo_zs_dln2;
	u8  smo_f2_dln2;
	u8  slp_pro_dln2;
	u8  slp_int_dln2;
	u8  slp_indwp_dln2;
	u8  slp_indwp_kdiv;
	u8  isoverflow_cmax;
	u8  pi_erpro_dln2;
	u8  pi_erint_dln2;

} motorfoc_Config;

extern motorfoc_Config motorfoc_ConfigTable[];

typedef struct {

	// system parameters
	struct
	{
		float    ts;          // foc cycle time

		float    vbus_gain ;  // voltage bit weight
		float    vphs_gain ;  // voltage bit weight
		
		float    ibus_gain ;  // current bit weight
		float    iphs_gain ;  // current bit weight
		
		float    xbus_fcut1;  // 1st filter Bus
		float    xbus_fcut2;  // 2nd filter Bus
		
		float    xphs_fcut1;  // 1st filter Motor Phases
		float    xphs_fcut2;  // 2nd filter Motor Phases
		
		float    ibus_limit;  // v-bus current limit
		float    iphs_limit;  // phase current limit
		
		float    deadtime  ;  // gate unit dead time

	} sys;

	// motor parameters
	struct
	{
		int8_t   manufacturer[MOTORFOC_AP_MANUFACTURER_LEN+1];
		int8_t   motorcode[MOTORFOC_AP_MOTORCODE_LEN+1];
		uint8_t  windings;    // motor type
		uint8_t  paipol;      // pair poles (1..n is poles/2)
		float    inductance;  // coil inductance
		float    resistance;  // coil resistance
		float    bemfke;      // back-emf Ke
		float    torquemax;   // maximum torque
		float    torquekc;    // torque constant
		float    maxspeed;    // maximum speed
		float    iscurrmax;   // maximum current for Is (h/w protection)
		float    iscurrhigh;  // maximum current
		uint8_t  controlmode; // control mode
		uint8_t  rotposmode;  // rotor position mode
	} mot;

	// speed loop
	struct
	{
		float    pgain;  // proportional gain
		float    igain;  // integrative  gain
	} slp;

	// current loop
	struct
	{
		float    pgain;  // proportional gain
		float    igain;  // integrative  gain
	} pi;

	// modulator
	struct
	{
		float    mod2angskw; // 2nd modulator angle skew
		uint8_t  modlevels;  // modulator levels
		uint8_t  modulator;  // modulator selector
	} mod;
	
	// pwm
	struct
	{
		float    freq;  // frequency
		float    gain;  // gain
		float    mdmax; // modulation limit
		uint8_t  ctrls; // controls
	} pwm;

	// rpfm
	struct
	{
		float    pwdth; // pulse width
		uint8_t  ctrls; // controls
	} rpfm;
	
	// manual mode parameters
	struct
	{
		float    spfcut1; // 1st filter speed fcut
		float    spfcut2; // 2nd filter speed fcut
	} man;
	
	// scalar mode parameters
	struct
	{
		float    accmax;  // acceleration limit
		float    sofcut1; // 1st filter output speed fcut
		float    sofcut2; // 2nd filter output speed fcut
		uint8_t  ctrls;   // controls
		//-
		float    spfcut1; // 1st filter speed fcut
		float    spfcut2; // 2nd filter speed fcut
	} mrt;
	
	// external sns
	struct
	{
		float    spfcut1; // 1st filter speed fcut
		float    spfcut2; // 2nd filter speed fcut
	} ext;
	
	// resolver sns
	struct
	{
		uint8_t  paipol;  // pair poles (1..n is poles/2)
		float    angofs;  // offset angle DEG!
		//-
		float    spfcut1; // 1st filter speed fcut
		float    spfcut2; // 2nd filter speed fcut
		//-
		float    excfreq; // exciter frequency
	} rsv;
	
	// hall sns
	struct
	{
		float    ctolim;   // time-out coasting mode
		float    anglelut[MOTORFOC_HLS_DPR_ANGLE_ELEM]; // angle lut DEG!
		//-
		float    spfcut1;   // 1st filter speed fcut
		float    spfcut2;   // 2nd filter speed fcut
	} hls;

	// encoder
	struct
	{
		uint16_t cyprnd;  // encoder cycles per round
		uint8_t  ctrls;   // controls
		//-
		float    spfcut1; // 1st filter speed fcut
		float    spfcut2; // 2nd filter speed fcut
	} enc;

	// sliding mode observer
	struct
	{
		float    is_maxerr; // max Is error
		float    zs_maxval; // max Zs value
		float    es1_fcut;  // fcut 1st filter
		float    es2_fclow; // low fcut 2nd filter
		float    es2_fcmul; // multiplier fcut 2nd filter
		float    angofs;    // angle offset
		uint8_t  ctrls;     // controls
		//-
		float    spfcut1;   // 1st filter speed fcut
		float    spfcut2;   // 2nd filter speed fcut
	} smo;
	
} motorfoc_ap_prm_t;


// control registers {r/w}
typedef struct {

	// global
	struct
	{
		u32 motcontrol;
		u8  paipol;
	} glb;
	
	// phy
	struct
	{
		u32 xbus_fk1;
		u32 xbus_fk2;

		u32 xphs_fk1;
		u32 xphs_fk2;

		u32 ibus_limit;
		u32 iphs_limit;
		
		u8  deadtval;
		
	} phy;
	
	// speed eval
	struct
	{
		u32 spd_fktau1;
		u32 spd_fktau2;
	} spd;
	
	// rotor position
	struct
	{
		s32 rot_angdef;
	} rot;
	
	// smo 
	struct
	{
		u32 smo_vs_mult ;
		u32 smo_is_mult ;
		u32 smo_zs_max  ;
		u32 smo_es1_kflt;
		u32 smo_es2_kflo;
		u32 smo_es2_kfmx;
		s16 smo_angofs;
	} smo;
	
	
	// speed loop
	struct
	{
		s32 slp_spdset;
		union { u32 u; float f; } slp_kmpro;
		union { u32 u; float f; } slp_kmint;
		u32 slp_outlim;
		s32 slp_kmultx;
		s32 slp_kmulty;
	} slp;
	
	// resolver sns
	struct
	{
		u8  rsv_m2rppk;
		s16 rsv_angofs;
	} rsv;
	
	// hall sns
	struct
	{
		u32 hls_ctolim;
		s16 hls_dprangle[MOTORFOC_HLS_DPR_ANGLE_ELEM];
	} hls;
	
	// encoder
	struct
	{
		u32 enc_cyprnd;
		u32 enc_angphs;
	} enc;
	
	// scalar mode
	struct
	{
		s32 mrt_speed ;
		u32 mrt_accmax;
		u32 mrt_fktau1;
		u32 mrt_fktau2;
	} mrt;
	
	// incoming is current
	struct
	{
		u32 ismodmax;
	} iis;
	
	// current loop
	struct
	{
		s32 pi_setvalx;
		s32 pi_setvaly;
		u32 pi_kmpro;
		u32 pi_kmint;
	} pi;
	
	// modulator
	struct
	{
		s16 mod2angskw;
	} mod;
	
	// pwm
	struct
	{
		u32 pwm_presc;
		u32 pwm_kmod;
		u32 pwm_mdmax;
		u8  pwm_ctrl;
	} pwm;
	
	// rpfm
	struct
	{
		u16 rpfm3p_ctrl;
	} rpfm;
	
} motorfoc_ap_ctl_t;

// status registers {read only}
typedef struct {

	// main
	struct
	{
		u32 ip_ident;
		
		u16 tsclocks;

		u32 motstatus;

	} glb;
	
	// phy
	struct
	{
		s32 val_iphs_a;
		s32 val_iphs_b;
		s32 val_iphs_c;
		s32 val_ibus_x;
		s32 val_vphs_a;
		s32 val_vphs_b;
		s32 val_vphs_c;
		s32 val_vbus_x;
		s32 val_vphs_n;
		
		u32 guswcnt;
		
	} phy;

	
	// speed eval
	struct
	{
		s32 spd_speed;
	} spd;
	
	// rotor position
	struct
	{
		s32 rot_angle;
	} rot;
	
	// smo 
	struct
	{
		u32 smo_es2_kflt;
		s32 smo_bemf_p  ;
		u32 smo_bemf_m  ;
		s32 smo_angle   ;
	} smo;
	
	// position 16:16
	struct
	{
		s32 pos_position;
	} pos;
	
	// external sns
	struct
	{
		s32 ext_angle;
	} ext;
	
	// resolver sns
	struct
	{
		s32 rsv_angle;
	} rsv;
	
	// hall sns
	struct
	{
		s32 hls_angle;
	} hls;
	
	// encoder
	struct
	{
		u32 enc_index;
		s32 enc_phase;
		s32 enc_phcpt;
		s32 enc_angle;
	} enc;
	
	// scalar mode
	struct
	{
		s32 mrt_spdout;
		s32 mrt_angle;
	} mrt;
	
	// incoming is current
	struct
	{
		u32 ismodval;
		s32 isangval;
	} iis;
	
	// outgoing vs voltage
	struct
	{
		u32 vsmodval;
		s32 vsangval;
	} ovs;
	
	// pwm
	struct
	{
		u32 pwm_mdval;
	} pwm;
	
} motorfoc_ap_sts_t;

typedef struct {

	// configuration parameters
	motorfoc_ap_prm_t prm;

	// control registers
	motorfoc_ap_ctl_t ctl;

	// status registers
	motorfoc_ap_sts_t sts;
	
	// ip drive source
	u8 ipdrvsource;
	
} motorfoc_ap_t;


// main descriptor
typedef struct {

	motorfoc_Config * Config;
	
	motorfoc_ap_t   * ApPtr;
	
	u32 IsReady;
	
} motorfoc;

/***************** Macros (Inline Functions) Definitions *******************/

#define MOTORFOC_mWriteReg(BaseAddress, RegOffset, Data) \
 	Xil_Out32((BaseAddress) + (RegOffset), (u32)(Data))

#define MOTORFOC_mReadReg(BaseAddress, RegOffset) \
 	Xil_In32((BaseAddress) + (RegOffset))

	
//
// Write registers
//

// #define MOTORFOC_mWrite_ip_ident(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_ip_ident_OFFSET    ,(Value))

// #define MOTORFOC_mWrite_tsclocks(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_tsclocks_OFFSET    ,(Value))

   #define MOTORFOC_mWrite_motcontrol(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_motcontrol_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_motstatus(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_motstatus_OFFSET   ,(Value))

// #define MOTORFOC_mWrite_val_iphs_a(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_val_iphs_a_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_val_iphs_b(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_val_iphs_b_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_val_iphs_c(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_val_iphs_c_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_val_ibus_x(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_val_ibus_x_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_val_vphs_a(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_val_vphs_a_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_val_vphs_b(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_val_vphs_b_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_val_vphs_c(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_val_vphs_c_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_val_vbus_x(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_val_vbus_x_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_val_vphs_n(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_val_vphs_n_OFFSET  ,(Value))

   #define MOTORFOC_mWrite_ofs_iphs_a(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_ofs_iphs_a_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_ofs_iphs_b(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_ofs_iphs_b_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_ofs_iphs_c(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_ofs_iphs_c_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_ofs_ibus_x(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_ofs_ibus_x_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_ofs_vphs_a(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_ofs_vphs_a_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_ofs_vphs_b(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_ofs_vphs_b_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_ofs_vphs_c(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_ofs_vphs_c_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_ofs_vbus_x(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_ofs_vbus_x_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_ofs_vphs_n(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_ofs_vphs_n_OFFSET  ,(Value))

   #define MOTORFOC_mWrite_xbus_fk1(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_xbus_fk1_OFFSET    ,(Value))
   #define MOTORFOC_mWrite_xbus_fk2(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_xbus_fk2_OFFSET    ,(Value))

   #define MOTORFOC_mWrite_xphs_fk1(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_xphs_fk1_OFFSET    ,(Value))
   #define MOTORFOC_mWrite_xphs_fk2(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_xphs_fk2_OFFSET    ,(Value))

   #define MOTORFOC_mWrite_ibus_limit(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_ibus_limit_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_iphs_limit(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_iphs_limit_OFFSET  ,(Value))

   #define MOTORFOC_mWrite_ismodmax(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_ismodmax_OFFSET    ,(Value))

// #define MOTORFOC_mWrite_ismodval(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_ismodval_OFFSET    ,(Value))
// #define MOTORFOC_mWrite_isangval(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_isangval_OFFSET    ,(Value))

// #define MOTORFOC_mWrite_vsmodval(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_vsmodval_OFFSET    ,(Value))
// #define MOTORFOC_mWrite_vsangval(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_vsangval_OFFSET    ,(Value))

   #define MOTORFOC_mWrite_rot_angdef(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_rot_angdef_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_rot_angle(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_rot_angle_OFFSET   ,(Value))

   #define MOTORFOC_mWrite_spd_fktau1(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_spd_fktau1_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_spd_fktau2(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_spd_fktau2_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_spd_speed(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_spd_speed_OFFSET   ,(Value))

   #define MOTORFOC_mWrite_smo_vs_mult(BaseAddress,Value)                  MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_smo_vs_mult_OFFSET ,(Value))
   #define MOTORFOC_mWrite_smo_is_mult(BaseAddress,Value)                  MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_smo_is_mult_OFFSET ,(Value))
   #define MOTORFOC_mWrite_smo_zs_max(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_smo_zs_max_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_smo_es1_kflt(BaseAddress,Value)                 MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_smo_es1_kflt_OFFSET,(Value))
   #define MOTORFOC_mWrite_smo_es2_kflo(BaseAddress,Value)                 MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_smo_es2_kflo_OFFSET,(Value))
   #define MOTORFOC_mWrite_smo_es2_kfmx(BaseAddress,Value)                 MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_smo_es2_kfmx_OFFSET,(Value))
// #define MOTORFOC_mWrite_smo_es2_kflt(BaseAddress,Value)                 MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_smo_es2_kflt_OFFSET,(Value))
   #define MOTORFOC_mWrite_smo_angofs(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_smo_angofs_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_smo_bemf_p(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_smo_bemf_p_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_smo_bemf_m(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_smo_bemf_m_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_smo_angle(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_smo_angle_OFFSET   ,(Value))

// #define MOTORFOC_mWrite_pos_position(BaseAddress,Value)                 MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_pos_position_OFFSET,(Value))

   #define MOTORFOC_mWrite_slp_spdset(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_slp_spdset_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_slp_kmpro(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_slp_kmpro_OFFSET   ,(Value))
   #define MOTORFOC_mWrite_slp_kmint(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_slp_kmint_OFFSET   ,(Value))
   #define MOTORFOC_mWrite_slp_outlim(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_slp_outlim_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_slp_kmultx(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_slp_kmultx_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_slp_kmulty(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_slp_kmulty_OFFSET  ,(Value))

   #define MOTORFOC_mWrite_paipol(BaseAddress,Value)                       MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_paipol_OFFSET      ,(Value))

// #define MOTORFOC_mWrite_ext_angle(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_ext_angle_OFFSET   ,(Value))

   #define MOTORFOC_mWrite_rsv_m2rppk(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_rsv_m2rppk_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_rsv_angofs(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_rsv_angofs_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_rsv_angle(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_rsv_angle_OFFSET   ,(Value))

   #define MOTORFOC_mWrite_hls_ctolim(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_hls_ctolim_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_hls_angle(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_hls_angle_OFFSET   ,(Value))

// #define MOTORFOC_mWrite_enc_index(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_enc_index_OFFSET   ,(Value))
// #define MOTORFOC_mWrite_enc_phase(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_enc_phase_OFFSET   ,(Value))
// #define MOTORFOC_mWrite_enc_phcpt(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_enc_phcpt_OFFSET   ,(Value))
   #define MOTORFOC_mWrite_enc_cyprnd(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_enc_cyprnd_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_enc_angphs(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_enc_angphs_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_enc_angle(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_enc_angle_OFFSET   ,(Value))

   #define MOTORFOC_mWrite_mrt_speed(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_mrt_speed_OFFSET   ,(Value))
   #define MOTORFOC_mWrite_mrt_accmax(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_mrt_accmax_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_mrt_fktau1(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_mrt_fktau1_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_mrt_fktau2(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_mrt_fktau2_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_mrt_spdout(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_mrt_spdout_OFFSET  ,(Value))
// #define MOTORFOC_mWrite_mrt_angle(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_mrt_angle_OFFSET   ,(Value))

   #define MOTORFOC_mWrite_pi_setvalx(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_pi_setvalx_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_pi_setvaly(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_pi_setvaly_OFFSET  ,(Value))
   #define MOTORFOC_mWrite_pi_kmpro(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_pi_kmpro_OFFSET    ,(Value))
   #define MOTORFOC_mWrite_pi_kmint(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_pi_kmint_OFFSET    ,(Value))

   #define MOTORFOC_mWrite_mod2angskw(BaseAddress,Value)                   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_mod2angskw_OFFSET  ,(Value))

   #define MOTORFOC_mWrite_pwm_kmod(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_pwm_kmod_OFFSET    ,(Value))
   #define MOTORFOC_mWrite_pwm_presc(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_pwm_presc_OFFSET   ,(Value))
   #define MOTORFOC_mWrite_pwm_mdmax(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_pwm_mdmax_OFFSET   ,(Value))
// #define MOTORFOC_mWrite_pwm_mdval(BaseAddress,Value)                    MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_pwm_mdval_OFFSET   ,(Value))
   #define MOTORFOC_mWrite_pwm_ctrl(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_pwm_ctrl_OFFSET    ,(Value))

   #define MOTORFOC_mWrite_rpfm3p_ctrl(BaseAddress,Value)                  MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_rpfm3p_ctrl_OFFSET ,(Value))

   #define MOTORFOC_mWrite_deadtval(BaseAddress,Value)                     MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_deadtval_OFFSET    ,(Value))
// #define MOTORFOC_mWrite_guswcnt(BaseAddress,Value)                      MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_guswcnt_OFFSET     ,(Value))


   #define MOTORFOC_mWrite_hall_sensor_angle_lut(BaseAddress,Offset,Value) MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_hall_sensor_angle_lut_OFFSET + (Offset),(Value))
   #define MOTORFOC_mWrite_pwm_modulator_table(BaseAddress,Offset,Value)   MOTORFOC_mWriteReg((BaseAddress), MOTORFOC_RA_pwm_modulator_table_OFFSET   + (Offset),(Value))

	
//
// Read registers
//

   #define MOTORFOC_mRead_ip_ident(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_ip_ident_OFFSET    )

   #define MOTORFOC_mRead_tsclocks(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_tsclocks_OFFSET    )

   #define MOTORFOC_mRead_motcontrol(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_motcontrol_OFFSET  )
   #define MOTORFOC_mRead_motstatus(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_motstatus_OFFSET   )

   #define MOTORFOC_mRead_val_iphs_a(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_val_iphs_a_OFFSET  )
   #define MOTORFOC_mRead_val_iphs_b(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_val_iphs_b_OFFSET  )
   #define MOTORFOC_mRead_val_iphs_c(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_val_iphs_c_OFFSET  )
   #define MOTORFOC_mRead_val_ibus_x(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_val_ibus_x_OFFSET  )
   #define MOTORFOC_mRead_val_vphs_a(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_val_vphs_a_OFFSET  )
   #define MOTORFOC_mRead_val_vphs_b(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_val_vphs_b_OFFSET  )
   #define MOTORFOC_mRead_val_vphs_c(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_val_vphs_c_OFFSET  )
   #define MOTORFOC_mRead_val_vbus_x(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_val_vbus_x_OFFSET  )
   #define MOTORFOC_mRead_val_vphs_n(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_val_vphs_n_OFFSET  )

   #define MOTORFOC_mRead_ofs_iphs_a(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_ofs_iphs_a_OFFSET  )
   #define MOTORFOC_mRead_ofs_iphs_b(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_ofs_iphs_b_OFFSET  )
   #define MOTORFOC_mRead_ofs_iphs_c(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_ofs_iphs_c_OFFSET  )
   #define MOTORFOC_mRead_ofs_ibus_x(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_ofs_ibus_x_OFFSET  )
   #define MOTORFOC_mRead_ofs_vphs_a(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_ofs_vphs_a_OFFSET  )
   #define MOTORFOC_mRead_ofs_vphs_b(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_ofs_vphs_b_OFFSET  )
   #define MOTORFOC_mRead_ofs_vphs_c(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_ofs_vphs_c_OFFSET  )
   #define MOTORFOC_mRead_ofs_vbus_x(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_ofs_vbus_x_OFFSET  )
   #define MOTORFOC_mRead_ofs_vphs_n(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_ofs_vphs_n_OFFSET  )

   #define MOTORFOC_mRead_xbus_fk1(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_xbus_fk1_OFFSET    )
   #define MOTORFOC_mRead_xbus_fk2(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_xbus_fk2_OFFSET    )

   #define MOTORFOC_mRead_xphs_fk1(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_xphs_fk1_OFFSET    )
   #define MOTORFOC_mRead_xphs_fk2(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_xphs_fk2_OFFSET    )

   #define MOTORFOC_mRead_ibus_limit(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_ibus_limit_OFFSET  )
   #define MOTORFOC_mRead_iphs_limit(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_iphs_limit_OFFSET  )

   #define MOTORFOC_mRead_ismodmax(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_ismodmax_OFFSET    )

   #define MOTORFOC_mRead_ismodval(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_ismodval_OFFSET    )
   #define MOTORFOC_mRead_isangval(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_isangval_OFFSET    )

   #define MOTORFOC_mRead_vsmodval(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_vsmodval_OFFSET    )
   #define MOTORFOC_mRead_vsangval(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_vsangval_OFFSET    )

   #define MOTORFOC_mRead_rot_angdef(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_rot_angdef_OFFSET  )
   #define MOTORFOC_mRead_rot_angle(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_rot_angle_OFFSET   )

   #define MOTORFOC_mRead_spd_fktau1(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_spd_fktau1_OFFSET  )
   #define MOTORFOC_mRead_spd_fktau2(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_spd_fktau2_OFFSET  )
   #define MOTORFOC_mRead_spd_speed(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_spd_speed_OFFSET   )

   #define MOTORFOC_mRead_smo_vs_mult(BaseAddress)                  MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_smo_vs_mult_OFFSET )
   #define MOTORFOC_mRead_smo_is_mult(BaseAddress)                  MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_smo_is_mult_OFFSET )
   #define MOTORFOC_mRead_smo_zs_max(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_smo_zs_max_OFFSET  )
   #define MOTORFOC_mRead_smo_es1_kflt(BaseAddress)                 MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_smo_es1_kflt_OFFSET)
   #define MOTORFOC_mRead_smo_es2_kflo(BaseAddress)                 MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_smo_es2_kflo_OFFSET)
   #define MOTORFOC_mRead_smo_es2_kfmx(BaseAddress)                 MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_smo_es2_kfmx_OFFSET)
   #define MOTORFOC_mRead_smo_es2_kflt(BaseAddress)                 MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_smo_es2_kflt_OFFSET)
   #define MOTORFOC_mRead_smo_angofs(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_smo_angofs_OFFSET  )
   #define MOTORFOC_mRead_smo_bemf_p(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_smo_bemf_p_OFFSET  )
   #define MOTORFOC_mRead_smo_bemf_m(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_smo_bemf_m_OFFSET  )
   #define MOTORFOC_mRead_smo_angle(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_smo_angle_OFFSET   )

   #define MOTORFOC_mRead_pos_position(BaseAddress)                 MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_pos_position_OFFSET)

   #define MOTORFOC_mRead_slp_spdset(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_slp_spdset_OFFSET  )
   #define MOTORFOC_mRead_slp_kmpro(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_slp_kmpro_OFFSET   )
   #define MOTORFOC_mRead_slp_kmint(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_slp_kmint_OFFSET   )
   #define MOTORFOC_mRead_slp_outlim(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_slp_outlim_OFFSET  )
   #define MOTORFOC_mRead_slp_kmultx(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_slp_kmultx_OFFSET  )
   #define MOTORFOC_mRead_slp_kmulty(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_slp_kmulty_OFFSET  )

   #define MOTORFOC_mRead_paipol(BaseAddress)                       MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_paipol_OFFSET      )

   #define MOTORFOC_mRead_ext_angle(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_ext_angle_OFFSET   )

   #define MOTORFOC_mRead_rsv_m2rppk(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_rsv_m2rppk_OFFSET  )
   #define MOTORFOC_mRead_rsv_angofs(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_rsv_angofs_OFFSET  )
   #define MOTORFOC_mRead_rsv_angle(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_rsv_angle_OFFSET   )

   #define MOTORFOC_mRead_hls_ctolim(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_hls_ctolim_OFFSET  )
   #define MOTORFOC_mRead_hls_angle(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_hls_angle_OFFSET   )

   #define MOTORFOC_mRead_enc_index(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_enc_index_OFFSET   )
   #define MOTORFOC_mRead_enc_phase(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_enc_phase_OFFSET   )
   #define MOTORFOC_mRead_enc_phcpt(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_enc_phcpt_OFFSET   )
   #define MOTORFOC_mRead_enc_cyprnd(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_enc_cyprnd_OFFSET  )
   #define MOTORFOC_mRead_enc_angphs(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_enc_angphs_OFFSET  )
   #define MOTORFOC_mRead_enc_angle(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_enc_angle_OFFSET   )

   #define MOTORFOC_mRead_mrt_speed(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_mrt_speed_OFFSET   )
   #define MOTORFOC_mRead_mrt_accmax(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_mrt_accmax_OFFSET  )
   #define MOTORFOC_mRead_mrt_fktau1(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_mrt_fktau1_OFFSET  )
   #define MOTORFOC_mRead_mrt_fktau2(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_mrt_fktau2_OFFSET  )
   #define MOTORFOC_mRead_mrt_spdout(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_mrt_spdout_OFFSET  )
   #define MOTORFOC_mRead_mrt_angle(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_mrt_angle_OFFSET   )

   #define MOTORFOC_mRead_pi_setvalx(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_pi_setvalx_OFFSET  )
   #define MOTORFOC_mRead_pi_setvaly(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_pi_setvaly_OFFSET  )
   #define MOTORFOC_mRead_pi_kmpro(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_pi_kmpro_OFFSET    )
   #define MOTORFOC_mRead_pi_kmint(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_pi_kmint_OFFSET    )

   #define MOTORFOC_mRead_mod2angskw(BaseAddress)                   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_mod2angskw_OFFSET  )

   #define MOTORFOC_mRead_pwm_kmod(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_pwm_kmod_OFFSET    )
   #define MOTORFOC_mRead_pwm_presc(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_pwm_presc_OFFSET   )
   #define MOTORFOC_mRead_pwm_mdmax(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_pwm_mdmax_OFFSET   )
   #define MOTORFOC_mRead_pwm_mdval(BaseAddress)                    MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_pwm_mdval_OFFSET   )
   #define MOTORFOC_mRead_pwm_ctrl(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_pwm_ctrl_OFFSET    )

   #define MOTORFOC_mRead_rpfm3p_ctrl(BaseAddress)                  MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_rpfm3p_ctrl_OFFSET )

   #define MOTORFOC_mRead_deadtval(BaseAddress)                     MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_deadtval_OFFSET    )
   #define MOTORFOC_mRead_guswcnt(BaseAddress)                      MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_guswcnt_OFFSET     )

   
   #define MOTORFOC_mRead_hall_sensor_angle_lut(BaseAddress,Offset) MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_hall_sensor_angle_lut_OFFSET + (Offset))
   #define MOTORFOC_mRead_pwm_modulator_table(BaseAddress,Offset)   MOTORFOC_mReadReg((BaseAddress), MOTORFOC_RA_pwm_modulator_table_OFFSET   + (Offset))
   
   
/* ************************* macro functions **********************************/

		
/************************** Function Prototypes ****************************/

motorfoc_Config *motorfoc_LookupConfig(u16 DeviceId);
int motorfoc_Initialize(motorfoc *InstancePtr, u16 DeviceId);
int motorfoc_Down(u16 DeviceId);

// sign-extend macro
#define MOTORFOC_sign_extend(v,b) (((v) & (1 << ((b)-1))) ? ((v) | (-(1 << (b)))) : ((v) & ((1 << (b))-1)))

#endif /** MOTORFOC_H */
