/*
-------------------------------------------------------------------------------------------
-- Copyright  2011-2018, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief motorfoc ap lib
/// \file motorfoc_apc.h
/// \version 0.51
/// \date jun 15, 2013
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2009-2018

#ifndef MOTORFOC_APC_H_
#define MOTORFOC_APC_H_

int motorfoc_ap_init(motorfoc *InstancePtr);

int motorfoc_ap_sts_read(motorfoc *InstancePtr);
int motorfoc_ap_ctl_read(motorfoc *InstancePtr);
int motorfoc_ap_ctl_write(motorfoc *InstancePtr);
int motorfoc_ap_compile(motorfoc *InstancePtr);

int motorfoc_ap_prm_set_modlevels(motorfoc *InstancePtr, int modlevels);
int motorfoc_ap_prm_get_modlevels(motorfoc *InstancePtr, int * modlevels);

int motorfoc_ap_sts_get_iscurr(motorfoc *InstancePtr, float * iscurr);
int motorfoc_ap_sts_get_vsvolts(motorfoc *InstancePtr, float * vsvolts);
int motorfoc_ap_sts_get_vs2israd(motorfoc *InstancePtr, float * vs2israd);
int motorfoc_ap_sts_get_speedrpm(motorfoc *InstancePtr, float * speedrpm);
int motorfoc_ap_sts_get_mrt_spdout(motorfoc *InstancePtr, float * speed);
int motorfoc_ap_sts_get_rotpos(motorfoc *InstancePtr, float * rotpos);
int motorfoc_ap_sts_get_modidx(motorfoc *InstancePtr, float * modidx);
int motorfoc_ap_sts_get_vbus(motorfoc *InstancePtr, float * vbus);
int motorfoc_ap_sts_get_ivd(motorfoc *InstancePtr, float * ivd);

int motorfoc_ap_ctl_get_smo_lpfsimout(motorfoc *InstancePtr,  float * module, float * angle, float * angofs);

int motorfoc_ap_ctl_get_slp_is2pi_rec(motorfoc *InstancePtr, float *x, float *y);
int motorfoc_ap_ctl_set_slp_is2pi_rec(motorfoc *InstancePtr, const float x, const float y, int wnow);

int motorfoc_ap_ctl_get_slp_is2pi_pol(motorfoc *InstancePtr, float *module, float *angle);
int motorfoc_ap_ctl_set_slp_is2pi_pol(motorfoc *InstancePtr, const float module, const float angle, int wnow);

int motorfoc_ap_ctl_get_pi_curset_rec(motorfoc *InstancePtr, float *x, float *y);
int motorfoc_ap_ctl_set_pi_curset_rec(motorfoc *InstancePtr, const float x, const float y, int wnow);

int motorfoc_ap_ctl_get_pi_curset_pol(motorfoc *InstancePtr, float *module, float *angle);
int motorfoc_ap_ctl_set_pi_curset_pol(motorfoc *InstancePtr, const float module, const float angle, int wnow);

int motorfoc_ap_ctl_set_mrt_speed(motorfoc *InstancePtr, float speed, int wnow);
int motorfoc_ap_ctl_get_mrt_speed(motorfoc *InstancePtr, float * speed);

int motorfoc_ap_ctl_set_slp_speed(motorfoc *InstancePtr, float speed, int wnow);
int motorfoc_ap_ctl_get_slp_speed(motorfoc *InstancePtr, float * speed);

int motorfoc_ap_sts_get_failure(motorfoc *InstancePtr, int * failure);

int motorfoc_ap_sts_get_modulator(motorfoc *InstancePtr, int * modulator);

int motorfoc_ap_ctl_set_ctrlmode(motorfoc *InstancePtr, int ctrlmode, int mrtopt, int wnow);
int motorfoc_ap_ctl_get_ctrlmode(motorfoc *InstancePtr, int * ctrlmode, int * mrtopt);

int motorfoc_ap_ctl_set_power(motorfoc *InstancePtr, int power, int wnow);
int motorfoc_ap_ctl_get_power(motorfoc *InstancePtr, int * power);

int motorfoc_ap_ctl_set_modulator(motorfoc *InstancePtr, int modulator, int wnow);
int motorfoc_ap_ctl_get_modulator(motorfoc *InstancePtr, int * modulator);

int motorfoc_ap_ctl_set_rotposmode(motorfoc *InstancePtr, int rotposmode, int wnow);
int motorfoc_ap_ctl_get_rotposmode(motorfoc *InstancePtr, int * rotposmode);

int motorfoc_ap_ctl_set_mrt_drive(motorfoc *InstancePtr, int drive, int wnow);
int motorfoc_ap_ctl_set_smo_drive(motorfoc *InstancePtr, int drive, int wnow);
int motorfoc_ap_ctl_set_enc_drive(motorfoc *InstancePtr, int drive, int wnow);

int motorfoc_ap_ctl_zerovphsx(motorfoc *InstancePtr);
int motorfoc_ap_ctl_zerovbusx(motorfoc *InstancePtr);
int motorfoc_ap_ctl_reseterr(motorfoc *InstancePtr);
int motorfoc_ap_ctl_zerocurr(motorfoc *InstancePtr);

#endif /* MOTORFOC_APC_H_ */
