/*
-------------------------------------------------------------------------------------------
-- Copyright  2011-2013, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/*****************************************************************************
* Filename:          C:\zdq\DiGi\mc\sw\kc705\kc705_mc_prj/drivers/netmot_drv_v1_00_a/src/netmot_drv.c
* Version:           1.00.a
* Description:       netmot_drv Driver Source File
* Date:              Fri Nov 18 19:16:10 2011 (by Create and Import Peripheral Wizard)
*****************************************************************************/


/***************************** Include Files *******************************/

#include "netmot_drv.h"

/************************** Function Definitions ***************************/

void netmot_drv_Read_Modify_Write(u32 BaseAddress, u32 RegOffset, u32 mask, u32 value)
{
	u32 tmp = NETMOT_DRV_mReadReg(BaseAddress,RegOffset);
	tmp &= ~(mask);
	tmp |= (value) & (mask);
	NETMOT_DRV_mWriteReg(BaseAddress, RegOffset, tmp);
}			

netmot_drv_Config *netmot_drv_LookupConfig(u16 DeviceId)
{
	netmot_drv_Config *CfgPtr = NULL;
	u32 Index;

	for (Index = 0; Index < XPAR_NETMOT_DRV_NUM_INSTANCES; Index++) {
		if (netmot_drv_ConfigTable[Index].DeviceId == DeviceId) {
			CfgPtr = &netmot_drv_ConfigTable[Index];
			break;
		}
	}

	return CfgPtr;
}

int netmot_drv_Initialize(netmot_drv *InstancePtr, u16 DeviceId)
{
	netmot_drv_Config *netmot_drv_ConfigPtr;/* Pointer to Configuration data. */

	/*
	 * Verify that each of the inputs are valid.
	 */
	Xil_AssertNonvoid(InstancePtr != NULL);

	/*
	 * Lookup the device configuration in the configuration table. Use this
	 * configuration info down below when initializing this driver.
	 */
	netmot_drv_ConfigPtr = netmot_drv_LookupConfig(DeviceId);
	if (netmot_drv_ConfigPtr == NULL) {
		return XST_DEVICE_NOT_FOUND;
	}
	
	// save param
	InstancePtr->Config = netmot_drv_ConfigPtr;
	
    // driver base address
    u32 BaseAddress = netmot_drv_ConfigPtr->BaseAddress;
    
    // disable coil, off outputs and leds
    NETMOT_DRV_mWrite_command(BaseAddress,0);

    // driver Ready
	InstancePtr->IsReady = XIL_COMPONENT_IS_READY;

	return XST_SUCCESS;
}

int netmot_drv_Down(u16 DeviceId)
{
	netmot_drv_Config *netmot_drv_ConfigPtr;
    
    // get configuration address
	netmot_drv_ConfigPtr = netmot_drv_LookupConfig(DeviceId);
	if (netmot_drv_ConfigPtr == NULL)
		return XST_DEVICE_NOT_FOUND;
        
    // driver base address
    u32 BaseAddress = netmot_drv_ConfigPtr->BaseAddress;
    
    // disable coil, off outputs and leds
    NETMOT_DRV_mWrite_command(BaseAddress,0);
    
	return XST_SUCCESS;
}

int netmot_drv_w4_cmd_ack(u32 BaseAddress)
{
	u32 nt;
	// clr TOK and W4 TOK
	NETMOT_DRV_command_clr(BaseAddress,NETMOT_DRV_CMD_TOK_VAL);
	nt = 0;
	while((NETMOT_DRV_mRead_status(BaseAddress) & NETMOT_DRV_STS_TOK_FLG) != 0 && nt < NETMOT_DRV_CMD_ACK_TMO_LOOP)
		nt++;
	// set TOK and W4 TOK
	NETMOT_DRV_command_set(BaseAddress,NETMOT_DRV_CMD_TOK_VAL);
	nt = 0;
	while((NETMOT_DRV_mRead_status(BaseAddress) & NETMOT_DRV_STS_TOK_FLG) == 0 && nt < NETMOT_DRV_CMD_ACK_TMO_LOOP)
		nt++;
	return (nt < NETMOT_DRV_CMD_ACK_TMO_LOOP) ? XST_SUCCESS : XST_FAILURE;
}

int netmot_drv_fpga_dna_clock_and_input(u32 BaseAddress)
{
	int idat;
	netmot_drv_w4_cmd_ack(BaseAddress);
	NETMOT_DRV_command_set(BaseAddress,NETMOT_DRV_CMD_DNA_CLK);
	netmot_drv_w4_cmd_ack(BaseAddress);
	NETMOT_DRV_command_clr(BaseAddress,NETMOT_DRV_CMD_DNA_CLK);
	netmot_drv_w4_cmd_ack(BaseAddress);
	idat = 0 != (NETMOT_DRV_mRead_status(BaseAddress) & NETMOT_DRV_STS_DNA_DATA);
	return idat;
}

int netmot_drv_fpga_dna_read(u32 BaseAddress, netmot_fpga_dna_t *p)
{
	u8  m;
	int i;
	
	NETMOT_DRV_command_clr(BaseAddress,NETMOT_DRV_CMD_DNA_READ|NETMOT_DRV_CMD_DNA_SHIFT|NETMOT_DRV_CMD_DNA_CLK);
	netmot_drv_fpga_dna_clock_and_input(BaseAddress);
	NETMOT_DRV_command_set(BaseAddress,NETMOT_DRV_CMD_DNA_READ );
	netmot_drv_fpga_dna_clock_and_input(BaseAddress);
	NETMOT_DRV_command_clr(BaseAddress,NETMOT_DRV_CMD_DNA_READ );
	memset(p,0,sizeof(*p));
	for(m = 0x01,i=0; i < ((int)(sizeof(p->raw)/sizeof(p->raw.data[0])));)
	{
		int t = netmot_drv_fpga_dna_clock_and_input(BaseAddress);
		if (t)
		{
			p->raw.data[i] |= m;
		}
		if (m == 0x01)
		{
			m = 0x80;
			i++;
		}
		else
		{
			m >>= 1;
		}
		NETMOT_DRV_command_set(BaseAddress,NETMOT_DRV_CMD_DNA_SHIFT);
	}
	return XST_SUCCESS;
}

void netmot_drv_fpga_setup_for_program_dev(u32 BaseAddress,u8 ena)
{
	NETMOT_DRV_mWrite_command(BaseAddress,0);
	netmot_drv_w4_cmd_ack(BaseAddress);
	NETMOT_DRV_mWrite_coil_ctrl(BaseAddress,ena ? (~0) : 0);
}
