/*
-------------------------------------------------------------------------------------------
-- Copyright  2011-2015, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

#ifndef NETMOT_DRV_H
#define NETMOT_DRV_H

/***************************** Include Files *******************************/

#include "xparameters.h"
#include "xil_types.h"
#include "xil_assert.h"
#include "xil_io.h"
#include "xstatus.h"
#include "xenv.h"

/***************************************************************************/

#include "netmot_drv_lx.h"

/**************************** Type Definitions *****************************/

typedef struct {

	u16 DeviceId;
	u32 BaseAddress;
	//-
	u32 systemclock;
	//-
	u8  carrier;
	u8  hw_link_ref_0;
	u8  hw_link_ref_1;
	
} netmot_drv_Config;

extern netmot_drv_Config netmot_drv_ConfigTable[];

typedef struct {

	netmot_drv_Config * Config;
	
	u32 IsReady;
	
} netmot_drv;


typedef union
{
	struct
	{
		 u8 data[8+1];
	} raw;
	struct
	{
		u8 dna[8];
		u8 ver;
	} dec;
    
 } netmot_fpga_dna_t;

/***************** Macros (Inline Functions) Definitions *******************/

/**
 *
 * Write a value to a NETMOT_DRV register. A 32 bit write is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is written.
 *
 * @param   BaseAddress is the base address of the NETMOT_DRV device.
 * @param   RegOffset is the register offset from the base to write to.
 * @param   Data is the data written to the register.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void NETMOT_DRV_mWriteReg(u32 BaseAddress, unsigned RegOffset, u32 Data)
 *
 */
#define NETMOT_DRV_mWriteReg(BaseAddress, RegOffset, Data) \
 	Xil_Out32((BaseAddress) + (RegOffset), (u32)(Data))

/**
 *
 * Read a value from a NETMOT_DRV register. A 32 bit read is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is read from the register. The most significant data
 * will be read as 0.
 *
 * @param   BaseAddress is the base address of the NETMOT_DRV device.
 * @param   RegOffset is the register offset from the base to write to.
 *
 * @return  Data is the data from the register.
 *
 * @note
 * C-style signature:
 * 	u32 NETMOT_DRV_mReadReg(u32 BaseAddress, unsigned RegOffset)
 *
 */
#define NETMOT_DRV_mReadReg(BaseAddress, RegOffset) \
 	Xil_In32((BaseAddress) + (RegOffset))


//
// Write registers
//

// #define NETMOT_DRV_mWrite_ip_ident(BaseAddress,Value)     NETMOT_DRV_mWriteReg(BaseAddress, NETMOT_DRV_RA_ip_ident_OFFSET    , Value)
   #define NETMOT_DRV_mWrite_command(BaseAddress,Value)      NETMOT_DRV_mWriteReg(BaseAddress, NETMOT_DRV_RA_command_OFFSET     , Value)
// #define NETMOT_DRV_mWrite_status(BaseAddress,Value)       NETMOT_DRV_mWriteReg(BaseAddress, NETMOT_DRV_RA_status_OFFSET      , Value)
   #define NETMOT_DRV_mWrite_coil_ctrl(BaseAddress,Value)    NETMOT_DRV_mWriteReg(BaseAddress, NETMOT_DRV_RA_coil_ctrl_OFFSET   , Value)
	
//
// Read registers
//

   #define NETMOT_DRV_mRead_ip_ident(BaseAddress)         NETMOT_DRV_mReadReg(BaseAddress, NETMOT_DRV_RA_ip_ident_OFFSET    )
   #define NETMOT_DRV_mRead_command(BaseAddress)          NETMOT_DRV_mReadReg(BaseAddress, NETMOT_DRV_RA_command_OFFSET     )
   #define NETMOT_DRV_mRead_status(BaseAddress)           NETMOT_DRV_mReadReg(BaseAddress, NETMOT_DRV_RA_status_OFFSET      )
   #define NETMOT_DRV_mRead_coil_ctrl(BaseAddress)        NETMOT_DRV_mReadReg(BaseAddress, NETMOT_DRV_RA_coil_ctrl_OFFSET   )

/************************** Function Prototypes ****************************/

void netmot_drv_Read_Modify_Write(u32 BaseAddress, u32 RegOffset, u32 mask, u32 value);

netmot_drv_Config *netmot_drv_LookupConfig(u16 DeviceId);
int netmot_drv_Initialize(netmot_drv *InstancePtr, u16 DeviceId);
int netmot_drv_Down(u16 DeviceId);

int netmot_drv_w4_cmd_ack(u32 BaseAddress);
int netmot_drv_fpga_dna_clock_and_input(u32 BaseAddress);
int netmot_drv_fpga_dna_read(u32 BaseAddress, netmot_fpga_dna_t *p);

void netmot_drv_fpga_setup_for_program_dev(u32 BaseAddress,u8 ena);

// ===========================================================================

// passive delay s/w waiting for command ack
// WARNING uP speed depending
#define NETMOT_DRV_CMD_ACK_TMO_LOOP 100

// netmot fpga firmware version required
#define NETMOT_DRV_FPGA_FW_VER_REQUEST  9

// ===========================================================================

//-
#define NETMOT_DRV_command_rmw(BaseAddress,mask,value)        netmot_drv_Read_Modify_Write((BaseAddress),NETMOT_DRV_RA_command_OFFSET,(mask),(value))
#define NETMOT_DRV_command_set(BaseAddress,mask)              NETMOT_DRV_command_rmw((BaseAddress),(mask),(mask))
#define NETMOT_DRV_command_clr(BaseAddress,mask)              NETMOT_DRV_command_rmw((BaseAddress),(mask),(0))

#endif /** NETMOT_DRV_H */
