/*
-------------------------------------------------------------------------------------------
-- Copyright  2011-2016, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

#ifndef NETMOT_DRV_LX_H
#define NETMOT_DRV_LX_H

// register offsets

#define NETMOT_DRV_RA_ip_ident_OFFSET        (4*0)
#define NETMOT_DRV_RA_command_OFFSET         (4*1)
#define NETMOT_DRV_RA_status_OFFSET          (4*2)
#define NETMOT_DRV_RA_coil_ctrl_OFFSET       (4*3)

// IP ident code
#define NETMOT_DRV_ip_ident_code_value   2


// command register definitions

#define NETMOT_DRV_CMD_TOK_VAL   (1 << 31)
#define NETMOT_DRV_CMD_DNA_READ  (1 << 30)
#define NETMOT_DRV_CMD_DNA_SHIFT (1 << 29)
#define NETMOT_DRV_CMD_DNA_CLK   (1 << 28)
#define NETMOT_DRV_CMD_RES_27    (1 << 27)
#define NETMOT_DRV_CMD_RES_26    (1 << 26)
#define NETMOT_DRV_CMD_RES_25    (1 << 25)
#define NETMOT_DRV_CMD_OUT_ENA   (1 << 24)

#define NETMOT_DRV_CMD_MOT_MASK  (0xFF)
#define NETMOT_DRV_CMD_MOT_BIT   (16)

#define NETMOT_DRV_CMD_MOT_ENA   (NETMOT_DRV_CMD_MOT_MASK << NETMOT_DRV_CMD_MOT_BIT)


#define NETMOT_DRV_CMD_COIL_3_R_ENA (1 << 23)
#define NETMOT_DRV_CMD_COIL_3_F_ENA (1 << 22)

#define NETMOT_DRV_CMD_COIL_2_R_ENA (1 << 21)
#define NETMOT_DRV_CMD_COIL_2_F_ENA (1 << 20)

#define NETMOT_DRV_CMD_COIL_1_R_ENA (1 << 19)
#define NETMOT_DRV_CMD_COIL_1_F_ENA (1 << 18)

#define NETMOT_DRV_CMD_COIL_0_R_ENA (1 << 17)
#define NETMOT_DRV_CMD_COIL_0_F_ENA (1 << 16)

#define NETMOT_DRV_CMD_LED_MASK  (0xFF)
#define NETMOT_DRV_CMD_LED_BIT   (8)

#define NETMOT_DRV_CMD_OUT_MASK  (0xFF)
#define NETMOT_DRV_CMD_OUT_BIT   (0)

// gear mask for m0/m1
#define NETMOT_DRV_COMMAND_GEAR_MASK_0 (NETMOT_DRV_CMD_COIL_0_F_ENA | NETMOT_DRV_CMD_COIL_0_R_ENA | NETMOT_DRV_CMD_COIL_1_F_ENA | NETMOT_DRV_CMD_COIL_1_R_ENA)
#define NETMOT_DRV_COMMAND_GEAR_MASK_1 (NETMOT_DRV_CMD_COIL_2_F_ENA | NETMOT_DRV_CMD_COIL_2_R_ENA | NETMOT_DRV_CMD_COIL_3_F_ENA | NETMOT_DRV_CMD_COIL_3_R_ENA)


// status register definitions

#define NETMOT_DRV_STS_TOK_FLG  (1 << 31)
#define NETMOT_DRV_STS_DNA_DATA (1 << 30)
#define NETMOT_DRV_STS_MCHB1    (1 << 29)
#define NETMOT_DRV_STS_MCHA1    (1 << 28)
#define NETMOT_DRV_STS_MIDX1    (1 << 27)
#define NETMOT_DRV_STS_MCHB0    (1 << 26)
#define NETMOT_DRV_STS_MCHA0    (1 << 25)
#define NETMOT_DRV_STS_MIDX0    (1 << 24)

#define NETMOT_DRV_STS_INP_MASK (0xFF)
#define NETMOT_DRV_STS_INP_BIT  (16)

#define NETMOT_DRV_STS_ADVCNT   (1 << 15)
#define NETMOT_DRV_STS_ADICNT   (1 << 14)

#define NETMOT_DRV_STS_RES_13   (1 << 13)
#define NETMOT_DRV_STS_RES_12   (1 << 12)
#define NETMOT_DRV_STS_RES_11   (1 << 11)
#define NETMOT_DRV_STS_RES_10   (1 << 10)

#define NETMOT_DRV_STS_VMOT_MASK (0x3FF)
#define NETMOT_DRV_STS_VMOT_BIT  (0)

// Motor COIL control

#define NETMOT_DRV_COIL_MAN_OFS  0
#define NETMOT_DRV_COIL_VAL_OFS  8

#define NETMOT_DRV_COIL_MAN_0  (1 <<  0)
#define NETMOT_DRV_COIL_MAN_1  (1 <<  1)
#define NETMOT_DRV_COIL_MAN_2  (1 <<  2)
#define NETMOT_DRV_COIL_MAN_3  (1 <<  3)
#define NETMOT_DRV_COIL_MAN_4  (1 <<  4)
#define NETMOT_DRV_COIL_MAN_5  (1 <<  5)
#define NETMOT_DRV_COIL_MAN_6  (1 <<  6)
#define NETMOT_DRV_COIL_MAN_7  (1 <<  7)

#define NETMOT_DRV_COIL_VAL_0  (1 <<  8)
#define NETMOT_DRV_COIL_VAL_1  (1 <<  9)
#define NETMOT_DRV_COIL_VAL_2  (1 << 10)
#define NETMOT_DRV_COIL_VAL_3  (1 << 11)
#define NETMOT_DRV_COIL_VAL_4  (1 << 12)
#define NETMOT_DRV_COIL_VAL_5  (1 << 13)
#define NETMOT_DRV_COIL_VAL_6  (1 << 14)
#define NETMOT_DRV_COIL_VAL_7  (1 << 15)

// ===========================================================================

// Dead time
#define NETMOT_DRV_DEF_DEADTIME   (100.0e-9)
#define NETMOT_DRV_MIN_DEADTIME   (100.0e-9)
#define NETMOT_DRV_MAX_DEADTIME   (100.0e-9)

// S/D analog conversion
#define NETMOT_DRV_SD_VBUS_GAIN   (295.41e-6)
#define NETMOT_DRV_SD_CURR_GAIN   (123.35e-6)
#define NETMOT_DRV_SD_CURR_TIME   (3.2e-6)

// current limit
#define NETMOT_DRV_DEF_CURR_LIMIT (7.21)


#endif /** NETMOT_DRV_LX_H */
