/*
-------------------------------------------------------------------------------------------
-- Copyright © 2011-2017, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/


#ifndef TEC53_DRV_H
#define TEC53_DRV_H

// =========================== Include Files ==============================

#include "xil_types.h"
#include "xil_assert.h"
#include "xil_io.h"
#include "xstatus.h"
#include "xenv.h"

// ===========================================================================

#include "tec53_drv_lx.h"

// =========================== Type Definitions ==============================

typedef struct {

	u16 DeviceId;
	u32 BaseAddress;
	//-
	u32 systemclock;
	//-
	u8  carrier;
	u8  hw_link_ref;
	u16 inv_acqtime_ns;
	
} tec53_drv_Config;

extern tec53_drv_Config tec53_drv_ConfigTable[];



typedef struct {

	tec53_drv_Config * Config;
	
	u32 IsReady;
	
} tec53_drv;

// ===========================================================================

#define TEC53_DRV_mWriteReg(BaseAddress, RegOffset, Data) \
  	Xil_Out32((BaseAddress) + (RegOffset), (u32)(Data))

#define TEC53_DRV_mReadReg(BaseAddress, RegOffset) \
    Xil_In32((BaseAddress) + (RegOffset))

// ===========================================================================


//
// Write registers
//

// #define TEC53_DRV_mWrite_ip_ident(BaseAddress,Value)         TEC53_DRV_mWriteReg((BaseAddress), TEC53_DRV_RA_ip_ident_OFFSET,(Value))
   
//
// Read registers
//

  #define TEC53_DRV_mRead_ip_ident(BaseAddress)                 TEC53_DRV_mReadReg((BaseAddress), TEC53_DRV_RA_ip_ident_OFFSET)

// ===========================================================================

tec53_drv_Config *tec53_drv_LookupConfig(u16 DeviceId);
int tec53_drv_Initialize(tec53_drv *InstancePtr, u16 DeviceId);
int tec53_drv_Down(u16 DeviceId);

// ===========================================================================

#endif // TEC53_DRV_H
