/*
-------------------------------------------------------------------------------------------
-- Copyright © 2011-2016, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

#ifndef TLIMOT_DRV_H
#define TLIMOT_DRV_H


/****************** Include Files ********************/

#include "xil_types.h"
#include "xil_assert.h"
#include "xil_io.h"
#include "xstatus.h"
#include "xenv.h"

/***************************************************************************/

#include "tlimot_drv_lx.h"

/**************************** Type Definitions *****************************/

typedef struct {

	u16 DeviceId;
	u32 BaseAddress;
	//-
	u32 systemclock;
	//-
	u8  carrier;
	u8  hw_link_ref;
	
} tlimot_drv_Config;

extern tlimot_drv_Config tlimot_drv_ConfigTable[];

typedef struct {

	tlimot_drv_Config * Config;
	
	u32 IsReady;
	
} tlimot_drv;

// ===========================================================================

#define TLIMOT_DRV_mWriteReg(BaseAddress, RegOffset, Data) \
  	Xil_Out32((BaseAddress) + (RegOffset), (u32)(Data))

#define TLIMOT_DRV_mReadReg(BaseAddress, RegOffset) \
    Xil_In32((BaseAddress) + (RegOffset))

// ===========================================================================


//
// Write registers
//

// #define TLIMOT_DRV_mWrite_ip_ident(BaseAddress,Value)         TLIMOT_DRV_mWriteReg((BaseAddress), TLIMOT_DRV_RA_ip_ident_OFFSET   ,(Value))
// #define TLIMOT_DRV_mWrite_status_reg(BaseAddress,Value)       TLIMOT_DRV_mWriteReg((BaseAddress), TLIMOT_DRV_RA_status_reg_OFFSET ,(Value))
   #define TLIMOT_DRV_mWrite_control_reg(BaseAddress,Value)      TLIMOT_DRV_mWriteReg((BaseAddress), TLIMOT_DRV_RA_control_reg_OFFSET,(Value))
   #define TLIMOT_DRV_mWrite_coil_ctrl(BaseAddress,Value)        TLIMOT_DRV_mWriteReg((BaseAddress), TLIMOT_DRV_RA_coil_ctrl_OFFSET  ,(Value))
   
//
// Read registers
//

   #define TLIMOT_DRV_mRead_ip_ident(BaseAddress)                TLIMOT_DRV_mReadReg((BaseAddress), TLIMOT_DRV_RA_ip_ident_OFFSET   )
   #define TLIMOT_DRV_mRead_status_reg(BaseAddress)              TLIMOT_DRV_mReadReg((BaseAddress), TLIMOT_DRV_RA_status_reg_OFFSET )
   #define TLIMOT_DRV_mRead_control_reg(BaseAddress)             TLIMOT_DRV_mReadReg((BaseAddress), TLIMOT_DRV_RA_control_reg_OFFSET)
   #define TLIMOT_DRV_mRead_coil_ctrl(BaseAddress)               TLIMOT_DRV_mReadReg((BaseAddress), TLIMOT_DRV_RA_coil_ctrl_OFFSET  )
   
/***************************************************************************/

   #define TLIMOT_DRV_Read_Modify_Write(BaseAddress,RegOffset,Mask,Value) {\
				u32 tmp = TLIMOT_DRV_mReadReg((BaseAddress), (RegOffset)); \
				tmp &= ~(Mask); \
				tmp |= (Mask) & (Value); \
				TLIMOT_DRV_mWriteReg((BaseAddress), (RegOffset) ,tmp); \
			}

/***************************************************************************/

// sign-extend macro
#define TLIMOT_DRV_sign_extend(v,b) (((v) & (1 << ((b)-1))) ? ((v) | (-(1 << (b)))) : ((v) & ((1 << (b))-1)))

// SIGMA DELTA ACQ
#define TLIMOT_DRV_sd_adi_vbus(value)      TLIMOT_DRV_sign_extend((value),18)
#define TLIMOT_DRV_sd_adi_curr_ab(value)   TLIMOT_DRV_sign_extend((value),18)

// fs=2^17 @ 0.320 mV Rb=499, Ra=200K*4+499, gain=1/1605.2; 0.320mV * 1605.2 = 513.666 Volts
#define TLIMOT_DRV_sd_dc_link_mvolts(value)   (((value) * (513666 >> 6)) >> (17-6))

// fs=2^17 @ 0.320 mV, shunt=0.01, jp 1-2 Fs=32
#define TLIMOT_DRV_sd_curr_ab_mamps(value)   (((value) * (32000 >> 1)) >> (17-1))

/***************************************************************************/

tlimot_drv_Config *tlimot_drv_LookupConfig(u16 DeviceId);
int tlimot_drv_Initialize(tlimot_drv *InstancePtr, u16 DeviceId);
int tlimot_drv_Down(u16 DeviceId);

// ===========================================================================

#endif // TLIMOT_DRV_H
