/*
-------------------------------------------------------------------------------------------
-- Copyright © 2011-2016, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/



/***************************** Include Files *******************************/

#include "uart4bus.h"

/************************** Function Definitions ***************************/

uart4bus_Config *uart4bus_LookupConfig(u16 DeviceId)
{
	uart4bus_Config *CfgPtr = NULL;
	u32 Index;

	for (Index = 0; Index < XPAR_UART4BUS_NUM_INSTANCES; Index++) {
		if (uart4bus_ConfigTable[Index].DeviceId == DeviceId) {
			CfgPtr = &uart4bus_ConfigTable[Index];
			break;
		}
	}

	return CfgPtr;
}

int uart4bus_Initialize(uart4bus *InstancePtr, u16 DeviceId)
{
	uart4bus_Config *uart4bus_ConfigPtr;/* Pointer to Configuration data. */

	/*
	* Verify that each of the inputs are valid.
	*/
	Xil_AssertNonvoid(InstancePtr != NULL);

	/*
	* Lookup the device configuration in the configuration table. Use this
	* configuration info down below when initializing this driver.
	*/
	uart4bus_ConfigPtr = uart4bus_LookupConfig(DeviceId);
	if (uart4bus_ConfigPtr == NULL) {
		return XST_DEVICE_NOT_FOUND;
	}

	// save param
	InstancePtr->Config = uart4bus_ConfigPtr;

	// driver Ready
	InstancePtr->IsReady = XIL_COMPONENT_IS_READY;

	return XST_SUCCESS;
}

int uart4bus_Down(u16 DeviceId)
{
	uart4bus_Config *uart4bus_ConfigPtr;

	// get configuration address
	uart4bus_ConfigPtr = uart4bus_LookupConfig(DeviceId);
	if (uart4bus_ConfigPtr == NULL) {
		return XST_DEVICE_NOT_FOUND;
	}

	return XST_SUCCESS;
}
