/*
-------------------------------------------------------------------------------------------
-- Copyright © 2011-2016, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

#ifndef UART4BUS_H
#define UART4BUS_H

// =========================== Include Files ==============================

#include "xil_types.h"
#include "xil_assert.h"
#include "xil_io.h"
#include "xstatus.h"
#include "xenv.h"

// ===========================================================================

#include "uart4bus_lx.h"

// =========================== Type Definitions ==============================

typedef struct {

	u16 DeviceId;
	u32 BaseAddress;
	//-
	u32 systemclock;
	//-
	u8  uart_num;
	//-
	u8  space_value;
	u8  edata_value;
	u8  clkdiv_width;
	u8  weight_width;
	u8  txfifo_depth;
	u8  rxfifo_depth;
	
} uart4bus_Config;

extern uart4bus_Config uart4bus_ConfigTable[];


typedef struct {

	uart4bus_Config * Config;
	
	u32 IsReady;
	
} uart4bus;

// ===========================================================================

#define UART4BUS_mWriteReg(BaseAddress, RegOffset, Data) \
  	Xil_Out32((BaseAddress) + (RegOffset), (u32)(Data))

#define UART4BUS_mReadReg(BaseAddress, RegOffset) \
    Xil_In32((BaseAddress) + (RegOffset))

// ===========================================================================

//
// Write registers
//

// #define UART4BUS_mWrite_ip_ident(BaseAddress,Value)              UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_ip_ident_OFFSET ,(Value))

   #define UART4BUS_mWrite_wordct_tx(BaseAddress,Value)             UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_wordct_tx_OFFSET,(Value))
   #define UART4BUS_mWrite_clkpre_tx(BaseAddress,Value)             UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_clkpre_tx_OFFSET,(Value))
   #define UART4BUS_mWrite_clkbit_tx(BaseAddress,Value)             UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_clkbit_tx_OFFSET,(Value))
   #define UART4BUS_mWrite_clkstp_tx(BaseAddress,Value)             UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_clkstp_tx_OFFSET,(Value))
   #define UART4BUS_mWrite_clkpos_tx(BaseAddress,Value)             UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_clkpos_tx_OFFSET,(Value))

   #define UART4BUS_mWrite_wordct_rx(BaseAddress,Value)             UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_wordct_rx_OFFSET,(Value))
   #define UART4BUS_mWrite_weight_rx(BaseAddress,Value)             UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_weight_rx_OFFSET,(Value))
   #define UART4BUS_mWrite_clkbit_rx(BaseAddress,Value)             UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_clkbit_rx_OFFSET,(Value))
   #define UART4BUS_mWrite_clkstp_rx(BaseAddress,Value)             UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_clkstp_rx_OFFSET,(Value))
   #define UART4BUS_mWrite_clkidl_rx(BaseAddress,Value)             UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_clkidl_rx_OFFSET,(Value))
   
// #define UART4BUS_mWrite_status_rx_ch(BaseAddress,Channel,Value)  UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_status_rx_ch_OFFSET(Channel) ,(Value))
// #define UART4BUS_mWrite_rcvfifo_rx_ch(BaseAddress,Channel,Value) UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_rcvfifo_rx_ch_OFFSET(Channel),(Value))
// #define UART4BUS_mWrite_status_tx_ch(BaseAddress,Channel,Value)  UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_status_tx_ch_OFFSET(Channel) ,(Value))
   #define UART4BUS_mWrite_sndfifo_tx_ch(BaseAddress,Channel,Value) UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_sndfifo_tx_ch_OFFSET(Channel),(Value))

// #define UART4BUS_mWrite_gpio_in(BaseAddress,Value)               UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_gpio_in_OFFSET ,(Value))
   #define UART4BUS_mWrite_gpio_out(BaseAddress,Value)              UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_gpio_out_OFFSET,(Value))
   #define UART4BUS_mWrite_gpio_tri(BaseAddress,Value)              UART4BUS_mWriteReg((BaseAddress),UART4BUS_RA_gpio_tri_OFFSET,(Value))
   
//
// Read registers
//

   #define UART4BUS_mRead_ip_ident(BaseAddress)                     UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_ip_ident_OFFSET )

   #define UART4BUS_mRead_wordct_tx(BaseAddress)                    UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_wordct_tx_OFFSET)
   #define UART4BUS_mRead_clkpre_tx(BaseAddress)                    UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_clkpre_tx_OFFSET)
   #define UART4BUS_mRead_clkbit_tx(BaseAddress)                    UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_clkbit_tx_OFFSET)
   #define UART4BUS_mRead_clkstp_tx(BaseAddress)                    UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_clkstp_tx_OFFSET)
   #define UART4BUS_mRead_clkpos_tx(BaseAddress)                    UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_clkpos_tx_OFFSET)

   #define UART4BUS_mRead_wordct_rx(BaseAddress)                    UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_wordct_rx_OFFSET)
   #define UART4BUS_mRead_weight_rx(BaseAddress)                    UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_weight_rx_OFFSET)
   #define UART4BUS_mRead_clkbit_rx(BaseAddress)                    UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_clkbit_rx_OFFSET)
   #define UART4BUS_mRead_clkstp_rx(BaseAddress)                    UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_clkstp_rx_OFFSET)
   #define UART4BUS_mRead_clkidl_rx(BaseAddress)                    UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_clkidl_rx_OFFSET)

   #define UART4BUS_mRead_status_rx_ch(BaseAddress,Channel)         UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_status_rx_ch_OFFSET(Channel) )
   #define UART4BUS_mRead_rcvfifo_rx_ch(BaseAddress,Channel)        UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_rcvfifo_rx_ch_OFFSET(Channel))
   #define UART4BUS_mRead_status_tx_ch(BaseAddress,Channel)         UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_status_tx_ch_OFFSET(Channel) )
   #define UART4BUS_mRead_sndfifo_tx_ch(BaseAddress,Channel)        UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_sndfifo_tx_ch_OFFSET(Channel))

   #define UART4BUS_mRead_gpio_in(BaseAddress)                      UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_gpio_in_OFFSET )
   #define UART4BUS_mRead_gpio_out(BaseAddress)                     UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_gpio_out_OFFSET)
   #define UART4BUS_mRead_gpio_tri(BaseAddress)                     UART4BUS_mReadReg((BaseAddress),UART4BUS_RA_gpio_tri_OFFSET)
   
// ===========================================================================

// max clock divisor
#define uart4bus_DivclkMax(ConfigPtr)                ((1 << (ConfigPtr)->clkdiv_width) - 1)

// clip to max
#define uart4bus_Clip2DivclkMax(ConfigPtr,divclk) \
	( { u32 _req_ = (u32)(divclk); u32 _lim_ = (u32)uart4bus_DivclkMax(ConfigPtr); _req_ < _lim_ ? _req_ : _lim_ ; } )


// TX set/get control word

#define uart4bus_SetSendCtrl(ConfigPtr,wordct)       UART4BUS_mWrite_wordct_tx((ConfigPtr)->BaseAddress,(wordct))
#define uart4bus_GetSendCtrl(ConfigPtr)              UART4BUS_mRead_wordct_tx((ConfigPtr)->BaseAddress)

// RX set/get control word

#define uart4bus_SetRecvCtrl(ConfigPtr,wordct)       UART4BUS_mWrite_wordct_rx((ConfigPtr)->BaseAddress,(wordct))
#define uart4bus_GetRecvCtrl(ConfigPtr)              UART4BUS_mRead_wordct_rx((ConfigPtr)->BaseAddress)

// RX timing

#define uart4bus_GetRecvFiltBit(ConfigPtr)           ((ConfigPtr)->systemclock / (UART4BUS_mRead_weight_rx((ConfigPtr)->BaseAddress) - UART4BUS_WEIGHT_ADJ_SET))
#define uart4bus_GetRecvFreqBit(ConfigPtr)           ((ConfigPtr)->systemclock / (UART4BUS_mRead_clkbit_rx((ConfigPtr)->BaseAddress) - UART4BUS_CLKBIT_ADJ_SET))
#define uart4bus_GetRecvStopBit(ConfigPtr)           ((ConfigPtr)->systemclock / (UART4BUS_mRead_clkstp_rx((ConfigPtr)->BaseAddress) - UART4BUS_CLKSTP_ADJ_SET))
#define uart4bus_GetRecvIdleBit(ConfigPtr)           ((ConfigPtr)->systemclock / (UART4BUS_mRead_clkidl_rx((ConfigPtr)->BaseAddress) - UART4BUS_CLKIDL_ADJ_SET))

#define uart4bus_SetRecvFiltBit(ConfigPtr,freqhz)    UART4BUS_mWrite_weight_rx((ConfigPtr)->BaseAddress,uart4bus_Clip2DivclkMax((ConfigPtr),(int)((ConfigPtr)->systemclock / (freqhz)) + UART4BUS_WEIGHT_ADJ_SET))
#define uart4bus_SetRecvFreqBit(ConfigPtr,freqhz)    UART4BUS_mWrite_clkbit_rx((ConfigPtr)->BaseAddress,uart4bus_Clip2DivclkMax((ConfigPtr),(int)((ConfigPtr)->systemclock / (freqhz)) + UART4BUS_CLKBIT_ADJ_SET))
#define uart4bus_SetRecvStopBit(ConfigPtr,freqhz)    UART4BUS_mWrite_clkstp_rx((ConfigPtr)->BaseAddress,uart4bus_Clip2DivclkMax((ConfigPtr),(int)((ConfigPtr)->systemclock / (freqhz)) + UART4BUS_CLKSTP_ADJ_SET))
#define uart4bus_SetRecvIdleBit(ConfigPtr,freqhz)    UART4BUS_mWrite_clkidl_rx((ConfigPtr)->BaseAddress,uart4bus_Clip2DivclkMax((ConfigPtr),(int)((ConfigPtr)->systemclock / (freqhz)) + UART4BUS_CLKIDL_ADJ_SET))


// TX timing

#define uart4bus_GetSendPreeBit(ConfigPtr)           ((ConfigPtr)->systemclock / (UART4BUS_mRead_clkpre_tx((ConfigPtr)->BaseAddress) - UART4BUS_CLKPRE_ADJ_SET))
#define uart4bus_GetSendFreqBit(ConfigPtr)           ((ConfigPtr)->systemclock / (UART4BUS_mRead_clkbit_tx((ConfigPtr)->BaseAddress) - UART4BUS_CLKBIT_ADJ_SET))
#define uart4bus_GetSendStopBit(ConfigPtr)           ((ConfigPtr)->systemclock / (UART4BUS_mRead_clkstp_tx((ConfigPtr)->BaseAddress) - UART4BUS_CLKSTP_ADJ_SET))
#define uart4bus_GetSendPostBit(ConfigPtr)           ((ConfigPtr)->systemclock / (UART4BUS_mRead_clkpos_tx((ConfigPtr)->BaseAddress) - UART4BUS_CLKPOS_ADJ_SET))

#define uart4bus_SetSendPreeBit(ConfigPtr,freqhz)    UART4BUS_mWrite_clkpre_tx((ConfigPtr)->BaseAddress,uart4bus_Clip2DivclkMax((ConfigPtr),(int)((ConfigPtr)->systemclock / (freqhz)) + UART4BUS_CLKPRE_ADJ_SET))
#define uart4bus_SetSendFreqBit(ConfigPtr,freqhz)    UART4BUS_mWrite_clkbit_tx((ConfigPtr)->BaseAddress,uart4bus_Clip2DivclkMax((ConfigPtr),(int)((ConfigPtr)->systemclock / (freqhz)) + UART4BUS_CLKBIT_ADJ_SET))
#define uart4bus_SetSendStopBit(ConfigPtr,freqhz)    UART4BUS_mWrite_clkstp_tx((ConfigPtr)->BaseAddress,uart4bus_Clip2DivclkMax((ConfigPtr),(int)((ConfigPtr)->systemclock / (freqhz)) + UART4BUS_CLKSTP_ADJ_SET))
#define uart4bus_SetSendPostBit(ConfigPtr,freqhz)    UART4BUS_mWrite_clkpos_tx((ConfigPtr)->BaseAddress,uart4bus_Clip2DivclkMax((ConfigPtr),(int)((ConfigPtr)->systemclock / (freqhz)) + UART4BUS_CLKPOS_ADJ_SET))


// ===========================================================================

// TX full setup
#define uart4bus_SetSendSetup(ConfigPtr,WordFormat,BaudRate,PrologTime,StopBits,EpilogTime) { \
		\
	uart4bus_SetSendPreeBit((ConfigPtr),(u32)(1.0f / (float)(PrologTime))); \
	uart4bus_SetSendFreqBit((ConfigPtr),(u32)(BaudRate)); \
	uart4bus_SetSendStopBit((ConfigPtr),(u32)((float)(BaudRate) / (float)(StopBits))); \
	uart4bus_SetSendPostBit((ConfigPtr),(u32)(1.0f / (float)(EpilogTime))); \
		\
    uart4bus_SetSendCtrl((ConfigPtr),(WordFormat)); \
}

// RX full setup
#define uart4bus_SetRecvSetup(ConfigPtr,WordFormat,BaudRate,FilterTime,StopBits,IdleBits) { \
		\
	uart4bus_SetRecvFiltBit((ConfigPtr),(u32)(1.0f / (float)(FilterTime))); \
	uart4bus_SetRecvFreqBit((ConfigPtr),(u32)(BaudRate)); \
	uart4bus_SetRecvStopBit((ConfigPtr),(u32)((float)(BaudRate) / (float)(StopBits))); \
	uart4bus_SetRecvIdleBit((ConfigPtr),(u32)((float)(BaudRate) / (float)(IdleBits))); \
		\
    uart4bus_SetRecvCtrl((ConfigPtr),(WordFormat)); \
}

// ===========================================================================

uart4bus_Config *uart4bus_LookupConfig(u16 DeviceId);
int uart4bus_Initialize(uart4bus *InstancePtr, u16 DeviceId);
int uart4bus_Down(u16 DeviceId);

// ===========================================================================

#endif // UART4BUS_H
