/*
-------------------------------------------------------------------------------------------
-- Copyright © 2011-2016, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/


#include <stdio.h>

// ===========================================================================

#include "uart4bus_apd.h"

// ===========================================================================


// display IP raw register
void uart4bus_DisplayRaw(const u32 BaseAddress, const int Channel, const u8 NoTouch)
{
	printf("ip_ident  =x%08lx\n", UART4BUS_mRead_ip_ident(BaseAddress));

	printf("wordct_tx =x%08lx\n", UART4BUS_mRead_wordct_tx(BaseAddress));
	printf("clkpre_tx =x%08lx\n", UART4BUS_mRead_clkpre_tx(BaseAddress));
	printf("clkbit_tx =x%08lx\n", UART4BUS_mRead_clkbit_tx(BaseAddress));
	printf("clkstp_tx =x%08lx\n", UART4BUS_mRead_clkstp_tx(BaseAddress));
	printf("clkpos_tx =x%08lx\n", UART4BUS_mRead_clkpos_tx(BaseAddress));

	printf("wordct_rx =x%08lx\n", UART4BUS_mRead_wordct_rx(BaseAddress));
	printf("weight_rx =x%08lx\n", UART4BUS_mRead_weight_rx(BaseAddress));
	printf("clkbit_rx =x%08lx\n", UART4BUS_mRead_clkbit_rx(BaseAddress));
	printf("clkstp_rx =x%08lx\n", UART4BUS_mRead_clkstp_rx(BaseAddress));
	printf("clkidl_rx =x%08lx\n", UART4BUS_mRead_clkidl_rx(BaseAddress));
	
	printf("status_rx [%d]=x%08lx\n",Channel, UART4BUS_mRead_status_rx_ch(BaseAddress,Channel));
	if (NoTouch == 0)
		printf("rcvfifo_rx[%d]=x%08lx\n",Channel, UART4BUS_mRead_rcvfifo_rx_ch(BaseAddress,Channel));

	printf("status_tx [%d]=x%08lx\n",Channel, UART4BUS_mRead_status_tx_ch(BaseAddress,Channel));
	printf("sndfifo_tx[%d]=x%08lx\n",Channel, UART4BUS_mRead_sndfifo_tx_ch(BaseAddress,Channel));

	printf("\n");
}

// display IP setup
void uart4bus_DisplaySetup(uart4bus_Config * ConfigPtr)
{
	if (ConfigPtr == NULL) {
		return;
	}

    // RX timing
    printf("RX Timing\n");
    printf("Filter Time =%g\n"  , 1.0f / (float)uart4bus_GetRecvFiltBit(ConfigPtr));
    printf("Baud Rate   =%lu\n" , uart4bus_GetRecvFreqBit(ConfigPtr));
    printf("Stop Bits   =%.3f\n", (float)uart4bus_GetRecvFreqBit(ConfigPtr) / (float)uart4bus_GetRecvStopBit(ConfigPtr));
    printf("Idle Bits   =%.3f\n", (float)uart4bus_GetRecvFreqBit(ConfigPtr) / (float)uart4bus_GetRecvIdleBit(ConfigPtr));
    printf("Bit-9       =%s\n"  ,
			(uart4bus_GetRecvCtrl(ConfigPtr) == UART4BUS_WORDCT_XX_NONE   ) ? "none"  :
			(uart4bus_GetRecvCtrl(ConfigPtr) == UART4BUS_WORDCT_XX_MARK   ) ? "mark"  :
			(uart4bus_GetRecvCtrl(ConfigPtr) == UART4BUS_WORDCT_XX_SPACE  ) ? "space" :
			(uart4bus_GetRecvCtrl(ConfigPtr) == UART4BUS_WORDCT_XX_PAREVEN) ? "even"  :
			(uart4bus_GetRecvCtrl(ConfigPtr) == UART4BUS_WORDCT_XX_PARODD ) ? "odd"   : "none");
    printf("\n");

    // TX timing
    printf("TX Timing\n");
    printf("Prolog Time =%g\n"  , 1.0f / (float)uart4bus_GetSendPreeBit(ConfigPtr));
    printf("Baud Rate   =%lu\n" , uart4bus_GetSendFreqBit(ConfigPtr));
    printf("Stop Bits   =%.3f\n", (float)uart4bus_GetSendFreqBit(ConfigPtr) / (float)uart4bus_GetSendStopBit(ConfigPtr));
    printf("Epilog Time =%g\n"  , 1.0f / (float)uart4bus_GetSendPostBit(ConfigPtr));
    printf("Bit-9       =%s\n"  ,
			(uart4bus_GetSendCtrl(ConfigPtr) == UART4BUS_WORDCT_XX_NONE   ) ? "none"  :
			(uart4bus_GetSendCtrl(ConfigPtr) == UART4BUS_WORDCT_XX_MARK   ) ? "mark"  :
			(uart4bus_GetSendCtrl(ConfigPtr) == UART4BUS_WORDCT_XX_SPACE  ) ? "space" :
			(uart4bus_GetSendCtrl(ConfigPtr) == UART4BUS_WORDCT_XX_PAREVEN) ? "even"  :
			(uart4bus_GetSendCtrl(ConfigPtr) == UART4BUS_WORDCT_XX_PARODD ) ? "odd"   : "none");
    printf("\n");
}
