/*
-------------------------------------------------------------------------------------------
-- Copyright © 2011-2017, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

#ifndef UART4BUS_LX_H
#define UART4BUS_LX_H

// register offsets

#define UART4BUS_RA_ip_ident_OFFSET     (4*0)

#define UART4BUS_RA_wordct_tx_OFFSET    (4*1)
#define UART4BUS_RA_clkpre_tx_OFFSET    (4*2)
#define UART4BUS_RA_clkbit_tx_OFFSET    (4*3)
#define UART4BUS_RA_clkstp_tx_OFFSET    (4*4)
#define UART4BUS_RA_clkpos_tx_OFFSET    (4*5)

#define UART4BUS_RA_wordct_rx_OFFSET    (4*6)
#define UART4BUS_RA_weight_rx_OFFSET    (4*7)
#define UART4BUS_RA_clkbit_rx_OFFSET    (4*8)
#define UART4BUS_RA_clkstp_rx_OFFSET    (4*9)
#define UART4BUS_RA_clkidl_rx_OFFSET    (4*10)

#define UART4BUS_RA_status_rx_0_OFFSET  (4*11)
#define UART4BUS_RA_rcvfifo_rx_0_OFFSET (4*12)
#define UART4BUS_RA_status_tx_0_OFFSET  (4*13)
#define UART4BUS_RA_sndfifo_tx_0_OFFSET (4*14)

#define UART4BUS_RA_status_rx_ch_OFFSET(Channel)    (UART4BUS_RA_status_rx_0_OFFSET  + (Channel) * 16)
#define UART4BUS_RA_rcvfifo_rx_ch_OFFSET(Channel)   (UART4BUS_RA_rcvfifo_rx_0_OFFSET + (Channel) * 16)
#define UART4BUS_RA_status_tx_ch_OFFSET(Channel)    (UART4BUS_RA_status_tx_0_OFFSET  + (Channel) * 16)
#define UART4BUS_RA_sndfifo_tx_ch_OFFSET(Channel)   (UART4BUS_RA_sndfifo_tx_0_OFFSET + (Channel) * 16)

#define UART4BUS_RA_gpio_in_OFFSET      (4*75)
#define UART4BUS_RA_gpio_out_OFFSET     (4*76)
#define UART4BUS_RA_gpio_tri_OFFSET     (4*77)

// IP ident code
#define UART4BUS_ip_ident_code_value   104

// ===========================================================================

// ---------------------------------------------------------------------------

// -- TX & RX word control
#define UART4BUS_WORDCT_XX_B9ENA     ( 0)
#define UART4BUS_WORDCT_XX_B9SEED    ( 1)
#define UART4BUS_WORDCT_XX_B9PRTY    ( 2)

// -- TX send fifo register
#define UART4BUS_SNDFIFO_TX_FFDT0    ( 8)

// -- TX specific
#define UART4BUS_STATUS_TX_FFDT0     ( 8)
#define UART4BUS_STATUS_TX_BUSY      ( 9)
#define UART4BUS_STATUS_TX_EDATA     (10)
#define UART4BUS_STATUS_TX_SDATA     (11)

// -- RX specific
#define UART4BUS_STATUS_RX_B9ERR     ( 8)
#define UART4BUS_STATUS_RX_FRERR     ( 9)
#define UART4BUS_STATUS_RX_IDLE      (10)
#define UART4BUS_STATUS_RX_SDATA     (11)

// -- TX & RX status register
#define UART4BUS_STATUS_XX_EMPTY     (12)
#define UART4BUS_STATUS_XX_FULL      (13)
#define UART4BUS_STATUS_XX_FLUSH     (14)
#define UART4BUS_STATUS_XX_SPARE_15  (15)
#define UART4BUS_STATUS_XX_LEVEL_LSB (16)
#define UART4BUS_STATUS_XX_LEVEL_MSB (23)

// ---------------------------------------------------------------------------

// ===========================================================================

#define UART4BUS_WORDCT_XX_NONE     (0 \
				| 0 * (1 << UART4BUS_WORDCT_XX_B9ENA )\
				| 0 * (1 << UART4BUS_WORDCT_XX_B9SEED) \
				| 0 * (1 << UART4BUS_WORDCT_XX_B9PRTY) )

#define UART4BUS_WORDCT_XX_MARK     (0 \
				| 1 * (1 << UART4BUS_WORDCT_XX_B9ENA )\
				| 1 * (1 << UART4BUS_WORDCT_XX_B9SEED) \
				| 0 * (1 << UART4BUS_WORDCT_XX_B9PRTY) )

#define UART4BUS_WORDCT_XX_SPACE    (0 \
				| 1 * (1 << UART4BUS_WORDCT_XX_B9ENA )\
				| 0 * (1 << UART4BUS_WORDCT_XX_B9SEED) \
				| 0 * (1 << UART4BUS_WORDCT_XX_B9PRTY) )

#define UART4BUS_WORDCT_XX_PAREVEN  (0 \
				| 1 * (1 << UART4BUS_WORDCT_XX_B9ENA )\
				| 0 * (1 << UART4BUS_WORDCT_XX_B9SEED) \
				| 1 * (1 << UART4BUS_WORDCT_XX_B9PRTY) )

#define UART4BUS_WORDCT_XX_PARODD   (0 \
				| 1 * (1 << UART4BUS_WORDCT_XX_B9ENA )\
				| 1 * (1 << UART4BUS_WORDCT_XX_B9SEED) \
				| 1 * (1 << UART4BUS_WORDCT_XX_B9PRTY) )
				
// transmit modes

#define uart4bus_sndfifo_tx_complex(f,v)     (((f) << UART4BUS_SNDFIFO_TX_FFDT0) | (v))

#define uart4bus_sndfifo_tx_data(value)      uart4bus_sndfifo_tx_complex(0,(value) & 255)
#define uart4bus_sndfifo_tx_space()          uart4bus_sndfifo_tx_complex(1,0)
#define uart4bus_sndfifo_tx_mark()           uart4bus_sndfifo_tx_complex(1,1)

// status register bit function

#define uart4bus_status_tx_ffdt0(status)     (((status) >> UART4BUS_STATUS_TX_FFDT0   ) &   1)
#define uart4bus_status_tx_busy(status)      (((status) >> UART4BUS_STATUS_TX_BUSY    ) &   1)
#define uart4bus_status_tx_edata(status)     (((status) >> UART4BUS_STATUS_TX_EDATA   ) &   1)
#define uart4bus_status_tx_sdata(status)     (((status) >> UART4BUS_STATUS_TX_SDATA   ) &   1)

#define uart4bus_status_rx_b9err(status)     (((status) >> UART4BUS_STATUS_RX_B9ERR   ) &   1)
#define uart4bus_status_rx_frerr(status)     (((status) >> UART4BUS_STATUS_RX_FRERR   ) &   1)
#define uart4bus_status_rx_idle(status)      (((status) >> UART4BUS_STATUS_RX_IDLE    ) &   1)
#define uart4bus_status_rx_sdata(status)     (((status) >> UART4BUS_STATUS_RX_SDATA   ) &   1)

#define uart4bus_status_xx_empty(status)     (((status) >> UART4BUS_STATUS_XX_EMPTY    ) &   1)
#define uart4bus_status_xx_full(status)      (((status) >> UART4BUS_STATUS_XX_FULL     ) &   1)
#define uart4bus_status_xx_flush(status)     (((status) >> UART4BUS_STATUS_XX_FLUSH    ) &   1)
#define uart4bus_status_xx_level(status)     (((status) >> UART4BUS_STATUS_XX_LEVEL_LSB) & 255)

// ===========================================================================

// add values local to register
// sub values register to local

#define UART4BUS_CLKPRE_ADJ_SET  (-3)
#define UART4BUS_CLKBIT_ADJ_SET  (-2)
#define UART4BUS_CLKSTP_ADJ_SET  (-2)
#define UART4BUS_CLKPOS_ADJ_SET  (-2)
#define UART4BUS_WEIGHT_ADJ_SET  (-2)
#define UART4BUS_CLKIDL_ADJ_SET  (-2)

// ===========================================================================


#endif // UART4BUS_LX_H
