/*
-------------------------------------------------------------------------------------------
-- Copyright  2011-2013, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

#ifndef ZEDODISP_H
#define ZEDODISP_H

/***************************** Include Files *******************************/

#include "xparameters.h"
#include "xil_types.h"
#include "xil_assert.h"
#include "xil_io.h"
#include "xstatus.h"
#include "xenv.h"

/***************************************************************************/

#include "zedodisp_lx.h"

/**************************** Type Definitions *****************************/

typedef struct {

	u16 DeviceId;
	u32 BaseAddress;
	
	u32 sclk_freq_hz;
	u8  fmem_read_en;
	
} zedodisp_Config;

extern zedodisp_Config zedodisp_ConfigTable[];

typedef struct {

	zedodisp_Config * Config;
	
	u32 IsReady;
	
} zedodisp;


/***************** Macros (Inline Functions) Definitions *******************/

/**
 *
 * Write a value to a ZEDODISP register. A 32 bit write is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is written.
 *
 * @param   BaseAddress is the base address of the ZEDODISP device.
 * @param   RegOffset is the register offset from the base to write to.
 * @param   Data is the data written to the register.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void ZEDODISP_mWriteReg(u32 BaseAddress, unsigned RegOffset, u32 Data)
 *
 */
#define ZEDODISP_mWriteReg(BaseAddress, RegOffset, Data) \
 	Xil_Out32((BaseAddress) + (RegOffset), (u32)(Data))

/**
 *
 * Read a value from a ZEDODISP register. A 32 bit read is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is read from the register. The most significant data
 * will be read as 0.
 *
 * @param   BaseAddress is the base address of the ZEDODISP device.
 * @param   RegOffset is the register offset from the base to write to.
 *
 * @return  Data is the data from the register.
 *
 * @note
 * C-style signature:
 * 	u32 ZEDODISP_mReadReg(u32 BaseAddress, unsigned RegOffset)
 *
 */
#define ZEDODISP_mReadReg(BaseAddress, RegOffset) \
 	Xil_In32((BaseAddress) + (RegOffset))


//
// Write registers
//

#define ZEDODISP_mWrite_fmem(BaseAddress,Offset,Value)     ZEDODISP_mWriteReg(BaseAddress, ZEDODISP_fmem_OFFSET     + (Offset), Value)
#define ZEDODISP_mWrite_ctrlst(BaseAddress,Value)          ZEDODISP_mWriteReg(BaseAddress, ZEDODISP_ctrlst_OFFSET   , Value)

	
//
// Read registers
//

#define ZEDODISP_mRead_fmem(BaseAddress,Offset)           ZEDODISP_mReadReg(BaseAddress, ZEDODISP_fmem_OFFSET     + (Offset))
#define ZEDODISP_mRead_ctrlst(BaseAddress)                ZEDODISP_mReadReg(BaseAddress, ZEDODISP_ctrlst_OFFSET)

/************************** Derived macros *********************************/

#define ZEDODISP_fmem_copyto(BaseAddress,Value) { \
				int o; \
				u32 * p = (u32 *)(Value); \
				for(o = 0; o < ZEDODISP_FMEM_SIZE; o += 4) { \
					u32 t = *p++; \
					ZEDODISP_mWrite_fmem(BaseAddress,o,t); \
				} \
			}

#define ZEDODISP_shutdown(BaseAddress) ZEDODISP_mWrite_ctrlst(BaseAddress,0)

#define ZEDODISP_ctrlst_rmw(BaseAddress,Mask,Value) { \
				u32 t32; \
				t32 = ZEDODISP_mRead_ctrlst(BaseAddress); \
				t32 &= ~(Mask); \
				t32 |= (Value) & (Mask); \
				ZEDODISP_mWrite_ctrlst(BaseAddress,t32); \
			}

#define ZEDODISP_ctrlst_dtcmd_set(BaseAddress,Value) \
			ZEDODISP_ctrlst_rmw(BaseAddress, \
				((1 << ZEDODISP_CTRLST_XREQ) | (0xFF << ZEDODISP_CTRLST_D0)), \
				((1 << ZEDODISP_CTRLST_XREQ) | (((Value) & 0xFF) << ZEDODISP_CTRLST_D0)))

#define ZEDODISP_ctrlst_dtcmd_idle(BaseAddress) ZEDODISP_ctrlst_rmw(BaseAddress,(1 << ZEDODISP_CTRLST_XREQ),0)

#define ZEDODISP_ctrlst_dc(BaseAddress,Value)   ZEDODISP_ctrlst_rmw(BaseAddress,(1 << ZEDODISP_CTRLST_DC),  (Value) ? (1 << ZEDODISP_CTRLST_DC)  :0)
#define ZEDODISP_ctrlst_res(BaseAddress,Value)  ZEDODISP_ctrlst_rmw(BaseAddress,(1 << ZEDODISP_CTRLST_RES), (Value) ? (1 << ZEDODISP_CTRLST_RES) :0)
#define ZEDODISP_ctrlst_vdd(BaseAddress,Value)  ZEDODISP_ctrlst_rmw(BaseAddress,(1 << ZEDODISP_CTRLST_VDD), (Value) ? (1 << ZEDODISP_CTRLST_VDD) :0)
#define ZEDODISP_ctrlst_vbat(BaseAddress,Value) ZEDODISP_ctrlst_rmw(BaseAddress,(1 << ZEDODISP_CTRLST_VBAT),(Value) ? (1 << ZEDODISP_CTRLST_VBAT):0)
#define ZEDODISP_ctrlst_auto(BaseAddress,Value) ZEDODISP_ctrlst_rmw(BaseAddress,(1 << ZEDODISP_CTRLST_AUTO),(Value) ? (1 << ZEDODISP_CTRLST_AUTO):0)

#define ZEDODISP_ctrlst_is_xack(BaseAddress) (0 != (ZEDODISP_mRead_ctrlst(BaseAddress) & (1 << ZEDODISP_CTRLST_XACK)))
#define ZEDODISP_ctrlst_is_busy(BaseAddress) (0 != (ZEDODISP_mRead_ctrlst(BaseAddress) & (1 << ZEDODISP_CTRLST_BUSY)))
			
/************************** Function Prototypes ****************************/

zedodisp_Config *zedodisp_LookupConfig(u16 DeviceId);
int zedodisp_Initialize(zedodisp *InstancePtr, u16 DeviceId);
int zedodisp_Down(u16 DeviceId);

#endif /** ZEDODISP_H */
