/*
-------------------------------------------------------------------------------------------
-- Copyright  2011-2014, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief DS1820 demo
/// \file ds1820_demo.c
/// \version 0.1
/// \date oct 09, 2014
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2009-2014


#include "xparameters.h"

#include "xtime_l.h"
#include "xuartps.h"

#include "ds1820.h"
#include "ds1820_apc.h"
#include "ds1820_apd.h"

#include <stdio.h>


int getcharnowait()
{
	int key = -1;
	if (XUartPs_IsReceiveData(STDIN_BASEADDRESS))
	{
		key = XUartPs_ReadReg(STDIN_BASEADDRESS, XUARTPS_FIFO_OFFSET) & 0xFF;
	}
	return key;
}


//-
//- DS1820 test program
//-

int main()
{
	int rc;
	ds1820 dev;

	ds1820 *InstancePtr = & dev;

	rc = ds1820_Initialize(InstancePtr,XPAR_DS1820_0_DEVICE_ID);
	if (XST_SUCCESS != rc)
		return XST_FAILURE;

	printf("\n" "%s %s" "\n",__DATE__, __TIME__);

	printf("\n");


	// flush stdin buffer
	while(-1 != getcharnowait())
		;

	// end-of-test
	u8 eot = 0;

	int lpcnt;
	for(lpcnt = 1;eot == 0; lpcnt++)
	{

		XTime tconvst,tlpsp;

		XTime_GetTime(&tlpsp);

		u8 id = 0;

		ds1820_Process_Run(InstancePtr,DS1820_CMD_INIT);
		u8 eos = 0;
		for(;eos==0;)
		{
			XTime te,tx;
			XTime_GetTime(&te);

			tx = te - tlpsp;
			if (tx >= (COUNTS_PER_SECOND *5))
			{
				printf("Timeout!\n");
				break;
			}

			int key = getcharnowait();
			if (key == 3 || key == 27)
			{
				eot = 1;
				break;
			}

			ds1820_Process_Run(InstancePtr,DS1820_CMD_RUN);

			switch(ds1820_Process_Event(InstancePtr))
			{
				case DS1820_EVENT_NONE:
					{
					}
					break;
				case DS1820_EVENT_TCONV_STARTED:
					{
						XTime_GetTime(&tconvst);
					}
					break;
				case DS1820_EVENT_TCONV_RUNNING:
					{
						tx = te - tconvst;
						if (tx >= (DS1820_TCONV_MAX_MS * (COUNTS_PER_SECOND / 1000)))
						{
							ds1820_Process_Run(InstancePtr,DS1820_CMD_KICK);
						}
					}
					break;
				case DS1820_EVENT_DEVICE_FOUND:
					{
						id++;
						printf("[%d] ",id);
						ds1820_ap_ident_disp(InstancePtr);
					}
					break;
				case DS1820_EVENT_TEMP_READED:
					{
						printf("[%d] ",id);
						ds1820_ap_temp_disp(InstancePtr);
						printf("\n");
					}
					break;
				case DS1820_EVENT_END_OF_SCAN:
					{
						printf("[%d Execute in %.3f]\n\n",lpcnt, (float)tx / (float)COUNTS_PER_SECOND);
						eos = 1;
					}
					break;
			}
		}
	}

    printf("\n" "That's all Folks!" "\n");

    return XST_SUCCESS;
}
