/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief board definition
/// \file board.h
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#ifndef BOARD_H_
#define BOARD_H_

#include "xparameters.h"

//
#ifdef XPAR_MPHY18_DRV_NUM_INSTANCES
	#define SYS_MPHY18_NUM XPAR_MPHY18_DRV_NUM_INSTANCES
#else
	#define SYS_MPHY18_NUM 0
#endif

//
#ifdef XPAR_MPHY17_DRV_NUM_INSTANCES
	#define SYS_MPHY17_NUM XPAR_MPHY17_DRV_NUM_INSTANCES
#else
	#define SYS_MPHY17_NUM 0
#endif

//
#ifdef XPAR_MPHY16_DRV_NUM_INSTANCES
	#define SYS_MPHY16_NUM XPAR_MPHY16_DRV_NUM_INSTANCES
#else
	#define SYS_MPHY16_NUM 0
#endif

//
#ifdef XPAR_TEP2_DRV_NUM_INSTANCES
	#define SYS_TEP2_NUM XPAR_TEP2_DRV_NUM_INSTANCES
#else
	#define SYS_TEP2_NUM 0
#endif

//
#ifdef XPAR_TEC53_DRV_NUM_INSTANCES
	#define SYS_TEC53_NUM XPAR_TEC53_DRV_NUM_INSTANCES
#else
	#define SYS_TEC53_NUM 0
#endif

//
#ifdef XPAR_S4SICINV_DRV_NUM_INSTANCES
	#define SYS_S4SICINV_NUM XPAR_S4SICINV_DRV_NUM_INSTANCES
#else
	#define SYS_S4SICINV_NUM 0
#endif

//
#ifdef XPAR_QINVSIC_DRV_NUM_INSTANCES
	#define SYS_QINVSIC_NUM XPAR_QINVSIC_DRV_NUM_INSTANCES
#else
	#define SYS_QINVSIC_NUM 0
#endif

//
#ifdef XPAR_MPHY11_DRV_NUM_INSTANCES
	#define SYS_MPHY11_NUM XPAR_MPHY11_DRV_NUM_INSTANCES
#else
	#define SYS_MPHY11_NUM 0
#endif

//
#ifdef XPAR_ZYGUSIC_DRV_NUM_INSTANCES
	#define SYS_ZYGUSIC_NUM XPAR_ZYGUSIC_DRV_NUM_INSTANCES
#else
	#define SYS_ZYGUSIC_NUM 0
#endif

//
#ifdef XPAR_TLIMOT_DRV_NUM_INSTANCES
	#define SYS_TLIMOT_NUM XPAR_TLIMOT_DRV_NUM_INSTANCES
#else
	#define SYS_TLIMOT_NUM 0
#endif

//
#ifdef XPAR_MLIEVB_DRV_NUM_INSTANCES
	#define SYS_MLIEVB_NUM XPAR_MLIEVB_DRV_NUM_INSTANCES
#else
	#define SYS_MLIEVB_NUM 0
#endif

//
#ifdef XPAR_MOTCON2_DRV_NUM_INSTANCES
	#define SYS_MOTCON2_NUM XPAR_MOTCON2_DRV_NUM_INSTANCES
#else
	#define SYS_MOTCON2_NUM 0
#endif

//
#ifdef XPAR_MOTCON1_DRV_NUM_INSTANCES
	#define SYS_MOTCON1_NUM XPAR_MOTCON1_DRV_NUM_INSTANCES
#else
	#define SYS_MOTCON1_NUM 0
#endif

//
#ifdef XPAR_SGXBCU_DRV_NUM_INSTANCES
	#define SYS_SGXBCU_NUM XPAR_SGXBCU_DRV_NUM_INSTANCES
#else
	#define SYS_SGXBCU_NUM 0
#endif

//
#ifdef XPAR_FMCMC1_DRV_NUM_INSTANCES
	#define SYS_FMCMC1_NUM XPAR_FMCMC1_DRV_NUM_INSTANCES
#else
	#define SYS_FMCMC1_NUM 0
#endif

//
#ifdef XPAR_NETMOT_DRV_NUM_INSTANCES
	#define SYS_NETMOT_NUM XPAR_NETMOT_DRV_NUM_INSTANCES
#else
	#define SYS_NETMOT_NUM 0
#endif

//
#ifdef XPAR_MOTORFOC_NUM_INSTANCES
	#define SYS_MOTORFOC_NUM XPAR_MOTORFOC_NUM_INSTANCES
#else
	#define SYS_MOTORFOC_NUM 0
#endif

// total # motor control cards
#define SYS_TOTAL_FMC_BOARDS (SYS_MPHY18_NUM + SYS_MPHY17_NUM + SYS_MPHY16_NUM + SYS_TEP2_NUM + SYS_TEC53_NUM + SYS_S4SICINV_NUM + SYS_QINVSIC_NUM + SYS_MPHY11_NUM + SYS_ZYGUSIC_NUM + SYS_TLIMOT_NUM + SYS_MLIEVB_NUM + SYS_MOTCON2_NUM + SYS_MOTCON1_NUM + SYS_SGXBCU_NUM + SYS_FMCMC1_NUM + SYS_NETMOT_NUM)

// carrier board

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_MPHY18_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_MPHY18_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_MPHY18_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_MPHY18_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_MPHY17_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_MPHY17_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_MPHY17_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_MPHY17_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_MPHY16_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_MPHY16_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_MPHY16_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_MPHY16_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_TEP2_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_TEP2_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_TEP2_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_TEP2_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_TEC53_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_TEC53_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_TEC53_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_TEC53_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_S4SICINV_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_S4SICINV_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_S4SICINV_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_S4SICINV_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_QINVSIC_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_QINVSIC_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_QINVSIC_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_QINVSIC_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_MPHY11_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_MPHY11_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_MPHY11_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_MPHY11_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_ZYGUSIC_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_ZYGUSIC_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_ZYGUSIC_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_ZYGUSIC_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_TLIMOT_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_TLIMOT_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_TLIMOT_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_TLIMOT_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_MLIEVB_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_MLIEVB_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_MLIEVB_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_MLIEVB_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_MOTCON2_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_MOTCON2_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_MOTCON2_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_MOTCON2_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_MOTCON1_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_MOTCON1_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_MOTCON1_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_MOTCON1_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_SGXBCU_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_SGXBCU_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_SGXBCU_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_SGXBCU_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_FMCMC1_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_FMCMC1_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_FMCMC1_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_FMCMC1_DRV_1_CARRIER
#endif

//
#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_NETMOT_DRV_0_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_NETMOT_DRV_0_CARRIER
#endif

#if !defined(SYS_CARRIER_IDENT) && defined(XPAR_NETMOT_DRV_1_CARRIER)
	#define  SYS_CARRIER_IDENT             XPAR_NETMOT_DRV_1_CARRIER
#endif

// what about carrier board ?
#if !defined(SYS_CARRIER_IDENT)
	#error "Systme Carrier Ident not defined"
#endif

#if SYS_CARRIER_IDENT == 1
	#define SYS_CARRIER_LYR178 1
	#define SYS_CARRIER_NAME "LYR178"
#endif

#if SYS_CARRIER_IDENT == 2
	#define SYS_CARRIER_SP605  1
	#define SYS_CARRIER_NAME "SP605"
#endif

#if SYS_CARRIER_IDENT == 3
	#define SYS_CARRIER_LX150T 1
	#define SYS_CARRIER_NAME "LX150T"
#endif

#if SYS_CARRIER_IDENT == 4
	#define SYS_CARRIER_LX75T 1
	#define SYS_CARRIER_NAME "LX75T"
#endif

#if SYS_CARRIER_IDENT == 5
	#define SYS_CARRIER_KC705 1
	#define SYS_CARRIER_NAME "KC705"
#endif

#if SYS_CARRIER_IDENT == 6
	#define SYS_CARRIER_ZC702 1
	#define SYS_CARRIER_NAME "ZC702"
#endif

#if SYS_CARRIER_IDENT == 7
	#define SYS_CARRIER_ZEDBOARD 1
	#define SYS_CARRIER_NAME "ZedBoard"
#endif

#if SYS_CARRIER_IDENT == 8
	#define SYS_CARRIER_TE0720 1
	#define SYS_CARRIER_NAME "TE0720"
#endif

#if SYS_CARRIER_IDENT == 9
	#define SYS_CARRIER_MICROZED 1
	#define SYS_CARRIER_NAME "MicroZed"
#endif

#if SYS_CARRIER_IDENT == 10
	#define SYS_CARRIER_PICOZED 1
	#define SYS_CARRIER_NAME "PicoZed"
#endif

#if SYS_CARRIER_IDENT == 11
	#define SYS_CARRIER_ZYBO 1
	#define SYS_CARRIER_NAME "ZYBO"
#endif

#if SYS_CARRIER_IDENT == 12
	#define SYS_CARRIER_ARTY 1
	#define SYS_CARRIER_NAME "Arty"
#endif

#if SYS_CARRIER_IDENT == 13
	#define SYS_CARRIER_QINVSIC 1
	#define SYS_CARRIER_NAME "QinvSiC"
#endif

#if SYS_CARRIER_IDENT == 14
	#define SYS_CARRIER_ARTYZ7 1
	#define SYS_CARRIER_NAME "ArtyZ7"
#endif

#if SYS_CARRIER_IDENT == 15
	#define SYS_CARRIER_ULTRAZED 1
	#define SYS_CARRIER_NAME "UltraZed"
#endif

#if SYS_CARRIER_IDENT == 16
	#define SYS_CARRIER_CORAZ7 1
	#define SYS_CARRIER_NAME "CoraZ7"
#endif

#if SYS_CARRIER_IDENT == 17
	#define SYS_CARRIER_MINIZED 1
	#define SYS_CARRIER_NAME "MiniZed"
#endif

#if SYS_CARRIER_IDENT == 18
	#define SYS_CARRIER_TOPICURP 1
	#define SYS_CARRIER_NAME "TopicURP"
#endif


//
#if !defined(XPAR_FPGA_CLOCK_FREQ_HZ) && defined(XPAR_MOTORFOC_0_SYSTEMCLOCK) && XPAR_MOTORFOC_0_SYSTEMCLOCK > 0
	#define  XPAR_FPGA_CLOCK_FREQ_HZ             XPAR_MOTORFOC_0_SYSTEMCLOCK
#endif


// fall to MicroBlaze ..
#if !defined(XPAR_FPGA_CLOCK_FREQ_HZ) && defined(XPAR_MICROBLAZE_CORE_CLOCK_FREQ_HZ) && XPAR_MICROBLAZE_CORE_CLOCK_FREQ_HZ > 0
	#define  XPAR_FPGA_CLOCK_FREQ_HZ             XPAR_MICROBLAZE_CORE_CLOCK_FREQ_HZ
#endif

#if !defined(XPAR_FPGA_CLOCK_FREQ_HZ)
	#error "Cannot get PL clock"
#endif

// EtherCAT

#ifdef XPAR_ESC_AXI_ETHERCAT_USER_0_BASEADDR
	#define XPAR_ETHERCAT_USER_0_BASEADDR XPAR_ESC_AXI_ETHERCAT_USER_0_BASEADDR
#endif

#ifdef XPAR_AXI_ETHERCAT_USER_0_S_AXI_BASEADDR
	#define XPAR_ETHERCAT_USER_0_BASEADDR XPAR_AXI_ETHERCAT_USER_0_S_AXI_BASEADDR
#endif

#ifdef XPAR_ETHERCAT_USER_0_BASEADDR
	#define SYS_ETHERCAT_NUM 1
#else
	#define SYS_ETHERCAT_NUM 0
#endif

#ifdef XPAR_DS1820_NUM_INSTANCES
	#define SYS_DS1820_NUM XPAR_DS1820_NUM_INSTANCES
#else
	#define SYS_DS1820_NUM 0
#endif

#ifdef XPAR_IVPLOGGER_NUM_INSTANCES
	#define SYS_IVPLOGGER_NUM XPAR_IVPLOGGER_NUM_INSTANCES
#else
	#define SYS_IVPLOGGER_NUM 0
#endif


// XADCPS

// def 1
#define SYS_QXADCPS_NUM 1

// no XADC -> undef
#if defined(SYS_QXADCPS_NUM) && !defined(XPAR_PS7_XADC_0_DEVICE_ID)
     #undef SYS_QXADCPS_NUM
#endif

// boards with XADC

#if defined(SYS_QXADCPS_NUM) && defined(XPAR_FMCMC1_DRV_0_XADC) && XPAR_FMCMC1_DRV_0_XADC > 0
     #undef SYS_QXADCPS_NUM
#endif

#if defined(SYS_QXADCPS_NUM) && defined(XPAR_FMCMC1_DRV_1_XADC) && XPAR_FMCMC1_DRV_1_XADC > 0
     #undef SYS_QXADCPS_NUM
#endif

#if defined(SYS_QXADCPS_NUM) && defined(XPAR_MOTCON1_DRV_0_XADC) && XPAR_MOTCON1_DRV_0_XADC > 0
     #undef SYS_QXADCPS_NUM
#endif

#if defined(SYS_QXADCPS_NUM) && defined(XPAR_MOTCON1_DRV_1_XADC) && XPAR_MOTCON1_DRV_1_XADC > 0
     #undef SYS_QXADCPS_NUM
#endif

#if defined(SYS_QXADCPS_NUM) && defined(XPAR_MOTCON2_DRV_0_XADC) && XPAR_MOTCON2_DRV_0_XADC > 0
     #undef SYS_QXADCPS_NUM
#endif

#if defined(SYS_QXADCPS_NUM) && defined(XPAR_MOTCON2_DRV_1_XADC) && XPAR_MOTCON2_DRV_1_XADC > 0
     #undef SYS_QXADCPS_NUM
#endif


// undef -> def 0
#if !defined(SYS_QXADCPS_NUM)
	#define SYS_QXADCPS_NUM 0
#endif

#if defined(SYS_CARRIER_TE0720) && SYS_CARRIER_TE0720 > 0
	#if SYS_S4SICINV_NUM == 0
		#define SYS_TE0701_NUM 1
	#endif
#endif


#ifdef XPAR_UART4BUS_NUM_INSTANCES
	#define SYS_UART4BUS_NUM XPAR_UART4BUS_NUM_INSTANCES
#else
	#define SYS_UART4BUS_NUM 0
#endif


#endif /*BOARD_H_*/
