/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief can agent
/// \file canagent.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "glovar.h"

#if SW_ENA_CAN_AGENT > 0

#include "mcaxcan.h"

// ---------------------------------------------------------------------------

// IP range size
#define MCA_CAN_BOARD_SIZE    (64*1024)
#define MCA_CAN_MOTOR_SIZE    (64*1024)

typedef struct
{
	u8 ip_ident;
	u8 instances;
} mca_can_ip_map_t;

static const mca_can_ip_map_t mca_can_ip_map_v [] =
{
	{SYSTEM_ip_ident_code_value,1},
#if SYS_MPHY18_NUM > 0
	{MPHY18_DRV_ip_ident_code_value,SYS_MPHY18_NUM},
#endif
#if SYS_MPHY17_NUM > 0
	{MPHY17_DRV_ip_ident_code_value,SYS_MPHY17_NUM},
#endif
#if SYS_MPHY16_NUM > 0
	{MPHY16_DRV_ip_ident_code_value,SYS_MPHY16_NUM},
#endif
#if SYS_TEP2_NUM > 0
	{TEP2_DRV_ip_ident_code_value,SYS_TEP2_NUM},
#endif
#if SYS_TEC53_NUM > 0
	{TEC53_DRV_ip_ident_code_value,SYS_TEC53_NUM},
#endif
#if SYS_S4SICINV_NUM > 0
	{S4SICINV_DRV_ip_ident_code_value,SYS_S4SICINV_NUM},
#endif
#if SYS_QINVSIC_NUM > 0
	{QINVSIC_DRV_ip_ident_code_value,SYS_QINVSIC_NUM},
#endif
#if SYS_MPHY11_NUM > 0
	{MPHY11_DRV_ip_ident_code_value,SYS_MPHY11_NUM},
#endif
#if SYS_ZYGUSIC_NUM > 0
	{ZYGUSIC_DRV_ip_ident_code_value,SYS_ZYGUSIC_NUM},
#endif
#if SYS_TLIMOT_NUM > 0
	{TLIMOT_DRV_ip_ident_code_value,SYS_TLIMOT_NUM},
#endif
#if SYS_MLIEVB_NUM > 0
	{MLIEVB_DRV_ip_ident_code_value,SYS_MLIEVB_NUM},
#endif
#if SYS_MOTCON2_NUM > 0
	{MOTCON2_DRV_ip_ident_code_value,SYS_MOTCON2_NUM},
#endif
#if SYS_MOTCON1_NUM > 0
	{MOTCON1_DRV_ip_ident_code_value,SYS_MOTCON1_NUM},
#endif
#if SYS_SGXBCU_NUM > 0
	{SGXBCU_DRV_ip_ident_code_value,SYS_SGXBCU_NUM},
#endif
#if SYS_FMCMC1_NUM > 0
	{FMCMC1_DRV_ip_ident_code_value,SYS_FMCMC1_NUM},
#endif
#if SYS_NETMOT_NUM > 0
	{NETMOT_DRV_ip_ident_code_value,SYS_NETMOT_NUM},
#endif
#if SYS_MOTORFOC_NUM > 0
	{MOTORFOC_ip_ident_code_value,SYS_MOTORFOC_NUM},
#endif

};

#define mca_can_ip_map_n (sizeof(mca_can_ip_map_v)/sizeof(mca_can_ip_map_v[0]))

// ---------------------------------------------------------------------------
// ---------------------------------------------------------------------------
// ---------------------------------------------------------------------------
// ---------------------------------------------------------------------------

// ---------------------------------------------------------------------------

#if SYS_MPHY18_NUM > 0

static const u16 mca_can_chrid2ofs_mphy18_stat_v[] =
{
	MPHY18_DRV_RA_status_reg_OFFSET,
	MPHY18_DRV_RA_ecu_temp_OFFSET,
	MPHY18_DRV_RA_rm_temp_OFFSET,
};

#define mca_can_chrid2ofs_mphy18_stat_n (sizeof(mca_can_chrid2ofs_mphy18_stat_v)/sizeof(mca_can_chrid2ofs_mphy18_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_mphy18_stat_n];
} mca_can_xstat_mphy18_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_MPHY17_NUM > 0

static const u16 mca_can_chrid2ofs_mphy17_stat_v[] =
{
	MPHY17_DRV_RA_status_reg_OFFSET,
	MPHY17_DRV_RA_ecu_temp_OFFSET,
	MPHY17_DRV_RA_rm_temp_OFFSET,
};

#define mca_can_chrid2ofs_mphy17_stat_n (sizeof(mca_can_chrid2ofs_mphy17_stat_v)/sizeof(mca_can_chrid2ofs_mphy17_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_mphy17_stat_n];
} mca_can_xstat_mphy17_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_MPHY16_NUM > 0

static const u16 mca_can_chrid2ofs_mphy16_stat_v[] =
{
	MPHY16_DRV_RA_status_reg_OFFSET,
	MPHY16_DRV_RA_digin_OFFSET,
	MPHY16_DRV_RA_vinrush_OFFSET,
	MPHY16_DRV_RA_ecu_temp_0_OFFSET,
	MPHY16_DRV_RA_ecu_temp_1_OFFSET,
	MPHY16_DRV_RA_ecu_temp_2_OFFSET,
	MPHY16_DRV_RA_ecu_temp_3_OFFSET,
	MPHY16_DRV_RA_ecu_temp_4_OFFSET,
	MPHY16_DRV_RA_rm_temp_0_OFFSET,
	MPHY16_DRV_RA_rm_temp_1_OFFSET,
	MPHY16_DRV_RA_rm_temp_2_OFFSET,
	MPHY16_DRV_RA_rm_temp_3_OFFSET,
	MPHY16_DRV_RA_fan_tacho_0_OFFSET,
	MPHY16_DRV_RA_fan_tacho_1_OFFSET,
	MPHY16_DRV_RA_fan_tacho_2_OFFSET,
};

#define mca_can_chrid2ofs_mphy16_stat_n (sizeof(mca_can_chrid2ofs_mphy16_stat_v)/sizeof(mca_can_chrid2ofs_mphy16_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_mphy16_stat_n];
} mca_can_xstat_mphy16_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_TEP2_NUM > 0

static const u16 mca_can_chrid2ofs_tep2_stat_v[] =
{
};

#define mca_can_chrid2ofs_tep2_stat_n (sizeof(mca_can_chrid2ofs_tep2_stat_v)/sizeof(mca_can_chrid2ofs_tep2_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_tep2_stat_n];
} mca_can_xstat_tep2_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_TEC53_NUM > 0

static const u16 mca_can_chrid2ofs_tec53_stat_v[] =
{
};

#define mca_can_chrid2ofs_tec53_stat_n (sizeof(mca_can_chrid2ofs_tec53_stat_v)/sizeof(mca_can_chrid2ofs_tec53_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_tec53_stat_n];
} mca_can_xstat_tec53_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_S4SICINV_NUM > 0

static const u16 mca_can_chrid2ofs_s4sicinv_stat_v[] =
{
	S4SICINV_DRV_RA_status_reg_OFFSET,
	S4SICINV_DRV_RA_digin_OFFSET,
	S4SICINV_DRV_RA_rm_temp_0_OFFSET,
	S4SICINV_DRV_RA_rm_temp_1_OFFSET,
	S4SICINV_DRV_RA_rm_temp_2_OFFSET,
	S4SICINV_DRV_RA_rm_temp_3_OFFSET,
	S4SICINV_DRV_RA_rm_temp_4_OFFSET,
	S4SICINV_DRV_RA_rm_temp_5_OFFSET,
	S4SICINV_DRV_RA_rm_temp_6_OFFSET,
	S4SICINV_DRV_RA_rm_temp_7_OFFSET,
};

#define mca_can_chrid2ofs_s4sicinv_stat_n (sizeof(mca_can_chrid2ofs_s4sicinv_stat_v)/sizeof(mca_can_chrid2ofs_s4sicinv_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_s4sicinv_stat_n];
} mca_can_xstat_s4sicinv_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_QINVSIC_NUM > 0

static const u16 mca_can_chrid2ofs_qinvsic_stat_v[] =
{
	QINVSIC_DRV_RA_status_reg_OFFSET,
	QINVSIC_DRV_RA_digin_OFFSET,
	QINVSIC_DRV_RA_rm_temp_0_OFFSET,
	QINVSIC_DRV_RA_rm_temp_1_OFFSET,
	QINVSIC_DRV_RA_rm_temp_2_OFFSET,
	QINVSIC_DRV_RA_rm_temp_3_OFFSET,
	QINVSIC_DRV_RA_rm_temp_4_OFFSET,
	QINVSIC_DRV_RA_rm_temp_5_OFFSET,
	QINVSIC_DRV_RA_rm_temp_6_OFFSET,
	QINVSIC_DRV_RA_rm_temp_7_OFFSET,
	QINVSIC_DRV_RA_rm_temp_8_OFFSET,
	QINVSIC_DRV_RA_rm_temp_8_OFFSET,
	QINVSIC_DRV_RA_rm_temp_10_OFFSET,
	QINVSIC_DRV_RA_rm_temp_11_OFFSET,
	QINVSIC_DRV_RA_rm_temp_12_OFFSET,
	QINVSIC_DRV_RA_rm_temp_13_OFFSET,
	QINVSIC_DRV_RA_rm_temp_14_OFFSET,
	QINVSIC_DRV_RA_rm_temp_15_OFFSET,
};

#define mca_can_chrid2ofs_qinvsic_stat_n (sizeof(mca_can_chrid2ofs_qinvsic_stat_v)/sizeof(mca_can_chrid2ofs_qinvsic_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_qinvsic_stat_n];
} mca_can_xstat_qinvsic_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_MPHY11_NUM > 0

static const u16 mca_can_chrid2ofs_mphy11_stat_v[] =
{
	MPHY11_DRV_RA_status_reg_OFFSET,
	MPHY11_DRV_RA_digin_OFFSET,
	MPHY11_DRV_RA_anin_0_OFFSET,
	MPHY11_DRV_RA_anin_1_OFFSET,
	MPHY11_DRV_RA_anin_2_OFFSET,
	MPHY11_DRV_RA_anin_3_OFFSET,
	MPHY11_DRV_RA_anin_4_OFFSET,
	MPHY11_DRV_RA_anin_5_OFFSET,
	MPHY11_DRV_RA_anin_6_OFFSET,
	MPHY11_DRV_RA_anin_7_OFFSET,
	MPHY11_DRV_RA_ecu_temp_0_OFFSET,
	MPHY11_DRV_RA_ecu_temp_1_OFFSET,
	MPHY11_DRV_RA_ecu_temp_2_OFFSET,
	MPHY11_DRV_RA_ecu_temp_3_OFFSET,
	MPHY11_DRV_RA_ecu_temp_4_OFFSET,
	MPHY11_DRV_RA_ecu_temp_5_OFFSET,
	MPHY11_DRV_RA_ecu_temp_6_OFFSET,
	MPHY11_DRV_RA_ecu_temp_7_OFFSET,
	MPHY11_DRV_RA_rm_temp_0_OFFSET,
	MPHY11_DRV_RA_rm_temp_1_OFFSET,
	MPHY11_DRV_RA_rm_temp_2_OFFSET,
	MPHY11_DRV_RA_rm_temp_3_OFFSET,
	MPHY11_DRV_RA_rm_temp_4_OFFSET,
	MPHY11_DRV_RA_rm_temp_5_OFFSET,
	MPHY11_DRV_RA_rm_temp_6_OFFSET,
	MPHY11_DRV_RA_rm_temp_7_OFFSET,
	MPHY11_DRV_RA_rm_temp_8_OFFSET,
	MPHY11_DRV_RA_rm_temp_9_OFFSET,
	MPHY11_DRV_RA_rm_temp_10_OFFSET,
	MPHY11_DRV_RA_rm_temp_11_OFFSET,
	MPHY11_DRV_RA_rm_temp_12_OFFSET,
	MPHY11_DRV_RA_rm_temp_13_OFFSET,
	MPHY11_DRV_RA_rm_temp_14_OFFSET,
	MPHY11_DRV_RA_rm_temp_15_OFFSET,
	MPHY11_DRV_RA_fan_tacho_0_OFFSET,
	MPHY11_DRV_RA_fan_tacho_1_OFFSET,
};

#define mca_can_chrid2ofs_mphy11_stat_n (sizeof(mca_can_chrid2ofs_mphy11_stat_v)/sizeof(mca_can_chrid2ofs_mphy11_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_mphy11_stat_n];
} mca_can_xstat_mphy11_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_ZYGUSIC_NUM > 0

static const u16 mca_can_chrid2ofs_zygusic_stat_v[] =
{
	ZYGUSIC_DRV_RA_status_reg_OFFSET,
};

#define mca_can_chrid2ofs_zygusic_stat_n (sizeof(mca_can_chrid2ofs_zygusic_stat_v)/sizeof(mca_can_chrid2ofs_zygusic_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_zygusic_stat_n];
} mca_can_xstat_zygusic_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_TLIMOT_NUM > 0

static const u16 mca_can_chrid2ofs_tlimot_stat_v[] =
{
	TLIMOT_DRV_RA_status_reg_OFFSET,
};

#define mca_can_chrid2ofs_tlimot_stat_n (sizeof(mca_can_chrid2ofs_tlimot_stat_v)/sizeof(mca_can_chrid2ofs_tlimot_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_tlimot_stat_n];
} mca_can_xstat_tlimot_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_MLIEVB_NUM > 0

static const u16 mca_can_chrid2ofs_mlievb_stat_v[] =
{
	MLIEVB_DRV_RA_status_reg_OFFSET,
	MLIEVB_DRV_RA_guswcnt_OFFSET,
};

#define mca_can_chrid2ofs_mlievb_stat_n (sizeof(mca_can_chrid2ofs_mlievb_stat_v)/sizeof(mca_can_chrid2ofs_mlievb_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_mlievb_stat_n];
} mca_can_xstat_mlievb_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_MOTCON2_NUM > 0

static const u16 mca_can_chrid2ofs_motcon2_stat_v[] =
{
	MOTCON2_DRV_RA_status_reg_OFFSET,
	MOTCON2_DRV_RA_guswcnt_0_OFFSET,
	MOTCON2_DRV_RA_guswcnt_1_OFFSET,
	MOTCON2_DRV_RA_oc_temp_val_OFFSET,
};

#define mca_can_chrid2ofs_motcon2_stat_n (sizeof(mca_can_chrid2ofs_motcon2_stat_v)/sizeof(mca_can_chrid2ofs_motcon2_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_motcon2_stat_n];
} mca_can_xstat_motcon2_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_MOTCON1_NUM > 0

static const u16 mca_can_chrid2ofs_motcon1_stat_v[] =
{
	MOTCON1_DRV_RA_status_reg_OFFSET,
	MOTCON1_DRV_RA_guswcnt_OFFSET,
	MOTCON1_DRV_RA_oc_temp_val_OFFSET,
};

#define mca_can_chrid2ofs_motcon1_stat_n (sizeof(mca_can_chrid2ofs_motcon1_stat_v)/sizeof(mca_can_chrid2ofs_motcon1_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_motcon1_stat_n];
} mca_can_xstat_motcon1_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_SGXBCU_NUM > 0

static const u16 mca_can_chrid2ofs_sgxbcu_stat_v[] =
{
	SGXBCU_DRV_RA_status_reg_OFFSET,
	SGXBCU_DRV_RA_guswcnt_OFFSET,
};

#define mca_can_chrid2ofs_sgxbcu_stat_n (sizeof(mca_can_chrid2ofs_sgxbcu_stat_v)/sizeof(mca_can_chrid2ofs_sgxbcu_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_sgxbcu_stat_n];
} mca_can_xstat_sgxbcu_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_FMCMC1_NUM > 0

static const u16 mca_can_chrid2ofs_fmcmc1_stat_v[] =
{
	FMCMC1_DRV_RA_status_reg_OFFSET,
	FMCMC1_DRV_RA_oc_temp_val_OFFSET,
	FMCMC1_DRV_RA_xadc_rctreg_OFFSET,
};

#define mca_can_chrid2ofs_fmcmc1_stat_n (sizeof(mca_can_chrid2ofs_fmcmc1_stat_v)/sizeof(mca_can_chrid2ofs_fmcmc1_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_fmcmc1_stat_n];
} mca_can_xstat_fmcmc1_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_NETMOT_NUM > 0

static const u16 mca_can_chrid2ofs_netmot_stat_v[] =
{
	NETMOT_DRV_RA_status_OFFSET,
};

#define mca_can_chrid2ofs_netmot_stat_n (sizeof(mca_can_chrid2ofs_netmot_stat_v)/sizeof(mca_can_chrid2ofs_netmot_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_netmot_stat_n];
} mca_can_xstat_netmot_t;

#endif

// ---------------------------------------------------------------------------

#if SYS_MOTORFOC_NUM > 0

static const u16 mca_can_chrid2ofs_motorfoc_stat_v[] =
{ 
	MOTORFOC_RA_tsclocks_OFFSET,
	MOTORFOC_RA_motstatus_OFFSET,
	MOTORFOC_RA_val_iphs_a_OFFSET,
	MOTORFOC_RA_val_iphs_b_OFFSET,
	MOTORFOC_RA_val_iphs_c_OFFSET,
	MOTORFOC_RA_val_ibus_x_OFFSET,
	MOTORFOC_RA_val_vphs_a_OFFSET,
	MOTORFOC_RA_val_vphs_b_OFFSET,
	MOTORFOC_RA_val_vphs_c_OFFSET,
	MOTORFOC_RA_val_vbus_x_OFFSET,
	MOTORFOC_RA_val_vphs_n_OFFSET,
	MOTORFOC_RA_ismodval_OFFSET,
	MOTORFOC_RA_isangval_OFFSET,
	MOTORFOC_RA_vsmodval_OFFSET,
	MOTORFOC_RA_vsangval_OFFSET,
	MOTORFOC_RA_rot_angle_OFFSET,
	MOTORFOC_RA_spd_speed_OFFSET,
	MOTORFOC_RA_smo_es2_kflt_OFFSET,
	MOTORFOC_RA_smo_bemf_p_OFFSET,
	MOTORFOC_RA_smo_bemf_m_OFFSET,
	MOTORFOC_RA_smo_angle_OFFSET,
	MOTORFOC_RA_pos_position_OFFSET,
	MOTORFOC_RA_ext_angle_OFFSET,
	MOTORFOC_RA_rsv_angle_OFFSET,
	MOTORFOC_RA_hls_angle_OFFSET,
	MOTORFOC_RA_enc_index_OFFSET,
	MOTORFOC_RA_enc_phase_OFFSET,
	MOTORFOC_RA_enc_phcpt_OFFSET,
	MOTORFOC_RA_enc_angle_OFFSET,
	MOTORFOC_RA_mrt_spdout_OFFSET,
	MOTORFOC_RA_mrt_angle_OFFSET,
	MOTORFOC_RA_pwm_mdval_OFFSET,
	MOTORFOC_RA_guswcnt_OFFSET,
};

#define mca_can_chrid2ofs_motorfoc_stat_n (sizeof(mca_can_chrid2ofs_motorfoc_stat_v)/sizeof(mca_can_chrid2ofs_motorfoc_stat_v[0]))

typedef struct
{
	u32 data[mca_can_chrid2ofs_motorfoc_stat_n];
} mca_can_xstat_motorfoc_t;

#endif

// ---------------------------------------------------------------------------

typedef struct
{
	u8  dev_index;
	u8  instance_id;
	u8  field_id;
	u32 field_data;
	u32 field_address;

} mca_can_acv_t;

// ---------------------------------------------------------------------------

typedef struct
{
	u8               task_status;

	u32              timer_tic_per_sec;
	u32              timer_tic_per_msec;
	
	u32              timer_tic_old_msec;
	u32              timer_msec;
	
	u32              twdg;
	u16              linktmo;
	
	#ifdef SW_SELF_RESTART_AGENT
	u32              timer_self_restart;
	#endif

	u8               can_crash;
	u8               sys_up;

	xx_xcan_frame_t  RxFrame;
	xx_xcan_frame_t  TxFrame;

	// running level
	u8               runlevel;
	
	struct
	{
		u32 tls;
		u32 timersend;
	} xgensnd;

	#if SYS_DS1820_NUM > 0
	struct
	{
		u32 tls;
		u32 timersend;
	} xds1820;
	#endif

	#if SYS_QXADCPS_NUM > 0
	struct
	{
		u32 tls;
		u32 timersend;
	} xqxadcps;
	#endif

	// send device IP map context
	struct
	{
		mca_can_acv_t acv;
		u8           state;
	} xmap;

	// send device IP status context
	struct
	{
		mca_can_acv_t acv;
		u32           loops;

		union
		{
			#if SYS_MPHY18_NUM > 0
			mca_can_xstat_mphy18_t   mphy18;
			#endif
			
			#if SYS_MPHY17_NUM > 0
			mca_can_xstat_mphy17_t   mphy17;
			#endif
			
			#if SYS_MPHY16_NUM > 0
			mca_can_xstat_mphy16_t   mphy16;
			#endif
			
			#if SYS_TEP2_NUM > 0
			mca_can_xstat_tep2_t     tep2;
			#endif

			#if SYS_TEC53_NUM > 0
			mca_can_xstat_tec53_t     tec53;
			#endif

			#if SYS_S4SICINV_NUM > 0
			mca_can_xstat_s4sicinv_t  s4sicinv;
			#endif

			#if SYS_QINVSIC_NUM > 0
			mca_can_xstat_qinvsic_t  qinvsic;
			#endif

			#if SYS_MPHY11_NUM > 0
			mca_can_xstat_mphy11_t   mphy11;
			#endif

			#if SYS_ZYGUSIC_NUM > 0
			mca_can_xstat_zygusic_t  zygusic;
			#endif
			
			#if SYS_TLIMOT_NUM > 0
			mca_can_xstat_tlimot_t   tlimot;
			#endif
			
			#if SYS_MLIEVB_NUM > 0
			mca_can_xstat_mlievb_t   mlievb;
			#endif
			
			#if SYS_MOTCON2_NUM > 0
			mca_can_xstat_motcon2_t  motcon2;
			#endif

			#if SYS_MOTCON1_NUM > 0
			mca_can_xstat_motcon1_t  motcon1;
			#endif
			
			#if SYS_SGXBCU_NUM > 0
			mca_can_xstat_sgxbcu_t   sgxbcu;
			#endif

			#if SYS_FMCMC1_NUM > 0
			mca_can_xstat_fmcmc1_t   fmcmc1;
			#endif

			#if SYS_NETMOT_NUM > 0
			mca_can_xstat_netmot_t   netmot;
			#endif
			
			#if SYS_MOTORFOC_NUM > 0
			mca_can_xstat_motorfoc_t motorfoc;
			#endif

		} park;


	} xstat;

	struct
	{
		u32  address;
		u32  data;
		u8   dosend;
	} xreadl;


} mca_can_env_t;


static mca_can_env_t mca_can_env_v;

#define MCA_CAN_V_DEF mca_can_env_t * mca_can_v_ref = & mca_can_env_v
#define MCA_CAN_V_REF mca_can_v_ref

// ---------------------------------------------------------------------------

#define mca_can_tsnd_ready(process)  ((u32)(xx_timer_cnt_val() - MCA_CAN_V_REF->process.tls) >= MCA_CAN_V_REF->process.timersend)
#define mca_can_tsnd_stamp(process)  { MCA_CAN_V_REF->process.tls = xx_timer_cnt_val(); }

// ---------------------------------------------------------------------------

void mca_can_dump_frame(xx_xcan_frame_t *pf)
{
	printf("id=x%8lX srr=%d ide=%d rtr=%d dlc=%d tst=%5d: ",pf->id,pf->srr,pf->ide,pf->rtr,pf->dlc,pf->tst);
	int x;
	for(x = 0; x < pf->dlc; x++)
	{
		printf("%02X ",pf->raw_b[XX_XCAN_PAYLOAD_IDX_BX(x)]);
	}
	printf("\n");
}

// ---------------------------------------------------------------------------

enum
{
	XCAN_XMAP_STATE_IDLE   = 0,
	XCAN_XMAP_STATE_START  = 1,
	XCAN_XMAP_STATE_DATA   = 2,
	XCAN_XMAP_STATE_FINISH = 3,
};

// ---------------------------------------------------------------------------

static void mca_can_acv_rewind(mca_can_acv_t *p)
{
	p->dev_index   = 0;
	p->instance_id = 0;
	p->field_id    = 0;
}

static int mca_can_acv_ip_type_chk_eof(mca_can_acv_t *p)
{
	return (p->dev_index < mca_can_ip_map_n) ? XST_SUCCESS : XST_FAILURE;
}

static int mca_can_acv_ip_instance_chk_eof(mca_can_acv_t *p)
{
	if (XST_SUCCESS != mca_can_acv_ip_type_chk_eof(p))
		return XST_FAILURE;

	return (p->instance_id < mca_can_ip_map_v[p->dev_index].instances) ? XST_SUCCESS : XST_FAILURE;
}

static int mca_can_acv_ip_type_next(mca_can_acv_t *p)
{
	if (XST_SUCCESS != mca_can_acv_ip_type_chk_eof(p))
		return XST_FAILURE;

	p->dev_index++;
	p->instance_id = 0;
	p->field_id    = 0;

	return mca_can_acv_ip_type_chk_eof(p);
}

static int mca_can_acv_ip_instance_next(mca_can_acv_t *p)
{
	if (XST_SUCCESS != mca_can_acv_ip_instance_chk_eof(p))
		return XST_FAILURE;

	p->instance_id++;
	p->field_id = 0;

	return mca_can_acv_ip_instance_chk_eof(p);
}

// ---------------------------------------------------------------------------
// ---------------------------------------------------------------------------
// ---------------------------------------------------------------------------

static void mca_can_xmap_rewind(void)
{
	MCA_CAN_V_DEF;
	mca_can_acv_rewind(&MCA_CAN_V_REF->xmap.acv);
}

static int mca_can_xmap_ip_instance_chk_eof(void)
{
	MCA_CAN_V_DEF;

	return mca_can_acv_ip_instance_chk_eof(&MCA_CAN_V_REF->xmap.acv);
}

static int mca_can_xmap_ip_type_next(void)
{
	MCA_CAN_V_DEF;

	return mca_can_acv_ip_type_next(&MCA_CAN_V_REF->xmap.acv);
}

static int mca_can_xmap_ip_instance_next(void)
{
	MCA_CAN_V_DEF;

	return mca_can_acv_ip_instance_next(&MCA_CAN_V_REF->xmap.acv);
}

static int mca_can_xmap_data_load(void)
{
	MCA_CAN_V_DEF;
	mca_can_acv_t *pQ = &MCA_CAN_V_REF->xmap.acv;

	if (XST_SUCCESS != mca_can_xmap_ip_instance_chk_eof())
		return XST_FAILURE;

	switch(mca_can_ip_map_v[pQ->dev_index].ip_ident)
	{
		default:
			{
				return XST_FAILURE;
			}
			break;

		case SYSTEM_ip_ident_code_value:
			{
				char carrier_name[32];
				memset(carrier_name,0,32);
				strcpy(carrier_name,SYS_CARRIER_NAME);
				switch(pQ->field_id)
				{
					case XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_0  : memcpy(&pQ->field_data,&carrier_name[0],4); break;
					case XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_1  : memcpy(&pQ->field_data,&carrier_name[4],4); break;
					case XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_2  : memcpy(&pQ->field_data,&carrier_name[8],4); break;
					case XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_3  : memcpy(&pQ->field_data,&carrier_name[12],4); break;
					case XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_4  : memcpy(&pQ->field_data,&carrier_name[16],4); break;
					case XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_5  : memcpy(&pQ->field_data,&carrier_name[20],4); break;
					case XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_6  : memcpy(&pQ->field_data,&carrier_name[24],4); break;
					case XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_7  : memcpy(&pQ->field_data,&carrier_name[28],4); break;
					case XCAN_X2X_PARAM_ID_SYSTEM_PROGRAM_VERSION : pQ->field_data = PROGRAM_VERS_NUM; break;
					case XCAN_X2X_PARAM_ID_SYSTEM_SYSTEM_CLOCK    : pQ->field_data = XPAR_FPGA_CLOCK_FREQ_HZ; break;
					default                                       : return XST_FAILURE;
				}
			}
			break;
			
			#if SYS_MPHY18_NUM > 0
			case MPHY18_DRV_ip_ident_code_value:
				{
					mphy18_drv_Config *CfgPtr = mphy18_drv_LookupConfig(pQ->instance_id);
					switch(pQ->field_id)
					{
						case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId         ; break;
						case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier          ; break;
						case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
						case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
						case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = CfgPtr->rslv_exc_dvclk   ; break;
						case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = CfgPtr->rslv_exc_dvclk   ; break;
						case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = 0                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = 0                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = 0                        ; break;
						default                                          : return XST_FAILURE;
					}
				}
				break;
			#endif
			
			#if SYS_MPHY17_NUM > 0
			case MPHY17_DRV_ip_ident_code_value:
				{
					mphy17_drv_Config *CfgPtr = mphy17_drv_LookupConfig(pQ->instance_id);
					switch(pQ->field_id)
					{
						case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId         ; break;
						case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier          ; break;
						case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
						case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
						case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = 1                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = 1                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = 0                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = 0                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = 0                        ; break;
						default                                          : return XST_FAILURE;
					}
				}
				break;
			#endif

			#if SYS_MPHY16_NUM > 0
			case MPHY16_DRV_ip_ident_code_value:
				{
					mphy16_drv_Config *CfgPtr = mphy16_drv_LookupConfig(pQ->instance_id);
					switch(pQ->field_id)
					{
						case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId         ; break;
						case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier          ; break;
						case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
						case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
						case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = 1                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = 1                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = CfgPtr->fan_pwm_freq     ; break;
						case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = CfgPtr->dout_pwm_freq    ; break;
						case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = 0                        ; break;
						default                                          : return XST_FAILURE;
					}
				}
				break;
			#endif
			
			#if SYS_TEP2_NUM > 0
			case TEP2_DRV_ip_ident_code_value:
				{
					tep2_drv_Config *CfgPtr = tep2_drv_LookupConfig(pQ->instance_id);
					switch(pQ->field_id)
					{
						case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId         ; break;
						case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier          ; break;
						case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
						case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
						case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = 1                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = 1                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = 0                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = 0                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = 0                        ; break;
						default                                          : return XST_FAILURE;
					}
				}
				break;
			#endif
			
			#if SYS_TEC53_NUM > 0
			case TEC53_DRV_ip_ident_code_value:
				{
					tec53_drv_Config *CfgPtr = tec53_drv_LookupConfig(pQ->instance_id);
					switch(pQ->field_id)
					{
						case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId         ; break;
						case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier          ; break;
						case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
						case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
						case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
						case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = 1                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = 1                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = 0                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = 0                        ; break;
						case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = 0                        ; break;
						default                                          : return XST_FAILURE;
					}
				}
				break;
			#endif

		#if SYS_S4SICINV_NUM > 0
		case S4SICINV_DRV_ip_ident_code_value:
			{
				s4sicinv_drv_Config *CfgPtr = s4sicinv_drv_LookupConfig(pQ->instance_id);
				switch(pQ->field_id)
				{
					case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId         ; break;
					case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier          ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = CfgPtr->rslv_exc_dvclk   ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = CfgPtr->rslv_exc_dvclk   ; break;
					case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = CfgPtr->fan_pwm_freq     ; break;
					case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = 0                        ; break;
					case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = 0                        ; break;
					default                                          : return XST_FAILURE;
				}
			}
			break;
		#endif

		#if SYS_QINVSIC_NUM > 0
		case QINVSIC_DRV_ip_ident_code_value:
			{
				qinvsic_drv_Config *CfgPtr = qinvsic_drv_LookupConfig(pQ->instance_id);
				switch(pQ->field_id)
				{
					case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId         ; break;
					case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier          ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = 0                        ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = 0                        ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = CfgPtr->rslv_exc_dvclk   ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = CfgPtr->rslv_exc_dvclk   ; break;
					case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = 0                        ; break;
					case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = CfgPtr->dout_pwm_freq    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = 0                        ; break;
					default                                          : return XST_FAILURE;
				}
			}
			break;
		#endif

		#if SYS_MPHY11_NUM > 0
		case MPHY11_DRV_ip_ident_code_value:
			{
				mphy11_drv_Config *CfgPtr = mphy11_drv_LookupConfig(pQ->instance_id);
				switch(pQ->field_id)
				{
					case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId         ; break;
					case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier          ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = 1                        ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = 1                        ; break;
					case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = CfgPtr->fan_pwm_freq     ; break;
					case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = CfgPtr->dout_pwm_freq    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = 0                        ; break;
					default                                          : return XST_FAILURE;
				}
			}
			break;
		#endif

		#if SYS_ZYGUSIC_NUM > 0
		case ZYGUSIC_DRV_ip_ident_code_value:
			{
				zygusic_drv_Config *CfgPtr = zygusic_drv_LookupConfig(pQ->instance_id);
				switch(pQ->field_id)
				{
					case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId         ; break;
					case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier          ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = CfgPtr->inv_acqtime_ns   ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = CfgPtr->rslv_exc_dvclk   ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = CfgPtr->rslv_exc_dvclk   ; break;
					case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = CfgPtr->fan_pwm_freq     ; break;
					case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = 0                        ; break;
					case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = 0                        ; break;
					default                                          : return XST_FAILURE;
				}
			}
			break;
		#endif

		#if SYS_TLIMOT_NUM > 0
		case TLIMOT_DRV_ip_ident_code_value:
			{
				tlimot_drv_Config *CfgPtr = tlimot_drv_LookupConfig(pQ->instance_id);
				switch(pQ->field_id)
				{
					case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId     ; break;
					case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = 1                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = 1                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = 0                    ; break;
					default                                          : return XST_FAILURE;
				}
			}
			break;
		#endif

		#if SYS_MLIEVB_NUM > 0
		case MLIEVB_DRV_ip_ident_code_value:
			{
				mlievb_drv_Config *CfgPtr = mlievb_drv_LookupConfig(pQ->instance_id);
				switch(pQ->field_id)
				{
					case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId     ; break;
					case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = 1                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = 1                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = 0                    ; break;
					default                                          : return XST_FAILURE;
				}
			}
			break;
		#endif

		#if SYS_MOTCON2_NUM > 0
		case MOTCON2_DRV_ip_ident_code_value:
			{
				motcon2_drv_Config *CfgPtr = motcon2_drv_LookupConfig(pQ->instance_id);
				switch(pQ->field_id)
				{
					case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId     ; break;
					case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref_0; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref_1; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = 1                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = 1                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = CfgPtr->xadc         ; break;
					default                                          : return XST_FAILURE;
				}
			}
			break;
		#endif

		#if SYS_MOTCON1_NUM > 0
		case MOTCON1_DRV_ip_ident_code_value:
			{
				motcon1_drv_Config *CfgPtr = motcon1_drv_LookupConfig(pQ->instance_id);
				switch(pQ->field_id)
				{
					case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId     ; break;
					case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = 1                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = 1                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = CfgPtr->xadc         ; break;
					default                                          : return XST_FAILURE;
				}
			}
			break;
		#endif

		#if SYS_SGXBCU_NUM > 0
		case SGXBCU_DRV_ip_ident_code_value:
			{
				sgxbcu_drv_Config *CfgPtr = sgxbcu_drv_LookupConfig(pQ->instance_id);
				switch(pQ->field_id)
				{
					case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId     ; break;
					case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = 1                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = 1                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = 0                    ; break;
					default                                          : return XST_FAILURE;
				}
			}
			break;
		#endif

		#if SYS_FMCMC1_NUM > 0
		case FMCMC1_DRV_ip_ident_code_value:
			{
				fmcmc1_drv_Config *CfgPtr = fmcmc1_drv_LookupConfig(pQ->instance_id);
				switch(pQ->field_id)
				{
					case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId     ; break;
					case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref_0; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref_1; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = 1                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = 1                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = CfgPtr->xadc         ; break;
					default                                          : return XST_FAILURE;
				}
			}
			break;
		#endif

		#if SYS_NETMOT_NUM > 0
		case NETMOT_DRV_ip_ident_code_value:
			{
				netmot_drv_Config *CfgPtr = netmot_drv_LookupConfig(pQ->instance_id);
				switch(pQ->field_id)
				{
					case XCAN_X2X_PARAM_ID_BOARD_DEVICEID            : pQ->field_data = CfgPtr->DeviceId     ; break;
					case XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         : pQ->field_data = CfgPtr->BaseAddress  ; break;
					case XCAN_X2X_PARAM_ID_BOARD_CARRIER             : pQ->field_data = CfgPtr->carrier      ; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       : pQ->field_data = CfgPtr->hw_link_ref_0; break;
					case XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       : pQ->field_data = CfgPtr->hw_link_ref_1; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    : pQ->field_data = 1                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    : pQ->field_data = 1                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       : pQ->field_data = 0                    ; break;
					case XCAN_X2X_PARAM_ID_BOARD_XADC                : pQ->field_data = 0                    ; break;
					default                                          : return XST_FAILURE;
				}
			}
			break;
		#endif

		#if SYS_MOTORFOC_NUM > 0
		case MOTORFOC_ip_ident_code_value:
			{
				motorfoc_Config *CfgPtr = motorfoc_LookupConfig(pQ->instance_id);
				switch(pQ->field_id)
				{
					case XCAN_X2X_PARAM_ID_MOTORFOC_DEVICEID          : pQ->field_data = CfgPtr->DeviceId         ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_BASEADDRESS       : pQ->field_data = CfgPtr->BaseAddress      ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_HW_LINK_REF       : pQ->field_data = CfgPtr->hw_link_ref      ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_INV_IN_MAP        : pQ->field_data = CfgPtr->inv_in_map       ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_INV_IN_NOT        : pQ->field_data = CfgPtr->inv_in_not       ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_INV_OFSV_MODE     : pQ->field_data = CfgPtr->inv_ofsv_mode    ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_INV_OVER_IPHS     : pQ->field_data = CfgPtr->inv_over_iphs    ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_INV_OVER_IBUS     : pQ->field_data = CfgPtr->inv_over_ibus    ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_INV_FILTER        : pQ->field_data = CfgPtr->inv_filter       ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_CLARKE_NPHS       : pQ->field_data = CfgPtr->clarke_nphs      ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_PWM_MODULATOR     : pQ->field_data = CfgPtr->pwm_modulator    ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_RPFM_MODULATOR    : pQ->field_data = CfgPtr->rpfm_modulator   ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_RPFM_3_LEVEL      : pQ->field_data = CfgPtr->rpfm_3_level     ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_RPFM_TPNC         : pQ->field_data = CfgPtr->rpfm_tpnc        ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_SMO_EVAL          : pQ->field_data = CfgPtr->smo_eval         ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_SPD_EVAL          : pQ->field_data = CfgPtr->spd_eval         ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_SPEED_CTRL        : pQ->field_data = CfgPtr->speed_ctrl       ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_RESOLVER          : pQ->field_data = CfgPtr->resolver         ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_HALLSENSOR        : pQ->field_data = CfgPtr->hallsensor       ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_ENCODER           : pQ->field_data = CfgPtr->encoder          ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_MANROT            : pQ->field_data = CfgPtr->manrot           ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_MRT_ACCMAX_DLN2   : pQ->field_data = CfgPtr->mrt_accmax_dln2  ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_SMO_ZS_DLN2       : pQ->field_data = CfgPtr->smo_zs_dln2      ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_SMO_F2_DLN2       : pQ->field_data = CfgPtr->smo_f2_dln2      ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_SLP_PRO_DLN2      : pQ->field_data = CfgPtr->slp_pro_dln2     ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_SLP_INT_DLN2      : pQ->field_data = CfgPtr->slp_int_dln2     ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_SLP_INDWP_DLN2    : pQ->field_data = CfgPtr->slp_indwp_dln2   ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_SLP_INDWP_KDIV    : pQ->field_data = CfgPtr->slp_indwp_kdiv   ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_ISOVERFLOW_CMAX   : pQ->field_data = CfgPtr->isoverflow_cmax  ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_PI_ERPRO_DLN2     : pQ->field_data = CfgPtr->pi_erpro_dln2    ; break;
					case XCAN_X2X_PARAM_ID_MOTORFOC_PI_ERINT_DLN2     : pQ->field_data = CfgPtr->pi_erint_dln2    ; break;
					default                                           : return XST_FAILURE;
				}
			}
			break;
		#endif

	}

	return XST_SUCCESS;
}

static int mca_can_xmap_data_next(void)
{
	MCA_CAN_V_DEF;

	if (XST_SUCCESS != mca_can_xmap_data_load())
		return XST_FAILURE;

	MCA_CAN_V_REF->xmap.acv.field_id++;

	if (XST_SUCCESS != mca_can_xmap_data_load())
		return XST_FAILURE;

	return XST_SUCCESS;
}

// ---------------------------------------------------------------------------

static void mca_can_application_stop(void)
{
	#if SYS_MOTORFOC_NUM > 0
	u16 id;
	for(id = 0;id < SYS_MOTORFOC_NUM; id++)
		motorfoc_Down(id);
	#endif
}

// ---------------------------------------------------------------------------

static void mca_can_xstat_rewind(void)
{
	MCA_CAN_V_DEF;

	mca_can_acv_rewind(&MCA_CAN_V_REF->xstat.acv);

}

static int mca_can_xstat_ip_instance_chk_eof(void)
{
	MCA_CAN_V_DEF;

	return mca_can_acv_ip_instance_chk_eof(&MCA_CAN_V_REF->xstat.acv);
}


static int mca_can_xstat_ip_type_next(void)
{
	MCA_CAN_V_DEF;

	return mca_can_acv_ip_type_next(&MCA_CAN_V_REF->xstat.acv);
}

static int mca_can_xstat_ip_instance_next(void)
{
	MCA_CAN_V_DEF;

	return mca_can_acv_ip_instance_next(&MCA_CAN_V_REF->xstat.acv);
}


static int mca_can_xstat_data_load(void)
{
	MCA_CAN_V_DEF;
	mca_can_acv_t *pQ = &MCA_CAN_V_REF->xstat.acv;

	if (XST_SUCCESS != mca_can_xstat_ip_instance_chk_eof())
		return XST_FAILURE;

	switch(mca_can_ip_map_v[pQ->dev_index].ip_ident)
	{
		default:
			{
				return XST_FAILURE;
			}
			break;

		case SYSTEM_ip_ident_code_value:
			{
				switch(pQ->field_id)
				{
					case XCAN_X2X_STAT_ID_SYSTEM_TIMER_MSEC :
						{
							pQ->field_address = XCAN_X2X_STAT_ID_SYSTEM_TIMER_MSEC;
							pQ->field_data    = MCA_CAN_V_REF->timer_msec;
						}
						break;
					case XCAN_X2X_STAT_ID_SYSTEM_STATUS_LOOP :
						{
							pQ->field_address = XCAN_X2X_STAT_ID_SYSTEM_STATUS_LOOP;
							pQ->field_data    = MCA_CAN_V_REF->xstat.loops;
						}
						break;
					default                                 : return XST_FAILURE;
				}
			}
			break;

#define MCA_CAN_XSTAT_DATA_LOAD_M(ip_id_code,ipname,ipxname) \
		case (ip_id_code): \
			{ \
				u32 baseaddress = ipxname##_LookupConfig(pQ->instance_id)->BaseAddress; \
				\
				if (0 == pQ->field_id) \
				{ \
					int i; \
					for(i = 0; i < ((int)mca_can_chrid2ofs_##ipname##_stat_n); i++) { \
						MCA_CAN_V_REF->xstat.park.ipname.data[i] = Xil_In32(baseaddress + mca_can_chrid2ofs_##ipname##_stat_v[i]); \
					} \
				}\
				if (pQ->field_id < mca_can_chrid2ofs_##ipname##_stat_n) \
				{ \
					pQ->field_address = baseaddress + mca_can_chrid2ofs_##ipname##_stat_v[pQ->field_id]; \
					pQ->field_data    =             MCA_CAN_V_REF->xstat.park.ipname.data[pQ->field_id]; \
					break; \
				} \
				return XST_FAILURE; \
			} \
			break;


		#if SYS_MPHY18_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(MPHY18_DRV_ip_ident_code_value,mphy18,mphy18_drv)
		#endif
		
		#if SYS_MPHY17_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(MPHY17_DRV_ip_ident_code_value,mphy17,mphy17_drv)
		#endif

		#if SYS_MPHY16_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(MPHY16_DRV_ip_ident_code_value,mphy16,mphy16_drv)
		#endif
		
		#if SYS_TEP2_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(TEP2_DRV_ip_ident_code_value,tep2,tep2_drv)
		#endif

		#if SYS_TEC53_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(TEC53_DRV_ip_ident_code_value,tec53,tec53_drv)
		#endif

		#if SYS_S4SICINV_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(S4SICINV_DRV_ip_ident_code_value,s4sicinv,s4sicinv_drv)
		#endif

		#if SYS_QINVSIC_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(QINVSIC_DRV_ip_ident_code_value,qinvsic,qinvsic_drv)
		#endif

		#if SYS_MPHY11_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(MPHY11_DRV_ip_ident_code_value,mphy11,mphy11_drv)
		#endif

		#if SYS_ZYGUSIC_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(ZYGUSIC_DRV_ip_ident_code_value,zygusic,zygusic_drv)
		#endif
			
		#if SYS_TLIMOT_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(TLIMOT_DRV_ip_ident_code_value,tlimot,tlimot_drv)
		#endif
			
		#if SYS_MLIEVB_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(MLIEVB_DRV_ip_ident_code_value,mlievb,mlievb_drv)
		#endif
			
		#if SYS_MOTCON2_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(MOTCON2_DRV_ip_ident_code_value,motcon2,motcon2_drv)
		#endif

		#if SYS_MOTCON1_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(MOTCON1_DRV_ip_ident_code_value,motcon1,motcon1_drv)
		#endif
			
		#if SYS_SGXBCU_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(SGXBCU_DRV_ip_ident_code_value,sgxbcu,sgxbcu_drv)
		#endif
			
		#if SYS_FMCMC1_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(FMCMC1_DRV_ip_ident_code_value,fmcmc1,fmcmc1_drv)
		#endif

		#if SYS_NETMOT_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(NETMOT_DRV_ip_ident_code_value,netmot,netmot_drv)
		#endif

		#if SYS_MOTORFOC_NUM > 0
		MCA_CAN_XSTAT_DATA_LOAD_M(MOTORFOC_ip_ident_code_value,motorfoc,motorfoc)
		#endif

	}
	return XST_SUCCESS;
}

static int mca_can_xstat_data_next(void)
{
	MCA_CAN_V_DEF;

	if (XST_SUCCESS != mca_can_xstat_data_load())
		return XST_FAILURE;

	MCA_CAN_V_REF->xstat.acv.field_id++;

	if (XST_SUCCESS != mca_can_xstat_data_load())
		return XST_FAILURE;

	return XST_SUCCESS;
}

// ---------------------------------------------------------------------------

static int mca_can_check_access(const u32 address)
{
	// (address % 4) = 0
	if ((address & 3) != 0)
		return XST_FAILURE;

	#define MCA_CAN_CHECKHW_ACCESS_M(instances,ipxname,ipsize,address) { \
		u16 id; \
		for(id = 0;id < (instances); id++) \
		{ \
			ipxname##_Config *CfgPtr = ipxname##_LookupConfig(id); \
				\
			u32 offset = (address) - CfgPtr->BaseAddress; \
			if (offset <= (ipsize)) \
				return XST_SUCCESS; \
		} \
	}

	#if SYS_MPHY18_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_MPHY18_NUM,mphy18_drv,MCA_CAN_BOARD_SIZE,address);
	#endif
	
	#if SYS_MPHY17_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_MPHY17_NUM,mphy17_drv,MCA_CAN_BOARD_SIZE,address);
	#endif

	#if SYS_MPHY16_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_MPHY16_NUM,mphy16_drv,MCA_CAN_BOARD_SIZE,address);
	#endif
	
	#if SYS_TEP2_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_TEP2_NUM,tep2_drv,MCA_CAN_BOARD_SIZE,address);
	#endif

	#if SYS_TEC53_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_TEC53_NUM,tec53_drv,MCA_CAN_BOARD_SIZE,address);
	#endif

	#if SYS_S4SICINV_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_S4SICINV_NUM,s4sicinv_drv,MCA_CAN_BOARD_SIZE,address);
	#endif

	#if SYS_QINVSIC_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_QINVSIC_NUM,qinvsic_drv,MCA_CAN_BOARD_SIZE,address);
	#endif

	#if SYS_MPHY11_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_MPHY11_NUM,mphy11_drv,MCA_CAN_BOARD_SIZE,address);
	#endif

	#if SYS_ZYGUSIC_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_ZYGUSIC_NUM,zygusic_drv,MCA_CAN_BOARD_SIZE,address);
	#endif
	
	#if SYS_TLIMOT_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_TLIMOT_NUM,tlimot_drv,MCA_CAN_BOARD_SIZE,address);
	#endif
	
	#if SYS_MLIEVB_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_MLIEVB_NUM,mlievb_drv,MCA_CAN_BOARD_SIZE,address);
	#endif
	
	#if SYS_MOTCON2_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_MOTCON2_NUM,motcon2_drv,MCA_CAN_BOARD_SIZE,address);
	#endif
	
	#if SYS_MOTCON1_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_MOTCON1_NUM,motcon1_drv,MCA_CAN_BOARD_SIZE,address);
	#endif
	
	#if SYS_SGXBCU_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_SGXBCU_NUM,sgxbcu_drv,MCA_CAN_BOARD_SIZE,address);
	#endif
	
	#if SYS_FMCMC1_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_FMCMC1_NUM,fmcmc1_drv,MCA_CAN_BOARD_SIZE,address);
	#endif

	#if SYS_NETMOT_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_NETMOT_NUM,netmot_drv,MCA_CAN_BOARD_SIZE,address);
	#endif

	#if SYS_MOTORFOC_NUM > 0
	MCA_CAN_CHECKHW_ACCESS_M(SYS_MOTORFOC_NUM,motorfoc,MCA_CAN_MOTOR_SIZE,address);
	#endif

	return XST_FAILURE;
}

// ---------------------------------------------------------------------------


static void mca_can_recv_process_m2a_sysf_set_runlevel(const u8 runlevel)
{
	MCA_CAN_V_DEF;

	if (MCA_CAN_V_REF->runlevel != runlevel)
	{
		switch(runlevel)
		{
			default:
				{
				}
				break;
			case XCAN_X2X_RUNLEVEL_STOP:
				{
					mca_can_application_stop();
				}
				break;
			case XCAN_X2X_RUNLEVEL_FULL:
				{
					mca_can_xstat_rewind();
				}
				break;
		}

		MCA_CAN_V_REF->runlevel = runlevel;
	}
}

// - system functions

static int mca_can_recv_process_m2a_sysf(void)
{
	MCA_CAN_V_DEF;

	if (MCA_CAN_V_REF->RxFrame.dlc >= 1)
	{
		switch(MCA_CAN_V_REF->RxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(0)])
		{
			default:
				{
				}
				break;

			case XCAN_M2A_CODE_RUNLEVEL:
				{
					if (MCA_CAN_V_REF->RxFrame.dlc >= 2)
					{
						u8 t8 = MCA_CAN_V_REF->RxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(1)];
						mca_can_recv_process_m2a_sysf_set_runlevel(t8);
					}
				}
				break;

			case XCAN_M2A_CODE_REQ4MAP:
				{
					MCA_CAN_V_REF->xmap.state = XCAN_XMAP_STATE_START;
				}
				break;

			case XCAN_M2A_CODE_TMGSETUP:
				{
					if (MCA_CAN_V_REF->RxFrame.dlc >= 8)
					{
						u8  heartbeat;
						u16 linktmo,sndgenus,sndacqus;

						heartbeat = MCA_CAN_V_REF->RxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(1)];

						linktmo   = MCA_CAN_V_REF->RxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(2)] << 8;
						linktmo  |= MCA_CAN_V_REF->RxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(3)];

						sndgenus  = MCA_CAN_V_REF->RxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(4)] << 8;
						sndgenus |= MCA_CAN_V_REF->RxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(5)];

						sndacqus  = MCA_CAN_V_REF->RxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(6)] << 8;
						sndacqus |= MCA_CAN_V_REF->RxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(7)];

						if (heartbeat)
							MCA_CAN_V_REF->twdg = MCA_CAN_V_REF->timer_msec;

						MCA_CAN_V_REF->linktmo = linktmo;
						MCA_CAN_V_REF->xgensnd.timersend  = ((MCA_CAN_V_REF->timer_tic_per_sec / 2000) * sndgenus) / 500;
					}
				}
				break;

			case XCAN_M2A_CODE_TERMINATE:
				{
					MCA_CAN_V_REF->task_status = MCA_CAN_TASK_STATUS_TERMINATE;
				}
				break;

			case XCAN_M2A_CODE_RESTART:
				{
					MCA_CAN_V_REF->task_status = MCA_CAN_TASK_STATUS_RESTART;
				}
				break;


		}
	}
	return XST_SUCCESS;
}

static int mca_can_recv_process_m2a_readl(void)
{
	MCA_CAN_V_DEF;

	if (MCA_CAN_V_REF->RxFrame.dlc >= 4)
	{
		u32 address = XX_XCAN_PAYLOAD_SWAP_U32(MCA_CAN_V_REF->RxFrame.raw_l[XX_XCAN_PAYLOAD_IDX_LX(0)]);
		//printf("request read @ %08lX\n",address);

		if (XST_SUCCESS == mca_can_check_access(address))
		{
			MCA_CAN_V_REF->xreadl.address = address;
			MCA_CAN_V_REF->xreadl.data    = Xil_In32(address);

			MCA_CAN_V_REF->xreadl.dosend  = 1;
		}
	}
	return XST_SUCCESS;

}

static int mca_can_recv_process_m2a_writel(void)
{
	MCA_CAN_V_DEF;

	if (MCA_CAN_V_REF->RxFrame.dlc >= 8)
	{
		u32 address = XX_XCAN_PAYLOAD_SWAP_U32(MCA_CAN_V_REF->RxFrame.raw_l[XX_XCAN_PAYLOAD_IDX_LX(0)]);
		u32 datal   = XX_XCAN_PAYLOAD_SWAP_U32(MCA_CAN_V_REF->RxFrame.raw_l[XX_XCAN_PAYLOAD_IDX_LX(1)]);
		//printf("request write @ %08lX x%08lX\n",address,datal);

		if (XST_SUCCESS == mca_can_check_access(address))
		{
			Xil_Out32(address,datal);
		}
	}
	return XST_SUCCESS;

}

// send status in round robin
static int mca_can_application_send_status(void)
{
	MCA_CAN_V_DEF;
	int rc = XST_SUCCESS;

	if (XST_SUCCESS != mca_can_xstat_data_load())
	{
		if (XST_SUCCESS != mca_can_xstat_ip_instance_next())
		{
			if (XST_SUCCESS != mca_can_xstat_ip_type_next())
			{
				MCA_CAN_V_REF->xstat.loops += 1;

				mca_can_xstat_rewind();
			}

		}
	}

	if (XST_SUCCESS != mca_can_xstat_data_load())
	{
		return XST_FAILURE;
	}

	//-
	MCA_CAN_V_REF->TxFrame.id  = XCAN_X2X_STD_PUT_IDENT(g_link_device,XCAN_A2M_STD_FUNC_READL);
	MCA_CAN_V_REF->TxFrame.srr = 0;
	MCA_CAN_V_REF->TxFrame.ide = 0;
	MCA_CAN_V_REF->TxFrame.rtr = 0;
	MCA_CAN_V_REF->TxFrame.dlc = 8;
	MCA_CAN_V_REF->TxFrame.raw_l[XX_XCAN_PAYLOAD_IDX_LX(0)] = XX_XCAN_PAYLOAD_SWAP_U32(MCA_CAN_V_REF->xstat.acv.field_address);
	MCA_CAN_V_REF->TxFrame.raw_l[XX_XCAN_PAYLOAD_IDX_LX(1)] = XX_XCAN_PAYLOAD_SWAP_U32(MCA_CAN_V_REF->xstat.acv.field_data);
	rc = xx_xcan_send_frame(&MCA_CAN_V_REF->TxFrame);
	mca_can_tsnd_stamp(xgensnd);
	//-
	mca_can_xstat_data_next();
	//-

	return rc;
}


static int mca_can_send_device_map(void)
{
	MCA_CAN_V_DEF;
	int rc = XST_SUCCESS;

	switch(MCA_CAN_V_REF->xmap.state)
	{
		default:
			{
			}
			break;
		case XCAN_XMAP_STATE_IDLE:
			{
			}
			break;
		case XCAN_XMAP_STATE_START:
			{
				MCA_CAN_V_REF->TxFrame.id  = XCAN_X2X_STD_PUT_IDENT(g_link_device,XCAN_A2M_STD_FUNC_SYSF);
				MCA_CAN_V_REF->TxFrame.srr = 0;
				MCA_CAN_V_REF->TxFrame.ide = 0;
				MCA_CAN_V_REF->TxFrame.rtr = 0;
				MCA_CAN_V_REF->TxFrame.dlc = 1;
				MCA_CAN_V_REF->TxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(0)] = XCAN_A2M_CODE_MAPOPEN;
				rc = xx_xcan_send_frame(&MCA_CAN_V_REF->TxFrame);
				mca_can_tsnd_stamp(xgensnd);
				//-
				mca_can_xmap_rewind();
				MCA_CAN_V_REF->xmap.state = XCAN_XMAP_STATE_DATA;
			}
			break;
		case XCAN_XMAP_STATE_DATA:
			{
				if (XST_SUCCESS != (rc = mca_can_xmap_data_load()))
				{
					if (XST_SUCCESS != (rc = mca_can_xmap_ip_instance_next()))
					{
						mca_can_xmap_ip_type_next();
					}
				}
				if (XST_SUCCESS != (rc = mca_can_xmap_data_load()))
				{
					MCA_CAN_V_REF->xmap.state = XCAN_XMAP_STATE_FINISH;
					break;
				}
				MCA_CAN_V_REF->TxFrame.id  = XCAN_X2X_STD_PUT_IDENT(g_link_device,XCAN_A2M_STD_FUNC_SYSF);
				MCA_CAN_V_REF->TxFrame.srr = 0;
				MCA_CAN_V_REF->TxFrame.ide = 0;
				MCA_CAN_V_REF->TxFrame.rtr = 0;
				MCA_CAN_V_REF->TxFrame.dlc = 8;
				MCA_CAN_V_REF->TxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(0)] = XCAN_A2M_CODE_MAPDATA;
				MCA_CAN_V_REF->TxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(1)] = mca_can_ip_map_v[MCA_CAN_V_REF->xmap.acv.dev_index].ip_ident;
				MCA_CAN_V_REF->TxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(2)] = MCA_CAN_V_REF->xmap.acv.instance_id;
				MCA_CAN_V_REF->TxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(3)] = MCA_CAN_V_REF->xmap.acv.field_id;
				MCA_CAN_V_REF->TxFrame.raw_l[XX_XCAN_PAYLOAD_IDX_LX(1)] = XX_XCAN_PAYLOAD_SWAP_U32(MCA_CAN_V_REF->xmap.acv.field_data);
				rc = xx_xcan_send_frame(&MCA_CAN_V_REF->TxFrame);
				mca_can_tsnd_stamp(xgensnd);
				//-
				mca_can_xmap_data_next();
			}
			break;
		case XCAN_XMAP_STATE_FINISH:
			{
				MCA_CAN_V_REF->TxFrame.id  = XCAN_X2X_STD_PUT_IDENT(g_link_device,XCAN_A2M_STD_FUNC_SYSF);
				MCA_CAN_V_REF->TxFrame.srr = 0;
				MCA_CAN_V_REF->TxFrame.ide = 0;
				MCA_CAN_V_REF->TxFrame.rtr = 0;
				MCA_CAN_V_REF->TxFrame.dlc = 1;
				MCA_CAN_V_REF->TxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(0)] = XCAN_A2M_CODE_MAPCLOSE;
				rc = xx_xcan_send_frame(&MCA_CAN_V_REF->TxFrame);
				mca_can_tsnd_stamp(xgensnd);
				//-
				MCA_CAN_V_REF->xmap.state = XCAN_XMAP_STATE_IDLE;
			}
			break;
	}

	return rc;
}

static int mca_can_send_single_word(void)
{
	MCA_CAN_V_DEF;
	int rc = XST_SUCCESS;

	if (MCA_CAN_V_REF->xreadl.dosend)
	{
		//-
		MCA_CAN_V_REF->TxFrame.id  = XCAN_X2X_STD_PUT_IDENT(g_link_device,XCAN_A2M_STD_FUNC_READL);
		MCA_CAN_V_REF->TxFrame.srr = 0;
		MCA_CAN_V_REF->TxFrame.ide = 0;
		MCA_CAN_V_REF->TxFrame.rtr = 0;
		MCA_CAN_V_REF->TxFrame.dlc = 8;
		MCA_CAN_V_REF->TxFrame.raw_l[XX_XCAN_PAYLOAD_IDX_LX(0)] = XX_XCAN_PAYLOAD_SWAP_U32(MCA_CAN_V_REF->xreadl.address);
		MCA_CAN_V_REF->TxFrame.raw_l[XX_XCAN_PAYLOAD_IDX_LX(1)] = XX_XCAN_PAYLOAD_SWAP_U32(MCA_CAN_V_REF->xreadl.data);
		rc = xx_xcan_send_frame(&MCA_CAN_V_REF->TxFrame);
		mca_can_tsnd_stamp(xgensnd);
		//-
		MCA_CAN_V_REF->xreadl.dosend = 0;
	}

	return rc;
}


#if SYS_DS1820_NUM > 0
static int mca_can_application_send_ds1820(void)
{
	MCA_CAN_V_DEF;
	int rc = XST_SUCCESS;

	u8 idx;
	union { u32 u; float r; } temp;
	if (XST_SUCCESS == ds1820_ape_get_temp(&idx,&temp.r))
	{
		//-
		MCA_CAN_V_REF->TxFrame.id  = XCAN_X2X_STD_PUT_IDENT(g_link_device,XCAN_A2M_STD_FUNC_SYSF);
		MCA_CAN_V_REF->TxFrame.srr = 0;
		MCA_CAN_V_REF->TxFrame.ide = 0;
		MCA_CAN_V_REF->TxFrame.rtr = 0;
		MCA_CAN_V_REF->TxFrame.dlc = 8;
		MCA_CAN_V_REF->TxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(0)] = XCAN_M2A_CODE_DS1820;
		MCA_CAN_V_REF->TxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(1)] = idx;
		MCA_CAN_V_REF->TxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(2)] = 0;
		MCA_CAN_V_REF->TxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(3)] = 0;
		MCA_CAN_V_REF->TxFrame.raw_l[XX_XCAN_PAYLOAD_IDX_LX(1)] = XX_XCAN_PAYLOAD_SWAP_U32(temp.u);
		rc = xx_xcan_send_frame(&MCA_CAN_V_REF->TxFrame);
		mca_can_tsnd_stamp(xds1820);
		//-

	}

	return rc;
}
#endif

#if SYS_QXADCPS_NUM > 0
static int mca_can_application_send_qxadcps(void)
{
	MCA_CAN_V_DEF;
	int rc = XST_SUCCESS;

	union { u32 u; float r; } temp;
	temp.r = g_qxadcps_oc_temp[0];
	//-
	MCA_CAN_V_REF->TxFrame.id  = XCAN_X2X_STD_PUT_IDENT(g_link_device,XCAN_A2M_STD_FUNC_SYSF);
	MCA_CAN_V_REF->TxFrame.srr = 0;
	MCA_CAN_V_REF->TxFrame.ide = 0;
	MCA_CAN_V_REF->TxFrame.rtr = 0;
	MCA_CAN_V_REF->TxFrame.dlc = 8;
	MCA_CAN_V_REF->TxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(0)] = XCAN_M2A_CODE_QXADCPS;
	MCA_CAN_V_REF->TxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(1)] = 0;
	MCA_CAN_V_REF->TxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(2)] = 0;
	MCA_CAN_V_REF->TxFrame.raw_b[XX_XCAN_PAYLOAD_IDX_BX(3)] = 0;
	MCA_CAN_V_REF->TxFrame.raw_l[XX_XCAN_PAYLOAD_IDX_LX(1)] = XX_XCAN_PAYLOAD_SWAP_U32(temp.u);
	rc = xx_xcan_send_frame(&MCA_CAN_V_REF->TxFrame);
	mca_can_tsnd_stamp(xqxadcps);
	//-

	return rc;
}
#endif


// recv process

static int mca_can_recv_process(void)
{
	MCA_CAN_V_DEF;

	if (MCA_CAN_V_REF->RxFrame.ide)
	{
		u8 device = XCAN_X2X_EXT_GET_DEVICE(MCA_CAN_V_REF->RxFrame.id);
		if (device != g_link_device) return 0;

		u32 code  = XCAN_X2X_EXT_GET_IDENT(MCA_CAN_V_REF->RxFrame.id);

		//printf("RX EXT : code -> x%lX\n",code);
		switch(code)
		{
			default:
				{
				}
				break;
		}
	}
	else
	{
		u8 device = XCAN_X2X_STD_GET_DEVICE(MCA_CAN_V_REF->RxFrame.id);
		if (device != g_link_device) return 0;

		u32 code  = XCAN_X2X_STD_GET_IDENT(MCA_CAN_V_REF->RxFrame.id);

		//printf("RX STD : code -> x%lX\n",code);
		switch(code)
		{
			case XCAN_M2A_STD_FUNC_SYSF:
				{
					mca_can_recv_process_m2a_sysf();
				}
				break;

			case XCAN_M2A_STD_FUNC_READL:
				{
					mca_can_recv_process_m2a_readl();
				}
				break;

			case XCAN_M2A_STD_FUNC_WRITEL:
				{
					mca_can_recv_process_m2a_writel();
				}
				break;

			default:
				{
				}
				break;
		}
	}

	return 0;
}


// send process

static int mca_can_send_process(void)
{
	MCA_CAN_V_DEF;
	int rc = XST_SUCCESS;

	rc = xx_xcan_send_full();
	if (rc != XST_SUCCESS) return rc;

	if (mca_can_tsnd_ready(xgensnd))
	{
		mca_can_send_single_word();
	}


	if (mca_can_tsnd_ready(xgensnd))
	{
		mca_can_send_device_map();
	}

	if (MCA_CAN_V_REF->runlevel == XCAN_X2X_RUNLEVEL_FULL)
	{
		if (mca_can_tsnd_ready(xgensnd))
		{
			mca_can_application_send_status();
		}

		#if SYS_DS1820_NUM > 0
		if (mca_can_tsnd_ready(xds1820))
		{
			mca_can_application_send_ds1820();
		}
		#endif

		#if SYS_QXADCPS_NUM > 0
		if (mca_can_tsnd_ready(xqxadcps))
		{
			mca_can_application_send_qxadcps();
		}
		#endif
	}

	return rc;
}

static void mca_can_runtime_checks(void)
{
	MCA_CAN_V_DEF;

	// watch dog can
	{
		u32 d = MCA_CAN_V_REF->timer_msec - MCA_CAN_V_REF->twdg;
		if (d >= MCA_CAN_V_REF->linktmo)
		{
			MCA_CAN_V_REF->twdg = MCA_CAN_V_REF->timer_msec - MCA_CAN_V_REF->linktmo;

			MCA_CAN_V_REF->can_crash = 1;

			if (MCA_CAN_V_REF->sys_up)
			{
				MCA_CAN_V_REF->sys_up = 0;
				motbrddv_system_down();
				print("0");
			}
		}
		else
		{
			MCA_CAN_V_REF->can_crash = 0;

			if (!MCA_CAN_V_REF->sys_up)
			{
				MCA_CAN_V_REF->sys_up = 1;
				motbrddv_system_enable();
				print("1");
			}
		}
	}

}

// counts milliseconds
static void mca_can_timer_process(void)
{
	MCA_CAN_V_DEF;

	for(;;)
	{
		u32 d = xx_timer_cnt_val() - MCA_CAN_V_REF->timer_tic_old_msec;
		if (d < MCA_CAN_V_REF->timer_tic_per_msec)
			break;
		MCA_CAN_V_REF->timer_tic_old_msec += MCA_CAN_V_REF->timer_tic_per_msec;
		MCA_CAN_V_REF->timer_msec += 1;

		// live dot
		if (0 == (MCA_CAN_V_REF->timer_msec % 1000)) print("^");
	}
}

static void mca_can_init(void)
{
	MCA_CAN_V_DEF;

	MCA_CAN_V_REF->task_status        = MCA_CAN_TASK_STATUS_RUNNING;

	MCA_CAN_V_REF->timer_tic_per_sec  = xx_timer_cnt_pser_sec();
	MCA_CAN_V_REF->timer_tic_per_msec = MCA_CAN_V_REF->timer_tic_per_sec / 1000;

	MCA_CAN_V_REF->timer_tic_old_msec = xx_timer_cnt_val();
	MCA_CAN_V_REF->timer_msec         = 0;

	MCA_CAN_V_REF->linktmo            = 1000;
	MCA_CAN_V_REF->twdg               = MCA_CAN_V_REF->timer_msec - MCA_CAN_V_REF->linktmo;

	MCA_CAN_V_REF->can_crash          = 1;
	MCA_CAN_V_REF->sys_up             = 0;

	//-

	MCA_CAN_V_REF->runlevel           = XCAN_X2X_RUNLEVEL_STOP;

	MCA_CAN_V_REF->xstat.loops        = 0;
	MCA_CAN_V_REF->xmap.state         = XCAN_XMAP_STATE_IDLE;
	MCA_CAN_V_REF->xreadl.dosend      = 0;
	
	MCA_CAN_V_REF->xgensnd.timersend  = MCA_CAN_V_REF->timer_tic_per_sec / 1000;
	MCA_CAN_V_REF->xgensnd.tls        = xx_timer_cnt_val();

	#if SYS_DS1820_NUM > 0
	MCA_CAN_V_REF->xds1820.timersend  = MCA_CAN_V_REF->timer_tic_per_sec / 1;
	MCA_CAN_V_REF->xds1820.tls        = xx_timer_cnt_val();
	#endif

	#if SYS_QXADCPS_NUM > 0
	MCA_CAN_V_REF->xqxadcps.timersend = MCA_CAN_V_REF->timer_tic_per_sec / 1;
	MCA_CAN_V_REF->xqxadcps.tls       = xx_timer_cnt_val();
	#endif

	#ifdef SW_SELF_RESTART_AGENT
	MCA_CAN_V_REF->timer_self_restart = MCA_CAN_V_REF->timer_msec;
	#endif

}

int mca_can_agent(void)
{
	MCA_CAN_V_DEF;

	u32 loop;
	int rc;

	print("\r\nStart X-CAN CTL agent\r\n");
	
	// off motors
	motbrddv_system_down();

	// init env
	mca_can_init();

	// Initialize CAN controller
	xx_xcan_init();

	// agent loop
	for(loop = 0;MCA_CAN_V_REF->task_status == MCA_CAN_TASK_STATUS_RUNNING;loop++)
	{

		// msec timer eval
		mca_can_timer_process();

		#if SYS_DS1820_NUM > 0
		ds1820_ape_proc();
		#endif

		#if SYS_QXADCPS_NUM > 0
		qxadcps_ape_proc();
		#endif

		// checks
		mca_can_runtime_checks();

		#ifdef SW_SELF_RESTART_AGENT
		if ((u32)(MCA_CAN_V_REF->timer_msec - MCA_CAN_V_REF->timer_self_restart) > LINKDEVICE_SPAWN_TIME_MS)
			if (g_link_phymode != LINK_PHYMODE_CAN)
				MCA_CAN_V_REF->task_status = MCA_CAN_TASK_STATUS_RESTART;
		#endif

		int key = con_tkey();
		if (27 == key)
			MCA_CAN_V_REF->task_status = MCA_CAN_TASK_STATUS_TERMINATE;

		// peek frame
		rc = xx_xcan_recv_frame(&MCA_CAN_V_REF->RxFrame);
		if (rc == XST_SUCCESS)
		{
			#ifdef SW_SELF_RESTART_AGENT
			MCA_CAN_V_REF->timer_self_restart = MCA_CAN_V_REF->timer_msec;
			#endif

			//mca_can_dump_frame(&MCA_CAN_V_REF->RxFrame);
			mca_can_recv_process();
		}

		mca_can_send_process();

	}

	// off can bus
	xx_xcan_down();

	// off motors
	motbrddv_system_down();

	return XST_SUCCESS;
}

int mca_can_task_status(void)
{
	MCA_CAN_V_DEF;
	return MCA_CAN_V_REF->task_status;
}

#endif
