/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief console utility
/// \file conutil.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include <stdio.h>
#include "xparameters.h"

#if defined(__MICROBLAZE__)
#include "xuartlite_l.h"
#endif

#if defined (__arm__) || defined (__aarch64__)
#include "xuartps.h"
#include "xil_printf.h"
#endif

#include "conutil.h"

int con_tkey(void)
{
	int key = -1;
    
    #if defined (__arm__) || defined (__aarch64__)
	if (XUartPs_IsReceiveData(STDIN_BASEADDRESS))
	{
		key = XUartPs_ReadReg(STDIN_BASEADDRESS, XUARTPS_FIFO_OFFSET) & 0xFF;
	}
    #endif
    
    #if defined(__MICROBLAZE__)
    if (!XUartLite_IsReceiveEmpty(STDIN_BASEADDRESS))
    {
        key = XUartLite_RecvByte(STDIN_BASEADDRESS) & 0xFF;
    }
    #endif
    
	return key;
}

int con_gkey(void)
{
	int key;
	do { key = con_tkey(); } while(key < 0);
	return key;
}

void con_anyk(void)
{
	print(" .... any key to cont..");
	con_gkey();
	print("\r\n");
}

int con_guint(int nchr,int *rval,int hex)
{
	int key;
	int ival;
	char istr[16];
	int i;

	if (nchr < 1 || nchr >= ((int)sizeof(istr)))
	{
		return 27;
	}

	for(i=0;i<nchr;i++)
		istr[i] = '0';
	istr[nchr] = 0;

	for(;;)
	{
		print(istr);
		print("  ");
		key = con_gkey();
		if (key == 27 || key == 13 || key == 3)
			break;
		if ((key >= '0' && key <= '9') || (key >= 'a' && key <= 'f' && hex) || (key >= 'A' && key <= 'F' && hex))
		{
			for(i=1;i<nchr;i++)
				istr[i-1] = istr[i];
			istr[nchr-1] = (char)key;
		}
		for(i=-2;i<nchr;i++)
			print("\b");
	}
	ival = 0;
	for(i=0;i<nchr;i++)
	{
		int ch = istr[i];
		if (hex == 0)
			ival *= 10;
		else
			ival <<= 4;
		if (ch >= '0' && ch <= '9')
			ch = ch - '0' + 0;
		if (ch >= 'A' && ch <= 'F')
			ch = ch - 'A' + 10;
		if (ch >= 'a' && ch <= 'f')
			ch = ch - 'a' + 10;
		ival += ch;
	}
	if (rval)
		* rval = ival;
	return key;
}

void con_dump(const void *start,const int limit,const int notail)
{
	int i,j,nb;
	const unsigned char *pb;
	unsigned char vb;

	if (notail)
	{
		nb = 0;
		pb = start;
		for(i=0;i < limit;i++)
		{
			vb = *pb++;
			if (vb)
				nb = i+1;
		}
	}
	else
	{
		nb = limit;
	}

	pb = start;
	for(i=0;i<nb;i+=16)
	{
		xil_printf("\r\n%3X  ",i);
		for(j=0;j<16;j++)
		{
			if (j == 8)
				print("- ");
			if (i+j < nb)
			{
				vb = pb[i+j];
				xil_printf("%X%X ",vb >> 4,vb & 15);
			}
			else
			{
				print("   ");
			}
		}
		print("  ");
		for(j=0;j<16;j++)
		{
			if (i+j < nb)
			{
				vb = pb[i+j];
				xil_printf("%c",(vb >= ' ' && vb <= '~') ? vb : '.');
			}
		}
	}
	print("\r\n");
}
