/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief ethernet agent
/// \file ethagent.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "glovar.h"

	// if extra parameters for motor control...
	#if SW_ENA_ENHANCED > 0
		#include <math.h>
		// never used function to include libm
		float jfloat2inml(float f) {remainderf(f,f); atan2f(f,f); return cosf(sinf(logf(expf(sqrtf(f))))); }
	#endif

#if SW_ENA_ETH_AGENT > 0

#include "mcaxeth.h"

typedef struct
{
	u8               task_status;

	u32              timer_tic_per_sec;
	u32              timer_tic_per_msec;

	u32              timer_tic_old_msec;
	u32              timer_msec;

	u32              twdg;
	u16              linktmo;

	u8               eth_crash;
	u8               sys_up;
    
    utcpip_ip_addr_t owner_addr;
    utcpip_in_port_t owner_port;
	u8               imanager;
	u8               managed;

	u32              tmng;
	u16              tmomsec;

	u32              tnetlive;
	
	u8             * pB;
	u16              lB;
	int              erPRT;


	utcpip_dev_t     udpip;

	#ifdef SW_SELF_RESTART_AGENT
	u32              timer_self_restart;
	#endif

} mca_eth_env_t;



// time-out manager

#define MCA_ETH_ALIVE_TIME_MSEC     5000 // for relinq speed

#define MCA_ETH_MANAGERTMO_MSEC_MIN  500 // minimun time for manager loose

static mca_eth_env_t mca_eth_env_v;

#define MCA_ETH_V_DEF mca_eth_env_t * mca_eth_v_ref = & mca_eth_env_v
#define MCA_ETH_V_REF mca_eth_v_ref

#define MCA_ETH_EVAL_MODSIZE_PROLOG() u32 _st_ = MCA_ETH_V_REF->lB
#define MCA_ETH_EVAL_MODSIZE_EPILOG() { xil_printf("\r\n SIZE = %d\r\n", _st_ - MCA_ETH_V_REF->lB); }

typedef union
{
	u32   u;
	float r;
} u32r32_t;

inline static u32 xdr_ext_level(void)
{
	MCA_ETH_V_DEF;
	return MCA_ETH_V_REF->lB;
}

static void xdr_ext_skip(const u16 skip)
{
	MCA_ETH_V_DEF;

	u16 n = (skip <= MCA_ETH_V_REF->lB) ? skip : MCA_ETH_V_REF->lB;
	MCA_ETH_V_REF->pB += n;
	MCA_ETH_V_REF->lB -= n;
}

static u32 xdr_ext_u8(void)
{
	MCA_ETH_V_DEF;

	u8 *p = MCA_ETH_V_REF->pB;
	u16 l = MCA_ETH_V_REF->lB;
	u32 t = 0;
	if (l == 0)
	{
		MCA_ETH_V_REF->erPRT = XST_FAILURE;
	}
	else
	{
		t = *p++;
		l--;
	}
	MCA_ETH_V_REF->pB = p;
	MCA_ETH_V_REF->lB = l;
	return t;
}

static u32 xdr_ext_u16(void)
{
	u32 t;
	t = xdr_ext_u8();
	t <<= 8;
	t |= xdr_ext_u8();
	return t;
}

static u32 xdr_ext_u24(void)
{
	u32 t;
	t = xdr_ext_u8();
	t <<= 16;
	t |= xdr_ext_u16();
	return t;
}

static u32 xdr_ext_u32(void)
{
	u32 t;
	t = xdr_ext_u16();
	t <<= 16;
	t |= xdr_ext_u16();
	return t;
}

#if SW_ENA_ENHANCED > 0

static float xdr_ext_r32(void)
{
	u32r32_t t;
	t.u = xdr_ext_u32();
	return t.r;
}

#endif

void xdr_ext_bytes(u8 *b,u32 l)
{
	while(l > 0)
	{
		*b++ = xdr_ext_u8();
		l--;
	}
}

static void xdr_ins_u8(const u32 v)
{
	MCA_ETH_V_DEF;

	u8 *p = MCA_ETH_V_REF->pB;
	u16 l = MCA_ETH_V_REF->lB;

	*p++ = v;
	l++;

	MCA_ETH_V_REF->pB = p;
	MCA_ETH_V_REF->lB = l;
}

static void xdr_ins_u16(const u32 v)
{
	xdr_ins_u8(v >> 8);
	xdr_ins_u8(v);
}

static void xdr_ins_u24(const u32 v)
{
	xdr_ins_u8(v >> 16);
	xdr_ins_u16(v);
}

static void xdr_ins_u32(const u32 v)
{
	xdr_ins_u16(v >> 16);
	xdr_ins_u16(v);
}

void xdr_ins_r32(const float v)
{
	u32r32_t t;
	t.r = v;
	xdr_ins_u32(t.u);
}


void xdr_ins_bytes(const u8 *b,u32 l)
{
	while(l > 0)
	{
		xdr_ins_u8(*b++);
		l--;
	}
}

static void xdr_ins_string(const char *s,u32 l)
{
	u8  v = 0;
	if (s)
	{
		while(l > 0)
		{
			v = *s++;
			if (v == 0) break;
			xdr_ins_u8(v);
			l--;
		}
	}
	while(l > 0)
	{
		xdr_ins_u8(v);
		l--;
	}
}

static int mca_eth_is_full_udpip(void)
{
	#if SW_ENA_ENHANCED > 0
	u16 id;
	for(id = 0; id < SYS_MOTORFOC_NUM; id++)
	{
		if (g_motorfoc_ap_v[id].ipdrvsource != MOTORFOC_AP_IPDRVSOURCE_UDPIP)
			return 0;
	}
	#endif
	return 1;
}

static void mca_eth_process_netsetup_tgm(void)
{
	MCA_ETH_V_DEF;

	if (!MCA_ETH_V_REF->imanager) return;

	if (mca_eth_is_full_udpip())
	{
		// overwrite IP
		g_eth_c_def_ip_address[0] = xdr_ext_u8();
		g_eth_c_def_ip_address[1] = xdr_ext_u8();
		g_eth_c_def_ip_address[2] = xdr_ext_u8();
		g_eth_c_def_ip_address[3] = xdr_ext_u8();
	}

	// no reply
	MCA_ETH_V_REF->lB = 0;
	MCA_ETH_V_REF->erPRT = XST_SUCCESS;
}

static void mca_eth_reply_simple(u32 code)
{
	MCA_ETH_V_DEF;

	MCA_ETH_V_REF->pB = MCA_ETH_V_REF->udpip.ctx.xnctl.xnctl_data;
	MCA_ETH_V_REF->lB = 0;
	MCA_ETH_V_REF->erPRT = XST_SUCCESS;
	xdr_ins_u8(XETHCTL_PROTO_TYPE);
	xdr_ins_u8(code);
}

static void mca_eth_reply_ident_tgm(void)
{
	mca_eth_reply_simple(XETHCTL_CODE_sIdent);
	//-
	xdr_ins_string(SYS_CARRIER_NAME,XETHCTL_CARRIER_NAME_LENGTH);
	xdr_ins_u16(PROGRAM_VERS_NUM);
	xdr_ins_u32(XPAR_FPGA_CLOCK_FREQ_HZ);
	
	#if SYS_MPHY18_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MPHY18_NUM; id++)
		{
			mphy18_drv_Config *CfgPtr = mphy18_drv_LookupConfig(id);
			//-
			xdr_ins_u8(MPHY18_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
			//-
			xdr_ins_u16(CfgPtr->inv_acqtime_ns);
			xdr_ins_u8(CfgPtr->rslv_exc_dvclk);
		}
	}
	#endif
	
	#if SYS_MPHY17_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MPHY17_NUM; id++)
		{
			mphy17_drv_Config *CfgPtr = mphy17_drv_LookupConfig(id);
			//-
			xdr_ins_u8(MPHY17_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
			//-
			xdr_ins_u16(CfgPtr->inv_acqtime_ns);
		}
	}
	#endif

	#if SYS_MPHY16_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MPHY16_NUM; id++)
		{
			mphy16_drv_Config *CfgPtr = mphy16_drv_LookupConfig(id);
			//-
			xdr_ins_u8(MPHY16_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
			//-
			xdr_ins_u16(CfgPtr->inv_acqtime_ns);
			xdr_ins_u16(CfgPtr->fan_pwm_freq);
			xdr_ins_u16(CfgPtr->dout_pwm_freq);
		}
	}
	#endif

	#if SYS_TEP2_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_TEP2_NUM; id++)
		{
			tep2_drv_Config *CfgPtr = tep2_drv_LookupConfig(id);
			//-
			xdr_ins_u8(TEP2_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
			//-
			xdr_ins_u16(CfgPtr->inv_acqtime_ns);
		}
	}
	#endif

	#if SYS_TEC53_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_TEC53_NUM; id++)
		{
			tec53_drv_Config *CfgPtr = tec53_drv_LookupConfig(id);
			//-
			xdr_ins_u8(TEC53_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
			//-
			xdr_ins_u16(CfgPtr->inv_acqtime_ns);
		}
	}
	#endif

	#if SYS_S4SICINV_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_S4SICINV_NUM; id++)
		{
			s4sicinv_drv_Config *CfgPtr = s4sicinv_drv_LookupConfig(id);
			//-
			xdr_ins_u8(S4SICINV_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
			//-
			xdr_ins_u16(CfgPtr->inv_acqtime_ns);
			xdr_ins_u8(CfgPtr->rslv_exc_dvclk);
			xdr_ins_u16(CfgPtr->fan_pwm_freq);
		}
	}
	#endif

	#if SYS_QINVSIC_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_QINVSIC_NUM; id++)
		{
			qinvsic_drv_Config *CfgPtr = qinvsic_drv_LookupConfig(id);
			//-
			xdr_ins_u8(QINVSIC_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
			xdr_ins_u8(CfgPtr->rslv_exc_dvclk);
			xdr_ins_u16(CfgPtr->dout_pwm_freq);
		}
	}
	#endif

	#if SYS_MPHY11_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MPHY11_NUM; id++)
		{
			mphy11_drv_Config *CfgPtr = mphy11_drv_LookupConfig(id);
			//-
			xdr_ins_u8(MPHY11_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
			//-
			xdr_ins_u16(CfgPtr->inv_acqtime_ns);
			xdr_ins_u16(CfgPtr->fan_pwm_freq);
			xdr_ins_u16(CfgPtr->dout_pwm_freq);
		}
	}
	#endif

	#if SYS_ZYGUSIC_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_ZYGUSIC_NUM; id++)
		{
			zygusic_drv_Config *CfgPtr = zygusic_drv_LookupConfig(id);
			//-
			xdr_ins_u8(ZYGUSIC_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
			//-
			xdr_ins_u16(CfgPtr->inv_acqtime_ns);
			xdr_ins_u8(CfgPtr->rslv_exc_dvclk);
			xdr_ins_u16(CfgPtr->fan_pwm_freq);
		}
	}
	#endif

	#if SYS_TLIMOT_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_TLIMOT_NUM; id++)
		{
			tlimot_drv_Config *CfgPtr = tlimot_drv_LookupConfig(id);
			//-
			xdr_ins_u8(TLIMOT_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
		}
	}
	#endif

	#if SYS_MLIEVB_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MLIEVB_NUM; id++)
		{
			mlievb_drv_Config *CfgPtr = mlievb_drv_LookupConfig(id);
			//-
			xdr_ins_u8(MLIEVB_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
		}
	}
	#endif

	#if SYS_MOTCON2_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MOTCON2_NUM; id++)
		{
			motcon2_drv_Config *CfgPtr = motcon2_drv_LookupConfig(id);
			//-
			xdr_ins_u8(MOTCON2_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref_0);
			xdr_ins_u8(CfgPtr->hw_link_ref_1);
			//-
			xdr_ins_u8(CfgPtr->xadc);
		}
	}
	#endif

	#if SYS_MOTCON1_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MOTCON1_NUM; id++)
		{
			motcon1_drv_Config *CfgPtr = motcon1_drv_LookupConfig(id);
			//-
			xdr_ins_u8(MOTCON1_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
			//-
			xdr_ins_u8(CfgPtr->xadc);
		}
	}
	#endif

	#if SYS_SGXBCU_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_SGXBCU_NUM; id++)
		{
			sgxbcu_drv_Config *CfgPtr = sgxbcu_drv_LookupConfig(id);
			//-
			xdr_ins_u8(SGXBCU_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
		}
	}
	#endif

	#if SYS_FMCMC1_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_FMCMC1_NUM; id++)
		{
			fmcmc1_drv_Config *CfgPtr = fmcmc1_drv_LookupConfig(id);
			//-
			xdr_ins_u8(FMCMC1_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref_0);
			xdr_ins_u8(CfgPtr->hw_link_ref_1);
			//-
			xdr_ins_u8(CfgPtr->xadc);
		}
	}
	#endif

	#if SYS_NETMOT_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_NETMOT_NUM; id++)
		{
			netmot_drv_Config *CfgPtr = netmot_drv_LookupConfig(id);
			//-
			xdr_ins_u8(NETMOT_DRV_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->carrier);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref_0);
			xdr_ins_u8(CfgPtr->hw_link_ref_1);
		}
	}
	#endif

	// -- motors
	{
		u16 id;
		for(id = 0; id < SYS_MOTORFOC_NUM; id++)
		{
			motorfoc_Config *CfgPtr = motorfoc_LookupConfig(id);
			//-
			xdr_ins_u8(MOTORFOC_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
			//-
			xdr_ins_u16(CfgPtr->inv_in_map);
			xdr_ins_u16(CfgPtr->inv_in_not);
			xdr_ins_u8(CfgPtr->inv_ofsv_mode);
			xdr_ins_u8(CfgPtr->inv_over_iphs);
			xdr_ins_u8(CfgPtr->inv_over_ibus);
			xdr_ins_u8(CfgPtr->inv_filter);
			xdr_ins_u8(CfgPtr->clarke_nphs);
			xdr_ins_u8(CfgPtr->pwm_modulator);
			xdr_ins_u8(CfgPtr->rpfm_modulator);
			xdr_ins_u8(CfgPtr->rpfm_3_level);
			xdr_ins_u8(CfgPtr->rpfm_tpnc);
			xdr_ins_u8(CfgPtr->smo_eval);
			xdr_ins_u8(CfgPtr->spd_eval);
			xdr_ins_u8(CfgPtr->speed_ctrl);
			xdr_ins_u8(CfgPtr->resolver);
			xdr_ins_u8(CfgPtr->hallsensor);
			xdr_ins_u8(CfgPtr->encoder);
			xdr_ins_u8(CfgPtr->manrot);
			xdr_ins_u8(CfgPtr->mrt_accmax_dln2);
			xdr_ins_u8(CfgPtr->smo_zs_dln2);
			xdr_ins_u8(CfgPtr->smo_f2_dln2);
			xdr_ins_u8(CfgPtr->slp_pro_dln2);
			xdr_ins_u8(CfgPtr->slp_int_dln2);
			xdr_ins_u8(CfgPtr->slp_indwp_dln2);
			xdr_ins_u8(CfgPtr->slp_indwp_kdiv);
			xdr_ins_u8(CfgPtr->isoverflow_cmax);
			xdr_ins_u8(CfgPtr->pi_erpro_dln2);
			xdr_ins_u8(CfgPtr->pi_erint_dln2);
		}
	}

	#if SYS_IVPLOGGER_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_IVPLOGGER_NUM; id++)
		{
			ivplogger_Config *CfgPtr = ivplogger_LookupConfig(id);
			//-
			xdr_ins_u8(IVPLOGGER_ip_ident_code_value);
			//-
			xdr_ins_u8(CfgPtr->hw_link_ref);
			//-
			xdr_ins_u16(CfgPtr->channel_map);
			xdr_ins_u32(CfgPtr->memory_base);
			xdr_ins_u32(CfgPtr->memory_size);
		}
	}
	#endif

}

static void mca_eth_process_ipssetup(void)
{
	//MCA_ETH_V_DEF;

	#if SW_ENA_ENHANCED > 0
	//--
	while(xdr_ext_level() > 0)
	{
		u8 ip_id_code = xdr_ext_u8();
		switch(ip_id_code)
		{
			default:
				{
					return;
				}
				break;

			case MOTORFOC_ip_ident_code_value:
				{
					// motor id
					u8 mot_id = xdr_ext_u8();
					//-
					if (mot_id >= SYS_MOTORFOC_NUM) continue;
					//-
					u8 skip = 0;
					#if SW_ENA_ENHANCED > 0
					skip = g_motorfoc_ap_v[mot_id].ipdrvsource != MOTORFOC_AP_IPDRVSOURCE_UDPIP;
					#endif
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(362);
					}
					else
					{
						//-
						motorfoc_ap_prm_t * r = &g_motorfoc_ap_v[mot_id].prm;
						//-
						int i;
						//-
						//- system parameters
						r->sys.ts         = xdr_ext_r32();
						//-
						r->sys.vbus_gain  = xdr_ext_r32();
						r->sys.vphs_gain  = xdr_ext_r32();
						//-
						r->sys.ibus_gain  = xdr_ext_r32();
						r->sys.iphs_gain  = xdr_ext_r32();
						//-
						r->sys.xbus_fcut1 = xdr_ext_r32();
						r->sys.xbus_fcut2 = xdr_ext_r32();
						//-
						r->sys.xphs_fcut1 = xdr_ext_r32();
						r->sys.xphs_fcut2 = xdr_ext_r32();
						//-
						r->sys.ibus_limit = xdr_ext_r32();
						r->sys.iphs_limit = xdr_ext_r32();
						//-
						r->sys.deadtime   = xdr_ext_r32();
						// motor parameters
						xdr_ext_bytes((u8 *)r->mot.manufacturer,MOTORFOC_AP_MANUFACTURER_LEN);
						xdr_ext_bytes((u8 *)r->mot.motorcode,MOTORFOC_AP_MOTORCODE_LEN);
						r->mot.windings = xdr_ext_u8();
						r->mot.paipol = xdr_ext_u8();
						r->mot.inductance = xdr_ext_r32();
						r->mot.resistance = xdr_ext_r32();
						r->mot.bemfke = xdr_ext_r32();
						r->mot.torquemax = xdr_ext_r32();
						r->mot.torquekc = xdr_ext_r32();
						r->mot.maxspeed = xdr_ext_r32();
						r->mot.iscurrmax = xdr_ext_r32();
						r->mot.iscurrhigh = xdr_ext_r32();
						r->mot.controlmode = xdr_ext_u8();
						r->mot.rotposmode = xdr_ext_u8();
						// speed loop
						r->slp.pgain = xdr_ext_r32();
						r->slp.igain = xdr_ext_r32();
						// pi regulator
						r->pi.pgain = xdr_ext_r32();
						r->pi.igain = xdr_ext_r32();
						// modulator
						r->mod.mod2angskw = xdr_ext_r32();
						r->mod.modlevels = xdr_ext_u8();
						r->mod.modulator = xdr_ext_u8();
						// pwm modulator
						r->pwm.freq = xdr_ext_r32();
						r->pwm.gain = xdr_ext_r32();
						r->pwm.mdmax = xdr_ext_r32();
						r->pwm.ctrls = xdr_ext_u8();
						// rpfm modulator
						r->rpfm.pwdth = xdr_ext_r32();
						r->rpfm.ctrls = xdr_ext_u8();
						// manual mode
						r->man.spfcut1 = xdr_ext_r32();
						r->man.spfcut2 = xdr_ext_r32();
						// scalar mode angle
						r->mrt.accmax = xdr_ext_r32();
						r->mrt.sofcut1 = xdr_ext_r32();
						r->mrt.sofcut2 = xdr_ext_r32();
						r->mrt.ctrls = xdr_ext_u8();
						r->mrt.spfcut1 = xdr_ext_r32();
						r->mrt.spfcut2 = xdr_ext_r32();
						// external
						r->ext.spfcut1 = xdr_ext_r32();
						r->ext.spfcut2 = xdr_ext_r32();
						// resolver
						r->rsv.paipol = xdr_ext_u8();
						r->rsv.angofs = xdr_ext_r32();
						r->rsv.spfcut1 = xdr_ext_r32();
						r->rsv.spfcut2 = xdr_ext_r32();
						r->rsv.excfreq = xdr_ext_r32();
						// hall sensor
						r->hls.ctolim = xdr_ext_r32();
						for(i=0;i<MOTORFOC_HLS_DPR_ANGLE_ELEM;i++)
						{
							r->hls.anglelut[i] = xdr_ext_r32();
						}
						r->hls.spfcut1 = xdr_ext_r32();
						r->hls.spfcut2 = xdr_ext_r32();
						// encoder parameters
						r->enc.cyprnd = xdr_ext_u16();
						r->enc.ctrls = xdr_ext_u8();
						r->enc.spfcut1 = xdr_ext_r32();
						r->enc.spfcut2 = xdr_ext_r32();
						// sliding mode observer
						r->smo.is_maxerr = xdr_ext_r32();
						r->smo.zs_maxval = xdr_ext_r32();
						r->smo.es1_fcut = xdr_ext_r32();
						r->smo.es2_fclow = xdr_ext_r32();
						r->smo.es2_fcmul = xdr_ext_r32();
						r->smo.angofs = xdr_ext_r32();
						r->smo.ctrls = xdr_ext_u8();
						r->smo.spfcut1 = xdr_ext_r32();
						r->smo.spfcut2 = xdr_ext_r32();
						//-
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
		}
	}

	#endif
	mca_eth_reply_simple(XETHCTL_CODE_sIpsSetup);
}

static void mca_eth_process_cctrl_tgm(void)
{
	MCA_ETH_V_DEF;

	if (!MCA_ETH_V_REF->imanager) return;

	// timeout for line-crash
	MCA_ETH_V_REF->linktmo = xdr_ext_u16();

	//--
	while(xdr_ext_level() > 0)
	{
		u8 ip_id_code = xdr_ext_u8();

		switch(ip_id_code)
		{
			default:
				{
					return;
				}
				break;
				
			#if SYS_MPHY18_NUM > 0
			case MPHY18_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(12);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_MPHY18_NUM; id++)
						{
							mphy18_drv_Config *CfgPtr = mphy18_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							u32 mphy18_base = CfgPtr->BaseAddress;
							u32 t32;
							//-
							t32 = xdr_ext_u8(); MPHY18_DRV_mWrite_control_reg(mphy18_base,t32);

							t32 = xdr_ext_u24(); MPHY18_DRV_mWrite_ecu_temp_fkt(mphy18_base,t32);
							t32 = xdr_ext_u24(); MPHY18_DRV_mWrite_rm_temp_fkt(mphy18_base,t32);

							t32 = xdr_ext_u32(); MPHY18_DRV_mWrite_rxg_speed(mphy18_base,t32);
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif
				
			#if SYS_MPHY17_NUM > 0
			case MPHY17_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(11);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_MPHY17_NUM; id++)
						{
							mphy17_drv_Config *CfgPtr = mphy17_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							u32 mphy17_base = CfgPtr->BaseAddress;
							u32 t32;
							//-
							t32 = xdr_ext_u8(); MPHY17_DRV_mWrite_control_reg(mphy17_base,t32);

							t32 = xdr_ext_u24(); MPHY17_DRV_mWrite_ecu_temp_fkt(mphy17_base,t32);
							t32 = xdr_ext_u24(); MPHY17_DRV_mWrite_rm_temp_fkt(mphy17_base,t32);

							t32 = xdr_ext_u8();  MPHY17_DRV_mWrite_rdc_m2rppk(mphy17_base,t32);
							t32 = xdr_ext_u16(); MPHY17_DRV_mWrite_rdc_angofs(mphy17_base,t32);
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif
				
			#if SYS_MPHY16_NUM > 0
			case MPHY16_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(31);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_MPHY16_NUM; id++)
						{
							mphy16_drv_Config *CfgPtr = mphy16_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							u32 mphy16_base = CfgPtr->BaseAddress;
							u32 t32;
							int i;
							//-
							t32 = xdr_ext_u8(); MPHY16_DRV_mWrite_control_reg(mphy16_base,t32);
							t32 = xdr_ext_u24(); MPHY16_DRV_mWrite_ecu_temp_fkt(mphy16_base,t32);
							t32 = xdr_ext_u24(); MPHY16_DRV_mWrite_rm_temp_fkt(mphy16_base,t32);
							t32 = xdr_ext_u24(); MPHY16_DRV_mWrite_vinrush_fkt(mphy16_base,t32);
							t32 = xdr_ext_u24(); MPHY16_DRV_mWrite_vinrush_max(mphy16_base,t32);
							t32 = xdr_ext_u24(); MPHY16_DRV_mWrite_vbus_min(mphy16_base,t32);
							for(i = 0; i < MPHY16_DRV_FAN_POWER_NUM; i++)
							{
								t32 = xdr_ext_u16(); MPHY16_DRV_mWrite_fan_power(mphy16_base,i,t32);
							}
							for(i = 0; i < MPHY16_DRV_DOUT_POWER_NUM; i++)
							{
								t32 = xdr_ext_u16(); MPHY16_DRV_mWrite_dout_power(mphy16_base,i,t32);
							}
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif

			#if SYS_TEP2_NUM > 0
			case TEP2_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(1);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_TEP2_NUM; id++)
						{
							tep2_drv_Config *CfgPtr = tep2_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							//u32 tep2_base = CfgPtr->BaseAddress;
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif
			
			#if SYS_TEC53_NUM > 0
			case TEC53_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(1);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_TEC53_NUM; id++)
						{
							tec53_drv_Config *CfgPtr = tec53_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							//u32 tec53_base = CfgPtr->BaseAddress;
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif

			
			#if SYS_S4SICINV_NUM > 0
			case S4SICINV_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(11);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_S4SICINV_NUM; id++)
						{
							s4sicinv_drv_Config *CfgPtr = s4sicinv_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							u32 s4sicinv_base = CfgPtr->BaseAddress;
							u32 t32;
							//-
							t32 = xdr_ext_u8(); S4SICINV_DRV_mWrite_control_reg(s4sicinv_base,t32);
							t32 = xdr_ext_u32(); S4SICINV_DRV_mWrite_rxg_speed(s4sicinv_base,t32);
							t32 = xdr_ext_u24(); S4SICINV_DRV_mWrite_rm_temp_fkt(s4sicinv_base,t32);
							t32 = xdr_ext_u16(); S4SICINV_DRV_mWrite_fan_power(s4sicinv_base,t32);
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif
			
			
			#if SYS_QINVSIC_NUM > 0
			case QINVSIC_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(17);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_QINVSIC_NUM; id++)
						{
							qinvsic_drv_Config *CfgPtr = qinvsic_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							u32 qinvsic_base = CfgPtr->BaseAddress;
							u32 t32;
							int i;
							//-
							t32 = xdr_ext_u8(); QINVSIC_DRV_mWrite_control_reg(qinvsic_base,t32);
							t32 = xdr_ext_u32(); QINVSIC_DRV_mWrite_rxg_speed(qinvsic_base,t32);
							t32 = xdr_ext_u24(); QINVSIC_DRV_mWrite_rm_temp_fkt(qinvsic_base,t32);
							for(i = 0; i < QINVSIC_DRV_DOUT_POWER_NUM; i++)
							{
								t32 = xdr_ext_u16(); QINVSIC_DRV_mWrite_dout_power(qinvsic_base,i,t32);
							}
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif
		
			#if SYS_MPHY11_NUM > 0
			case MPHY11_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(39);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_MPHY11_NUM; id++)
						{
							mphy11_drv_Config *CfgPtr = mphy11_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							u32 mphy11_base = CfgPtr->BaseAddress;
							u32 t32;
							int i;
							//-
							t32 = xdr_ext_u8(); MPHY11_DRV_mWrite_control_reg(mphy11_base,t32);
							t32 = xdr_ext_u24(); MPHY11_DRV_mWrite_anin_fkt(mphy11_base,t32);
							t32 = xdr_ext_u24(); MPHY11_DRV_mWrite_ecu_temp_fkt(mphy11_base,t32);
							t32 = xdr_ext_u24(); MPHY11_DRV_mWrite_rm_temp_fkt(mphy11_base,t32);
							for(i = 0; i < MPHY11_DRV_FAN_POWER_NUM; i++)
							{
								t32 = xdr_ext_u16(); MPHY11_DRV_mWrite_fan_power(mphy11_base,i,t32);
							}
							for(i = 0; i < MPHY11_DRV_DOUT_POWER_NUM; i++)
							{
								t32 = xdr_ext_u16(); MPHY11_DRV_mWrite_dout_power(mphy11_base,i,t32);
							}
							for(i = 0; i < MPHY11_DRV_AOUT_POWER_NUM; i++)
							{
								t32 = xdr_ext_u16(); MPHY11_DRV_mWrite_aout_power(mphy11_base,i,t32);
							}
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif
			
			
			#if SYS_ZYGUSIC_NUM > 0
			case ZYGUSIC_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(8);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_ZYGUSIC_NUM; id++)
						{
							zygusic_drv_Config *CfgPtr = zygusic_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							u32 zygusic_base = CfgPtr->BaseAddress;
							u32 t32;
							//-
							t32 = xdr_ext_u8(); ZYGUSIC_DRV_mWrite_control_reg(zygusic_base,t32);
							t32 = xdr_ext_u32(); ZYGUSIC_DRV_mWrite_rxg_speed(zygusic_base,t32);
							t32 = xdr_ext_u16(); ZYGUSIC_DRV_mWrite_fan_power(zygusic_base,t32);
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif
			
			#if SYS_TLIMOT_NUM > 0
			case TLIMOT_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(3);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_TLIMOT_NUM; id++)
						{
							tlimot_drv_Config *CfgPtr = tlimot_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							u32 tlimot_base = CfgPtr->BaseAddress;
							u32 t32;
							//-
							t32 = xdr_ext_u16(); TLIMOT_DRV_mWrite_control_reg(tlimot_base,t32);
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif
		
			#if SYS_MLIEVB_NUM > 0
			case MLIEVB_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(3);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_MLIEVB_NUM; id++)
						{
							mlievb_drv_Config *CfgPtr = mlievb_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							u32 mlievb_base = CfgPtr->BaseAddress;
							u32 t32;
							//-
							t32 = xdr_ext_u16(); MLIEVB_DRV_mWrite_control_reg(mlievb_base,t32);
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif

			#if SYS_MOTCON2_NUM > 0
			case MOTCON2_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(5);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_MOTCON2_NUM; id++)
						{
							motcon2_drv_Config *CfgPtr = motcon2_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							u32 motcon2_base = CfgPtr->BaseAddress;
							u32 t32;
							//-
							t32 = xdr_ext_u8(); MOTCON2_DRV_mWrite_control_reg(motcon2_base,t32);
							t32 = xdr_ext_u24(); MOTCON2_DRV_mWrite_oc_temp_fkt(motcon2_base,t32);
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif
			
			#if SYS_MOTCON1_NUM > 0
			case MOTCON1_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(5);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_MOTCON1_NUM; id++)
						{
							motcon1_drv_Config *CfgPtr = motcon1_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							u32 motcon1_base = CfgPtr->BaseAddress;
							u32 t32;
							//-
							t32 = xdr_ext_u8(); MOTCON1_DRV_mWrite_control_reg(motcon1_base,t32);
							t32 = xdr_ext_u24(); MOTCON1_DRV_mWrite_oc_temp_fkt(motcon1_base,t32);
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif
		
			#if SYS_SGXBCU_NUM > 0
			case SGXBCU_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(2);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_SGXBCU_NUM; id++)
						{
							sgxbcu_drv_Config *CfgPtr = sgxbcu_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							u32 sgxbcu_base = CfgPtr->BaseAddress;
							u32 t32;
							//-
							t32 = xdr_ext_u8(); SGXBCU_DRV_mWrite_control_reg(sgxbcu_base,t32);
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif

			#if SYS_FMCMC1_NUM > 0
			case FMCMC1_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(18);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_FMCMC1_NUM; id++)
						{
							fmcmc1_drv_Config *CfgPtr = fmcmc1_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							u32 fmcmc1_base = CfgPtr->BaseAddress;
							u32 t32;
							//-
							t32 = xdr_ext_u24(); FMCMC1_DRV_mWrite_sd_ctrl_0(fmcmc1_base,t32);
							t32 = xdr_ext_u24(); FMCMC1_DRV_mWrite_sd_ctrl_1(fmcmc1_base,t32);
							//-
							t32 = xdr_ext_u24(); FMCMC1_DRV_mWrite_oc_temp_fkt(fmcmc1_base,t32);
							//-
							t32 = xdr_ext_u16(); FMCMC1_DRV_mWrite_xa_synclkdiv(fmcmc1_base,t32);
							//-
							t32 = xdr_ext_u32(); FMCMC1_DRV_mWrite_xadc_wctreg(fmcmc1_base,t32);
							//-
							t32 = xdr_ext_u8(); FMCMC1_DRV_mWrite_gpio_o(fmcmc1_base,t32);
							t32 = xdr_ext_u8(); FMCMC1_DRV_mWrite_gpio_t(fmcmc1_base,t32);
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif

			#if SYS_NETMOT_NUM > 0
			case NETMOT_DRV_ip_ident_code_value:
				{
					//-
					u8 skip = !mca_eth_is_full_udpip();
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(3);
					}
					else
					{
						u8 DeviceId = xdr_ext_u8();
						u16 id;
						for(id = 0;id < SYS_NETMOT_NUM; id++)
						{
							netmot_drv_Config *CfgPtr = netmot_drv_LookupConfig(id);
							//-
							if (DeviceId != CfgPtr->DeviceId) continue;
							//-
							u32 netmot_base = CfgPtr->BaseAddress;
							u32 t32;
							//-
							t32  = xdr_ext_u8() << NETMOT_DRV_CMD_OUT_BIT;
							t32 |= xdr_ext_u8() << NETMOT_DRV_CMD_LED_BIT;

							NETMOT_DRV_command_rmw(netmot_base,(0xFF << NETMOT_DRV_CMD_LED_BIT) | (0xFF << NETMOT_DRV_CMD_OUT_BIT),t32);
						}
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;
			#endif

			case MOTORFOC_ip_ident_code_value:
				{
					u8 mot_id = xdr_ext_u8();
					if (mot_id >= SYS_MOTORFOC_NUM) continue;
					//-
					u8 skip = 0;
					#if SW_ENA_ENHANCED > 0
					skip = g_motorfoc_ap_v[mot_id].ipdrvsource != MOTORFOC_AP_IPDRVSOURCE_UDPIP;
					#endif
					//-
					//MCA_ETH_EVAL_MODSIZE_PROLOG();
					//-
					if (skip)
					{
						xdr_ext_skip(165);
					}
					else
					{
						//-
						u32 mfoc_base = motorfoc_LookupConfig(mot_id)->BaseAddress;
						//-
						u32 t32;
						int i;
						//-
						t32 = xdr_ext_u32(); MOTORFOC_mWrite_motcontrol(mfoc_base,t32);
						//-
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_xbus_fk1(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_xbus_fk2(mfoc_base,t32);
						//-
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_xphs_fk1(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_xphs_fk2(mfoc_base,t32);
						//-
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_ibus_limit(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_iphs_limit(mfoc_base,t32);
						//-
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_ismodmax(mfoc_base,t32);
						//-
						t32 = xdr_ext_u32(); MOTORFOC_mWrite_rot_angdef(mfoc_base,t32);
						//-
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_spd_fktau1(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_spd_fktau2(mfoc_base,t32);
						//-
						t32 = xdr_ext_u32(); MOTORFOC_mWrite_smo_vs_mult(mfoc_base,t32);
						t32 = xdr_ext_u32(); MOTORFOC_mWrite_smo_is_mult(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_smo_zs_max(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_smo_es1_kflt(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_smo_es2_kflo(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_smo_es2_kfmx(mfoc_base,t32);
						t32 = xdr_ext_u16(); MOTORFOC_mWrite_smo_angofs(mfoc_base,t32);
						//-
						t32 = xdr_ext_u32(); MOTORFOC_mWrite_slp_spdset(mfoc_base,t32);
						t32 = xdr_ext_u32(); MOTORFOC_mWrite_slp_kmpro(mfoc_base,t32);
						t32 = xdr_ext_u32(); MOTORFOC_mWrite_slp_kmint(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_slp_outlim(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_slp_kmultx(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_slp_kmulty(mfoc_base,t32);
						//-
						t32 = xdr_ext_u8(); MOTORFOC_mWrite_paipol(mfoc_base,t32);
						//-
						t32 = xdr_ext_u8(); MOTORFOC_mWrite_rsv_m2rppk(mfoc_base,t32);
						t32 = xdr_ext_u16(); MOTORFOC_mWrite_rsv_angofs(mfoc_base,t32);
						//-
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_hls_ctolim(mfoc_base,t32);
						//-
						for(i = 0; i < MOTORFOC_HLS_DPR_ANGLE_ELEM;i++)
						{
							t32 = xdr_ext_u16(); MOTORFOC_mWrite_hall_sensor_angle_lut(mfoc_base,i << 2,t32);
						}
						//-
						t32 = xdr_ext_u16(); MOTORFOC_mWrite_enc_cyprnd(mfoc_base,t32);
						t32 = xdr_ext_u32(); MOTORFOC_mWrite_enc_angphs(mfoc_base,t32);
						//-
						t32 = xdr_ext_u32(); MOTORFOC_mWrite_mrt_speed(mfoc_base,t32);
						t32 = xdr_ext_u32(); MOTORFOC_mWrite_mrt_accmax(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_mrt_fktau1(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_mrt_fktau2(mfoc_base,t32);
						//-
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_pi_setvalx(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_pi_setvaly(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_pi_kmpro(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_pi_kmint(mfoc_base,t32);
						//-
						t32 = xdr_ext_u16(); MOTORFOC_mWrite_mod2angskw(mfoc_base,t32);
						//-
						t32 = xdr_ext_u32(); MOTORFOC_mWrite_pwm_kmod(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_pwm_presc(mfoc_base,t32);
						t32 = xdr_ext_u24(); MOTORFOC_mWrite_pwm_mdmax(mfoc_base,t32);
						t32 = xdr_ext_u8(); MOTORFOC_mWrite_pwm_ctrl(mfoc_base,t32);
						//-
						t32 = xdr_ext_u16(); MOTORFOC_mWrite_rpfm3p_ctrl(mfoc_base,t32);
						//-
						t32 = xdr_ext_u8(); MOTORFOC_mWrite_deadtval(mfoc_base,t32);
					}
					//-
					//MCA_ETH_EVAL_MODSIZE_EPILOG();
				}
				break;

		}
	}


	//- reset watch dog
	MCA_ETH_V_REF->twdg = MCA_ETH_V_REF->timer_msec;
}


static void mca_eth_reply_sstat_tgm(void)
{
	MCA_ETH_V_DEF;

	mca_eth_reply_simple(XETHCTL_CODE_sStat);

	// I am manager
	xdr_ins_u8(MCA_ETH_V_REF->imanager);

	// link timeout
	xdr_ins_u16(MCA_ETH_V_REF->linktmo);

	// timer value
	xdr_ins_u32(MCA_ETH_V_REF->timer_msec);

	// -- boards
	
	#if SYS_MPHY18_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MPHY18_NUM; id++)
		{
			mphy18_drv_Config *CfgPtr = mphy18_drv_LookupConfig(id);
			//-
			xdr_ins_u8(MPHY18_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			u32 mphy18_base = CfgPtr->BaseAddress;
			//-
			xdr_ins_u8(MPHY18_DRV_mRead_status_reg(mphy18_base));

			xdr_ins_u16(MPHY18_DRV_mRead_ecu_temp(mphy18_base));
			xdr_ins_u16(MPHY18_DRV_mRead_rm_temp(mphy18_base));
		}
	}
	#endif
	
	#if SYS_MPHY17_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MPHY17_NUM; id++)
		{
			mphy17_drv_Config *CfgPtr = mphy17_drv_LookupConfig(id);
			//-
			xdr_ins_u8(MPHY17_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			u32 mphy17_base = CfgPtr->BaseAddress;
			//-
			xdr_ins_u8(MPHY17_DRV_mRead_status_reg(mphy17_base));

			xdr_ins_u16(MPHY17_DRV_mRead_ecu_temp(mphy17_base));
			xdr_ins_u16(MPHY17_DRV_mRead_rm_temp(mphy17_base));
		}
	}
	#endif
	
	#if SYS_MPHY16_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MPHY16_NUM; id++)
		{
			mphy16_drv_Config *CfgPtr = mphy16_drv_LookupConfig(id);
			//-
			xdr_ins_u8(MPHY16_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			u32 mphy16_base = CfgPtr->BaseAddress;
			int i;
			//-
			xdr_ins_u8(MPHY16_DRV_mRead_status_reg(mphy16_base));
			xdr_ins_u8(MPHY16_DRV_mRead_digin(mphy16_base));
			xdr_ins_u24(MPHY16_DRV_mRead_vinrush(mphy16_base));
			for(i = 0; i < MPHY16_DRV_ECU_TEMP_NUM; i++)
			{
				xdr_ins_u16(MPHY16_DRV_mRead_ecu_temp(mphy16_base,i));
			}
			for(i = 0; i < MPHY16_DRV_RM_TEMP_NUM; i++)
			{
				xdr_ins_u16(MPHY16_DRV_mRead_rm_temp(mphy16_base,i));
			}
			//-
			for(i = 0; i < MPHY16_DRV_FAN_TACHO_NUM; i++)
			{
				xdr_ins_u16(MPHY16_DRV_mRead_fan_tacho(mphy16_base,i));
			}
		}
	}
	#endif

	#if SYS_TEP2_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_TEP2_NUM; id++)
		{
			tep2_drv_Config *CfgPtr = tep2_drv_LookupConfig(id);
			//-
			xdr_ins_u8(TEP2_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			//u32 tep2_base = CfgPtr->BaseAddress;
		}
	}
	#endif

	#if SYS_TEC53_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_TEC53_NUM; id++)
		{
			tec53_drv_Config *CfgPtr = tec53_drv_LookupConfig(id);
			//-
			xdr_ins_u8(TEC53_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			//u32 tec53_base = CfgPtr->BaseAddress;
		}
	}
	#endif
	
	#if SYS_S4SICINV_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_S4SICINV_NUM; id++)
		{
			s4sicinv_drv_Config *CfgPtr = s4sicinv_drv_LookupConfig(id);
			//-
			xdr_ins_u8(S4SICINV_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			u32 s4sicinv_base = CfgPtr->BaseAddress;
			int i;
			//-
			xdr_ins_u8(S4SICINV_DRV_mRead_status_reg(s4sicinv_base));
			xdr_ins_u8(S4SICINV_DRV_mRead_digin(s4sicinv_base));
			for(i = 0; i < S4SICINV_DRV_RM_TEMP_NUM; i++)
			{
				xdr_ins_u16(S4SICINV_DRV_mRead_rm_temp(s4sicinv_base,i));
			}
		}
	}
	#endif

	#if SYS_QINVSIC_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_QINVSIC_NUM; id++)
		{
			qinvsic_drv_Config *CfgPtr = qinvsic_drv_LookupConfig(id);
			//-
			xdr_ins_u8(QINVSIC_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			u32 qinvsic_base = CfgPtr->BaseAddress;
			int i;
			//-
			xdr_ins_u8(QINVSIC_DRV_mRead_status_reg(qinvsic_base));
			xdr_ins_u8(QINVSIC_DRV_mRead_digin(qinvsic_base));
			for(i = 0; i < QINVSIC_DRV_RM_TEMP_NUM; i++)
			{
				xdr_ins_u16(QINVSIC_DRV_mRead_rm_temp(qinvsic_base,i));
			}
		}
	}
	#endif

	#if SYS_MPHY11_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MPHY11_NUM; id++)
		{
			mphy11_drv_Config *CfgPtr = mphy11_drv_LookupConfig(id);
			//-
			xdr_ins_u8(MPHY11_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			u32 mphy11_base = CfgPtr->BaseAddress;
			int i;
			//-
			xdr_ins_u8(MPHY11_DRV_mRead_status_reg(mphy11_base));
			xdr_ins_u8(MPHY11_DRV_mRead_digin(mphy11_base));
			for(i = 0; i < MPHY11_DRV_ANIN_NUM; i++)
			{
				xdr_ins_u16(MPHY11_DRV_mRead_anin(mphy11_base,i));
			}
			for(i = 0; i < MPHY11_DRV_ECU_TEMP_NUM; i++)
			{
				xdr_ins_u16(MPHY11_DRV_mRead_ecu_temp(mphy11_base,i));
			}
			for(i = 0; i < MPHY11_DRV_RM_TEMP_NUM; i++)
			{
				xdr_ins_u16(MPHY11_DRV_mRead_rm_temp(mphy11_base,i));
			}
			//-
			for(i = 0; i < MPHY11_DRV_FAN_TACHO_NUM; i++)
			{
				xdr_ins_u16(MPHY11_DRV_mRead_fan_tacho(mphy11_base,i));
			}
		}
	}
	#endif

	#if SYS_ZYGUSIC_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_ZYGUSIC_NUM; id++)
		{
			zygusic_drv_Config *CfgPtr = zygusic_drv_LookupConfig(id);
			//-
			xdr_ins_u8(ZYGUSIC_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			u32 zygusic_base = CfgPtr->BaseAddress;
			//-
			xdr_ins_u8(ZYGUSIC_DRV_mRead_status_reg(zygusic_base));
		}
	}
	#endif

	#if SYS_TLIMOT_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_TLIMOT_NUM; id++)
		{
			tlimot_drv_Config *CfgPtr = tlimot_drv_LookupConfig(id);
			//-
			xdr_ins_u8(TLIMOT_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			u32 tlimot_base = CfgPtr->BaseAddress;
			//-
			xdr_ins_u8(TLIMOT_DRV_mRead_status_reg(tlimot_base));
		}
	}
	#endif

	#if SYS_MLIEVB_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MLIEVB_NUM; id++)
		{
			mlievb_drv_Config *CfgPtr = mlievb_drv_LookupConfig(id);
			//-
			xdr_ins_u8(MLIEVB_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			u32 mlievb_base = CfgPtr->BaseAddress;
			//-
			xdr_ins_u8(MLIEVB_DRV_mRead_status_reg(mlievb_base));
		}
	}
	#endif

	#if SYS_MOTCON2_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MOTCON2_NUM; id++)
		{
			motcon2_drv_Config *CfgPtr = motcon2_drv_LookupConfig(id);
			//-
			xdr_ins_u8(MOTCON2_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			u32 motcon2_base = CfgPtr->BaseAddress;
			//-
			xdr_ins_u8(MOTCON2_DRV_mRead_status_reg(motcon2_base));
			xdr_ins_u24(MOTCON2_DRV_mRead_oc_temp_val(motcon2_base));
		}
	}
	#endif

	#if SYS_MOTCON1_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MOTCON1_NUM; id++)
		{
			motcon1_drv_Config *CfgPtr = motcon1_drv_LookupConfig(id);
			//-
			xdr_ins_u8(MOTCON1_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			u32 motcon1_base = CfgPtr->BaseAddress;
			//-
			xdr_ins_u8(MOTCON1_DRV_mRead_status_reg(motcon1_base));
			xdr_ins_u24(MOTCON1_DRV_mRead_oc_temp_val(motcon1_base));
		}
	}
	#endif

	#if SYS_SGXBCU_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_SGXBCU_NUM; id++)
		{
			sgxbcu_drv_Config *CfgPtr = sgxbcu_drv_LookupConfig(id);
			//-
			xdr_ins_u8(SGXBCU_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			u32 sgxbcu_base = CfgPtr->BaseAddress;
			//-
			xdr_ins_u8(SGXBCU_DRV_mRead_status_reg(sgxbcu_base));
		}
	}
	#endif

	#if SYS_FMCMC1_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_FMCMC1_NUM; id++)
		{
			fmcmc1_drv_Config *CfgPtr = fmcmc1_drv_LookupConfig(id);
			//-
			xdr_ins_u8(FMCMC1_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			u32 fmcmc1_base = CfgPtr->BaseAddress;
			//-
			xdr_ins_u8(FMCMC1_DRV_mRead_status_reg(fmcmc1_base));
			//-
			xdr_ins_u24(FMCMC1_DRV_mRead_oc_temp_val(fmcmc1_base));
			//-
			xdr_ins_u32(FMCMC1_DRV_mRead_xadc_rctreg(fmcmc1_base));
			//-
			xdr_ins_u8(FMCMC1_DRV_mRead_dip_sw(fmcmc1_base));
			xdr_ins_u8(FMCMC1_DRV_mRead_gpio_i(fmcmc1_base));
		}
	}
	#endif

	#if SYS_NETMOT_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_NETMOT_NUM; id++)
		{
			netmot_drv_Config *CfgPtr = netmot_drv_LookupConfig(id);
			//-
			xdr_ins_u8(NETMOT_DRV_ip_ident_code_value);
			xdr_ins_u8(CfgPtr->DeviceId);
			//-
			u32 netmot_base = CfgPtr->BaseAddress;
			//-
			xdr_ins_u32(NETMOT_DRV_mRead_status(netmot_base));
		}
	}
	#endif


	// -- motors
	{
		u16 id;
		for(id = 0; id < SYS_MOTORFOC_NUM; id++)
		{
			//-
			xdr_ins_u8(MOTORFOC_ip_ident_code_value);
			xdr_ins_u8(id);
			//-
			u32 mfoc_base = motorfoc_LookupConfig(id)->BaseAddress;
			//-
			xdr_ins_u16(MOTORFOC_mRead_tsclocks(mfoc_base)   );
			//-
			xdr_ins_u32(MOTORFOC_mRead_motstatus(mfoc_base)   );
			//-
			xdr_ins_u24(MOTORFOC_mRead_val_iphs_a(mfoc_base)  );
			xdr_ins_u24(MOTORFOC_mRead_val_iphs_b(mfoc_base)  );
			xdr_ins_u24(MOTORFOC_mRead_val_iphs_c(mfoc_base)  );
			xdr_ins_u24(MOTORFOC_mRead_val_ibus_x(mfoc_base)  );
			xdr_ins_u24(MOTORFOC_mRead_val_vphs_a(mfoc_base)  );
			xdr_ins_u24(MOTORFOC_mRead_val_vphs_b(mfoc_base)  );
			xdr_ins_u24(MOTORFOC_mRead_val_vphs_c(mfoc_base)  );
			xdr_ins_u24(MOTORFOC_mRead_val_vbus_x(mfoc_base)  );
			xdr_ins_u24(MOTORFOC_mRead_val_vphs_n(mfoc_base)  );
			//-
			xdr_ins_u24(MOTORFOC_mRead_ismodval(mfoc_base)    );
			xdr_ins_u32(MOTORFOC_mRead_isangval(mfoc_base)    );
			//-
			xdr_ins_u24(MOTORFOC_mRead_vsmodval(mfoc_base)    );
			xdr_ins_u32(MOTORFOC_mRead_vsangval(mfoc_base)    );
			//-
			xdr_ins_u32(MOTORFOC_mRead_rot_angle(mfoc_base)   );
			//-
			xdr_ins_u32(MOTORFOC_mRead_spd_speed(mfoc_base)   );
			//-
			xdr_ins_u24(MOTORFOC_mRead_smo_es2_kflt(mfoc_base));
			xdr_ins_u32(MOTORFOC_mRead_smo_bemf_p(mfoc_base)  );
			xdr_ins_u24(MOTORFOC_mRead_smo_bemf_m(mfoc_base)  );
			xdr_ins_u32(MOTORFOC_mRead_smo_angle(mfoc_base)   );
			//-
			xdr_ins_u32(MOTORFOC_mRead_pos_position(mfoc_base));
			//-
			xdr_ins_u32(MOTORFOC_mRead_ext_angle(mfoc_base)  );
			//-
			xdr_ins_u32(MOTORFOC_mRead_rsv_angle(mfoc_base)  );
			//-
			xdr_ins_u32(MOTORFOC_mRead_hls_angle(mfoc_base)  );
			//-
			xdr_ins_u32(MOTORFOC_mRead_enc_index(mfoc_base)  );
			xdr_ins_u32(MOTORFOC_mRead_enc_phase(mfoc_base)  );
			xdr_ins_u32(MOTORFOC_mRead_enc_phcpt(mfoc_base)  );
			xdr_ins_u32(MOTORFOC_mRead_enc_angle(mfoc_base)  );
			//-
			xdr_ins_u32(MOTORFOC_mRead_mrt_spdout(mfoc_base) );
			xdr_ins_u32(MOTORFOC_mRead_mrt_angle(mfoc_base)  );
			//-
			xdr_ins_u24(MOTORFOC_mRead_pwm_mdval(mfoc_base)  );
			//-
			xdr_ins_u32(MOTORFOC_mRead_guswcnt(mfoc_base) );
			//-
			u8 ctrsrc = MOTORFOC_AP_IPDRVSOURCE_UDPIP;
			#if SW_ENA_ENHANCED > 0
			ctrsrc = g_motorfoc_ap_v[id].ipdrvsource;
			#endif
			xdr_ins_u8(ctrsrc);
		}
	}

	#if SYS_DS1820_NUM > 0
	{
		u8    idx;
		float temp;
		if (XST_SUCCESS == ds1820_ape_get_temp(&idx,&temp))
		{
			xdr_ins_u8(DS1820_ip_ident_code_value);
			//-
			xdr_ins_u8(idx);
			xdr_ins_r32(temp);
		}
	}
	#endif

	#if SYS_IVPLOGGER_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_IVPLOGGER_NUM; id++)
		{
			//-
			u32 ivp_base = ivplogger_LookupConfig(id)->BaseAddress;
			//-
			xdr_ins_u8(IVPLOGGER_ip_ident_code_value);
			//-
			xdr_ins_u8(id);
			//--
			xdr_ins_u8(IVPLOGGER_mRead_acq_ctrlst(ivp_base));
			xdr_ins_u32(IVPLOGGER_mRead_acq_level(ivp_base));
		}
	}
	#endif
	
	#if SYS_QXADCPS_NUM > 0
	{
		xdr_ins_u8(QXADCPS_ip_ident_code_value);
		//-
		xdr_ins_r32(g_qxadcps_oc_temp[0]);
	}
	#endif

}

//-

static void mca_eth_process_cgetdpr(void)
{
	u8  mot_id = xdr_ext_u8();
	u16 start  = xdr_ext_u16();

	u32 mfoc_base = motorfoc_LookupConfig(mot_id)->BaseAddress;
	mca_eth_reply_simple(XETHCTL_CODE_sGetDpr);

	xdr_ins_u8(mot_id);
	xdr_ins_u16(start);

	int i;
	for(i=0;i<256;i++)
	{
		u32 addr = start + i;
		u32 t32 = MOTORFOC_mRead_pwm_modulator_table(mfoc_base,addr << 2);
		xdr_ins_u24(t32);
	}
}

//-

static void mca_eth_process_csetdpr(void)
{
	MCA_ETH_V_DEF;

	if (!MCA_ETH_V_REF->imanager) return;

	u8  mot_id = xdr_ext_u8();
	u16 start  = xdr_ext_u16();

	//-
	u8 skip = 0;
	#if SW_ENA_ENHANCED > 0
	skip = g_motorfoc_ap_v[mot_id].ipdrvsource != MOTORFOC_AP_IPDRVSOURCE_UDPIP;
	#endif
	//-

	if (!skip)
	{
		u32 mfoc_base = motorfoc_LookupConfig(mot_id)->BaseAddress;
		int i;
		for(i=0;i<256;i++)
		{
			u32 t32 = xdr_ext_u24();
			u32 addr = start + i;
			MOTORFOC_mWrite_pwm_modulator_table(mfoc_base,addr << 2,t32);
		}
	}

	mca_eth_reply_simple(XETHCTL_CODE_sSetDpr);

	xdr_ins_u8(mot_id);
	xdr_ins_u16(start);
}

//-

#if SYS_IVPLOGGER_NUM > 0

static void mca_eth_process_cgetivpl(void)
{
	MCA_ETH_V_DEF;

	if (!MCA_ETH_V_REF->imanager) return;

	u8  ivp_id    = xdr_ext_u8();
	u32 offset    = xdr_ext_u32();
	u8  n_synmod  = xdr_ext_u8();
	u32 n_logsize = xdr_ext_u32();

	u32 ivp_base = ivplogger_LookupConfig(ivp_id)->BaseAddress;

	mca_eth_reply_simple(XETHCTL_CODE_sGetIvpL);

	IVPLOGGER_mWrite_acq_ctrlst(ivp_base,n_synmod & (IVPLOGGER_ACQ_CTRLST_SYNMOD1 | IVPLOGGER_ACQ_CTRLST_SYNMOD0));

	xdr_ins_u8(ivp_id);
	xdr_ins_u32(offset);
	xdr_ins_u8(IVPLOGGER_mRead_acq_ctrlst(ivp_base));
	u32 logsize = IVPLOGGER_mRead_mem_size(ivp_base);
	xdr_ins_u32(logsize);

	// get channel map and evaluate record size
	u16 channel_map = ivplogger_LookupConfig(ivp_id)->channel_map;
	u32 record_size = 0;
	{
		u16 t = channel_map;
		while(t != 0)
		{
			if ((t & 1) != 0)
				record_size += sizeof(s16);
			t >>= 1;
		}
	}

	// pnt to 1st record field
	s16 * pRFLD = (s16 *)(UINTPTR)(IVPLOGGER_mRead_mem_base(ivp_base) + offset);

	Xil_DCacheInvalidateRange((INTPTR)pRFLD,UTCPIP_UDP_DATA_MAX);

	u32 adrofs = offset;
	for(;;)
	{
		if ((adrofs + record_size) > logsize)
		{
			IVPLOGGER_mWrite_mem_size(ivp_base,n_logsize);
			IVPLOGGER_mWrite_acq_ctrlst(ivp_base,n_synmod | IVPLOGGER_ACQ_CTRLST_START);
			break;
		}

		// over Ethernet telegram length
		if ((MCA_ETH_V_REF->lB + record_size) > UTCPIP_UDP_DATA_MAX)
			break;

		// serialize record
		{
			u32 n;
			for(n = 0; n < record_size; n += sizeof(s16))
				xdr_ins_u16(*pRFLD++);
		}

		adrofs += record_size;
	}
}

#endif

//-

static void mca_eth_recv_process(utcpip_dev_t *ref)
{
	MCA_ETH_V_DEF;

	MCA_ETH_V_REF->pB = ref->ctx.xnctl.xnctl_data;
	MCA_ETH_V_REF->lB = ref->ctx.xnctl.xnctl_len;
	MCA_ETH_V_REF->erPRT = XST_SUCCESS;
	u8 proto_type = xdr_ext_u8();
	u8 proto_code = xdr_ext_u8();
	if (proto_type != XETHCTL_PROTO_TYPE)
	{
		MCA_ETH_V_REF->erPRT = XST_FAILURE;
	}
	if (MCA_ETH_V_REF->erPRT == XST_SUCCESS)
	{
		//----
		if (MCA_ETH_V_REF->managed == 0)
		{
			MCA_ETH_V_REF->owner_addr = ref->ctx.xnctl.xnctl_ip_address;
			MCA_ETH_V_REF->owner_port = ref->ctx.xnctl.xnctl_udp_sport;
			MCA_ETH_V_REF->managed    = 1;
		}
		MCA_ETH_V_REF->imanager = (MCA_ETH_V_REF->owner_addr.s_addr == ref->ctx.xnctl.xnctl_ip_address.s_addr) && (MCA_ETH_V_REF->owner_port == ref->ctx.xnctl.xnctl_udp_sport);
		if (MCA_ETH_V_REF->imanager)
		{
			u16 tmomsec = MCA_ETH_V_REF->linktmo;
			if (tmomsec < MCA_ETH_MANAGERTMO_MSEC_MIN)
				tmomsec = MCA_ETH_MANAGERTMO_MSEC_MIN;
			MCA_ETH_V_REF->tmng = MCA_ETH_V_REF->timer_msec;
			MCA_ETH_V_REF->tmomsec = tmomsec;
		}
		//----
		switch(proto_code)
		{
			case XETHCTL_CODE_cTerminate:
				{
					MCA_ETH_V_REF->task_status = MCA_ETH_TASK_STATUS_TERMINATE;
				}
				break;
			case XETHCTL_CODE_cRestart:
				{
					MCA_ETH_V_REF->task_status = MCA_ETH_TASK_STATUS_RESTART;
				}
				break;
			case XETHCTL_CODE_cNetSetup:
				{
					mca_eth_process_netsetup_tgm();
				}
				break;
			case XETHCTL_CODE_cIdent:
				{
					mca_eth_reply_ident_tgm();
				}
				break;

			case XETHCTL_CODE_cIpsSetup:
				{
					mca_eth_process_ipssetup();
				}
				break;

			case XETHCTL_CODE_cCtrl:
				{
					mca_eth_process_cctrl_tgm();
					mca_eth_reply_sstat_tgm();
				}
				break;

			case XETHCTL_CODE_cGetDpr:
				{
					mca_eth_process_cgetdpr();
				}
				break;

			case XETHCTL_CODE_cSetDpr:
				{
					mca_eth_process_csetdpr();
				}
				break;

			#if SYS_IVPLOGGER_NUM > 0
			case XETHCTL_CODE_cGetIvpL:
				{
					mca_eth_process_cgetivpl();
				}
				break;
			#endif

			case XETHCTL_CODE_cRpcEnvSetup:
				{
					u8 device_address = xdr_ext_u8();
					u8 phy_mode       = xdr_ext_u8();
					nvenviron_docmd(NVENVIRON_STATE_ERASE_START);
					while(XST_SUCCESS != nvenviron_exec_done())
						nvenviron_process();
					nvenviron_linkdev_address(device_address);
					nvenviron_linkdev_phymode(phy_mode);
					nvenviron_docmd(NVENVIRON_STATE_PROGRAM_START);
					mca_eth_reply_simple(XETHCTL_CODE_sRpcEnvSetup);
					MCA_ETH_V_REF->task_status = MCA_ETH_TASK_STATUS_RESTART;
				}
				break;

			case XETHCTL_CODE_cRpcEnvErase:
				{
					nvenviron_docmd(NVENVIRON_STATE_ERASE_START);
					while(XST_SUCCESS != nvenviron_exec_done())
						nvenviron_process();
					mca_eth_reply_simple(XETHCTL_CODE_sRpcEnvErase);
					MCA_ETH_V_REF->task_status = MCA_ETH_TASK_STATUS_RESTART;
				}
				break;

			default:
				{
					xil_printf("Unhandled code = %d\r\n",proto_code);
					MCA_ETH_V_REF->erPRT = XST_FAILURE;
				}
				break;
		}
	}

	if (MCA_ETH_V_REF->erPRT == XST_FAILURE)
		mca_eth_reply_simple(XETHCTL_CODE_xInvalid);

	ref->ctx.xnctl.xnctl_len = MCA_ETH_V_REF->lB;
}


static void mca_eth_runtime_checks(void)
{
	MCA_ETH_V_DEF;

	// watch dog link manager
	{
		if (MCA_ETH_V_REF->managed)
		{
			u32 d = MCA_ETH_V_REF->timer_msec - MCA_ETH_V_REF->tmng;
			if (d >= MCA_ETH_V_REF->tmomsec)
			{
				MCA_ETH_V_REF->managed = 0;
			}
		}
	}

	u8 noethcr = 0;

	QAP_API_RUNTIME_CHECKS();

	#if SYS_ETHERCAT_NUM > 0
	noethcr = 1;
	#endif

	// watch dog ethernet
	{
		u32 d = MCA_ETH_V_REF->timer_msec - MCA_ETH_V_REF->twdg;
		if (d >= MCA_ETH_V_REF->linktmo)
		{
			MCA_ETH_V_REF->twdg = MCA_ETH_V_REF->timer_msec - MCA_ETH_V_REF->linktmo;

			MCA_ETH_V_REF->eth_crash = 1;

			if (!noethcr)
			{
				if (MCA_ETH_V_REF->sys_up)
				{
					MCA_ETH_V_REF->sys_up = 0;
					motbrddv_system_down();
					print("0");
				}
				return;
			}
		}
		else
		{
			MCA_ETH_V_REF->eth_crash = 0;
		}
	}

	if (!MCA_ETH_V_REF->sys_up)
	{
		MCA_ETH_V_REF->sys_up = 1;
		motbrddv_system_enable();
		print("1");
	}
}

// counts milliseconds
static void mca_eth_timer_process(void)
{
	MCA_ETH_V_DEF;

	for(;;)
	{
		u32 d = xx_timer_cnt_val() - MCA_ETH_V_REF->timer_tic_old_msec;
		if (d < MCA_ETH_V_REF->timer_tic_per_msec)
			break;
		MCA_ETH_V_REF->timer_tic_old_msec += MCA_ETH_V_REF->timer_tic_per_msec;
		MCA_ETH_V_REF->timer_msec += 1;

		// live dot
		if (0 == (MCA_ETH_V_REF->timer_msec % 1000)) print(".");
	}
}

static void mca_eth_setup_show(void)
{
	xil_printf("DEV: %d\r\n",g_link_device);

	xil_printf("MAC: %02X.%02X.%02X-%02X.%02X.%02X",
			g_eth_c_def_mac_address[0],
			g_eth_c_def_mac_address[1],
			g_eth_c_def_mac_address[2],
			g_eth_c_def_mac_address[3],
			g_eth_c_def_mac_address[4],
			g_eth_c_def_mac_address[5]);

	xil_printf(" IP: %d.%d.%d.%d\r\n",
			g_eth_c_def_ip_address[0],
			g_eth_c_def_ip_address[1],
			g_eth_c_def_ip_address[2],
			g_eth_c_def_ip_address[3]);
}

static int mca_eth_mot_owner(void)
{
	#if SYS_ETHERCAT_NUM > 0
	MCA_ETH_V_DEF;
	esc_env_t * pE = & g_esc_env_v;
	u16 id;
	for(id = 0; id < SYS_MOTORFOC_NUM && id < ETHERCAT_MOT_NUM; id++)
	{
		motorfoc *pM = &g_motorfoc_v[id];

		u8 owndef = MOTORFOC_AP_IPDRVSOURCE_UDPIP;

		// parameters shall ok
		if (pM->ApPtr->prm.sys.ts > 0.0f)
		{
			// EtherCAT shall be operational
			if (pE->Last_AL_Control == ETHERCAT_ESC_AL_ST_MACH_OPER)
			{
				// ownership shall ON
				if ((pE->ctl_v.motor[id].ctl & ETHERCAT_MOT_CTL_CTL_OWN_MSK) == ETHERCAT_MOT_CTL_CTL_OWN_REQ)
				{
					// power request shall OFF
					if ((pE->ctl_v.motor[id].ctl & ETHERCAT_MOT_CTL_CTL_PWR_MSK) == ETHERCAT_MOT_CTL_CTL_PWR_OFF)
					{
						int power;
						motorfoc_ap_ctl_get_power(pM,&power);
						if (power == 0)
						{
							owndef = MOTORFOC_AP_IPDRVSOURCE_LOCAL;
						}
					}
					else
					{
						owndef = pM->ApPtr->ipdrvsource;
					}
				}
			}
		}

		// set request
		pM->ApPtr->ipdrvsource = owndef;


		// if crash...
		if ((pM->ApPtr->ipdrvsource == MOTORFOC_AP_IPDRVSOURCE_UDPIP && MCA_ETH_V_REF->eth_crash) ||
			(pM->ApPtr->ipdrvsource == MOTORFOC_AP_IPDRVSOURCE_LOCAL && pE->Last_AL_Control != ETHERCAT_ESC_AL_ST_MACH_OPER))
		{
			motorfoc_Down(id);
		}


	}
	#endif

	QAP_API_MOT_OWNER();

	return XST_SUCCESS;
}

static void mca_eth_init(void)
{
	MCA_ETH_V_DEF;

	MCA_ETH_V_REF->task_status        = MCA_ETH_TASK_STATUS_RUNNING;

	MCA_ETH_V_REF->timer_tic_per_sec  = xx_timer_cnt_pser_sec();
	MCA_ETH_V_REF->timer_tic_per_msec = MCA_ETH_V_REF->timer_tic_per_sec / 1000;

	MCA_ETH_V_REF->timer_tic_old_msec = xx_timer_cnt_val();
	MCA_ETH_V_REF->timer_msec         = 0;

	MCA_ETH_V_REF->linktmo            = 1000;
	MCA_ETH_V_REF->twdg               = MCA_ETH_V_REF->timer_msec - MCA_ETH_V_REF->linktmo;

	MCA_ETH_V_REF->eth_crash          = 1;
	MCA_ETH_V_REF->sys_up             = 0;

	//-

	MCA_ETH_V_REF->managed            = 0;
	MCA_ETH_V_REF->tnetlive           = MCA_ETH_V_REF->timer_msec;

	#ifdef SW_SELF_RESTART_AGENT
	MCA_ETH_V_REF->timer_self_restart = MCA_ETH_V_REF->timer_msec;
	#endif
}

int mca_eth_agent(void)
{
	MCA_ETH_V_DEF;
	int rc = XST_SUCCESS;
	u32 loop;

	print("\r\nStart X-Net CTL agent\r\n");

	// off motors
	motbrddv_system_down();

	// env initialize
	mca_eth_init();

	// stop ETH
	xx_ethmac_down();

	// set mac address
	xx_ethmac_set_mac_addr(g_eth_c_def_mac_address);

	//- display
	mca_eth_setup_show();

	// init ETH
	xx_ethmac_init();


	QAP_API_INITIALIZATION();

	for(loop = 0;MCA_ETH_V_REF->task_status == MCA_ETH_TASK_STATUS_RUNNING;loop++)
	{
		// msec timer eval
		mca_eth_timer_process();

		#if SYS_ETHERCAT_NUM > 0
		ethercat_proc();
		#endif

		#if SYS_DS1820_NUM > 0
		ds1820_ape_proc();
		#endif

		#if SYS_QXADCPS_NUM > 0
		qxadcps_ape_proc();
		#endif

		#if SW_ENA_NVRAM_ENV > 0
		nvenviron_process();
		#endif

		mca_eth_mot_owner();

		// checks
		mca_eth_runtime_checks();

		#ifdef SW_SELF_RESTART_AGENT
		if ((u32)(MCA_ETH_V_REF->timer_msec - MCA_ETH_V_REF->timer_self_restart) > LINKDEVICE_SPAWN_TIME_MS)
			if (g_link_phymode != LINK_PHYMODE_ETH)
				MCA_ETH_V_REF->task_status = MCA_ETH_TASK_STATUS_RESTART;
		#endif

		int key = con_tkey();
		if (27 == key)
			MCA_ETH_V_REF->task_status = MCA_ETH_TASK_STATUS_TERMINATE;

		QAP_API_SCHEDULATIONS();

		{
			u32 d = MCA_ETH_V_REF->timer_msec - MCA_ETH_V_REF->tnetlive;
			if (d >= MCA_ETH_ALIVE_TIME_MSEC)
			{
				MCA_ETH_V_REF->tnetlive = MCA_ETH_V_REF->timer_msec;
				//print("^");
				xx_ethmac_relinq_speed();
			}
		}
		{
			u32 rrecv = sizeof(g_eth_trx_buffer);
			u8 rok;
			xx_ethmac_recv_tgm(g_eth_trx_buffer,&rrecv,&rok);
			if (rok && rrecv > 0)
			{
				// reset wdt eth
				MCA_ETH_V_REF->tnetlive = MCA_ETH_V_REF->timer_msec;

				// confirm mac address from MAC
				xx_ethmac_get_mac_addr(g_eth_c_def_mac_address);

				// process received eth frame

				MCA_ETH_V_REF->udpip.mac_address.b.b1 = g_eth_c_def_mac_address[0];
				MCA_ETH_V_REF->udpip.mac_address.b.b2 = g_eth_c_def_mac_address[1];
				MCA_ETH_V_REF->udpip.mac_address.b.b3 = g_eth_c_def_mac_address[2];
				MCA_ETH_V_REF->udpip.mac_address.b.b4 = g_eth_c_def_mac_address[3];
				MCA_ETH_V_REF->udpip.mac_address.b.b5 = g_eth_c_def_mac_address[4];
				MCA_ETH_V_REF->udpip.mac_address.b.b6 = g_eth_c_def_mac_address[5];

				MCA_ETH_V_REF->udpip.ip_address.b.b1  = g_eth_c_def_ip_address[0];
				MCA_ETH_V_REF->udpip.ip_address.b.b2  = g_eth_c_def_ip_address[1];
				MCA_ETH_V_REF->udpip.ip_address.b.b3  = g_eth_c_def_ip_address[2];
				MCA_ETH_V_REF->udpip.ip_address.b.b4  = g_eth_c_def_ip_address[3];

				MCA_ETH_V_REF->udpip.ctx.xnctl.xnctl_udp_dport = XETHCTL_UDP_PORT;

				MCA_ETH_V_REF->udpip.trx_msg_pnt       = &g_eth_trx_buffer[0];
				MCA_ETH_V_REF->udpip.trx_msg_len       = rrecv;

				MCA_ETH_V_REF->udpip.verbose          = 0;

				utcpip_client(&MCA_ETH_V_REF->udpip);

				switch(MCA_ETH_V_REF->udpip.decoded)
				{
					case UTCPIP_RECEIVED_ARP_REQUEST:
						{
							utcpip_build_arp_reply(&MCA_ETH_V_REF->udpip);
						}
						break;
					case UTCPIP_RECEIVED_ICMP_ECHO_REQUEST:
						{
							utcpip_build_icmp_echo_reply(&MCA_ETH_V_REF->udpip);
						}
						break;
					case UTCPIP_RECEIVED_XNCTL_REQUEST:
						{
							#ifdef SW_SELF_RESTART_AGENT
							MCA_ETH_V_REF->timer_self_restart = MCA_ETH_V_REF->timer_msec;
							#endif

							u8 xdn = 0;

							#ifdef XPAR_XQSPIPS_0_DEVICE_ID
							zq_qspips_eth_process(&g_zq_qspips,MCA_ETH_V_REF->udpip.ctx.xnctl.xnctl_data,&MCA_ETH_V_REF->udpip.ctx.xnctl.xnctl_len,&xdn);
							#endif

							if (!xdn)
							{
								QAP_API_UDPIP_PROTOCOL_FILTER();
							}

							if (!xdn)
								mca_eth_recv_process(&MCA_ETH_V_REF->udpip);
							if (MCA_ETH_V_REF->udpip.ctx.xnctl.xnctl_len > 0)
								utcpip_build_xnctl_reply(&MCA_ETH_V_REF->udpip);
							else
								MCA_ETH_V_REF->udpip.trx_msg_len = 0;
						}
						break;
					default:
						{
							MCA_ETH_V_REF->udpip.trx_msg_len = 0;
						}
						break;
				}

				if (MCA_ETH_V_REF->udpip.trx_msg_len > 0)
				{
					int ntry;
					for(ntry = 1; ntry <= 2; ntry++)
					{
						u8 tok;
						xx_ethmac_send_tgm(MCA_ETH_V_REF->udpip.trx_msg_pnt,MCA_ETH_V_REF->udpip.trx_msg_len,&tok);
						if (tok) break;
						xil_printf("\r\nzq_ethmac_send_tgm() [%d] Error\r\n",ntry);
					}
				}
			}
		}

	}

	// off motors
	motbrddv_system_down();

	// eoj
	return rc;
}

int mca_eth_task_status(void)
{
	MCA_ETH_V_DEF;
	return MCA_ETH_V_REF->task_status;
}

#endif
