/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief application interactive
/// \file ethercat.h
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#ifndef ETHERCAT_H_
#define ETHERCAT_H_


#define ETHERCAT_MOT_NUM 4

typedef struct
{
	u16   ctl;

#define ETHERCAT_MOT_CTL_CTL_ROT_MAN  ( 0 << 0)
#define ETHERCAT_MOT_CTL_CTL_ROT_MRT  ( 1 << 0)
#define ETHERCAT_MOT_CTL_CTL_ROT_ENC  ( 2 << 0)
#define ETHERCAT_MOT_CTL_CTL_ROT_SMO  ( 3 << 0)
#define ETHERCAT_MOT_CTL_CTL_ROT_HLS  ( 4 << 0)
#define ETHERCAT_MOT_CTL_CTL_ROT_RSV  ( 5 << 0)
#define ETHERCAT_MOT_CTL_CTL_ROT_EXT  ( 6 << 0)
#define ETHERCAT_MOT_CTL_CTL_ROT_MSK  (15 << 0)

#define ETHERCAT_MOT_CTL_CTL_CTM_TRQ  (0 <<  4)
#define ETHERCAT_MOT_CTL_CTL_CTM_SPD  (1 <<  4)
#define ETHERCAT_MOT_CTL_CTL_CTM_POS  (2 <<  4)
#define ETHERCAT_MOT_CTL_CTL_CTM_MSK  (3 <<  4)

#define ETHERCAT_MOT_CTL_CTL_MOD_PWM  (0 <<  8)
#define ETHERCAT_MOT_CTL_CTL_MOD_RPFM (1 <<  8)
#define ETHERCAT_MOT_CTL_CTL_MOD_MSK  (3 <<  8)

#define ETHERCAT_MOT_CTL_CTL_LVM_2    (0 << 10)
#define ETHERCAT_MOT_CTL_CTL_LVM_3    (1 << 10)
#define ETHERCAT_MOT_CTL_CTL_LVM_MSK  (3 << 10)

#define ETHERCAT_MOT_CTL_CTL_PWR_OFF  (0 << 12)
#define ETHERCAT_MOT_CTL_CTL_PWR_ON   (1 << 12)
#define ETHERCAT_MOT_CTL_CTL_PWR_MSK  (1 << 12)

#define ETHERCAT_MOT_CTL_CTL_OWN_REL  (0 << 13)
#define ETHERCAT_MOT_CTL_CTL_OWN_REQ  (1 << 13)
#define ETHERCAT_MOT_CTL_CTL_OWN_MSK  (1 << 13)

#define ETHERCAT_MOT_CTL_CTL_FLX_DRV  (0 << 14)
#define ETHERCAT_MOT_CTL_CTL_FLX_SET  (1 << 14)
#define ETHERCAT_MOT_CTL_CTL_FLX_MSK  (1 << 14)

	float current;
	float speed;
	float position;

} eact_mot_ctl_t;

typedef struct
{
	u16   sts;

#define ETHERCAT_MOT_STS_STS_ROT_MAN  ( 0 << 0)
#define ETHERCAT_MOT_STS_STS_ROT_MRT  ( 1 << 0)
#define ETHERCAT_MOT_STS_STS_ROT_ENC  ( 2 << 0)
#define ETHERCAT_MOT_STS_STS_ROT_SMO  ( 3 << 0)
#define ETHERCAT_MOT_STS_STS_ROT_HLS  ( 4 << 0)
#define ETHERCAT_MOT_STS_STS_ROT_RSV  ( 5 << 0)
#define ETHERCAT_MOT_STS_STS_ROT_EXT  ( 6 << 0)
#define ETHERCAT_MOT_STS_STS_ROT_MSK  (15 << 0)

#define ETHERCAT_MOT_STS_STS_CTM_TRQ  (0 <<  4)
#define ETHERCAT_MOT_STS_STS_CTM_SPD  (1 <<  4)
#define ETHERCAT_MOT_STS_STS_CTM_POS  (2 <<  4)
#define ETHERCAT_MOT_STS_STS_CTM_MSK  (3 <<  4)

#define ETHERCAT_MOT_STS_STS_MOD_PWM  (0 <<  8)
#define ETHERCAT_MOT_STS_STS_MOD_RPFM (1 <<  8)
#define ETHERCAT_MOT_STS_STS_MOD_MSK  (3 <<  8)

#define ETHERCAT_MOT_STS_STS_LVM_2    (0 << 10)
#define ETHERCAT_MOT_STS_STS_LVM_3    (1 << 10)
#define ETHERCAT_MOT_STS_STS_LVM_MSK  (3 << 10)

#define ETHERCAT_MOT_STS_STS_PWR_OFF  (0 << 12)
#define ETHERCAT_MOT_STS_STS_PWR_ON   (1 << 12)
#define ETHERCAT_MOT_STS_STS_PWR_MSK  (1 << 12)

#define ETHERCAT_MOT_STS_STS_OWN_NOT  (0 << 13)
#define ETHERCAT_MOT_STS_STS_OWN_ACK  (1 << 13)
#define ETHERCAT_MOT_STS_STS_OWN_MSK  (1 << 13)

#define ETHERCAT_MOT_STS_STS_FAIL_OFF (0 << 15)
#define ETHERCAT_MOT_STS_STS_FAIL_ON  (1 << 15)
#define ETHERCAT_MOT_STS_STS_FAIL_MSK (1 << 15)

	float current;
	float speed;
	float position;
	float modindex;
	float vbus;

} ecat_mot_sts_t;

typedef struct
{
	eact_mot_ctl_t motor[ETHERCAT_MOT_NUM];

} esc_apl_ctl_t;

typedef struct
{
	ecat_mot_sts_t motor[ETHERCAT_MOT_NUM];

	float octemp;

} esc_apl_sts_t;


typedef struct
{

	esc_apl_ctl_t ctl_v;
	esc_apl_ctl_t ctl_o;

	esc_apl_sts_t sts_v;

	u8 Last_AL_Control;
	#define ETHERCAT_ESC_AL_ST_MACH_INIT  1
	#define ETHERCAT_ESC_AL_ST_MACH_BOOT  3
	#define ETHERCAT_ESC_AL_ST_MACH_PREOP 2
	#define ETHERCAT_ESC_AL_ST_MACH_SAFE  4
	#define ETHERCAT_ESC_AL_ST_MACH_OPER  8
	#define ETHERCAT_ESC_AL_ST_MACH_INVALID 0xFF

	u8  warm;

} esc_env_t;


int ethercat_init(void);
int ethercat_e2prdy(void);
int ethercat_disp(void);
int ethercat_proc(void);

#endif /* ETHERCAT_H_ */
