/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief global variables
/// \file glovar.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "glovar.h"

int           g_init_error_line;

u8            g_link_device;
u8            g_link_phymode;

u8            g_board_id;

u8            g_eth_c_def_mac_address[6];
u8            g_eth_c_def_ip_address [4];

u8            g_eth_trx_buffer[ETH_TRX_TGM_SIZE] __attribute__ ((aligned(4)));

#ifdef XPAR_XQSPIPS_0_DEVICE_ID
	zq_qspips_t   g_zq_qspips;
#endif

#if SYS_MOTORFOC_NUM > 0
	motorfoc      g_motorfoc_v   [SYS_MOTORFOC_NUM];
	#if SW_ENA_ENHANCED > 0
	motorfoc_ap_t g_motorfoc_ap_v[SYS_MOTORFOC_NUM];
	#endif
#endif

#if SYS_MPHY18_NUM > 0
	mphy18_drv g_mphy18_drv_v[SYS_MPHY18_NUM];
	#if SW_ENA_ENHANCED > 0
	mphy18_drv_ap_t g_mphy18_drv_ap_v[SYS_MPHY18_NUM];
	#endif
#endif

#if SYS_MPHY17_NUM > 0
	mphy17_drv g_mphy17_drv_v[SYS_MPHY17_NUM];
	#if SW_ENA_ENHANCED > 0
	mphy17_drv_ap_t g_mphy17_drv_ap_v[SYS_MPHY17_NUM];
	#endif
#endif

#if SYS_MPHY16_NUM > 0
	mphy16_drv g_mphy16_drv_v[SYS_MPHY16_NUM];
	#if SW_ENA_ENHANCED > 0
	mphy16_drv_ap_t g_mphy16_drv_ap_v[SYS_MPHY16_NUM];
	#endif
#endif

#if SYS_TEP2_NUM > 0
	tep2_drv g_tep2_drv_v[SYS_TEP2_NUM];
#endif

#if SYS_TEC53_NUM > 0
	tec53_drv g_tec53_drv_v[SYS_TEC53_NUM];
#endif

#if SYS_S4SICINV_NUM > 0
	s4sicinv_drv g_s4sicinv_drv_v[SYS_S4SICINV_NUM];
	#if SW_ENA_ENHANCED > 0
	s4sicinv_drv_ap_t g_s4sicinv_drv_ap_v[SYS_S4SICINV_NUM];
	#endif
#endif

#if SYS_QINVSIC_NUM > 0
	qinvsic_drv g_qinvsic_drv_v[SYS_QINVSIC_NUM];
	#if SW_ENA_ENHANCED > 0
	qinvsic_drv_ap_t g_qinvsic_drv_ap_v[SYS_QINVSIC_NUM];
	#endif
#endif

#if SYS_MPHY11_NUM > 0
	mphy11_drv g_mphy11_drv_v[SYS_MPHY11_NUM];
	#if SW_ENA_ENHANCED > 0
	mphy11_drv_ap_t g_mphy11_drv_ap_v[SYS_MPHY11_NUM];
	#endif
#endif

#if SYS_ZYGUSIC_NUM > 0
	zygusic_drv g_zygusic_drv_v[SYS_ZYGUSIC_NUM];
#endif

#if SYS_TLIMOT_NUM > 0
	tlimot_drv g_tlimot_drv_v[SYS_TLIMOT_NUM];
#endif

#if SYS_MLIEVB_NUM > 0
	mlievb_drv g_mlievb_drv_v[SYS_MLIEVB_NUM];
#endif

#if SYS_MOTCON2_NUM > 0
	motcon2_drv g_motcon2_drv_v[SYS_MOTCON2_NUM];
#endif

#if SYS_MOTCON1_NUM > 0
	motcon1_drv g_motcon1_drv_v[SYS_MOTCON1_NUM];
#endif

#if SYS_SGXBCU_NUM > 0
	sgxbcu_drv g_sgxbcu_drv_v[SYS_SGXBCU_NUM];
#endif

#if SYS_NETMOT_NUM > 0
	netmot_drv g_netmot_drv_v[SYS_NETMOT_NUM];
#endif

#if SYS_FMCMC1_NUM > 0
	fmcmc1_drv      g_fmcmc1_drv_v   [SYS_FMCMC1_NUM];
	#if SW_ENA_ENHANCED > 0
	fmcmc1_drv_ap_t g_fmcmc1_drv_ap_v[SYS_FMCMC1_NUM];
	#endif
#endif


#if SYS_ETHERCAT_NUM > 0
	esc_env_t g_esc_env_v;
#endif

#if SYS_DS1820_NUM > 0
	ds1820       g_ds1820_v    [SYS_DS1820_NUM];
	ds1820ape_t  g_ds1820ape_v [SYS_DS1820_NUM];
	ds1820_env_t g_ds1820_env_v;
#endif

#if SYS_IVPLOGGER_NUM > 0
	ivplogger    g_ivplogger_v[SYS_IVPLOGGER_NUM];
#endif

#if SYS_QXADCPS_NUM > 0
	float        g_qxadcps_oc_temp[SYS_QXADCPS_NUM];
#endif

#if SW_ENA_NVRAM_ENV > 0
	nvenviron_t  g_nvenviron;
#endif

//-
