/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief global variables
/// \file glovar.h
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020


#ifndef GLOVAR_H_
#define GLOVAR_H_

#include <stdio.h>
#include "xparameters.h"
#include "xil_cache.h"

#ifdef XPAR_PS7_XADC_0_DEVICE_ID
#include "xadcps.h"
#endif

#ifdef XPAR_XQSPIPS_0_DEVICE_ID
#include "xqspips.h"
#include "zq_qspips.h"
#endif

#if defined(__MICROBLAZE__)
#endif

#if defined (__arm__) || defined (__aarch64__)
#include "xil_printf.h"
#endif

#include "board.h"

#define SW_ENA_PL_INIT 0  // ZYNQ PL Init from QSPI bitstrem partition
#define SW_ENA_OCM_RAM 0  // ZYNQ OCM only (no DDR)

#if defined (__arm__) || defined (__aarch64__) && SYS_MOTORFOC_NUM > 0
	#define SW_ENA_ENHANCED 1
#else
	#define SW_ENA_ENHANCED 0
#endif

#define SW_ENA_ETH_AGENT 1
#define SW_ENA_CAN_AGENT 1

#if ! defined(XPAR_PS7_CAN_0_DEVICE_ID)
#undef  SW_ENA_CAN_AGENT
#define SW_ENA_CAN_AGENT 0
#endif

#if (SW_ENA_CAN_AGENT > 0) && (SW_ENA_ETH_AGENT > 0)
	#define SW_SELF_RESTART_AGENT
#endif

#define SW_ENA_NVRAM_ENV 1

#include "conutil.h"
#include "xx_timer.h"
#include "xx_ethmac.h"
#include "xx_xcan.h"
#include "xx_iic.h"

#if SYS_QXADCPS_NUM > 0
#include "qxadcps_ape.h"
#endif

#if SYS_UART4BUS_NUM > 0
	#include "uart4bus.h"
	#include "uart4bus_apd.h"
#endif

#if SYS_MOTORFOC_NUM > 0
#include "motorfoc.h"
#include "motorfoc_apc.h"
#include "motorfoc_apd.h"
#include "motorfoc_utl.h"
#endif

#if SYS_MPHY18_NUM > 0
#include "mphy18_drv.h"
#include "mphy18_drv_apc.h"
#include "mphy18_drv_apd.h"
#endif

#if SYS_MPHY17_NUM > 0
#include "mphy17_drv.h"
#include "mphy17_drv_apc.h"
#include "mphy17_drv_apd.h"
#endif

#if SYS_MPHY16_NUM > 0
#include "mphy16_drv.h"
#include "mphy16_drv_apc.h"
#include "mphy16_drv_apd.h"
#endif

#if SYS_TEP2_NUM > 0
#include "tep2_drv.h"
#endif

#if SYS_TEC53_NUM > 0
#include "tec53_drv.h"
#endif

#if SYS_S4SICINV_NUM > 0
#include "s4sicinv_drv.h"
#include "s4sicinv_drv_apc.h"
#include "s4sicinv_drv_apd.h"
#include "s4sicinv_drv_utl.h"
#endif

#if SYS_QINVSIC_NUM > 0
#include "qinvsic_drv.h"
#include "qinvsic_drv_apc.h"
#include "qinvsic_drv_apd.h"
#endif

#if SYS_MPHY11_NUM > 0
#include "mphy11_drv.h"
#include "mphy11_drv_apc.h"
#include "mphy11_drv_apd.h"
#endif

#if SYS_ZYGUSIC_NUM > 0
#include "zygusic_drv.h"
#endif

#if SYS_TLIMOT_NUM > 0
#include "tlimot_drv.h"
#include "tlimot_drv_apc.h"
#endif

#if SYS_MLIEVB_NUM > 0
#include "mlievb_drv.h"
#include "mlievb_drv_apc.h"
#endif

#if SYS_MOTCON2_NUM > 0
#include "motcon2_drv.h"
#endif

#if SYS_MOTCON1_NUM > 0
#include "motcon1_drv.h"
#endif

#if SYS_SGXBCU_NUM > 0
#include "sgxbcu_drv.h"
#endif

#if SYS_NETMOT_NUM > 0
#include "netmot_drv.h"
#include "netmot_drv_apc.h"
#endif

#if SYS_FMCMC1_NUM > 0
#include "fmcmc1_drv.h"
#include "fmcmc1_drv_apc.h"
#include "fmcmc1_drv_apd.h"
#endif

#if SYS_ETHERCAT_NUM > 0
#include "ethercat.h"
#endif

#if SYS_DS1820_NUM > 0
#include "ds1820.h"
#include "ds1820_apc.h"
#include "ds1820_apd.h"
#include "ds1820ape.h"
#endif

#if SYS_IVPLOGGER_NUM > 0
#include "ivplogger.h"
#include "ivploggerape.h"
#endif

#include "mcaident.h"
#include "conutil.h"
#include "motbrddv.h"
#include "utcpip.h"
#include "ethagent.h"

#if (SW_ENA_CAN_AGENT > 0)
	#include "canagent.h"
#endif

#include "interact.h"

//
// application embedded code
//

#if 0

	#include "qap_header.h"

#else

	#define QAP_APPLICATION_BANNER ""

	#define QAP_API_RUNTIME_CHECKS()
	#define QAP_API_MOT_OWNER()
	#define QAP_API_INITIALIZATION()
	#define QAP_API_SCHEDULATIONS()
	#define QAP_API_UDPIP_PROTOCOL_FILTER()

#endif

// environment data
#include "nvenviron.h"


// include of test programs
#if 0

	#if SYS_MPHY18_NUM > 0
		#include "mphy18_test.h"
	#endif
	#if SYS_MPHY17_NUM > 0
		#include "mphy17_test.h"
	#endif
	#if SYS_MPHY16_NUM > 0
		#include "mphy16_test.h"
	#endif
	#if SYS_MPHY11_NUM > 0
		#include "mphy11_test.h"
	#endif
	#if SYS_S4SICINV_NUM > 0
		#include "s4sicinv_test.h"
	#endif

#endif

#define ETH_TRX_TGM_SIZE 1536

extern int           g_init_error_line;

extern u8            g_link_device;
extern u8            g_link_phymode;

extern u8            g_board_id;

extern u8            g_eth_c_def_mac_address[6];
extern u8            g_eth_c_def_ip_address [4];

extern u8            g_eth_trx_buffer[ETH_TRX_TGM_SIZE];

#ifdef XPAR_XQSPIPS_0_DEVICE_ID
	extern zq_qspips_t   g_zq_qspips;
#endif


#if SYS_MOTORFOC_NUM > 0
	extern motorfoc      g_motorfoc_v   [SYS_MOTORFOC_NUM];
	#if SW_ENA_ENHANCED > 0
	extern motorfoc_ap_t g_motorfoc_ap_v[SYS_MOTORFOC_NUM];
	#endif
#endif

#if SYS_MPHY18_NUM > 0
	extern mphy18_drv g_mphy18_drv_v[SYS_MPHY18_NUM];
	#if SW_ENA_ENHANCED > 0
	extern mphy18_drv_ap_t g_mphy18_drv_ap_v[SYS_MPHY18_NUM];
	#endif
#endif

#if SYS_MPHY17_NUM > 0
	extern mphy17_drv g_mphy17_drv_v[SYS_MPHY17_NUM];
	#if SW_ENA_ENHANCED > 0
	extern mphy17_drv_ap_t g_mphy17_drv_ap_v[SYS_MPHY17_NUM];
	#endif
#endif

#if SYS_MPHY16_NUM > 0
	extern mphy16_drv g_mphy16_drv_v[SYS_MPHY16_NUM];
	#if SW_ENA_ENHANCED > 0
	extern mphy16_drv_ap_t g_mphy16_drv_ap_v[SYS_MPHY16_NUM];
	#endif
#endif

#if SYS_TEP2_NUM > 0
	extern tep2_drv g_tep2_drv_v[SYS_TEP2_NUM];
#endif

#if SYS_TEC53_NUM > 0
	extern tec53_drv g_tec53_drv_v[SYS_TEC53_NUM];
#endif

#if SYS_S4SICINV_NUM > 0
	extern s4sicinv_drv g_s4sicinv_drv_v[SYS_S4SICINV_NUM];
	#if SW_ENA_ENHANCED > 0
	extern s4sicinv_drv_ap_t g_s4sicinv_drv_ap_v[SYS_S4SICINV_NUM];
	#endif
#endif

#if SYS_QINVSIC_NUM > 0
	extern qinvsic_drv g_qinvsic_drv_v[SYS_QINVSIC_NUM];
	#if SW_ENA_ENHANCED > 0
	extern qinvsic_drv_ap_t g_qinvsic_drv_ap_v[SYS_QINVSIC_NUM];
	#endif
#endif

#if SYS_MPHY11_NUM > 0
	extern mphy11_drv g_mphy11_drv_v[SYS_MPHY11_NUM];
	#if SW_ENA_ENHANCED > 0
	extern mphy11_drv_ap_t g_mphy11_drv_ap_v[SYS_MPHY11_NUM];
	#endif
#endif

#if SYS_ZYGUSIC_NUM > 0
	extern zygusic_drv g_zygusic_drv_v[SYS_ZYGUSIC_NUM];
#endif

#if SYS_TLIMOT_NUM > 0
	extern tlimot_drv g_tlimot_drv_v[SYS_TLIMOT_NUM];
#endif

#if SYS_MLIEVB_NUM > 0
	extern mlievb_drv g_mlievb_drv_v[SYS_MLIEVB_NUM];
#endif

#if SYS_MOTCON2_NUM > 0
	extern motcon2_drv g_motcon2_drv_v[SYS_MOTCON2_NUM];
#endif

#if SYS_MOTCON1_NUM > 0
	extern motcon1_drv g_motcon1_drv_v[SYS_MOTCON1_NUM];
#endif

#if SYS_SGXBCU_NUM > 0
	extern sgxbcu_drv g_sgxbcu_drv_v[SYS_SGXBCU_NUM];
#endif

#if SYS_NETMOT_NUM > 0
	extern netmot_drv g_netmot_drv_v[SYS_NETMOT_NUM];
#endif

#if SYS_FMCMC1_NUM > 0
	extern fmcmc1_drv      g_fmcmc1_drv_v   [SYS_FMCMC1_NUM];
	#if SW_ENA_ENHANCED > 0
	extern fmcmc1_drv_ap_t g_fmcmc1_drv_ap_v[SYS_FMCMC1_NUM];
	#endif
#endif


#if SYS_ETHERCAT_NUM > 0
	extern esc_env_t g_esc_env_v;
#endif

#if SYS_DS1820_NUM > 0
	extern ds1820       g_ds1820_v    [SYS_DS1820_NUM];
	extern ds1820ape_t  g_ds1820ape_v [SYS_DS1820_NUM];
	extern ds1820_env_t g_ds1820_env_v;
#endif

#if SYS_IVPLOGGER_NUM > 0
	extern ivplogger    g_ivplogger_v[SYS_IVPLOGGER_NUM];
#endif

#if SYS_QXADCPS_NUM > 0
	extern float        g_qxadcps_oc_temp[SYS_QXADCPS_NUM];
#endif

#if SW_ENA_NVRAM_ENV > 0
	extern nvenviron_t  g_nvenviron;
#endif

//-

#endif /*GLOVAR_H_*/
