/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief application interactive task
/// \file interact.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "glovar.h"

typedef struct
{
	const char *pdesc;
	int (*pfun)(void);
} menudef_t;


#if SYS_TOTAL_FMC_BOARDS == 2

static int motctl_boardsel(void)
{
	int code;
	print("Enter board ID(0/1): ");
	int key = con_guint(1,&code,0);
	if (13 == key)
	{
		if (code == 0 || code == 1)
			g_board_id = code;
	}
	return XST_SUCCESS;
}

#endif

#if SYS_NETMOT_NUM > 0
static int netmot_fpga_dna_ident_show(void)
{
	if (g_board_id < SYS_NETMOT_NUM)
		netmot_drv_ap_fpga_dna_show(&g_netmot_drv_v[g_board_id]);
	return XST_SUCCESS;
}
#endif

#if SYS_MLIEVB_NUM > 0
static int mlievb_fpga_dna_ident_show(void)
{
	if (g_board_id < SYS_MLIEVB_NUM)
		mlievb_drv_ap_fpga_dna_show(&g_mlievb_drv_v[g_board_id]);
	return XST_SUCCESS;
}
#endif


static const menudef_t menulist[] =
{
	#if SYS_TOTAL_FMC_BOARDS == 2
	{"Enter Board ID(0/1)",motctl_boardsel},
	#endif

	#if SYS_MLIEVB_NUM > 0
	{"DNA Ident mlievb",mlievb_fpga_dna_ident_show},
	#endif
	
	#if SYS_NETMOT_NUM > 0
	{"DNA Ident netmot",netmot_fpga_dna_ident_show},
	#endif

	#if SYS_NETMOT_NUM > 0
	{"FPGA Program netmot",netmot_drv_ap_fpga_setup_4_program},
	#endif

	#if SW_ENA_ETH_AGENT > 0
	{"X-Net CTRL",mca_eth_agent},
	#endif

	#if SW_ENA_CAN_AGENT > 0
	{"X-CAN CTRL",mca_can_agent},
	#endif

	{"nv-environ",nvenviron_interact},

	#ifdef MPHY18_TEST_PROMPT
	{MPHY18_TEST_PROMPT,mphy18_test},
	#endif
	#ifdef MPHY17_TEST_PROMPT
	{MPHY17_TEST_PROMPT,mphy17_test},
	#endif
	#ifdef MPHY16_TEST_PROMPT
	{MPHY16_TEST_PROMPT,mphy16_test},
	#endif
	#ifdef MPHY11_TEST_PROMPT
	{MPHY11_TEST_PROMPT,mphy11_test},
	#endif
	#ifdef S4SICINV_TEST_PROMPT
	{S4SICINV_TEST_PROMPT,s4sicinv_test},
	#endif

	
	{0,0}
};

#define MENU_OFFSET_EXEC 1

//-

XStatus motctl_interactive(void)
{
	int show;
	int i;
	
	show = 0;
	for(;;)
	{
		int key;
		int code;
		
		if (show)
		{
			motctl_disp_ident();
			con_anyk();
			show = 0;
		}
		//
		#if SYS_TOTAL_FMC_BOARDS == 2
		xil_printf("\r\n Board ID = %d",g_board_id);
		#endif
		print("\r\n EXT exit");
		for(i=0;;i++)
		{
			if (menulist[i].pfun == 0) break;
			xil_printf("\r\n %2X  %s",(i+MENU_OFFSET_EXEC),menulist[i].pdesc);
		}
		print("\r\n");
		key = con_guint(1,&code,1);
		print("\r\n");
		if (key == 27)
		{
			print("Really want to exit (y/n) ? ");
			if ('y' == con_gkey())
				break;
		}
		if (key == 13)
		{
			show = 1;
			for(i=0;;i++)
			{
				if (menulist[i].pfun == 0) break;
				if (code == (i+MENU_OFFSET_EXEC))
				{
					show = 0;
					menulist[i].pfun();
					con_anyk();
					break;
					
				}
			}
		}
	}

	// =========================================
	print("\r\nThat's all folks!.\r\n");
	
	return XST_SUCCESS;
}


