/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief IVP Logger Application I/F
/// \file ivplogger.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "glovar.h"

#if SYS_IVPLOGGER_NUM > 0

#if SYS_IVPLOGGER_NUM > 8
	#error  "SYS_IVPLOGGER_NUM > 8 !"
#endif

#if  defined(XPAR_IVPLOGGER_0_MEMORY_BASE) && XPAR_IVPLOGGER_0_MEMORY_BASE == 0
	#define LOCAL_IVPLOGGER_0_MEMORY_SIZE     XPAR_IVPLOGGER_0_MEMORY_SIZE
#else
	#define LOCAL_IVPLOGGER_0_MEMORY_SIZE  0
#endif

#if  defined(XPAR_IVPLOGGER_1_MEMORY_BASE) && XPAR_IVPLOGGER_1_MEMORY_BASE == 0
	#define LOCAL_IVPLOGGER_1_MEMORY_SIZE     XPAR_IVPLOGGER_1_MEMORY_SIZE
#else
	#define LOCAL_IVPLOGGER_1_MEMORY_SIZE  0
#endif


#if  defined(XPAR_IVPLOGGER_2_MEMORY_BASE) && XPAR_IVPLOGGER_2_MEMORY_BASE == 0
	#define LOCAL_IVPLOGGER_2_MEMORY_SIZE     XPAR_IVPLOGGER_2_MEMORY_SIZE
#else
	#define LOCAL_IVPLOGGER_2_MEMORY_SIZE  0
#endif

#if  defined(XPAR_IVPLOGGER_3_MEMORY_BASE) && XPAR_IVPLOGGER_3_MEMORY_BASE == 0
	#define LOCAL_IVPLOGGER_3_MEMORY_SIZE     XPAR_IVPLOGGER_3_MEMORY_SIZE
#else
	#define LOCAL_IVPLOGGER_3_MEMORY_SIZE  0
#endif

#if  defined(XPAR_IVPLOGGER_4_MEMORY_BASE) && XPAR_IVPLOGGER_4_MEMORY_BASE == 0
	#define LOCAL_IVPLOGGER_4_MEMORY_SIZE     XPAR_IVPLOGGER_4_MEMORY_SIZE
#else
	#define LOCAL_IVPLOGGER_4_MEMORY_SIZE  0
#endif

#if  defined(XPAR_IVPLOGGER_5_MEMORY_BASE) && XPAR_IVPLOGGER_5_MEMORY_BASE == 0
	#define LOCAL_IVPLOGGER_5_MEMORY_SIZE     XPAR_IVPLOGGER_5_MEMORY_SIZE
#else
	#define LOCAL_IVPLOGGER_5_MEMORY_SIZE  0
#endif

#if  defined(XPAR_IVPLOGGER_6_MEMORY_BASE) && XPAR_IVPLOGGER_6_MEMORY_BASE == 0
	#define LOCAL_IVPLOGGER_6_MEMORY_SIZE     XPAR_IVPLOGGER_6_MEMORY_SIZE
#else
	#define LOCAL_IVPLOGGER_6_MEMORY_SIZE  0
#endif

#if  defined(XPAR_IVPLOGGER_7_MEMORY_BASE) && XPAR_IVPLOGGER_7_MEMORY_BASE == 0
	#define LOCAL_IVPLOGGER_7_MEMORY_SIZE     XPAR_IVPLOGGER_7_MEMORY_SIZE
#else
	#define LOCAL_IVPLOGGER_7_MEMORY_SIZE  0
#endif

#define LOCAL_IVPLOGGER_X_MEMORY_SIZE (LOCAL_IVPLOGGER_0_MEMORY_SIZE+LOCAL_IVPLOGGER_1_MEMORY_SIZE+LOCAL_IVPLOGGER_2_MEMORY_SIZE+LOCAL_IVPLOGGER_3_MEMORY_SIZE+LOCAL_IVPLOGGER_4_MEMORY_SIZE+LOCAL_IVPLOGGER_5_MEMORY_SIZE+LOCAL_IVPLOGGER_6_MEMORY_SIZE+LOCAL_IVPLOGGER_7_MEMORY_SIZE)

#if LOCAL_IVPLOGGER_X_MEMORY_SIZE > 0

	static u8  ivplogger_ape_pool[LOCAL_IVPLOGGER_X_MEMORY_SIZE] __attribute__ ((aligned (1024)));
	static u32 ivplogger_ape_used;
	static u32 ivplogger_ape_free;

	static void ivplogger_ape_mem_init(void)
	{
		ivplogger_ape_used = 0;
		ivplogger_ape_free = LOCAL_IVPLOGGER_X_MEMORY_SIZE;
	}

	static UINTPTR ivplogger_ape_mem_alloc(const u32 size)
	{
		if (size > ivplogger_ape_free)
			return 0;
		UINTPTR p = (UINTPTR)&ivplogger_ape_pool[ivplogger_ape_used];
		ivplogger_ape_used += size;
		ivplogger_ape_free -= size;
		return p;
	}

#else

	static void ivplogger_ape_mem_init(void)
	{
	}

	static UINTPTR ivplogger_ape_mem_alloc(const u32 size)
	{
		(void)size;
		return 0;
	}

#endif


int ivplogger_ape_init(void)
{
	int rc = XST_SUCCESS;

	ivplogger_ape_mem_init();

	u16 id;
	for(id = 0;id < SYS_IVPLOGGER_NUM; id++)
	{
		ivplogger * InstancePtr = &g_ivplogger_v[id];
		rc = ivplogger_Initialize(InstancePtr,id);
		if (XST_SUCCESS != rc)
			return XST_FAILURE;
		UINTPTR p = InstancePtr->Config->memory_base;
		if (!p)
		{
			p = ivplogger_ape_mem_alloc(InstancePtr->Config->memory_size);
			if (!p)
			{
				xil_printf("ivplogger_ape_mem_alloc()=NULL\r\n");
				return XST_FAILURE;
			}
		}
		if (p != (p & 0xFFFFFFFF))
		{
			xil_printf("ivplogger_ape_mem_alloc()=p_mem over 4G\r\n");
			return XST_FAILURE;
		}
		IVPLOGGER_mWrite_mem_base(InstancePtr->Config->BaseAddress,(u32)p);
	}

	return XST_SUCCESS;
}

#endif
