/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief Microblaze Timer driver
/// \file mb_timer.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "xparameters.h"

#if defined(__MICROBLAZE__)

#include <stdio.h>

#include "xstatus.h"
#include "xtmrctr.h"

#include "xx_timer.h"

static XTmrCtr g_timer;

#define USER_TIMER_AUX  1 // aux timer

#define XX_TIMER_DEF XTmrCtr * p_timer = & g_timer
#define XX_TIMER_REF           p_timer


int xx_timer_init(void)
{
	XX_TIMER_DEF;
	int rc;

	// initialize
	rc = XTmrCtr_Initialize(XX_TIMER_REF,XPAR_TMRCTR_0_DEVICE_ID);
	if (rc != XST_SUCCESS)
	{
		xil_printf("XTmrCtr_Initialize()=%d\r\n",rc);
		return XST_FAILURE;
	}

	// set timer options
	XTmrCtr_SetOptions(XX_TIMER_REF,USER_TIMER_AUX,XTC_AUTO_RELOAD_OPTION);

	// set timer value
	XTmrCtr_SetResetValue(XX_TIMER_REF,USER_TIMER_AUX,0);

	// start timer
	XTmrCtr_Start(XX_TIMER_REF,USER_TIMER_AUX);

	return XST_SUCCESS;
}

void xx_timer_down(void)
{
}

u32 xx_timer_cnt_pser_sec(void)
{
	return XPAR_MICROBLAZE_CORE_CLOCK_FREQ_HZ;
}

u32 xx_timer_cnt_val(void)
{
	XX_TIMER_DEF;
	return XTmrCtr_GetValue(XX_TIMER_REF,USER_TIMER_AUX);
}

#endif
