/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
-- 
--QDESYS 
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief s/w identification
/// \file mcaident.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "glovar.h"

void motctl_disp_ident(void)
{
	xil_printf("\r\nCarrier  : " SYS_CARRIER_NAME "\r\n");

	xil_printf("CPU type : "
				#if defined(__MICROBLAZE__)
					"MICROBLAZE"
				#endif
				#if defined(__arm__) && !defined(ARMR5)
					"Cortex-A9"
				#endif
				#if defined(__arm__) && defined(ARMR5)
					"Cortex-R5"
				#endif
				#if defined(__aarch64__)
					"Cortex-A53"
				#endif
				"\r\n");


	#if SYS_MPHY18_NUM > 0
	xil_printf("M-phy 18 : %d\r\n",SYS_MPHY18_NUM);
	#endif

	#if SYS_MPHY17_NUM > 0
	xil_printf("M-phy 17 : %d\r\n",SYS_MPHY17_NUM);
	#endif

	#if SYS_MPHY16_NUM > 0
	xil_printf("M-phy 16 : %d\r\n",SYS_MPHY16_NUM);
	#endif
	
	#if SYS_TEP2_NUM > 0
	xil_printf("TEP2     : %d\r\n",SYS_TEP2_NUM);
	#endif

	#if SYS_TEC53_NUM > 0
	xil_printf("TEC53    : %d\r\n",SYS_TEC53_NUM);
	#endif

	#if SYS_S4SICINV_NUM > 0
	xil_printf("S4SICINV : %d\r\n",SYS_S4SICINV_NUM);
	#endif

	#if SYS_QINVSIC_NUM > 0
	xil_printf("QINVSIC  : %d\r\n",SYS_QINVSIC_NUM);
	#endif

	#if SYS_MPHY11_NUM > 0
	xil_printf("M-phy 11 : %d\r\n",SYS_MPHY11_NUM);
	#endif

	#if SYS_ZYGUSIC_NUM > 0
	xil_printf("ZYGUSIC  : %d\r\n",SYS_ZYGUSIC_NUM);
	#endif

	#if SYS_TLIMOT_NUM > 0
	xil_printf("TLIMOT   : %d\r\n",SYS_TLIMOT_NUM);
	#endif

	#if SYS_MLIEVB_NUM > 0
	xil_printf("MLIEVB   : %d\r\n",SYS_MLIEVB_NUM);
	#endif

	#if SYS_MOTCON2_NUM > 0
	xil_printf("MOTCON2  : %d\r\n",SYS_MOTCON2_NUM);
	#endif

	#if SYS_MOTCON1_NUM > 0
	xil_printf("MOTCON1  : %d\r\n",SYS_MOTCON1_NUM);
	#endif

	#if SYS_SGXBCU_NUM > 0
	xil_printf("SGXBCU   : %d\r\n",SYS_SGXBCU_NUM);
	#endif

	#if SYS_FMCMC1_NUM > 0
	xil_printf("FMCMC1   : %d\r\n",SYS_FMCMC1_NUM);
	#endif

	#if SYS_NETMOT_NUM > 0
	xil_printf("NETMOT   : %d\r\n",SYS_NETMOT_NUM);
	#endif

	#if SYS_MOTORFOC_NUM > 0
	xil_printf("MOTORFOC : %d\r\n",SYS_MOTORFOC_NUM);
	#endif

	#if SYS_IVPLOGGER_NUM > 0
	xil_printf("IVPLOGGER: %d\r\n",SYS_IVPLOGGER_NUM);
	#endif

	#if SYS_DS1820_NUM > 0
	xil_printf("DS1820   : %d\r\n",SYS_DS1820_NUM);
	#endif

	#if SYS_ETHERCAT_NUM > 0
	xil_printf("ET1815 IP Core\r\n");
	#endif

	xil_printf("%s",
		"Program  : "PROGRAM_NAME "\r\n"
		"Version  : "PROGRAM_VERS_STR "\r\n"
		QAP_APPLICATION_BANNER
		"Built    : "PROGRAM_BUILT "\r\n"
		);
}
