/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief motor control agent can protocol defines
/// \file mcaxcan.h
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#ifndef MCAXCAN_H_
#define MCAXCAN_H_

// ---------------------------------------------------------------------------
// CAN application protocol constants
// ---------------------------------------------------------------------------

#define SYSTEM_ip_ident_code_value 0

enum
{
	XCAN_M2A_STD_BASE = 0x600,
	XCAN_A2M_STD_BASE = 0x700,
};

// main function code
enum
{
	XCAN_M2A_STD_FUNC_SYSF   = XCAN_M2A_STD_BASE + 0,
	XCAN_A2M_STD_FUNC_SYSF   = XCAN_A2M_STD_BASE + 0,

	XCAN_M2A_STD_FUNC_READL  = XCAN_M2A_STD_BASE + 1,
	XCAN_A2M_STD_FUNC_READL  = XCAN_A2M_STD_BASE + 1,

	XCAN_M2A_STD_FUNC_WRITEL = XCAN_M2A_STD_BASE + 2,
	//XCAN_A2M_STD_FUNC_WRITEL = XCAN_A2M_STD_BASE + 2,

};

// build CAN ID from device/function codes

#define XCAN_X2X_STD_PUT_IDENT(device,func)   (((device) <<  4) | (func))
#define XCAN_X2X_EXT_PUT_IDENT(device,func)   (((device) << 20) | (func))

// extract device/function from CAN ID

#define XCAN_X2X_STD_GET_IDENT(id)   ((id) & 0x70F)
#define XCAN_X2X_STD_GET_DEVICE(id)  (((id) >> 4) & 0xF)

#define XCAN_X2X_EXT_GET_IDENT(id)   ((id) & 0x1F0FF000)
#define XCAN_X2X_EXT_GET_DEVICE(id)  (((id) >> 20) & 0xF)



// sysf subcodes
enum
{
	XCAN_M2A_CODE_RUNLEVEL   = 0,
	XCAN_M2A_CODE_REQ4MAP    = 1,
	XCAN_A2M_CODE_MAPOPEN    = 2,
	XCAN_A2M_CODE_MAPDATA    = 3,
	XCAN_A2M_CODE_MAPCLOSE   = 4,
	XCAN_M2A_CODE_TMGSETUP   = 5,
	XCAN_M2A_CODE_DS1820     = 6,
	XCAN_M2A_CODE_TERMINATE  = 7,
	XCAN_M2A_CODE_RESTART    = 8,
	XCAN_M2A_CODE_QXADCPS    = 9,
};

// parameters for system
enum
{
	XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_0  = 0,
	XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_1  = 1,
	XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_2  = 2,
	XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_3  = 3,
	XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_4  = 4,
	XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_5  = 5,
	XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_6  = 6,
	XCAN_X2X_PARAM_ID_SYSTEM_CARRIER_NAME_7  = 7,
	XCAN_X2X_PARAM_ID_SYSTEM_PROGRAM_VERSION = 8,
	XCAN_X2X_PARAM_ID_SYSTEM_SYSTEM_CLOCK    = 9,
};


// parameters for board IP
enum
{
	XCAN_X2X_PARAM_ID_BOARD_DEVICEID            = 0,
	XCAN_X2X_PARAM_ID_BOARD_BASEADDRESS         = 1,
	XCAN_X2X_PARAM_ID_BOARD_CARRIER             = 2,
	XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_0       = 3,
	XCAN_X2X_PARAM_ID_BOARD_HW_LINK_REF_1       = 4,
	XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_0    = 5,
	XCAN_X2X_PARAM_ID_BOARD_INV_ACQTIME_NS_1    = 6,
	XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_0    = 7,
	XCAN_X2X_PARAM_ID_BOARD_RSLV_EXC_DVCLK_1    = 8,
	XCAN_X2X_PARAM_ID_BOARD_FAN_PWM_FREQ        = 9,
	XCAN_X2X_PARAM_ID_BOARD_DOUT_PWM_FREQ       = 10,
	XCAN_X2X_PARAM_ID_BOARD_XADC                = 11,
};

// parameters for motor IP
enum
{
	XCAN_X2X_PARAM_ID_MOTORFOC_DEVICEID          = 0,
	XCAN_X2X_PARAM_ID_MOTORFOC_BASEADDRESS       = 1,
	XCAN_X2X_PARAM_ID_MOTORFOC_HW_LINK_REF       = 2,
	XCAN_X2X_PARAM_ID_MOTORFOC_INV_IN_MAP        = 3,
	XCAN_X2X_PARAM_ID_MOTORFOC_INV_IN_NOT        = 4,
	XCAN_X2X_PARAM_ID_MOTORFOC_INV_OFSV_MODE     = 5,
	XCAN_X2X_PARAM_ID_MOTORFOC_INV_OVER_IPHS     = 6,
	XCAN_X2X_PARAM_ID_MOTORFOC_INV_OVER_IBUS     = 7,
	XCAN_X2X_PARAM_ID_MOTORFOC_INV_FILTER        = 8,
	XCAN_X2X_PARAM_ID_MOTORFOC_CLARKE_NPHS       = 9,
	XCAN_X2X_PARAM_ID_MOTORFOC_PWM_MODULATOR     = 10,
	XCAN_X2X_PARAM_ID_MOTORFOC_RPFM_MODULATOR    = 11,
	XCAN_X2X_PARAM_ID_MOTORFOC_RPFM_3_LEVEL      = 12,
	XCAN_X2X_PARAM_ID_MOTORFOC_RPFM_TPNC         = 13,
	XCAN_X2X_PARAM_ID_MOTORFOC_SMO_EVAL          = 14,
	XCAN_X2X_PARAM_ID_MOTORFOC_SPD_EVAL          = 15,
	XCAN_X2X_PARAM_ID_MOTORFOC_SPEED_CTRL        = 16,
	XCAN_X2X_PARAM_ID_MOTORFOC_RESOLVER          = 17,
	XCAN_X2X_PARAM_ID_MOTORFOC_HALLSENSOR        = 18,
	XCAN_X2X_PARAM_ID_MOTORFOC_ENCODER           = 19,
	XCAN_X2X_PARAM_ID_MOTORFOC_MANROT            = 20,
	XCAN_X2X_PARAM_ID_MOTORFOC_MRT_ACCMAX_DLN2   = 21,
	XCAN_X2X_PARAM_ID_MOTORFOC_SMO_ZS_DLN2       = 22,
	XCAN_X2X_PARAM_ID_MOTORFOC_SMO_F2_DLN2       = 23,
	XCAN_X2X_PARAM_ID_MOTORFOC_SLP_PRO_DLN2      = 24,
	XCAN_X2X_PARAM_ID_MOTORFOC_SLP_INT_DLN2      = 25,
	XCAN_X2X_PARAM_ID_MOTORFOC_SLP_INDWP_DLN2    = 26,
	XCAN_X2X_PARAM_ID_MOTORFOC_SLP_INDWP_KDIV    = 27,
	XCAN_X2X_PARAM_ID_MOTORFOC_ISOVERFLOW_CMAX   = 28,
	XCAN_X2X_PARAM_ID_MOTORFOC_PI_ERPRO_DLN2     = 29,
	XCAN_X2X_PARAM_ID_MOTORFOC_PI_ERINT_DLN2     = 30
};

// status for system
enum
{
	XCAN_X2X_STAT_ID_SYSTEM_TIMER_MSEC  = 0,
	XCAN_X2X_STAT_ID_SYSTEM_STATUS_LOOP = 1,
};

// running level
enum
{
	XCAN_X2X_RUNLEVEL_STOP   = 0,
	XCAN_X2X_RUNLEVEL_FULL   = 1
};

// ---------------------------------------------------------------------------

#endif /* MCAXCAN_H_ */
