/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief extra driver
/// \file motbrddv.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "glovar.h"

int motbrddv_system_init(void)
{
	#if SYS_MOTORFOC_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MOTORFOC_NUM; id++)
		{
			motorfoc_Initialize(&g_motorfoc_v[id],id);
			#if SW_ENA_ENHANCED > 0
			g_motorfoc_v[id].ApPtr = & g_motorfoc_ap_v[id];
			motorfoc_ap_init(&g_motorfoc_v[id]);
			#endif
		}
	}
	#endif
	
	#if SYS_MPHY18_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MPHY18_NUM;id++)
		{
			mphy18_drv_Initialize(&g_mphy18_drv_v[id],id);
			#if SW_ENA_ENHANCED > 0
			g_mphy18_drv_v[id].ApPtr = & g_mphy18_drv_ap_v[id];
			mphy18_drv_ap_init(&g_mphy18_drv_v[id]);
			#endif
		}
	}
	#endif
	
	#if SYS_MPHY17_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MPHY17_NUM;id++)
		{
			mphy17_drv_Initialize(&g_mphy17_drv_v[id],id);
			#if SW_ENA_ENHANCED > 0
			g_mphy17_drv_v[id].ApPtr = & g_mphy17_drv_ap_v[id];
			mphy17_drv_ap_init(&g_mphy17_drv_v[id]);
			#endif
		}
	}
	#endif
	
	#if SYS_MPHY16_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MPHY16_NUM;id++)
		{
			mphy16_drv_Initialize(&g_mphy16_drv_v[id],id);
			#if SW_ENA_ENHANCED > 0
			g_mphy16_drv_v[id].ApPtr = & g_mphy16_drv_ap_v[id];
			mphy16_drv_ap_init(&g_mphy16_drv_v[id]);
			#endif
		}
	}
	#endif

	#if SYS_TEP2_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_TEP2_NUM;id++)
		{
			tep2_drv_Initialize(&g_tep2_drv_v[id],id);
		}
	}
	#endif

	#if SYS_TEC53_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_TEC53_NUM;id++)
		{
			tec53_drv_Initialize(&g_tec53_drv_v[id],id);
		}
	}
	#endif

	#if SYS_S4SICINV_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_S4SICINV_NUM;id++)
		{
			s4sicinv_drv_Initialize(&g_s4sicinv_drv_v[id],id);
			#if SW_ENA_ENHANCED > 0
			g_s4sicinv_drv_v[id].ApPtr = & g_s4sicinv_drv_ap_v[id];
			s4sicinv_drv_ap_init(&g_s4sicinv_drv_v[id]);
			#endif
		}
	}
	#endif

	#if SYS_QINVSIC_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_QINVSIC_NUM;id++)
		{
			qinvsic_drv_Initialize(&g_qinvsic_drv_v[id],id);
			#if SW_ENA_ENHANCED > 0
			g_qinvsic_drv_v[id].ApPtr = & g_qinvsic_drv_ap_v[id];
			qinvsic_drv_ap_init(&g_qinvsic_drv_v[id]);
			#endif
		}
	}
	#endif

	#if SYS_MPHY11_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MPHY11_NUM;id++)
		{
			mphy11_drv_Initialize(&g_mphy11_drv_v[id],id);
			#if SW_ENA_ENHANCED > 0
			g_mphy11_drv_v[id].ApPtr = & g_mphy11_drv_ap_v[id];
			mphy11_drv_ap_init(&g_mphy11_drv_v[id]);
			#endif
		}
	}
	#endif

	#if SYS_ZYGUSIC_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_ZYGUSIC_NUM;id++)
		{
			zygusic_drv_Initialize(&g_zygusic_drv_v[id],id);
		}
	}
	#endif
	
	#if SYS_TLIMOT_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_TLIMOT_NUM;id++)
		{
			tlimot_drv_Initialize(&g_tlimot_drv_v[id],id);
		}
	}
	#endif
	
	#if SYS_MLIEVB_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MLIEVB_NUM;id++)
		{
			mlievb_drv_Initialize(&g_mlievb_drv_v[id],id);
		}
	}
	#endif

	#if SYS_MOTCON2_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MOTCON2_NUM;id++)
		{
			motcon2_drv_Initialize(&g_motcon2_drv_v[id],id);
		}
	}
	#endif
	
	#if SYS_MOTCON1_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MOTCON1_NUM;id++)
		{
			motcon1_drv_Initialize(&g_motcon1_drv_v[id],id);
		}
	}
	#endif
	
	#if SYS_SGXBCU_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_SGXBCU_NUM;id++)
		{
			sgxbcu_drv_Initialize(&g_sgxbcu_drv_v[id],id);
		}
	}
	#endif

	#if SYS_NETMOT_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_NETMOT_NUM;id++)
		{
			netmot_drv_Initialize(&g_netmot_drv_v[id],id);
		}
	}
	#endif

	#if SYS_FMCMC1_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_FMCMC1_NUM;id++)
		{
			fmcmc1_drv_Initialize(&g_fmcmc1_drv_v[id],id);
			#if SW_ENA_ENHANCED > 0
			g_fmcmc1_drv_v[id].ApPtr = & g_fmcmc1_drv_ap_v[id];
			fmcmc1_drv_ap_init(&g_fmcmc1_drv_v[id]);
			#endif
		}
	}
	#endif
	return XST_SUCCESS;
}

int motbrddv_system_down(void)
{
	#if SYS_MOTORFOC_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MOTORFOC_NUM; id++)
		{
			motorfoc_Down(id);
		}
	}
	#endif
	
	#if SYS_MPHY18_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MPHY18_NUM;id++)
		{
			mphy18_drv_Down(id);
		}
	}
	#endif
	
	#if SYS_MPHY17_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MPHY17_NUM;id++)
		{
			mphy17_drv_Down(id);
		}
	}
	#endif
	
	#if SYS_MPHY16_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MPHY16_NUM;id++)
		{
			mphy16_drv_Down(id);
		}
	}
	#endif

	#if SYS_TEP2_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_TEP2_NUM;id++)
		{
			tep2_drv_Down(id);
		}
	}
	#endif

	#if SYS_TEC53_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_TEC53_NUM;id++)
		{
			tec53_drv_Down(id);
		}
	}
	#endif

	#if SYS_S4SICINV_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_S4SICINV_NUM;id++)
		{
			s4sicinv_drv_Down(id);
		}
	}
	#endif

	#if SYS_QINVSIC_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_QINVSIC_NUM;id++)
		{
			qinvsic_drv_Down(id);
		}
	}
	#endif

	#if SYS_MPHY11_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MPHY11_NUM;id++)
		{
			mphy11_drv_Down(id);
		}
	}
	#endif

	#if SYS_ZYGUSIC_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_ZYGUSIC_NUM;id++)
		{
			zygusic_drv_Down(id);
		}
	}
	#endif
	
	#if SYS_TLIMOT_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_TLIMOT_NUM;id++)
		{
			tlimot_drv_Down(id);
		}
	}
	#endif
	
	#if SYS_MLIEVB_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MLIEVB_NUM;id++)
		{
			mlievb_drv_Down(id);
		}
	}
	#endif
	
	#if SYS_MOTCON2_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MOTCON2_NUM;id++)
		{
			motcon2_drv_Down(id);
		}
	}
	#endif
	
	#if SYS_MOTCON1_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MOTCON1_NUM;id++)
		{
			motcon1_drv_Down(id);
		}
	}
	#endif
	
	#if SYS_SGXBCU_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_SGXBCU_NUM;id++)
		{
			sgxbcu_drv_Down(id);
		}
	}
	#endif

	#if SYS_NETMOT_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_NETMOT_NUM;id++)
		{
			netmot_drv_Down(id);
		}
	}
	#endif

	#if SYS_FMCMC1_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_FMCMC1_NUM;id++)
		{
			fmcmc1_drv_Down(id);
		}
	}
	#endif

	return XST_SUCCESS;
}

int motbrddv_system_enable(void)
{
	#if SYS_MPHY18_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MPHY18_NUM; id++)
		{
			u32 mphy18_base = mphy18_drv_LookupConfig(id)->BaseAddress;
			MPHY18_DRV_mWrite_coil_ctrl(mphy18_base,MPHY18_DRV_COIL_CTRL_GX_AUTO);
		}
	}
	#endif
	
	#if SYS_MPHY17_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MPHY17_NUM; id++)
		{
			u32 mphy17_base = mphy17_drv_LookupConfig(id)->BaseAddress;
			MPHY17_DRV_mWrite_coil_ctrl(mphy17_base,MPHY17_DRV_COIL_CTRL_GX_AUTO);
		}
	}
	#endif

	#if SYS_MPHY16_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MPHY16_NUM; id++)
		{
			u32 mphy16_base = mphy16_drv_LookupConfig(id)->BaseAddress;
			MPHY16_DRV_mWrite_coil_ctrl(mphy16_base,MPHY16_DRV_COIL_CTRL_GX_AUTO);
		}
	}
	#endif

	#if SYS_TEP2_NUM > 0
	{
	}
	#endif

	#if SYS_TEC53_NUM > 0
	{
	}
	#endif

	#if SYS_S4SICINV_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_S4SICINV_NUM; id++)
		{
			u32 s4sicinv_base = s4sicinv_drv_LookupConfig(id)->BaseAddress;
			S4SICINV_DRV_mWrite_coil_ctrl(s4sicinv_base,S4SICINV_DRV_COIL_CTRL_GX_AUTO);
		}
	}
	#endif

	#if SYS_QINVSIC_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_QINVSIC_NUM; id++)
		{
			u32 qinvsic_base = qinvsic_drv_LookupConfig(id)->BaseAddress;
			QINVSIC_DRV_mWrite_coil_ctrl(qinvsic_base,QINVSIC_DRV_COIL_CTRL_GX_AUTO);
		}
	}
	#endif

	#if SYS_MPHY11_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MPHY11_NUM; id++)
		{
			u32 mphy11_base = mphy11_drv_LookupConfig(id)->BaseAddress;
			MPHY11_DRV_mWrite_coil_ctrl(mphy11_base,MPHY11_DRV_COIL_CTRL_GX_AUTO);
		}
	}
	#endif

	#if SYS_ZYGUSIC_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_ZYGUSIC_NUM; id++)
		{
			u32 zygusic_base = zygusic_drv_LookupConfig(id)->BaseAddress;
			ZYGUSIC_DRV_mWrite_coil_ctrl(zygusic_base,ZYGUSIC_DRV_COIL_CTRL_GX_AUTO);
		}
	}
	#endif

	#if SYS_TLIMOT_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_TLIMOT_NUM; id++)
		{
			u32 tlimot_base = tlimot_drv_LookupConfig(id)->BaseAddress;
			TLIMOT_DRV_mWrite_coil_ctrl(tlimot_base,TLIMOT_DRV_COIL_CTRL_GX_AUTO);
		}
	}
	#endif

	#if SYS_MLIEVB_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MLIEVB_NUM; id++)
		{
			u32 mlievb_base = mlievb_drv_LookupConfig(id)->BaseAddress;
			MLIEVB_DRV_mWrite_coil_ctrl(mlievb_base,MLIEVB_DRV_COIL_CTRL_GX_AUTO);
		}
	}
	#endif

	#if SYS_MOTCON2_NUM > 0
	{
	}
	#endif

	#if SYS_MOTCON1_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_MOTCON1_NUM; id++)
		{
			u32 motcon1_base = motcon1_drv_LookupConfig(id)->BaseAddress;
			u32 t = 0
					| MOTCON1_DRV_C_COIL_CTRL_GX_AUTO
					| MOTCON1_DRV_C_COIL_CTRL_GX_GEAR
					;
			MOTCON1_DRV_mWrite_coil_ctrl(motcon1_base,t);
		}
	}
	#endif

	#if SYS_SGXBCU_NUM > 0
	{
	}
	#endif

	#if SYS_NETMOT_NUM > 0
	{
		u16 id;
		for(id = 0;id < SYS_NETMOT_NUM; id++)
		{
			u32 netmot_base = netmot_drv_LookupConfig(id)->BaseAddress;

			#define NMDRVMASKMOT (NETMOT_DRV_CMD_OUT_ENA | NETMOT_DRV_COMMAND_GEAR_MASK_0 | NETMOT_DRV_COMMAND_GEAR_MASK_1)
			NETMOT_DRV_command_rmw(netmot_base,NMDRVMASKMOT,NMDRVMASKMOT);
		}
	}
	#endif

	#if SYS_FMCMC1_NUM > 0
	{
	}
	#endif

	return XST_SUCCESS;
}
