/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief non volatile memory environ
/// \file nvenviron.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "glovar.h"

void nvenviron_linkdev_phymode(const u8 ym)
{
	g_link_phymode = ym;
}

void nvenviron_linkdev_address(const u8 da)
{
	if (da <= 15)
	{
		g_link_device = da;

		g_eth_c_def_mac_address[0] = FD_ETH_MAC_ADDRESS_1;
		g_eth_c_def_mac_address[1] = FD_ETH_MAC_ADDRESS_2;
		g_eth_c_def_mac_address[2] = FD_ETH_MAC_ADDRESS_3;
		g_eth_c_def_mac_address[3] = FD_ETH_MAC_ADDRESS_4;
		g_eth_c_def_mac_address[4] = FD_ETH_MAC_ADDRESS_5;
		g_eth_c_def_mac_address[5] = FD_ETH_MAC_ADDRESS_6 + da;

		g_eth_c_def_ip_address [0] = FD_ETH_IP_ADDRESS_1;
		g_eth_c_def_ip_address [1] = FD_ETH_IP_ADDRESS_2;
		g_eth_c_def_ip_address [2] = FD_ETH_IP_ADDRESS_3;
		g_eth_c_def_ip_address [3] = FD_ETH_IP_ADDRESS_4 + da;
	}
}



#if SW_ENA_NVRAM_ENV > 0

#ifdef XPAR_XQSPIPS_0_DEVICE_ID


static u16 nvenviron_evalcksum(void)
{
	// basic env payload
	nvenviron_basic_payload_t *pEB = & g_nvenviron.nvram.basic.payload;

	// cksum seed
	u16 cksum = NENVIRON_NVRAM_CKSUM;
	{
		u32  n;
		u16 *p = (u16 *)pEB;
		for(n = 0; n < sizeof(*pEB); n += 2)
		{
			cksum += *p++;
		}
	}
	return cksum;
}

#define nvenviron_invalidate() { \
	g_nvenviron.nvram.basic.magic = (u16)(~(NENVIRON_NVRAM_MAGIC)); \
}


static void nvenviron_makeok(void)
{
	g_nvenviron.nvram.basic.magic = NENVIRON_NVRAM_MAGIC;
	g_nvenviron.nvram.basic.cksum = nvenviron_evalcksum();
}

static int nvenviron_check(void)
{
	// integrity check

	// magic code
	if (g_nvenviron.nvram.basic.magic != NENVIRON_NVRAM_MAGIC)
	{
		g_nvenviron.checkok = 0;
		return XST_FAILURE;
	}

	u16 cksum = nvenviron_evalcksum();

	// cksum
	if (g_nvenviron.nvram.basic.cksum != cksum)
	{
		g_nvenviron.checkok = 0;
		return XST_FAILURE;
	}

	//xil_printf("NV-RAM load okay\r\n");

	g_nvenviron.checkok = 1;

	return XST_SUCCESS;
}




static int nvenviron_dumpcurrent(void)
{
	// basic env payload
	nvenviron_basic_payload_t *pEB = & g_nvenviron.nvram.basic.payload;
	//-
	pEB->link.device  = g_link_device;
	pEB->link.phymode = g_link_phymode;
	//-
	#if SYS_MOTORFOC_NUM > 0
	{
		u16 id;
		for(id = 0; id < SYS_MOTORFOC_NUM; id++)
		{
			u32 base = motorfoc_LookupConfig(id)->BaseAddress;
			s32 offset_values[MOTORFOC_IVD_NUM];
			offset_values[MOTORFOC_IVD_IPHS_A] = MOTORFOC_mRead_ofs_iphs_a(base);
			offset_values[MOTORFOC_IVD_IPHS_B] = MOTORFOC_mRead_ofs_iphs_b(base);
			offset_values[MOTORFOC_IVD_IPHS_C] = MOTORFOC_mRead_ofs_iphs_c(base);
			offset_values[MOTORFOC_IVD_IBUS_X] = MOTORFOC_mRead_ofs_ibus_x(base);
			offset_values[MOTORFOC_IVD_VPHS_A] = MOTORFOC_mRead_ofs_vphs_a(base);
			offset_values[MOTORFOC_IVD_VPHS_B] = MOTORFOC_mRead_ofs_vphs_b(base);
			offset_values[MOTORFOC_IVD_VPHS_C] = MOTORFOC_mRead_ofs_vphs_c(base);
			offset_values[MOTORFOC_IVD_VBUS_X] = MOTORFOC_mRead_ofs_vbus_x(base);
			offset_values[MOTORFOC_IVD_VPHS_N] = MOTORFOC_mRead_ofs_vphs_n(base);
			int x;
            for(x = 0; x < MOTORFOC_IVD_NUM;x++)
            {
                pEB->motorfoc[id].ivp_offset[x] = MOTORFOC_sign_extend(offset_values[x],18);
            }
		}
	}
	#endif

	return XST_SUCCESS;
}

static int nvenviron_loadsaved(void)
{
	if (XST_SUCCESS == nvenviron_check())
	{
		// basic env payload
		nvenviron_basic_payload_t *pEB = & g_nvenviron.nvram.basic.payload;
		//-
		nvenviron_linkdev_phymode(pEB->link.phymode);
		nvenviron_linkdev_address(pEB->link.device);
		//-
		#if SYS_MOTORFOC_NUM > 0
		{
			u16 id;
			for(id = 0; id < SYS_MOTORFOC_NUM; id++)
			{
				u32 base = motorfoc_LookupConfig(id)->BaseAddress;
				s32 offset_values[MOTORFOC_IVD_NUM];
				int x;
	            for(x = 0; x < MOTORFOC_IVD_NUM;x++)
	            {
	                offset_values[x] = pEB->motorfoc[id].ivp_offset[x];
	            }
				MOTORFOC_mWrite_ofs_iphs_a(base,offset_values[MOTORFOC_IVD_IPHS_A]);
				MOTORFOC_mWrite_ofs_iphs_b(base,offset_values[MOTORFOC_IVD_IPHS_B]);
				MOTORFOC_mWrite_ofs_iphs_c(base,offset_values[MOTORFOC_IVD_IPHS_C]);
				MOTORFOC_mWrite_ofs_ibus_x(base,offset_values[MOTORFOC_IVD_IBUS_X]);
				MOTORFOC_mWrite_ofs_vphs_a(base,offset_values[MOTORFOC_IVD_VPHS_A]);
				MOTORFOC_mWrite_ofs_vphs_b(base,offset_values[MOTORFOC_IVD_VPHS_B]);
				MOTORFOC_mWrite_ofs_vphs_c(base,offset_values[MOTORFOC_IVD_VPHS_C]);
				MOTORFOC_mWrite_ofs_vbus_x(base,offset_values[MOTORFOC_IVD_VBUS_X]);
				MOTORFOC_mWrite_ofs_vphs_n(base,offset_values[MOTORFOC_IVD_VPHS_N]);
			}
		}
		#endif
	}
	else
	{
		nvenviron_linkdev_phymode(FD_LINK_PHYMODE);
		nvenviron_linkdev_address(FD_LINK_ADDRESS);
	}
	
	return XST_SUCCESS;
}


#define NVENVIRON_NVRAM_SIZE    sizeof(nvenviron_nvram_t)

static u32 nvenviron_address(void)
{
	u32 addr = g_zq_qspips.flashIdent->flash_size - NVENVIRON_NVRAM_SIZE;
	addr -= addr & (g_zq_qspips.flashIdent->sec_size-1);
	return addr;
}


#if 0
	static u8 l_old_state;
	#define NVENVIRON_NVRAM_SPY_STATE
#endif

int nvenviron_process(void)
{
	int rc;

	#ifdef NVENVIRON_NVRAM_SPY_STATE
	if (l_old_state != g_nvenviron.state)
	{
		xil_printf("NVS=%d\r\n",g_nvenviron.state);
		l_old_state = g_nvenviron.state;
	}
	#endif

	switch(g_nvenviron.state)
	{
		case NVENVIRON_STATE_IDLE:
			{
			}
			break;
		case NVENVIRON_STATE_ERASE_START:
			{
				// enable write
				if (XST_SUCCESS != (rc = zq_qspips_write_enable(&g_zq_qspips)))
				{
					g_nvenviron.state = NVENVIRON_STATE_ERROR;
					break;
				}

				// erase sector
				if (XST_SUCCESS != (rc = zq_qspips_sector_erase(&g_zq_qspips,nvenviron_address())))
				{
					g_nvenviron.state = NVENVIRON_STATE_ERROR;
					break;
				}

				// nv-memory is NOT valid
				nvenviron_invalidate();

				g_nvenviron.state = NVENVIRON_STATE_ERASE_W4_DONE;
			}
			break;
		case NVENVIRON_STATE_ERASE_W4_DONE:
			{
				u8 sr1;
				if (XST_SUCCESS != (rc = zq_qspips_flash_read_sr1(&g_zq_qspips,&sr1)))
				{
					g_nvenviron.state = NVENVIRON_STATE_ERROR;
					break;
				}
				if ((sr1 & 0x01) == 0)
				{
					g_nvenviron.erased = 1;
					g_nvenviron.state = NVENVIRON_STATE_IDLE;
					break;
				}
			}
			break;


		case NVENVIRON_STATE_PROGRAM_START:
			{
				if (!g_nvenviron.erased)
				{
					xil_printf("Not erased!\r\n");
					g_nvenviron.state = NVENVIRON_STATE_ERROR;
					break;
				}

				// current value to nvram
				nvenviron_dumpcurrent();

				// make ok
				nvenviron_makeok();

				// nvram
				nvenviron_nvram_t *pNV = & g_nvenviron.nvram;

				// init for loop program
				g_nvenviron.address  = nvenviron_address();
				g_nvenviron.localmem = (u32)pNV;
				g_nvenviron.nbytes   = NVENVIRON_NVRAM_SIZE;

				g_nvenviron.state = NVENVIRON_STATE_PROGRAM_LOOP_PROG;
			}
			break;


		case NVENVIRON_STATE_PROGRAM_LOOP_PROG:
			{
				// end-of-loop
				if (g_nvenviron.nbytes == 0)
				{
					// done
					g_nvenviron.state = NVENVIRON_STATE_IDLE;
					break;
				}

				// enable write
				if (XST_SUCCESS != (rc = zq_qspips_write_enable(&g_zq_qspips)))
				{
					g_nvenviron.state = NVENVIRON_STATE_ERROR;
					break;
				}

				u32 bytecount = (g_nvenviron.nbytes < QSPI_FLASH_PAGE_SIZE) ? g_nvenviron.nbytes : QSPI_FLASH_PAGE_SIZE;

				memcpy(&g_zq_qspips.flashBuffer[ZQ_QSPIPS_BUFFER_OFFSET_WRITE],(void *)g_nvenviron.localmem,bytecount);

				if (XST_SUCCESS  != (rc = zq_qspips_page_program(&g_zq_qspips, g_nvenviron.address, bytecount)))
				{
					g_nvenviron.state = NVENVIRON_STATE_ERROR;
					break;
				}

				//xil_printf("prg: ad=x%X la=x%X nb=%d\r\n",	g_nvenviron.address  ,	g_nvenviron.localmem ,	g_nvenviron.nbytes   );


				g_nvenviron.state = NVENVIRON_STATE_PROGRAM_W4_PROG;
			}
			break;

		case NVENVIRON_STATE_PROGRAM_W4_PROG:
			{
				u8 sr1;
				if (XST_SUCCESS != (rc = zq_qspips_flash_read_sr1(&g_zq_qspips,&sr1)))
				{
					g_nvenviron.state = NVENVIRON_STATE_ERROR;
					break;
				}
				//xil_printf("QSPI sr1=x%X\r\n",sr1);
				if ((sr1 & 0x01) == 0)
				{
					u32 bytecount = (g_nvenviron.nbytes < QSPI_FLASH_PAGE_SIZE) ? g_nvenviron.nbytes : QSPI_FLASH_PAGE_SIZE;

					g_nvenviron.address  += bytecount;
					g_nvenviron.localmem += bytecount;
					g_nvenviron.nbytes   -= bytecount;

					g_nvenviron.state = NVENVIRON_STATE_PROGRAM_LOOP_PROG;
					break;
				}
			}
			break;


		case NVENVIRON_STATE_READ_START:
			{
				// nvram
				nvenviron_nvram_t *pNV = & g_nvenviron.nvram;

				// init read loop
				g_nvenviron.address  = nvenviron_address();
				g_nvenviron.localmem = (u32)pNV;
				g_nvenviron.nbytes   = NVENVIRON_NVRAM_SIZE;

				// loop
				while(g_nvenviron.nbytes > 0)
				{
					u32 bytecount = (g_nvenviron.nbytes < ZQ_QSPIPS_BUFFER_SIZE) ? g_nvenviron.nbytes : ZQ_QSPIPS_BUFFER_SIZE;

					//xil_printf("zq_qspips_read(,x%X,%d)\r\n", g_nvenviron.address, bytecount);

					if (XST_SUCCESS  != (rc = zq_qspips_read(&g_zq_qspips, g_nvenviron.address, bytecount)))
					{
						g_nvenviron.state = NVENVIRON_STATE_ERROR;
						break;
					}

					memcpy((void *)g_nvenviron.localmem,&g_zq_qspips.flashBuffer[ZQ_QSPIPS_BUFFER_OFFSET_READ],bytecount);

					g_nvenviron.address  += bytecount;
					g_nvenviron.localmem += bytecount;
					g_nvenviron.nbytes   -= bytecount;
				}

				//xil_printf("magic=%X mode=%d device=%d cksum=x%X\r\n",pNV->magic,pNV->link.phymode,pNV->link.device,pNV->cksum);

				// integrity check

				if (XST_SUCCESS != nvenviron_check())
				{
					g_nvenviron.state = NVENVIRON_STATE_LOAD_SAVED;
					break;
				}

				//xil_printf("NV-RAM load okay\r\n");

				g_nvenviron.state = NVENVIRON_STATE_LOAD_SAVED;
				break;
			}
			break;

		case NVENVIRON_STATE_LOAD_SAVED:
			{
				nvenviron_loadsaved();

				g_nvenviron.state = NVENVIRON_STATE_IDLE;
			}
			break;


		case NVENVIRON_STATE_ERROR:
			{
			}
			break;

		default:
			{
			}
			break;
	}
	return XST_SUCCESS;
}

int nvenviron_exec_done(void)
{
	int rc = XST_SUCCESS;
	switch(g_nvenviron.state)
	{
		case NVENVIRON_STATE_IDLE:
		case NVENVIRON_STATE_ERROR:
			{
				rc = XST_SUCCESS;
			}
			break;
		default:
			{
				rc = XST_DEVICE_BUSY;
			}
			break;
	}
	return rc;
}

int nvenviron_docmd(const int reqstate)
{
	int rc;

	if (XST_SUCCESS != (rc = nvenviron_exec_done()))
		return rc;

	switch(reqstate)
	{
		case NVENVIRON_STATE_IDLE:
		case NVENVIRON_STATE_ERASE_START:
		case NVENVIRON_STATE_PROGRAM_START:
		case NVENVIRON_STATE_READ_START:
		case NVENVIRON_STATE_LOAD_SAVED:
		case NVENVIRON_STATE_ERROR:
			{
				g_nvenviron.state = reqstate;
				rc = XST_SUCCESS;
			}
			break;
		default:
			{
				rc = XST_FAILURE;
			}
			break;
	}
	return rc;
}


int nvenviron_load(void)
{
	while(XST_SUCCESS != nvenviron_exec_done())
		nvenviron_process();
	nvenviron_docmd(NVENVIRON_STATE_READ_START);
	while(XST_SUCCESS != nvenviron_exec_done())
		nvenviron_process();
	return XST_SUCCESS;
}

int nvenviron_initialize(void)
{
	g_nvenviron.erased = 0;
	g_nvenviron.state  = NVENVIRON_STATE_IDLE;

	return XST_SUCCESS;
}


#define NENVIRON_NVRAM_FCT

#endif
#endif

#ifndef NENVIRON_NVRAM_FCT


	int nvenviron_process(void)
	{
		return XST_SUCCESS;
	}

	int nvenviron_exec_done(void)
	{
		return XST_SUCCESS;
	}

	int nvenviron_docmd(const int reqstate)
	{
		(void)reqstate; // unused
		return XST_SUCCESS;
	}
	int nvenviron_load(void)
	{
		nvenviron_linkdev_phymode(LINK_PHYMODE_ETH);
		nvenviron_linkdev_address(0);
		return XST_SUCCESS;
	}
	int nvenviron_initialize(void)
	{
		return XST_SUCCESS;
	}

#endif






int nvenviron_interact(void)
{
	for(;;)
	{
		xil_printf(
				"\r\n"
				"== environ size=%d : dev=%d mode=%s ==" "\r\n"
				"a - auto" "\r\n"
				"e - eth" "\r\n"
				"c - can" "\r\n"
				"n - dev number" "\r\n"
				"o - display ivp offset" "\r\n"
				"s - save nvram" "\r\n"
				"l - load nvram" "\r\n"
				"x - erase nvram" "\r\n",
				sizeof(nvenviron_nvram_t),
				g_link_device,
				(g_link_phymode == LINK_PHYMODE_ETH) ? "eth" :
				(g_link_phymode == LINK_PHYMODE_CAN) ? "can" : "auto"
				);

		int key;
		do
		{
			nvenviron_process();
			key = con_tkey();
		} while(key < 0);

		if (27 == key) break;
		switch(key)
		{
			case 'a':
				{
					nvenviron_linkdev_phymode(LINK_PHYMODE_AUTO);
				}
				break;
			case 'e':
				{
					nvenviron_linkdev_phymode(LINK_PHYMODE_ETH);
				}
				break;
			case 'c':
				{
					nvenviron_linkdev_phymode(LINK_PHYMODE_CAN);
				}
				break;
			case 'n':
				{
					int value;
					print("Enter Device # : ");
					key = con_guint(2,&value,0);
					if (13 == key)
						nvenviron_linkdev_address(value);
				}
				break;
			case 'o':
				#if SYS_MOTORFOC_NUM > 0
				{
					u16 id;
					for(id = 0; id < SYS_MOTORFOC_NUM; id++)
					{
						xil_printf("m[%d]\r\n",id);
						u32 base = motorfoc_LookupConfig(id)->BaseAddress;
						xil_printf("iphs_a = %d\r\n", MOTORFOC_sign_extend(MOTORFOC_mRead_ofs_iphs_a(base),18));
						xil_printf("iphs_b = %d\r\n", MOTORFOC_sign_extend(MOTORFOC_mRead_ofs_iphs_b(base),18));
						xil_printf("iphs_c = %d\r\n", MOTORFOC_sign_extend(MOTORFOC_mRead_ofs_iphs_c(base),18));
						xil_printf("ibus_x = %d\r\n", MOTORFOC_sign_extend(MOTORFOC_mRead_ofs_ibus_x(base),18));
						xil_printf("vphs_a = %d\r\n", MOTORFOC_sign_extend(MOTORFOC_mRead_ofs_vphs_a(base),18));
						xil_printf("vphs_b = %d\r\n", MOTORFOC_sign_extend(MOTORFOC_mRead_ofs_vphs_b(base),18));
						xil_printf("vphs_c = %d\r\n", MOTORFOC_sign_extend(MOTORFOC_mRead_ofs_vphs_c(base),18));
						xil_printf("vbus_x = %d\r\n", MOTORFOC_sign_extend(MOTORFOC_mRead_ofs_vbus_x(base),18));
						xil_printf("vphs_n = %d\r\n", MOTORFOC_sign_extend(MOTORFOC_mRead_ofs_vphs_n(base),18));
						xil_printf("\r\n");
					}
				}
				#endif
				break;
			case 's' :
				{
					nvenviron_docmd(NVENVIRON_STATE_PROGRAM_START);
				}
				break;
			case 'l' :
				{
					nvenviron_docmd(NVENVIRON_STATE_READ_START);
				}
				break;
			case 'x' :
				{
					nvenviron_docmd(NVENVIRON_STATE_ERASE_START);
				}
				break;

			default:
				{
				}
				break;
		}
	}
	return XST_SUCCESS;
}
