/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief non volatile memory environ
/// \file nvenviron.h
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#ifndef SRC_NVENVIRON_H_
#define SRC_NVENVIRON_H_

// time for spawn
#define LINKDEVICE_SPAWN_TIME_MS    8000

enum {
	LINK_PHYMODE_AUTO = 0,
	LINK_PHYMODE_ETH = 1,
	LINK_PHYMODE_CAN = 2
};

// brief link phy mode
#define FD_LINK_PHYMODE  LINK_PHYMODE_ETH

// brief link device index
#define FD_LINK_ADDRESS  0

// brief ETH ethernet MAC address (factory)
#define FD_ETH_MAC_ADDRESS_1  0x00
#define FD_ETH_MAC_ADDRESS_2  0x0a
#define FD_ETH_MAC_ADDRESS_3  0x35
#define FD_ETH_MAC_ADDRESS_4  0xea
#define FD_ETH_MAC_ADDRESS_5  0x34
#define FD_ETH_MAC_ADDRESS_6  0x6c

// brief ETH tcp/ip IP address (factory)
#define FD_ETH_IP_ADDRESS_1 192
#define FD_ETH_IP_ADDRESS_2 168
#define FD_ETH_IP_ADDRESS_3 1
#define FD_ETH_IP_ADDRESS_4 100


#ifndef QAP_APPLICATION_NVE_DEFINE
#define QAP_APPLICATION_NVE_DEFINE
#endif


// basic environment payload
typedef struct
{

	struct
	{
		u8  device;
		u8  phymode;
	} link;

	#if SYS_MOTORFOC_NUM > 0
	struct
	{
		s32 ivp_offset[MOTORFOC_IVD_NUM];
	} motorfoc[SYS_MOTORFOC_NUM];
	#endif

} nvenviron_basic_payload_t;


// basic environment
typedef struct
{

#define NENVIRON_NVRAM_MAGIC 0x0001
	u16 magic;

	nvenviron_basic_payload_t payload;

#define NENVIRON_NVRAM_CKSUM 0xFFFF
	u16 cksum;

} nvenviron_basic_t;


// whole NVRAM
typedef struct
{
	nvenviron_basic_t basic;
	QAP_APPLICATION_NVE_DEFINE;
} nvenviron_nvram_t;


typedef struct
{
	nvenviron_nvram_t nvram;
	u8                erased;
	u8                checkok;
	u8                state;
	#define NVENVIRON_STATE_IDLE              0
	#define NVENVIRON_STATE_ERASE_START       1
	#define NVENVIRON_STATE_ERASE_W4_DONE     2
	#define NVENVIRON_STATE_PROGRAM_START     3
	#define NVENVIRON_STATE_PROGRAM_LOOP_PROG 4
	#define NVENVIRON_STATE_PROGRAM_W4_PROG   5
	#define NVENVIRON_STATE_READ_START        6
	#define NVENVIRON_STATE_LOAD_SAVED        7
	#define NVENVIRON_STATE_ERROR             8
	
	u16               nbytes;
	u32               address;
	u32               localmem;

} nvenviron_t;



// function prototypes

void nvenviron_linkdev_phymode(const u8 ym);
void nvenviron_linkdev_address(const u8 da);
int  nvenviron_process(void);
int  nvenviron_exec_done(void);
int  nvenviron_docmd(const int reqstate);
int  nvenviron_load(void);
int  nvenviron_initialize(void);
int  nvenviron_interact(void);

#endif /* SRC_NVENVIRON_H_ */
