/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief qxadcps application engine
/// \file qxadcps_ape.c
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#include "glovar.h"

#if SYS_QXADCPS_NUM > 0

static XAdcPs xadc;
#define QXADCPS_OC_TEMP_K (0.05F)

int qxadcps_ape_init(void)
{
	int rc;

	// reference
	XAdcPs_Config * ConfigPtr = XAdcPs_LookupConfig(XPAR_PS7_XADC_0_DEVICE_ID);
	if (NULL == ConfigPtr) return XST_FAILURE;

	// Initialize
	rc = XAdcPs_CfgInitialize(&xadc,ConfigPtr,XPAR_PS7_XADC_0_BASEADDR);
	if (XST_SUCCESS != rc) return rc;

	//stop sequencer
    XAdcPs_SetSequencerMode(&xadc, XADCPS_SEQ_MODE_SINGCHAN);

    //disable alarms
    XAdcPs_SetAlarmEnables(&xadc, 0x0);

    //configure sequencer to just sample internal on chip parameters
    XAdcPs_SetSeqInputMode(&xadc, XADCPS_SEQ_MODE_SAFE);

    //configure the channel enables we want to monitor
    XAdcPs_SetSeqChEnables(&xadc,
    		XADCPS_CH_TEMP | XADCPS_CH_VCCINT | XADCPS_CH_VCCAUX | XADCPS_CH_VBRAM | XADCPS_CH_VCCPINT | XADCPS_CH_VCCPAUX | XADCPS_CH_VCCPDRO);

	return XST_SUCCESS;
}

int qxadcps_ape_proc(void)
{
    u32 TempRawData = XAdcPs_GetAdcData(&xadc, XADCPS_CH_TEMP);
    float temp      = XAdcPs_RawToTemperature(TempRawData);
    g_qxadcps_oc_temp[0] += (temp - g_qxadcps_oc_temp[0]) * QXADCPS_OC_TEMP_K;
    return XST_SUCCESS;
}

void qxadcps_ape_display(void)
{
	printf("\n");

    u32 TempRawData = XAdcPs_GetAdcData(&xadc, XADCPS_CH_TEMP);
    float TempData    = XAdcPs_RawToTemperature(TempRawData);
    printf("Raw Temp    %5lu Real Temp    %6.3f\n", TempRawData, TempData);

    u32 VccIntRawData = XAdcPs_GetAdcData(&xadc, XADCPS_CH_VCCINT);
    float VccIntData    = XAdcPs_RawToVoltage(VccIntRawData);
    printf("Raw VccInt  %5lu Real VccInt  %6.3f\n", VccIntRawData, VccIntData);

    u32 VccAuxRawData = XAdcPs_GetAdcData(&xadc, XADCPS_CH_VCCAUX);
    float VccAuxData    = XAdcPs_RawToVoltage(VccAuxRawData);
    printf("Raw VccAux  %5lu Real VccAux  %6.3f\n", VccAuxRawData, VccAuxData);

    // VrefPRawData = XAdcPs_GetAdcData(&xadc, XADCPS_CH_VREFP);
    // VrefPData = XAdcPs_RawToVoltage(VrefPRawData);
    // printf("Raw VRefP %5lu Real VRefP %6.3f\n", VrefPRawData, VrefPData);

	// VrefNRawData = XAdcPs_GetAdcData(&xadc, XADCPS_CH_VREFN);
	// VrefNData = XAdcPs_RawToVoltage(VrefNRawData);
	// printf("Raw VRefN %5lu Real VRefN %6.3f\n", VrefNRawData, VrefNData);

    u32 VBramRawData = XAdcPs_GetAdcData(&xadc, XADCPS_CH_VBRAM);
    float VBramData    = XAdcPs_RawToVoltage(VBramRawData);
    printf("Raw VccBram %5lu Real VccBram %6.3f\n", VBramRawData, VBramData);

    u32 VccPIntRawData = XAdcPs_GetAdcData(&xadc, XADCPS_CH_VCCPINT);
    float VccPIntData = XAdcPs_RawToVoltage(VccPIntRawData);
    printf("Raw VccPInt %5lu Real VccPInt %6.3f\n", VccPIntRawData, VccPIntData);

    u32 VccPAuxRawData = XAdcPs_GetAdcData(&xadc,XADCPS_CH_VCCPAUX);
    float VccPAuxData = XAdcPs_RawToVoltage(VccPAuxRawData);
    printf("Raw VccPAux %5lu Real VccPAux %6.3f\n", VccPAuxRawData,VccPAuxData);

    u32 VDDRRawData = XAdcPs_GetAdcData(&xadc, XADCPS_CH_VCCPDRO);
    float VDDRData    = XAdcPs_RawToVoltage(VDDRRawData);
    printf("Raw VccDDR  %5lu Real VccDDR  %6.3f\n", VDDRRawData,VDDRData);

}

#else

int qxadcps_ape_init(void)
{
	return XST_SUCCESS;
}

int qxadcps_ape_proc(void)
{
    return XST_SUCCESS;
}

int qxadcps_ape_get_temp(float * temp)
{
	(void)temp; // unused
    return XST_NO_DATA;
}

void qxadcps_ape_display(void)
{
}

#endif

