/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief CAN driver
/// \file xx_xcan.h
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#ifndef XX_XCAN_H_
#define XX_XCAN_H_

#include "xil_types.h"
#include "xstatus.h"

typedef struct
{
	union
	{
		u32 raw_l[16/4];
		u16 raw_w[16/2];
		u8  raw_b[16/1];
	};

	#define XX_XCAN_PAYLOAD_IDX_L (8/4)
	#define XX_XCAN_PAYLOAD_IDX_W (8/2)
	#define XX_XCAN_PAYLOAD_IDX_B (8/1)

	u32 id ;
	u16 tst;
	u8  srr;
	u8  ide;
	u8  rtr;
	u8  dlc;


#define XX_XCAN_PAYLOAD_IDX_BX(index)   (XX_XCAN_PAYLOAD_IDX_B + (index))
#define XX_XCAN_PAYLOAD_IDX_WX(index)   (XX_XCAN_PAYLOAD_IDX_W + (index))
#define XX_XCAN_PAYLOAD_IDX_LX(index)   (XX_XCAN_PAYLOAD_IDX_L + (index))

#define XX_XCAN_PAYLOAD_SWAP_U32(value) (0 \
		| (((value) >> 24) & 0x000000FF) \
		| (((value) >>  8) & 0x0000FF00) \
		| (((value) <<  8) & 0x00FF0000) \
		| (((value) << 24) & 0xFF00000F) \
		)

#define XX_XCAN_PAYLOAD_SWAP_U16(value) (0 \
		| (((value) >>  8) & 0x00FF) \
		| (((value) <<  8) & 0xFF00) \
		)


} xx_xcan_frame_t;

int xx_xcan_send_full(void);
int xx_xcan_recv_ready(void);

int xx_xcan_recv_frame(xx_xcan_frame_t *pf);
int xx_xcan_send_frame(xx_xcan_frame_t *pf);

int xx_xcan_show_info(void);

int  xx_xcan_init(void);
void xx_xcan_down(void);

#endif /* XX_XCAN_H_ */
