/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief ZYNQ IICPS alternaltive driver
/// \file zq_iicps.h
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020


#ifndef ZQ_IICPS_H_
#define ZQ_IICPS_H_

#include "xparameters.h"

#if defined (__arm__) || defined (__aarch64__)

#if defined(XPAR_XIICPS_0_BASEADDR) || defined(XPAR_XIICPS_1_BASEADDR)

#include "xiicps_hw.h"
#include "xiicps.h"

int i2cps_get_isr(const u32 BaseAddress);
int i2cps_get_sr(const u32 BaseAddress);
int i2cps_is_busy(const u32 BaseAddress);
int i2cps_is_comp(const u32 BaseAddress);
int i2cps_get_baud(const u32 BaseAddress, const u32 inclockhz);
int i2cps_set_baud(const u32 BaseAddress, const u32 inclockhz, const u32 baudrate);
int i2cps_set_timeout(const u32 BaseAddress, const u8 timeout);
int i2cps_init(const u32 BaseAddress, const u32 inclockhz, const u32 baudrate, const u8 timeout);
int i2cps_is_arb_lost(const u32 BaseAddress);
int i2cps_is_send_done(const u32 BaseAddress);
int i2cps_send(const u32 BaseAddress, const u16 addr, const u8 * buffer, const u8 length, const u8 hold);
int i2cps_recv_arm(const u32 BaseAddress, const u16 addr, const u8 length);
int i2cps_recv_get(const u32 BaseAddress, u8 *buffer, const u8 length);

#endif

#endif

#endif /* ZQ_IICPS_H_ */
