/*
-------------------------------------------------------------------------------------------
-- Copyright (c) 2007-2020, QDESYS
-- This file contains confidential and proprietary information of QDESYS and is
-- protected under European and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by QDESYS, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND QDESYS HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) QDESYS shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or QDESYS had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- QDESYS products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of QDESYS products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
--QDESYS
--Via Mazzano 14/a
-- 37024 Negrar Verona Italy
--
-- info@qdesys.com
--
*/

/// \brief zynq qspi_ps
/// \file zq_qspips.h
/// \version 1.01
/// \date Apr  6, 2018
/// \author Diego Quagreda
/// \author CopyRight (c) QDESYS 2007-2020

#ifndef SRC_ZQ_QSPIPS_H_
#define SRC_ZQ_QSPIPS_H_

// - protocol definition

#define ZQ_QSPIPS_PROTO_TYPE                 254
#define ZQ_QSPIPS_PROTO_VERSION                1

#define ZQ_QSPIPS_PROTO_CODE_FLS_IDENT         0
#define ZQ_QSPIPS_PROTO_CODE_FLS_STATUS        1
#define ZQ_QSPIPS_PROTO_CODE_FLS_READ          2
#define ZQ_QSPIPS_PROTO_CODE_FLS_WRITE         3
#define ZQ_QSPIPS_PROTO_CODE_FLS_SCT_ERASE     4
#define ZQ_QSPIPS_PROTO_CODE_FLS_GET           5
#define ZQ_QSPIPS_PROTO_CODE_FLS_PUT           6

#define ZQ_QSPIPS_PROTO_CODE_FLS_DATA_LENGTH  1024

// -

// page size (program)
#define QSPI_FLASH_PAGE_SIZE              256   // move do flash descriptor ?!?!?!


// flash definition
typedef struct
{
	const char *name;
	const u8    id[6];
	const u8    id_len;
	const u32   sec_size;
	const u16   sectors;
	const u32   flash_size;
} zq_qspips_flash_type_t;


typedef struct
{
	XQspiPs dev;

	#define ZQ_QSPIPS_BUFFER_SIZE       ZQ_QSPIPS_PROTO_CODE_FLS_DATA_LENGTH
	u8  flashBuffer[ 1 + 3 + 1 +  ZQ_QSPIPS_BUFFER_SIZE ];
	#define ZQ_QSPIPS_BUFFER_OFFSET_READ  5
	#define ZQ_QSPIPS_BUFFER_OFFSET_WRITE 4

	const zq_qspips_flash_type_t * flashIdent;
	
} zq_qspips_t;

// function prototypes

int zq_qspips_flash_read_sr1(zq_qspips_t *pDrv, u8 * sr1);
int zq_qspips_write_enable(zq_qspips_t *pDrv);
int zq_qspips_read(zq_qspips_t *pDrv, const u32 address, const u32 bytecount);
int zq_qspips_sector_erase(zq_qspips_t *pDrv, const u32 address);
int zq_qspips_page_program(zq_qspips_t *pDrv, const u32 address, const u32 bytecount);
int zq_qspips_initialize(zq_qspips_t *pDrv);
int zq_qspips_eth_process(zq_qspips_t *pDrv, u8 *p_data,u16 *p_len, u8 *p_xdn);

#endif /* SRC_ZQ_QSPIPS_H_ */
