import axi_vip_pkg::*;
import axi_vip_master_pkg::*;

module axi_vip_master_commander(i_wr_cmd_valid, i_wr_cmd_address, i_wr_cmd_length, i_wr_cmd_data, o_wr_cmd_done,
   i_rd_cmd_valid, i_rd_cmd_address, i_rd_cmd_length, o_rd_cmd_done, o_rd_cmd_data);

   input i_wr_cmd_valid;
   input [31:0] i_wr_cmd_address;
   input [7:0] i_wr_cmd_length;
   input [32767:0] i_wr_cmd_data;
   output o_wr_cmd_done;

   input i_rd_cmd_valid;
   input [31:0] i_rd_cmd_address;
   input [7:0] i_rd_cmd_length;
   output o_rd_cmd_done;
   output [32767:0] o_rd_cmd_data;

   /*************************************************************************************************
  * <component_name>_mst_t for master agent
  * <component_name> can be easily found in vivado bd design: click on the instance, 
  * Then click CONFIG under Properties window and Component_Name will be shown
  * More details please refer PG267 section about "Useful Coding Guidelines and Examples"
  * for more details.
  *************************************************************************************************/
   axi_vip_master_mst_t                               agent;

   /*************************************************************************************************
  * Declare variables which will be used in API and parital randomization for transaction generation
  * and data read back from driver.
  *************************************************************************************************/
   axi_transaction                                          wr_trans; // Write transaction
   axi_transaction                                          rd_trans; // Read transaction
   xil_axi_uint                                             mtestWID; // Write ID  
   xil_axi_ulong                                            mtestWADDR; // Write ADDR  
   xil_axi_len_t                                            mtestWBurstLength; // Write Burst Length   
   xil_axi_size_t                                           mtestWDataSize; // Write SIZE  
   xil_axi_burst_t                                          mtestWBurstType; // Write Burst Type
   xil_axi_lock_t                                           mtestWLock; // Write Lock type  
   xil_axi_resp_t [255:0]                                   mtestWResp; // WRESP type  
   xil_axi_resp_t [255:0]                                   mtestBResp; // BRESP type
   xil_axi_uint                                             mtestRID; // Read ID  
   xil_axi_ulong                                            mtestRADDR; // Read ADDR  
   xil_axi_len_t                                            mtestRBurstLength; // Read Burst Length   
   xil_axi_size_t                                           mtestRDataSize; // Read SIZE  
   xil_axi_burst_t                                          mtestRBurstType; // Read Burst Type
   xil_axi_lock_t                                           mtestRLock; // Read Lock type  
   xil_axi_resp_t [255:0]                                   mtestRResp; // RRESP type

   xil_axi_data_beat [255:0]                                mtestWUSER; // Write user  
   xil_axi_data_beat                                        mtestAWUSER; // Write Awuser 
   xil_axi_data_beat                                        mtestARUSER; // Read Aruser 
   xil_axi_data_beat [255:0]                                mtestRUSER; // Read ruser 
   /************************************************************************************************
  * A burst can not cross 4KB address boundry for AXI4
  * Maximum data bits = 4*1024*8 =32768
  * Write Data Value for WRITE_BURST transaction
  * Read Data Value for READ_BURST transaction
  ************************************************************************************************/
   bit [32767:0]                                          mtestWData; // Write Data 
   bit [32767:0]                                          Rdatablock; // Read data block

   logic r_wr_cmd_done;
   logic r_rd_cmd_done;

   initial begin
      /***********************************************************************************************
    * Before agent is newed, user has to run simulation with an empty testbench to find the hierarchy
    * path of the AXI VIP's instance.Message like
    * "Xilinx AXI VIP Found at Path: my_ip_exdes_tb.DUT.ex_design.axi_vip_mst.inst" will be printed 
    * out. Pass this path to the new function. 
    ***********************************************************************************************/
      agent = new("master vip agent",tb_dyplo_axi_top.MASTER_GP.inst.IF);
      agent.start_master(); // agent start to run

      r_wr_cmd_done = 0;
      r_rd_cmd_done = 0;

   end

   assign o_wr_cmd_done = r_wr_cmd_done;
   assign o_rd_cmd_done = r_rd_cmd_done;
   assign o_rd_cmd_data = Rdatablock;

   always @ (posedge i_wr_cmd_valid) begin

      mtestWID = 0;
      mtestWADDR = i_wr_cmd_address;
      mtestWBurstLength = i_wr_cmd_length;
      mtestWDataSize = xil_axi_size_t'(xil_clog2((32)/8));
      mtestWBurstType = XIL_AXI_BURST_TYPE_INCR;
      mtestWData = i_wr_cmd_data;

      agent.AXI4_WRITE_BURST( mtestWID,
         mtestWADDR,
         mtestWBurstLength,
         mtestWDataSize,
         mtestWBurstType,
         mtestWLock,
         3,
         0,
         0,
         0,
         mtestAWUSER,
         mtestWData,
         mtestWUSER,
         mtestBResp
      );

      r_wr_cmd_done = 1;
      wait(i_wr_cmd_valid == 0);
      r_wr_cmd_done = 0;

   end

   always @ (posedge i_rd_cmd_valid) begin

      mtestRID = 0;
      mtestRADDR = i_rd_cmd_address;
      mtestRBurstLength = i_rd_cmd_length;
      mtestRDataSize = xil_axi_size_t'(xil_clog2((32)/8));
      mtestRBurstType = XIL_AXI_BURST_TYPE_INCR;

      agent.AXI4_READ_BURST(  mtestRID,
         mtestRADDR,
         mtestRBurstLength,
         mtestRDataSize,
         mtestRBurstType,
         mtestRLock,
         3,
         0,
         0,
         0,
         mtestARUSER,
         Rdatablock,
         mtestRResp,
         mtestRUSER
      );

      r_rd_cmd_done = 1;
      wait(i_rd_cmd_valid == 0);
      r_rd_cmd_done = 0;

   end

endmodule