-- (c) Copyright 2011 - 2012 Xilinx, Inc. All rights reserved.
--
-- This file contains confidential and proprietary information
-- of Xilinx, Inc. and is protected under U.S. and
-- international copyright and other intellectual property
-- laws.
--
-- DISCLAIMER
-- This disclaimer is not a license and does not grant any
-- rights to the materials distributed herewith. Except as
-- otherwise provided in a valid license issued to you by
-- Xilinx, and to the maximum extent permitted by applicable
-- law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
-- WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
-- AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
-- BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
-- INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
-- (2) Xilinx shall not be liable (whether in contract or tort,
-- including negligence, or under any other theory of
-- liability) for any loss or damage of any kind or nature
-- related to, arising under or in connection with these
-- materials, including for any direct, or any indirect,
-- special, incidental, or consequential loss or damage
-- (including loss of data, profits, goodwill, or any type of
-- loss or damage suffered as a result of any action brought
-- by a third party) even if such damage or loss was
-- reasonably foreseeable or Xilinx had been advised of the
-- possibility of the same.
--
-- CRITICAL APPLICATIONS
-- Xilinx products are not designed or intended to be fail-
-- safe, or for use in any application requiring fail-safe
-- performance, such as life-support or safety devices or
-- systems, Class III medical devices, nuclear facilities,
-- applications related to the deployment of airbags, or any
-- other applications that could lead to death, personal
-- injury, or severe property or environmental damage
-- (individually and collectively, "Critical
-- Applications"). Customer assumes the sole risk and
-- liability of any use of Xilinx products in Critical
-- Applications, subject only to applicable laws and
-- regulations governing limitations on product liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
-- PART OF THIS FILE AT ALL TIMES.

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_textio.all;

use std.textio.all;

package bfm_axi4_full_master is
   constant C_M_AXI_1_NAME                       : string  := "MASTER_0";
   constant C_M_AXI_1_DATA_WIDTH                 : integer := 32;
   constant C_M_AXI_1_ADDR_WIDTH                 : integer := 32;
   constant C_M_AXI_1_ID_WIDTH                   : integer := 4;
   constant C_M_AXI_1_AWUSER_WIDTH               : integer := 1;
   constant C_M_AXI_1_ARUSER_WIDTH               : integer := 1;
   constant C_M_AXI_1_RUSER_WIDTH                : integer := 1;
   constant C_M_AXI_1_WUSER_WIDTH                : integer := 1;
   constant C_M_AXI_1_BUSER_WIDTH                : integer := 1;
   constant C_INTERCONNECT_M_AXI_1_READ_ISSUING  : integer := 8;
   constant C_INTERCONNECT_M_AXI_1_WRITE_ISSUING : integer := 8;
   constant C_M_AXI_1_EXCLUSIVE_ACCESS_SUPPORTED : integer := 0;

   constant MAX_BURST_LENGTH : integer := 255;
   constant WUSER_BUS_WIDTH  : integer := 1;
   constant RESP_BUS_WIDTH   : integer := 2;
   constant CLK_PER          : time    := 20 ns;

   -- Internal Signals for Master
   signal M_AXI_1_ACLK     : std_logic                                           := '0';
   signal M_AXI_1_ARESETN  : std_logic                                           := '0';
   signal M_AXI_1_AWID     : std_logic_vector(C_M_AXI_1_ID_WIDTH - 1 downto 0);
   signal M_AXI_1_AWADDR   : std_logic_vector(C_M_AXI_1_ADDR_WIDTH - 1 downto 0);
   signal M_AXI_1_AWLEN    : std_logic_vector(7 downto 0);
   signal M_AXI_1_AWSIZE   : std_logic_vector(2 downto 0);
   signal M_AXI_1_AWBURST  : std_logic_vector(1 downto 0);
   signal M_AXI_1_AWLOCK   : std_logic;
   signal M_AXI_1_AWCACHE  : std_logic_vector(3 downto 0);
   signal M_AXI_1_AWPROT   : std_logic_vector(2 downto 0);
   signal M_AXI_1_AWREGION : std_logic_vector(3 downto 0);
   signal M_AXI_1_AWQOS    : std_logic_vector(3 downto 0);
   signal M_AXI_1_AWUSER   : std_logic_vector(C_M_AXI_1_AWUSER_WIDTH - 1 downto 0);
   signal M_AXI_1_AWVALID  : std_logic;
   signal M_AXI_1_AWREADY  : std_logic;
   signal M_AXI_1_WDATA    : std_logic_vector(C_M_AXI_1_DATA_WIDTH - 1 downto 0);
   signal M_AXI_1_WSTRB    : std_logic_vector(3 downto 0);
   signal M_AXI_1_WLAST    : std_logic;
   signal M_AXI_1_WUSER    : std_logic_vector(C_M_AXI_1_WUSER_WIDTH - 1 downto 0);
   signal M_AXI_1_WVALID   : std_logic;
   signal M_AXI_1_WREADY   : std_logic;
   signal M_AXI_1_BID      : std_logic_vector(C_M_AXI_1_ID_WIDTH - 1 downto 0);
   signal M_AXI_1_BRESP    : std_logic_vector(1 downto 0);
   signal M_AXI_1_BVALID   : std_logic;
   signal M_AXI_1_BUSER    : std_logic_vector(C_M_AXI_1_BUSER_WIDTH - 1 downto 0) := (others => '0');
   signal M_AXI_1_BREADY   : std_logic;
   signal M_AXI_1_ARID     : std_logic_vector(C_M_AXI_1_ID_WIDTH - 1 downto 0);
   signal M_AXI_1_ARADDR   : std_logic_vector(C_M_AXI_1_ADDR_WIDTH - 1 downto 0);
   signal M_AXI_1_ARLEN    : std_logic_vector(7 downto 0);
   signal M_AXI_1_ARSIZE   : std_logic_vector(2 downto 0);
   signal M_AXI_1_ARBURST  : std_logic_vector(1 downto 0);
   signal M_AXI_1_ARLOCK   : std_logic;
   signal M_AXI_1_ARCACHE  : std_logic_vector(3 downto 0);
   signal M_AXI_1_ARPROT   : std_logic_vector(2 downto 0);
   signal M_AXI_1_ARREGION : std_logic_vector(3 downto 0);
   signal M_AXI_1_ARQOS    : std_logic_vector(3 downto 0);
   signal M_AXI_1_ARUSER   : std_logic_vector(C_M_AXI_1_ARUSER_WIDTH - 1 downto 0);
   signal M_AXI_1_ARVALID  : std_logic;
   signal M_AXI_1_ARREADY  : std_logic;
   signal M_AXI_1_RID      : std_logic_vector(C_M_AXI_1_ID_WIDTH - 1 downto 0);
   signal M_AXI_1_RDATA    : std_logic_vector(C_M_AXI_1_DATA_WIDTH - 1 downto 0) := (others => '0');
   signal M_AXI_1_RRESP    : std_logic_vector(1 downto 0);
   signal M_AXI_1_RLAST    : std_logic;
   signal M_AXI_1_RUSER    : std_logic_vector(C_M_AXI_1_RUSER_WIDTH - 1 downto 0) := (others => '0');
   signal M_AXI_1_RVALID   : std_logic;
   signal M_AXI_1_RREADY   : std_logic;

   signal M_AXI_2_AWID     : std_logic_vector(C_M_AXI_1_ID_WIDTH - 1 downto 0);
   signal M_AXI_2_AWADDR   : std_logic_vector(C_M_AXI_1_ADDR_WIDTH - 1 downto 0);
   signal M_AXI_2_AWLEN    : std_logic_vector(7 downto 0);
   signal M_AXI_2_AWSIZE   : std_logic_vector(2 downto 0);
   signal M_AXI_2_AWBURST  : std_logic_vector(1 downto 0);
   signal M_AXI_2_AWLOCK   : std_logic;
   signal M_AXI_2_AWCACHE  : std_logic_vector(3 downto 0);
   signal M_AXI_2_AWPROT   : std_logic_vector(2 downto 0);
   signal M_AXI_2_AWREGION : std_logic_vector(3 downto 0);
   signal M_AXI_2_AWQOS    : std_logic_vector(3 downto 0);
   signal M_AXI_2_AWUSER   : std_logic_vector(C_M_AXI_1_AWUSER_WIDTH - 1 downto 0);
   signal M_AXI_2_AWVALID  : std_logic;
   signal M_AXI_2_AWREADY  : std_logic;
   signal M_AXI_2_WDATA    : std_logic_vector(C_M_AXI_1_DATA_WIDTH - 1 downto 0);
   signal M_AXI_2_WSTRB    : std_logic_vector(3 downto 0);
   signal M_AXI_2_WLAST    : std_logic;
   signal M_AXI_2_WUSER    : std_logic_vector(C_M_AXI_1_WUSER_WIDTH - 1 downto 0);
   signal M_AXI_2_WVALID   : std_logic;
   signal M_AXI_2_WREADY   : std_logic;
   signal M_AXI_2_BID      : std_logic_vector(C_M_AXI_1_ID_WIDTH - 1 downto 0);
   signal M_AXI_2_BRESP    : std_logic_vector(1 downto 0);
   signal M_AXI_2_BVALID   : std_logic;
   signal M_AXI_2_BUSER    : std_logic_vector(C_M_AXI_1_BUSER_WIDTH - 1 downto 0);
   signal M_AXI_2_BREADY   : std_logic;
   signal M_AXI_2_ARID     : std_logic_vector(C_M_AXI_1_ID_WIDTH - 1 downto 0);
   signal M_AXI_2_ARADDR   : std_logic_vector(C_M_AXI_1_ADDR_WIDTH - 1 downto 0);
   signal M_AXI_2_ARLEN    : std_logic_vector(7 downto 0);
   signal M_AXI_2_ARSIZE   : std_logic_vector(2 downto 0);
   signal M_AXI_2_ARBURST  : std_logic_vector(1 downto 0);
   signal M_AXI_2_ARLOCK   : std_logic;
   signal M_AXI_2_ARCACHE  : std_logic_vector(3 downto 0);
   signal M_AXI_2_ARPROT   : std_logic_vector(2 downto 0);
   signal M_AXI_2_ARREGION : std_logic_vector(3 downto 0);
   signal M_AXI_2_ARQOS    : std_logic_vector(3 downto 0);
   signal M_AXI_2_ARUSER   : std_logic_vector(C_M_AXI_1_ARUSER_WIDTH - 1 downto 0);
   signal M_AXI_2_ARVALID  : std_logic;
   signal M_AXI_2_ARREADY  : std_logic;
   signal M_AXI_2_RID      : std_logic_vector(C_M_AXI_1_ID_WIDTH - 1 downto 0);
   signal M_AXI_2_RDATA    : std_logic_vector(C_M_AXI_1_DATA_WIDTH - 1 downto 0) := (others => '0');
   signal M_AXI_2_RRESP    : std_logic_vector(1 downto 0);
   signal M_AXI_2_RLAST    : std_logic;
   signal M_AXI_2_RUSER    : std_logic_vector(C_M_AXI_1_RUSER_WIDTH - 1 downto 0);
   signal M_AXI_2_RVALID   : std_logic;
   signal M_AXI_2_RREADY   : std_logic;
   
   signal M_AXI_GP0_AWID     : std_logic_vector(C_M_AXI_1_ID_WIDTH - 1 downto 0);
   signal M_AXI_GP0_AWADDR   : std_logic_vector(C_M_AXI_1_ADDR_WIDTH - 1 downto 0);
   signal M_AXI_GP0_AWLEN    : std_logic_vector(7 downto 0);
   signal M_AXI_GP0_AWSIZE   : std_logic_vector(2 downto 0);
   signal M_AXI_GP0_AWBURST  : std_logic_vector(1 downto 0);
   signal M_AXI_GP0_AWLOCK   : std_logic_vector(0 downto 0);
   signal M_AXI_GP0_AWCACHE  : std_logic_vector(3 downto 0);
   signal M_AXI_GP0_AWPROT   : std_logic_vector(2 downto 0);
   signal M_AXI_GP0_AWREGION : std_logic_vector(3 downto 0);
   signal M_AXI_GP0_AWQOS    : std_logic_vector(3 downto 0);
   signal M_AXI_GP0_AWUSER   : std_logic_vector(C_M_AXI_1_AWUSER_WIDTH - 1 downto 0);
   signal M_AXI_GP0_AWVALID  : std_logic;
   signal M_AXI_GP0_AWREADY  : std_logic;
   signal M_AXI_GP0_WDATA    : std_logic_vector(C_M_AXI_1_DATA_WIDTH - 1 downto 0);
   signal M_AXI_GP0_WSTRB    : std_logic_vector(3 downto 0);
   signal M_AXI_GP0_WLAST    : std_logic;
   signal M_AXI_GP0_WUSER    : std_logic_vector(C_M_AXI_1_WUSER_WIDTH - 1 downto 0);
   signal M_AXI_GP0_WVALID   : std_logic;
   signal M_AXI_GP0_WREADY   : std_logic;
   signal M_AXI_GP0_BID      : std_logic_vector(C_M_AXI_1_ID_WIDTH - 1 downto 0);
   signal M_AXI_GP0_BRESP    : std_logic_vector(1 downto 0);
   signal M_AXI_GP0_BVALID   : std_logic;
   signal M_AXI_GP0_BUSER    : std_logic_vector(C_M_AXI_1_BUSER_WIDTH - 1 downto 0);
   signal M_AXI_GP0_BREADY   : std_logic;
   signal M_AXI_GP0_ARID     : std_logic_vector(C_M_AXI_1_ID_WIDTH - 1 downto 0);
   signal M_AXI_GP0_ARADDR   : std_logic_vector(C_M_AXI_1_ADDR_WIDTH - 1 downto 0);
   signal M_AXI_GP0_ARLEN    : std_logic_vector(7 downto 0);
   signal M_AXI_GP0_ARSIZE   : std_logic_vector(2 downto 0);
   signal M_AXI_GP0_ARBURST  : std_logic_vector(1 downto 0);
   signal M_AXI_GP0_ARLOCK   : std_logic_vector(0 downto 0);
   signal M_AXI_GP0_ARCACHE  : std_logic_vector(3 downto 0);
   signal M_AXI_GP0_ARPROT   : std_logic_vector(2 downto 0);
   signal M_AXI_GP0_ARREGION : std_logic_vector(3 downto 0);
   signal M_AXI_GP0_ARQOS    : std_logic_vector(3 downto 0);
   signal M_AXI_GP0_ARUSER   : std_logic_vector(C_M_AXI_1_ARUSER_WIDTH - 1 downto 0);
   signal M_AXI_GP0_ARVALID  : std_logic;
   signal M_AXI_GP0_ARREADY  : std_logic;
   signal M_AXI_GP0_RID      : std_logic_vector(C_M_AXI_1_ID_WIDTH - 1 downto 0);
   signal M_AXI_GP0_RDATA    : std_logic_vector(C_M_AXI_1_DATA_WIDTH - 1 downto 0) := (others => '0');
   signal M_AXI_GP0_RRESP    : std_logic_vector(1 downto 0);
   signal M_AXI_GP0_RLAST    : std_logic;
   signal M_AXI_GP0_RUSER    : std_logic_vector(C_M_AXI_1_RUSER_WIDTH - 1 downto 0);
   signal M_AXI_GP0_RVALID   : std_logic;
   signal M_AXI_GP0_RREADY   : std_logic;   

   signal MTESTCACHETYPE              : std_logic;
   signal MTESTPROTECTIONTYPE         : std_logic;
   signal MTESTREGION                 : std_logic;
   signal MTESTQOS                    : std_logic;
   signal MTESTAWUSER                 : std_logic;
   signal MTESTARUSER                 : std_logic;
   signal MTESTBUSER                  : std_logic := '0';
   signal V_WUSER                     : std_logic_vector(255 downto 0)                                                       := (others => '0');
   signal WRITE_DONE                  : std_logic;
   signal READ_DONE                   : std_logic;
   signal MTESTID                     : std_logic_vector(3 downto 0);
   signal STESTID                     : std_logic_vector(3 downto 0);
   signal MTESTADDR_W                 : std_logic_vector(C_M_AXI_1_ADDR_WIDTH - 1 downto 0);
   signal MTESTBURSTLENGTH_W          : std_logic_vector(((C_M_AXI_1_DATA_WIDTH / 8) * (MAX_BURST_LENGTH + 1)) - 1 downto 0) := (others => '0');
   signal BURST_SIZE_4_BYTES          : std_logic_vector(3 downto 0);
   signal BURST_TYPE_W                : std_logic_vector(1 downto 0);
   signal MTESTADDR_R                 : std_logic_vector(C_M_AXI_1_ADDR_WIDTH - 1 downto 0);
   signal MTESTBURSTLENGTH_R          : std_logic_vector(((C_M_AXI_1_DATA_WIDTH / 8) * (MAX_BURST_LENGTH + 1)) - 1 downto 0) := (others => '0');
   signal BURST_TYPE_R                : std_logic_vector(1 downto 0);
   signal LOCK_TYPE                   : std_logic;
   signal FOURBIT                     : std_logic;
   signal THREEBIT                    : std_logic;
   signal RD_DATA                     : std_logic_vector((C_M_AXI_1_DATA_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0)       := (others => '0');
   signal WRITE_TASK                  : std_logic;
   signal READ_TASK                   : std_logic;
   signal WRITE_BURST_CONCURRENT      : std_logic;
   signal WRITE_BURST_CONCURRENT_DONE : std_logic;
   signal MTESTDATASIZE               : std_logic_vector(10 downto 0)                                                        := (others => '0');
   signal RESPONSE                    : std_logic_vector(RESP_BUS_WIDTH - 1 downto 0)                                        := (others => '0');
   signal VRESPONSE                   : std_logic_vector(511 downto 0)                                                       := (others => '0');
   signal CHECK_RESPONSE              : std_logic_vector(RESP_BUS_WIDTH - 1 downto 0)                                        := (others => '0');
   signal V_RUSER                     : std_logic_vector((C_M_AXI_1_RUSER_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0)      := (others => '0');
   signal TEST_DATA                   : std_logic_vector((C_M_AXI_1_DATA_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);

   signal WRITE_ID_TAG                          : std_logic_vector(3 downto 0);
   signal READ_ID_TAG                           : std_logic_vector(3 downto 0);
   signal ERROR_VRESPONSE                       : std_logic_vector((RESP_BUS_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
   signal WRITE_BURST_RES                       : std_logic;
   signal READ_BURST_RES                        : std_logic;
   signal WRITE_BURST_RESP_CTRL                 : std_logic;
   signal WRITE_BURST_RESP_CTRL_DONE            : std_logic;
   signal READ_BURST_RESP_CTRL                  : std_logic;
   signal READ_BURST_RESP_CTRL_DONE             : std_logic;
   signal RESPONSE_TYPE                         : std_logic_vector(1 downto 0);
   signal SET_READ_BURST_DATA_TRANSFER_GAP      : std_logic;
   signal SET_READ_BURST_DATA_TRANSFER_GAP_DONE : std_logic;
   signal SET_WRITE_RESPONSE_GAP                : std_logic;
   signal READ_DATA_TRANSFER_COMPLETE           : std_logic;
   signal SET_READ_RESPONSE_GAP                 : std_logic;
   signal GAP_DATA                              : std_logic_vector(7 downto 0) := (others => '0');

   signal PENDING_TRANSACTIONS_COUNT                     : std_logic_vector(3 downto 0);
   signal WRITE_DATA_TRANSFER_COMPLETE                   : std_logic;
   signal WRITE_BURST_DATA_TRANSFER_GAP                  : std_logic;
   signal WRITE_BURST_DATA_TRANSFER_GAP_DONE             : std_logic;
   signal M_GAP_DATA                                     : std_logic_vector(7 downto 0) := "00000000";
   signal LEN                                            : integer;
   signal J                                              : integer;
   signal REPORT_STATUS                                  : std_logic;
   signal REPORT_STATUS_DONE                             : std_logic;
   signal NUMBER_OF_EXPECTED_ERRORS_WARNINGS_AND_PENDING : std_logic;

   signal M_NO_OF_EXPECTED_ERRORS_WARNINGS_AND_PENDING : std_logic := '0';

   PROCEDURE COMPARE_WUSER(signal v_wuser   : in std_logic_vector((WUSER_BUS_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
                           signal s_v_wuser : in std_logic_vector((WUSER_BUS_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
                           variable AWLEN   : IN integer);
   PROCEDURE COMPARE_RUSER(signal V_RUSER   : in std_logic_vector((WUSER_BUS_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
                           signal S_V_RUSER : in std_logic_vector((WUSER_BUS_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
                           variable ARLEN   : IN integer);
   PROCEDURE CHECK_RESPONSE_OKAY(variable check_resp : IN std_logic_vector(RESP_BUS_WIDTH - 1 downto 0));
   PROCEDURE CHECK_RESPONSE_VECTOR_OKAY(signal VRESPONSE : IN std_logic_vector((RESP_BUS_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
                                        signal LEN       : IN integer);
   PROCEDURE STOP_SIMULATION;
   PROCEDURE PRINT_DATA(signal SLAVE_FINISHED : IN std_logic_vector(3 downto 0));
   PROCEDURE COMPARE_DATA(signal WR_DATA  : IN std_logic_vector((C_M_AXI_1_DATA_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
                          signal RD_DATA  : IN std_logic_vector((C_M_AXI_1_DATA_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
                          variable ARSIZE : IN integer;
                          variable ARLEN  : IN integer);
   PROCEDURE REPORT_MASTER_STATUS(signal NO_OF_EXPECTED_ERRORS_WARNINGS_AND_PENDING : IN std_logic;
                                  variable test                                     : IN integer);

   component cdn_axi4_master_bfm_wrap is
      generic(
         C_M_AXI_NAME                       : string  := "MASTER_0";
         C_M_AXI_DATA_WIDTH                 : integer := 32;
         C_M_AXI_ADDR_WIDTH                 : integer := 32;
         C_M_AXI_ID_WIDTH                   : integer := 4;
         C_M_AXI_AWUSER_WIDTH               : integer := 1;
         C_M_AXI_ARUSER_WIDTH               : integer := 1;
         C_M_AXI_RUSER_WIDTH                : integer := 1;
         C_M_AXI_WUSER_WIDTH                : integer := 1;
         C_M_AXI_BUSER_WIDTH                : integer := 1;
         C_INTERCONNECT_M_AXI_READ_ISSUING  : integer := 8;
         C_INTERCONNECT_M_AXI_WRITE_ISSUING : integer := 8;
         C_M_AXI_EXCLUSIVE_ACCESS_SUPPORTED : integer := 0
      );
      port(
         M_AXI_ACLK                                     : in  std_logic;
         M_AXI_ARESETN                                  : in  std_logic;
         M_AXI_AWID                                     : out std_logic_vector(C_M_AXI_ID_WIDTH - 1 downto 0); -- Master Write address ID.
         M_AXI_AWADDR                                   : out std_logic_vector(C_M_AXI_ADDR_WIDTH - 1 downto 0); -- Master Write address.
         M_AXI_AWLEN                                    : out std_logic_vector(7 downto 0); -- Master Burst length.
         M_AXI_AWSIZE                                   : out std_logic_vector(2 downto 0); -- Master Burst size.
         M_AXI_AWBURST                                  : out std_logic_vector(1 downto 0); -- Master Burst type.
         M_AXI_AWLOCK                                   : out std_logic; -- Master Lock type.
         M_AXI_AWCACHE                                  : out std_logic_vector(3 downto 0); -- Master Cache type.
         M_AXI_AWPROT                                   : out std_logic_vector(2 downto 0); -- Master Protection type.
         M_AXI_AWREGION                                 : out std_logic_vector(3 downto 0); -- Master Region signals.
         M_AXI_AWQOS                                    : out std_logic_vector(3 downto 0); -- Master QoS signals.
         M_AXI_AWUSER                                   : out std_logic_vector(C_M_AXI_AWUSER_WIDTH - 1 downto 0); -- Master User defined signals.
         M_AXI_AWVALID                                  : out std_logic; -- Master Write address valid.
         M_AXI_AWREADY                                  : in  std_logic; -- Slave Write address ready.
         M_AXI_WDATA                                    : out std_logic_vector(C_M_AXI_DATA_WIDTH - 1 downto 0); -- Master Write data.
         M_AXI_WSTRB                                    : out std_logic_vector(3 downto 0); -- Master Write strobes.
         M_AXI_WLAST                                    : out std_logic; -- Master Write last.
         M_AXI_WUSER                                    : out std_logic_vector(C_M_AXI_WUSER_WIDTH - 1 downto 0); -- Master Write User defined signals.
         M_AXI_WVALID                                   : out std_logic; -- Master Write valid.
         M_AXI_WREADY                                   : in  std_logic; -- Slave Write ready.
         M_AXI_BID                                      : in  std_logic_vector(C_M_AXI_ID_WIDTH - 1 downto 0); -- Slave Response ID.
         M_AXI_BRESP                                    : in  std_logic_vector(1 downto 0); -- Slave Write response.
         M_AXI_BVALID                                   : in  std_logic; -- Slave Write response valid.
         M_AXI_BUSER                                    : in  std_logic_vector(C_M_AXI_BUSER_WIDTH - 1 downto 0); -- Slave Write user defined signals.
         M_AXI_BREADY                                   : out std_logic; -- Master Response ready.
         M_AXI_ARID                                     : out std_logic_vector(C_M_AXI_ID_WIDTH - 1 downto 0); -- Master Read address ID.
         M_AXI_ARADDR                                   : out std_logic_vector(C_M_AXI_ADDR_WIDTH - 1 downto 0); -- Master Read address.
         M_AXI_ARLEN                                    : out std_logic_vector(7 downto 0); -- Master Burst length.
         M_AXI_ARSIZE                                   : out std_logic_vector(2 downto 0); -- Master Burst size.
         M_AXI_ARBURST                                  : out std_logic_vector(1 downto 0); -- Master Burst type.
         M_AXI_ARLOCK                                   : out std_logic; -- Master Lock typ
         M_AXI_ARCACHE                                  : out std_logic_vector(3 downto 0); -- Master Cache type.
         M_AXI_ARPROT                                   : out std_logic_vector(2 downto 0); -- Master Protection type.
         M_AXI_ARREGION                                 : out std_logic_vector(3 downto 0); -- Master Region signals.
         M_AXI_ARQOS                                    : out std_logic_vector(3 downto 0); -- Master QoS signals.
         M_AXI_ARUSER                                   : out std_logic_vector(C_M_AXI_ARUSER_WIDTH - 1 downto 0); -- Master User defined signals.
         M_AXI_ARVALID                                  : out std_logic; -- Master Read address valid.
         M_AXI_ARREADY                                  : in  std_logic; -- Slave Read address ready.
         M_AXI_RID                                      : in  std_logic_vector(C_M_AXI_ID_WIDTH - 1 downto 0); -- Slave Read ID tag.
         M_AXI_RDATA                                    : in  std_logic_vector(C_M_AXI_DATA_WIDTH - 1 downto 0); -- Slave Read data.
         M_AXI_RRESP                                    : in  std_logic_vector(1 downto 0); -- Slave Read response.
         M_AXI_RLAST                                    : in  std_logic; -- Slave Read last.
         M_AXI_RUSER                                    : in  std_logic_vector(C_M_AXI_RUSER_WIDTH - 1 downto 0); -- Slave Read user defined signals.
         M_AXI_RVALID                                   : in  std_logic; -- Slave Read valid.
         M_AXI_RREADY                                   : out std_logic; -- Master Read ready.
         MTESTCACHETYPE                                 : in  std_logic;
         MTESTPROTECTIONTYPE                            : in  std_logic;
         MTESTREGION                                    : in  std_logic;
         MTESTQOS                                       : in  std_logic;
         MTESTAWUSER                                    : in  std_logic;
         MTESTARUSER                                    : in  std_logic;
         MTESTBUSER                                     : out std_logic;
         V_WUSER                                        : in  std_logic_vector(255 downto 0);
         WRITE_DONE                                     : out std_logic;
         READ_DONE                                      : out std_logic;
         MTESTID                                        : in  std_logic_vector(3 downto 0);
         STESTID                                        : in  std_logic_vector(3 downto 0);
         MTESTADDR_W                                    : in  std_logic_vector(C_M_AXI_ADDR_WIDTH - 1 downto 0);
         MTESTBURSTLENGTH_W                             : in  std_logic_vector(((C_M_AXI_DATA_WIDTH / 8) * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
         BURST_SIZE_4_BYTES                             : in  std_logic_vector(3 downto 0);
         BURST_TYPE_W                                   : in  std_logic_vector(1 downto 0);
         MTESTADDR_R                                    : in  std_logic_vector(C_M_AXI_ADDR_WIDTH - 1 downto 0);
         MTESTBURSTLENGTH_R                             : in  std_logic_vector(((C_M_AXI_DATA_WIDTH / 8) * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
         BURST_TYPE_R                                   : in  std_logic_vector(1 downto 0);
         LOCK_TYPE                                      : in  std_logic;
         FOURBIT                                        : in  std_logic;
         THREEBIT                                       : in  std_logic;
         RD_DATA                                        : out std_logic_vector((C_M_AXI_DATA_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
         WRITE_TASK                                     : in  std_logic;
         READ_TASK                                      : in  std_logic;
         WRITE_BURST_CONCURRENT                         : in  std_logic;
         WRITE_BURST_CONCURRENT_DONE                    : out std_logic;
         MTESTDATASIZE                                  : in  std_logic_vector(10 downto 0);
         RESPONSE                                       : out std_logic_vector(RESP_BUS_WIDTH - 1 downto 0);
         VRESPONSE                                      : out std_logic_vector(511 downto 0);
         V_RUSER                                        : out std_logic_vector((C_M_AXI_RUSER_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
         TEST_DATA                                      : in  std_logic_vector((C_M_AXI_DATA_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
         WRITE_DATA_TRANSFER_COMPLETE                   : out std_logic;
         WRITE_BURST_DATA_TRANSFER_GAP                  : in  std_logic;
         WRITE_BURST_DATA_TRANSFER_GAP_DONE             : out std_logic;
         GAP_DATA                                       : in  std_logic_vector(7 downto 0);
         REPORT_STATUS                                  : in  std_logic;
         REPORT_STATUS_DONE                             : out std_logic;
         NUMBER_OF_EXPECTED_ERRORS_WARNINGS_AND_PENDING : out std_logic
      );
   end component;

end;

package body bfm_axi4_full_master is
   PROCEDURE COMPARE_RUSER(signal V_RUSER   : in std_logic_vector((WUSER_BUS_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
                           signal S_V_RUSER : in std_logic_vector((WUSER_BUS_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
                           variable ARLEN   : IN integer) IS
      variable outline_user : line;
   BEGIN
      IF v_ruser(ARLEN downto 0) /= s_v_ruser(ARLEN downto 0) THEN
         REPORT "TESTBENCH ERROR! RUSER data expected is not equal to actual. ";
         write(outline_user, string'("** v_ruser = "));
         write(outline_user, v_ruser);
         write(outline_user, string'("** s_v_ruser = "));
         write(outline_user, s_v_ruser);
         write(outline_user, string'("******** MISMATCH********* "));
         writeline(output, outline_user);
      END IF;
   END PROCEDURE COMPARE_RUSER;

   PROCEDURE COMPARE_WUSER(signal v_wuser   : in std_logic_vector((WUSER_BUS_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
                           signal s_v_wuser : in std_logic_vector((WUSER_BUS_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
                           variable AWLEN   : IN integer) IS
      variable outline_user : line;
   BEGIN
      IF v_wuser(AWLEN downto 0) /= s_v_wuser(AWLEN downto 0) THEN
         REPORT "TESTBENCH ERROR! WUSER data expected is not equal to actual. ";
         write(outline_user, string'("** v_wuser = "));
         write(outline_user, v_wuser);
         write(outline_user, string'("** s_v_wuser = "));
         write(outline_user, s_v_wuser);
         write(outline_user, string'("******** MISMATCH********* "));
         writeline(output, outline_user);
      ELSE
         write(outline_user, string'(" WUSER comparision "));
         write(outline_user, string'("** PASS ** "));
         writeline(output, outline_user);
      END IF;
   END PROCEDURE COMPARE_WUSER;

   PROCEDURE CHECK_RESPONSE_OKAY(variable check_resp : IN std_logic_vector(RESP_BUS_WIDTH - 1 downto 0)) IS
      variable outline1 : line;
   BEGIN
      IF response /= "00" THEN
         REPORT "Uacceptable values... Wrong Response";
         RETURN;                        -- Exit the procedure
      ELSE
         write(outline1, string'("*****************************************"));
         write(outline1, string'("==========MASTER TO SLAVE RESPONSE VALUE :: OK "));
         write(outline1, response);
         write(outline1, string'(" :: PASS :: *****************************************"));
      END IF;
      writeline(output, outline1);

   END PROCEDURE CHECK_RESPONSE_OKAY;

   PROCEDURE COMPARE_DATA(signal wr_data  : IN std_logic_vector((C_M_AXI_1_DATA_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
                          signal rd_data  : IN std_logic_vector((C_M_AXI_1_DATA_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
                          variable ARSIZE : IN integer;
                          variable ARLEN  : IN integer) IS
      variable outline2 : line;
      variable usb      : integer;
      variable size     : integer;
      variable length   : integer;
   BEGIN
      IF ARSIZE = 0 THEN
         size := 1;
      ELSIF ARSIZE = 1 THEN
         size := 2;
      ELSIF ARSIZE = 2 THEN
         size := 4;
      ELSIF ARSIZE = 3 THEN
         size := 8;
      ELSIF ARSIZE = 4 THEN
         size := 16;
      ELSIF ARSIZE = 5 THEN
         size := 32;
      ELSIF ARSIZE = 6 THEN
         size := 64;
      ELSIF ARSIZE = 7 THEN
         size := 128;
      END IF;

      length := ARLEN + 1;

      usb := ((size * 8) * length) - 1;
      write(outline2, string'("***********************  FUNCTION COMPARE DATA  ******************************"));
      writeline(output, outline2);
      write(outline2, string'("ARSIZE : "));
      write(outline2, ARSIZE);
      write(outline2, string'("     ARLENGTH :"));
      write(outline2, ARLEN);
      write(outline2, string'(" Comparing data width (SIZE*8)*LENGTH) :"));
      write(outline2, usb);
      writeline(output, outline2);
      writeline(output, outline2);

      IF wr_data(usb downto 0) /= rd_data(usb downto 0) THEN
         write(outline2, string'("******************************************************************************************"));
         writeline(output, outline2);
         write(outline2, string'("***********************   READ AND WRITE DATA MATCH :: FAIL (MIS MATCH RD AND WR DATA) ******************************"));
         writeline(output, outline2);
         write(outline2, string'("******************************************************************************************"));
         writeline(output, outline2);
         write(outline2, wr_data);
         write(outline2, string'(" ============="));
         write(outline2, rd_data);
         write(outline2, string'(" ============="));
         writeline(output, outline2);
         RETURN;                        -- Exit the procedure
      ELSE
         write(outline2, string'("******************************************************************************************"));
         writeline(output, outline2);
         write(outline2, string'("***********************   READ AND WRITE DATA MATCH :: PASS  ******************************"));
         writeline(output, outline2);
         write(outline2, string'("******************************************************************************************"));
         writeline(output, outline2);
      END IF;

   END PROCEDURE compare_data;

   PROCEDURE stop_simulation is
   begin
      wait for 100 ns;
      assert false report "simulation ended" severity failure;
   end stop_simulation;

   PROCEDURE print_data(signal SLAVE_FINISHED : IN std_logic_vector(3 downto 0)) IS
      variable outline_tr : line;
   BEGIN
      write(outline_tr, string'("##============================================ ::"));
      write(outline_tr, string'("  EXAMPLE TEST  :: "));
      write(outline_tr, SLAVE_FINISHED);
      write(outline_tr, string'("  FINISHED WITH SLAVE ID :: "));
      write(outline_tr, SLAVE_FINISHED);
      write(outline_tr, string'("============================================ ::"));
      writeline(output, outline_tr);
   END PROCEDURE print_data;

   PROCEDURE CHECK_RESPONSE_VECTOR_OKAY(signal vresponse : IN std_logic_vector((RESP_BUS_WIDTH * (MAX_BURST_LENGTH + 1)) - 1 downto 0);
                                        signal LEN       : IN integer) IS
      variable outliner : line;
      variable a        : integer;
      variable b        : integer;
   BEGIN
      for j in 0 to LEN loop
         a := j * RESP_BUS_WIDTH + 1;
         b := j * RESP_BUS_WIDTH;
         write(outliner, vresponse(a downto b));
         write(outliner, string'("***"));
         IF vresponse(a downto b) /= "00" THEN
            REPORT "Uacceptable values... Wrong Response";
            write(outliner, string'("************************ ::VRESPONSE FAIL ************************"));
            write(outliner, vresponse(a downto b));
            writeline(output, outliner);
            RETURN;                     -- Exit the procedure
         ELSE
            write(outliner, string'("************************************"));
            write(outliner, string'("****** MASTER TO SLAVE RESPONSE VALUE :: VRESPONSE ["));
            write(outliner, a);
            write(outliner, string'(":"));
            write(outliner, b);
            write(outliner, string'("] = "));
            write(outliner, vresponse(a downto b));
            write(outliner, string'("*******:: PASS ::*****************************"));
         END IF;
         writeline(output, outliner);
      end loop;

   END PROCEDURE CHECK_RESPONSE_VECTOR_OKAY;
   --------------------------------------------------------------------------
   -- TEST BENCH LEVEL API: REPORT_MASTER_STATUS
   --------------------------------------------------------------------------
   -- Description:
   -- REPORT_MASTER_STATUS(number_of_expected_errors_warnings_and_pending)
   -- This task calls the masters report_status function which returns the
   -- total of the errors + warnings + pending counters. This return number
   -- is compared with the input
   -- number_of_expected_errors_warnings_and_pending
   --------------------------------------------------------------------------
   PROCEDURE REPORT_MASTER_STATUS(signal NO_OF_EXPECTED_ERRORS_WARNINGS_AND_PENDING : IN std_logic;
                                  variable test                                     : IN integer) IS
      variable outline1 : line;
   BEGIN
      If NO_OF_EXPECTED_ERRORS_WARNINGS_AND_PENDING /= NUMBER_OF_EXPECTED_ERRORS_WARNINGS_AND_PENDING THEN
         write(outline1, string'("---------------------------------------------------------------"));
         writeline(output, outline1);
         write(outline1, string'("EXAMPLE TEST "));
         write(outline1, test);
         write(outline1, string'("  : MASTER FAILED"));
         writeline(output, outline1);
         write(outline1, string'("---------------------------------------------------------------"));
         writeline(output, outline1);

         write(outline1, string'("---------------------------------------------------------------"));
         writeline(output, outline1);
         write(outline1, string'(" EXPECTED  : "));
         write(outline1, NO_OF_EXPECTED_ERRORS_WARNINGS_AND_PENDING);
         writeline(output, outline1);
         write(outline1, string'(" ACTUAL (MASTER) : "));
         write(outline1, NUMBER_OF_EXPECTED_ERRORS_WARNINGS_AND_PENDING);
         writeline(output, outline1);
         write(outline1, string'("---------------------------------------------------------------"));
         writeline(output, outline1);
      ELSE
         write(outline1, string'("---------------------------------------------------------------"));
         writeline(output, outline1);
         write(outline1, string'("EXAMPLE TEST "));
         write(outline1, test);
         write(outline1, string'("  : MASTER PASSED"));
         writeline(output, outline1);
         write(outline1, string'("---------------------------------------------------------------"));
         writeline(output, outline1);

      END IF;

   END PROCEDURE REPORT_MASTER_STATUS;

end package body;

