----------------------------------------------------------------------------------
-- Company:
-- Engineer:
--
-- Create Date: 01/16/2015 10:32:46 AM
-- Design Name:
-- Module Name: dyplo_fifo_32_to_64_tb - Behavioral
-- Project Name:
-- Target Devices:
-- Tool Versions:
-- Description:
--
-- Dependencies:
--
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
--
----------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

entity dyplo_fifo_64_to_32_tb is
end dyplo_fifo_64_to_32_tb;

architecture testbench of dyplo_fifo_64_to_32_tb is

  component dyplo_fifo_64_to_32
  generic(
    FIFO_DEPTH            : integer := 256;
    FIFO_TYPE             : string  := "DISTRIBUTED"
  );
  port(
    fifo_clk              : in  std_logic;
    fifo_rst              : in  std_logic;
    fifo_wdata            : in  std_logic_vector(63 downto 0);
    fifo_wuser            : in   std_logic_vector(3 downto 0);
    fifo_wdata_complete    : in  std_logic;
    fifo_wr               : in  std_logic;
    fifo_rdata            : out std_logic_vector(31 downto 0);
    fifo_ruser            : out std_logic_vector(3 downto 0);
    fifo_rd               : in  std_logic;
    fifo_empty            : out std_logic;
    fifo_full             : out std_logic;
    fifo_vacant            : out std_logic_vector(11 downto 0)
  );
  end component;

  signal clk              : std_logic := '0';
  signal rst              : std_logic := '1';

  signal wdata            : std_logic_vector(63 downto 0) := (others => '0');
  signal wuser            : std_logic_vector(3 downto 0) := (others => '0');
  signal wdata_complete   : std_logic := '1';
  signal write            : std_logic := '0';

  signal rdata            : std_logic_vector(31 downto 0);
  signal ruser            : std_logic_vector(3 downto 0) := (others => '0');
  signal read             : std_logic := '0';

  signal empty            : std_logic;
  signal full             : std_logic;
  signal vacant           : std_logic_vector(11 downto 0);

begin

  process(clk)
  begin
      clk <= not clk after 20 ns;
  end process;

  process
  begin
    wait for 120 ns;
    rst <= '0';

    wait until falling_edge(clk);

    for i in 0 to 7 loop
      write <= '1';
      wait until falling_edge(clk);
      wdata(31 downto 0) <= std_logic_vector(unsigned(wdata(31 downto 0)) + 1);
      wdata(63 downto 32) <= std_logic_vector(unsigned(wdata(63 downto 32)) + 2);
    end loop;

    write <= '0';
    wait until falling_edge(clk);

    for i in 0 to 15 loop
      read <= '1';
      wait until falling_edge(clk);
    end loop;

    read <= '0';

-- FIRST STAGE TEST DONE

    wait until falling_edge(clk);

    for i in 0 to 4 loop
      write <= '1';
      wait until falling_edge(clk);
      wdata <= std_logic_vector(unsigned(wdata) + 1);
    end loop;

    write <= '0';
    wait until falling_edge(clk);

    for i in 0 to 3 loop
      write <= '1';
      wait until falling_edge(clk);
      wdata <= std_logic_vector(unsigned(wdata) + 1);
    end loop;

    write <= '0';
    wait until falling_edge(clk);

    wait until falling_edge(clk);

    for i in 0 to 4 loop
      read <= '1';
      wait until falling_edge(clk);
    end loop;

    read      <= '0';

-- SECOND STAGE TEST DONE

    wait until falling_edge(clk);
    write <= '1';

    wait until falling_edge(clk);
    write     <= '0';


    wait until falling_edge(clk);
    read  <=  '1';
    wait until falling_edge(clk);
    read      <= '0';

    wait until falling_edge(clk);

    for i in 0 to 9 loop
      write <= '1';
      wait until falling_edge(clk);
      wdata <= std_logic_vector(unsigned(wdata) + 1);
    end loop;

    for i in 0 to 4 loop
      read <= '1';
      wait until falling_edge(clk);
      wdata <= std_logic_vector(unsigned(wdata) + 1);
    end loop;

    read  <= '0';
    write <= '0';


    wait;

  end process;


  fifo : dyplo_fifo_64_to_32
  generic map(
    FIFO_DEPTH            => 32,
    FIFO_TYPE             => "DISTRIBUTED"
  )
  port map(
    fifo_clk              => clk,
    fifo_rst              => rst,
    fifo_wdata            => wdata,
    fifo_wuser            => wuser,
    fifo_wdata_complete   => wdata_complete,
    fifo_wr               => write,
    fifo_rdata            => rdata,
    fifo_ruser            => ruser,
    fifo_rd               => read,
    fifo_empty            => empty,
    fifo_full             => full,
    fifo_vacant           => vacant
  );
end testbench;
