-- 	File: lane_input_simulator.vhd
--
--	© COPYRIGHT 2014 TOPIC EMBEDDED PRODUCTS B.V. ALL RIGHTS RESERVED.
--
--	This file contains confidential and proprietary information of
--	Topic Embedded Products B.V. and is protected under Dutch and
--	International copyright and other international intellectual property laws.
--
--	Disclaimer
--
--	This disclaimer is not a license and does not grant any rights to the
--	materials distributed herewith. Except as otherwise provided in a valid
--	license issued to you by Topic Embedded Products B.V., and to the maximum
--	extend permitted by applicable law:
--
--	1.	Dyplo is furnished on an "as is", as available basis. Topic makes no
--	warranty, express or implied, with respect to the capability of Dyplo. All
--	warranties of any type, express or implied, including the warranties of
--	merchantability, fitness for a particular purpose and non-infringement of
--	third party rights are expressly disclaimed.
--
--	2.	Topic's maximum total liability shall be limited to general money
--	damages in an amount not to exceed the total amount paid for in the year
--	in which the damages have occurred.  Under no circumstances including
--	negligence shall Topic be liable for direct, indirect, incidental, special,
--	consequential or punitive damages, or for loss of profits, revenue, or data,
--	that are directly or indirectly related to the use of, or the inability to
--	access and use Dyplo and related services, whether in an action in contract,
--	tort, product liability, strict liability, statute or otherwise even if
--	Topic has been advised of the possibility of those damages.
--
--	This copyright notice and disclaimer must be retained as part of this file at all times.


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library dyplo;
use dyplo.dyplo_types.all;
use dyplo.dyplo_config.all;

library dyplo_test;
use dyplo_test.test_types.all;

entity lane_input_simulator is
	generic(
		CONNECT_LANES : natural range 1 to 4  := 2;
		CONNECT_ID    : natural range 0 to 31 := 1
	);
	port(
		clk               : in  std_logic;
		rst               : in  std_logic;
		lane_mode         : in  lane_mode_type;
		lane_destinations : in  lane_destinations_type;
		lanes             : out data_lanes_type;
		send_to_lanes     : out unsigned_matrix_4_8_4_64
	);
end lane_input_simulator;

architecture Behavioral of lane_input_simulator is
begin
	lanes_gen : for i in 0 to CONNECT_LANES - 1 generate
		signal output_counters : unsigned_matrix_8_4_64 := (others => (others => (others => '0')));
		signal lane_tvalid  : std_logic             := '0';
	begin
		process(clk)
			variable gap_cnt : natural := 0;
		begin
			if (rising_edge(clk)) then
				if (rst = '1') then
					output_counters 	<= (others => (others => (others => '0')));
					lane_tvalid 		<= '0';
				else
					case (lane_mode(i)) is
						---------------------------------
						when 0 =>       -- don't output data on this lane
							lane_tvalid <= '0';
						---------------------------------
						when 1 =>       -- output a little data on this lane (1 word each 10 clocks)
							if (gap_cnt = 9) then
								lane_tvalid <= '1';
								gap_cnt     := 0;
							else
								lane_tvalid <= '0';
								gap_cnt     := gap_cnt + 1;
							end if;
						---------------------------------
						when 2 =>       -- output some more data on this lane (1 word each 3 clocks)
							if (gap_cnt = 2) then
								lane_tvalid <= '1';
								gap_cnt     := 0;
							else
								lane_tvalid <= '0';
								gap_cnt     := gap_cnt + 1;
							end if;
						---------------------------------
						when 3 =>       -- completely fill this lane (1 word each clock)
							lane_tvalid <= '1';
					end case;

					if (lane_tvalid = '1') then
						output_counters(to_integer(unsigned(lane_destinations(i)(c_int_destination_width - 1 downto c_stream_id_width))))(to_integer(unsigned(lane_destinations(i)(c_stream_id_width - 1 downto 0)))) <= output_counters(to_integer(unsigned(lane_destinations(i)(c_int_destination_width - 1 downto c_stream_id_width))))(to_integer(unsigned(lane_destinations(i)(c_stream_id_width - 1 downto 0)))) + 1;
					end if;

				end if;
			end if;
		end process;

		lanes(i).tvalid     <= lane_tvalid;
		lanes(i).tdata      <= std_logic_vector(output_counters(to_integer(unsigned(lane_destinations(i)(c_int_destination_width - 1 downto c_stream_id_width))))(to_integer(unsigned(lane_destinations(i)(c_stream_id_width - 1 downto 0)))));
		lanes(i).tnode_id   <= lane_destinations(i)(c_int_destination_width - 1 downto c_stream_id_width);
		lanes(i).tstream_id <= lane_destinations(i)(c_stream_id_width - 1 downto 0);
		lanes(i).tuser      <= (others => '0');
		lanes(i).tfail      <= '0';

		send_to_lanes(i)    <= output_counters;

	end generate;	

end Behavioral;







