----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 09/15/2014 04:01:03 PM
-- Design Name: 
-- Module Name: aurora_controller - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity aurora_controller is
  generic ( 
    CC_PERIOD       : in  integer   := 5000;
    CC_DURATION     : in  integer   := 6
  );
  port (
    reset_in                : in    std_logic;
    user_clk                : in    std_logic;
    init_clk                : in    std_logic;

    system_reset_out        : out   std_logic;
    gt_reset_out            : out   std_logic;    
    -----------------------------------
    do_cc                   : out   std_logic 
  );
end aurora_controller;

architecture Behavioral of aurora_controller is

  signal period_counter         : integer := CC_PERIOD - 1;
  signal duration_counter       : integer := CC_DURATION - 1; 
  
  signal cc_active              : std_logic := '0';
  
  signal do_gt_reset            : std_logic;
  signal gt_reset_busy          : std_logic;
  signal gt_reset_cnt           : integer := 6;
  
  type gt_reset_state_type is (WAIT_FOR_RESET, RESET);
  signal gt_reset_state         : gt_reset_state_type := WAIT_FOR_RESET;

begin 

  process(user_clk)
  begin
    if(rising_edge(user_clk)) then
      if(reset_in = '1') then
         period_counter   <= CC_PERIOD - 1;
         duration_counter <= CC_DURATION - 1;
         cc_active        <= '0';
         do_gt_reset      <= '1';
      else
        if(cc_active = '1') then
        
          if(duration_counter = 0) then
            cc_active         <= '0';
            duration_counter  <= CC_DURATION - 1;
          else
            duration_counter  <= duration_counter - 1; 
          end if;  
          
        else
          if(period_counter = 0) then
            cc_active       <= '1';
            period_counter  <= CC_PERIOD - 1;
          else
            period_counter  <= period_counter - 1; 
          end if; 
        end if;
        
        if(gt_reset_busy = '1') then
          do_gt_reset <= '0';
        end if;
        
      end if;
    end if;
  end process;
  
  do_cc <= cc_active;
  
  system_reset_out  <= reset_in;
  
  process(init_clk)
  begin
    if(rising_edge(init_clk)) then
      if(reset_in = '1') then
        gt_reset_busy   <= '0';
        gt_reset_state  <= WAIT_FOR_RESET;
        gt_reset_out    <= '0';
      else
      
        case(gt_reset_state) is
        
          when WAIT_FOR_RESET =>
            if(do_gt_reset = '1') then
              gt_reset_out    <= '1';
              gt_reset_cnt    <= 6;
              gt_reset_busy   <= '1';
            
              gt_reset_state  <= RESET;
            end if;
          
          when RESET =>
            if(gt_reset_cnt /= 0) then
              gt_reset_cnt <= gt_reset_cnt - 1; 
            else
              if(do_gt_reset = '0') then
                gt_reset_out    <= '0';
                gt_reset_busy   <= '0';
                
                gt_reset_state  <= WAIT_FOR_RESET;
              end if;
            end if;
            
        end case;
      end if;
    end if;
  end process;

end Behavioral;
