LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;

ENTITY axis_data_counter IS
   GENERIC(
      p_axis_tdata_width : natural := 24
   );
   PORT(
      AXIS_ACLK     : IN  STD_LOGIC;
      -- input stream
      S_AXIS_TDATA  : IN  STD_LOGIC_VECTOR(p_axis_tdata_width - 1 DOWNTO 0);
      S_AXIS_TVALID : IN  STD_LOGIC;
      S_AXIS_TREADY : OUT STD_LOGIC;
      S_AXIS_TLAST  : IN  STD_LOGIC;
      -- output stream
      M_AXIS_TDATA  : OUT STD_LOGIC_VECTOR(p_axis_tdata_width - 1 DOWNTO 0);
      M_AXIS_TVALID : OUT STD_LOGIC;
      M_AXIS_TREADY : IN  STD_LOGIC;
      M_AXIS_TLAST  : OUT STD_LOGIC;
      -- data count
      o_count       : OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
   );
END ENTITY;

ARCHITECTURE Behavioral OF axis_data_counter IS
   SIGNAL r_cnt : unsigned(31 DOWNTO 0) := (OTHERS => '0');
BEGIN

   PROCESS(AXIS_ACLK)
   BEGIN
      IF (rising_edge(AXIS_ACLK)) THEN
         IF (S_AXIS_TVALID = '1' AND M_AXIS_TREADY = '1') THEN
            r_cnt <= r_cnt + 1;
         END IF;
      END IF;
   END PROCESS;

   o_count <= std_logic_vector(r_cnt);

   M_AXIS_TDATA  <= S_AXIS_TDATA;
   M_AXIS_TLAST  <= S_AXIS_TLAST;
   M_AXIS_TVALID <= S_AXIS_TVALID;
   S_AXIS_TREADY <= M_AXIS_TREADY;

END Behavioral;
