LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;

ENTITY axis_video_add_tuser IS
   GENERIC(
      p_axis_tdata_width      : natural := 24;
      p_axis_tuser_width_in   : natural := 1;
      p_axis_added_tuser_bits : natural := 1;
      p_video_max_height      : natural := 1080
   );
   PORT(
      AXIS_ACLK      : IN  Std_logic;
      -- resolution info
      i_video_height : IN  std_logic_vector(11 DOWNTO 0);
      -- status output
      o_frame_passed : OUT std_logic := '0';
      -- input stream
      S_AXIS_TDATA   : IN  STD_LOGIC_VECTOR(p_axis_tdata_width - 1 DOWNTO 0);
      S_AXIS_TVALID  : IN  STD_LOGIC;
      S_AXIS_TREADY  : OUT STD_LOGIC;
      S_AXIS_TLAST   : IN  STD_LOGIC;
      S_AXIS_TUSER   : IN  STD_LOGIC_VECTOR(p_axis_tuser_width_in - 1 DOWNTO 0);
      -- output stream
      M_AXIS_TDATA   : OUT STD_LOGIC_VECTOR(p_axis_tdata_width - 1 DOWNTO 0);
      M_AXIS_TVALID  : OUT STD_LOGIC;
      M_AXIS_TREADY  : IN  STD_LOGIC;
      M_AXIS_TLAST   : OUT STD_LOGIC;
      M_AXIS_TUSER   : OUT STD_LOGIC_VECTOR((p_axis_tuser_width_in + p_axis_added_tuser_bits) - 1 DOWNTO 0)
   );
END ENTITY;

ARCHITECTURE Behavioral OF axis_video_add_tuser IS
   SIGNAL n_eof         : std_logic;
   SIGNAL r_tlast_count : natural RANGE 0 TO p_video_max_height - 1;
   
   signal cdc_video_height : std_logic_vector(i_video_height'range);
   signal r_video_height : std_logic_vector(i_video_height'range);
BEGIN

   PROCESS(AXIS_ACLK)
   BEGIN
      IF (rising_edge(AXIS_ACLK)) THEN
         -- Assign the r_stop_input_stream at the entering of the first pixel
         IF (S_AXIS_TVALID = '1' AND M_AXIS_TREADY = '1') THEN
            IF (S_AXIS_TUSER(0) = '1') THEN
               r_tlast_count <= 0;
               r_video_height <= cdc_video_height;
            ELSIF (S_AXIS_TLAST = '1') THEN
               r_tlast_count <= r_tlast_count + 1;
            END IF;
            
            o_frame_passed  <= n_eof;
         ELSE
            o_frame_passed  <= '0';
         END IF;
      END IF;
   END PROCESS;

   n_eof <= S_AXIS_TLAST WHEN r_tlast_count = to_integer(unsigned(r_video_height) - 1) ELSE '0';

   M_AXIS_TDATA                                     <= S_AXIS_TDATA;
   M_AXIS_TUSER(p_axis_tuser_width_in - 1 DOWNTO 0) <= S_AXIS_TUSER(p_axis_tuser_width_in - 1 DOWNTO 0);
   M_AXIS_TUSER(p_axis_tuser_width_in)              <= n_eof;
   M_AXIS_TLAST                                     <= S_AXIS_TLAST;
   M_AXIS_TVALID                                    <= S_AXIS_TVALID;
   S_AXIS_TREADY                                    <= M_AXIS_TREADY;
   
   cdc_video_height <= i_video_height;
      
END Behavioral;
