LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY axis_video_frame_dropper IS
   GENERIC(
      p_axis_tdata_width    : natural := 24;
      p_axis_tuser_width    : natural := 1
   );
   PORT(
      AXIS_ACLK      : IN  STD_LOGIC;
      -- input stream
      S_AXIS_TDATA   : IN  STD_LOGIC_VECTOR(p_axis_tdata_width - 1 DOWNTO 0);
      S_AXIS_TVALID  : IN  STD_LOGIC;
      S_AXIS_TREADY  : OUT STD_LOGIC;
      S_AXIS_TLAST   : IN  STD_LOGIC;
      S_AXIS_TUSER   : IN  STD_LOGIC_VECTOR(p_axis_tuser_width - 1 DOWNTO 0);
      -- output stream
      M_AXIS_TDATA   : OUT STD_LOGIC_VECTOR(p_axis_tdata_width - 1 DOWNTO 0);
      M_AXIS_TVALID  : OUT STD_LOGIC;
      M_AXIS_TREADY  : IN  STD_LOGIC;
      M_AXIS_TLAST   : OUT STD_LOGIC;
      M_AXIS_TUSER   : OUT STD_LOGIC_VECTOR(p_axis_tuser_width - 1 DOWNTO 0)
   );
END ENTITY;

ARCHITECTURE Behavioral OF axis_video_frame_dropper IS

    SIGNAL r_flush_frame : BOOLEAN := false;
    ALIAS n_sof : std_logic is S_AXIS_TUSER(0);

BEGIN

   PROCESS(AXIS_ACLK)
   BEGIN
      IF (rising_edge(AXIS_ACLK)) THEN
         IF (r_flush_frame) THEN
             IF (S_AXIS_TVALID = '1' AND n_sof = '1') THEN
                r_flush_frame <= false;
             END IF;
         ELSE
             IF (S_AXIS_TVALID = '1' AND M_AXIS_TREADY = '0') THEN
                r_flush_frame <= true;
             END IF;
         END IF;
      END IF;
   END PROCESS;
   
   M_AXIS_TDATA     <= S_AXIS_TDATA;
   M_AXIS_TUSER     <= S_AXIS_TUSER;
   M_AXIS_TLAST     <= S_AXIS_TLAST;
   M_AXIS_TVALID    <= S_AXIS_TVALID WHEN (NOT r_flush_frame) OR n_sof = '1' ELSE '0';
   S_AXIS_TREADY    <= '1';

END Behavioral;
