
################################################################
# This is a generated script based on design: xdp_trd
#
# Though there are limitations about the generated script,
# the main purpose of this utility is to make learning
# IP Integrator Tcl commands easier.
################################################################

namespace eval _tcl {
proc get_script_folder {} {
   set script_path [file normalize [info script]]
   set script_folder [file dirname $script_path]
   return $script_folder
}
}
variable script_folder
set script_folder [_tcl::get_script_folder]

################################################################
# Check if script is running in correct Vivado version.
################################################################
set scripts_vivado_version 2019.2
set current_vivado_version [version -short]

if { [string first $scripts_vivado_version $current_vivado_version] == -1 } {
   puts ""
   common::send_msg_id "BD_TCL-1002" "WARNING" "This script was generated using Vivado <$scripts_vivado_version> without IP versions in the create_bd_cell commands, but is now being run in <$current_vivado_version> of Vivado. There may have been major IP version changes between Vivado <$scripts_vivado_version> and <$current_vivado_version>, which could impact the parameter settings of the IPs."

}

################################################################
# START
################################################################

# To test this script, run the following commands from Vivado Tcl console:
# source xdp_trd_script.tcl


# The design that will be created by this Tcl script contains the following 
# module references:
# aurora_controller, axis_data_counter, aurora_controller, axis_data_counter, axis_video_frame_dropper, axis_video_add_tuser_frame_count

# Please add the sources of those modules before sourcing this Tcl script.

# If there is no project opened, this script will create a
# project, but make sure you do not have an existing project
# <./myproj/project_1.xpr> in the current working folder.

set list_projs [get_projects -quiet]
if { $list_projs eq "" } {
   create_project project_1 myproj -part xczu7ev-fbvb900-2-i
}


# CHANGE DESIGN NAME HERE
variable design_name
set design_name xdp_trd

# If you do not already have an existing IP Integrator design open,
# you can create a design using the following command:
#    create_bd_design $design_name

# Creating design if needed
set errMsg ""
set nRet 0

set cur_design [current_bd_design -quiet]
set list_cells [get_bd_cells -quiet]

if { ${design_name} eq "" } {
   # USE CASES:
   #    1) Design_name not set

   set errMsg "Please set the variable <design_name> to a non-empty value."
   set nRet 1

} elseif { ${cur_design} ne "" && ${list_cells} eq "" } {
   # USE CASES:
   #    2): Current design opened AND is empty AND names same.
   #    3): Current design opened AND is empty AND names diff; design_name NOT in project.
   #    4): Current design opened AND is empty AND names diff; design_name exists in project.

   if { $cur_design ne $design_name } {
      common::send_msg_id "BD_TCL-001" "INFO" "Changing value of <design_name> from <$design_name> to <$cur_design> since current design is empty."
      set design_name [get_property NAME $cur_design]
   }
   common::send_msg_id "BD_TCL-002" "INFO" "Constructing design in IPI design <$cur_design>..."

} elseif { ${cur_design} ne "" && $list_cells ne "" && $cur_design eq $design_name } {
   # USE CASES:
   #    5) Current design opened AND has components AND same names.

   set errMsg "Design <$design_name> already exists in your project, please set the variable <design_name> to another value."
   set nRet 1
} elseif { [get_files -quiet ${design_name}.bd] ne "" } {
   # USE CASES: 
   #    6) Current opened design, has components, but diff names, design_name exists in project.
   #    7) No opened design, design_name exists in project.

   set errMsg "Design <$design_name> already exists in your project, please set the variable <design_name> to another value."
   set nRet 2

} else {
   # USE CASES:
   #    8) No opened design, design_name not in project.
   #    9) Current opened design, has components, but diff names, design_name not in project.

   common::send_msg_id "BD_TCL-003" "INFO" "Currently there is no design <$design_name> in project, so creating one..."

   create_bd_design $design_name

   common::send_msg_id "BD_TCL-004" "INFO" "Making design <$design_name> as current_bd_design."
   current_bd_design $design_name

}

common::send_msg_id "BD_TCL-005" "INFO" "Currently the variable <design_name> is equal to \"$design_name\"."

if { $nRet != 0 } {
   catch {common::send_msg_id "BD_TCL-114" "ERROR" $errMsg}
   return $nRet
}

set bCheckIPsPassed 1
##################################################################
# CHECK IPs
##################################################################
set bCheckIPs 1
if { $bCheckIPs == 1 } {
   set list_check_ips "\ 
xilinx.com:ip:xlconstant:*\
xilinx.com:ip:axi_gpio:*\
xilinx.com:ip:axi_iic:*\
qdesys.com:motor_control:axi_ivplogger:*\
qdesys.com:motor_control:axi_motorfoc:*\
xilinx.com:ip:smartconnect:*\
qdesys.com:motor_control:axi_tep2_drv:*\
xilinx.com:ip:axi_uart16550:*\
topicembeddedproducts.com:ip:topic_fanctrl:*\
xilinx.com:ip:axis_data_fifo:*\
xilinx.com:ip:c_counter_binary:*\
xilinx.com:ip:util_ds_buf:*\
xilinx.com:ip:clk_wiz:*\
topicembeddedproducts.com:dyplo:dyplo_axi:*\
xilinx.com:ip:proc_sys_reset:*\
xilinx.com:ip:vcu:*\
xilinx.com:ip:xlslice:*\
xilinx.com:ip:aurora_64b66b:*\
xilinx.com:ip:axis_subset_converter:*\
xilinx.com:ip:v_frmbuf_rd:*\
xilinx.com:ip:v_hdmi_tx_ss:*\
xilinx.com:ip:vid_phy_controller:*\
xilinx.com:ip:mipi_csi2_rx_subsystem:*\
xilinx.com:ip:v_demosaic:*\
xilinx.com:ip:v_frmbuf_wr:*\
xilinx.com:ip:v_proc_ss:*\
xilinx.com:ip:xlconcat:*\
xilinx.com:ip:zynq_ultra_ps_e:*\
"

   set list_ips_missing ""
   common::send_msg_id "BD_TCL-006" "INFO" "Checking if the following IPs exist in the project's IP catalog: $list_check_ips ."

   foreach ip_vlnv $list_check_ips {
      set ip_obj [get_ipdefs -all $ip_vlnv]
      if { $ip_obj eq "" } {
         lappend list_ips_missing $ip_vlnv
      }
   }

   if { $list_ips_missing ne "" } {
      catch {common::send_msg_id "BD_TCL-115" "ERROR" "The following IPs are not found in the IP Catalog:\n  $list_ips_missing\n\nResolution: Please add the repository containing the IP(s) to the project." }
      set bCheckIPsPassed 0
   }

}

##################################################################
# CHECK Modules
##################################################################
set bCheckModules 1
if { $bCheckModules == 1 } {
   set list_check_mods "\ 
aurora_controller\
axis_data_counter\
aurora_controller\
axis_data_counter\
axis_video_frame_dropper\
axis_video_add_tuser_frame_count\
"

   set list_mods_missing ""
   common::send_msg_id "BD_TCL-006" "INFO" "Checking if the following modules exist in the project's sources: $list_check_mods ."

   foreach mod_vlnv $list_check_mods {
      if { [can_resolve_reference $mod_vlnv] == 0 } {
         lappend list_mods_missing $mod_vlnv
      }
   }

   if { $list_mods_missing ne "" } {
      catch {common::send_msg_id "BD_TCL-115" "ERROR" "The following module(s) are not found in the project: $list_mods_missing" }
      common::send_msg_id "BD_TCL-008" "INFO" "Please add source files for the missing module(s) above."
      set bCheckIPsPassed 0
   }
}

if { $bCheckIPsPassed != 1 } {
  common::send_msg_id "BD_TCL-1003" "WARNING" "Will not continue with creation of design due to the error(s) above."
  return 3
}

##################################################################
# DESIGN PROCs
##################################################################


# Hierarchical cell: emio_gpio_distr
proc create_hier_cell_emio_gpio_distr { parentCell nameHier } {

  variable script_folder

  if { $parentCell eq "" || $nameHier eq "" } {
     catch {common::send_msg_id "BD_TCL-102" "ERROR" "create_hier_cell_emio_gpio_distr() - Empty argument(s)!"}
     return
  }

  # Get object for parentCell
  set parentObj [get_bd_cells $parentCell]
  if { $parentObj == "" } {
     catch {common::send_msg_id "BD_TCL-100" "ERROR" "Unable to find parent cell <$parentCell>!"}
     return
  }

  # Make sure parentObj is hier blk
  set parentType [get_property TYPE $parentObj]
  if { $parentType ne "hier" } {
     catch {common::send_msg_id "BD_TCL-101" "ERROR" "Parent <$parentObj> has TYPE = <$parentType>. Expected to be <hier>."}
     return
  }

  # Save current instance; Restore later
  set oldCurInst [current_bd_instance .]

  # Set parent object as current
  current_bd_instance $parentObj

  # Create cell and set as current instance
  set hier_obj [create_bd_cell -type hier $nameHier]
  current_bd_instance $hier_obj

  # Create interface pins

  # Create pins
  create_bd_pin -dir I BT_DEV_WAKE
  create_bd_pin -dir O -from 0 -to 0 BT_EN
  create_bd_pin -dir O -from 0 -to 0 Dout
  create_bd_pin -dir O -from 0 -to 0 EIO_LED1
  create_bd_pin -dir O -from 0 -to 0 EIO_LED2
  create_bd_pin -dir O -from 0 -to 0 SDIO_WIFI_EN
  create_bd_pin -dir I -from 0 -to 0 SENS_ACCEL_INT0
  create_bd_pin -dir I -from 0 -to 0 SENS_ACCEL_INT1
  create_bd_pin -dir O -from 0 -to 0 SENS_GPS_DSEL
  create_bd_pin -dir I -from 0 -to 0 SENS_GPS_PIO11
  create_bd_pin -dir I -from 0 -to 0 SENS_GPS_PIO13_EINT
  create_bd_pin -dir I -from 0 -to 0 SENS_GPS_PIO14
  create_bd_pin -dir I -from 0 -to 0 SENS_GPS_PIO15
  create_bd_pin -dir O -from 0 -to 0 SENS_GPS_RESET_N
  create_bd_pin -dir I -from 0 -to 0 SENS_GYRO_INT0
  create_bd_pin -dir I -from 0 -to 0 SENS_GYRO_INT1
  create_bd_pin -dir I -from 0 -to 0 SENS_MAGN_DRDY
  create_bd_pin -dir I -from 0 -to 0 SENS_MAGN_INT
  create_bd_pin -dir O -from 0 -to 0 aurora_raptor_reset
  create_bd_pin -dir O -from 0 -to 0 aurora_xdp_reset
  create_bd_pin -dir O -from 94 -to 0 emio_gpio_i
  create_bd_pin -dir I -from 94 -to 0 emio_gpio_o
  create_bd_pin -dir O -from 0 -to 0 hdmi_tx_refclk_rdy
  create_bd_pin -dir O -from 0 -to 0 mipi_0_csc_rst_n
  create_bd_pin -dir O -from 0 -to 0 mipi_0_demosaic_rst_n
  create_bd_pin -dir O -from 0 -to 0 mipi_0_frmbuf_rst_n
  create_bd_pin -dir O -from 0 -to 0 mipi_0_rst
  create_bd_pin -dir O -from 0 -to 0 mipi_0_scaler_rst_n
  create_bd_pin -dir O -from 0 -to 0 mipi_1_csc_rst_n
  create_bd_pin -dir O -from 0 -to 0 mipi_1_demosaic_rst_n
  create_bd_pin -dir O -from 0 -to 0 mipi_1_frmbuf_rst_n
  create_bd_pin -dir O -from 0 -to 0 mipi_1_rst
  create_bd_pin -dir I -from 0 -to 0 switch_pl_0
  create_bd_pin -dir O -from 0 -to 0 vcu_0_rst_n
  create_bd_pin -dir I -from 3 -to 0 xdp_version

  # Create instance: GND_59b, and set properties
  set GND_59b [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant GND_59b ]
  set_property -dict [ list \
   CONFIG.CONST_VAL {0} \
   CONFIG.CONST_WIDTH {59} \
 ] $GND_59b

  # Create instance: bit_0, and set properties
  set bit_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_0 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {0} \
   CONFIG.DIN_TO {0} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_0

  # Create instance: bit_1, and set properties
  set bit_1 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_1 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {1} \
   CONFIG.DIN_TO {1} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_1

  # Create instance: bit_3, and set properties
  set bit_3 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_3 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {3} \
   CONFIG.DIN_TO {3} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_3

  # Create instance: bit_4, and set properties
  set bit_4 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_4 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {4} \
   CONFIG.DIN_TO {4} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_4

  # Create instance: bit_6, and set properties
  set bit_6 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_6 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {6} \
   CONFIG.DIN_TO {6} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_6

  # Create instance: bit_7, and set properties
  set bit_7 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_7 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {7} \
   CONFIG.DIN_TO {7} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_7

  # Create instance: bit_18, and set properties
  set bit_18 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_18 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {18} \
   CONFIG.DIN_TO {18} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_18

  # Create instance: bit_19, and set properties
  set bit_19 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_19 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {19} \
   CONFIG.DIN_TO {19} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_19

  # Create instance: bit_20, and set properties
  set bit_20 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_20 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {20} \
   CONFIG.DIN_TO {20} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_20

  # Create instance: bit_21, and set properties
  set bit_21 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_21 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {21} \
   CONFIG.DIN_TO {21} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_21

  # Create instance: bit_22, and set properties
  set bit_22 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_22 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {22} \
   CONFIG.DIN_TO {22} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_22

  # Create instance: bit_23, and set properties
  set bit_23 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_23 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {23} \
   CONFIG.DIN_TO {23} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_23

  # Create instance: bit_24, and set properties
  set bit_24 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_24 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {24} \
   CONFIG.DIN_TO {24} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_24

  # Create instance: bit_25, and set properties
  set bit_25 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_25 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {25} \
   CONFIG.DIN_TO {25} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_25

  # Create instance: bit_26, and set properties
  set bit_26 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_26 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {26} \
   CONFIG.DIN_TO {26} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_26

  # Create instance: bit_27, and set properties
  set bit_27 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_27 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {27} \
   CONFIG.DIN_TO {27} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_27

  # Create instance: bit_28, and set properties
  set bit_28 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_28 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {28} \
   CONFIG.DIN_TO {28} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_28

  # Create instance: bit_29, and set properties
  set bit_29 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_29 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {29} \
   CONFIG.DIN_TO {29} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_29

  # Create instance: bit_30, and set properties
  set bit_30 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_30 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {30} \
   CONFIG.DIN_TO {30} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_30

  # Create instance: bit_31, and set properties
  set bit_31 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice bit_31 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {31} \
   CONFIG.DIN_TO {31} \
   CONFIG.DIN_WIDTH {95} \
   CONFIG.DOUT_WIDTH {1} \
 ] $bit_31

  # Create instance: xlconcat_0, and set properties
  set xlconcat_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlconcat xlconcat_0 ]
  set_property -dict [ list \
   CONFIG.NUM_PORTS {32} \
 ] $xlconcat_0

  # Create instance: xlconcat_1, and set properties
  set xlconcat_1 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlconcat xlconcat_1 ]
  set_property -dict [ list \
   CONFIG.IN0_WIDTH {32} \
   CONFIG.IN1_WIDTH {4} \
   CONFIG.IN2_WIDTH {59} \
   CONFIG.NUM_PORTS {3} \
 ] $xlconcat_1

  # Create port connections
  connect_bd_net -net BT_DEV_WAKE_1 [get_bd_pins BT_DEV_WAKE] [get_bd_pins xlconcat_0/In5]
  connect_bd_net -net GND_63b_dout [get_bd_pins GND_59b/dout] [get_bd_pins xlconcat_1/In2]
  connect_bd_net -net SENS_ACCEL_INT0_1 [get_bd_pins SENS_ACCEL_INT0] [get_bd_pins xlconcat_0/In12]
  connect_bd_net -net SENS_ACCEL_INT1_1 [get_bd_pins SENS_ACCEL_INT1] [get_bd_pins xlconcat_0/In13]
  connect_bd_net -net SENS_GPS_PIO11_1 [get_bd_pins SENS_GPS_PIO11] [get_bd_pins xlconcat_0/In8]
  connect_bd_net -net SENS_GPS_PIO13_EINT_1 [get_bd_pins SENS_GPS_PIO13_EINT] [get_bd_pins xlconcat_0/In9]
  connect_bd_net -net SENS_GPS_PIO14_1 [get_bd_pins SENS_GPS_PIO14] [get_bd_pins xlconcat_0/In10]
  connect_bd_net -net SENS_GPS_PIO15_1 [get_bd_pins SENS_GPS_PIO15] [get_bd_pins xlconcat_0/In11]
  connect_bd_net -net SENS_GYRO_INT0_1 [get_bd_pins SENS_GYRO_INT0] [get_bd_pins xlconcat_0/In14]
  connect_bd_net -net SENS_GYRO_INT1_1 [get_bd_pins SENS_GYRO_INT1] [get_bd_pins xlconcat_0/In15]
  connect_bd_net -net SENS_MAGN_DRDY_1 [get_bd_pins SENS_MAGN_DRDY] [get_bd_pins xlconcat_0/In16]
  connect_bd_net -net SENS_MAGN_INT_1 [get_bd_pins SENS_MAGN_INT] [get_bd_pins xlconcat_0/In17]
  connect_bd_net -net bit_0_Dout [get_bd_pins mipi_0_rst] [get_bd_pins bit_0/Dout] [get_bd_pins xlconcat_0/In0]
  connect_bd_net -net bit_18_Dout [get_bd_pins SENS_GPS_DSEL] [get_bd_pins bit_18/Dout] [get_bd_pins xlconcat_0/In18]
  connect_bd_net -net bit_19_Dout [get_bd_pins SENS_GPS_RESET_N] [get_bd_pins bit_19/Dout] [get_bd_pins xlconcat_0/In19]
  connect_bd_net -net bit_1_Dout [get_bd_pins mipi_1_rst] [get_bd_pins bit_1/Dout] [get_bd_pins xlconcat_0/In1]
  connect_bd_net -net bit_20_Dout [get_bd_pins EIO_LED1] [get_bd_pins bit_20/Dout] [get_bd_pins xlconcat_0/In20]
  connect_bd_net -net bit_21_Dout [get_bd_pins EIO_LED2] [get_bd_pins bit_21/Dout] [get_bd_pins xlconcat_0/In21]
  connect_bd_net -net bit_22_Dout [get_bd_pins SDIO_WIFI_EN] [get_bd_pins bit_22/Dout] [get_bd_pins xlconcat_0/In22]
  connect_bd_net -net bit_23_Dout [get_bd_pins mipi_0_scaler_rst_n] [get_bd_pins bit_23/Dout] [get_bd_pins xlconcat_0/In23]
  connect_bd_net -net bit_24_Dout [get_bd_pins mipi_0_frmbuf_rst_n] [get_bd_pins bit_24/Dout] [get_bd_pins xlconcat_0/In24]
  connect_bd_net -net bit_25_Dout [get_bd_pins mipi_1_frmbuf_rst_n] [get_bd_pins bit_25/Dout] [get_bd_pins xlconcat_0/In25]
  connect_bd_net -net bit_26_Dout [get_bd_pins mipi_0_demosaic_rst_n] [get_bd_pins bit_26/Dout] [get_bd_pins xlconcat_0/In26]
  connect_bd_net -net bit_27_Dout [get_bd_pins mipi_1_demosaic_rst_n] [get_bd_pins bit_27/Dout] [get_bd_pins xlconcat_0/In27]
  connect_bd_net -net bit_28_Dout [get_bd_pins vcu_0_rst_n] [get_bd_pins bit_28/Dout] [get_bd_pins xlconcat_0/In28]
  connect_bd_net -net bit_29_Dout [get_bd_pins Dout] [get_bd_pins bit_29/Dout] [get_bd_pins xlconcat_0/In29]
  connect_bd_net -net bit_30_Dout [get_bd_pins mipi_0_csc_rst_n] [get_bd_pins bit_30/Dout] [get_bd_pins xlconcat_0/In30]
  connect_bd_net -net bit_31_Dout [get_bd_pins mipi_1_csc_rst_n] [get_bd_pins bit_31/Dout] [get_bd_pins xlconcat_0/In31]
  connect_bd_net -net bit_3_Dout [get_bd_pins aurora_raptor_reset] [get_bd_pins bit_3/Dout] [get_bd_pins xlconcat_0/In3]
  connect_bd_net -net bit_4_Dout [get_bd_pins aurora_xdp_reset] [get_bd_pins bit_4/Dout] [get_bd_pins xlconcat_0/In4]
  connect_bd_net -net bit_6_Dout [get_bd_pins BT_EN] [get_bd_pins bit_6/Dout] [get_bd_pins xlconcat_0/In6]
  connect_bd_net -net bit_7_Dout [get_bd_pins hdmi_tx_refclk_rdy] [get_bd_pins bit_7/Dout] [get_bd_pins xlconcat_0/In7]
  connect_bd_net -net switch_pl_0_1 [get_bd_pins switch_pl_0] [get_bd_pins xlconcat_0/In2]
  connect_bd_net -net xdp_version_1 [get_bd_pins xdp_version] [get_bd_pins xlconcat_1/In1]
  connect_bd_net -net xlconcat_0_dout [get_bd_pins xlconcat_0/dout] [get_bd_pins xlconcat_1/In0]
  connect_bd_net -net xlconcat_1_dout [get_bd_pins emio_gpio_i] [get_bd_pins xlconcat_1/dout]
  connect_bd_net -net zynq_ultra_ps_e_0_emio_gpio_o [get_bd_pins emio_gpio_o] [get_bd_pins bit_0/Din] [get_bd_pins bit_1/Din] [get_bd_pins bit_18/Din] [get_bd_pins bit_19/Din] [get_bd_pins bit_20/Din] [get_bd_pins bit_21/Din] [get_bd_pins bit_22/Din] [get_bd_pins bit_23/Din] [get_bd_pins bit_24/Din] [get_bd_pins bit_25/Din] [get_bd_pins bit_26/Din] [get_bd_pins bit_27/Din] [get_bd_pins bit_28/Din] [get_bd_pins bit_29/Din] [get_bd_pins bit_3/Din] [get_bd_pins bit_30/Din] [get_bd_pins bit_31/Din] [get_bd_pins bit_4/Din] [get_bd_pins bit_6/Din] [get_bd_pins bit_7/Din]

  # Restore current instance
  current_bd_instance $oldCurInst
}

# Hierarchical cell: zynq_ps
proc create_hier_cell_zynq_ps { parentCell nameHier } {

  variable script_folder

  if { $parentCell eq "" || $nameHier eq "" } {
     catch {common::send_msg_id "BD_TCL-102" "ERROR" "create_hier_cell_zynq_ps() - Empty argument(s)!"}
     return
  }

  # Get object for parentCell
  set parentObj [get_bd_cells $parentCell]
  if { $parentObj == "" } {
     catch {common::send_msg_id "BD_TCL-100" "ERROR" "Unable to find parent cell <$parentCell>!"}
     return
  }

  # Make sure parentObj is hier blk
  set parentType [get_property TYPE $parentObj]
  if { $parentType ne "hier" } {
     catch {common::send_msg_id "BD_TCL-101" "ERROR" "Parent <$parentObj> has TYPE = <$parentType>. Expected to be <hier>."}
     return
  }

  # Save current instance; Restore later
  set oldCurInst [current_bd_instance .]

  # Set parent object as current
  current_bd_instance $parentObj

  # Create cell and set as current instance
  set hier_obj [create_bd_cell -type hier $nameHier]
  current_bd_instance $hier_obj

  # Create interface pins
  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:aximm_rtl:1.0 M_AXI_HPM0_FPD

  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:sdio_rtl:1.0 SDIO_WIFI

  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:aximm_rtl:1.0 S_AXI_HP0_FPD

  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:aximm_rtl:1.0 S_AXI_HP2_FPD

  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:aximm_rtl:1.0 S_AXI_HP3_FPD

  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:aximm_rtl:1.0 S_AXI_HPC0_FPD

  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:aximm_rtl:1.0 S_AXI_HPC1_FPD

  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:uart_rtl:1.0 UART_1


  # Create pins
  create_bd_pin -dir I -from 0 -to 0 BT_DEV_WAKE
  create_bd_pin -dir O -from 0 -to 0 BT_EN
  create_bd_pin -dir O -from 0 -to 0 -type rst Dout
  create_bd_pin -dir O -from 0 -to 0 EIO_LED1
  create_bd_pin -dir O -from 0 -to 0 EIO_LED2
  create_bd_pin -dir O -from 0 -to 0 SDIO_WIFI_EN
  create_bd_pin -dir I -from 0 -to 0 SENS_ACCEL_INT0
  create_bd_pin -dir I -from 0 -to 0 SENS_ACCEL_INT1
  create_bd_pin -dir O -from 0 -to 0 SENS_GPS_DSEL
  create_bd_pin -dir I -from 0 -to 0 SENS_GPS_PIO11
  create_bd_pin -dir I -from 0 -to 0 SENS_GPS_PIO13_EINT
  create_bd_pin -dir I -from 0 -to 0 SENS_GPS_PIO14
  create_bd_pin -dir I -from 0 -to 0 SENS_GPS_PIO15
  create_bd_pin -dir O -from 0 -to 0 SENS_GPS_RESET_N
  create_bd_pin -dir I -from 0 -to 0 SENS_GYRO_INT0
  create_bd_pin -dir I -from 0 -to 0 SENS_GYRO_INT1
  create_bd_pin -dir I -from 0 -to 0 SENS_MAGN_DRDY
  create_bd_pin -dir I -from 0 -to 0 SENS_MAGN_INT
  create_bd_pin -dir O SENS_SPI0_CS_ACCEL
  create_bd_pin -dir O SENS_SPI0_CS_GYRO
  create_bd_pin -dir I SENS_SPI0_MISO
  create_bd_pin -dir O SENS_SPI0_MOSI
  create_bd_pin -dir O SENS_SPI0_SCK
  create_bd_pin -dir O SENS_SPI1_CS_ENV
  create_bd_pin -dir O SENS_SPI1_CS_MAGN
  create_bd_pin -dir I SENS_SPI1_MISO
  create_bd_pin -dir O SENS_SPI1_MOSI
  create_bd_pin -dir O SENS_SPI1_SCK
  create_bd_pin -dir O -from 0 -to 0 aurora_raptor_reset
  create_bd_pin -dir O -from 0 -to 0 -type rst aurora_xdp_reset
  create_bd_pin -dir O -from 0 -to 0 hdmi_tx_ref_clk_rdy
  create_bd_pin -dir O -from 0 -to 0 -type rst interconnect_aresetn
  create_bd_pin -dir O -from 0 -to 0 mipi_0_csc_rst_n
  create_bd_pin -dir O -from 0 -to 0 mipi_0_demosaic_rst_n
  create_bd_pin -dir O -from 0 -to 0 mipi_0_frmbuf_rst_n
  create_bd_pin -dir O -from 0 -to 0 mipi_0_rst
  create_bd_pin -dir O -from 0 -to 0 mipi_1_csc_rst_n
  create_bd_pin -dir O -from 0 -to 0 -type rst mipi_1_demosaic_rst_n
  create_bd_pin -dir O -from 0 -to 0 mipi_1_frmbuf_rst_n
  create_bd_pin -dir O -from 0 -to 0 mipi_1_rst
  create_bd_pin -dir O -from 0 -to 0 -type rst peripheral_aresetn_clk0
  create_bd_pin -dir O -from 0 -to 0 -type rst peripheral_aresetn_clk1
  create_bd_pin -dir O -type clk pl_clk0
  create_bd_pin -dir O -type clk pl_clk1
  create_bd_pin -dir I -from 0 -to 0 pl_ps_irq0
  create_bd_pin -dir I -from 0 -to 0 pl_ps_irq1
  create_bd_pin -dir I -type intr pl_ps_irq2
  create_bd_pin -dir I -type intr pl_ps_irq3
  create_bd_pin -dir I -type intr pl_ps_irq4
  create_bd_pin -dir I -type intr pl_ps_irq5
  create_bd_pin -dir I -type intr pl_ps_irq6
  create_bd_pin -dir I -type intr pl_ps_irq7
  create_bd_pin -dir I -type intr pl_ps_irq8
  create_bd_pin -dir I -from 0 -to 0 pl_ps_irq9
  create_bd_pin -dir I -from 0 -to 0 pl_ps_irq10
  create_bd_pin -dir I -type intr pl_ps_irq11
  create_bd_pin -dir I -type intr pl_ps_irq12
  create_bd_pin -dir I -type intr pl_ps_irq13
  create_bd_pin -dir I -type intr pl_ps_irq14
  create_bd_pin -dir I -from 0 -to 0 pl_ps_irq15
  create_bd_pin -dir O -type rst pl_resetn0
  create_bd_pin -dir I saxihp0_fpd_aclk
  create_bd_pin -dir I -type clk saxihpc0_fpd_aclk
  create_bd_pin -dir I -from 0 -to 0 switch_pl_0
  create_bd_pin -dir O -from 0 -to 0 vcu_0_rst_n
  create_bd_pin -dir I -from 3 -to 0 xdp_version

  # Create instance: emio_gpio_distr
  create_hier_cell_emio_gpio_distr $hier_obj emio_gpio_distr

  # Create instance: irq_concat_0, and set properties
  set irq_concat_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlconcat irq_concat_0 ]
  set_property -dict [ list \
   CONFIG.NUM_PORTS {8} \
 ] $irq_concat_0

  # Create instance: irq_concat_1, and set properties
  set irq_concat_1 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlconcat irq_concat_1 ]
  set_property -dict [ list \
   CONFIG.NUM_PORTS {8} \
 ] $irq_concat_1

  # Create instance: rst_ps8_0_149M, and set properties
  set rst_ps8_0_149M [ create_bd_cell -type ip -vlnv xilinx.com:ip:proc_sys_reset rst_ps8_0_149M ]

  # Create instance: rst_ps8_0_99M, and set properties
  set rst_ps8_0_99M [ create_bd_cell -type ip -vlnv xilinx.com:ip:proc_sys_reset rst_ps8_0_99M ]

  # Create instance: xlconstant_axcache, and set properties
  set xlconstant_axcache [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant xlconstant_axcache ]
  set_property -dict [ list \
   CONFIG.CONST_VAL {15} \
   CONFIG.CONST_WIDTH {4} \
 ] $xlconstant_axcache

  # Create instance: xlconstant_axprot, and set properties
  set xlconstant_axprot [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant xlconstant_axprot ]
  set_property -dict [ list \
   CONFIG.CONST_VAL {2} \
   CONFIG.CONST_WIDTH {3} \
 ] $xlconstant_axprot

  # Create instance: zynq_ultra_ps_e_0, and set properties
  set zynq_ultra_ps_e_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:zynq_ultra_ps_e zynq_ultra_ps_e_0 ]
  set_property -dict [ list \
   CONFIG.CAN0_BOARD_INTERFACE {custom} \
   CONFIG.CAN1_BOARD_INTERFACE {custom} \
   CONFIG.CSU_BOARD_INTERFACE {custom} \
   CONFIG.DP_BOARD_INTERFACE {custom} \
   CONFIG.GEM0_BOARD_INTERFACE {custom} \
   CONFIG.GEM1_BOARD_INTERFACE {custom} \
   CONFIG.GEM2_BOARD_INTERFACE {custom} \
   CONFIG.GEM3_BOARD_INTERFACE {custom} \
   CONFIG.GPIO_BOARD_INTERFACE {custom} \
   CONFIG.IIC0_BOARD_INTERFACE {custom} \
   CONFIG.IIC1_BOARD_INTERFACE {custom} \
   CONFIG.NAND_BOARD_INTERFACE {custom} \
   CONFIG.PCIE_BOARD_INTERFACE {custom} \
   CONFIG.PJTAG_BOARD_INTERFACE {custom} \
   CONFIG.PMU_BOARD_INTERFACE {custom} \
   CONFIG.PSU_BANK_0_IO_STANDARD {LVCMOS18} \
   CONFIG.PSU_BANK_1_IO_STANDARD {LVCMOS33} \
   CONFIG.PSU_BANK_2_IO_STANDARD {LVCMOS18} \
   CONFIG.PSU_BANK_3_IO_STANDARD {LVCMOS18} \
   CONFIG.PSU_DDR_RAM_HIGHADDR {0xFFFFFFFF} \
   CONFIG.PSU_DDR_RAM_HIGHADDR_OFFSET {0x800000000} \
   CONFIG.PSU_DDR_RAM_LOWADDR_OFFSET {0x80000000} \
   CONFIG.PSU_DYNAMIC_DDR_CONFIG_EN {0} \
   CONFIG.PSU_IMPORT_BOARD_PRESET {} \
   CONFIG.PSU_MIO_0_DIRECTION {out} \
   CONFIG.PSU_MIO_0_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_0_INPUT_TYPE {cmos} \
   CONFIG.PSU_MIO_0_POLARITY {Default} \
   CONFIG.PSU_MIO_0_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_0_SLEW {slow} \
   CONFIG.PSU_MIO_10_DIRECTION {inout} \
   CONFIG.PSU_MIO_10_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_10_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_10_POLARITY {Default} \
   CONFIG.PSU_MIO_10_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_10_SLEW {slow} \
   CONFIG.PSU_MIO_11_DIRECTION {inout} \
   CONFIG.PSU_MIO_11_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_11_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_11_POLARITY {Default} \
   CONFIG.PSU_MIO_11_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_11_SLEW {slow} \
   CONFIG.PSU_MIO_12_DIRECTION {out} \
   CONFIG.PSU_MIO_12_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_12_INPUT_TYPE {cmos} \
   CONFIG.PSU_MIO_12_POLARITY {Default} \
   CONFIG.PSU_MIO_12_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_12_SLEW {slow} \
   CONFIG.PSU_MIO_13_DIRECTION {inout} \
   CONFIG.PSU_MIO_13_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_13_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_13_POLARITY {Default} \
   CONFIG.PSU_MIO_13_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_13_SLEW {slow} \
   CONFIG.PSU_MIO_14_DIRECTION {inout} \
   CONFIG.PSU_MIO_14_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_14_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_14_POLARITY {Default} \
   CONFIG.PSU_MIO_14_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_14_SLEW {slow} \
   CONFIG.PSU_MIO_15_DIRECTION {inout} \
   CONFIG.PSU_MIO_15_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_15_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_15_POLARITY {Default} \
   CONFIG.PSU_MIO_15_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_15_SLEW {slow} \
   CONFIG.PSU_MIO_16_DIRECTION {inout} \
   CONFIG.PSU_MIO_16_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_16_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_16_POLARITY {Default} \
   CONFIG.PSU_MIO_16_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_16_SLEW {slow} \
   CONFIG.PSU_MIO_17_DIRECTION {inout} \
   CONFIG.PSU_MIO_17_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_17_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_17_POLARITY {Default} \
   CONFIG.PSU_MIO_17_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_17_SLEW {slow} \
   CONFIG.PSU_MIO_18_DIRECTION {inout} \
   CONFIG.PSU_MIO_18_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_18_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_18_POLARITY {Default} \
   CONFIG.PSU_MIO_18_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_18_SLEW {slow} \
   CONFIG.PSU_MIO_19_DIRECTION {inout} \
   CONFIG.PSU_MIO_19_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_19_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_19_POLARITY {Default} \
   CONFIG.PSU_MIO_19_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_19_SLEW {slow} \
   CONFIG.PSU_MIO_1_DIRECTION {inout} \
   CONFIG.PSU_MIO_1_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_1_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_1_POLARITY {Default} \
   CONFIG.PSU_MIO_1_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_1_SLEW {slow} \
   CONFIG.PSU_MIO_20_DIRECTION {inout} \
   CONFIG.PSU_MIO_20_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_20_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_20_POLARITY {Default} \
   CONFIG.PSU_MIO_20_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_20_SLEW {slow} \
   CONFIG.PSU_MIO_21_DIRECTION {inout} \
   CONFIG.PSU_MIO_21_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_21_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_21_POLARITY {Default} \
   CONFIG.PSU_MIO_21_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_21_SLEW {slow} \
   CONFIG.PSU_MIO_22_DIRECTION {out} \
   CONFIG.PSU_MIO_22_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_22_INPUT_TYPE {cmos} \
   CONFIG.PSU_MIO_22_POLARITY {Default} \
   CONFIG.PSU_MIO_22_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_22_SLEW {slow} \
   CONFIG.PSU_MIO_23_DIRECTION {out} \
   CONFIG.PSU_MIO_23_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_23_INPUT_TYPE {cmos} \
   CONFIG.PSU_MIO_23_POLARITY {Default} \
   CONFIG.PSU_MIO_23_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_23_SLEW {slow} \
   CONFIG.PSU_MIO_24_DIRECTION {out} \
   CONFIG.PSU_MIO_24_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_24_INPUT_TYPE {cmos} \
   CONFIG.PSU_MIO_24_POLARITY {Default} \
   CONFIG.PSU_MIO_24_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_24_SLEW {slow} \
   CONFIG.PSU_MIO_25_DIRECTION {in} \
   CONFIG.PSU_MIO_25_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_25_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_25_POLARITY {Default} \
   CONFIG.PSU_MIO_25_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_25_SLEW {fast} \
   CONFIG.PSU_MIO_26_DIRECTION {inout} \
   CONFIG.PSU_MIO_26_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_26_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_26_POLARITY {Default} \
   CONFIG.PSU_MIO_26_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_26_SLEW {slow} \
   CONFIG.PSU_MIO_27_DIRECTION {in} \
   CONFIG.PSU_MIO_27_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_27_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_27_POLARITY {Default} \
   CONFIG.PSU_MIO_27_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_27_SLEW {fast} \
   CONFIG.PSU_MIO_28_DIRECTION {in} \
   CONFIG.PSU_MIO_28_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_28_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_28_POLARITY {Default} \
   CONFIG.PSU_MIO_28_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_28_SLEW {fast} \
   CONFIG.PSU_MIO_29_DIRECTION {inout} \
   CONFIG.PSU_MIO_29_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_29_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_29_POLARITY {Default} \
   CONFIG.PSU_MIO_29_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_29_SLEW {slow} \
   CONFIG.PSU_MIO_2_DIRECTION {inout} \
   CONFIG.PSU_MIO_2_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_2_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_2_POLARITY {Default} \
   CONFIG.PSU_MIO_2_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_2_SLEW {slow} \
   CONFIG.PSU_MIO_30_DIRECTION {inout} \
   CONFIG.PSU_MIO_30_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_30_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_30_POLARITY {Default} \
   CONFIG.PSU_MIO_30_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_30_SLEW {slow} \
   CONFIG.PSU_MIO_31_DIRECTION {inout} \
   CONFIG.PSU_MIO_31_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_31_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_31_POLARITY {Default} \
   CONFIG.PSU_MIO_31_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_31_SLEW {slow} \
   CONFIG.PSU_MIO_32_DIRECTION {inout} \
   CONFIG.PSU_MIO_32_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_32_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_32_POLARITY {Default} \
   CONFIG.PSU_MIO_32_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_32_SLEW {slow} \
   CONFIG.PSU_MIO_33_DIRECTION {inout} \
   CONFIG.PSU_MIO_33_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_33_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_33_POLARITY {Default} \
   CONFIG.PSU_MIO_33_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_33_SLEW {slow} \
   CONFIG.PSU_MIO_34_DIRECTION {inout} \
   CONFIG.PSU_MIO_34_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_34_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_34_POLARITY {Default} \
   CONFIG.PSU_MIO_34_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_34_SLEW {slow} \
   CONFIG.PSU_MIO_35_DIRECTION {inout} \
   CONFIG.PSU_MIO_35_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_35_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_35_POLARITY {Default} \
   CONFIG.PSU_MIO_35_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_35_SLEW {slow} \
   CONFIG.PSU_MIO_36_DIRECTION {inout} \
   CONFIG.PSU_MIO_36_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_36_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_36_POLARITY {Default} \
   CONFIG.PSU_MIO_36_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_36_SLEW {slow} \
   CONFIG.PSU_MIO_37_DIRECTION {inout} \
   CONFIG.PSU_MIO_37_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_37_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_37_POLARITY {Default} \
   CONFIG.PSU_MIO_37_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_37_SLEW {slow} \
   CONFIG.PSU_MIO_38_DIRECTION {inout} \
   CONFIG.PSU_MIO_38_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_38_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_38_POLARITY {Default} \
   CONFIG.PSU_MIO_38_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_38_SLEW {slow} \
   CONFIG.PSU_MIO_39_DIRECTION {inout} \
   CONFIG.PSU_MIO_39_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_39_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_39_POLARITY {Default} \
   CONFIG.PSU_MIO_39_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_39_SLEW {slow} \
   CONFIG.PSU_MIO_3_DIRECTION {inout} \
   CONFIG.PSU_MIO_3_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_3_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_3_POLARITY {Default} \
   CONFIG.PSU_MIO_3_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_3_SLEW {slow} \
   CONFIG.PSU_MIO_40_DIRECTION {inout} \
   CONFIG.PSU_MIO_40_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_40_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_40_POLARITY {Default} \
   CONFIG.PSU_MIO_40_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_40_SLEW {slow} \
   CONFIG.PSU_MIO_41_DIRECTION {inout} \
   CONFIG.PSU_MIO_41_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_41_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_41_POLARITY {Default} \
   CONFIG.PSU_MIO_41_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_41_SLEW {slow} \
   CONFIG.PSU_MIO_42_DIRECTION {inout} \
   CONFIG.PSU_MIO_42_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_42_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_42_POLARITY {Default} \
   CONFIG.PSU_MIO_42_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_42_SLEW {slow} \
   CONFIG.PSU_MIO_43_DIRECTION {inout} \
   CONFIG.PSU_MIO_43_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_43_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_43_POLARITY {Default} \
   CONFIG.PSU_MIO_43_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_43_SLEW {slow} \
   CONFIG.PSU_MIO_44_DIRECTION {inout} \
   CONFIG.PSU_MIO_44_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_44_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_44_POLARITY {Default} \
   CONFIG.PSU_MIO_44_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_44_SLEW {slow} \
   CONFIG.PSU_MIO_45_DIRECTION {inout} \
   CONFIG.PSU_MIO_45_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_45_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_45_POLARITY {Default} \
   CONFIG.PSU_MIO_45_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_45_SLEW {slow} \
   CONFIG.PSU_MIO_46_DIRECTION {inout} \
   CONFIG.PSU_MIO_46_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_46_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_46_POLARITY {Default} \
   CONFIG.PSU_MIO_46_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_46_SLEW {slow} \
   CONFIG.PSU_MIO_47_DIRECTION {inout} \
   CONFIG.PSU_MIO_47_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_47_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_47_POLARITY {Default} \
   CONFIG.PSU_MIO_47_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_47_SLEW {slow} \
   CONFIG.PSU_MIO_48_DIRECTION {inout} \
   CONFIG.PSU_MIO_48_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_48_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_48_POLARITY {Default} \
   CONFIG.PSU_MIO_48_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_48_SLEW {slow} \
   CONFIG.PSU_MIO_49_DIRECTION {inout} \
   CONFIG.PSU_MIO_49_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_49_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_49_POLARITY {Default} \
   CONFIG.PSU_MIO_49_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_49_SLEW {slow} \
   CONFIG.PSU_MIO_4_DIRECTION {inout} \
   CONFIG.PSU_MIO_4_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_4_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_4_POLARITY {Default} \
   CONFIG.PSU_MIO_4_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_4_SLEW {slow} \
   CONFIG.PSU_MIO_50_DIRECTION {inout} \
   CONFIG.PSU_MIO_50_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_50_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_50_POLARITY {Default} \
   CONFIG.PSU_MIO_50_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_50_SLEW {slow} \
   CONFIG.PSU_MIO_51_DIRECTION {inout} \
   CONFIG.PSU_MIO_51_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_51_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_51_POLARITY {Default} \
   CONFIG.PSU_MIO_51_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_51_SLEW {slow} \
   CONFIG.PSU_MIO_52_DIRECTION {in} \
   CONFIG.PSU_MIO_52_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_52_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_52_POLARITY {Default} \
   CONFIG.PSU_MIO_52_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_52_SLEW {fast} \
   CONFIG.PSU_MIO_53_DIRECTION {in} \
   CONFIG.PSU_MIO_53_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_53_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_53_POLARITY {Default} \
   CONFIG.PSU_MIO_53_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_53_SLEW {fast} \
   CONFIG.PSU_MIO_54_DIRECTION {inout} \
   CONFIG.PSU_MIO_54_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_54_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_54_POLARITY {Default} \
   CONFIG.PSU_MIO_54_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_54_SLEW {slow} \
   CONFIG.PSU_MIO_55_DIRECTION {in} \
   CONFIG.PSU_MIO_55_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_55_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_55_POLARITY {Default} \
   CONFIG.PSU_MIO_55_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_55_SLEW {fast} \
   CONFIG.PSU_MIO_56_DIRECTION {inout} \
   CONFIG.PSU_MIO_56_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_56_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_56_POLARITY {Default} \
   CONFIG.PSU_MIO_56_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_56_SLEW {slow} \
   CONFIG.PSU_MIO_57_DIRECTION {inout} \
   CONFIG.PSU_MIO_57_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_57_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_57_POLARITY {Default} \
   CONFIG.PSU_MIO_57_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_57_SLEW {slow} \
   CONFIG.PSU_MIO_58_DIRECTION {out} \
   CONFIG.PSU_MIO_58_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_58_INPUT_TYPE {cmos} \
   CONFIG.PSU_MIO_58_POLARITY {Default} \
   CONFIG.PSU_MIO_58_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_58_SLEW {slow} \
   CONFIG.PSU_MIO_59_DIRECTION {inout} \
   CONFIG.PSU_MIO_59_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_59_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_59_POLARITY {Default} \
   CONFIG.PSU_MIO_59_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_59_SLEW {slow} \
   CONFIG.PSU_MIO_5_DIRECTION {out} \
   CONFIG.PSU_MIO_5_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_5_INPUT_TYPE {cmos} \
   CONFIG.PSU_MIO_5_POLARITY {Default} \
   CONFIG.PSU_MIO_5_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_5_SLEW {slow} \
   CONFIG.PSU_MIO_60_DIRECTION {inout} \
   CONFIG.PSU_MIO_60_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_60_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_60_POLARITY {Default} \
   CONFIG.PSU_MIO_60_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_60_SLEW {slow} \
   CONFIG.PSU_MIO_61_DIRECTION {inout} \
   CONFIG.PSU_MIO_61_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_61_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_61_POLARITY {Default} \
   CONFIG.PSU_MIO_61_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_61_SLEW {slow} \
   CONFIG.PSU_MIO_62_DIRECTION {inout} \
   CONFIG.PSU_MIO_62_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_62_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_62_POLARITY {Default} \
   CONFIG.PSU_MIO_62_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_62_SLEW {slow} \
   CONFIG.PSU_MIO_63_DIRECTION {inout} \
   CONFIG.PSU_MIO_63_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_63_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_63_POLARITY {Default} \
   CONFIG.PSU_MIO_63_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_63_SLEW {slow} \
   CONFIG.PSU_MIO_64_DIRECTION {inout} \
   CONFIG.PSU_MIO_64_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_64_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_64_POLARITY {Default} \
   CONFIG.PSU_MIO_64_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_64_SLEW {slow} \
   CONFIG.PSU_MIO_65_DIRECTION {out} \
   CONFIG.PSU_MIO_65_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_65_INPUT_TYPE {cmos} \
   CONFIG.PSU_MIO_65_POLARITY {Default} \
   CONFIG.PSU_MIO_65_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_65_SLEW {slow} \
   CONFIG.PSU_MIO_66_DIRECTION {inout} \
   CONFIG.PSU_MIO_66_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_66_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_66_POLARITY {Default} \
   CONFIG.PSU_MIO_66_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_66_SLEW {slow} \
   CONFIG.PSU_MIO_67_DIRECTION {inout} \
   CONFIG.PSU_MIO_67_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_67_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_67_POLARITY {Default} \
   CONFIG.PSU_MIO_67_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_67_SLEW {slow} \
   CONFIG.PSU_MIO_68_DIRECTION {out} \
   CONFIG.PSU_MIO_68_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_68_INPUT_TYPE {cmos} \
   CONFIG.PSU_MIO_68_POLARITY {Default} \
   CONFIG.PSU_MIO_68_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_68_SLEW {slow} \
   CONFIG.PSU_MIO_69_DIRECTION {inout} \
   CONFIG.PSU_MIO_69_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_69_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_69_POLARITY {Default} \
   CONFIG.PSU_MIO_69_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_69_SLEW {slow} \
   CONFIG.PSU_MIO_6_DIRECTION {out} \
   CONFIG.PSU_MIO_6_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_6_INPUT_TYPE {cmos} \
   CONFIG.PSU_MIO_6_POLARITY {Default} \
   CONFIG.PSU_MIO_6_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_6_SLEW {slow} \
   CONFIG.PSU_MIO_70_DIRECTION {inout} \
   CONFIG.PSU_MIO_70_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_70_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_70_POLARITY {Default} \
   CONFIG.PSU_MIO_70_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_70_SLEW {slow} \
   CONFIG.PSU_MIO_71_DIRECTION {inout} \
   CONFIG.PSU_MIO_71_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_71_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_71_POLARITY {Default} \
   CONFIG.PSU_MIO_71_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_71_SLEW {slow} \
   CONFIG.PSU_MIO_72_DIRECTION {inout} \
   CONFIG.PSU_MIO_72_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_72_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_72_POLARITY {Default} \
   CONFIG.PSU_MIO_72_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_72_SLEW {slow} \
   CONFIG.PSU_MIO_73_DIRECTION {inout} \
   CONFIG.PSU_MIO_73_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_73_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_73_POLARITY {Default} \
   CONFIG.PSU_MIO_73_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_73_SLEW {slow} \
   CONFIG.PSU_MIO_74_DIRECTION {inout} \
   CONFIG.PSU_MIO_74_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_74_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_74_POLARITY {Default} \
   CONFIG.PSU_MIO_74_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_74_SLEW {slow} \
   CONFIG.PSU_MIO_75_DIRECTION {inout} \
   CONFIG.PSU_MIO_75_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_75_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_75_POLARITY {Default} \
   CONFIG.PSU_MIO_75_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_75_SLEW {slow} \
   CONFIG.PSU_MIO_76_DIRECTION {inout} \
   CONFIG.PSU_MIO_76_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_76_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_76_POLARITY {Default} \
   CONFIG.PSU_MIO_76_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_76_SLEW {slow} \
   CONFIG.PSU_MIO_77_DIRECTION {inout} \
   CONFIG.PSU_MIO_77_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_77_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_77_POLARITY {Default} \
   CONFIG.PSU_MIO_77_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_77_SLEW {slow} \
   CONFIG.PSU_MIO_7_DIRECTION {out} \
   CONFIG.PSU_MIO_7_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_7_INPUT_TYPE {cmos} \
   CONFIG.PSU_MIO_7_POLARITY {Default} \
   CONFIG.PSU_MIO_7_PULLUPDOWN {disable} \
   CONFIG.PSU_MIO_7_SLEW {slow} \
   CONFIG.PSU_MIO_8_DIRECTION {inout} \
   CONFIG.PSU_MIO_8_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_8_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_8_POLARITY {Default} \
   CONFIG.PSU_MIO_8_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_8_SLEW {slow} \
   CONFIG.PSU_MIO_9_DIRECTION {inout} \
   CONFIG.PSU_MIO_9_DRIVE_STRENGTH {12} \
   CONFIG.PSU_MIO_9_INPUT_TYPE {schmitt} \
   CONFIG.PSU_MIO_9_POLARITY {Default} \
   CONFIG.PSU_MIO_9_PULLUPDOWN {pullup} \
   CONFIG.PSU_MIO_9_SLEW {slow} \
   CONFIG.PSU_MIO_TREE_PERIPHERALS {Quad SPI Flash#Quad SPI Flash#Quad SPI Flash#Quad SPI Flash#Quad SPI Flash#Quad SPI Flash#Feedback Clk#Quad SPI Flash#Quad SPI Flash#Quad SPI Flash#Quad SPI Flash#Quad SPI Flash#Quad SPI Flash#SD 0#SD 0#SD 0#SD 0#SD 0#SD 0#SD 0#SD 0#SD 0#SD 0#SD 0#UART 1#UART 1#GPIO1 MIO#VIDEO REF CLK#PSS ALT REF CLK#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#GPIO1 MIO#USB 0#USB 0#USB 0#USB 0#USB 0#USB 0#USB 0#USB 0#USB 0#USB 0#USB 0#USB 0#GPIO2 MIO#USB0 Reset#GPIO2 MIO#GPIO2 MIO#PCIE#GPIO2 MIO#GPIO2 MIO#GPIO2 MIO#GPIO2 MIO#GPIO2 MIO#I2C 0#I2C 0#GPIO2 MIO#GPIO2 MIO} \
   CONFIG.PSU_MIO_TREE_SIGNALS {sclk_out#miso_mo1#mo2#mo3#mosi_mi0#n_ss_out#clk_for_lpbk#n_ss_out_upper#mo_upper[0]#mo_upper[1]#mo_upper[2]#mo_upper[3]#sclk_out_upper#sdio0_data_out[0]#sdio0_data_out[1]#sdio0_data_out[2]#sdio0_data_out[3]#sdio0_data_out[4]#sdio0_data_out[5]#sdio0_data_out[6]#sdio0_data_out[7]#sdio0_cmd_out#sdio0_clk_out#sdio0_bus_pow#txd#rxd#gpio1[26]#videorefclk#pssaltrefclk#gpio1[29]#gpio1[30]#gpio1[31]#gpio1[32]#gpio1[33]#gpio1[34]#gpio1[35]#gpio1[36]#gpio1[37]#gpio1[38]#gpio1[39]#gpio1[40]#gpio1[41]#gpio1[42]#gpio1[43]#gpio1[44]#gpio1[45]#gpio1[46]#gpio1[47]#gpio1[48]#gpio1[49]#gpio1[50]#gpio1[51]#ulpi_clk_in#ulpi_dir#ulpi_tx_data[2]#ulpi_nxt#ulpi_tx_data[0]#ulpi_tx_data[1]#ulpi_stp#ulpi_tx_data[3]#ulpi_tx_data[4]#ulpi_tx_data[5]#ulpi_tx_data[6]#ulpi_tx_data[7]#gpio2[64]#reset#gpio2[66]#gpio2[67]#reset_n#gpio2[69]#gpio2[70]#gpio2[71]#gpio2[72]#gpio2[73]#scl_out#sda_out#gpio2[76]#gpio2[77]} \
   CONFIG.PSU_PERIPHERAL_BOARD_PRESET {} \
   CONFIG.PSU_SD0_INTERNAL_BUS_WIDTH {8} \
   CONFIG.PSU_SD1_INTERNAL_BUS_WIDTH {4} \
   CONFIG.PSU_SMC_CYCLE_T0 {NA} \
   CONFIG.PSU_SMC_CYCLE_T1 {NA} \
   CONFIG.PSU_SMC_CYCLE_T2 {NA} \
   CONFIG.PSU_SMC_CYCLE_T3 {NA} \
   CONFIG.PSU_SMC_CYCLE_T4 {NA} \
   CONFIG.PSU_SMC_CYCLE_T5 {NA} \
   CONFIG.PSU_SMC_CYCLE_T6 {NA} \
   CONFIG.PSU_USB3__DUAL_CLOCK_ENABLE {1} \
   CONFIG.PSU_VALUE_SILVERSION {3} \
   CONFIG.PSU__ACPU0__POWER__ON {1} \
   CONFIG.PSU__ACPU1__POWER__ON {1} \
   CONFIG.PSU__ACPU2__POWER__ON {1} \
   CONFIG.PSU__ACPU3__POWER__ON {1} \
   CONFIG.PSU__ACTUAL__IP {1} \
   CONFIG.PSU__ACT_DDR_FREQ_MHZ {1199.988037} \
   CONFIG.PSU__AFI0_COHERENCY {0} \
   CONFIG.PSU__AFI1_COHERENCY {0} \
   CONFIG.PSU__AUX_REF_CLK__FREQMHZ {33.333} \
   CONFIG.PSU__CAN0_LOOP_CAN1__ENABLE {0} \
   CONFIG.PSU__CAN0__GRP_CLK__ENABLE {0} \
   CONFIG.PSU__CAN0__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__CAN0__PERIPHERAL__IO {EMIO} \
   CONFIG.PSU__CAN1__GRP_CLK__ENABLE {0} \
   CONFIG.PSU__CAN1__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__CAN1__PERIPHERAL__IO {EMIO} \
   CONFIG.PSU__CRF_APB__ACPU_CTRL__ACT_FREQMHZ {1333.320068} \
   CONFIG.PSU__CRF_APB__ACPU_CTRL__DIVISOR0 {1} \
   CONFIG.PSU__CRF_APB__ACPU_CTRL__FREQMHZ {1334} \
   CONFIG.PSU__CRF_APB__ACPU_CTRL__SRCSEL {APLL} \
   CONFIG.PSU__CRF_APB__ACPU__FRAC_ENABLED {0} \
   CONFIG.PSU__CRF_APB__AFI0_REF_CTRL__ACT_FREQMHZ {667} \
   CONFIG.PSU__CRF_APB__AFI0_REF_CTRL__DIVISOR0 {2} \
   CONFIG.PSU__CRF_APB__AFI0_REF_CTRL__FREQMHZ {667} \
   CONFIG.PSU__CRF_APB__AFI0_REF_CTRL__SRCSEL {DPLL} \
   CONFIG.PSU__CRF_APB__AFI0_REF__ENABLE {0} \
   CONFIG.PSU__CRF_APB__AFI1_REF_CTRL__ACT_FREQMHZ {667} \
   CONFIG.PSU__CRF_APB__AFI1_REF_CTRL__DIVISOR0 {2} \
   CONFIG.PSU__CRF_APB__AFI1_REF_CTRL__FREQMHZ {667} \
   CONFIG.PSU__CRF_APB__AFI1_REF_CTRL__SRCSEL {DPLL} \
   CONFIG.PSU__CRF_APB__AFI1_REF__ENABLE {0} \
   CONFIG.PSU__CRF_APB__AFI2_REF_CTRL__ACT_FREQMHZ {667} \
   CONFIG.PSU__CRF_APB__AFI2_REF_CTRL__DIVISOR0 {2} \
   CONFIG.PSU__CRF_APB__AFI2_REF_CTRL__FREQMHZ {667} \
   CONFIG.PSU__CRF_APB__AFI2_REF_CTRL__SRCSEL {DPLL} \
   CONFIG.PSU__CRF_APB__AFI2_REF__ENABLE {0} \
   CONFIG.PSU__CRF_APB__AFI3_REF_CTRL__ACT_FREQMHZ {667} \
   CONFIG.PSU__CRF_APB__AFI3_REF_CTRL__DIVISOR0 {2} \
   CONFIG.PSU__CRF_APB__AFI3_REF_CTRL__FREQMHZ {667} \
   CONFIG.PSU__CRF_APB__AFI3_REF_CTRL__SRCSEL {DPLL} \
   CONFIG.PSU__CRF_APB__AFI3_REF__ENABLE {0} \
   CONFIG.PSU__CRF_APB__AFI4_REF_CTRL__ACT_FREQMHZ {667} \
   CONFIG.PSU__CRF_APB__AFI4_REF_CTRL__DIVISOR0 {2} \
   CONFIG.PSU__CRF_APB__AFI4_REF_CTRL__FREQMHZ {667} \
   CONFIG.PSU__CRF_APB__AFI4_REF_CTRL__SRCSEL {DPLL} \
   CONFIG.PSU__CRF_APB__AFI4_REF__ENABLE {0} \
   CONFIG.PSU__CRF_APB__AFI5_REF_CTRL__ACT_FREQMHZ {667} \
   CONFIG.PSU__CRF_APB__AFI5_REF_CTRL__DIVISOR0 {2} \
   CONFIG.PSU__CRF_APB__AFI5_REF_CTRL__FREQMHZ {667} \
   CONFIG.PSU__CRF_APB__AFI5_REF_CTRL__SRCSEL {DPLL} \
   CONFIG.PSU__CRF_APB__AFI5_REF__ENABLE {0} \
   CONFIG.PSU__CRF_APB__APLL_CTRL__DIV2 {1} \
   CONFIG.PSU__CRF_APB__APLL_CTRL__FBDIV {80} \
   CONFIG.PSU__CRF_APB__APLL_CTRL__FRACDATA {0.000000} \
   CONFIG.PSU__CRF_APB__APLL_CTRL__FRACFREQ {27.138} \
   CONFIG.PSU__CRF_APB__APLL_CTRL__SRCSEL {PSS_REF_CLK} \
   CONFIG.PSU__CRF_APB__APLL_FRAC_CFG__ENABLED {0} \
   CONFIG.PSU__CRF_APB__APLL_TO_LPD_CTRL__DIVISOR0 {3} \
   CONFIG.PSU__CRF_APB__APM_CTRL__ACT_FREQMHZ {1} \
   CONFIG.PSU__CRF_APB__APM_CTRL__DIVISOR0 {1} \
   CONFIG.PSU__CRF_APB__APM_CTRL__FREQMHZ {1} \
   CONFIG.PSU__CRF_APB__DBG_FPD_CTRL__ACT_FREQMHZ {199.998001} \
   CONFIG.PSU__CRF_APB__DBG_FPD_CTRL__DIVISOR0 {2} \
   CONFIG.PSU__CRF_APB__DBG_FPD_CTRL__FREQMHZ {200} \
   CONFIG.PSU__CRF_APB__DBG_FPD_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRF_APB__DBG_TRACE_CTRL__ACT_FREQMHZ {250} \
   CONFIG.PSU__CRF_APB__DBG_TRACE_CTRL__DIVISOR0 {5} \
   CONFIG.PSU__CRF_APB__DBG_TRACE_CTRL__FREQMHZ {250} \
   CONFIG.PSU__CRF_APB__DBG_TRACE_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRF_APB__DBG_TSTMP_CTRL__ACT_FREQMHZ {199.998001} \
   CONFIG.PSU__CRF_APB__DBG_TSTMP_CTRL__DIVISOR0 {2} \
   CONFIG.PSU__CRF_APB__DBG_TSTMP_CTRL__FREQMHZ {200} \
   CONFIG.PSU__CRF_APB__DBG_TSTMP_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRF_APB__DDR_CTRL__ACT_FREQMHZ {599.994019} \
   CONFIG.PSU__CRF_APB__DDR_CTRL__DIVISOR0 {2} \
   CONFIG.PSU__CRF_APB__DDR_CTRL__FREQMHZ {1200} \
   CONFIG.PSU__CRF_APB__DDR_CTRL__SRCSEL {DPLL} \
   CONFIG.PSU__CRF_APB__DPDMA_REF_CTRL__ACT_FREQMHZ {599.994019} \
   CONFIG.PSU__CRF_APB__DPDMA_REF_CTRL__DIVISOR0 {2} \
   CONFIG.PSU__CRF_APB__DPDMA_REF_CTRL__FREQMHZ {667} \
   CONFIG.PSU__CRF_APB__DPDMA_REF_CTRL__SRCSEL {DPLL} \
   CONFIG.PSU__CRF_APB__DPLL_CTRL__DIV2 {1} \
   CONFIG.PSU__CRF_APB__DPLL_CTRL__FBDIV {72} \
   CONFIG.PSU__CRF_APB__DPLL_CTRL__FRACDATA {0.000000} \
   CONFIG.PSU__CRF_APB__DPLL_CTRL__FRACFREQ {27.138} \
   CONFIG.PSU__CRF_APB__DPLL_CTRL__SRCSEL {PSS_REF_CLK} \
   CONFIG.PSU__CRF_APB__DPLL_FRAC_CFG__ENABLED {0} \
   CONFIG.PSU__CRF_APB__DPLL_TO_LPD_CTRL__DIVISOR0 {3} \
   CONFIG.PSU__CRF_APB__DP_AUDIO_REF_CTRL__ACT_FREQMHZ {24.999750} \
   CONFIG.PSU__CRF_APB__DP_AUDIO_REF_CTRL__DIVISOR0 {63} \
   CONFIG.PSU__CRF_APB__DP_AUDIO_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRF_APB__DP_AUDIO_REF_CTRL__FREQMHZ {25} \
   CONFIG.PSU__CRF_APB__DP_AUDIO_REF_CTRL__SRCSEL {RPLL} \
   CONFIG.PSU__CRF_APB__DP_AUDIO__FRAC_ENABLED {0} \
   CONFIG.PSU__CRF_APB__DP_STC_REF_CTRL__ACT_FREQMHZ {26.666400} \
   CONFIG.PSU__CRF_APB__DP_STC_REF_CTRL__DIVISOR0 {6} \
   CONFIG.PSU__CRF_APB__DP_STC_REF_CTRL__DIVISOR1 {10} \
   CONFIG.PSU__CRF_APB__DP_STC_REF_CTRL__FREQMHZ {27} \
   CONFIG.PSU__CRF_APB__DP_STC_REF_CTRL__SRCSEL {RPLL} \
   CONFIG.PSU__CRF_APB__DP_VIDEO_REF_CTRL__ACT_FREQMHZ {299.997000} \
   CONFIG.PSU__CRF_APB__DP_VIDEO_REF_CTRL__DIVISOR0 {5} \
   CONFIG.PSU__CRF_APB__DP_VIDEO_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRF_APB__DP_VIDEO_REF_CTRL__FREQMHZ {300} \
   CONFIG.PSU__CRF_APB__DP_VIDEO_REF_CTRL__SRCSEL {VPLL} \
   CONFIG.PSU__CRF_APB__DP_VIDEO__FRAC_ENABLED {0} \
   CONFIG.PSU__CRF_APB__GDMA_REF_CTRL__ACT_FREQMHZ {599.994019} \
   CONFIG.PSU__CRF_APB__GDMA_REF_CTRL__DIVISOR0 {2} \
   CONFIG.PSU__CRF_APB__GDMA_REF_CTRL__FREQMHZ {667} \
   CONFIG.PSU__CRF_APB__GDMA_REF_CTRL__SRCSEL {DPLL} \
   CONFIG.PSU__CRF_APB__GPU_REF_CTRL__ACT_FREQMHZ {599.994019} \
   CONFIG.PSU__CRF_APB__GPU_REF_CTRL__DIVISOR0 {2} \
   CONFIG.PSU__CRF_APB__GPU_REF_CTRL__FREQMHZ {600} \
   CONFIG.PSU__CRF_APB__GPU_REF_CTRL__SRCSEL {DPLL} \
   CONFIG.PSU__CRF_APB__GTGREF0_REF_CTRL__ACT_FREQMHZ {-1} \
   CONFIG.PSU__CRF_APB__GTGREF0_REF_CTRL__DIVISOR0 {-1} \
   CONFIG.PSU__CRF_APB__GTGREF0_REF_CTRL__FREQMHZ {-1} \
   CONFIG.PSU__CRF_APB__GTGREF0_REF_CTRL__SRCSEL {NA} \
   CONFIG.PSU__CRF_APB__GTGREF0__ENABLE {NA} \
   CONFIG.PSU__CRF_APB__PCIE_REF_CTRL__ACT_FREQMHZ {249.997498} \
   CONFIG.PSU__CRF_APB__PCIE_REF_CTRL__DIVISOR0 {2} \
   CONFIG.PSU__CRF_APB__PCIE_REF_CTRL__FREQMHZ {250} \
   CONFIG.PSU__CRF_APB__PCIE_REF_CTRL__SRCSEL {RPLL} \
   CONFIG.PSU__CRF_APB__SATA_REF_CTRL__ACT_FREQMHZ {249.997500} \
   CONFIG.PSU__CRF_APB__SATA_REF_CTRL__DIVISOR0 {5} \
   CONFIG.PSU__CRF_APB__SATA_REF_CTRL__FREQMHZ {250} \
   CONFIG.PSU__CRF_APB__SATA_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRF_APB__TOPSW_LSBUS_CTRL__ACT_FREQMHZ {99.999001} \
   CONFIG.PSU__CRF_APB__TOPSW_LSBUS_CTRL__DIVISOR0 {12} \
   CONFIG.PSU__CRF_APB__TOPSW_LSBUS_CTRL__FREQMHZ {100} \
   CONFIG.PSU__CRF_APB__TOPSW_LSBUS_CTRL__SRCSEL {DPLL} \
   CONFIG.PSU__CRF_APB__TOPSW_MAIN_CTRL__ACT_FREQMHZ {444.440033} \
   CONFIG.PSU__CRF_APB__TOPSW_MAIN_CTRL__DIVISOR0 {3} \
   CONFIG.PSU__CRF_APB__TOPSW_MAIN_CTRL__FREQMHZ {533.4} \
   CONFIG.PSU__CRF_APB__TOPSW_MAIN_CTRL__SRCSEL {APLL} \
   CONFIG.PSU__CRF_APB__VPLL_CTRL__DIV2 {1} \
   CONFIG.PSU__CRF_APB__VPLL_CTRL__FBDIV {90} \
   CONFIG.PSU__CRF_APB__VPLL_CTRL__FRACDATA {0.000000} \
   CONFIG.PSU__CRF_APB__VPLL_CTRL__FRACFREQ {27.138} \
   CONFIG.PSU__CRF_APB__VPLL_CTRL__SRCSEL {PSS_REF_CLK} \
   CONFIG.PSU__CRF_APB__VPLL_FRAC_CFG__ENABLED {0} \
   CONFIG.PSU__CRF_APB__VPLL_TO_LPD_CTRL__DIVISOR0 {3} \
   CONFIG.PSU__CRL_APB__ADMA_REF_CTRL__ACT_FREQMHZ {499.994995} \
   CONFIG.PSU__CRL_APB__ADMA_REF_CTRL__DIVISOR0 {3} \
   CONFIG.PSU__CRL_APB__ADMA_REF_CTRL__FREQMHZ {533.4} \
   CONFIG.PSU__CRL_APB__ADMA_REF_CTRL__SRCSEL {RPLL} \
   CONFIG.PSU__CRL_APB__AFI6_REF_CTRL__ACT_FREQMHZ {500} \
   CONFIG.PSU__CRL_APB__AFI6_REF_CTRL__DIVISOR0 {3} \
   CONFIG.PSU__CRL_APB__AFI6_REF_CTRL__FREQMHZ {500} \
   CONFIG.PSU__CRL_APB__AFI6_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__AFI6__ENABLE {0} \
   CONFIG.PSU__CRL_APB__AMS_REF_CTRL__ACT_FREQMHZ {49.999500} \
   CONFIG.PSU__CRL_APB__AMS_REF_CTRL__DIVISOR0 {24} \
   CONFIG.PSU__CRL_APB__AMS_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__AMS_REF_CTRL__FREQMHZ {50} \
   CONFIG.PSU__CRL_APB__AMS_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__CAN0_REF_CTRL__ACT_FREQMHZ {99.999001} \
   CONFIG.PSU__CRL_APB__CAN0_REF_CTRL__DIVISOR0 {12} \
   CONFIG.PSU__CRL_APB__CAN0_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__CAN0_REF_CTRL__FREQMHZ {100} \
   CONFIG.PSU__CRL_APB__CAN0_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__CAN1_REF_CTRL__ACT_FREQMHZ {99.999001} \
   CONFIG.PSU__CRL_APB__CAN1_REF_CTRL__DIVISOR0 {12} \
   CONFIG.PSU__CRL_APB__CAN1_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__CAN1_REF_CTRL__FREQMHZ {100} \
   CONFIG.PSU__CRL_APB__CAN1_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__CPU_R5_CTRL__ACT_FREQMHZ {499.994995} \
   CONFIG.PSU__CRL_APB__CPU_R5_CTRL__DIVISOR0 {3} \
   CONFIG.PSU__CRL_APB__CPU_R5_CTRL__FREQMHZ {500} \
   CONFIG.PSU__CRL_APB__CPU_R5_CTRL__SRCSEL {RPLL} \
   CONFIG.PSU__CRL_APB__CSU_PLL_CTRL__ACT_FREQMHZ {180} \
   CONFIG.PSU__CRL_APB__CSU_PLL_CTRL__DIVISOR0 {3} \
   CONFIG.PSU__CRL_APB__CSU_PLL_CTRL__FREQMHZ {180} \
   CONFIG.PSU__CRL_APB__CSU_PLL_CTRL__SRCSEL {SysOsc} \
   CONFIG.PSU__CRL_APB__DBG_LPD_CTRL__ACT_FREQMHZ {249.997498} \
   CONFIG.PSU__CRL_APB__DBG_LPD_CTRL__DIVISOR0 {6} \
   CONFIG.PSU__CRL_APB__DBG_LPD_CTRL__FREQMHZ {250} \
   CONFIG.PSU__CRL_APB__DBG_LPD_CTRL__SRCSEL {RPLL} \
   CONFIG.PSU__CRL_APB__DEBUG_R5_ATCLK_CTRL__ACT_FREQMHZ {1000} \
   CONFIG.PSU__CRL_APB__DEBUG_R5_ATCLK_CTRL__DIVISOR0 {6} \
   CONFIG.PSU__CRL_APB__DEBUG_R5_ATCLK_CTRL__FREQMHZ {1000} \
   CONFIG.PSU__CRL_APB__DEBUG_R5_ATCLK_CTRL__SRCSEL {RPLL} \
   CONFIG.PSU__CRL_APB__DLL_REF_CTRL__ACT_FREQMHZ {1499.984985} \
   CONFIG.PSU__CRL_APB__DLL_REF_CTRL__FREQMHZ {1500} \
   CONFIG.PSU__CRL_APB__DLL_REF_CTRL__SRCSEL {RPLL} \
   CONFIG.PSU__CRL_APB__GEM0_REF_CTRL__ACT_FREQMHZ {124.998750} \
   CONFIG.PSU__CRL_APB__GEM0_REF_CTRL__DIVISOR0 {12} \
   CONFIG.PSU__CRL_APB__GEM0_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__GEM0_REF_CTRL__FREQMHZ {125} \
   CONFIG.PSU__CRL_APB__GEM0_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__GEM1_REF_CTRL__ACT_FREQMHZ {124.998750} \
   CONFIG.PSU__CRL_APB__GEM1_REF_CTRL__DIVISOR0 {12} \
   CONFIG.PSU__CRL_APB__GEM1_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__GEM1_REF_CTRL__FREQMHZ {125} \
   CONFIG.PSU__CRL_APB__GEM1_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__GEM2_REF_CTRL__ACT_FREQMHZ {124.998750} \
   CONFIG.PSU__CRL_APB__GEM2_REF_CTRL__DIVISOR0 {12} \
   CONFIG.PSU__CRL_APB__GEM2_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__GEM2_REF_CTRL__FREQMHZ {125} \
   CONFIG.PSU__CRL_APB__GEM2_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__GEM3_REF_CTRL__ACT_FREQMHZ {124.998750} \
   CONFIG.PSU__CRL_APB__GEM3_REF_CTRL__DIVISOR0 {12} \
   CONFIG.PSU__CRL_APB__GEM3_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__GEM3_REF_CTRL__FREQMHZ {125} \
   CONFIG.PSU__CRL_APB__GEM3_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__GEM_TSU_REF_CTRL__ACT_FREQMHZ {249.997500} \
   CONFIG.PSU__CRL_APB__GEM_TSU_REF_CTRL__DIVISOR0 {4} \
   CONFIG.PSU__CRL_APB__GEM_TSU_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__GEM_TSU_REF_CTRL__FREQMHZ {250} \
   CONFIG.PSU__CRL_APB__GEM_TSU_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__I2C0_REF_CTRL__ACT_FREQMHZ {99.999001} \
   CONFIG.PSU__CRL_APB__I2C0_REF_CTRL__DIVISOR0 {12} \
   CONFIG.PSU__CRL_APB__I2C0_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__I2C0_REF_CTRL__FREQMHZ {100} \
   CONFIG.PSU__CRL_APB__I2C0_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__I2C1_REF_CTRL__ACT_FREQMHZ {99.999001} \
   CONFIG.PSU__CRL_APB__I2C1_REF_CTRL__DIVISOR0 {12} \
   CONFIG.PSU__CRL_APB__I2C1_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__I2C1_REF_CTRL__FREQMHZ {100} \
   CONFIG.PSU__CRL_APB__I2C1_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__IOPLL_CTRL__DIV2 {1} \
   CONFIG.PSU__CRL_APB__IOPLL_CTRL__FBDIV {72} \
   CONFIG.PSU__CRL_APB__IOPLL_CTRL__FRACDATA {0.000000} \
   CONFIG.PSU__CRL_APB__IOPLL_CTRL__FRACFREQ {27.138} \
   CONFIG.PSU__CRL_APB__IOPLL_CTRL__SRCSEL {PSS_REF_CLK} \
   CONFIG.PSU__CRL_APB__IOPLL_FRAC_CFG__ENABLED {0} \
   CONFIG.PSU__CRL_APB__IOPLL_TO_FPD_CTRL__DIVISOR0 {3} \
   CONFIG.PSU__CRL_APB__IOU_SWITCH_CTRL__ACT_FREQMHZ {249.997498} \
   CONFIG.PSU__CRL_APB__IOU_SWITCH_CTRL__DIVISOR0 {6} \
   CONFIG.PSU__CRL_APB__IOU_SWITCH_CTRL__FREQMHZ {267} \
   CONFIG.PSU__CRL_APB__IOU_SWITCH_CTRL__SRCSEL {RPLL} \
   CONFIG.PSU__CRL_APB__LPD_LSBUS_CTRL__ACT_FREQMHZ {99.999001} \
   CONFIG.PSU__CRL_APB__LPD_LSBUS_CTRL__DIVISOR0 {12} \
   CONFIG.PSU__CRL_APB__LPD_LSBUS_CTRL__FREQMHZ {100} \
   CONFIG.PSU__CRL_APB__LPD_LSBUS_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__LPD_SWITCH_CTRL__ACT_FREQMHZ {499.994995} \
   CONFIG.PSU__CRL_APB__LPD_SWITCH_CTRL__DIVISOR0 {3} \
   CONFIG.PSU__CRL_APB__LPD_SWITCH_CTRL__FREQMHZ {500} \
   CONFIG.PSU__CRL_APB__LPD_SWITCH_CTRL__SRCSEL {RPLL} \
   CONFIG.PSU__CRL_APB__NAND_REF_CTRL__ACT_FREQMHZ {100} \
   CONFIG.PSU__CRL_APB__NAND_REF_CTRL__DIVISOR0 {15} \
   CONFIG.PSU__CRL_APB__NAND_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__NAND_REF_CTRL__FREQMHZ {100} \
   CONFIG.PSU__CRL_APB__NAND_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__OCM_MAIN_CTRL__ACT_FREQMHZ {500} \
   CONFIG.PSU__CRL_APB__OCM_MAIN_CTRL__DIVISOR0 {3} \
   CONFIG.PSU__CRL_APB__OCM_MAIN_CTRL__FREQMHZ {500} \
   CONFIG.PSU__CRL_APB__OCM_MAIN_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__PCAP_CTRL__ACT_FREQMHZ {199.998001} \
   CONFIG.PSU__CRL_APB__PCAP_CTRL__DIVISOR0 {6} \
   CONFIG.PSU__CRL_APB__PCAP_CTRL__FREQMHZ {200} \
   CONFIG.PSU__CRL_APB__PCAP_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__PL0_REF_CTRL__ACT_FREQMHZ {99.999001} \
   CONFIG.PSU__CRL_APB__PL0_REF_CTRL__DIVISOR0 {12} \
   CONFIG.PSU__CRL_APB__PL0_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__PL0_REF_CTRL__FREQMHZ {100} \
   CONFIG.PSU__CRL_APB__PL0_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__PL1_REF_CTRL__ACT_FREQMHZ {149.998505} \
   CONFIG.PSU__CRL_APB__PL1_REF_CTRL__DIVISOR0 {8} \
   CONFIG.PSU__CRL_APB__PL1_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__PL1_REF_CTRL__FREQMHZ {150} \
   CONFIG.PSU__CRL_APB__PL1_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__PL2_REF_CTRL__ACT_FREQMHZ {199.998001} \
   CONFIG.PSU__CRL_APB__PL2_REF_CTRL__DIVISOR0 {6} \
   CONFIG.PSU__CRL_APB__PL2_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__PL2_REF_CTRL__FREQMHZ {200} \
   CONFIG.PSU__CRL_APB__PL2_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__PL3_REF_CTRL__ACT_FREQMHZ {299.997009} \
   CONFIG.PSU__CRL_APB__PL3_REF_CTRL__DIVISOR0 {4} \
   CONFIG.PSU__CRL_APB__PL3_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__PL3_REF_CTRL__FREQMHZ {300} \
   CONFIG.PSU__CRL_APB__PL3_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__QSPI_REF_CTRL__ACT_FREQMHZ {299.997009} \
   CONFIG.PSU__CRL_APB__QSPI_REF_CTRL__DIVISOR0 {4} \
   CONFIG.PSU__CRL_APB__QSPI_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__QSPI_REF_CTRL__FREQMHZ {300} \
   CONFIG.PSU__CRL_APB__QSPI_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__RPLL_CTRL__DIV2 {1} \
   CONFIG.PSU__CRL_APB__RPLL_CTRL__FBDIV {90} \
   CONFIG.PSU__CRL_APB__RPLL_CTRL__FRACDATA {0.000000} \
   CONFIG.PSU__CRL_APB__RPLL_CTRL__FRACFREQ {27.138} \
   CONFIG.PSU__CRL_APB__RPLL_CTRL__SRCSEL {PSS_REF_CLK} \
   CONFIG.PSU__CRL_APB__RPLL_FRAC_CFG__ENABLED {0} \
   CONFIG.PSU__CRL_APB__RPLL_TO_FPD_CTRL__DIVISOR0 {3} \
   CONFIG.PSU__CRL_APB__SDIO0_REF_CTRL__ACT_FREQMHZ {199.998001} \
   CONFIG.PSU__CRL_APB__SDIO0_REF_CTRL__DIVISOR0 {6} \
   CONFIG.PSU__CRL_APB__SDIO0_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__SDIO0_REF_CTRL__FREQMHZ {200} \
   CONFIG.PSU__CRL_APB__SDIO0_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__SDIO1_REF_CTRL__ACT_FREQMHZ {199.998001} \
   CONFIG.PSU__CRL_APB__SDIO1_REF_CTRL__DIVISOR0 {6} \
   CONFIG.PSU__CRL_APB__SDIO1_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__SDIO1_REF_CTRL__FREQMHZ {200} \
   CONFIG.PSU__CRL_APB__SDIO1_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__SPI0_REF_CTRL__ACT_FREQMHZ {199.998001} \
   CONFIG.PSU__CRL_APB__SPI0_REF_CTRL__DIVISOR0 {6} \
   CONFIG.PSU__CRL_APB__SPI0_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__SPI0_REF_CTRL__FREQMHZ {200} \
   CONFIG.PSU__CRL_APB__SPI0_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__SPI1_REF_CTRL__ACT_FREQMHZ {199.998001} \
   CONFIG.PSU__CRL_APB__SPI1_REF_CTRL__DIVISOR0 {6} \
   CONFIG.PSU__CRL_APB__SPI1_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__SPI1_REF_CTRL__FREQMHZ {200} \
   CONFIG.PSU__CRL_APB__SPI1_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__TIMESTAMP_REF_CTRL__ACT_FREQMHZ {99.999001} \
   CONFIG.PSU__CRL_APB__TIMESTAMP_REF_CTRL__DIVISOR0 {12} \
   CONFIG.PSU__CRL_APB__TIMESTAMP_REF_CTRL__FREQMHZ {100} \
   CONFIG.PSU__CRL_APB__TIMESTAMP_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__UART0_REF_CTRL__ACT_FREQMHZ {99.999001} \
   CONFIG.PSU__CRL_APB__UART0_REF_CTRL__DIVISOR0 {12} \
   CONFIG.PSU__CRL_APB__UART0_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__UART0_REF_CTRL__FREQMHZ {100} \
   CONFIG.PSU__CRL_APB__UART0_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__UART1_REF_CTRL__ACT_FREQMHZ {99.999001} \
   CONFIG.PSU__CRL_APB__UART1_REF_CTRL__DIVISOR0 {12} \
   CONFIG.PSU__CRL_APB__UART1_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__UART1_REF_CTRL__FREQMHZ {100} \
   CONFIG.PSU__CRL_APB__UART1_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__USB0_BUS_REF_CTRL__ACT_FREQMHZ {249.997498} \
   CONFIG.PSU__CRL_APB__USB0_BUS_REF_CTRL__DIVISOR0 {6} \
   CONFIG.PSU__CRL_APB__USB0_BUS_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__USB0_BUS_REF_CTRL__FREQMHZ {250} \
   CONFIG.PSU__CRL_APB__USB0_BUS_REF_CTRL__SRCSEL {RPLL} \
   CONFIG.PSU__CRL_APB__USB1_BUS_REF_CTRL__ACT_FREQMHZ {249.997500} \
   CONFIG.PSU__CRL_APB__USB1_BUS_REF_CTRL__DIVISOR0 {6} \
   CONFIG.PSU__CRL_APB__USB1_BUS_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__USB1_BUS_REF_CTRL__FREQMHZ {250} \
   CONFIG.PSU__CRL_APB__USB1_BUS_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__USB3_DUAL_REF_CTRL__ACT_FREQMHZ {19.999800} \
   CONFIG.PSU__CRL_APB__USB3_DUAL_REF_CTRL__DIVISOR0 {60} \
   CONFIG.PSU__CRL_APB__USB3_DUAL_REF_CTRL__DIVISOR1 {1} \
   CONFIG.PSU__CRL_APB__USB3_DUAL_REF_CTRL__FREQMHZ {20} \
   CONFIG.PSU__CRL_APB__USB3_DUAL_REF_CTRL__SRCSEL {IOPLL} \
   CONFIG.PSU__CRL_APB__USB3__ENABLE {1} \
   CONFIG.PSU__CSUPMU__PERIPHERAL__VALID {0} \
   CONFIG.PSU__CSU_COHERENCY {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_0__ENABLE {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_0__ERASE_BBRAM {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_10__ENABLE {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_10__ERASE_BBRAM {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_11__ENABLE {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_11__ERASE_BBRAM {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_12__ENABLE {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_12__ERASE_BBRAM {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_1__ENABLE {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_1__ERASE_BBRAM {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_2__ENABLE {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_2__ERASE_BBRAM {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_3__ENABLE {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_3__ERASE_BBRAM {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_4__ENABLE {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_4__ERASE_BBRAM {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_5__ENABLE {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_5__ERASE_BBRAM {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_6__ENABLE {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_6__ERASE_BBRAM {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_7__ENABLE {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_7__ERASE_BBRAM {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_8__ENABLE {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_8__ERASE_BBRAM {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_9__ENABLE {0} \
   CONFIG.PSU__CSU__CSU_TAMPER_9__ERASE_BBRAM {0} \
   CONFIG.PSU__CSU__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__DDRC__ADDR_MIRROR {0} \
   CONFIG.PSU__DDRC__AL {0} \
   CONFIG.PSU__DDRC__BANK_ADDR_COUNT {2} \
   CONFIG.PSU__DDRC__BG_ADDR_COUNT {1} \
   CONFIG.PSU__DDRC__BRC_MAPPING {ROW_BANK_COL} \
   CONFIG.PSU__DDRC__BUS_WIDTH {64 Bit} \
   CONFIG.PSU__DDRC__CL {17} \
   CONFIG.PSU__DDRC__CLOCK_STOP_EN {0} \
   CONFIG.PSU__DDRC__COL_ADDR_COUNT {10} \
   CONFIG.PSU__DDRC__COMPONENTS {Components} \
   CONFIG.PSU__DDRC__CWL {16} \
   CONFIG.PSU__DDRC__DDR3L_T_REF_RANGE {NA} \
   CONFIG.PSU__DDRC__DDR3_T_REF_RANGE {NA} \
   CONFIG.PSU__DDRC__DDR4_ADDR_MAPPING {0} \
   CONFIG.PSU__DDRC__DDR4_CAL_MODE_ENABLE {0} \
   CONFIG.PSU__DDRC__DDR4_CRC_CONTROL {0} \
   CONFIG.PSU__DDRC__DDR4_MAXPWR_SAVING_EN {0} \
   CONFIG.PSU__DDRC__DDR4_T_REF_MODE {0} \
   CONFIG.PSU__DDRC__DDR4_T_REF_RANGE {Normal (0-85)} \
   CONFIG.PSU__DDRC__DEEP_PWR_DOWN_EN {0} \
   CONFIG.PSU__DDRC__DEVICE_CAPACITY {8192 MBits} \
   CONFIG.PSU__DDRC__DIMM_ADDR_MIRROR {0} \
   CONFIG.PSU__DDRC__DM_DBI {DM_NO_DBI} \
   CONFIG.PSU__DDRC__DQMAP_0_3 {0} \
   CONFIG.PSU__DDRC__DQMAP_12_15 {0} \
   CONFIG.PSU__DDRC__DQMAP_16_19 {0} \
   CONFIG.PSU__DDRC__DQMAP_20_23 {0} \
   CONFIG.PSU__DDRC__DQMAP_24_27 {0} \
   CONFIG.PSU__DDRC__DQMAP_28_31 {0} \
   CONFIG.PSU__DDRC__DQMAP_32_35 {0} \
   CONFIG.PSU__DDRC__DQMAP_36_39 {0} \
   CONFIG.PSU__DDRC__DQMAP_40_43 {0} \
   CONFIG.PSU__DDRC__DQMAP_44_47 {0} \
   CONFIG.PSU__DDRC__DQMAP_48_51 {0} \
   CONFIG.PSU__DDRC__DQMAP_4_7 {0} \
   CONFIG.PSU__DDRC__DQMAP_52_55 {0} \
   CONFIG.PSU__DDRC__DQMAP_56_59 {0} \
   CONFIG.PSU__DDRC__DQMAP_60_63 {0} \
   CONFIG.PSU__DDRC__DQMAP_64_67 {0} \
   CONFIG.PSU__DDRC__DQMAP_68_71 {0} \
   CONFIG.PSU__DDRC__DQMAP_8_11 {0} \
   CONFIG.PSU__DDRC__DRAM_WIDTH {16 Bits} \
   CONFIG.PSU__DDRC__ECC {Enabled} \
   CONFIG.PSU__DDRC__ECC_SCRUB {0} \
   CONFIG.PSU__DDRC__ENABLE {1} \
   CONFIG.PSU__DDRC__ENABLE_2T_TIMING {0} \
   CONFIG.PSU__DDRC__ENABLE_DP_SWITCH {0} \
   CONFIG.PSU__DDRC__ENABLE_LP4_HAS_ECC_COMP {ERR: 1  | 0} \
   CONFIG.PSU__DDRC__ENABLE_LP4_SLOWBOOT {0} \
   CONFIG.PSU__DDRC__EN_2ND_CLK {0} \
   CONFIG.PSU__DDRC__FGRM {1X} \
   CONFIG.PSU__DDRC__FREQ_MHZ {1} \
   CONFIG.PSU__DDRC__LPDDR3_DUALRANK_SDP {0} \
   CONFIG.PSU__DDRC__LPDDR3_T_REF_RANGE {NA} \
   CONFIG.PSU__DDRC__LPDDR4_T_REF_RANGE {NA} \
   CONFIG.PSU__DDRC__LP_ASR {manual normal} \
   CONFIG.PSU__DDRC__MEMORY_TYPE {DDR 4} \
   CONFIG.PSU__DDRC__PARITY_ENABLE {0} \
   CONFIG.PSU__DDRC__PER_BANK_REFRESH {0} \
   CONFIG.PSU__DDRC__PHY_DBI_MODE {0} \
   CONFIG.PSU__DDRC__PLL_BYPASS {0} \
   CONFIG.PSU__DDRC__PWR_DOWN_EN {0} \
   CONFIG.PSU__DDRC__RANK_ADDR_COUNT {0} \
   CONFIG.PSU__DDRC__RD_DQS_CENTER {0} \
   CONFIG.PSU__DDRC__ROW_ADDR_COUNT {16} \
   CONFIG.PSU__DDRC__SB_TARGET {15-15-15} \
   CONFIG.PSU__DDRC__SELF_REF_ABORT {0} \
   CONFIG.PSU__DDRC__SPEED_BIN {DDR4_2400T} \
   CONFIG.PSU__DDRC__STATIC_RD_MODE {0} \
   CONFIG.PSU__DDRC__TRAIN_DATA_EYE {1} \
   CONFIG.PSU__DDRC__TRAIN_READ_GATE {1} \
   CONFIG.PSU__DDRC__TRAIN_WRITE_LEVEL {1} \
   CONFIG.PSU__DDRC__T_FAW {30.0} \
   CONFIG.PSU__DDRC__T_RAS_MIN {32} \
   CONFIG.PSU__DDRC__T_RC {46.16} \
   CONFIG.PSU__DDRC__T_RCD {17} \
   CONFIG.PSU__DDRC__T_RP {17} \
   CONFIG.PSU__DDRC__VENDOR_PART {OTHERS} \
   CONFIG.PSU__DDRC__VIDEO_BUFFER_SIZE {0} \
   CONFIG.PSU__DDRC__VREF {1} \
   CONFIG.PSU__DDR_HIGH_ADDRESS_GUI_ENABLE {1} \
   CONFIG.PSU__DDR_QOS_ENABLE {0} \
   CONFIG.PSU__DDR_QOS_FIX_HP0_RDQOS {} \
   CONFIG.PSU__DDR_QOS_FIX_HP0_WRQOS {} \
   CONFIG.PSU__DDR_QOS_FIX_HP1_RDQOS {} \
   CONFIG.PSU__DDR_QOS_FIX_HP1_WRQOS {} \
   CONFIG.PSU__DDR_QOS_FIX_HP2_RDQOS {} \
   CONFIG.PSU__DDR_QOS_FIX_HP2_WRQOS {} \
   CONFIG.PSU__DDR_QOS_FIX_HP3_RDQOS {} \
   CONFIG.PSU__DDR_QOS_FIX_HP3_WRQOS {} \
   CONFIG.PSU__DDR_QOS_HP0_RDQOS {} \
   CONFIG.PSU__DDR_QOS_HP0_WRQOS {} \
   CONFIG.PSU__DDR_QOS_HP1_RDQOS {} \
   CONFIG.PSU__DDR_QOS_HP1_WRQOS {} \
   CONFIG.PSU__DDR_QOS_HP2_RDQOS {} \
   CONFIG.PSU__DDR_QOS_HP2_WRQOS {} \
   CONFIG.PSU__DDR_QOS_HP3_RDQOS {} \
   CONFIG.PSU__DDR_QOS_HP3_WRQOS {} \
   CONFIG.PSU__DDR_QOS_RD_HPR_THRSHLD {} \
   CONFIG.PSU__DDR_QOS_RD_LPR_THRSHLD {} \
   CONFIG.PSU__DDR_QOS_WR_THRSHLD {} \
   CONFIG.PSU__DDR_SW_REFRESH_ENABLED {1} \
   CONFIG.PSU__DDR__INTERFACE__FREQMHZ {600.000} \
   CONFIG.PSU__DEVICE_TYPE {EV} \
   CONFIG.PSU__DISPLAYPORT__LANE0__ENABLE {0} \
   CONFIG.PSU__DISPLAYPORT__LANE0__IO {<Select>} \
   CONFIG.PSU__DISPLAYPORT__LANE1__ENABLE {0} \
   CONFIG.PSU__DISPLAYPORT__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__DLL__ISUSED {1} \
   CONFIG.PSU__DPAUX__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__DPAUX__PERIPHERAL__IO {<Select>} \
   CONFIG.PSU__DP__LANE_SEL {<Select>} \
   CONFIG.PSU__DP__REF_CLK_FREQ {<Select>} \
   CONFIG.PSU__DP__REF_CLK_SEL {<Select>} \
   CONFIG.PSU__ENABLE__DDR__REFRESH__SIGNALS {0} \
   CONFIG.PSU__ENET0__FIFO__ENABLE {0} \
   CONFIG.PSU__ENET0__GRP_MDIO__ENABLE {0} \
   CONFIG.PSU__ENET0__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__ENET0__PERIPHERAL__IO {<Select>} \
   CONFIG.PSU__ENET0__PTP__ENABLE {0} \
   CONFIG.PSU__ENET0__TSU__ENABLE {0} \
   CONFIG.PSU__ENET1__FIFO__ENABLE {0} \
   CONFIG.PSU__ENET1__GRP_MDIO__ENABLE {0} \
   CONFIG.PSU__ENET1__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__ENET1__PERIPHERAL__IO {<Select>} \
   CONFIG.PSU__ENET1__PTP__ENABLE {0} \
   CONFIG.PSU__ENET1__TSU__ENABLE {0} \
   CONFIG.PSU__ENET2__FIFO__ENABLE {0} \
   CONFIG.PSU__ENET2__GRP_MDIO__ENABLE {0} \
   CONFIG.PSU__ENET2__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__ENET2__PERIPHERAL__IO {<Select>} \
   CONFIG.PSU__ENET2__PTP__ENABLE {0} \
   CONFIG.PSU__ENET2__TSU__ENABLE {0} \
   CONFIG.PSU__ENET3__FIFO__ENABLE {0} \
   CONFIG.PSU__ENET3__GRP_MDIO__ENABLE {0} \
   CONFIG.PSU__ENET3__GRP_MDIO__IO {<Select>} \
   CONFIG.PSU__ENET3__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__ENET3__PERIPHERAL__IO {<Select>} \
   CONFIG.PSU__ENET3__PTP__ENABLE {0} \
   CONFIG.PSU__ENET3__TSU__ENABLE {0} \
   CONFIG.PSU__EN_AXI_STATUS_PORTS {0} \
   CONFIG.PSU__EN_EMIO_TRACE {0} \
   CONFIG.PSU__EP__IP {0} \
   CONFIG.PSU__EXPAND__CORESIGHT {0} \
   CONFIG.PSU__EXPAND__FPD_SLAVES {0} \
   CONFIG.PSU__EXPAND__GIC {0} \
   CONFIG.PSU__EXPAND__LOWER_LPS_SLAVES {0} \
   CONFIG.PSU__EXPAND__UPPER_LPS_SLAVES {0} \
   CONFIG.PSU__FPDMASTERS_COHERENCY {0} \
   CONFIG.PSU__FPD_SLCR__WDT1__ACT_FREQMHZ {99.999001} \
   CONFIG.PSU__FPD_SLCR__WDT1__FREQMHZ {99.999001} \
   CONFIG.PSU__FPD_SLCR__WDT_CLK_SEL__SELECT {APB} \
   CONFIG.PSU__FPGA_PL0_ENABLE {1} \
   CONFIG.PSU__FPGA_PL1_ENABLE {1} \
   CONFIG.PSU__FPGA_PL2_ENABLE {1} \
   CONFIG.PSU__FPGA_PL3_ENABLE {1} \
   CONFIG.PSU__FP__POWER__ON {1} \
   CONFIG.PSU__FTM__CTI_IN_0 {0} \
   CONFIG.PSU__FTM__CTI_IN_1 {0} \
   CONFIG.PSU__FTM__CTI_IN_2 {0} \
   CONFIG.PSU__FTM__CTI_IN_3 {0} \
   CONFIG.PSU__FTM__CTI_OUT_0 {0} \
   CONFIG.PSU__FTM__CTI_OUT_1 {0} \
   CONFIG.PSU__FTM__CTI_OUT_2 {0} \
   CONFIG.PSU__FTM__CTI_OUT_3 {0} \
   CONFIG.PSU__FTM__GPI {0} \
   CONFIG.PSU__FTM__GPO {0} \
   CONFIG.PSU__GEM0_COHERENCY {0} \
   CONFIG.PSU__GEM0_ROUTE_THROUGH_FPD {0} \
   CONFIG.PSU__GEM1_COHERENCY {0} \
   CONFIG.PSU__GEM1_ROUTE_THROUGH_FPD {0} \
   CONFIG.PSU__GEM2_COHERENCY {0} \
   CONFIG.PSU__GEM2_ROUTE_THROUGH_FPD {0} \
   CONFIG.PSU__GEM3_COHERENCY {0} \
   CONFIG.PSU__GEM3_ROUTE_THROUGH_FPD {0} \
   CONFIG.PSU__GEM__TSU__ENABLE {0} \
   CONFIG.PSU__GEN_IPI_0__MASTER {APU} \
   CONFIG.PSU__GEN_IPI_10__MASTER {NONE} \
   CONFIG.PSU__GEN_IPI_1__MASTER {RPU0} \
   CONFIG.PSU__GEN_IPI_2__MASTER {RPU1} \
   CONFIG.PSU__GEN_IPI_3__MASTER {PMU} \
   CONFIG.PSU__GEN_IPI_4__MASTER {PMU} \
   CONFIG.PSU__GEN_IPI_5__MASTER {PMU} \
   CONFIG.PSU__GEN_IPI_6__MASTER {PMU} \
   CONFIG.PSU__GEN_IPI_7__MASTER {NONE} \
   CONFIG.PSU__GEN_IPI_8__MASTER {NONE} \
   CONFIG.PSU__GEN_IPI_9__MASTER {NONE} \
   CONFIG.PSU__GPIO0_MIO__IO {MIO 0 .. 25} \
   CONFIG.PSU__GPIO0_MIO__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__GPIO1_MIO__IO {MIO 26 .. 51} \
   CONFIG.PSU__GPIO1_MIO__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__GPIO2_MIO__IO {MIO 52 .. 77} \
   CONFIG.PSU__GPIO2_MIO__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__GPIO_EMIO_WIDTH {95} \
   CONFIG.PSU__GPIO_EMIO__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__GPIO_EMIO__PERIPHERAL__IO {95} \
   CONFIG.PSU__GPIO_EMIO__WIDTH {[94:0]} \
   CONFIG.PSU__GPU_PP0__POWER__ON {1} \
   CONFIG.PSU__GPU_PP1__POWER__ON {1} \
   CONFIG.PSU__GT_REF_CLK__FREQMHZ {33.333} \
   CONFIG.PSU__GT__LINK_SPEED {<Select>} \
   CONFIG.PSU__GT__PRE_EMPH_LVL_4 {} \
   CONFIG.PSU__GT__VLT_SWNG_LVL_4 {} \
   CONFIG.PSU__HIGH_ADDRESS__ENABLE {1} \
   CONFIG.PSU__HPM0_FPD__NUM_READ_THREADS {4} \
   CONFIG.PSU__HPM0_FPD__NUM_WRITE_THREADS {4} \
   CONFIG.PSU__HPM0_LPD__NUM_READ_THREADS {4} \
   CONFIG.PSU__HPM0_LPD__NUM_WRITE_THREADS {4} \
   CONFIG.PSU__HPM1_FPD__NUM_READ_THREADS {4} \
   CONFIG.PSU__HPM1_FPD__NUM_WRITE_THREADS {4} \
   CONFIG.PSU__I2C0_LOOP_I2C1__ENABLE {0} \
   CONFIG.PSU__I2C0__GRP_INT__ENABLE {0} \
   CONFIG.PSU__I2C0__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__I2C0__PERIPHERAL__IO {MIO 74 .. 75} \
   CONFIG.PSU__I2C1__GRP_INT__ENABLE {0} \
   CONFIG.PSU__I2C1__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__I2C1__PERIPHERAL__IO {EMIO} \
   CONFIG.PSU__IOU_SLCR__IOU_TTC_APB_CLK__TTC0_SEL {APB} \
   CONFIG.PSU__IOU_SLCR__IOU_TTC_APB_CLK__TTC1_SEL {APB} \
   CONFIG.PSU__IOU_SLCR__IOU_TTC_APB_CLK__TTC2_SEL {APB} \
   CONFIG.PSU__IOU_SLCR__IOU_TTC_APB_CLK__TTC3_SEL {PSS_REF_CLK} \
   CONFIG.PSU__IOU_SLCR__TTC0__ACT_FREQMHZ {100.000000} \
   CONFIG.PSU__IOU_SLCR__TTC0__FREQMHZ {100.000000} \
   CONFIG.PSU__IOU_SLCR__TTC1__ACT_FREQMHZ {100.000000} \
   CONFIG.PSU__IOU_SLCR__TTC1__FREQMHZ {100.000000} \
   CONFIG.PSU__IOU_SLCR__TTC2__ACT_FREQMHZ {100.000000} \
   CONFIG.PSU__IOU_SLCR__TTC2__FREQMHZ {100.000000} \
   CONFIG.PSU__IOU_SLCR__TTC3__ACT_FREQMHZ {33.333000} \
   CONFIG.PSU__IOU_SLCR__TTC3__FREQMHZ {33.333000} \
   CONFIG.PSU__IOU_SLCR__WDT0__ACT_FREQMHZ {99.999001} \
   CONFIG.PSU__IOU_SLCR__WDT0__FREQMHZ {99.999001} \
   CONFIG.PSU__IOU_SLCR__WDT_CLK_SEL__SELECT {APB} \
   CONFIG.PSU__IRQ_P2F_ADMA_CHAN__INT {0} \
   CONFIG.PSU__IRQ_P2F_AIB_AXI__INT {0} \
   CONFIG.PSU__IRQ_P2F_AMS__INT {0} \
   CONFIG.PSU__IRQ_P2F_APM_FPD__INT {0} \
   CONFIG.PSU__IRQ_P2F_APU_COMM__INT {0} \
   CONFIG.PSU__IRQ_P2F_APU_CPUMNT__INT {0} \
   CONFIG.PSU__IRQ_P2F_APU_CTI__INT {0} \
   CONFIG.PSU__IRQ_P2F_APU_EXTERR__INT {0} \
   CONFIG.PSU__IRQ_P2F_APU_IPI__INT {0} \
   CONFIG.PSU__IRQ_P2F_APU_L2ERR__INT {0} \
   CONFIG.PSU__IRQ_P2F_APU_PMU__INT {0} \
   CONFIG.PSU__IRQ_P2F_APU_REGS__INT {0} \
   CONFIG.PSU__IRQ_P2F_ATB_LPD__INT {0} \
   CONFIG.PSU__IRQ_P2F_CAN0__INT {0} \
   CONFIG.PSU__IRQ_P2F_CAN1__INT {0} \
   CONFIG.PSU__IRQ_P2F_CLKMON__INT {0} \
   CONFIG.PSU__IRQ_P2F_CSUPMU_WDT__INT {0} \
   CONFIG.PSU__IRQ_P2F_CSU_DMA__INT {0} \
   CONFIG.PSU__IRQ_P2F_CSU__INT {0} \
   CONFIG.PSU__IRQ_P2F_DDR_SS__INT {0} \
   CONFIG.PSU__IRQ_P2F_DPDMA__INT {0} \
   CONFIG.PSU__IRQ_P2F_DPORT__INT {0} \
   CONFIG.PSU__IRQ_P2F_EFUSE__INT {0} \
   CONFIG.PSU__IRQ_P2F_ENT0_WAKEUP__INT {0} \
   CONFIG.PSU__IRQ_P2F_ENT0__INT {0} \
   CONFIG.PSU__IRQ_P2F_ENT1_WAKEUP__INT {0} \
   CONFIG.PSU__IRQ_P2F_ENT1__INT {0} \
   CONFIG.PSU__IRQ_P2F_ENT2_WAKEUP__INT {0} \
   CONFIG.PSU__IRQ_P2F_ENT2__INT {0} \
   CONFIG.PSU__IRQ_P2F_ENT3_WAKEUP__INT {0} \
   CONFIG.PSU__IRQ_P2F_ENT3__INT {0} \
   CONFIG.PSU__IRQ_P2F_FPD_APB__INT {0} \
   CONFIG.PSU__IRQ_P2F_FPD_ATB_ERR__INT {0} \
   CONFIG.PSU__IRQ_P2F_FP_WDT__INT {0} \
   CONFIG.PSU__IRQ_P2F_GDMA_CHAN__INT {0} \
   CONFIG.PSU__IRQ_P2F_GPIO__INT {0} \
   CONFIG.PSU__IRQ_P2F_GPU__INT {0} \
   CONFIG.PSU__IRQ_P2F_I2C0__INT {0} \
   CONFIG.PSU__IRQ_P2F_I2C1__INT {0} \
   CONFIG.PSU__IRQ_P2F_LPD_APB__INT {0} \
   CONFIG.PSU__IRQ_P2F_LPD_APM__INT {0} \
   CONFIG.PSU__IRQ_P2F_LP_WDT__INT {0} \
   CONFIG.PSU__IRQ_P2F_NAND__INT {0} \
   CONFIG.PSU__IRQ_P2F_OCM_ERR__INT {0} \
   CONFIG.PSU__IRQ_P2F_PCIE_DMA__INT {0} \
   CONFIG.PSU__IRQ_P2F_PCIE_LEGACY__INT {0} \
   CONFIG.PSU__IRQ_P2F_PCIE_MSC__INT {0} \
   CONFIG.PSU__IRQ_P2F_PCIE_MSI__INT {0} \
   CONFIG.PSU__IRQ_P2F_PL_IPI__INT {0} \
   CONFIG.PSU__IRQ_P2F_QSPI__INT {0} \
   CONFIG.PSU__IRQ_P2F_R5_CORE0_ECC_ERR__INT {0} \
   CONFIG.PSU__IRQ_P2F_R5_CORE1_ECC_ERR__INT {0} \
   CONFIG.PSU__IRQ_P2F_RPU_IPI__INT {0} \
   CONFIG.PSU__IRQ_P2F_RPU_PERMON__INT {0} \
   CONFIG.PSU__IRQ_P2F_RTC_ALARM__INT {0} \
   CONFIG.PSU__IRQ_P2F_RTC_SECONDS__INT {0} \
   CONFIG.PSU__IRQ_P2F_SATA__INT {0} \
   CONFIG.PSU__IRQ_P2F_SDIO0_WAKE__INT {0} \
   CONFIG.PSU__IRQ_P2F_SDIO0__INT {0} \
   CONFIG.PSU__IRQ_P2F_SDIO1_WAKE__INT {0} \
   CONFIG.PSU__IRQ_P2F_SDIO1__INT {0} \
   CONFIG.PSU__IRQ_P2F_SPI0__INT {0} \
   CONFIG.PSU__IRQ_P2F_SPI1__INT {0} \
   CONFIG.PSU__IRQ_P2F_TTC0__INT0 {0} \
   CONFIG.PSU__IRQ_P2F_TTC0__INT1 {0} \
   CONFIG.PSU__IRQ_P2F_TTC0__INT2 {0} \
   CONFIG.PSU__IRQ_P2F_TTC1__INT0 {0} \
   CONFIG.PSU__IRQ_P2F_TTC1__INT1 {0} \
   CONFIG.PSU__IRQ_P2F_TTC1__INT2 {0} \
   CONFIG.PSU__IRQ_P2F_TTC2__INT0 {0} \
   CONFIG.PSU__IRQ_P2F_TTC2__INT1 {0} \
   CONFIG.PSU__IRQ_P2F_TTC2__INT2 {0} \
   CONFIG.PSU__IRQ_P2F_TTC3__INT0 {0} \
   CONFIG.PSU__IRQ_P2F_TTC3__INT1 {0} \
   CONFIG.PSU__IRQ_P2F_TTC3__INT2 {0} \
   CONFIG.PSU__IRQ_P2F_UART0__INT {0} \
   CONFIG.PSU__IRQ_P2F_UART1__INT {0} \
   CONFIG.PSU__IRQ_P2F_USB3_ENDPOINT__INT0 {0} \
   CONFIG.PSU__IRQ_P2F_USB3_ENDPOINT__INT1 {0} \
   CONFIG.PSU__IRQ_P2F_USB3_OTG__INT0 {0} \
   CONFIG.PSU__IRQ_P2F_USB3_OTG__INT1 {0} \
   CONFIG.PSU__IRQ_P2F_USB3_PMU_WAKEUP__INT {0} \
   CONFIG.PSU__IRQ_P2F_XMPU_FPD__INT {0} \
   CONFIG.PSU__IRQ_P2F_XMPU_LPD__INT {0} \
   CONFIG.PSU__IRQ_P2F__INTF_FPD_SMMU__INT {0} \
   CONFIG.PSU__IRQ_P2F__INTF_PPD_CCI__INT {0} \
   CONFIG.PSU__L2_BANK0__POWER__ON {1} \
   CONFIG.PSU__LPDMA0_COHERENCY {0} \
   CONFIG.PSU__LPDMA1_COHERENCY {0} \
   CONFIG.PSU__LPDMA2_COHERENCY {0} \
   CONFIG.PSU__LPDMA3_COHERENCY {0} \
   CONFIG.PSU__LPDMA4_COHERENCY {0} \
   CONFIG.PSU__LPDMA5_COHERENCY {0} \
   CONFIG.PSU__LPDMA6_COHERENCY {0} \
   CONFIG.PSU__LPDMA7_COHERENCY {0} \
   CONFIG.PSU__LPD_SLCR__CSUPMU_WDT_CLK_SEL__SELECT {APB} \
   CONFIG.PSU__LPD_SLCR__CSUPMU__ACT_FREQMHZ {100} \
   CONFIG.PSU__LPD_SLCR__CSUPMU__FREQMHZ {100} \
   CONFIG.PSU__MAXIGP0__DATA_WIDTH {128} \
   CONFIG.PSU__MAXIGP1__DATA_WIDTH {128} \
   CONFIG.PSU__MAXIGP2__DATA_WIDTH {32} \
   CONFIG.PSU__M_AXI_GP0_SUPPORTS_NARROW_BURST {1} \
   CONFIG.PSU__M_AXI_GP1_SUPPORTS_NARROW_BURST {1} \
   CONFIG.PSU__M_AXI_GP2_SUPPORTS_NARROW_BURST {1} \
   CONFIG.PSU__NAND_COHERENCY {0} \
   CONFIG.PSU__NAND_ROUTE_THROUGH_FPD {0} \
   CONFIG.PSU__NAND__CHIP_ENABLE__ENABLE {0} \
   CONFIG.PSU__NAND__DATA_STROBE__ENABLE {0} \
   CONFIG.PSU__NAND__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__NAND__READY0_BUSY__ENABLE {0} \
   CONFIG.PSU__NAND__READY1_BUSY__ENABLE {0} \
   CONFIG.PSU__NAND__READY_BUSY__ENABLE {0} \
   CONFIG.PSU__NUM_FABRIC_RESETS {1} \
   CONFIG.PSU__OCM_BANK0__POWER__ON {1} \
   CONFIG.PSU__OCM_BANK1__POWER__ON {1} \
   CONFIG.PSU__OCM_BANK2__POWER__ON {1} \
   CONFIG.PSU__OCM_BANK3__POWER__ON {1} \
   CONFIG.PSU__OVERRIDE__BASIC_CLOCK {0} \
   CONFIG.PSU__PCIE__ACS_VIOLAION {0} \
   CONFIG.PSU__PCIE__ACS_VIOLATION {0} \
   CONFIG.PSU__PCIE__AER_CAPABILITY {0} \
   CONFIG.PSU__PCIE__ATOMICOP_EGRESS_BLOCKED {0} \
   CONFIG.PSU__PCIE__BAR0_64BIT {0} \
   CONFIG.PSU__PCIE__BAR0_ENABLE {0} \
   CONFIG.PSU__PCIE__BAR0_PREFETCHABLE {0} \
   CONFIG.PSU__PCIE__BAR0_SCALE {<Select>} \
   CONFIG.PSU__PCIE__BAR0_SIZE {<Select>} \
   CONFIG.PSU__PCIE__BAR0_TYPE {<Select>} \
   CONFIG.PSU__PCIE__BAR0_VAL {0x0} \
   CONFIG.PSU__PCIE__BAR1_64BIT {0} \
   CONFIG.PSU__PCIE__BAR1_ENABLE {0} \
   CONFIG.PSU__PCIE__BAR1_PREFETCHABLE {0} \
   CONFIG.PSU__PCIE__BAR1_VAL {0x0} \
   CONFIG.PSU__PCIE__BAR2_64BIT {0} \
   CONFIG.PSU__PCIE__BAR2_ENABLE {0} \
   CONFIG.PSU__PCIE__BAR2_PREFETCHABLE {0} \
   CONFIG.PSU__PCIE__BAR2_VAL {0x0} \
   CONFIG.PSU__PCIE__BAR3_64BIT {0} \
   CONFIG.PSU__PCIE__BAR3_ENABLE {0} \
   CONFIG.PSU__PCIE__BAR3_PREFETCHABLE {0} \
   CONFIG.PSU__PCIE__BAR3_VAL {0x0} \
   CONFIG.PSU__PCIE__BAR4_64BIT {0} \
   CONFIG.PSU__PCIE__BAR4_ENABLE {0} \
   CONFIG.PSU__PCIE__BAR4_PREFETCHABLE {0} \
   CONFIG.PSU__PCIE__BAR4_VAL {0x0} \
   CONFIG.PSU__PCIE__BAR5_64BIT {0} \
   CONFIG.PSU__PCIE__BAR5_ENABLE {0} \
   CONFIG.PSU__PCIE__BAR5_PREFETCHABLE {0} \
   CONFIG.PSU__PCIE__BAR5_VAL {0x0} \
   CONFIG.PSU__PCIE__BRIDGE_BAR_INDICATOR {<Select>} \
   CONFIG.PSU__PCIE__CLASS_CODE_BASE {0x06} \
   CONFIG.PSU__PCIE__CLASS_CODE_INTERFACE {0x0} \
   CONFIG.PSU__PCIE__CLASS_CODE_SUB {0x4} \
   CONFIG.PSU__PCIE__CLASS_CODE_VALUE {0x60400} \
   CONFIG.PSU__PCIE__COMPLETER_ABORT {0} \
   CONFIG.PSU__PCIE__COMPLTION_TIMEOUT {0} \
   CONFIG.PSU__PCIE__CORRECTABLE_INT_ERR {0} \
   CONFIG.PSU__PCIE__CRS_SW_VISIBILITY {0} \
   CONFIG.PSU__PCIE__DEVICE_ID {0xD021} \
   CONFIG.PSU__PCIE__DEVICE_PORT_TYPE {Root Port} \
   CONFIG.PSU__PCIE__ECRC_CHECK {0} \
   CONFIG.PSU__PCIE__ECRC_ERR {0} \
   CONFIG.PSU__PCIE__ECRC_GEN {0} \
   CONFIG.PSU__PCIE__EROM_ENABLE {0} \
   CONFIG.PSU__PCIE__EROM_VAL {0x0} \
   CONFIG.PSU__PCIE__FLOW_CONTROL_ERR {0} \
   CONFIG.PSU__PCIE__FLOW_CONTROL_PROTOCOL_ERR {0} \
   CONFIG.PSU__PCIE__HEADER_LOG_OVERFLOW {0} \
   CONFIG.PSU__PCIE__INTX_GENERATION {1} \
   CONFIG.PSU__PCIE__INTX_PIN {<Select>} \
   CONFIG.PSU__PCIE__LANE0__ENABLE {1} \
   CONFIG.PSU__PCIE__LANE0__IO {GT Lane0} \
   CONFIG.PSU__PCIE__LANE1__ENABLE {1} \
   CONFIG.PSU__PCIE__LANE1__IO {GT Lane1} \
   CONFIG.PSU__PCIE__LANE2__ENABLE {0} \
   CONFIG.PSU__PCIE__LANE3__ENABLE {0} \
   CONFIG.PSU__PCIE__LINK_SPEED {5.0 Gb/s} \
   CONFIG.PSU__PCIE__MAXIMUM_LINK_WIDTH {x2} \
   CONFIG.PSU__PCIE__MAX_PAYLOAD_SIZE {256 bytes} \
   CONFIG.PSU__PCIE__MC_BLOCKED_TLP {0} \
   CONFIG.PSU__PCIE__MSIX_BAR_INDICATOR {} \
   CONFIG.PSU__PCIE__MSIX_CAPABILITY {0} \
   CONFIG.PSU__PCIE__MSIX_PBA_BAR_INDICATOR {} \
   CONFIG.PSU__PCIE__MSIX_PBA_OFFSET {0} \
   CONFIG.PSU__PCIE__MSIX_TABLE_OFFSET {0} \
   CONFIG.PSU__PCIE__MSIX_TABLE_SIZE {0} \
   CONFIG.PSU__PCIE__MSI_64BIT_ADDR_CAPABLE {0} \
   CONFIG.PSU__PCIE__MSI_CAPABILITY {0} \
   CONFIG.PSU__PCIE__MULTIHEADER {0} \
   CONFIG.PSU__PCIE__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__PCIE__PERIPHERAL__ENDPOINT_ENABLE {0} \
   CONFIG.PSU__PCIE__PERIPHERAL__ENDPOINT_IO {<Select>} \
   CONFIG.PSU__PCIE__PERIPHERAL__ROOTPORT_ENABLE {1} \
   CONFIG.PSU__PCIE__PERIPHERAL__ROOTPORT_IO {MIO 68} \
   CONFIG.PSU__PCIE__PERM_ROOT_ERR_UPDATE {0} \
   CONFIG.PSU__PCIE__RECEIVER_ERR {0} \
   CONFIG.PSU__PCIE__RECEIVER_OVERFLOW {0} \
   CONFIG.PSU__PCIE__REF_CLK_FREQ {100} \
   CONFIG.PSU__PCIE__REF_CLK_SEL {Ref Clk0} \
   CONFIG.PSU__PCIE__RESET__POLARITY {Active Low} \
   CONFIG.PSU__PCIE__REVISION_ID {0x0} \
   CONFIG.PSU__PCIE__SUBSYSTEM_ID {0x7} \
   CONFIG.PSU__PCIE__SUBSYSTEM_VENDOR_ID {0x10EE} \
   CONFIG.PSU__PCIE__SURPRISE_DOWN {0} \
   CONFIG.PSU__PCIE__TLP_PREFIX_BLOCKED {0} \
   CONFIG.PSU__PCIE__UNCORRECTABL_INT_ERR {0} \
   CONFIG.PSU__PCIE__VENDOR_ID {0x10EE} \
   CONFIG.PSU__PJTAG__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__PL_CLK0_BUF {TRUE} \
   CONFIG.PSU__PL_CLK1_BUF {TRUE} \
   CONFIG.PSU__PL_CLK2_BUF {FALSE} \
   CONFIG.PSU__PL_CLK3_BUF {FALSE} \
   CONFIG.PSU__PL__POWER__ON {1} \
   CONFIG.PSU__PMU_COHERENCY {0} \
   CONFIG.PSU__PMU__AIBACK__ENABLE {0} \
   CONFIG.PSU__PMU__EMIO_GPI__ENABLE {0} \
   CONFIG.PSU__PMU__EMIO_GPO__ENABLE {0} \
   CONFIG.PSU__PMU__GPI0__ENABLE {0} \
   CONFIG.PSU__PMU__GPI1__ENABLE {0} \
   CONFIG.PSU__PMU__GPI2__ENABLE {0} \
   CONFIG.PSU__PMU__GPI3__ENABLE {0} \
   CONFIG.PSU__PMU__GPI4__ENABLE {0} \
   CONFIG.PSU__PMU__GPI5__ENABLE {0} \
   CONFIG.PSU__PMU__GPO0__ENABLE {0} \
   CONFIG.PSU__PMU__GPO0__IO {<Select>} \
   CONFIG.PSU__PMU__GPO1__ENABLE {0} \
   CONFIG.PSU__PMU__GPO1__IO {<Select>} \
   CONFIG.PSU__PMU__GPO2__ENABLE {0} \
   CONFIG.PSU__PMU__GPO2__IO {<Select>} \
   CONFIG.PSU__PMU__GPO2__POLARITY {low} \
   CONFIG.PSU__PMU__GPO3__ENABLE {0} \
   CONFIG.PSU__PMU__GPO3__IO {<Select>} \
   CONFIG.PSU__PMU__GPO3__POLARITY {low} \
   CONFIG.PSU__PMU__GPO4__ENABLE {0} \
   CONFIG.PSU__PMU__GPO4__IO {<Select>} \
   CONFIG.PSU__PMU__GPO4__POLARITY {low} \
   CONFIG.PSU__PMU__GPO5__ENABLE {0} \
   CONFIG.PSU__PMU__GPO5__IO {<Select>} \
   CONFIG.PSU__PMU__GPO5__POLARITY {low} \
   CONFIG.PSU__PMU__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__PMU__PLERROR__ENABLE {0} \
   CONFIG.PSU__PRESET_APPLIED {0} \
   CONFIG.PSU__PROTECTION__DDR_SEGMENTS {NONE} \
   CONFIG.PSU__PROTECTION__DEBUG {0} \
   CONFIG.PSU__PROTECTION__ENABLE {0} \
   CONFIG.PSU__PROTECTION__FPD_SEGMENTS {SA:0xFD1A0000 ;SIZE:1280;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware|SA:0xFD000000 ;SIZE:64;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware|SA:0xFD010000 ;SIZE:64;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware|SA:0xFD020000 ;SIZE:64;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware|SA:0xFD030000 ;SIZE:64;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware|SA:0xFD040000 ;SIZE:64;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware|SA:0xFD050000 ;SIZE:64;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware|SA:0xFD610000 ;SIZE:512;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware|SA:0xFD5D0000 ;SIZE:64;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware} \
   CONFIG.PSU__PROTECTION__LOCK_UNUSED_SEGMENTS {0} \
   CONFIG.PSU__PROTECTION__LPD_SEGMENTS {SA:0xFF980000 ;SIZE:64;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware|SA:0xFF5E0000 ;SIZE:2560;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware|SA:0xFFCC0000 ;SIZE:64;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware|SA:0xFF180000 ;SIZE:768;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware|SA:0xFF410000 ;SIZE:640;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware|SA:0xFFA70000 ;SIZE:64;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware|SA:0xFF9A0000 ;SIZE:64;UNIT:KB ;RegionTZ:Secure ;WrAllowed:Read/Write;subsystemId:PMU Firmware} \
   CONFIG.PSU__PROTECTION__MASTERS {USB1:NonSecure;0|USB0:NonSecure;1|S_AXI_LPD:NA;0|S_AXI_HPC1_FPD:NA;1|S_AXI_HPC0_FPD:NA;1|S_AXI_HP3_FPD:NA;1|S_AXI_HP2_FPD:NA;1|S_AXI_HP1_FPD:NA;1|S_AXI_HP0_FPD:NA;1|S_AXI_ACP:NA;0|S_AXI_ACE:NA;0|SD1:NonSecure;1|SD0:NonSecure;1|SATA1:NonSecure;0|SATA0:NonSecure;0|RPU1:Secure;1|RPU0:Secure;1|QSPI:NonSecure;1|PMU:NA;1|PCIe:NonSecure;1|NAND:NonSecure;0|LDMA:NonSecure;1|GPU:NonSecure;1|GEM3:NonSecure;0|GEM2:NonSecure;0|GEM1:NonSecure;0|GEM0:NonSecure;0|FDMA:NonSecure;1|DP:NonSecure;0|DAP:NA;1|Coresight:NA;1|CSU:NA;1|APU:NA;1} \
   CONFIG.PSU__PROTECTION__MASTERS_TZ {GEM0:NonSecure|SD1:NonSecure|GEM2:NonSecure|GEM1:NonSecure|GEM3:NonSecure|PCIe:NonSecure|DP:NonSecure|NAND:NonSecure|GPU:NonSecure|USB1:NonSecure|USB0:NonSecure|LDMA:NonSecure|FDMA:NonSecure|QSPI:NonSecure|SD0:NonSecure} \
   CONFIG.PSU__PROTECTION__OCM_SEGMENTS {NONE} \
   CONFIG.PSU__PROTECTION__PRESUBSYSTEMS {NONE} \
   CONFIG.PSU__PROTECTION__SLAVES {LPD;USB3_1_XHCI;FE300000;FE3FFFFF;0|LPD;USB3_1;FF9E0000;FF9EFFFF;0|LPD;USB3_0_XHCI;FE200000;FE2FFFFF;1|LPD;USB3_0;FF9D0000;FF9DFFFF;1|LPD;UART1;FF010000;FF01FFFF;1|LPD;UART0;FF000000;FF00FFFF;1|LPD;TTC3;FF140000;FF14FFFF;1|LPD;TTC2;FF130000;FF13FFFF;1|LPD;TTC1;FF120000;FF12FFFF;1|LPD;TTC0;FF110000;FF11FFFF;1|FPD;SWDT1;FD4D0000;FD4DFFFF;1|LPD;SWDT0;FF150000;FF15FFFF;1|LPD;SPI1;FF050000;FF05FFFF;1|LPD;SPI0;FF040000;FF04FFFF;1|FPD;SMMU_REG;FD5F0000;FD5FFFFF;1|FPD;SMMU;FD800000;FDFFFFFF;1|FPD;SIOU;FD3D0000;FD3DFFFF;1|FPD;SERDES;FD400000;FD47FFFF;1|LPD;SD1;FF170000;FF17FFFF;1|LPD;SD0;FF160000;FF16FFFF;1|FPD;SATA;FD0C0000;FD0CFFFF;0|LPD;RTC;FFA60000;FFA6FFFF;1|LPD;RSA_CORE;FFCE0000;FFCEFFFF;1|LPD;RPU;FF9A0000;FF9AFFFF;1|LPD;R5_TCM_RAM_GLOBAL;FFE00000;FFE3FFFF;1|LPD;R5_1_Instruction_Cache;FFEC0000;FFECFFFF;1|LPD;R5_1_Data_Cache;FFED0000;FFEDFFFF;1|LPD;R5_1_BTCM_GLOBAL;FFEB0000;FFEBFFFF;1|LPD;R5_1_ATCM_GLOBAL;FFE90000;FFE9FFFF;1|LPD;R5_0_Instruction_Cache;FFE40000;FFE4FFFF;1|LPD;R5_0_Data_Cache;FFE50000;FFE5FFFF;1|LPD;R5_0_BTCM_GLOBAL;FFE20000;FFE2FFFF;1|LPD;R5_0_ATCM_GLOBAL;FFE00000;FFE0FFFF;1|LPD;QSPI_Linear_Address;C0000000;DFFFFFFF;1|LPD;QSPI;FF0F0000;FF0FFFFF;1|LPD;PMU_RAM;FFDC0000;FFDDFFFF;1|LPD;PMU_GLOBAL;FFD80000;FFDBFFFF;1|FPD;PCIE_MAIN;FD0E0000;FD0EFFFF;1|FPD;PCIE_LOW;E0000000;EFFFFFFF;1|FPD;PCIE_HIGH2;8000000000;BFFFFFFFFF;1|FPD;PCIE_HIGH1;600000000;7FFFFFFFF;1|FPD;PCIE_DMA;FD0F0000;FD0FFFFF;1|FPD;PCIE_ATTRIB;FD480000;FD48FFFF;1|LPD;OCM_XMPU_CFG;FFA70000;FFA7FFFF;1|LPD;OCM_SLCR;FF960000;FF96FFFF;1|OCM;OCM;FFFC0000;FFFFFFFF;1|LPD;NAND;FF100000;FF10FFFF;0|LPD;MBISTJTAG;FFCF0000;FFCFFFFF;1|LPD;LPD_XPPU_SINK;FF9C0000;FF9CFFFF;1|LPD;LPD_XPPU;FF980000;FF98FFFF;1|LPD;LPD_SLCR_SECURE;FF4B0000;FF4DFFFF;1|LPD;LPD_SLCR;FF410000;FF4AFFFF;1|LPD;LPD_GPV;FE100000;FE1FFFFF;1|LPD;LPD_DMA_7;FFAF0000;FFAFFFFF;1|LPD;LPD_DMA_6;FFAE0000;FFAEFFFF;1|LPD;LPD_DMA_5;FFAD0000;FFADFFFF;1|LPD;LPD_DMA_4;FFAC0000;FFACFFFF;1|LPD;LPD_DMA_3;FFAB0000;FFABFFFF;1|LPD;LPD_DMA_2;FFAA0000;FFAAFFFF;1|LPD;LPD_DMA_1;FFA90000;FFA9FFFF;1|LPD;LPD_DMA_0;FFA80000;FFA8FFFF;1|LPD;IPI_CTRL;FF380000;FF3FFFFF;1|LPD;IOU_SLCR;FF180000;FF23FFFF;1|LPD;IOU_SECURE_SLCR;FF240000;FF24FFFF;1|LPD;IOU_SCNTRS;FF260000;FF26FFFF;1|LPD;IOU_SCNTR;FF250000;FF25FFFF;1|LPD;IOU_GPV;FE000000;FE0FFFFF;1|LPD;I2C1;FF030000;FF03FFFF;1|LPD;I2C0;FF020000;FF02FFFF;1|FPD;GPU;FD4B0000;FD4BFFFF;1|LPD;GPIO;FF0A0000;FF0AFFFF;1|LPD;GEM3;FF0E0000;FF0EFFFF;0|LPD;GEM2;FF0D0000;FF0DFFFF;0|LPD;GEM1;FF0C0000;FF0CFFFF;0|LPD;GEM0;FF0B0000;FF0BFFFF;0|FPD;FPD_XMPU_SINK;FD4F0000;FD4FFFFF;1|FPD;FPD_XMPU_CFG;FD5D0000;FD5DFFFF;1|FPD;FPD_SLCR_SECURE;FD690000;FD6CFFFF;1|FPD;FPD_SLCR;FD610000;FD68FFFF;1|FPD;FPD_GPV;FD700000;FD7FFFFF;1|FPD;FPD_DMA_CH7;FD570000;FD57FFFF;1|FPD;FPD_DMA_CH6;FD560000;FD56FFFF;1|FPD;FPD_DMA_CH5;FD550000;FD55FFFF;1|FPD;FPD_DMA_CH4;FD540000;FD54FFFF;1|FPD;FPD_DMA_CH3;FD530000;FD53FFFF;1|FPD;FPD_DMA_CH2;FD520000;FD52FFFF;1|FPD;FPD_DMA_CH1;FD510000;FD51FFFF;1|FPD;FPD_DMA_CH0;FD500000;FD50FFFF;1|LPD;EFUSE;FFCC0000;FFCCFFFF;1|FPD;Display Port;FD4A0000;FD4AFFFF;0|FPD;DPDMA;FD4C0000;FD4CFFFF;0|FPD;DDR_XMPU5_CFG;FD050000;FD05FFFF;1|FPD;DDR_XMPU4_CFG;FD040000;FD04FFFF;1|FPD;DDR_XMPU3_CFG;FD030000;FD03FFFF;1|FPD;DDR_XMPU2_CFG;FD020000;FD02FFFF;1|FPD;DDR_XMPU1_CFG;FD010000;FD01FFFF;1|FPD;DDR_XMPU0_CFG;FD000000;FD00FFFF;1|FPD;DDR_QOS_CTRL;FD090000;FD09FFFF;1|FPD;DDR_PHY;FD080000;FD08FFFF;1|DDR;DDR_LOW;0;7FFFFFFF;1|DDR;DDR_HIGH;800000000;87FFFFFFF;1|FPD;DDDR_CTRL;FD070000;FD070FFF;1|LPD;Coresight;FE800000;FEFFFFFF;1|LPD;CSU_DMA;FFC80000;FFC9FFFF;1|LPD;CSU;FFCA0000;FFCAFFFF;0|LPD;CRL_APB;FF5E0000;FF85FFFF;1|FPD;CRF_APB;FD1A0000;FD2DFFFF;1|FPD;CCI_REG;FD5E0000;FD5EFFFF;1|FPD;CCI_GPV;FD6E0000;FD6EFFFF;1|LPD;CAN1;FF070000;FF07FFFF;1|LPD;CAN0;FF060000;FF06FFFF;1|FPD;APU;FD5C0000;FD5CFFFF;1|LPD;APM_INTC_IOU;FFA20000;FFA2FFFF;1|LPD;APM_FPD_LPD;FFA30000;FFA3FFFF;1|FPD;APM_5;FD490000;FD49FFFF;1|FPD;APM_0;FD0B0000;FD0BFFFF;1|LPD;APM2;FFA10000;FFA1FFFF;1|LPD;APM1;FFA00000;FFA0FFFF;1|LPD;AMS;FFA50000;FFA5FFFF;1|FPD;AFI_5;FD3B0000;FD3BFFFF;1|FPD;AFI_4;FD3A0000;FD3AFFFF;1|FPD;AFI_3;FD390000;FD39FFFF;1|FPD;AFI_2;FD380000;FD38FFFF;1|FPD;AFI_1;FD370000;FD37FFFF;1|FPD;AFI_0;FD360000;FD36FFFF;1|LPD;AFIFM6;FF9B0000;FF9BFFFF;1|FPD;ACPU_GIC;F9010000;F907FFFF;1} \
   CONFIG.PSU__PROTECTION__SUBSYSTEMS {PMU Firmware:PMU} \
   CONFIG.PSU__PSS_ALT_REF_CLK__ENABLE {1} \
   CONFIG.PSU__PSS_ALT_REF_CLK__FREQMHZ {27} \
   CONFIG.PSU__PSS_ALT_REF_CLK__IO {MIO 28} \
   CONFIG.PSU__PSS_REF_CLK__FREQMHZ {33.333} \
   CONFIG.PSU__QSPI_COHERENCY {0} \
   CONFIG.PSU__QSPI_ROUTE_THROUGH_FPD {0} \
   CONFIG.PSU__QSPI__GRP_FBCLK__ENABLE {1} \
   CONFIG.PSU__QSPI__GRP_FBCLK__IO {MIO 6} \
   CONFIG.PSU__QSPI__PERIPHERAL__DATA_MODE {x4} \
   CONFIG.PSU__QSPI__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__QSPI__PERIPHERAL__IO {MIO 0 .. 12} \
   CONFIG.PSU__QSPI__PERIPHERAL__MODE {Dual Parallel} \
   CONFIG.PSU__REPORT__DBGLOG {0} \
   CONFIG.PSU__RPU_COHERENCY {0} \
   CONFIG.PSU__RPU__POWER__ON {1} \
   CONFIG.PSU__SATA__LANE0__ENABLE {0} \
   CONFIG.PSU__SATA__LANE1__ENABLE {0} \
   CONFIG.PSU__SATA__LANE1__IO {<Select>} \
   CONFIG.PSU__SATA__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__SATA__REF_CLK_FREQ {<Select>} \
   CONFIG.PSU__SATA__REF_CLK_SEL {<Select>} \
   CONFIG.PSU__SAXIGP0__DATA_WIDTH {128} \
   CONFIG.PSU__SAXIGP1__DATA_WIDTH {128} \
   CONFIG.PSU__SAXIGP2__DATA_WIDTH {128} \
   CONFIG.PSU__SAXIGP3__DATA_WIDTH {128} \
   CONFIG.PSU__SAXIGP4__DATA_WIDTH {128} \
   CONFIG.PSU__SAXIGP5__DATA_WIDTH {128} \
   CONFIG.PSU__SAXIGP6__DATA_WIDTH {128} \
   CONFIG.PSU__SD0_COHERENCY {0} \
   CONFIG.PSU__SD0_ROUTE_THROUGH_FPD {0} \
   CONFIG.PSU__SD0__DATA_TRANSFER_MODE {8Bit} \
   CONFIG.PSU__SD0__GRP_CD__ENABLE {0} \
   CONFIG.PSU__SD0__GRP_POW__ENABLE {1} \
   CONFIG.PSU__SD0__GRP_POW__IO {MIO 23} \
   CONFIG.PSU__SD0__GRP_WP__ENABLE {0} \
   CONFIG.PSU__SD0__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__SD0__PERIPHERAL__IO {MIO 13 .. 22} \
   CONFIG.PSU__SD0__RESET__ENABLE {1} \
   CONFIG.PSU__SD0__SLOT_TYPE {eMMC} \
   CONFIG.PSU__SD1_COHERENCY {0} \
   CONFIG.PSU__SD1_ROUTE_THROUGH_FPD {0} \
   CONFIG.PSU__SD1__DATA_TRANSFER_MODE {4Bit} \
   CONFIG.PSU__SD1__GRP_CD__ENABLE {0} \
   CONFIG.PSU__SD1__GRP_CD__IO {<Select>} \
   CONFIG.PSU__SD1__GRP_POW__ENABLE {0} \
   CONFIG.PSU__SD1__GRP_POW__IO {<Select>} \
   CONFIG.PSU__SD1__GRP_WP__ENABLE {0} \
   CONFIG.PSU__SD1__GRP_WP__IO {<Select>} \
   CONFIG.PSU__SD1__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__SD1__PERIPHERAL__IO {EMIO} \
   CONFIG.PSU__SD1__RESET__ENABLE {0} \
   CONFIG.PSU__SD1__SLOT_TYPE {SD 2.0} \
   CONFIG.PSU__SPI0_LOOP_SPI1__ENABLE {0} \
   CONFIG.PSU__SPI0__GRP_SS0__ENABLE {1} \
   CONFIG.PSU__SPI0__GRP_SS0__IO {EMIO} \
   CONFIG.PSU__SPI0__GRP_SS1__ENABLE {1} \
   CONFIG.PSU__SPI0__GRP_SS1__IO {EMIO} \
   CONFIG.PSU__SPI0__GRP_SS2__ENABLE {0} \
   CONFIG.PSU__SPI0__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__SPI0__PERIPHERAL__IO {EMIO} \
   CONFIG.PSU__SPI1__GRP_SS0__ENABLE {1} \
   CONFIG.PSU__SPI1__GRP_SS0__IO {EMIO} \
   CONFIG.PSU__SPI1__GRP_SS1__ENABLE {1} \
   CONFIG.PSU__SPI1__GRP_SS1__IO {EMIO} \
   CONFIG.PSU__SPI1__GRP_SS2__ENABLE {0} \
   CONFIG.PSU__SPI1__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__SPI1__PERIPHERAL__IO {EMIO} \
   CONFIG.PSU__SWDT0__CLOCK__ENABLE {0} \
   CONFIG.PSU__SWDT0__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__SWDT0__PERIPHERAL__IO {NA} \
   CONFIG.PSU__SWDT0__RESET__ENABLE {0} \
   CONFIG.PSU__SWDT1__CLOCK__ENABLE {0} \
   CONFIG.PSU__SWDT1__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__SWDT1__PERIPHERAL__IO {NA} \
   CONFIG.PSU__SWDT1__RESET__ENABLE {0} \
   CONFIG.PSU__TCM0A__POWER__ON {1} \
   CONFIG.PSU__TCM0B__POWER__ON {1} \
   CONFIG.PSU__TCM1A__POWER__ON {1} \
   CONFIG.PSU__TCM1B__POWER__ON {1} \
   CONFIG.PSU__TESTSCAN__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__TRACE_PIPELINE_WIDTH {8} \
   CONFIG.PSU__TRACE__INTERNAL_WIDTH {32} \
   CONFIG.PSU__TRACE__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__TRISTATE__INVERTED {1} \
   CONFIG.PSU__TSU__BUFG_PORT_PAIR {0} \
   CONFIG.PSU__TTC0__CLOCK__ENABLE {0} \
   CONFIG.PSU__TTC0__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__TTC0__PERIPHERAL__IO {NA} \
   CONFIG.PSU__TTC0__WAVEOUT__ENABLE {0} \
   CONFIG.PSU__TTC1__CLOCK__ENABLE {0} \
   CONFIG.PSU__TTC1__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__TTC1__PERIPHERAL__IO {NA} \
   CONFIG.PSU__TTC1__WAVEOUT__ENABLE {0} \
   CONFIG.PSU__TTC2__CLOCK__ENABLE {0} \
   CONFIG.PSU__TTC2__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__TTC2__PERIPHERAL__IO {NA} \
   CONFIG.PSU__TTC2__WAVEOUT__ENABLE {0} \
   CONFIG.PSU__TTC3__CLOCK__ENABLE {0} \
   CONFIG.PSU__TTC3__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__TTC3__PERIPHERAL__IO {NA} \
   CONFIG.PSU__TTC3__WAVEOUT__ENABLE {0} \
   CONFIG.PSU__UART0_LOOP_UART1__ENABLE {0} \
   CONFIG.PSU__UART0__BAUD_RATE {115200} \
   CONFIG.PSU__UART0__MODEM__ENABLE {0} \
   CONFIG.PSU__UART0__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__UART0__PERIPHERAL__IO {EMIO} \
   CONFIG.PSU__UART1__BAUD_RATE {115200} \
   CONFIG.PSU__UART1__MODEM__ENABLE {0} \
   CONFIG.PSU__UART1__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__UART1__PERIPHERAL__IO {MIO 24 .. 25} \
   CONFIG.PSU__USB0_COHERENCY {0} \
   CONFIG.PSU__USB0__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__USB0__PERIPHERAL__IO {MIO 52 .. 63} \
   CONFIG.PSU__USB0__REF_CLK_FREQ {26} \
   CONFIG.PSU__USB0__REF_CLK_SEL {Ref Clk2} \
   CONFIG.PSU__USB0__RESET__ENABLE {1} \
   CONFIG.PSU__USB0__RESET__IO {MIO 65} \
   CONFIG.PSU__USB1_COHERENCY {0} \
   CONFIG.PSU__USB1__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__USB1__PERIPHERAL__IO {<Select>} \
   CONFIG.PSU__USB1__REF_CLK_FREQ {<Select>} \
   CONFIG.PSU__USB1__REF_CLK_SEL {<Select>} \
   CONFIG.PSU__USB1__RESET__ENABLE {0} \
   CONFIG.PSU__USB2_0__EMIO__ENABLE {0} \
   CONFIG.PSU__USB2_1__EMIO__ENABLE {0} \
   CONFIG.PSU__USB3_0__EMIO__ENABLE {0} \
   CONFIG.PSU__USB3_0__PERIPHERAL__ENABLE {1} \
   CONFIG.PSU__USB3_0__PERIPHERAL__IO {GT Lane2} \
   CONFIG.PSU__USB3_1__EMIO__ENABLE {0} \
   CONFIG.PSU__USB3_1__PERIPHERAL__ENABLE {0} \
   CONFIG.PSU__USB3_1__PERIPHERAL__IO {<Select>} \
   CONFIG.PSU__USB__RESET__MODE {Separate MIO Pin} \
   CONFIG.PSU__USB__RESET__POLARITY {Active Low} \
   CONFIG.PSU__USE_DIFF_RW_CLK_GP0 {0} \
   CONFIG.PSU__USE_DIFF_RW_CLK_GP1 {0} \
   CONFIG.PSU__USE_DIFF_RW_CLK_GP2 {0} \
   CONFIG.PSU__USE_DIFF_RW_CLK_GP3 {0} \
   CONFIG.PSU__USE_DIFF_RW_CLK_GP4 {0} \
   CONFIG.PSU__USE_DIFF_RW_CLK_GP5 {0} \
   CONFIG.PSU__USE_DIFF_RW_CLK_GP6 {0} \
   CONFIG.PSU__USE__ADMA {0} \
   CONFIG.PSU__USE__APU_LEGACY_INTERRUPT {0} \
   CONFIG.PSU__USE__AUDIO {0} \
   CONFIG.PSU__USE__CLK {0} \
   CONFIG.PSU__USE__CLK0 {0} \
   CONFIG.PSU__USE__CLK1 {0} \
   CONFIG.PSU__USE__CLK2 {0} \
   CONFIG.PSU__USE__CLK3 {0} \
   CONFIG.PSU__USE__CROSS_TRIGGER {0} \
   CONFIG.PSU__USE__DDR_INTF_REQUESTED {0} \
   CONFIG.PSU__USE__DEBUG__TEST {0} \
   CONFIG.PSU__USE__EVENT_RPU {0} \
   CONFIG.PSU__USE__FABRIC__RST {1} \
   CONFIG.PSU__USE__FTM {0} \
   CONFIG.PSU__USE__GDMA {0} \
   CONFIG.PSU__USE__IRQ {0} \
   CONFIG.PSU__USE__IRQ0 {1} \
   CONFIG.PSU__USE__IRQ1 {1} \
   CONFIG.PSU__USE__M_AXI_GP0 {1} \
   CONFIG.PSU__USE__M_AXI_GP1 {0} \
   CONFIG.PSU__USE__M_AXI_GP2 {0} \
   CONFIG.PSU__USE__PROC_EVENT_BUS {0} \
   CONFIG.PSU__USE__RPU_LEGACY_INTERRUPT {0} \
   CONFIG.PSU__USE__RST0 {0} \
   CONFIG.PSU__USE__RST1 {0} \
   CONFIG.PSU__USE__RST2 {0} \
   CONFIG.PSU__USE__RST3 {0} \
   CONFIG.PSU__USE__RTC {0} \
   CONFIG.PSU__USE__STM {0} \
   CONFIG.PSU__USE__S_AXI_ACE {0} \
   CONFIG.PSU__USE__S_AXI_ACP {0} \
   CONFIG.PSU__USE__S_AXI_GP0 {1} \
   CONFIG.PSU__USE__S_AXI_GP1 {1} \
   CONFIG.PSU__USE__S_AXI_GP2 {1} \
   CONFIG.PSU__USE__S_AXI_GP3 {1} \
   CONFIG.PSU__USE__S_AXI_GP4 {1} \
   CONFIG.PSU__USE__S_AXI_GP5 {1} \
   CONFIG.PSU__USE__S_AXI_GP6 {0} \
   CONFIG.PSU__USE__USB3_0_HUB {0} \
   CONFIG.PSU__USE__USB3_1_HUB {0} \
   CONFIG.PSU__USE__VIDEO {0} \
   CONFIG.PSU__VIDEO_REF_CLK__ENABLE {1} \
   CONFIG.PSU__VIDEO_REF_CLK__FREQMHZ {27} \
   CONFIG.PSU__VIDEO_REF_CLK__IO {MIO 27} \
   CONFIG.QSPI_BOARD_INTERFACE {custom} \
   CONFIG.SATA_BOARD_INTERFACE {custom} \
   CONFIG.SD0_BOARD_INTERFACE {custom} \
   CONFIG.SD1_BOARD_INTERFACE {custom} \
   CONFIG.SPI0_BOARD_INTERFACE {custom} \
   CONFIG.SPI1_BOARD_INTERFACE {custom} \
   CONFIG.SUBPRESET1 {Custom} \
   CONFIG.SUBPRESET2 {Custom} \
   CONFIG.SWDT0_BOARD_INTERFACE {custom} \
   CONFIG.SWDT1_BOARD_INTERFACE {custom} \
   CONFIG.TRACE_BOARD_INTERFACE {custom} \
   CONFIG.TTC0_BOARD_INTERFACE {custom} \
   CONFIG.TTC1_BOARD_INTERFACE {custom} \
   CONFIG.TTC2_BOARD_INTERFACE {custom} \
   CONFIG.TTC3_BOARD_INTERFACE {custom} \
   CONFIG.UART0_BOARD_INTERFACE {custom} \
   CONFIG.UART1_BOARD_INTERFACE {custom} \
   CONFIG.USB0_BOARD_INTERFACE {custom} \
   CONFIG.USB1_BOARD_INTERFACE {custom} \
 ] $zynq_ultra_ps_e_0

  # Create interface connections
  connect_bd_intf_net -intf_net Conn1 [get_bd_intf_pins UART_1] [get_bd_intf_pins zynq_ultra_ps_e_0/UART_0]
  connect_bd_intf_net -intf_net Conn2 [get_bd_intf_pins SDIO_WIFI] [get_bd_intf_pins zynq_ultra_ps_e_0/SDIO_1]
  connect_bd_intf_net -intf_net Conn3 [get_bd_intf_pins S_AXI_HPC0_FPD] [get_bd_intf_pins zynq_ultra_ps_e_0/S_AXI_HPC0_FPD]
  connect_bd_intf_net -intf_net Conn4 [get_bd_intf_pins S_AXI_HP2_FPD] [get_bd_intf_pins zynq_ultra_ps_e_0/S_AXI_HP2_FPD]
  connect_bd_intf_net -intf_net Conn5 [get_bd_intf_pins S_AXI_HP3_FPD] [get_bd_intf_pins zynq_ultra_ps_e_0/S_AXI_HP3_FPD]
  connect_bd_intf_net -intf_net Conn6 [get_bd_intf_pins S_AXI_HPC1_FPD] [get_bd_intf_pins zynq_ultra_ps_e_0/S_AXI_HPC1_FPD]
  connect_bd_intf_net -intf_net axi_smc_M00_AXI [get_bd_intf_pins S_AXI_HP0_FPD] [get_bd_intf_pins zynq_ultra_ps_e_0/S_AXI_HP0_FPD]
  connect_bd_intf_net -intf_net zynq_ultra_ps_e_0_M_AXI_HPM0_FPD [get_bd_intf_pins M_AXI_HPM0_FPD] [get_bd_intf_pins zynq_ultra_ps_e_0/M_AXI_HPM0_FPD]

  # Create port connections
  connect_bd_net -net BT_DEV_WAKE_1 [get_bd_pins BT_DEV_WAKE] [get_bd_pins emio_gpio_distr/BT_DEV_WAKE]
  connect_bd_net -net In0_1 [get_bd_pins pl_ps_irq0] [get_bd_pins irq_concat_0/In0]
  connect_bd_net -net In0_2 [get_bd_pins pl_ps_irq8] [get_bd_pins irq_concat_1/In0]
  connect_bd_net -net In10_1 [get_bd_pins pl_ps_irq13] [get_bd_pins irq_concat_1/In5]
  connect_bd_net -net In11_1 [get_bd_pins pl_ps_irq14] [get_bd_pins irq_concat_1/In6]
  connect_bd_net -net In12_1 [get_bd_pins pl_ps_irq15] [get_bd_pins irq_concat_1/In7]
  connect_bd_net -net In1_1 [get_bd_pins pl_ps_irq1] [get_bd_pins irq_concat_0/In1]
  connect_bd_net -net In1_2 [get_bd_pins pl_ps_irq9] [get_bd_pins irq_concat_1/In1]
  connect_bd_net -net In2_1 [get_bd_pins pl_ps_irq10] [get_bd_pins irq_concat_1/In2]
  connect_bd_net -net In2_2 [get_bd_pins pl_ps_irq2] [get_bd_pins irq_concat_0/In2]
  connect_bd_net -net In3_1 [get_bd_pins pl_ps_irq3] [get_bd_pins irq_concat_0/In3]
  connect_bd_net -net In4_1 [get_bd_pins pl_ps_irq4] [get_bd_pins irq_concat_0/In4]
  connect_bd_net -net In5_1 [get_bd_pins pl_ps_irq5] [get_bd_pins irq_concat_0/In5]
  connect_bd_net -net In6_1 [get_bd_pins pl_ps_irq6] [get_bd_pins irq_concat_0/In6]
  connect_bd_net -net In7_1 [get_bd_pins pl_ps_irq7] [get_bd_pins irq_concat_0/In7]
  connect_bd_net -net In8_1 [get_bd_pins pl_ps_irq11] [get_bd_pins irq_concat_1/In3]
  connect_bd_net -net In9_1 [get_bd_pins pl_ps_irq12] [get_bd_pins irq_concat_1/In4]
  connect_bd_net -net SENS_ACCEL_INT0_1 [get_bd_pins SENS_ACCEL_INT0] [get_bd_pins emio_gpio_distr/SENS_ACCEL_INT0]
  connect_bd_net -net SENS_ACCEL_INT1_1 [get_bd_pins SENS_ACCEL_INT1] [get_bd_pins emio_gpio_distr/SENS_ACCEL_INT1]
  connect_bd_net -net SENS_GPS_PIO11_1 [get_bd_pins SENS_GPS_PIO11] [get_bd_pins emio_gpio_distr/SENS_GPS_PIO11]
  connect_bd_net -net SENS_GPS_PIO13_EINT_1 [get_bd_pins SENS_GPS_PIO13_EINT] [get_bd_pins emio_gpio_distr/SENS_GPS_PIO13_EINT]
  connect_bd_net -net SENS_GPS_PIO14_1 [get_bd_pins SENS_GPS_PIO14] [get_bd_pins emio_gpio_distr/SENS_GPS_PIO14]
  connect_bd_net -net SENS_GPS_PIO15_1 [get_bd_pins SENS_GPS_PIO15] [get_bd_pins emio_gpio_distr/SENS_GPS_PIO15]
  connect_bd_net -net SENS_GYRO_INT0_1 [get_bd_pins SENS_GYRO_INT0] [get_bd_pins emio_gpio_distr/SENS_GYRO_INT0]
  connect_bd_net -net SENS_GYRO_INT1_1 [get_bd_pins SENS_GYRO_INT1] [get_bd_pins emio_gpio_distr/SENS_GYRO_INT1]
  connect_bd_net -net SENS_MAGN_DRDY_1 [get_bd_pins SENS_MAGN_DRDY] [get_bd_pins emio_gpio_distr/SENS_MAGN_DRDY]
  connect_bd_net -net SENS_MAGN_INT_1 [get_bd_pins SENS_MAGN_INT] [get_bd_pins emio_gpio_distr/SENS_MAGN_INT]
  connect_bd_net -net SENS_SPI0_MISO_1 [get_bd_pins SENS_SPI0_MISO] [get_bd_pins zynq_ultra_ps_e_0/emio_spi0_m_i]
  connect_bd_net -net SENS_SPI1_MISO_1 [get_bd_pins SENS_SPI1_MISO] [get_bd_pins zynq_ultra_ps_e_0/emio_spi1_m_i]
  connect_bd_net -net emio_gpio_distr_BT_EN [get_bd_pins BT_EN] [get_bd_pins emio_gpio_distr/BT_EN]
  connect_bd_net -net emio_gpio_distr_Dout [get_bd_pins aurora_raptor_reset] [get_bd_pins emio_gpio_distr/aurora_raptor_reset]
  connect_bd_net -net emio_gpio_distr_Dout1 [get_bd_pins aurora_xdp_reset] [get_bd_pins emio_gpio_distr/aurora_xdp_reset]
  connect_bd_net -net emio_gpio_distr_Dout2 [get_bd_pins hdmi_tx_ref_clk_rdy] [get_bd_pins emio_gpio_distr/hdmi_tx_refclk_rdy]
  connect_bd_net -net emio_gpio_distr_Dout3 [get_bd_pins mipi_1_frmbuf_rst_n] [get_bd_pins emio_gpio_distr/mipi_1_frmbuf_rst_n]
  connect_bd_net -net emio_gpio_distr_Dout4 [get_bd_pins mipi_0_frmbuf_rst_n] [get_bd_pins emio_gpio_distr/mipi_0_frmbuf_rst_n]
  connect_bd_net -net emio_gpio_distr_Dout5 [get_bd_pins mipi_1_demosaic_rst_n] [get_bd_pins emio_gpio_distr/mipi_1_demosaic_rst_n]
  connect_bd_net -net emio_gpio_distr_Dout6 [get_bd_pins mipi_0_demosaic_rst_n] [get_bd_pins emio_gpio_distr/mipi_0_demosaic_rst_n]
  connect_bd_net -net emio_gpio_distr_Dout7 [get_bd_pins Dout] [get_bd_pins emio_gpio_distr/Dout]
  connect_bd_net -net emio_gpio_distr_EIO_LED1 [get_bd_pins EIO_LED1] [get_bd_pins emio_gpio_distr/EIO_LED1]
  connect_bd_net -net emio_gpio_distr_EIO_LED2 [get_bd_pins EIO_LED2] [get_bd_pins emio_gpio_distr/EIO_LED2]
  connect_bd_net -net emio_gpio_distr_SDIO_WIFI_EN [get_bd_pins SDIO_WIFI_EN] [get_bd_pins emio_gpio_distr/SDIO_WIFI_EN]
  connect_bd_net -net emio_gpio_distr_SENS_GPS_DSEL [get_bd_pins SENS_GPS_DSEL] [get_bd_pins emio_gpio_distr/SENS_GPS_DSEL]
  connect_bd_net -net emio_gpio_distr_SENS_GPS_RESET_N [get_bd_pins SENS_GPS_RESET_N] [get_bd_pins emio_gpio_distr/SENS_GPS_RESET_N]
  connect_bd_net -net emio_gpio_distr_emio_gpio_i [get_bd_pins emio_gpio_distr/emio_gpio_i] [get_bd_pins zynq_ultra_ps_e_0/emio_gpio_i]
  connect_bd_net -net emio_gpio_distr_mipi_0_csc_rst_n [get_bd_pins mipi_0_csc_rst_n] [get_bd_pins emio_gpio_distr/mipi_0_csc_rst_n]
  connect_bd_net -net emio_gpio_distr_mipi_0_rst [get_bd_pins mipi_0_rst] [get_bd_pins emio_gpio_distr/mipi_0_rst]
  connect_bd_net -net emio_gpio_distr_mipi_1_csc_rst_n [get_bd_pins mipi_1_csc_rst_n] [get_bd_pins emio_gpio_distr/mipi_1_csc_rst_n]
  connect_bd_net -net emio_gpio_distr_mipi_1_rst [get_bd_pins mipi_1_rst] [get_bd_pins emio_gpio_distr/mipi_1_rst]
  connect_bd_net -net emio_gpio_distr_vcu_0_rst_n [get_bd_pins vcu_0_rst_n] [get_bd_pins emio_gpio_distr/vcu_0_rst_n]
  connect_bd_net -net rst_ps8_0_149M_peripheral_aresetn [get_bd_pins peripheral_aresetn_clk1] [get_bd_pins rst_ps8_0_149M/peripheral_aresetn]
  connect_bd_net -net rst_ps8_0_99M_interconnect_aresetn [get_bd_pins interconnect_aresetn] [get_bd_pins rst_ps8_0_99M/interconnect_aresetn]
  connect_bd_net -net rst_ps8_0_99M_peripheral_aresetn [get_bd_pins peripheral_aresetn_clk0] [get_bd_pins rst_ps8_0_99M/peripheral_aresetn]
  connect_bd_net -net saxihp0_fpd_aclk_1 [get_bd_pins saxihp0_fpd_aclk] [get_bd_pins zynq_ultra_ps_e_0/saxihp0_fpd_aclk]
  connect_bd_net -net saxihpc0_fpd_aclk_1 [get_bd_pins saxihpc0_fpd_aclk] [get_bd_pins zynq_ultra_ps_e_0/saxihp1_fpd_aclk] [get_bd_pins zynq_ultra_ps_e_0/saxihp2_fpd_aclk] [get_bd_pins zynq_ultra_ps_e_0/saxihp3_fpd_aclk] [get_bd_pins zynq_ultra_ps_e_0/saxihpc0_fpd_aclk] [get_bd_pins zynq_ultra_ps_e_0/saxihpc1_fpd_aclk]
  connect_bd_net -net switch_pl_0_1 [get_bd_pins switch_pl_0] [get_bd_pins emio_gpio_distr/switch_pl_0]
  connect_bd_net -net xdp_version_1 [get_bd_pins xdp_version] [get_bd_pins emio_gpio_distr/xdp_version]
  connect_bd_net -net xlconcat_0_dout [get_bd_pins irq_concat_0/dout] [get_bd_pins zynq_ultra_ps_e_0/pl_ps_irq0]
  connect_bd_net -net xlconcat_1_dout [get_bd_pins irq_concat_1/dout] [get_bd_pins zynq_ultra_ps_e_0/pl_ps_irq1]
  connect_bd_net -net xlconstant_axcache_dout [get_bd_pins xlconstant_axcache/dout] [get_bd_pins zynq_ultra_ps_e_0/saxigp1_arcache] [get_bd_pins zynq_ultra_ps_e_0/saxigp1_awcache]
  connect_bd_net -net xlconstant_axprot_dout [get_bd_pins xlconstant_axprot/dout] [get_bd_pins zynq_ultra_ps_e_0/saxigp1_arprot] [get_bd_pins zynq_ultra_ps_e_0/saxigp1_awprot]
  connect_bd_net -net zynq_ultra_ps_e_0_emio_gpio_o1 [get_bd_pins emio_gpio_distr/emio_gpio_o] [get_bd_pins zynq_ultra_ps_e_0/emio_gpio_o]
  connect_bd_net -net zynq_ultra_ps_e_0_emio_spi0_m_o [get_bd_pins SENS_SPI0_MOSI] [get_bd_pins zynq_ultra_ps_e_0/emio_spi0_m_o]
  connect_bd_net -net zynq_ultra_ps_e_0_emio_spi0_sclk_o [get_bd_pins SENS_SPI0_SCK] [get_bd_pins zynq_ultra_ps_e_0/emio_spi0_sclk_o]
  connect_bd_net -net zynq_ultra_ps_e_0_emio_spi0_ss1_o_n [get_bd_pins SENS_SPI0_CS_ACCEL] [get_bd_pins zynq_ultra_ps_e_0/emio_spi0_ss1_o_n]
  connect_bd_net -net zynq_ultra_ps_e_0_emio_spi0_ss_o_n [get_bd_pins SENS_SPI0_CS_GYRO] [get_bd_pins zynq_ultra_ps_e_0/emio_spi0_ss_o_n]
  connect_bd_net -net zynq_ultra_ps_e_0_emio_spi1_m_o [get_bd_pins SENS_SPI1_MOSI] [get_bd_pins zynq_ultra_ps_e_0/emio_spi1_m_o]
  connect_bd_net -net zynq_ultra_ps_e_0_emio_spi1_sclk_o [get_bd_pins SENS_SPI1_SCK] [get_bd_pins zynq_ultra_ps_e_0/emio_spi1_sclk_o]
  connect_bd_net -net zynq_ultra_ps_e_0_emio_spi1_ss1_o_n [get_bd_pins SENS_SPI1_CS_MAGN] [get_bd_pins zynq_ultra_ps_e_0/emio_spi1_ss1_o_n]
  connect_bd_net -net zynq_ultra_ps_e_0_emio_spi1_ss_o_n [get_bd_pins SENS_SPI1_CS_ENV] [get_bd_pins zynq_ultra_ps_e_0/emio_spi1_ss_o_n]
  connect_bd_net -net zynq_ultra_ps_e_0_pl_clk0 [get_bd_pins pl_clk0] [get_bd_pins rst_ps8_0_99M/slowest_sync_clk] [get_bd_pins zynq_ultra_ps_e_0/maxihpm0_fpd_aclk] [get_bd_pins zynq_ultra_ps_e_0/pl_clk0]
  connect_bd_net -net zynq_ultra_ps_e_0_pl_clk1 [get_bd_pins pl_clk1] [get_bd_pins rst_ps8_0_149M/slowest_sync_clk] [get_bd_pins zynq_ultra_ps_e_0/pl_clk1]
  connect_bd_net -net zynq_ultra_ps_e_0_pl_resetn0 [get_bd_pins pl_resetn0] [get_bd_pins rst_ps8_0_149M/ext_reset_in] [get_bd_pins rst_ps8_0_99M/ext_reset_in] [get_bd_pins zynq_ultra_ps_e_0/pl_resetn0]

  # Restore current instance
  current_bd_instance $oldCurInst
}

# Hierarchical cell: mipi_camera_1
proc create_hier_cell_mipi_camera_1 { parentCell nameHier } {

  variable script_folder

  if { $parentCell eq "" || $nameHier eq "" } {
     catch {common::send_msg_id "BD_TCL-102" "ERROR" "create_hier_cell_mipi_camera_1() - Empty argument(s)!"}
     return
  }

  # Get object for parentCell
  set parentObj [get_bd_cells $parentCell]
  if { $parentObj == "" } {
     catch {common::send_msg_id "BD_TCL-100" "ERROR" "Unable to find parent cell <$parentCell>!"}
     return
  }

  # Make sure parentObj is hier blk
  set parentType [get_property TYPE $parentObj]
  if { $parentType ne "hier" } {
     catch {common::send_msg_id "BD_TCL-101" "ERROR" "Parent <$parentObj> has TYPE = <$parentType>. Expected to be <hier>."}
     return
  }

  # Save current instance; Restore later
  set oldCurInst [current_bd_instance .]

  # Set parent object as current
  current_bd_instance $parentObj

  # Create cell and set as current instance
  set hier_obj [create_bd_cell -type hier $nameHier]
  current_bd_instance $hier_obj

  # Create interface pins
  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:iic_rtl:1.0 MIPI_IIC

  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:axis_rtl:1.0 M_AXIS_CAM

  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:aximm_rtl:1.0 S_AXI

  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:mipi_phy_rtl:1.0 mipi_csi


  # Create pins
  create_bd_pin -dir I ARESETN
  create_bd_pin -dir I clkoutphy_in
  create_bd_pin -dir I csc_rst_in
  create_bd_pin -dir O -type intr csirxss_csi_irq_0
  create_bd_pin -dir I demosaic_rst_n
  create_bd_pin -dir I -type clk dphy_clk_200M
  create_bd_pin -dir O iic2intc_irpt
  create_bd_pin -dir O -type intr interrupt_demosaic
  create_bd_pin -dir I -type clk lite_aclk
  create_bd_pin -dir I lite_aresetn
  create_bd_pin -dir I pll_lock_in
  create_bd_pin -dir I video_aclk
  create_bd_pin -dir I video_aresetn

  # Create instance: axi_iic_0, and set properties
  set axi_iic_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_iic axi_iic_0 ]
  set_property -dict [ list \
   CONFIG.C_SCL_INERTIAL_DELAY {5} \
   CONFIG.C_SDA_INERTIAL_DELAY {5} \
 ] $axi_iic_0

  # Create instance: axi_interconnect_0, and set properties
  set axi_interconnect_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_interconnect axi_interconnect_0 ]
  set_property -dict [ list \
   CONFIG.NUM_MI {4} \
 ] $axi_interconnect_0

  # Create instance: axis_subset_converter_0, and set properties
  set axis_subset_converter_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axis_subset_converter axis_subset_converter_0 ]
  set_property -dict [ list \
   CONFIG.M_HAS_TLAST {1} \
   CONFIG.M_TDATA_NUM_BYTES {2} \
   CONFIG.M_TDEST_WIDTH {10} \
   CONFIG.M_TUSER_WIDTH {1} \
   CONFIG.S_HAS_TLAST {1} \
   CONFIG.S_TDATA_NUM_BYTES {3} \
   CONFIG.S_TDEST_WIDTH {10} \
   CONFIG.S_TUSER_WIDTH {1} \
   CONFIG.TDATA_REMAP {tdata[19:12],tdata[9:2]} \
   CONFIG.TDEST_REMAP {tdest[9:0]} \
   CONFIG.TLAST_REMAP {tlast[0]} \
   CONFIG.TUSER_REMAP {tuser[0:0]} \
 ] $axis_subset_converter_0

  # Create instance: axis_subset_converter_1, and set properties
  set axis_subset_converter_1 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axis_subset_converter axis_subset_converter_1 ]
  set_property -dict [ list \
   CONFIG.M_HAS_TLAST {1} \
   CONFIG.M_TDATA_NUM_BYTES {8} \
   CONFIG.M_TUSER_WIDTH {1} \
   CONFIG.S_HAS_TLAST {1} \
   CONFIG.S_TDATA_NUM_BYTES {6} \
   CONFIG.S_TUSER_WIDTH {1} \
   CONFIG.TDATA_REMAP {8'b0,tdata[47:40],tdata[31:24],tdata[39:32],8'b0,tdata[23:16],tdata[7:0],tdata[15:8]} \
   CONFIG.TLAST_REMAP {tlast[0]} \
   CONFIG.TUSER_REMAP {tuser[0:0]} \
 ] $axis_subset_converter_1

  # Create instance: axis_video_frame_dro_0, and set properties
  set block_name axis_video_frame_dropper
  set block_cell_name axis_video_frame_dro_0
  if { [catch {set axis_video_frame_dro_0 [create_bd_cell -type module -reference $block_name $block_cell_name] } errmsg] } {
     catch {common::send_msg_id "BD_TCL-105" "ERROR" "Unable to add referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   } elseif { $axis_video_frame_dro_0 eq "" } {
     catch {common::send_msg_id "BD_TCL-106" "ERROR" "Unable to referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   }
    set_property -dict [ list \
   CONFIG.p_axis_tdata_width {64} \
 ] $axis_video_frame_dro_0

  # Create instance: mipi_csi2_rx_subsystem_0, and set properties
  set mipi_csi2_rx_subsystem_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:mipi_csi2_rx_subsystem mipi_csi2_rx_subsystem_0 ]
  set_property -dict [ list \
   CONFIG.CLK_LANE_IO_LOC {AD2} \
   CONFIG.CLK_LANE_IO_LOC_NAME {IO_L19P_T3L_N0_DBC_AD9P_66} \
   CONFIG.CMN_NUM_LANES {4} \
   CONFIG.CMN_NUM_PIXELS {2} \
   CONFIG.CMN_PXL_FORMAT {RAW10} \
   CONFIG.CSI_BUF_DEPTH {4096} \
   CONFIG.C_CLK_LANE_IO_POSITION {39} \
   CONFIG.C_CSI_EN_ACTIVELANES {true} \
   CONFIG.C_CSI_FILTER_USERDATATYPE {true} \
   CONFIG.C_DATA_LANE0_IO_POSITION {41} \
   CONFIG.C_DATA_LANE1_IO_POSITION {43} \
   CONFIG.C_DATA_LANE2_IO_POSITION {45} \
   CONFIG.C_DATA_LANE3_IO_POSITION {47} \
   CONFIG.C_DPHY_LANES {4} \
   CONFIG.C_EN_TIMEOUT_REGS {false} \
   CONFIG.C_HS_LINE_RATE {1440} \
   CONFIG.C_HS_SETTLE_NS {141} \
   CONFIG.DATA_LANE0_IO_LOC {AD1} \
   CONFIG.DATA_LANE0_IO_LOC_NAME {IO_L20P_T3L_N2_AD1P_66} \
   CONFIG.DATA_LANE1_IO_LOC {AC3} \
   CONFIG.DATA_LANE1_IO_LOC_NAME {IO_L21P_T3L_N4_AD8P_66} \
   CONFIG.DATA_LANE2_IO_LOC {AA3} \
   CONFIG.DATA_LANE2_IO_LOC_NAME {IO_L22P_T3U_N6_DBC_AD0P_66} \
   CONFIG.DATA_LANE3_IO_LOC {AB1} \
   CONFIG.DATA_LANE3_IO_LOC_NAME {IO_L23P_T3U_N8_66} \
   CONFIG.DPY_EN_REG_IF {false} \
   CONFIG.DPY_LINE_RATE {1440} \
   CONFIG.HP_IO_BANK_SELECTION {66} \
   CONFIG.SupportLevel {0} \
 ] $mipi_csi2_rx_subsystem_0

  # Create instance: v_demosaic_0, and set properties
  set v_demosaic_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:v_demosaic v_demosaic_0 ]
  set_property -dict [ list \
   CONFIG.ENABLE_ZIPPER_REMOVAL {false} \
   CONFIG.SAMPLES_PER_CLOCK {2} \
   CONFIG.USE_URAM {0} \
 ] $v_demosaic_0

  # Create instance: v_proc_ss_csc, and set properties
  set v_proc_ss_csc [ create_bd_cell -type ip -vlnv xilinx.com:ip:v_proc_ss v_proc_ss_csc ]
  set_property -dict [ list \
   CONFIG.C_COLORSPACE_SUPPORT {0} \
   CONFIG.C_CSC_ENABLE_WINDOW {false} \
   CONFIG.C_DEINT_MOTION_ADAPTIVE {false} \
   CONFIG.C_ENABLE_CSC {true} \
   CONFIG.C_MAX_DATA_WIDTH {8} \
   CONFIG.C_SCALER_ALGORITHM {2} \
   CONFIG.C_TOPOLOGY {3} \
   CONFIG.C_USE_URAM {1} \
 ] $v_proc_ss_csc

  # Create instance: video_tuser_cnt_0, and set properties
  set block_name axis_video_add_tuser_frame_count
  set block_cell_name video_tuser_cnt_0
  if { [catch {set video_tuser_cnt_0 [create_bd_cell -type module -reference $block_name $block_cell_name] } errmsg] } {
     catch {common::send_msg_id "BD_TCL-105" "ERROR" "Unable to add referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   } elseif { $video_tuser_cnt_0 eq "" } {
     catch {common::send_msg_id "BD_TCL-106" "ERROR" "Unable to referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   }
    set_property -dict [ list \
   CONFIG.p_axis_added_tuser_bits {4} \
   CONFIG.p_axis_tdata_width {64} \
 ] $video_tuser_cnt_0

  # Create interface connections
  connect_bd_intf_net -intf_net Conn1 [get_bd_intf_pins M_AXIS_CAM] [get_bd_intf_pins video_tuser_cnt_0/M_AXIS]
  connect_bd_intf_net -intf_net S_AXI1_1 [get_bd_intf_pins S_AXI] [get_bd_intf_pins axi_interconnect_0/S00_AXI]
  connect_bd_intf_net -intf_net axi_iic_0_IIC [get_bd_intf_pins MIPI_IIC] [get_bd_intf_pins axi_iic_0/IIC]
  connect_bd_intf_net -intf_net axi_interconnect_0_M00_AXI [get_bd_intf_pins axi_interconnect_0/M00_AXI] [get_bd_intf_pins mipi_csi2_rx_subsystem_0/csirxss_s_axi]
  connect_bd_intf_net -intf_net axi_interconnect_0_M01_AXI [get_bd_intf_pins axi_iic_0/S_AXI] [get_bd_intf_pins axi_interconnect_0/M01_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M02_AXI [get_bd_intf_pins axi_interconnect_0/M02_AXI] [get_bd_intf_pins v_demosaic_0/s_axi_CTRL]
  connect_bd_intf_net -intf_net axi_interconnect_0_M03_AXI [get_bd_intf_pins axi_interconnect_0/M03_AXI] [get_bd_intf_pins v_proc_ss_csc/s_axi_ctrl]
  connect_bd_intf_net -intf_net axis_subset_converter_0_M_AXIS [get_bd_intf_pins axis_subset_converter_0/M_AXIS] [get_bd_intf_pins v_demosaic_0/s_axis_video]
  connect_bd_intf_net -intf_net axis_subset_converter_1_M_AXIS [get_bd_intf_pins axis_subset_converter_1/M_AXIS] [get_bd_intf_pins axis_video_frame_dro_0/S_AXIS]
  connect_bd_intf_net -intf_net axis_video_frame_dro_0_M_AXIS [get_bd_intf_pins axis_video_frame_dro_0/M_AXIS] [get_bd_intf_pins video_tuser_cnt_0/S_AXIS]
  connect_bd_intf_net -intf_net mipi_csi2_rx_subsystem_0_video_out [get_bd_intf_pins axis_subset_converter_0/S_AXIS] [get_bd_intf_pins mipi_csi2_rx_subsystem_0/video_out]
  connect_bd_intf_net -intf_net mipi_csi_1 [get_bd_intf_pins mipi_csi] [get_bd_intf_pins mipi_csi2_rx_subsystem_0/mipi_phy_if]
  connect_bd_intf_net -intf_net v_demosaic_0_m_axis_video [get_bd_intf_pins v_demosaic_0/m_axis_video] [get_bd_intf_pins v_proc_ss_csc/s_axis]
  connect_bd_intf_net -intf_net v_proc_ss_csc_m_axis [get_bd_intf_pins axis_subset_converter_1/S_AXIS] [get_bd_intf_pins v_proc_ss_csc/m_axis]

  # Create port connections
  connect_bd_net -net ARESETN_1 [get_bd_pins ARESETN] [get_bd_pins axi_interconnect_0/ARESETN]
  connect_bd_net -net axi_iic_0_iic2intc_irpt [get_bd_pins iic2intc_irpt] [get_bd_pins axi_iic_0/iic2intc_irpt]
  connect_bd_net -net clkoutphy_in_1 [get_bd_pins clkoutphy_in] [get_bd_pins mipi_csi2_rx_subsystem_0/clkoutphy_in]
  connect_bd_net -net csc_rst_in_1 [get_bd_pins csc_rst_in] [get_bd_pins v_proc_ss_csc/aresetn]
  connect_bd_net -net demosaic_rst_n_1 [get_bd_pins demosaic_rst_n] [get_bd_pins v_demosaic_0/ap_rst_n]
  connect_bd_net -net dphy_clk_200M_0_1 [get_bd_pins dphy_clk_200M] [get_bd_pins mipi_csi2_rx_subsystem_0/dphy_clk_200M]
  connect_bd_net -net lite_aclk_1 [get_bd_pins lite_aclk] [get_bd_pins axi_iic_0/s_axi_aclk] [get_bd_pins axi_interconnect_0/ACLK] [get_bd_pins axi_interconnect_0/M00_ACLK] [get_bd_pins axi_interconnect_0/M01_ACLK] [get_bd_pins axi_interconnect_0/S00_ACLK] [get_bd_pins mipi_csi2_rx_subsystem_0/lite_aclk]
  connect_bd_net -net lite_aresetn_1 [get_bd_pins lite_aresetn] [get_bd_pins axi_iic_0/s_axi_aresetn] [get_bd_pins axi_interconnect_0/M00_ARESETN] [get_bd_pins axi_interconnect_0/M01_ARESETN] [get_bd_pins axi_interconnect_0/S00_ARESETN] [get_bd_pins mipi_csi2_rx_subsystem_0/lite_aresetn]
  connect_bd_net -net mipi_csi2_rx_subsystem_0_csirxss_csi_irq [get_bd_pins csirxss_csi_irq_0] [get_bd_pins mipi_csi2_rx_subsystem_0/csirxss_csi_irq]
  connect_bd_net -net pll_lock_in_1 [get_bd_pins pll_lock_in] [get_bd_pins mipi_csi2_rx_subsystem_0/pll_lock_in]
  connect_bd_net -net v_demosaic_0_interrupt [get_bd_pins interrupt_demosaic] [get_bd_pins v_demosaic_0/interrupt]
  connect_bd_net -net video_aclk_1 [get_bd_pins video_aclk] [get_bd_pins axi_interconnect_0/M02_ACLK] [get_bd_pins axi_interconnect_0/M03_ACLK] [get_bd_pins axis_subset_converter_0/aclk] [get_bd_pins axis_subset_converter_1/aclk] [get_bd_pins axis_video_frame_dro_0/AXIS_ACLK] [get_bd_pins mipi_csi2_rx_subsystem_0/video_aclk] [get_bd_pins v_demosaic_0/ap_clk] [get_bd_pins v_proc_ss_csc/aclk] [get_bd_pins video_tuser_cnt_0/AXIS_ACLK]
  connect_bd_net -net video_aresetn_1 [get_bd_pins video_aresetn] [get_bd_pins axi_interconnect_0/M02_ARESETN] [get_bd_pins axi_interconnect_0/M03_ARESETN] [get_bd_pins axis_subset_converter_0/aresetn] [get_bd_pins axis_subset_converter_1/aresetn] [get_bd_pins mipi_csi2_rx_subsystem_0/video_aresetn]

  # Restore current instance
  current_bd_instance $oldCurInst
}

# Hierarchical cell: mipi_camera_0
proc create_hier_cell_mipi_camera_0 { parentCell nameHier } {

  variable script_folder

  if { $parentCell eq "" || $nameHier eq "" } {
     catch {common::send_msg_id "BD_TCL-102" "ERROR" "create_hier_cell_mipi_camera_0() - Empty argument(s)!"}
     return
  }

  # Get object for parentCell
  set parentObj [get_bd_cells $parentCell]
  if { $parentObj == "" } {
     catch {common::send_msg_id "BD_TCL-100" "ERROR" "Unable to find parent cell <$parentCell>!"}
     return
  }

  # Make sure parentObj is hier blk
  set parentType [get_property TYPE $parentObj]
  if { $parentType ne "hier" } {
     catch {common::send_msg_id "BD_TCL-101" "ERROR" "Parent <$parentObj> has TYPE = <$parentType>. Expected to be <hier>."}
     return
  }

  # Save current instance; Restore later
  set oldCurInst [current_bd_instance .]

  # Set parent object as current
  current_bd_instance $parentObj

  # Create cell and set as current instance
  set hier_obj [create_bd_cell -type hier $nameHier]
  current_bd_instance $hier_obj

  # Create interface pins
  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:iic_rtl:1.0 MIPI_IIC

  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:aximm_rtl:1.0 S_AXI

  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:aximm_rtl:1.0 m_axi_mm_video_0

  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:mipi_phy_rtl:1.0 mipi_csi


  # Create pins
  create_bd_pin -dir I ARESETN
  create_bd_pin -dir O -type clk clkoutphy_out_0
  create_bd_pin -dir I csc_rst_in
  create_bd_pin -dir O -type intr csirxss_csi_irq_0
  create_bd_pin -dir I demosaic_rst_n
  create_bd_pin -dir I -type clk dphy_clk_200M
  create_bd_pin -dir O fb_write_irq
  create_bd_pin -dir I frmbuf_rst_n
  create_bd_pin -dir O iic2intc_irpt
  create_bd_pin -dir O -type intr interrupt_demosaic
  create_bd_pin -dir I -type clk lite_aclk
  create_bd_pin -dir I lite_aresetn
  create_bd_pin -dir O pll_lock_out_0
  create_bd_pin -dir I video_aclk
  create_bd_pin -dir I video_aresetn

  # Create instance: axi_iic_0, and set properties
  set axi_iic_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_iic axi_iic_0 ]
  set_property -dict [ list \
   CONFIG.C_SCL_INERTIAL_DELAY {5} \
   CONFIG.C_SDA_INERTIAL_DELAY {5} \
 ] $axi_iic_0

  # Create instance: axi_interconnect_0, and set properties
  set axi_interconnect_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_interconnect axi_interconnect_0 ]
  set_property -dict [ list \
   CONFIG.NUM_MI {5} \
 ] $axi_interconnect_0

  # Create instance: axis_subset_converter_0, and set properties
  set axis_subset_converter_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axis_subset_converter axis_subset_converter_0 ]
  set_property -dict [ list \
   CONFIG.M_HAS_TLAST {1} \
   CONFIG.M_TDATA_NUM_BYTES {2} \
   CONFIG.M_TDEST_WIDTH {10} \
   CONFIG.M_TUSER_WIDTH {1} \
   CONFIG.S_HAS_TLAST {1} \
   CONFIG.S_TDATA_NUM_BYTES {3} \
   CONFIG.S_TDEST_WIDTH {10} \
   CONFIG.S_TUSER_WIDTH {1} \
   CONFIG.TDATA_REMAP {tdata[19:12],tdata[9:2]} \
   CONFIG.TDEST_REMAP {tdest[9:0]} \
   CONFIG.TLAST_REMAP {tlast[0]} \
   CONFIG.TUSER_REMAP {tuser[0:0]} \
 ] $axis_subset_converter_0

  # Create instance: mipi_csi2_rx_subsystem_0, and set properties
  set mipi_csi2_rx_subsystem_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:mipi_csi2_rx_subsystem mipi_csi2_rx_subsystem_0 ]
  set_property -dict [ list \
   CONFIG.CLK_LANE_IO_LOC {AC12} \
   CONFIG.CLK_LANE_IO_LOC_NAME {IO_L1P_T0L_N0_DBC_66} \
   CONFIG.CMN_NUM_LANES {4} \
   CONFIG.CMN_NUM_PIXELS {2} \
   CONFIG.CMN_PXL_FORMAT {RAW10} \
   CONFIG.CSI_BUF_DEPTH {4096} \
   CONFIG.C_CLK_LANE_IO_POSITION {0} \
   CONFIG.C_CSI_EN_ACTIVELANES {true} \
   CONFIG.C_CSI_FILTER_USERDATATYPE {true} \
   CONFIG.C_DATA_LANE0_IO_POSITION {2} \
   CONFIG.C_DATA_LANE1_IO_POSITION {4} \
   CONFIG.C_DATA_LANE2_IO_POSITION {6} \
   CONFIG.C_DATA_LANE3_IO_POSITION {8} \
   CONFIG.C_DPHY_LANES {4} \
   CONFIG.C_EN_TIMEOUT_REGS {false} \
   CONFIG.C_HS_LINE_RATE {1440} \
   CONFIG.C_HS_SETTLE_NS {141} \
   CONFIG.DATA_LANE0_IO_LOC {AA12} \
   CONFIG.DATA_LANE0_IO_LOC_NAME {IO_L2P_T0L_N2_66} \
   CONFIG.DATA_LANE1_IO_LOC {AB11} \
   CONFIG.DATA_LANE1_IO_LOC_NAME {IO_L3P_T0L_N4_AD15P_66} \
   CONFIG.DATA_LANE2_IO_LOC {AD11} \
   CONFIG.DATA_LANE2_IO_LOC_NAME {IO_L4P_T0U_N6_DBC_AD7P_66} \
   CONFIG.DATA_LANE3_IO_LOC {AB10} \
   CONFIG.DATA_LANE3_IO_LOC_NAME {IO_L5P_T0U_N8_AD14P_66} \
   CONFIG.DPY_EN_REG_IF {false} \
   CONFIG.DPY_LINE_RATE {1440} \
   CONFIG.HP_IO_BANK_SELECTION {66} \
   CONFIG.SupportLevel {1} \
 ] $mipi_csi2_rx_subsystem_0

  # Create instance: v_demosaic_0, and set properties
  set v_demosaic_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:v_demosaic v_demosaic_0 ]
  set_property -dict [ list \
   CONFIG.ENABLE_ZIPPER_REMOVAL {false} \
   CONFIG.SAMPLES_PER_CLOCK {2} \
   CONFIG.USE_URAM {0} \
 ] $v_demosaic_0

  # Create instance: v_frmbuf_wr_0, and set properties
  set v_frmbuf_wr_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:v_frmbuf_wr v_frmbuf_wr_0 ]
  set_property -dict [ list \
   CONFIG.HAS_BGR8 {1} \
   CONFIG.HAS_BGRX8 {0} \
   CONFIG.HAS_RGBX8 {0} \
   CONFIG.HAS_UYVY8 {1} \
   CONFIG.HAS_Y8 {1} \
   CONFIG.HAS_YUV8 {1} \
   CONFIG.HAS_YUVX8 {1} \
   CONFIG.HAS_YUYV8 {1} \
   CONFIG.HAS_Y_UV8 {1} \
   CONFIG.HAS_Y_UV8_420 {1} \
   CONFIG.MAX_NR_PLANES {2} \
 ] $v_frmbuf_wr_0

  # Create instance: v_proc_ss_csc, and set properties
  set v_proc_ss_csc [ create_bd_cell -type ip -vlnv xilinx.com:ip:v_proc_ss v_proc_ss_csc ]
  set_property -dict [ list \
   CONFIG.C_COLORSPACE_SUPPORT {0} \
   CONFIG.C_CSC_ENABLE_WINDOW {false} \
   CONFIG.C_DEINT_MOTION_ADAPTIVE {false} \
   CONFIG.C_ENABLE_CSC {true} \
   CONFIG.C_MAX_DATA_WIDTH {8} \
   CONFIG.C_SCALER_ALGORITHM {2} \
   CONFIG.C_TOPOLOGY {3} \
   CONFIG.C_USE_URAM {1} \
 ] $v_proc_ss_csc

  # Create interface connections
  connect_bd_intf_net -intf_net S_AXI1_1 [get_bd_intf_pins S_AXI] [get_bd_intf_pins axi_interconnect_0/S00_AXI]
  connect_bd_intf_net -intf_net axi_iic_0_IIC [get_bd_intf_pins MIPI_IIC] [get_bd_intf_pins axi_iic_0/IIC]
  connect_bd_intf_net -intf_net axi_interconnect_0_M00_AXI [get_bd_intf_pins axi_interconnect_0/M00_AXI] [get_bd_intf_pins mipi_csi2_rx_subsystem_0/csirxss_s_axi]
  connect_bd_intf_net -intf_net axi_interconnect_0_M01_AXI [get_bd_intf_pins axi_iic_0/S_AXI] [get_bd_intf_pins axi_interconnect_0/M01_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M02_AXI [get_bd_intf_pins axi_interconnect_0/M02_AXI] [get_bd_intf_pins v_demosaic_0/s_axi_CTRL]
  connect_bd_intf_net -intf_net axi_interconnect_0_M03_AXI [get_bd_intf_pins axi_interconnect_0/M03_AXI] [get_bd_intf_pins v_proc_ss_csc/s_axi_ctrl]
  connect_bd_intf_net -intf_net axi_interconnect_0_M04_AXI [get_bd_intf_pins axi_interconnect_0/M04_AXI] [get_bd_intf_pins v_frmbuf_wr_0/s_axi_CTRL]
  connect_bd_intf_net -intf_net axis_subset_converter_0_M_AXIS [get_bd_intf_pins axis_subset_converter_0/M_AXIS] [get_bd_intf_pins v_demosaic_0/s_axis_video]
  connect_bd_intf_net -intf_net mipi_csi2_rx_subsystem_0_video_out [get_bd_intf_pins axis_subset_converter_0/S_AXIS] [get_bd_intf_pins mipi_csi2_rx_subsystem_0/video_out]
  connect_bd_intf_net -intf_net mipi_csi_1 [get_bd_intf_pins mipi_csi] [get_bd_intf_pins mipi_csi2_rx_subsystem_0/mipi_phy_if]
  connect_bd_intf_net -intf_net v_demosaic_0_m_axis_video [get_bd_intf_pins v_demosaic_0/m_axis_video] [get_bd_intf_pins v_proc_ss_csc/s_axis]
  connect_bd_intf_net -intf_net v_frmbuf_wr_0_m_axi_mm_video [get_bd_intf_pins m_axi_mm_video_0] [get_bd_intf_pins v_frmbuf_wr_0/m_axi_mm_video]
  connect_bd_intf_net -intf_net v_proc_ss_csc_m_axis [get_bd_intf_pins v_frmbuf_wr_0/s_axis_video] [get_bd_intf_pins v_proc_ss_csc/m_axis]

  # Create port connections
  connect_bd_net -net ARESETN_1 [get_bd_pins ARESETN] [get_bd_pins axi_interconnect_0/ARESETN]
  connect_bd_net -net axi_iic_0_iic2intc_irpt [get_bd_pins iic2intc_irpt] [get_bd_pins axi_iic_0/iic2intc_irpt]
  connect_bd_net -net csc_rst_in_1 [get_bd_pins csc_rst_in] [get_bd_pins v_proc_ss_csc/aresetn]
  connect_bd_net -net demosaic_rst_n_1 [get_bd_pins demosaic_rst_n] [get_bd_pins v_demosaic_0/ap_rst_n]
  connect_bd_net -net dphy_clk_200M_0_1 [get_bd_pins dphy_clk_200M] [get_bd_pins mipi_csi2_rx_subsystem_0/dphy_clk_200M]
  connect_bd_net -net frmbuf_rst_n_1 [get_bd_pins frmbuf_rst_n] [get_bd_pins v_frmbuf_wr_0/ap_rst_n]
  connect_bd_net -net lite_aclk_1 [get_bd_pins lite_aclk] [get_bd_pins axi_iic_0/s_axi_aclk] [get_bd_pins axi_interconnect_0/ACLK] [get_bd_pins axi_interconnect_0/M00_ACLK] [get_bd_pins axi_interconnect_0/M01_ACLK] [get_bd_pins axi_interconnect_0/S00_ACLK] [get_bd_pins mipi_csi2_rx_subsystem_0/lite_aclk]
  connect_bd_net -net lite_aresetn_1 [get_bd_pins lite_aresetn] [get_bd_pins axi_iic_0/s_axi_aresetn] [get_bd_pins axi_interconnect_0/M00_ARESETN] [get_bd_pins axi_interconnect_0/M01_ARESETN] [get_bd_pins axi_interconnect_0/S00_ARESETN] [get_bd_pins mipi_csi2_rx_subsystem_0/lite_aresetn]
  connect_bd_net -net mipi_csi2_rx_subsystem_0_clkoutphy_out [get_bd_pins clkoutphy_out_0] [get_bd_pins mipi_csi2_rx_subsystem_0/clkoutphy_out]
  connect_bd_net -net mipi_csi2_rx_subsystem_0_csirxss_csi_irq [get_bd_pins csirxss_csi_irq_0] [get_bd_pins mipi_csi2_rx_subsystem_0/csirxss_csi_irq]
  connect_bd_net -net mipi_csi2_rx_subsystem_0_pll_lock_out [get_bd_pins pll_lock_out_0] [get_bd_pins mipi_csi2_rx_subsystem_0/pll_lock_out]
  connect_bd_net -net v_demosaic_0_interrupt [get_bd_pins interrupt_demosaic] [get_bd_pins v_demosaic_0/interrupt]
  connect_bd_net -net v_frmbuf_wr_0_interrupt [get_bd_pins fb_write_irq] [get_bd_pins v_frmbuf_wr_0/interrupt]
  connect_bd_net -net video_aclk_1 [get_bd_pins video_aclk] [get_bd_pins axi_interconnect_0/M02_ACLK] [get_bd_pins axi_interconnect_0/M03_ACLK] [get_bd_pins axi_interconnect_0/M04_ACLK] [get_bd_pins axis_subset_converter_0/aclk] [get_bd_pins mipi_csi2_rx_subsystem_0/video_aclk] [get_bd_pins v_demosaic_0/ap_clk] [get_bd_pins v_frmbuf_wr_0/ap_clk] [get_bd_pins v_proc_ss_csc/aclk]
  connect_bd_net -net video_aresetn_1 [get_bd_pins video_aresetn] [get_bd_pins axi_interconnect_0/M02_ARESETN] [get_bd_pins axi_interconnect_0/M03_ARESETN] [get_bd_pins axi_interconnect_0/M04_ARESETN] [get_bd_pins axis_subset_converter_0/aresetn] [get_bd_pins mipi_csi2_rx_subsystem_0/video_aresetn]

  # Restore current instance
  current_bd_instance $oldCurInst
}

# Hierarchical cell: hdmi_out
proc create_hier_cell_hdmi_out { parentCell nameHier } {

  variable script_folder

  if { $parentCell eq "" || $nameHier eq "" } {
     catch {common::send_msg_id "BD_TCL-102" "ERROR" "create_hier_cell_hdmi_out() - Empty argument(s)!"}
     return
  }

  # Get object for parentCell
  set parentObj [get_bd_cells $parentCell]
  if { $parentObj == "" } {
     catch {common::send_msg_id "BD_TCL-100" "ERROR" "Unable to find parent cell <$parentCell>!"}
     return
  }

  # Make sure parentObj is hier blk
  set parentType [get_property TYPE $parentObj]
  if { $parentType ne "hier" } {
     catch {common::send_msg_id "BD_TCL-101" "ERROR" "Parent <$parentObj> has TYPE = <$parentType>. Expected to be <hier>."}
     return
  }

  # Save current instance; Restore later
  set oldCurInst [current_bd_instance .]

  # Set parent object as current
  current_bd_instance $parentObj

  # Create cell and set as current instance
  set hier_obj [create_bd_cell -type hier $nameHier]
  current_bd_instance $hier_obj

  # Create interface pins
  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:iic_rtl:1.0 HDMI_DDC

  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:aximm_rtl:1.0 M_AXI_MM2S

  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:aximm_rtl:1.0 S_AXI


  # Create pins
  create_bd_pin -dir I ARESETN
  create_bd_pin -dir I GND
  create_bd_pin -dir I HDMI_HPD
  create_bd_pin -dir O -from 3 -to 0 HDMI_TX_DAT_N_OUT
  create_bd_pin -dir O -from 3 -to 0 HDMI_TX_DAT_P_OUT
  create_bd_pin -dir I -from 0 -to 0 ap_rst_n
  create_bd_pin -dir I axi_resetn
  create_bd_pin -dir O frmbuf_rd_irq
  create_bd_pin -dir O hdmitx_irq
  create_bd_pin -dir I m_axi_mm2s_aclk
  create_bd_pin -dir I -type clk mgtrefclk0_pad_n_in_0
  create_bd_pin -dir I -type clk mgtrefclk0_pad_p_in_0
  create_bd_pin -dir I s_axi_lite_aclk
  create_bd_pin -dir I -from 0 -to 0 tx_refclk_rdy
  create_bd_pin -dir O vhy_irq
  create_bd_pin -dir I vid_phy_tx_axi4s_aresetn

  # Create instance: axi_interconnect_0, and set properties
  set axi_interconnect_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_interconnect axi_interconnect_0 ]
  set_property -dict [ list \
   CONFIG.NUM_MI {3} \
 ] $axi_interconnect_0

  # Create instance: remove_alpha_channel, and set properties
  set remove_alpha_channel [ create_bd_cell -type ip -vlnv xilinx.com:ip:axis_subset_converter remove_alpha_channel ]
  set_property -dict [ list \
   CONFIG.M_HAS_TKEEP {1} \
   CONFIG.M_HAS_TLAST {1} \
   CONFIG.M_TDATA_NUM_BYTES {6} \
   CONFIG.M_TUSER_WIDTH {1} \
   CONFIG.S_HAS_TKEEP {1} \
   CONFIG.S_HAS_TLAST {1} \
   CONFIG.S_TDATA_NUM_BYTES {8} \
   CONFIG.S_TUSER_WIDTH {1} \
   CONFIG.TDATA_REMAP {tdata[55:32],tdata[23:0]} \
   CONFIG.TKEEP_REMAP {tkeep[5:0]} \
   CONFIG.TLAST_REMAP {tlast[0]} \
   CONFIG.TUSER_REMAP {tuser[0:0]} \
 ] $remove_alpha_channel

  # Create instance: v_frmbuf_rd_0, and set properties
  set v_frmbuf_rd_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:v_frmbuf_rd v_frmbuf_rd_0 ]
  set_property -dict [ list \
   CONFIG.HAS_ALPHA {1} \
   CONFIG.HAS_BGR8 {0} \
   CONFIG.HAS_BGRA8 {1} \
   CONFIG.HAS_BGRX8 {0} \
   CONFIG.HAS_RGB8 {0} \
   CONFIG.HAS_RGBA8 {1} \
   CONFIG.HAS_RGBX8 {0} \
   CONFIG.NUM_VIDEO_COMPONENTS {4} \
 ] $v_frmbuf_rd_0

  # Create instance: v_hdmi_tx_ss_0, and set properties
  set v_hdmi_tx_ss_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:v_hdmi_tx_ss v_hdmi_tx_ss_0 ]

  # Create instance: vid_phy_controller_0, and set properties
  set vid_phy_controller_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:vid_phy_controller vid_phy_controller_0 ]
  set_property -dict [ list \
   CONFIG.CHANNEL_ENABLE {X0Y8 X0Y9 X0Y10 X0Y11} \
   CONFIG.CHANNEL_SITE {X0Y8} \
   CONFIG.C_INPUT_PIXELS_PER_CLOCK {2} \
   CONFIG.C_NIDRU {false} \
   CONFIG.C_RX_REFCLK_SEL {1} \
   CONFIG.C_Rx_No_Of_Channels {3} \
   CONFIG.C_Rx_Protocol {None} \
   CONFIG.C_TX_REFCLK_SEL {0} \
   CONFIG.C_Tx_No_Of_Channels {4} \
   CONFIG.C_Tx_Protocol {HDMI} \
   CONFIG.C_Use_GT_CH4_HDMI {true} \
   CONFIG.C_Use_Oddr_for_Tmds_Clkout {true} \
   CONFIG.C_vid_phy_rx_axi4s_ch_INT_TDATA_WIDTH {20} \
   CONFIG.C_vid_phy_rx_axi4s_ch_TDATA_WIDTH {20} \
   CONFIG.C_vid_phy_tx_axi4s_ch_INT_TDATA_WIDTH {20} \
   CONFIG.C_vid_phy_tx_axi4s_ch_TDATA_WIDTH {20} \
   CONFIG.Rx_GT_Line_Rate {5.94} \
   CONFIG.Rx_Max_GT_Line_Rate {5.94} \
   CONFIG.Transceiver_Width {2} \
   CONFIG.Tx_GT_Line_Rate {5.94} \
   CONFIG.Tx_GT_Ref_Clock_Freq {297} \
   CONFIG.Tx_Max_GT_Line_Rate {5.94} \
 ] $vid_phy_controller_0

  # Create interface connections
  connect_bd_intf_net -intf_net Conn1 [get_bd_intf_pins HDMI_DDC] [get_bd_intf_pins v_hdmi_tx_ss_0/DDC_OUT]
  connect_bd_intf_net -intf_net S_AXI_1 [get_bd_intf_pins S_AXI] [get_bd_intf_pins axi_interconnect_0/S00_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M00_AXI [get_bd_intf_pins axi_interconnect_0/M00_AXI] [get_bd_intf_pins vid_phy_controller_0/vid_phy_axi4lite]
  connect_bd_intf_net -intf_net axi_interconnect_0_M01_AXI [get_bd_intf_pins axi_interconnect_0/M01_AXI] [get_bd_intf_pins v_hdmi_tx_ss_0/S_AXI_CPU_IN]
  connect_bd_intf_net -intf_net axi_interconnect_0_M02_AXI [get_bd_intf_pins axi_interconnect_0/M02_AXI] [get_bd_intf_pins v_frmbuf_rd_0/s_axi_CTRL]
  connect_bd_intf_net -intf_net rgb_2_rbg_M_AXIS [get_bd_intf_pins remove_alpha_channel/M_AXIS] [get_bd_intf_pins v_hdmi_tx_ss_0/VIDEO_IN]
  connect_bd_intf_net -intf_net v_frmbuf_rd_0_m_axi_mm_video [get_bd_intf_pins M_AXI_MM2S] [get_bd_intf_pins v_frmbuf_rd_0/m_axi_mm_video]
  connect_bd_intf_net -intf_net v_frmbuf_rd_0_m_axis_video [get_bd_intf_pins remove_alpha_channel/S_AXIS] [get_bd_intf_pins v_frmbuf_rd_0/m_axis_video]
  connect_bd_intf_net -intf_net v_hdmi_tx_ss_0_LINK_DATA0_OUT [get_bd_intf_pins v_hdmi_tx_ss_0/LINK_DATA0_OUT] [get_bd_intf_pins vid_phy_controller_0/vid_phy_tx_axi4s_ch0]
  connect_bd_intf_net -intf_net v_hdmi_tx_ss_0_LINK_DATA1_OUT [get_bd_intf_pins v_hdmi_tx_ss_0/LINK_DATA1_OUT] [get_bd_intf_pins vid_phy_controller_0/vid_phy_tx_axi4s_ch1]
  connect_bd_intf_net -intf_net v_hdmi_tx_ss_0_LINK_DATA2_OUT [get_bd_intf_pins v_hdmi_tx_ss_0/LINK_DATA2_OUT] [get_bd_intf_pins vid_phy_controller_0/vid_phy_tx_axi4s_ch2]
  connect_bd_intf_net -intf_net vid_phy_controller_0_vid_phy_status_sb_tx [get_bd_intf_pins v_hdmi_tx_ss_0/SB_STATUS_IN] [get_bd_intf_pins vid_phy_controller_0/vid_phy_status_sb_tx]

  # Create port connections
  connect_bd_net -net ARESETN_1 [get_bd_pins ARESETN] [get_bd_pins axi_interconnect_0/ARESETN]
  connect_bd_net -net GND_1 [get_bd_pins GND] [get_bd_pins v_hdmi_tx_ss_0/AUDIO_IN_tvalid] [get_bd_pins v_hdmi_tx_ss_0/acr_valid] [get_bd_pins v_hdmi_tx_ss_0/fid] [get_bd_pins v_hdmi_tx_ss_0/s_axis_audio_aclk] [get_bd_pins v_hdmi_tx_ss_0/s_axis_audio_aresetn]
  connect_bd_net -net HDMI_HPD_1 [get_bd_pins HDMI_HPD] [get_bd_pins v_hdmi_tx_ss_0/hpd]
  connect_bd_net -net ap_rst_n_1 [get_bd_pins ap_rst_n] [get_bd_pins v_frmbuf_rd_0/ap_rst_n]
  connect_bd_net -net axi_resetn_1 [get_bd_pins axi_resetn] [get_bd_pins axi_interconnect_0/M00_ARESETN] [get_bd_pins axi_interconnect_0/M01_ARESETN] [get_bd_pins axi_interconnect_0/S00_ARESETN] [get_bd_pins v_hdmi_tx_ss_0/s_axi_cpu_aresetn] [get_bd_pins vid_phy_controller_0/vid_phy_axi4lite_aresetn] [get_bd_pins vid_phy_controller_0/vid_phy_sb_aresetn]
  connect_bd_net -net m_axi_mm2s_aclk_1 [get_bd_pins m_axi_mm2s_aclk] [get_bd_pins axi_interconnect_0/M02_ACLK] [get_bd_pins remove_alpha_channel/aclk] [get_bd_pins v_frmbuf_rd_0/ap_clk] [get_bd_pins v_hdmi_tx_ss_0/s_axis_video_aclk]
  connect_bd_net -net mgtrefclk0_pad_n_in_0_1 [get_bd_pins mgtrefclk0_pad_n_in_0] [get_bd_pins vid_phy_controller_0/mgtrefclk0_pad_n_in]
  connect_bd_net -net mgtrefclk0_pad_p_in_0_1 [get_bd_pins mgtrefclk0_pad_p_in_0] [get_bd_pins vid_phy_controller_0/mgtrefclk0_pad_p_in]
  connect_bd_net -net s_axi_lite_aclk_1 [get_bd_pins s_axi_lite_aclk] [get_bd_pins axi_interconnect_0/ACLK] [get_bd_pins axi_interconnect_0/M00_ACLK] [get_bd_pins axi_interconnect_0/M01_ACLK] [get_bd_pins axi_interconnect_0/S00_ACLK] [get_bd_pins v_hdmi_tx_ss_0/s_axi_cpu_aclk] [get_bd_pins vid_phy_controller_0/drpclk] [get_bd_pins vid_phy_controller_0/vid_phy_axi4lite_aclk] [get_bd_pins vid_phy_controller_0/vid_phy_sb_aclk]
  connect_bd_net -net tx_refclk_rdy_1 [get_bd_pins tx_refclk_rdy] [get_bd_pins vid_phy_controller_0/tx_refclk_rdy]
  connect_bd_net -net v_frmbuf_rd_0_interrupt [get_bd_pins frmbuf_rd_irq] [get_bd_pins v_frmbuf_rd_0/interrupt]
  connect_bd_net -net v_hdmi_tx_ss_0_irq [get_bd_pins hdmitx_irq] [get_bd_pins v_hdmi_tx_ss_0/irq]
  connect_bd_net -net vid_phy_controller_0_irq [get_bd_pins vhy_irq] [get_bd_pins vid_phy_controller_0/irq]
  connect_bd_net -net vid_phy_controller_0_phy_txn_out [get_bd_pins HDMI_TX_DAT_N_OUT] [get_bd_pins vid_phy_controller_0/phy_txn_out]
  connect_bd_net -net vid_phy_controller_0_phy_txp_out [get_bd_pins HDMI_TX_DAT_P_OUT] [get_bd_pins vid_phy_controller_0/phy_txp_out]
  connect_bd_net -net vid_phy_controller_0_tx_video_clk [get_bd_pins v_hdmi_tx_ss_0/video_clk] [get_bd_pins vid_phy_controller_0/tx_video_clk]
  connect_bd_net -net vid_phy_controller_0_txoutclk [get_bd_pins v_hdmi_tx_ss_0/link_clk] [get_bd_pins vid_phy_controller_0/txoutclk] [get_bd_pins vid_phy_controller_0/vid_phy_tx_axi4s_aclk]
  connect_bd_net -net vid_phy_tx_axi4s_aresetn_1 [get_bd_pins vid_phy_tx_axi4s_aresetn] [get_bd_pins axi_interconnect_0/M02_ARESETN] [get_bd_pins remove_alpha_channel/aresetn] [get_bd_pins v_hdmi_tx_ss_0/s_axis_video_aresetn] [get_bd_pins vid_phy_controller_0/vid_phy_tx_axi4s_aresetn]

  # Restore current instance
  current_bd_instance $oldCurInst
}

# Hierarchical cell: aurora_xdp
proc create_hier_cell_aurora_xdp { parentCell nameHier } {

  variable script_folder

  if { $parentCell eq "" || $nameHier eq "" } {
     catch {common::send_msg_id "BD_TCL-102" "ERROR" "create_hier_cell_aurora_xdp() - Empty argument(s)!"}
     return
  }

  # Get object for parentCell
  set parentObj [get_bd_cells $parentCell]
  if { $parentObj == "" } {
     catch {common::send_msg_id "BD_TCL-100" "ERROR" "Unable to find parent cell <$parentCell>!"}
     return
  }

  # Make sure parentObj is hier blk
  set parentType [get_property TYPE $parentObj]
  if { $parentType ne "hier" } {
     catch {common::send_msg_id "BD_TCL-101" "ERROR" "Parent <$parentObj> has TYPE = <$parentType>. Expected to be <hier>."}
     return
  }

  # Save current instance; Restore later
  set oldCurInst [current_bd_instance .]

  # Set parent object as current
  current_bd_instance $parentObj

  # Create cell and set as current instance
  set hier_obj [create_bd_cell -type hier $nameHier]
  current_bd_instance $hier_obj

  # Create interface pins
  create_bd_intf_pin -mode Slave -vlnv xilinx.com:display_aurora:GT_Serial_Transceiver_Pins_RX_rtl:1.0 AURORA_FROM_XDP

  create_bd_intf_pin -mode Master -vlnv xilinx.com:display_aurora:GT_Serial_Transceiver_Pins_TX_rtl:1.0 AURORA_TO_XDP

  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:diff_clock_rtl:1.0 Q0_REFCLK0


  # Create pins
  create_bd_pin -dir I GND
  create_bd_pin -dir I -type clk init_clk
  create_bd_pin -dir O -from 31 -to 0 m_axis_tdata
  create_bd_pin -dir I -from 0 -to 0 -type rst reset

  # Create instance: VCC, and set properties
  set VCC [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant VCC ]

  # Create instance: aurora_64b66b_0, and set properties
  set aurora_64b66b_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:aurora_64b66b aurora_64b66b_0 ]
  set_property -dict [ list \
   CONFIG.CHANNEL_ENABLE {X0Y4 X0Y5 X0Y6 X0Y7} \
   CONFIG.C_AURORA_LANES {4} \
   CONFIG.C_GT_LOC_2 {2} \
   CONFIG.C_GT_LOC_3 {3} \
   CONFIG.C_GT_LOC_4 {4} \
   CONFIG.C_LINE_RATE {10} \
   CONFIG.C_REFCLK_FREQUENCY {100} \
   CONFIG.SupportLevel {1} \
   CONFIG.drp_mode {Disabled} \
   CONFIG.interface_mode {Streaming} \
 ] $aurora_64b66b_0

  # Create instance: aurora_controller_0, and set properties
  set block_name aurora_controller
  set block_cell_name aurora_controller_0
  if { [catch {set aurora_controller_0 [create_bd_cell -type module -reference $block_name $block_cell_name] } errmsg] } {
     catch {common::send_msg_id "BD_TCL-105" "ERROR" "Unable to add referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   } elseif { $aurora_controller_0 eq "" } {
     catch {common::send_msg_id "BD_TCL-106" "ERROR" "Unable to referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   }
  
  # Create instance: axis_data_counter_0, and set properties
  set block_name axis_data_counter
  set block_cell_name axis_data_counter_0
  if { [catch {set axis_data_counter_0 [create_bd_cell -type module -reference $block_name $block_cell_name] } errmsg] } {
     catch {common::send_msg_id "BD_TCL-105" "ERROR" "Unable to add referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   } elseif { $axis_data_counter_0 eq "" } {
     catch {common::send_msg_id "BD_TCL-106" "ERROR" "Unable to referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   }
    set_property -dict [ list \
   CONFIG.p_axis_tdata_width {256} \
 ] $axis_data_counter_0

  # Create instance: axis_data_fifo_0, and set properties
  set axis_data_fifo_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axis_data_fifo axis_data_fifo_0 ]
  set_property -dict [ list \
   CONFIG.FIFO_DEPTH {16} \
   CONFIG.HAS_TKEEP {0} \
   CONFIG.HAS_TLAST {0} \
   CONFIG.HAS_TSTRB {0} \
   CONFIG.IS_ACLK_ASYNC {1} \
   CONFIG.TDATA_NUM_BYTES {4} \
   CONFIG.TDEST_WIDTH {0} \
   CONFIG.TID_WIDTH {0} \
   CONFIG.TUSER_WIDTH {0} \
 ] $axis_data_fifo_0

  # Create interface connections
  connect_bd_intf_net -intf_net GT_DIFF_REFCLK1_0_1 [get_bd_intf_pins Q0_REFCLK0] [get_bd_intf_pins aurora_64b66b_0/GT_DIFF_REFCLK1]
  connect_bd_intf_net -intf_net GT_SERIAL_RX_0_1 [get_bd_intf_pins AURORA_FROM_XDP] [get_bd_intf_pins aurora_64b66b_0/GT_SERIAL_RX]
  connect_bd_intf_net -intf_net aurora_64b66b_0_GT_SERIAL_TX [get_bd_intf_pins AURORA_TO_XDP] [get_bd_intf_pins aurora_64b66b_0/GT_SERIAL_TX]
  connect_bd_intf_net -intf_net aurora_64b66b_0_USER_DATA_M_AXIS_RX [get_bd_intf_pins aurora_64b66b_0/USER_DATA_M_AXIS_RX] [get_bd_intf_pins axis_data_counter_0/S_AXIS]
  connect_bd_intf_net -intf_net axis_data_counter_0_M_AXIS [get_bd_intf_pins aurora_64b66b_0/USER_DATA_S_AXIS_TX] [get_bd_intf_pins axis_data_counter_0/M_AXIS]

  # Create port connections
  connect_bd_net -net GND_dout [get_bd_pins GND] [get_bd_pins aurora_64b66b_0/gt_rxcdrovrden_in] [get_bd_pins aurora_64b66b_0/power_down]
  connect_bd_net -net VCC_dout [get_bd_pins VCC/dout] [get_bd_pins aurora_64b66b_0/s_axi_tx_tvalid] [get_bd_pins axis_data_counter_0/M_AXIS_TREADY] [get_bd_pins axis_data_fifo_0/m_axis_tready] [get_bd_pins axis_data_fifo_0/s_axis_aresetn] [get_bd_pins axis_data_fifo_0/s_axis_tvalid]
  connect_bd_net -net aurora_64b66b_0_user_clk_out [get_bd_pins aurora_64b66b_0/user_clk_out] [get_bd_pins aurora_controller_0/user_clk] [get_bd_pins axis_data_counter_0/AXIS_ACLK] [get_bd_pins axis_data_fifo_0/s_axis_aclk]
  connect_bd_net -net aurora_controller_0_gt_reset_out [get_bd_pins aurora_64b66b_0/pma_init] [get_bd_pins aurora_controller_0/gt_reset_out]
  connect_bd_net -net aurora_controller_0_system_reset_out [get_bd_pins aurora_64b66b_0/reset_pb] [get_bd_pins aurora_controller_0/system_reset_out]
  connect_bd_net -net axis_data_counter_0_o_count [get_bd_pins axis_data_counter_0/o_count] [get_bd_pins axis_data_fifo_0/s_axis_tdata]
  connect_bd_net -net axis_data_fifo_0_m_axis_tdata [get_bd_pins m_axis_tdata] [get_bd_pins axis_data_fifo_0/m_axis_tdata]
  connect_bd_net -net reset_1 [get_bd_pins reset] [get_bd_pins aurora_controller_0/reset_in]
  connect_bd_net -net zynq_ultra_ps_e_0_pl_clk0 [get_bd_pins init_clk] [get_bd_pins aurora_64b66b_0/init_clk] [get_bd_pins aurora_controller_0/init_clk] [get_bd_pins axis_data_fifo_0/m_axis_aclk]

  # Restore current instance
  current_bd_instance $oldCurInst
}

# Hierarchical cell: aurora_raptor
proc create_hier_cell_aurora_raptor { parentCell nameHier } {

  variable script_folder

  if { $parentCell eq "" || $nameHier eq "" } {
     catch {common::send_msg_id "BD_TCL-102" "ERROR" "create_hier_cell_aurora_raptor() - Empty argument(s)!"}
     return
  }

  # Get object for parentCell
  set parentObj [get_bd_cells $parentCell]
  if { $parentObj == "" } {
     catch {common::send_msg_id "BD_TCL-100" "ERROR" "Unable to find parent cell <$parentCell>!"}
     return
  }

  # Make sure parentObj is hier blk
  set parentType [get_property TYPE $parentObj]
  if { $parentType ne "hier" } {
     catch {common::send_msg_id "BD_TCL-101" "ERROR" "Parent <$parentObj> has TYPE = <$parentType>. Expected to be <hier>."}
     return
  }

  # Save current instance; Restore later
  set oldCurInst [current_bd_instance .]

  # Set parent object as current
  current_bd_instance $parentObj

  # Create cell and set as current instance
  set hier_obj [create_bd_cell -type hier $nameHier]
  current_bd_instance $hier_obj

  # Create interface pins
  create_bd_intf_pin -mode Slave -vlnv xilinx.com:display_aurora:GT_Serial_Transceiver_Pins_RX_rtl:1.0 AURORA_FROM_RAPTOR

  create_bd_intf_pin -mode Master -vlnv xilinx.com:display_aurora:GT_Serial_Transceiver_Pins_TX_rtl:1.0 AURORA_TO_RAPTOR

  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:diff_clock_rtl:1.0 Q2_REFCLK0


  # Create pins
  create_bd_pin -dir I GND
  create_bd_pin -dir I -type clk init_clk
  create_bd_pin -dir O -from 31 -to 0 m_axis_tdata
  create_bd_pin -dir I -from 0 -to 0 -type rst reset

  # Create instance: VCC, and set properties
  set VCC [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant VCC ]

  # Create instance: aurora_64b66b_0, and set properties
  set aurora_64b66b_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:aurora_64b66b aurora_64b66b_0 ]
  set_property -dict [ list \
   CONFIG.CHANNEL_ENABLE {X0Y12 X0Y13 X0Y14 X0Y15} \
   CONFIG.C_AURORA_LANES {4} \
   CONFIG.C_GT_LOC_2 {2} \
   CONFIG.C_GT_LOC_3 {3} \
   CONFIG.C_GT_LOC_4 {4} \
   CONFIG.C_LINE_RATE {10} \
   CONFIG.C_REFCLK_FREQUENCY {100} \
   CONFIG.C_REFCLK_SOURCE {MGTREFCLK0_of_Quad_X0Y3} \
   CONFIG.C_START_LANE {X0Y12} \
   CONFIG.C_START_QUAD {Quad_X0Y3} \
   CONFIG.SupportLevel {1} \
   CONFIG.drp_mode {Disabled} \
   CONFIG.interface_mode {Streaming} \
 ] $aurora_64b66b_0

  # Create instance: aurora_controller_0, and set properties
  set block_name aurora_controller
  set block_cell_name aurora_controller_0
  if { [catch {set aurora_controller_0 [create_bd_cell -type module -reference $block_name $block_cell_name] } errmsg] } {
     catch {common::send_msg_id "BD_TCL-105" "ERROR" "Unable to add referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   } elseif { $aurora_controller_0 eq "" } {
     catch {common::send_msg_id "BD_TCL-106" "ERROR" "Unable to referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   }
  
  # Create instance: axis_data_counter_0, and set properties
  set block_name axis_data_counter
  set block_cell_name axis_data_counter_0
  if { [catch {set axis_data_counter_0 [create_bd_cell -type module -reference $block_name $block_cell_name] } errmsg] } {
     catch {common::send_msg_id "BD_TCL-105" "ERROR" "Unable to add referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   } elseif { $axis_data_counter_0 eq "" } {
     catch {common::send_msg_id "BD_TCL-106" "ERROR" "Unable to referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   }
    set_property -dict [ list \
   CONFIG.p_axis_tdata_width {256} \
 ] $axis_data_counter_0

  # Create instance: axis_data_fifo_0, and set properties
  set axis_data_fifo_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axis_data_fifo axis_data_fifo_0 ]
  set_property -dict [ list \
   CONFIG.FIFO_DEPTH {16} \
   CONFIG.HAS_TKEEP {0} \
   CONFIG.HAS_TLAST {0} \
   CONFIG.HAS_TSTRB {0} \
   CONFIG.IS_ACLK_ASYNC {1} \
   CONFIG.TDATA_NUM_BYTES {4} \
   CONFIG.TDEST_WIDTH {0} \
   CONFIG.TID_WIDTH {0} \
   CONFIG.TUSER_WIDTH {0} \
 ] $axis_data_fifo_0

  # Create interface connections
  connect_bd_intf_net -intf_net GT_DIFF_REFCLK1_0_1 [get_bd_intf_pins Q2_REFCLK0] [get_bd_intf_pins aurora_64b66b_0/GT_DIFF_REFCLK1]
  connect_bd_intf_net -intf_net GT_SERIAL_RX_0_1 [get_bd_intf_pins AURORA_FROM_RAPTOR] [get_bd_intf_pins aurora_64b66b_0/GT_SERIAL_RX]
  connect_bd_intf_net -intf_net aurora_64b66b_0_GT_SERIAL_TX [get_bd_intf_pins AURORA_TO_RAPTOR] [get_bd_intf_pins aurora_64b66b_0/GT_SERIAL_TX]
  connect_bd_intf_net -intf_net aurora_64b66b_0_USER_DATA_M_AXIS_RX [get_bd_intf_pins aurora_64b66b_0/USER_DATA_M_AXIS_RX] [get_bd_intf_pins axis_data_counter_0/S_AXIS]
  connect_bd_intf_net -intf_net axis_data_counter_0_M_AXIS [get_bd_intf_pins aurora_64b66b_0/USER_DATA_S_AXIS_TX] [get_bd_intf_pins axis_data_counter_0/M_AXIS]

  # Create port connections
  connect_bd_net -net GND_dout [get_bd_pins GND] [get_bd_pins aurora_64b66b_0/gt_rxcdrovrden_in] [get_bd_pins aurora_64b66b_0/power_down]
  connect_bd_net -net VCC_dout [get_bd_pins VCC/dout] [get_bd_pins aurora_64b66b_0/s_axi_tx_tvalid] [get_bd_pins axis_data_counter_0/M_AXIS_TREADY] [get_bd_pins axis_data_fifo_0/m_axis_tready] [get_bd_pins axis_data_fifo_0/s_axis_aresetn] [get_bd_pins axis_data_fifo_0/s_axis_tvalid]
  connect_bd_net -net aurora_64b66b_0_user_clk_out [get_bd_pins aurora_64b66b_0/user_clk_out] [get_bd_pins aurora_controller_0/user_clk] [get_bd_pins axis_data_counter_0/AXIS_ACLK] [get_bd_pins axis_data_fifo_0/s_axis_aclk]
  connect_bd_net -net aurora_controller_0_gt_reset_out [get_bd_pins aurora_64b66b_0/pma_init] [get_bd_pins aurora_controller_0/gt_reset_out]
  connect_bd_net -net aurora_controller_0_system_reset_out [get_bd_pins aurora_64b66b_0/reset_pb] [get_bd_pins aurora_controller_0/system_reset_out]
  connect_bd_net -net axis_data_counter_0_o_count [get_bd_pins axis_data_counter_0/o_count] [get_bd_pins axis_data_fifo_0/s_axis_tdata]
  connect_bd_net -net axis_data_fifo_0_m_axis_tdata [get_bd_pins m_axis_tdata] [get_bd_pins axis_data_fifo_0/m_axis_tdata]
  connect_bd_net -net reset_1 [get_bd_pins reset] [get_bd_pins aurora_controller_0/reset_in]
  connect_bd_net -net zynq_ultra_ps_e_0_pl_clk0 [get_bd_pins init_clk] [get_bd_pins aurora_64b66b_0/init_clk] [get_bd_pins aurora_controller_0/init_clk] [get_bd_pins axis_data_fifo_0/m_axis_aclk]

  # Restore current instance
  current_bd_instance $oldCurInst
}


# Procedure to create entire design; Provide argument to make
# procedure reusable. If parentCell is "", will use root.
proc create_root_design { parentCell } {

  variable script_folder
  variable design_name

  if { $parentCell eq "" } {
     set parentCell [get_bd_cells /]
  }

  # Get object for parentCell
  set parentObj [get_bd_cells $parentCell]
  if { $parentObj == "" } {
     catch {common::send_msg_id "BD_TCL-100" "ERROR" "Unable to find parent cell <$parentCell>!"}
     return
  }

  # Make sure parentObj is hier blk
  set parentType [get_property TYPE $parentObj]
  if { $parentType ne "hier" } {
     catch {common::send_msg_id "BD_TCL-101" "ERROR" "Parent <$parentObj> has TYPE = <$parentType>. Expected to be <hier>."}
     return
  }

  # Save current instance; Restore later
  set oldCurInst [current_bd_instance .]

  # Set parent object as current
  current_bd_instance $parentObj


  # Create interface ports
  set AURORA_FROM_RAPTOR [ create_bd_intf_port -mode Slave -vlnv xilinx.com:display_aurora:GT_Serial_Transceiver_Pins_RX_rtl:1.0 AURORA_FROM_RAPTOR ]

  set AURORA_FROM_XDP [ create_bd_intf_port -mode Slave -vlnv xilinx.com:display_aurora:GT_Serial_Transceiver_Pins_RX_rtl:1.0 AURORA_FROM_XDP ]

  set AURORA_TO_RAPTOR [ create_bd_intf_port -mode Master -vlnv xilinx.com:display_aurora:GT_Serial_Transceiver_Pins_TX_rtl:1.0 AURORA_TO_RAPTOR ]

  set AURORA_TO_XDP [ create_bd_intf_port -mode Master -vlnv xilinx.com:display_aurora:GT_Serial_Transceiver_Pins_TX_rtl:1.0 AURORA_TO_XDP ]

  set EIO_GPIO [ create_bd_intf_port -mode Master -vlnv xilinx.com:interface:gpio_rtl:1.0 EIO_GPIO ]

  set EIO_IIC [ create_bd_intf_port -mode Master -vlnv xilinx.com:interface:iic_rtl:1.0 EIO_IIC ]

  set HDMI_DDC [ create_bd_intf_port -mode Master -vlnv xilinx.com:interface:iic_rtl:1.0 HDMI_DDC ]

  set MIPI_0_IIC [ create_bd_intf_port -mode Master -vlnv xilinx.com:interface:iic_rtl:1.0 MIPI_0_IIC ]

  set MIPI_1_IIC [ create_bd_intf_port -mode Master -vlnv xilinx.com:interface:iic_rtl:1.0 MIPI_1_IIC ]

  set MIPI_GPIO [ create_bd_intf_port -mode Master -vlnv xilinx.com:interface:gpio_rtl:1.0 MIPI_GPIO ]

  set Q0_REFCLK0 [ create_bd_intf_port -mode Slave -vlnv xilinx.com:interface:diff_clock_rtl:1.0 Q0_REFCLK0 ]

  set Q2_REFCLK0 [ create_bd_intf_port -mode Slave -vlnv xilinx.com:interface:diff_clock_rtl:1.0 Q2_REFCLK0 ]

  set RAPTOR_GPIO [ create_bd_intf_port -mode Master -vlnv xilinx.com:interface:gpio_rtl:1.0 RAPTOR_GPIO ]

  set SCIO_GPIO [ create_bd_intf_port -mode Master -vlnv xilinx.com:interface:gpio_rtl:1.0 SCIO_GPIO ]

  set SDIO_WIFI [ create_bd_intf_port -mode Master -vlnv xilinx.com:interface:sdio_rtl:1.0 SDIO_WIFI ]

  set SENS_GPS_IIC [ create_bd_intf_port -mode Master -vlnv xilinx.com:interface:iic_rtl:1.0 SENS_GPS_IIC ]

  set SENS_GPS_UART [ create_bd_intf_port -mode Master -vlnv xilinx.com:interface:uart_rtl:1.0 SENS_GPS_UART ]

  set clk_200M [ create_bd_intf_port -mode Slave -vlnv xilinx.com:interface:diff_clock_rtl:1.0 clk_200M ]
  set_property -dict [ list \
   CONFIG.FREQ_HZ {200000000} \
   ] $clk_200M

  set clk_fpga [ create_bd_intf_port -mode Slave -vlnv xilinx.com:interface:diff_clock_rtl:1.0 clk_fpga ]
  set_property -dict [ list \
   CONFIG.FREQ_HZ {200000000} \
   ] $clk_fpga

  set mipi_0_csi [ create_bd_intf_port -mode Slave -vlnv xilinx.com:interface:mipi_phy_rtl:1.0 mipi_0_csi ]

  set mipi_1_csi [ create_bd_intf_port -mode Slave -vlnv xilinx.com:interface:mipi_phy_rtl:1.0 mipi_1_csi ]

  set tep2_0 [ create_bd_intf_port -mode Master -vlnv qdesys.com:motor_control:tep2phy_rtl:1.0 tep2_0 ]

  set tep2_1 [ create_bd_intf_port -mode Master -vlnv qdesys.com:motor_control:tep2phy_rtl:1.0 tep2_1 ]

  set tep2_2 [ create_bd_intf_port -mode Master -vlnv qdesys.com:motor_control:tep2phy_rtl:1.0 tep2_2 ]

  set tep2_3 [ create_bd_intf_port -mode Master -vlnv qdesys.com:motor_control:tep2phy_rtl:1.0 tep2_3 ]


  # Create ports
  set BT_DEV_WAKE [ create_bd_port -dir I BT_DEV_WAKE ]
  set BT_EN [ create_bd_port -dir O -from 0 -to 0 BT_EN ]
  set CTS_FROM_BT_UART [ create_bd_port -dir I CTS_FROM_BT_UART ]
  set EIO_LED1 [ create_bd_port -dir O -from 0 -to 0 EIO_LED1 ]
  set EIO_LED2 [ create_bd_port -dir O -from 0 -to 0 EIO_LED2 ]
  set HDMI_5V_EN [ create_bd_port -dir O -from 0 -to 0 HDMI_5V_EN ]
  set HDMI_HPD [ create_bd_port -dir I HDMI_HPD ]
  set HDMI_RT_OE [ create_bd_port -dir O -from 0 -to 0 HDMI_RT_OE ]
  set HDMI_TX_DAT_N_OUT [ create_bd_port -dir O -from 3 -to 0 HDMI_TX_DAT_N_OUT ]
  set HDMI_TX_DAT_P_OUT [ create_bd_port -dir O -from 3 -to 0 HDMI_TX_DAT_P_OUT ]
  set Q1_REFCLK0_n [ create_bd_port -dir I Q1_REFCLK0_n ]
  set Q1_REFCLK0_p [ create_bd_port -dir I Q1_REFCLK0_p ]
  set RTS_TO_BT_UART [ create_bd_port -dir O RTS_TO_BT_UART ]
  set SDIO_WIFI_EN [ create_bd_port -dir O -from 0 -to 0 SDIO_WIFI_EN ]
  set SDIO_WIFI_WAKE [ create_bd_port -dir I SDIO_WIFI_WAKE ]
  set SDIO_WIFI_clk_pin [ create_bd_port -dir IO -type clk SDIO_WIFI_clk_pin ]
  set SENS_ACCEL_INT0 [ create_bd_port -dir I SENS_ACCEL_INT0 ]
  set SENS_ACCEL_INT1 [ create_bd_port -dir I SENS_ACCEL_INT1 ]
  set SENS_GPS_DSEL [ create_bd_port -dir O -from 0 -to 0 SENS_GPS_DSEL ]
  set SENS_GPS_PIO11 [ create_bd_port -dir I -from 0 -to 0 SENS_GPS_PIO11 ]
  set SENS_GPS_PIO13_EINT [ create_bd_port -dir I -from 0 -to 0 SENS_GPS_PIO13_EINT ]
  set SENS_GPS_PIO14 [ create_bd_port -dir I -from 0 -to 0 SENS_GPS_PIO14 ]
  set SENS_GPS_PIO15 [ create_bd_port -dir I -from 0 -to 0 SENS_GPS_PIO15 ]
  set SENS_GPS_RESET_N [ create_bd_port -dir O -from 0 -to 0 SENS_GPS_RESET_N ]
  set SENS_GYRO_INT0 [ create_bd_port -dir I SENS_GYRO_INT0 ]
  set SENS_GYRO_INT1 [ create_bd_port -dir I SENS_GYRO_INT1 ]
  set SENS_MAGN_DRDY [ create_bd_port -dir I SENS_MAGN_DRDY ]
  set SENS_MAGN_INT [ create_bd_port -dir I SENS_MAGN_INT ]
  set SENS_SPI0_CS_ACCEL [ create_bd_port -dir O SENS_SPI0_CS_ACCEL ]
  set SENS_SPI0_CS_GYRO [ create_bd_port -dir O -from 0 -to 0 SENS_SPI0_CS_GYRO ]
  set SENS_SPI0_MISO [ create_bd_port -dir I SENS_SPI0_MISO ]
  set SENS_SPI0_MOSI [ create_bd_port -dir O SENS_SPI0_MOSI ]
  set SENS_SPI0_SCK [ create_bd_port -dir O SENS_SPI0_SCK ]
  set SENS_SPI1_CS_ENV [ create_bd_port -dir O SENS_SPI1_CS_ENV ]
  set SENS_SPI1_CS_MAGN [ create_bd_port -dir O SENS_SPI1_CS_MAGN ]
  set SENS_SPI1_MISO [ create_bd_port -dir I SENS_SPI1_MISO ]
  set SENS_SPI1_MOSI [ create_bd_port -dir O SENS_SPI1_MOSI ]
  set SENS_SPI1_SCK [ create_bd_port -dir O SENS_SPI1_SCK ]
  set TX_FROM_BT_UART [ create_bd_port -dir I TX_FROM_BT_UART ]
  set TX_TO_BT_UART [ create_bd_port -dir O TX_TO_BT_UART ]
  set fan_pwm [ create_bd_port -dir O -from 0 -to 0 fan_pwm ]
  set fan_tacho [ create_bd_port -dir I -from 0 -to 0 fan_tacho ]
  set mipi_0_clk_tmp [ create_bd_port -dir O -from 0 -to 0 mipi_0_clk_tmp ]
  set mipi_0_rst [ create_bd_port -dir O -from 0 -to 0 mipi_0_rst ]
  set mipi_1_clk_tmp [ create_bd_port -dir O -from 0 -to 0 mipi_1_clk_tmp ]
  set mipi_1_rst [ create_bd_port -dir O -from 0 -to 0 mipi_1_rst ]
  set switch_pl_0 [ create_bd_port -dir I switch_pl_0 ]
  set user_led [ create_bd_port -dir O -from 0 -to 0 user_led ]
  set xdp_version [ create_bd_port -dir I -from 3 -to 0 xdp_version ]

  # Create instance: GND, and set properties
  set GND [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant GND ]
  set_property -dict [ list \
   CONFIG.CONST_VAL {0} \
 ] $GND

  # Create instance: aurora_raptor
  create_hier_cell_aurora_raptor [current_bd_instance .] aurora_raptor

  # Create instance: aurora_xdp
  create_hier_cell_aurora_xdp [current_bd_instance .] aurora_xdp

  # Create instance: axi_gpio_eio, and set properties
  set axi_gpio_eio [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_gpio axi_gpio_eio ]
  set_property -dict [ list \
   CONFIG.C_GPIO_WIDTH {13} \
 ] $axi_gpio_eio

  # Create instance: axi_gpio_mgt_speeds, and set properties
  set axi_gpio_mgt_speeds [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_gpio axi_gpio_mgt_speeds ]
  set_property -dict [ list \
   CONFIG.C_ALL_INPUTS {1} \
   CONFIG.C_ALL_INPUTS_2 {1} \
   CONFIG.C_IS_DUAL {1} \
 ] $axi_gpio_mgt_speeds

  # Create instance: axi_gpio_mipi, and set properties
  set axi_gpio_mipi [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_gpio axi_gpio_mipi ]
  set_property -dict [ list \
   CONFIG.C_ALL_OUTPUTS {0} \
   CONFIG.C_GPIO_WIDTH {8} \
 ] $axi_gpio_mipi

  # Create instance: axi_gpio_raptor, and set properties
  set axi_gpio_raptor [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_gpio axi_gpio_raptor ]
  set_property -dict [ list \
   CONFIG.C_GPIO_WIDTH {12} \
 ] $axi_gpio_raptor

  # Create instance: axi_gpio_scio, and set properties
  set axi_gpio_scio [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_gpio axi_gpio_scio ]
  set_property -dict [ list \
   CONFIG.C_GPIO_WIDTH {12} \
 ] $axi_gpio_scio

  # Create instance: axi_iic_1, and set properties
  set axi_iic_1 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_iic axi_iic_1 ]
  set_property -dict [ list \
   CONFIG.C_SCL_INERTIAL_DELAY {5} \
   CONFIG.C_SDA_INERTIAL_DELAY {5} \
 ] $axi_iic_1

  # Create instance: axi_iic_gps, and set properties
  set axi_iic_gps [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_iic axi_iic_gps ]
  set_property -dict [ list \
   CONFIG.C_SCL_INERTIAL_DELAY {5} \
   CONFIG.C_SDA_INERTIAL_DELAY {5} \
 ] $axi_iic_gps

  # Create instance: axi_interconnect_0, and set properties
  set axi_interconnect_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_interconnect axi_interconnect_0 ]
  set_property -dict [ list \
   CONFIG.NUM_MI {23} \
 ] $axi_interconnect_0

  # Create instance: axi_ivplogger_0, and set properties
  set axi_ivplogger_0 [ create_bd_cell -type ip -vlnv qdesys.com:motor_control:axi_ivplogger axi_ivplogger_0 ]
  set_property -dict [ list \
   CONFIG.C_CHANNEL_MAP {0x2FF} \
   CONFIG.C_MEMORY_SIZE {0x00100000} \
 ] $axi_ivplogger_0

  # Create instance: axi_motorfoc_0, and set properties
  set axi_motorfoc_0 [ create_bd_cell -type ip -vlnv qdesys.com:motor_control:axi_motorfoc axi_motorfoc_0 ]
  set_property -dict [ list \
   CONFIG.C_ENCODER {0} \
   CONFIG.C_HALLSENSOR {0} \
   CONFIG.C_INV_IN_MAP {0x0FF} \
   CONFIG.C_INV_IN_NOT {0x007} \
   CONFIG.C_INV_OFSV_MODE {3} \
   CONFIG.C_RESOLVER {0} \
   CONFIG.C_RPFM_3_LEVEL {0} \
   CONFIG.C_RPFM_TPNC {0} \
 ] $axi_motorfoc_0

  # Create instance: axi_motorfoc_1, and set properties
  set axi_motorfoc_1 [ create_bd_cell -type ip -vlnv qdesys.com:motor_control:axi_motorfoc axi_motorfoc_1 ]
  set_property -dict [ list \
   CONFIG.C_ENCODER {0} \
   CONFIG.C_HALLSENSOR {0} \
   CONFIG.C_HW_LINK_REF {1} \
   CONFIG.C_INV_IN_MAP {0x0FF} \
   CONFIG.C_INV_IN_NOT {0x007} \
   CONFIG.C_INV_OFSV_MODE {3} \
   CONFIG.C_RESOLVER {0} \
   CONFIG.C_RPFM_3_LEVEL {0} \
   CONFIG.C_RPFM_TPNC {0} \
 ] $axi_motorfoc_1

  # Create instance: axi_motorfoc_2, and set properties
  set axi_motorfoc_2 [ create_bd_cell -type ip -vlnv qdesys.com:motor_control:axi_motorfoc axi_motorfoc_2 ]
  set_property -dict [ list \
   CONFIG.C_ENCODER {0} \
   CONFIG.C_HALLSENSOR {0} \
   CONFIG.C_HW_LINK_REF {2} \
   CONFIG.C_INV_IN_MAP {0x0FF} \
   CONFIG.C_INV_IN_NOT {0x007} \
   CONFIG.C_INV_OFSV_MODE {3} \
   CONFIG.C_RESOLVER {0} \
   CONFIG.C_RPFM_3_LEVEL {0} \
   CONFIG.C_RPFM_TPNC {0} \
 ] $axi_motorfoc_2

  # Create instance: axi_motorfoc_3, and set properties
  set axi_motorfoc_3 [ create_bd_cell -type ip -vlnv qdesys.com:motor_control:axi_motorfoc axi_motorfoc_3 ]
  set_property -dict [ list \
   CONFIG.C_ENCODER {0} \
   CONFIG.C_HALLSENSOR {0} \
   CONFIG.C_HW_LINK_REF {3} \
   CONFIG.C_INV_IN_MAP {0x0FF} \
   CONFIG.C_INV_IN_NOT {0x007} \
   CONFIG.C_INV_OFSV_MODE {3} \
   CONFIG.C_RESOLVER {0} \
   CONFIG.C_RPFM_3_LEVEL {0} \
   CONFIG.C_RPFM_TPNC {0} \
 ] $axi_motorfoc_3

  # Create instance: axi_smc, and set properties
  set axi_smc [ create_bd_cell -type ip -vlnv xilinx.com:ip:smartconnect axi_smc ]
  set_property -dict [ list \
   CONFIG.NUM_CLKS {2} \
   CONFIG.NUM_SI {3} \
 ] $axi_smc

  # Create instance: axi_smc1, and set properties
  set axi_smc1 [ create_bd_cell -type ip -vlnv xilinx.com:ip:smartconnect axi_smc1 ]
  set_property -dict [ list \
   CONFIG.NUM_SI {1} \
 ] $axi_smc1

  # Create instance: axi_smc2, and set properties
  set axi_smc2 [ create_bd_cell -type ip -vlnv xilinx.com:ip:smartconnect axi_smc2 ]
  set_property -dict [ list \
   CONFIG.NUM_SI {2} \
 ] $axi_smc2

  # Create instance: axi_smc3, and set properties
  set axi_smc3 [ create_bd_cell -type ip -vlnv xilinx.com:ip:smartconnect axi_smc3 ]
  set_property -dict [ list \
   CONFIG.NUM_SI {2} \
 ] $axi_smc3

  # Create instance: axi_smc_1, and set properties
  set axi_smc_1 [ create_bd_cell -type ip -vlnv xilinx.com:ip:smartconnect axi_smc_1 ]
  set_property -dict [ list \
   CONFIG.NUM_SI {2} \
 ] $axi_smc_1

  # Create instance: axi_tep2_drv_0, and set properties
  set axi_tep2_drv_0 [ create_bd_cell -type ip -vlnv qdesys.com:motor_control:axi_tep2_drv axi_tep2_drv_0 ]
  set_property -dict [ list \
   CONFIG.C_CARRIER {18} \
 ] $axi_tep2_drv_0

  # Create instance: axi_tep2_drv_1, and set properties
  set axi_tep2_drv_1 [ create_bd_cell -type ip -vlnv qdesys.com:motor_control:axi_tep2_drv axi_tep2_drv_1 ]
  set_property -dict [ list \
   CONFIG.C_CARRIER {18} \
   CONFIG.C_HW_LINK_REF {1} \
 ] $axi_tep2_drv_1

  # Create instance: axi_tep2_drv_2, and set properties
  set axi_tep2_drv_2 [ create_bd_cell -type ip -vlnv qdesys.com:motor_control:axi_tep2_drv axi_tep2_drv_2 ]
  set_property -dict [ list \
   CONFIG.C_CARRIER {18} \
   CONFIG.C_HW_LINK_REF {2} \
 ] $axi_tep2_drv_2

  # Create instance: axi_tep2_drv_3, and set properties
  set axi_tep2_drv_3 [ create_bd_cell -type ip -vlnv qdesys.com:motor_control:axi_tep2_drv axi_tep2_drv_3 ]
  set_property -dict [ list \
   CONFIG.C_CARRIER {18} \
   CONFIG.C_HW_LINK_REF {3} \
 ] $axi_tep2_drv_3

  # Create instance: axi_uart16550_0, and set properties
  set axi_uart16550_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_uart16550 axi_uart16550_0 ]

  # Create instance: axil_pwm_fan_control_0, and set properties
  set axil_pwm_fan_control_0 [ create_bd_cell -type ip -vlnv topicembeddedproducts.com:ip:topic_fanctrl axil_pwm_fan_control_0 ]
  set_property -dict [ list \
   CONFIG.p_number_of_fans {1} \
 ] $axil_pwm_fan_control_0

  # Create instance: axis_data_fifo_0, and set properties
  set axis_data_fifo_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axis_data_fifo axis_data_fifo_0 ]
  set_property -dict [ list \
   CONFIG.FIFO_DEPTH {2048} \
 ] $axis_data_fifo_0

  # Create instance: c_counter_binary_0, and set properties
  set c_counter_binary_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:c_counter_binary c_counter_binary_0 ]
  set_property -dict [ list \
   CONFIG.Output_Width {26} \
 ] $c_counter_binary_0

  # Create instance: clk_fpga_bufg, and set properties
  set clk_fpga_bufg [ create_bd_cell -type ip -vlnv xilinx.com:ip:util_ds_buf clk_fpga_bufg ]
  set_property -dict [ list \
   CONFIG.C_BUF_TYPE {BUFG} \
 ] $clk_fpga_bufg

  # Create instance: clk_wiz_0, and set properties
  set clk_wiz_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:clk_wiz clk_wiz_0 ]
  set_property -dict [ list \
   CONFIG.CLKOUT1_JITTER {253.036} \
   CONFIG.CLKOUT1_REQUESTED_OUT_FREQ {24.000} \
   CONFIG.MMCM_CLKOUT0_DIVIDE_F {51.625} \
   CONFIG.MMCM_DIVCLK_DIVIDE {8} \
   CONFIG.USE_LOCKED {false} \
   CONFIG.USE_RESET {false} \
 ] $clk_wiz_0

  # Create instance: clk_wiz_1, and set properties
  set clk_wiz_1 [ create_bd_cell -type ip -vlnv xilinx.com:ip:clk_wiz clk_wiz_1 ]
  set_property -dict [ list \
   CONFIG.CLKOUT1_JITTER {206.083} \
   CONFIG.CLKOUT1_PHASE_ERROR {302.151} \
   CONFIG.CLKOUT1_REQUESTED_OUT_FREQ {33.000} \
   CONFIG.CLKOUT2_JITTER {150.138} \
   CONFIG.CLKOUT2_PHASE_ERROR {302.151} \
   CONFIG.CLKOUT2_REQUESTED_OUT_FREQ {333.000} \
   CONFIG.CLKOUT2_USED {true} \
   CONFIG.CLK_OUT1_PORT {clk_33Mhz} \
   CONFIG.CLK_OUT2_PORT {clk_333Mhz} \
   CONFIG.MMCM_CLKFBOUT_MULT_F {66.625} \
   CONFIG.MMCM_CLKOUT0_DIVIDE_F {40.375} \
   CONFIG.MMCM_CLKOUT1_DIVIDE {4} \
   CONFIG.MMCM_DIVCLK_DIVIDE {5} \
   CONFIG.NUM_OUT_CLKS {2} \
   CONFIG.RESET_PORT {resetn} \
   CONFIG.RESET_TYPE {ACTIVE_LOW} \
 ] $clk_wiz_1

  # Create instance: const_VCC, and set properties
  set const_VCC [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant const_VCC ]
  set_property -dict [ list \
   CONFIG.CONST_VAL {1} \
 ] $const_VCC

  # Create instance: dyplo_axi_0, and set properties
  set dyplo_axi_0 [ create_bd_cell -type ip -vlnv topicembeddedproducts.com:dyplo:dyplo_axi dyplo_axi_0 ]
  set_property -dict [ list \
   CONFIG.CPU_TO_FPGA_STREAMS {4} \
   CONFIG.DEVICE_ARCHITECTURE {ULTRASCALE} \
   CONFIG.DMA_NODES {2} \
   CONFIG.FIXED_NODES {0} \
   CONFIG.FPGA_TO_CPU_STREAMS {4} \
   CONFIG.IO_NODES {1} \
   CONFIG.IO_NODE_1_OUTSTREAMS {1} \
   CONFIG.IO_NODE_2_OUTSTREAMS {1} \
   CONFIG.MONITOR {true} \
   CONFIG.PR_NODES {0} \
   CONFIG.STATIC_BITSTREAM_ID {0x4534} \
 ] $dyplo_axi_0

  # Create instance: hdmi_out
  create_hier_cell_hdmi_out [current_bd_instance .] hdmi_out

  # Create instance: io_buf_sdio1_clk, and set properties
  set io_buf_sdio1_clk [ create_bd_cell -type ip -vlnv xilinx.com:ip:util_ds_buf io_buf_sdio1_clk ]
  set_property -dict [ list \
   CONFIG.C_BUF_TYPE {IOBUF} \
 ] $io_buf_sdio1_clk

  # Create instance: mipi_camera_0
  create_hier_cell_mipi_camera_0 [current_bd_instance .] mipi_camera_0

  # Create instance: mipi_camera_1
  create_hier_cell_mipi_camera_1 [current_bd_instance .] mipi_camera_1

  # Create instance: proc_sys_reset_0, and set properties
  set proc_sys_reset_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:proc_sys_reset proc_sys_reset_0 ]

  # Create instance: strb_0x3, and set properties
  set strb_0x3 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant strb_0x3 ]
  set_property -dict [ list \
   CONFIG.CONST_VAL {3} \
   CONFIG.CONST_WIDTH {2} \
 ] $strb_0x3

  # Create instance: util_ds_buf_0, and set properties
  set util_ds_buf_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:util_ds_buf util_ds_buf_0 ]

  # Create instance: util_ds_buf_1, and set properties
  set util_ds_buf_1 [ create_bd_cell -type ip -vlnv xilinx.com:ip:util_ds_buf util_ds_buf_1 ]

  # Create instance: util_ds_buf_2, and set properties
  set util_ds_buf_2 [ create_bd_cell -type ip -vlnv xilinx.com:ip:util_ds_buf util_ds_buf_2 ]
  set_property -dict [ list \
   CONFIG.C_BUF_TYPE {BUFG} \
 ] $util_ds_buf_2

  # Create instance: vcu_0, and set properties
  set vcu_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:vcu vcu_0 ]

  # Create instance: xlslice_0, and set properties
  set xlslice_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice xlslice_0 ]
  set_property -dict [ list \
   CONFIG.DIN_FROM {25} \
   CONFIG.DIN_TO {25} \
   CONFIG.DIN_WIDTH {26} \
   CONFIG.DOUT_WIDTH {1} \
 ] $xlslice_0

  # Create instance: zynq_ps
  create_hier_cell_zynq_ps [current_bd_instance .] zynq_ps

  # Create interface connections
  connect_bd_intf_net -intf_net AURORA_FROM_RAPTOR_0_1 [get_bd_intf_ports AURORA_FROM_RAPTOR] [get_bd_intf_pins aurora_raptor/AURORA_FROM_RAPTOR]
  connect_bd_intf_net -intf_net CLK_IN_D_0_1 [get_bd_intf_ports clk_200M] [get_bd_intf_pins util_ds_buf_0/CLK_IN_D]
  connect_bd_intf_net -intf_net CLK_IN_D_0_2 [get_bd_intf_ports clk_fpga] [get_bd_intf_pins util_ds_buf_1/CLK_IN_D]
  connect_bd_intf_net -intf_net Conn [get_bd_intf_pins axi_motorfoc_0/MCIF] [get_bd_intf_pins axi_tep2_drv_0/MCIF]
connect_bd_intf_net -intf_net [get_bd_intf_nets Conn] [get_bd_intf_pins axi_ivplogger_0/MCIF] [get_bd_intf_pins axi_motorfoc_0/MCIF]
  connect_bd_intf_net -intf_net GT_DIFF_REFCLK1_0_1 [get_bd_intf_ports Q0_REFCLK0] [get_bd_intf_pins aurora_xdp/Q0_REFCLK0]
  connect_bd_intf_net -intf_net GT_SERIAL_RX_0_1 [get_bd_intf_ports AURORA_FROM_XDP] [get_bd_intf_pins aurora_xdp/AURORA_FROM_XDP]
  connect_bd_intf_net -intf_net Q2_REFCLK0_1 [get_bd_intf_ports Q2_REFCLK0] [get_bd_intf_pins aurora_raptor/Q2_REFCLK0]
  connect_bd_intf_net -intf_net S_AXI_1 [get_bd_intf_pins axi_interconnect_0/M01_AXI] [get_bd_intf_pins mipi_camera_0/S_AXI]
  connect_bd_intf_net -intf_net aurora_64b66b_0_GT_SERIAL_TX [get_bd_intf_ports AURORA_TO_XDP] [get_bd_intf_pins aurora_xdp/AURORA_TO_XDP]
  connect_bd_intf_net -intf_net aurora_raptor_AURORA_TO_RAPTOR [get_bd_intf_ports AURORA_TO_RAPTOR] [get_bd_intf_pins aurora_raptor/AURORA_TO_RAPTOR]
  connect_bd_intf_net -intf_net axi_gpio_0_GPIO [get_bd_intf_ports MIPI_GPIO] [get_bd_intf_pins axi_gpio_mipi/GPIO]
  connect_bd_intf_net -intf_net axi_gpio_eio_GPIO [get_bd_intf_ports EIO_GPIO] [get_bd_intf_pins axi_gpio_eio/GPIO]
  connect_bd_intf_net -intf_net axi_gpio_raptor_GPIO [get_bd_intf_ports RAPTOR_GPIO] [get_bd_intf_pins axi_gpio_raptor/GPIO]
  connect_bd_intf_net -intf_net axi_gpio_scio_GPIO [get_bd_intf_ports SCIO_GPIO] [get_bd_intf_pins axi_gpio_scio/GPIO]
  connect_bd_intf_net -intf_net axi_iic_1_IIC [get_bd_intf_ports EIO_IIC] [get_bd_intf_pins axi_iic_1/IIC]
  connect_bd_intf_net -intf_net axi_iic_gps_IIC [get_bd_intf_ports SENS_GPS_IIC] [get_bd_intf_pins axi_iic_gps/IIC]
  connect_bd_intf_net -intf_net axi_interconnect_0_M00_AXI [get_bd_intf_pins axi_interconnect_0/M00_AXI] [get_bd_intf_pins hdmi_out/S_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M02_AXI [get_bd_intf_pins axi_gpio_mgt_speeds/S_AXI] [get_bd_intf_pins axi_interconnect_0/M02_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M03_AXI [get_bd_intf_pins axi_gpio_eio/S_AXI] [get_bd_intf_pins axi_interconnect_0/M03_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M04_AXI [get_bd_intf_pins axi_iic_1/S_AXI] [get_bd_intf_pins axi_interconnect_0/M04_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M05_AXI [get_bd_intf_pins axi_gpio_mipi/S_AXI] [get_bd_intf_pins axi_interconnect_0/M05_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M06_AXI [get_bd_intf_pins axi_iic_gps/S_AXI] [get_bd_intf_pins axi_interconnect_0/M06_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M07_AXI [get_bd_intf_pins axi_interconnect_0/M07_AXI] [get_bd_intf_pins axi_ivplogger_0/S_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M08_AXI [get_bd_intf_pins axi_gpio_scio/S_AXI] [get_bd_intf_pins axi_interconnect_0/M08_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M09_AXI [get_bd_intf_pins axi_gpio_raptor/S_AXI] [get_bd_intf_pins axi_interconnect_0/M09_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M10_AXI [get_bd_intf_pins axi_interconnect_0/M10_AXI] [get_bd_intf_pins axi_uart16550_0/S_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M11_AXI [get_bd_intf_pins axi_interconnect_0/M11_AXI] [get_bd_intf_pins axil_pwm_fan_control_0/S_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M12_AXI [get_bd_intf_pins axi_interconnect_0/M12_AXI] [get_bd_intf_pins vcu_0/S_AXI_LITE]
  connect_bd_intf_net -intf_net axi_interconnect_0_M13_AXI [get_bd_intf_pins axi_interconnect_0/M13_AXI] [get_bd_intf_pins dyplo_axi_0/S_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M14_AXI [get_bd_intf_pins axi_interconnect_0/M14_AXI] [get_bd_intf_pins mipi_camera_1/S_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M15_AXI [get_bd_intf_pins axi_interconnect_0/M15_AXI] [get_bd_intf_pins axi_tep2_drv_3/S_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M16_AXI [get_bd_intf_pins axi_interconnect_0/M16_AXI] [get_bd_intf_pins axi_motorfoc_0/S_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M17_AXI [get_bd_intf_pins axi_interconnect_0/M17_AXI] [get_bd_intf_pins axi_motorfoc_1/S_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M18_AXI [get_bd_intf_pins axi_interconnect_0/M18_AXI] [get_bd_intf_pins axi_motorfoc_2/S_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M19_AXI [get_bd_intf_pins axi_interconnect_0/M19_AXI] [get_bd_intf_pins axi_motorfoc_3/S_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M20_AXI [get_bd_intf_pins axi_interconnect_0/M20_AXI] [get_bd_intf_pins axi_tep2_drv_0/S_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M21_AXI [get_bd_intf_pins axi_interconnect_0/M21_AXI] [get_bd_intf_pins axi_tep2_drv_1/S_AXI]
  connect_bd_intf_net -intf_net axi_interconnect_0_M22_AXI [get_bd_intf_pins axi_interconnect_0/M22_AXI] [get_bd_intf_pins axi_tep2_drv_2/S_AXI]
  connect_bd_intf_net -intf_net axi_ivplogger_0_M_AXI [get_bd_intf_pins axi_ivplogger_0/M_AXI] [get_bd_intf_pins axi_smc/S01_AXI]
  connect_bd_intf_net -intf_net axi_smc1_M00_AXI [get_bd_intf_pins axi_smc1/M00_AXI] [get_bd_intf_pins zynq_ps/S_AXI_HPC0_FPD]
  connect_bd_intf_net -intf_net axi_smc2_M00_AXI [get_bd_intf_pins axi_smc2/M00_AXI] [get_bd_intf_pins zynq_ps/S_AXI_HP2_FPD]
  connect_bd_intf_net -intf_net axi_smc3_M00_AXI [get_bd_intf_pins axi_smc3/M00_AXI] [get_bd_intf_pins zynq_ps/S_AXI_HP3_FPD]
  connect_bd_intf_net -intf_net axi_smc_1_M00_AXI [get_bd_intf_pins axi_smc_1/M00_AXI] [get_bd_intf_pins zynq_ps/S_AXI_HPC1_FPD]
  connect_bd_intf_net -intf_net axi_smc_M00_AXI [get_bd_intf_pins axi_smc/M00_AXI] [get_bd_intf_pins zynq_ps/S_AXI_HP0_FPD]
  connect_bd_intf_net -intf_net axi_tep2_drv_0_MPHY [get_bd_intf_ports tep2_0] [get_bd_intf_pins axi_tep2_drv_0/MPHY]
  connect_bd_intf_net -intf_net axi_tep2_drv_1_MCIF [get_bd_intf_pins axi_motorfoc_1/MCIF] [get_bd_intf_pins axi_tep2_drv_1/MCIF]
  connect_bd_intf_net -intf_net axi_tep2_drv_1_MPHY [get_bd_intf_ports tep2_1] [get_bd_intf_pins axi_tep2_drv_1/MPHY]
  connect_bd_intf_net -intf_net axi_tep2_drv_2_MCIF [get_bd_intf_pins axi_motorfoc_2/MCIF] [get_bd_intf_pins axi_tep2_drv_2/MCIF]
  connect_bd_intf_net -intf_net axi_tep2_drv_2_MPHY [get_bd_intf_ports tep2_2] [get_bd_intf_pins axi_tep2_drv_2/MPHY]
  connect_bd_intf_net -intf_net axi_tep2_drv_3_MCIF [get_bd_intf_pins axi_motorfoc_3/MCIF] [get_bd_intf_pins axi_tep2_drv_3/MCIF]
  connect_bd_intf_net -intf_net axi_tep2_drv_3_MPHY [get_bd_intf_ports tep2_3] [get_bd_intf_pins axi_tep2_drv_3/MPHY]
  connect_bd_intf_net -intf_net axis_data_fifo_0_M_AXIS [get_bd_intf_pins axis_data_fifo_0/M_AXIS] [get_bd_intf_pins dyplo_axi_0/io_node_1_in_stream_1]
  connect_bd_intf_net -intf_net dyplo_axi_0_M_AXI_0 [get_bd_intf_pins axi_smc_1/S00_AXI] [get_bd_intf_pins dyplo_axi_0/M_AXI_0]
  connect_bd_intf_net -intf_net dyplo_axi_0_M_AXI_1 [get_bd_intf_pins axi_smc_1/S01_AXI] [get_bd_intf_pins dyplo_axi_0/M_AXI_1]
  connect_bd_intf_net -intf_net hdmi_out_HDMI_DDC [get_bd_intf_ports HDMI_DDC] [get_bd_intf_pins hdmi_out/HDMI_DDC]
  connect_bd_intf_net -intf_net hdmi_out_M_AXI_MM2S [get_bd_intf_pins axi_smc/S00_AXI] [get_bd_intf_pins hdmi_out/M_AXI_MM2S]
  connect_bd_intf_net -intf_net mipi_0_csi_1 [get_bd_intf_ports mipi_0_csi] [get_bd_intf_pins mipi_camera_0/mipi_csi]
  connect_bd_intf_net -intf_net mipi_1_csi_1 [get_bd_intf_ports mipi_1_csi] [get_bd_intf_pins mipi_camera_1/mipi_csi]
  connect_bd_intf_net -intf_net mipi_camera_0_MIPI_IIC [get_bd_intf_ports MIPI_0_IIC] [get_bd_intf_pins mipi_camera_0/MIPI_IIC]
  connect_bd_intf_net -intf_net mipi_camera_0_m_axi_mm_video_0 [get_bd_intf_pins axi_smc/S02_AXI] [get_bd_intf_pins mipi_camera_0/m_axi_mm_video_0]
  connect_bd_intf_net -intf_net mipi_camera_1_MIPI_IIC [get_bd_intf_ports MIPI_1_IIC] [get_bd_intf_pins mipi_camera_1/MIPI_IIC]
  connect_bd_intf_net -intf_net mipi_camera_1_M_AXIS_CAM [get_bd_intf_pins axis_data_fifo_0/S_AXIS] [get_bd_intf_pins mipi_camera_1/M_AXIS_CAM]
  connect_bd_intf_net -intf_net vcu_0_M_AXI_DEC0 [get_bd_intf_pins axi_smc3/S00_AXI] [get_bd_intf_pins vcu_0/M_AXI_DEC0]
  connect_bd_intf_net -intf_net vcu_0_M_AXI_DEC1 [get_bd_intf_pins axi_smc3/S01_AXI] [get_bd_intf_pins vcu_0/M_AXI_DEC1]
  connect_bd_intf_net -intf_net vcu_0_M_AXI_ENC0 [get_bd_intf_pins axi_smc2/S00_AXI] [get_bd_intf_pins vcu_0/M_AXI_ENC0]
  connect_bd_intf_net -intf_net vcu_0_M_AXI_ENC1 [get_bd_intf_pins axi_smc2/S01_AXI] [get_bd_intf_pins vcu_0/M_AXI_ENC1]
  connect_bd_intf_net -intf_net vcu_0_M_AXI_MCU [get_bd_intf_pins axi_smc1/S00_AXI] [get_bd_intf_pins vcu_0/M_AXI_MCU]
  connect_bd_intf_net -intf_net zynq_ps_M_AXI_HPM0_FPD [get_bd_intf_pins axi_interconnect_0/S00_AXI] [get_bd_intf_pins zynq_ps/M_AXI_HPM0_FPD]
  connect_bd_intf_net -intf_net zynq_ps_SDIO_WIFI [get_bd_intf_ports SDIO_WIFI] [get_bd_intf_pins zynq_ps/SDIO_WIFI]
  connect_bd_intf_net -intf_net zynq_ps_UART_1_0 [get_bd_intf_ports SENS_GPS_UART] [get_bd_intf_pins zynq_ps/UART_1]

  # Create port connections
  connect_bd_net -net BT_DEV_WAKE_1 [get_bd_ports BT_DEV_WAKE] [get_bd_pins zynq_ps/BT_DEV_WAKE]
  connect_bd_net -net GND_dout [get_bd_pins GND/dout] [get_bd_pins aurora_raptor/GND] [get_bd_pins aurora_xdp/GND] [get_bd_pins hdmi_out/GND]
  connect_bd_net -net HDMI_HPD_1 [get_bd_ports HDMI_HPD] [get_bd_pins hdmi_out/HDMI_HPD]
  connect_bd_net -net Net [get_bd_ports SDIO_WIFI_clk_pin] [get_bd_pins io_buf_sdio1_clk/IOBUF_IO_IO]
  connect_bd_net -net Q1_REFCLK0_n_1 [get_bd_ports Q1_REFCLK0_n] [get_bd_pins hdmi_out/mgtrefclk0_pad_n_in_0]
  connect_bd_net -net Q1_REFCLK0_p_1 [get_bd_ports Q1_REFCLK0_p] [get_bd_pins hdmi_out/mgtrefclk0_pad_p_in_0]
  connect_bd_net -net SDIO_WIFI_busvolt -boundary_type upper [get_bd_pins zynq_ps/SDIO_WIFI_busvolt]
  set_property HDL_ATTRIBUTE.MARK_DEBUG {true} [get_bd_nets SDIO_WIFI_busvolt]
  connect_bd_net -net SDIO_WIFI_clk_fb_1 [get_bd_pins io_buf_sdio1_clk/IOBUF_IO_O] [get_bd_pins zynq_ps/SDIO_WIFI_clk_fb]
  connect_bd_net -net SDIO_WIFI_led -boundary_type upper [get_bd_pins zynq_ps/SDIO_WIFI_led]
  set_property HDL_ATTRIBUTE.MARK_DEBUG {true} [get_bd_nets SDIO_WIFI_led]
  connect_bd_net -net SENS_ACCEL_INT0_1 [get_bd_ports SENS_ACCEL_INT0] [get_bd_pins zynq_ps/SENS_ACCEL_INT0]
  connect_bd_net -net SENS_ACCEL_INT1_1 [get_bd_ports SENS_ACCEL_INT1] [get_bd_pins zynq_ps/SENS_ACCEL_INT1]
  connect_bd_net -net SENS_GPS_PIO11_1 [get_bd_ports SENS_GPS_PIO11] [get_bd_pins zynq_ps/SENS_GPS_PIO11]
  connect_bd_net -net SENS_GPS_PIO13_EINT_1 [get_bd_ports SENS_GPS_PIO13_EINT] [get_bd_pins zynq_ps/SENS_GPS_PIO13_EINT]
  connect_bd_net -net SENS_GPS_PIO14_1 [get_bd_ports SENS_GPS_PIO14] [get_bd_pins zynq_ps/SENS_GPS_PIO14]
  connect_bd_net -net SENS_GPS_PIO15_1 [get_bd_ports SENS_GPS_PIO15] [get_bd_pins zynq_ps/SENS_GPS_PIO15]
  connect_bd_net -net SENS_GYRO_INT0_1 [get_bd_ports SENS_GYRO_INT0] [get_bd_pins zynq_ps/SENS_GYRO_INT0]
  connect_bd_net -net SENS_GYRO_INT1_1 [get_bd_ports SENS_GYRO_INT1] [get_bd_pins zynq_ps/SENS_GYRO_INT1]
  connect_bd_net -net SENS_MAGN_DRDY_1 [get_bd_ports SENS_MAGN_DRDY] [get_bd_pins zynq_ps/SENS_MAGN_DRDY]
  connect_bd_net -net SENS_MAGN_INT_1 [get_bd_ports SENS_MAGN_INT] [get_bd_pins zynq_ps/SENS_MAGN_INT]
  connect_bd_net -net SENS_SPI0_MISO_1 [get_bd_ports SENS_SPI0_MISO] [get_bd_pins zynq_ps/SENS_SPI0_MISO]
  connect_bd_net -net SENS_SPI1_MISO_1 [get_bd_ports SENS_SPI1_MISO] [get_bd_pins zynq_ps/SENS_SPI1_MISO]
  connect_bd_net -net ap_rst_n_1 [get_bd_pins hdmi_out/ap_rst_n] [get_bd_pins zynq_ps/Dout]
  connect_bd_net -net aurora_raptor_m_axis_tdata [get_bd_pins aurora_raptor/m_axis_tdata] [get_bd_pins axi_gpio_mgt_speeds/gpio2_io_i]
  connect_bd_net -net aurora_xdp_m_axis_tdata [get_bd_pins aurora_xdp/m_axis_tdata] [get_bd_pins axi_gpio_mgt_speeds/gpio_io_i]
  connect_bd_net -net axi_iic_0_iic2intc_irpt [get_bd_pins axi_iic_gps/iic2intc_irpt] [get_bd_pins zynq_ps/pl_ps_irq10]
  connect_bd_net -net axi_iic_1_iic2intc_irpt [get_bd_pins axi_iic_1/iic2intc_irpt] [get_bd_pins zynq_ps/pl_ps_irq1]
  connect_bd_net -net axi_uart16550_0_ip2intc_irpt [get_bd_pins axi_uart16550_0/ip2intc_irpt] [get_bd_pins zynq_ps/pl_ps_irq0]
  connect_bd_net -net axi_uart16550_0_rtsn [get_bd_ports RTS_TO_BT_UART] [get_bd_pins axi_uart16550_0/rtsn]
  connect_bd_net -net axi_uart16550_0_sout [get_bd_ports TX_TO_BT_UART] [get_bd_pins axi_uart16550_0/sout]
  connect_bd_net -net axil_pwm_fan_control_0_o_fan_pwm [get_bd_ports fan_pwm] [get_bd_pins axil_pwm_fan_control_0/o_fan_pwm]
  connect_bd_net -net c_counter_binary_0_Q [get_bd_pins c_counter_binary_0/Q] [get_bd_pins xlslice_0/Din]
  connect_bd_net -net clk_fpga_bufg_BUFG_O [get_bd_pins c_counter_binary_0/CLK] [get_bd_pins clk_fpga_bufg/BUFG_O]
  connect_bd_net -net clk_wiz_1_clk_333Mhz [get_bd_pins axi_smc/aclk] [get_bd_pins axi_smc1/aclk] [get_bd_pins axi_smc2/aclk] [get_bd_pins axi_smc3/aclk] [get_bd_pins axi_smc_1/aclk] [get_bd_pins axis_data_fifo_0/s_axis_aclk] [get_bd_pins clk_wiz_1/clk_333Mhz] [get_bd_pins dyplo_axi_0/M_AXI_0_ACLK] [get_bd_pins dyplo_axi_0/M_AXI_1_ACLK] [get_bd_pins dyplo_axi_0/dyplo_clock] [get_bd_pins dyplo_axi_0/io_clk] [get_bd_pins hdmi_out/m_axi_mm2s_aclk] [get_bd_pins mipi_camera_0/video_aclk] [get_bd_pins mipi_camera_1/video_aclk] [get_bd_pins proc_sys_reset_0/slowest_sync_clk] [get_bd_pins vcu_0/m_axi_dec_aclk] [get_bd_pins vcu_0/m_axi_enc_aclk] [get_bd_pins vcu_0/m_axi_mcu_aclk] [get_bd_pins zynq_ps/saxihp0_fpd_aclk] [get_bd_pins zynq_ps/saxihpc0_fpd_aclk]
  connect_bd_net -net clk_wiz_1_clk_33Mhz [get_bd_pins clk_wiz_1/clk_33Mhz] [get_bd_pins vcu_0/pll_ref_clk]
  connect_bd_net -net clk_wiz_1_locked [get_bd_pins clk_wiz_1/locked] [get_bd_pins proc_sys_reset_0/dcm_locked]
  connect_bd_net -net const_VCC_dout [get_bd_ports HDMI_5V_EN] [get_bd_ports HDMI_RT_OE] [get_bd_pins const_VCC/dout]
  connect_bd_net -net csc_rst_in_1 [get_bd_pins mipi_camera_0/csc_rst_in] [get_bd_pins zynq_ps/mipi_0_csc_rst_n]
  connect_bd_net -net csc_rst_in_2 [get_bd_pins mipi_camera_1/csc_rst_in] [get_bd_pins zynq_ps/mipi_1_csc_rst_n]
  connect_bd_net -net ctsn_0_1 [get_bd_ports CTS_FROM_BT_UART] [get_bd_pins axi_uart16550_0/ctsn]
  connect_bd_net -net demosaic_rst_n_1 [get_bd_pins mipi_camera_0/demosaic_rst_n] [get_bd_pins zynq_ps/mipi_0_demosaic_rst_n]
  connect_bd_net -net demosaic_rst_n_2 [get_bd_pins mipi_camera_1/demosaic_rst_n] [get_bd_pins zynq_ps/mipi_1_demosaic_rst_n]
  connect_bd_net -net dphy_clk_200M_1 [get_bd_pins mipi_camera_0/dphy_clk_200M] [get_bd_pins mipi_camera_1/dphy_clk_200M] [get_bd_pins util_ds_buf_2/BUFG_O]
  connect_bd_net -net dyplo_axi_0_dyplo_irq [get_bd_pins dyplo_axi_0/dyplo_irq] [get_bd_pins zynq_ps/pl_ps_irq15]
  connect_bd_net -net hdmi_out_HDMI_TX_DAT_N_OUT [get_bd_ports HDMI_TX_DAT_N_OUT] [get_bd_pins hdmi_out/HDMI_TX_DAT_N_OUT]
  connect_bd_net -net hdmi_out_HDMI_TX_DAT_P_OUT [get_bd_ports HDMI_TX_DAT_P_OUT] [get_bd_pins hdmi_out/HDMI_TX_DAT_P_OUT]
  connect_bd_net -net i_fan_tacho_0_1 [get_bd_ports fan_tacho] [get_bd_pins axil_pwm_fan_control_0/i_fan_tacho]
  connect_bd_net -net mipi_camera_0_clkoutphy_out_0 [get_bd_pins mipi_camera_0/clkoutphy_out_0] [get_bd_pins mipi_camera_1/clkoutphy_in]
  connect_bd_net -net mipi_camera_0_csirxss_csi_irq_0 [get_bd_pins mipi_camera_0/csirxss_csi_irq_0] [get_bd_pins zynq_ps/pl_ps_irq2]
  connect_bd_net -net mipi_camera_0_iic2intc_irpt [get_bd_pins mipi_camera_0/iic2intc_irpt] [get_bd_pins zynq_ps/pl_ps_irq3]
  connect_bd_net -net mipi_camera_0_interrupt_demosaic [get_bd_pins mipi_camera_0/interrupt_demosaic] [get_bd_pins zynq_ps/pl_ps_irq11]
  connect_bd_net -net mipi_camera_0_pll_lock_out_0 [get_bd_pins mipi_camera_0/pll_lock_out_0] [get_bd_pins mipi_camera_1/pll_lock_in]
  connect_bd_net -net mipi_camera_1_csirxss_csi_irq_0 [get_bd_pins mipi_camera_1/csirxss_csi_irq_0] [get_bd_pins zynq_ps/pl_ps_irq5]
  connect_bd_net -net mipi_camera_1_iic2intc_irpt [get_bd_pins mipi_camera_1/iic2intc_irpt] [get_bd_pins zynq_ps/pl_ps_irq6]
  connect_bd_net -net mipi_camera_1_interrupt_demosaic [get_bd_pins mipi_camera_1/interrupt_demosaic] [get_bd_pins zynq_ps/pl_ps_irq12]
  connect_bd_net -net pl_ps_irq13_1 [get_bd_pins hdmi_out/hdmitx_irq] [get_bd_pins zynq_ps/pl_ps_irq13]
  connect_bd_net -net pl_ps_irq14_1 [get_bd_pins mipi_camera_0/fb_write_irq] [get_bd_pins zynq_ps/pl_ps_irq14]
  connect_bd_net -net pl_ps_irq7_1 [get_bd_pins hdmi_out/frmbuf_rd_irq] [get_bd_pins zynq_ps/pl_ps_irq7]
  connect_bd_net -net pl_ps_irq8_1 [get_bd_pins hdmi_out/vhy_irq] [get_bd_pins zynq_ps/pl_ps_irq8]
  connect_bd_net -net proc_sys_reset_0_interconnect_aresetn [get_bd_pins axi_smc1/aresetn] [get_bd_pins axi_smc2/aresetn] [get_bd_pins axi_smc3/aresetn] [get_bd_pins axis_data_fifo_0/s_axis_aresetn] [get_bd_pins dyplo_axi_0/M_AXI_0_ARESETN] [get_bd_pins dyplo_axi_0/M_AXI_1_ARESETN] [get_bd_pins proc_sys_reset_0/interconnect_aresetn]
  connect_bd_net -net proc_sys_reset_0_peripheral_aresetn [get_bd_pins axi_smc_1/aresetn] [get_bd_pins hdmi_out/vid_phy_tx_axi4s_aresetn] [get_bd_pins mipi_camera_0/video_aresetn] [get_bd_pins mipi_camera_1/video_aresetn] [get_bd_pins proc_sys_reset_0/peripheral_aresetn]
  connect_bd_net -net reset_1 [get_bd_pins aurora_raptor/reset] [get_bd_pins zynq_ps/aurora_raptor_reset]
  connect_bd_net -net rst_ps8_0_149M_peripheral_aresetn [get_bd_pins axi_smc/aresetn] [get_bd_pins zynq_ps/peripheral_aresetn_clk1]
  connect_bd_net -net rst_ps8_0_99M_interconnect_aresetn [get_bd_pins axi_interconnect_0/ARESETN] [get_bd_pins hdmi_out/ARESETN] [get_bd_pins mipi_camera_0/ARESETN] [get_bd_pins mipi_camera_1/ARESETN] [get_bd_pins zynq_ps/interconnect_aresetn]
  connect_bd_net -net rst_ps8_0_99M_peripheral_aresetn [get_bd_pins axi_gpio_eio/s_axi_aresetn] [get_bd_pins axi_gpio_mipi/s_axi_aresetn] [get_bd_pins axi_gpio_raptor/s_axi_aresetn] [get_bd_pins axi_gpio_scio/s_axi_aresetn] [get_bd_pins axi_iic_1/s_axi_aresetn] [get_bd_pins axi_iic_gps/s_axi_aresetn] [get_bd_pins axi_interconnect_0/M00_ARESETN] [get_bd_pins axi_interconnect_0/M01_ARESETN] [get_bd_pins axi_interconnect_0/M02_ARESETN] [get_bd_pins axi_interconnect_0/M03_ARESETN] [get_bd_pins axi_interconnect_0/M04_ARESETN] [get_bd_pins axi_interconnect_0/M05_ARESETN] [get_bd_pins axi_interconnect_0/M06_ARESETN] [get_bd_pins axi_interconnect_0/M07_ARESETN] [get_bd_pins axi_interconnect_0/M08_ARESETN] [get_bd_pins axi_interconnect_0/M09_ARESETN] [get_bd_pins axi_interconnect_0/M10_ARESETN] [get_bd_pins axi_interconnect_0/M11_ARESETN] [get_bd_pins axi_interconnect_0/M12_ARESETN] [get_bd_pins axi_interconnect_0/M13_ARESETN] [get_bd_pins axi_interconnect_0/M14_ARESETN] [get_bd_pins axi_interconnect_0/M15_ARESETN] [get_bd_pins axi_interconnect_0/M16_ARESETN] [get_bd_pins axi_interconnect_0/M17_ARESETN] [get_bd_pins axi_interconnect_0/M18_ARESETN] [get_bd_pins axi_interconnect_0/M19_ARESETN] [get_bd_pins axi_interconnect_0/M20_ARESETN] [get_bd_pins axi_interconnect_0/M21_ARESETN] [get_bd_pins axi_interconnect_0/M22_ARESETN] [get_bd_pins axi_interconnect_0/S00_ARESETN] [get_bd_pins axi_ivplogger_0/M_AXI_ARESETN] [get_bd_pins axi_ivplogger_0/S_AXI_ARESETN] [get_bd_pins axi_motorfoc_0/S_AXI_ARESETN] [get_bd_pins axi_motorfoc_1/S_AXI_ARESETN] [get_bd_pins axi_motorfoc_2/S_AXI_ARESETN] [get_bd_pins axi_motorfoc_3/S_AXI_ARESETN] [get_bd_pins axi_tep2_drv_0/S_AXI_ARESETN] [get_bd_pins axi_tep2_drv_1/S_AXI_ARESETN] [get_bd_pins axi_tep2_drv_2/S_AXI_ARESETN] [get_bd_pins axi_tep2_drv_3/S_AXI_ARESETN] [get_bd_pins axi_uart16550_0/s_axi_aresetn] [get_bd_pins axil_pwm_fan_control_0/S_AXI_ARESETN] [get_bd_pins dyplo_axi_0/S_AXI_ARESETN] [get_bd_pins hdmi_out/axi_resetn] [get_bd_pins mipi_camera_0/lite_aresetn] [get_bd_pins mipi_camera_1/lite_aresetn] [get_bd_pins zynq_ps/peripheral_aresetn_clk0]
  connect_bd_net -net sin_0_1 [get_bd_ports TX_FROM_BT_UART] [get_bd_pins axi_uart16550_0/sin]
  connect_bd_net -net strb_0x3_dout [get_bd_pins dyplo_axi_0/io_node_1_in_stream_1_tstrb] [get_bd_pins strb_0x3/dout]
  connect_bd_net -net switch_pl_0 [get_bd_ports switch_pl_0] [get_bd_pins zynq_ps/switch_pl_0]
  connect_bd_net -net util_ds_buf_0_IBUF_OUT [get_bd_pins util_ds_buf_0/IBUF_OUT] [get_bd_pins util_ds_buf_2/BUFG_I]
  connect_bd_net -net util_ds_buf_1_IBUF_OUT [get_bd_pins clk_fpga_bufg/BUFG_I] [get_bd_pins util_ds_buf_1/IBUF_OUT]
  connect_bd_net -net util_ds_buf_2_BUFG_O [get_bd_ports mipi_0_clk_tmp] [get_bd_ports mipi_1_clk_tmp] [get_bd_pins clk_wiz_0/clk_out1]
  connect_bd_net -net vcu_0_vcu_host_interrupt [get_bd_pins vcu_0/vcu_host_interrupt] [get_bd_pins zynq_ps/pl_ps_irq9]
  connect_bd_net -net xdp_version_1 [get_bd_ports xdp_version] [get_bd_pins zynq_ps/xdp_version]
  connect_bd_net -net xlslice_0_Dout1 [get_bd_ports user_led] [get_bd_pins xlslice_0/Dout]
  connect_bd_net -net zynq_ps_Dout1 [get_bd_pins aurora_xdp/reset] [get_bd_pins zynq_ps/aurora_xdp_reset]
  connect_bd_net -net zynq_ps_Dout_1 [get_bd_ports BT_EN] [get_bd_pins zynq_ps/BT_EN]
  connect_bd_net -net zynq_ps_EIO_LED1 [get_bd_ports EIO_LED1] [get_bd_pins zynq_ps/EIO_LED1]
  connect_bd_net -net zynq_ps_EIO_LED2 [get_bd_ports EIO_LED2] [get_bd_pins zynq_ps/EIO_LED2]
  connect_bd_net -net zynq_ps_SDIO_WIFI_EN [get_bd_ports SDIO_WIFI_EN] [get_bd_pins zynq_ps/SDIO_WIFI_EN]
  connect_bd_net -net zynq_ps_SDIO_WIFI_clk [get_bd_pins io_buf_sdio1_clk/IOBUF_IO_I] [get_bd_pins zynq_ps/SDIO_WIFI_clk]
  connect_bd_net -net zynq_ps_SENS_GPS_DSEL [get_bd_ports SENS_GPS_DSEL] [get_bd_pins zynq_ps/SENS_GPS_DSEL]
  connect_bd_net -net zynq_ps_SENS_GPS_RESET_N [get_bd_ports SENS_GPS_RESET_N] [get_bd_pins zynq_ps/SENS_GPS_RESET_N]
  connect_bd_net -net zynq_ps_SENS_SPI0_CS_ACCEL [get_bd_ports SENS_SPI0_CS_ACCEL] [get_bd_pins zynq_ps/SENS_SPI0_CS_ACCEL]
  connect_bd_net -net zynq_ps_SENS_SPI0_CS_GYRO [get_bd_ports SENS_SPI0_CS_GYRO] [get_bd_pins zynq_ps/SENS_SPI0_CS_GYRO]
  connect_bd_net -net zynq_ps_SENS_SPI0_MOSI [get_bd_ports SENS_SPI0_MOSI] [get_bd_pins zynq_ps/SENS_SPI0_MOSI]
  connect_bd_net -net zynq_ps_SENS_SPI0_SCK [get_bd_ports SENS_SPI0_SCK] [get_bd_pins zynq_ps/SENS_SPI0_SCK]
  connect_bd_net -net zynq_ps_SENS_SPI1_CS_ENV [get_bd_ports SENS_SPI1_CS_ENV] [get_bd_pins zynq_ps/SENS_SPI1_CS_ENV]
  connect_bd_net -net zynq_ps_SENS_SPI1_CS_MAGN [get_bd_ports SENS_SPI1_CS_MAGN] [get_bd_pins zynq_ps/SENS_SPI1_CS_MAGN]
  connect_bd_net -net zynq_ps_SENS_SPI1_MOSI [get_bd_ports SENS_SPI1_MOSI] [get_bd_pins zynq_ps/SENS_SPI1_MOSI]
  connect_bd_net -net zynq_ps_SENS_SPI1_SCK [get_bd_ports SENS_SPI1_SCK] [get_bd_pins zynq_ps/SENS_SPI1_SCK]
  connect_bd_net -net zynq_ps_hdmi_tx_ref_clk_rdy [get_bd_pins hdmi_out/tx_refclk_rdy] [get_bd_pins zynq_ps/hdmi_tx_ref_clk_rdy]
  connect_bd_net -net zynq_ps_mipi_0_frmbuf_rst_n [get_bd_pins mipi_camera_0/frmbuf_rst_n] [get_bd_pins zynq_ps/mipi_0_frmbuf_rst_n]
  connect_bd_net -net zynq_ps_mipi_0_rst [get_bd_ports mipi_0_rst] [get_bd_pins zynq_ps/mipi_0_rst]
  connect_bd_net -net zynq_ps_mipi_1_rst [get_bd_ports mipi_1_rst] [get_bd_pins zynq_ps/mipi_1_rst]
  connect_bd_net -net zynq_ps_pl_resetn0 [get_bd_pins clk_wiz_1/resetn] [get_bd_pins proc_sys_reset_0/ext_reset_in] [get_bd_pins zynq_ps/pl_resetn0]
  connect_bd_net -net zynq_ps_vcu_0_rst_n [get_bd_pins vcu_0/vcu_resetn] [get_bd_pins zynq_ps/vcu_0_rst_n]
  connect_bd_net -net zynq_ultra_ps_e_0_pl_clk0 [get_bd_pins aurora_raptor/init_clk] [get_bd_pins aurora_xdp/init_clk] [get_bd_pins axi_gpio_eio/s_axi_aclk] [get_bd_pins axi_gpio_mgt_speeds/s_axi_aclk] [get_bd_pins axi_gpio_mipi/s_axi_aclk] [get_bd_pins axi_gpio_raptor/s_axi_aclk] [get_bd_pins axi_gpio_scio/s_axi_aclk] [get_bd_pins axi_iic_1/s_axi_aclk] [get_bd_pins axi_iic_gps/s_axi_aclk] [get_bd_pins axi_interconnect_0/ACLK] [get_bd_pins axi_interconnect_0/M00_ACLK] [get_bd_pins axi_interconnect_0/M01_ACLK] [get_bd_pins axi_interconnect_0/M02_ACLK] [get_bd_pins axi_interconnect_0/M03_ACLK] [get_bd_pins axi_interconnect_0/M04_ACLK] [get_bd_pins axi_interconnect_0/M05_ACLK] [get_bd_pins axi_interconnect_0/M06_ACLK] [get_bd_pins axi_interconnect_0/M07_ACLK] [get_bd_pins axi_interconnect_0/M08_ACLK] [get_bd_pins axi_interconnect_0/M09_ACLK] [get_bd_pins axi_interconnect_0/M10_ACLK] [get_bd_pins axi_interconnect_0/M11_ACLK] [get_bd_pins axi_interconnect_0/M12_ACLK] [get_bd_pins axi_interconnect_0/M13_ACLK] [get_bd_pins axi_interconnect_0/M14_ACLK] [get_bd_pins axi_interconnect_0/M15_ACLK] [get_bd_pins axi_interconnect_0/M16_ACLK] [get_bd_pins axi_interconnect_0/M17_ACLK] [get_bd_pins axi_interconnect_0/M18_ACLK] [get_bd_pins axi_interconnect_0/M19_ACLK] [get_bd_pins axi_interconnect_0/M20_ACLK] [get_bd_pins axi_interconnect_0/M21_ACLK] [get_bd_pins axi_interconnect_0/M22_ACLK] [get_bd_pins axi_interconnect_0/S00_ACLK] [get_bd_pins axi_ivplogger_0/M_AXI_ACLK] [get_bd_pins axi_ivplogger_0/S_AXI_ACLK] [get_bd_pins axi_motorfoc_0/S_AXI_ACLK] [get_bd_pins axi_motorfoc_1/S_AXI_ACLK] [get_bd_pins axi_motorfoc_2/S_AXI_ACLK] [get_bd_pins axi_motorfoc_3/S_AXI_ACLK] [get_bd_pins axi_smc/aclk1] [get_bd_pins axi_tep2_drv_0/S_AXI_ACLK] [get_bd_pins axi_tep2_drv_1/S_AXI_ACLK] [get_bd_pins axi_tep2_drv_2/S_AXI_ACLK] [get_bd_pins axi_tep2_drv_3/S_AXI_ACLK] [get_bd_pins axi_uart16550_0/s_axi_aclk] [get_bd_pins axil_pwm_fan_control_0/S_AXI_ACLK] [get_bd_pins clk_wiz_0/clk_in1] [get_bd_pins clk_wiz_1/clk_in1] [get_bd_pins dyplo_axi_0/S_AXI_ACLK] [get_bd_pins dyplo_axi_0/dna_port_clk] [get_bd_pins hdmi_out/s_axi_lite_aclk] [get_bd_pins mipi_camera_0/lite_aclk] [get_bd_pins mipi_camera_1/lite_aclk] [get_bd_pins vcu_0/s_axi_lite_aclk] [get_bd_pins zynq_ps/pl_clk0]

  # Create address segments
  assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces axi_ivplogger_0/M_AXI] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP2/HP0_DDR_LOW] -force
  assign_bd_address -offset 0xFF000000 -range 0x01000000 -target_address_space [get_bd_addr_spaces axi_ivplogger_0/M_AXI] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP2/HP0_LPS_OCM] -force
  assign_bd_address -offset 0xC0000000 -range 0x20000000 -target_address_space [get_bd_addr_spaces axi_ivplogger_0/M_AXI] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP2/HP0_QSPI] -force
  assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces dyplo_axi_0/M_AXI_0] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP1/HPC1_DDR_LOW] -force
  assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces dyplo_axi_0/M_AXI_1] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP1/HPC1_DDR_LOW] -force
  assign_bd_address -offset 0x000800000000 -range 0x000800000000 -target_address_space [get_bd_addr_spaces vcu_0/EncData1] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP4/HP2_DDR_HIGH] -force
  assign_bd_address -offset 0x000800000000 -range 0x000800000000 -target_address_space [get_bd_addr_spaces vcu_0/EncData0] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP4/HP2_DDR_HIGH] -force
  assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces vcu_0/EncData1] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP4/HP2_DDR_LOW] -force
  assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces vcu_0/EncData0] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP4/HP2_DDR_LOW] -force
  assign_bd_address -offset 0xFF000000 -range 0x01000000 -target_address_space [get_bd_addr_spaces vcu_0/EncData1] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP4/HP2_LPS_OCM] -force
  assign_bd_address -offset 0xFF000000 -range 0x01000000 -target_address_space [get_bd_addr_spaces vcu_0/EncData0] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP4/HP2_LPS_OCM] -force
  assign_bd_address -offset 0xE0000000 -range 0x10000000 -target_address_space [get_bd_addr_spaces vcu_0/EncData1] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP4/HP2_PCIE_LOW] -force
  assign_bd_address -offset 0xE0000000 -range 0x10000000 -target_address_space [get_bd_addr_spaces vcu_0/EncData0] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP4/HP2_PCIE_LOW] -force
  assign_bd_address -offset 0xC0000000 -range 0x20000000 -target_address_space [get_bd_addr_spaces vcu_0/EncData1] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP4/HP2_QSPI] -force
  assign_bd_address -offset 0xC0000000 -range 0x20000000 -target_address_space [get_bd_addr_spaces vcu_0/EncData0] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP4/HP2_QSPI] -force
  assign_bd_address -offset 0x000800000000 -range 0x000800000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData0] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP5/HP3_DDR_HIGH] -force
  assign_bd_address -offset 0x000800000000 -range 0x000800000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData1] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP5/HP3_DDR_HIGH] -force
  assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData0] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP5/HP3_DDR_LOW] -force
  assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData1] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP5/HP3_DDR_LOW] -force
  assign_bd_address -offset 0xFF000000 -range 0x01000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData1] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP5/HP3_LPS_OCM] -force
  assign_bd_address -offset 0xFF000000 -range 0x01000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData0] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP5/HP3_LPS_OCM] -force
  assign_bd_address -offset 0xE0000000 -range 0x10000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData1] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP5/HP3_PCIE_LOW] -force
  assign_bd_address -offset 0xE0000000 -range 0x10000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData0] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP5/HP3_PCIE_LOW] -force
  assign_bd_address -offset 0xC0000000 -range 0x20000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData1] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP5/HP3_QSPI] -force
  assign_bd_address -offset 0xC0000000 -range 0x20000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData0] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP5/HP3_QSPI] -force
  assign_bd_address -offset 0x000800000000 -range 0x000800000000 -target_address_space [get_bd_addr_spaces vcu_0/Code] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP0/HPC0_DDR_HIGH] -force
  assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces vcu_0/Code] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP0/HPC0_DDR_LOW] -force
  assign_bd_address -offset 0xFF000000 -range 0x01000000 -target_address_space [get_bd_addr_spaces vcu_0/Code] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP0/HPC0_LPS_OCM] -force
  assign_bd_address -offset 0xE0000000 -range 0x10000000 -target_address_space [get_bd_addr_spaces vcu_0/Code] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP0/HPC0_PCIE_LOW] -force
  assign_bd_address -offset 0xC0000000 -range 0x20000000 -target_address_space [get_bd_addr_spaces vcu_0/Code] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP0/HPC0_QSPI] -force
  assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces hdmi_out/v_frmbuf_rd_0/Data_m_axi_mm_video] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP2/HP0_DDR_LOW] -force
  assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces mipi_camera_0/v_frmbuf_wr_0/Data_m_axi_mm_video] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP2/HP0_DDR_LOW] -force
  assign_bd_address -offset 0xA0007000 -range 0x00001000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_gpio_eio/S_AXI/Reg] -force
  assign_bd_address -offset 0xA0002000 -range 0x00001000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_gpio_mgt_speeds/S_AXI/Reg] -force
  assign_bd_address -offset 0xA000A000 -range 0x00001000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_gpio_mipi/S_AXI/Reg] -force
  assign_bd_address -offset 0xA0008000 -range 0x00001000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_gpio_raptor/S_AXI/Reg] -force
  assign_bd_address -offset 0xA0009000 -range 0x00001000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_gpio_scio/S_AXI/Reg] -force
  assign_bd_address -offset 0xA0130000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs mipi_camera_0/axi_iic_0/S_AXI/Reg] -force
  assign_bd_address -offset 0xA01B0000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs mipi_camera_1/axi_iic_0/S_AXI/Reg] -force
  assign_bd_address -offset 0xA0004000 -range 0x00001000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_iic_1/S_AXI/Reg] -force
  assign_bd_address -offset 0xA000D000 -range 0x00001000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_iic_gps/S_AXI/Reg] -force
  assign_bd_address -offset 0xA0070000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_ivplogger_0/S_AXI/reg0] -force
  assign_bd_address -offset 0xA0020000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_motorfoc_0/S_AXI/reg0] -force
  assign_bd_address -offset 0xA0030000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_motorfoc_1/S_AXI/reg0] -force
  assign_bd_address -offset 0xA0050000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_motorfoc_2/S_AXI/reg0] -force
  assign_bd_address -offset 0xA0060000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_motorfoc_3/S_AXI/reg0] -force
  assign_bd_address -offset 0xA00B0000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_tep2_drv_0/S_AXI/reg0] -force
  assign_bd_address -offset 0xA00C0000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_tep2_drv_1/S_AXI/reg0] -force
  assign_bd_address -offset 0xA00D0000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_tep2_drv_2/S_AXI/reg0] -force
  assign_bd_address -offset 0xA00E0000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_tep2_drv_3/S_AXI/reg0] -force
  assign_bd_address -offset 0xA0010000 -range 0x00002000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_uart16550_0/S_AXI/Reg] -force
  assign_bd_address -offset 0xA0012000 -range 0x00001000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axil_pwm_fan_control_0/S_AXI/reg0] -force
  assign_bd_address -offset 0xA0400000 -range 0x00200000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs dyplo_axi_0/S_AXI/reg0] -force
  assign_bd_address -offset 0xA00F0000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs mipi_camera_0/mipi_csi2_rx_subsystem_0/csirxss_s_axi/Reg] -force
  assign_bd_address -offset 0xA0180000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs mipi_camera_1/mipi_csi2_rx_subsystem_0/csirxss_s_axi/Reg] -force
  assign_bd_address -offset 0xA0140000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs mipi_camera_0/v_demosaic_0/s_axi_CTRL/Reg] -force
  assign_bd_address -offset 0xA01C0000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs mipi_camera_1/v_demosaic_0/s_axi_CTRL/Reg] -force
  assign_bd_address -offset 0xA0150000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs hdmi_out/v_frmbuf_rd_0/s_axi_CTRL/Reg] -force
  assign_bd_address -offset 0xA0110000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs mipi_camera_0/v_frmbuf_wr_0/s_axi_CTRL/Reg] -force
  assign_bd_address -offset 0xA0160000 -range 0x00020000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs hdmi_out/v_hdmi_tx_ss_0/S_AXI_CPU_IN/Reg] -force
  assign_bd_address -offset 0xA0100000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs mipi_camera_0/v_proc_ss_csc/s_axi_ctrl/Reg] -force
  assign_bd_address -offset 0xA0040000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs mipi_camera_1/v_proc_ss_csc/s_axi_ctrl/Reg] -force
  assign_bd_address -offset 0xA0200000 -range 0x00100000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs vcu_0/S_AXI_LITE/Reg] -force
  assign_bd_address -offset 0xA01A0000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ps/zynq_ultra_ps_e_0/Data] [get_bd_addr_segs hdmi_out/vid_phy_controller_0/vid_phy_axi4lite/Reg] -force

  # Exclude Address Segments
  exclude_bd_addr_seg -target_address_space [get_bd_addr_spaces axi_ivplogger_0/M_AXI] [get_bd_addr_segs zynq_ps/zynq_ultra_ps_e_0/SAXIGP2/HP0_DDR_HIGH]


  # Restore current instance
  current_bd_instance $oldCurInst

  validate_bd_design
  save_bd_design
}
# End of create_root_design()


##################################################################
# MAIN FLOW
##################################################################

create_root_design ""


